Package: gridmicrotex
Title: Native 'LaTeX' Math Rendering for Grid Graphics
Version: 0.1.1
Authors@R: c(
    person("Alim", "Dayim", role = c("aut", "cre"),
           email = "ad938@cam.ac.uk",
           comment = c(ORCID = "0000-0001-9998-7463")),
    person("Nano", "Michael", role = "cph",
           comment = "Author of included 'MicroTeX' library"),
    person(family = "Bundled math font authors", role = "cph",
           comment = "See inst/COPYRIGHTS for the full list of authors of the bundled math fonts."))
Description: Renders 'LaTeX' math equations as native R grid graphics objects
    (grobs) using the 'MicroTeX' 'C++' library as the layout engine. Produces
    resolution-independent vector output that works on any R graphics device,
    with no external 'LaTeX' installation required. Markdown labels and block
    documents that mix prose formatting with math are also rendered, for use
    with both 'grid' and 'ggplot2'.
License: MIT + file LICENSE
Encoding: UTF-8
SystemRequirements: C++17, FreeType (>= 2.9), pkg-config, FriBidi
        (optional)
Depends: R (>= 4.2.0)
LinkingTo: Rcpp, systemfonts
Imports: commonmark, grDevices, grid, Rcpp, systemfonts, tools, utils,
        xml2
Suggests: ggplot2 (>= 4.0.0), grImport2, jpeg, knitr, png, ragg,
        rmarkdown, rsvg, S7, svglite, testthat (>= 3.0.0), textshaping,
        vdiffr
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/adayim/gridmicrotex,
        https://adayim.github.io/gridmicrotex/
BugReports: https://github.com/adayim/gridmicrotex/issues
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-08-23 23:52:23 UTC; alim
Author: Alim Dayim [aut, cre] (ORCID: <https://orcid.org/0000-0001-9998-7463>),
  Nano Michael [cph] (Author of included 'MicroTeX' library),
  Bundled math font authors [cph] (See inst/COPYRIGHTS for the full list
    of authors of the bundled math fonts.)
Maintainer: Alim Dayim <ad938@cam.ac.uk>
Repository: CRAN
Date/Publication: 2026-08-24 07:30:20 UTC

# Generated by roxygen2: do not edit by hand

S3method(ascentDetails,latexgrob)
S3method(descentDetails,latexgrob)
S3method(editDetails,latexgrob)
S3method(heightDetails,gridmicrotex_labels)
S3method(heightDetails,latexgrob)
S3method(heightDetails,markdownbox)
S3method(makeContent,latexgrob)
S3method(makeContent,markdownbox)
S3method(print,gridmicrotex_markdown_style)
S3method(print,gridmicrotex_md_style)
S3method(print,latex_tree)
S3method(widthDetails,gridmicrotex_labels)
S3method(widthDetails,latexgrob)
S3method(widthDetails,markdownbox)
S3method(xDetails,latexgrob)
S3method(yDetails,latexgrob)
export(GeomLatex)
export(GeomMarkdown)
export(available_highlighters)
export(available_math_fonts)
export(check_fonts)
export(check_math_fonts)
export(clear_macros)
export(define_macro)
export(element_latex)
export(element_markdown)
export(geom_latex)
export(geom_markdown)
export(grid.latex)
export(grid.markdown)
export(grobMark)
export(latex_cache_clear)
export(latex_cache_info)
export(latex_cache_limit)
export(latex_dims)
export(latex_grob)
export(latex_options)
export(latex_tree)
export(latex_wrap)
export(list_macros)
export(load_font)
export(load_math_font)
export(markdown_box_grob)
export(markdown_grob)
export(markdown_style)
export(md_style)
export(register_highlighter)
export(reset_latex_options)
importFrom(Rcpp,sourceCpp)
importFrom(grid,ascentDetails)
importFrom(grid,descentDetails)
importFrom(grid,editDetails)
importFrom(grid,heightDetails)
importFrom(grid,makeContent)
importFrom(grid,widthDetails)
importFrom(grid,xDetails)
importFrom(grid,yDetails)
useDynLib(gridmicrotex, .registration = TRUE)

# gridmicrotex

<!-- badges: start -->

[![R-CMD-check](https://github.com/adayim/gridmicrotex/workflows/R-CMD-check/badge.svg)](https://github.com/adayim/gridmicrotex/actions)
[![CRAN
status](https://www.r-pkg.org/badges/version/gridmicrotex)](https://CRAN.R-project.org/package=gridmicrotex)
[![CRAN
download](https://cranlogs.r-pkg.org/badges/grand-total/gridmicrotex)](https://cran.r-project.org/package=gridmicrotex)
[![codecov](https://codecov.io/gh/adayim/gridmicrotex/branch/main/graph/badge.svg?token=mzvaYDMPNc)](https://app.codecov.io/gh/adayim/gridmicrotex)
<!-- badges: end -->

Render LaTeX math expressions — and markdown containing them — as native
R **grid** graphics objects, with no external LaTeX installation
required.

gridmicrotex embeds the
[MicroTeX](https://github.com/NanoMichael/MicroTeX) C++ layout engine to
parse LaTeX, compute the full box model, and produce
resolution-independent vector output (paths, lines, rectangles) that
works on any R graphics device.

## Disclaimer

**A note on development**: This package was developed as a proof of
concept for AI-assisted package creation. I designed the architecture
and specification, and the core C++ integration (via
[MicroTeX](https://github.com/NanoMichael/MicroTeX)) was largely
facilitated by AI, with my review and oversight of the design and final
outputs. I am sharing it because it works, and I hope that others will
find it useful. Contributions, bug reports and improvements from the
community are very welcome.

## Installation

Install the development version from GitHub:

``` r
# install.packages("devtools")
devtools::install_github("adayim/gridmicrotex")
```

## Examples

``` r
library(gridmicrotex)
library(grid)

grid.newpage()
grid.latex("x = \\frac{\\textcolor{red}{-b} \\pm \\sqrt{b^{2} - 4ac}}{2a}", 
           gp = grid::gpar(fontsize = 30))
```

<img src="man/figures/README-example-basic-1.png" alt="" width="50%" />

By default, the input is treated as LaTeX math mode (“mixed” mode),
which wraps non-math text in `\text{}` and preserves math expressions
as-is. Use `$...$` or `\\(...\\)` delimiters to render math. The
`"x = "` in the equation above treated as text. Use
`input_mode = "math"` to treat the whole string as math mode and render
text with `\\text{}`. You can change this with global option
`latex_options(input_mode = "math")`.

### Composing with other grobs

The grob can be placed alongside other grid objects:

``` r
latex_options(input_mode = "math")
g <- latex_grob("\\frac{a}{b}", gp = grid::gpar(fontsize = 30))
grid.newpage()
# A blue box behind the formula
grid.rect(
  x = 0.5, y = 0.5,
  width = grobWidth(g) + unit(10, "bigpts"),
  height = grobHeight(g) + unit(10, "bigpts"),
  gp = gpar(fill = "#e8f0fe", col = "#4285f4", lwd = 2)
)

# The formula itself
grid.draw(g)
```

<img src="man/figures/README-example-compose-1.png" alt="" width="30%" />

### Multiple expressions

``` r
exprs <- c(
  "E = mc^{2}",
  "e^{i\\pi} + 1 = 0",
  "\\nabla \\times \\vec{E} = -\\frac{\\partial \\vec{B}}{\\partial t}",
  "\\binom{n}{k} = \\frac{n!}{k!(n-k)!}"
)

grid.newpage()
for (i in seq_along(exprs)) {
  grid.latex(
    exprs[i],
    x = unit(0.5, "npc"),
    y = unit(1 - i / (length(exprs) + 1), "npc"),
    gp = grid::gpar(fontsize = 28)
  )
}
```

<img src="man/figures/README-example-multiple-1.png" alt="" width="40%" />

### Mixed text and math

You can use `r"()"` raw strings to write LaTeX with regular newlines and
quotes without escaping. Use `\text{}` to embed regular text within math
expressions:

``` r
grid.newpage()
grid.latex(
  r"(f(x) = \begin{cases} x^2 & \text{if } x \geq 0 \\ -x & \text{otherwise} \end{cases})",
  gp = grid::gpar(fontsize = 26)
)
```

<img src="man/figures/README-example-mixed-definition-1.png" alt="" width="50%" />

## Markdown

`grid.markdown()` renders markdown, with `$math$` inline. Unlike the
other markdown-in-grid packages, the maths is real LaTeX rather than
plotmath:

``` r
grid.newpage()
grid.markdown(
  "The **fitted** slope is $\\hat{\\beta}_1 = 0.42$ (*p* < 0.001).",
  x = 0.02, hjust = 0, gp = gpar(fontsize = 20)
)
```

<img src="man/figures/README-example-markdown-1.png" alt="" width="70%" />

Colour, font and size come from inline HTML, as they must: markdown
itself defines no syntax for them.

``` r
grid.newpage()
grid.markdown(
  paste0('Set in <span style="font-family:serif">serif</span>, ',
         'H<sub>0</sub> <u>rejected</u> at ',
         '<span style="color:#B22222">5%</span>.'),
  x = 0.02, hjust = 0, gp = gpar(fontsize = 20)
)
```

<img src="man/figures/README-example-markdown-html-1.png" alt="" width="70%" />

`markdown_box_grob()` lays out a whole document — headings, lists,
quotes, code, tables and images — as a stack of grobs. See
`vignette("markdown")`.

## ggplot2 integration

Use `geom_latex()` to place LaTeX labels at data coordinates, and
`element_latex()` for LaTeX-rendered axis titles:

``` r
library(ggplot2)
# Add a LaTeX table as an annotation
tab_str <- r"(\begin{tabular}{c|c} \text{A} & B^2 \\ \hline 1 & \cellcolor{#00bde5}2 \\ 3 & 4 \end{tabular})"

df <- data.frame(x = 1:3, y = 1:3,
                 eq = c("x^2", "\\frac{a}{b}", "\\sum_{i=1}^n x_i"))
ggplot(df, aes(x, y, label = eq)) + 
  geom_latex() +
  annotate("latex", x = 1, y = 2.7, label = tab_str, size = 12) +
  labs(x = "$\\beta_1 \\cdot x + \\beta_0$") +
  theme(axis.title.x = element_latex())
```

<img src="man/figures/README-example-ggplot2-geom-1.png" alt="" width="80%" />

ggplot2 is a soft dependency — the core functions work without it. See
`vignette("ggplot2-integration")` for more examples.

## Comparison

| Approach         | LaTeX required? | Device independent? | Math coverage | Markdown |
|:-----------------|:---------------:|:-------------------:|:-------------:|:--------:|
| `tikzDevice`     |       Yes       |         No          |     Full      |    No    |
| `xdvir`          |       Yes       |         No          |     Full      |    No    |
| `latexpdf`       |       Yes       |         No          | Full (tables) |    No    |
| `latex2exp`      |       No        |         Yes         |    Limited    |    No    |
| `plotmath`       |       No        |         Yes         |    Limited    |    No    |
| `gridtext`       |       No        |         Yes         |     None      |   Yes    |
| `marquee`        |       No        |         Yes         |     None      |   Yes    |
| **gridmicrotex** |     **No**      |       **Yes**       |   **Broad**   | **Yes**  |

## How it works

1.  Your LaTeX string is parsed by MicroTeX’s C++ engine into a TeX box
    model
2.  A custom `Graphics2D` recorder captures every draw operation (glyph
    paths, lines, rectangles) with exact coordinates
3.  The layout crosses the C++/R boundary as a data frame
4.  R converts each record into native grid primitives (`pathGrob`,
    `segmentsGrob`, `rectGrob`)
5.  The result is a `gTree` that renders on any device at any resolution

By default, math glyphs are rendered in typeface mode as native text
using the selected math font, which keeps PDF/SVG output selectable and
searchable on devices with font embedding support.

When `render_mode = "path"` is used (or when automatic fallback is
triggered on unsupported devices), glyphs are drawn as filled vector
paths for consistent rendering everywhere.

Make sure to use `ragg::agg_png()`, `svglite::svglite()` or
`grDevices::cairo_pdf()` for best results, as some older devices may not
support the full range of path operations.

## Graphics backend

The default graphics device on Windows (`windows()`) and macOS
(`quartz()`) may not find the bundled math fonts, producing warnings
like:

    font family not found in Windows font database

To avoid this, switch to a modern graphics backend that uses
[systemfonts](https://CRAN.R-project.org/package=systemfonts) for font
resolution:

``` r
# For knitr / R Markdown — add to your setup chunk:
knitr::opts_chunk$set(dev = "ragg_png")

# For interactive use:
options(device = function(...) ragg::agg_png(tempfile(fileext = ".png"), ...))
```

Recommended backends:

| Backend | Format | Package |
|:---|:---|:---|
| `ragg::agg_png()` | PNG | [ragg](https://CRAN.R-project.org/package=ragg) |
| `svglite::svglite()` | SVG | [svglite](https://CRAN.R-project.org/package=svglite) |
| `grDevices::cairo_pdf()` | PDF | Base R (Cairo build) |

Alternatively, use `render_mode = "path"` to bypass font lookup entirely
— glyphs are drawn as vector paths, which works on all devices but
produces non-selectable text in PDF/SVG.

# gridmicrotex

<!-- badges: start -->

[![R-CMD-check](https://github.com/adayim/gridmicrotex/workflows/R-CMD-check/badge.svg)](https://github.com/adayim/gridmicrotex/actions)
[![CRAN
status](https://www.r-pkg.org/badges/version/gridmicrotex)](https://CRAN.R-project.org/package=gridmicrotex)
[![CRAN
download](https://cranlogs.r-pkg.org/badges/grand-total/gridmicrotex)](https://cran.r-project.org/package=gridmicrotex)
[![codecov](https://codecov.io/gh/adayim/gridmicrotex/branch/main/graph/badge.svg?token=mzvaYDMPNc)](https://app.codecov.io/gh/adayim/gridmicrotex)
<!-- badges: end -->

Render LaTeX math expressions as native R **grid** graphics objects,
with no external LaTeX installation required.

gridmicrotex embeds the
[MicroTeX](https://github.com/NanoMichael/MicroTeX) C++ layout engine to
parse LaTeX, compute the full box model, and produce
resolution-independent vector output (paths, lines, rectangles) that
works on any R graphics device.

## Disclaimer

**A note on development**: This package was developed as a proof of
concept for AI-assisted package creation. I designed the architecture
and specification, and the core C++ integration (via
[MicroTeX](https://github.com/NanoMichael/MicroTeX)) was largely
facilitated by AI, with my review and oversight of the design and final
outputs. I am sharing it because it works, and I hope that others will
find it useful. Contributions, bug reports and improvements from the
community are very welcome.

## Installation

Install the development version from GitHub:

``` r
# install.packages("devtools")
devtools::install_github("adayim/gridmicrotex")
```

## Examples

``` r
library(gridmicrotex)
library(grid)

grid.newpage()
grid.latex("x = \\frac{\\textcolor{red}{-b} \\pm \\sqrt{b^{2} - 4ac}}{2a}", 
           gp = grid::gpar(fontsize = 30))
```

<img src="man/figures/README-example-basic-1.png" alt="" width="50%" />

By default, the input is treated as LaTeX math mode (“mixed” mode),
which wraps non-math text in `\text{}` and preserves math expressions
as-is. Use `$...$` or `\\(...\\)` delimiters to render math. The
`"x = "` in the equation above treated as text. Use
`input_mode = "math"` to treat the whole string as math mode and render
text with `\\text{}`. You can change this with global option
`latex_options(input_mode = "math")`.

### Composing with other grobs

The grob can be placed alongside other grid objects:

``` r
latex_options(input_mode = "math")
g <- latex_grob("\\frac{a}{b}", gp = grid::gpar(fontsize = 30))
grid.newpage()
# A blue box behind the formula
grid.rect(
  x = 0.5, y = 0.5,
  width = grobWidth(g) + unit(10, "bigpts"),
  height = grobHeight(g) + unit(10, "bigpts"),
  gp = gpar(fill = "#e8f0fe", col = "#4285f4", lwd = 2)
)

# The formula itself
grid.draw(g)
```

<img src="man/figures/README-example-compose-1.png" alt="" width="30%" />

### Multiple expressions

``` r
exprs <- c(
  "E = mc^{2}",
  "e^{i\\pi} + 1 = 0",
  "\\nabla \\times \\vec{E} = -\\frac{\\partial \\vec{B}}{\\partial t}",
  "\\binom{n}{k} = \\frac{n!}{k!(n-k)!}"
)

grid.newpage()
for (i in seq_along(exprs)) {
  grid.latex(
    exprs[i],
    x = unit(0.5, "npc"),
    y = unit(1 - i / (length(exprs) + 1), "npc"),
    gp = grid::gpar(fontsize = 28)
  )
}
```

<img src="man/figures/README-example-multiple-1.png" alt="" width="40%" />

### Mixed text and math

You can use `r"()"` raw strings to write LaTeX with regular newlines and
quotes without escaping. Use `\text{}` to embed regular text within math
expressions:

``` r
grid.newpage()
grid.latex(
  r"(f(x) = \begin{cases} x^2 & \text{if } x \geq 0 \\ -x & \text{otherwise} \end{cases})",
  gp = grid::gpar(fontsize = 26)
)
```

<img src="man/figures/README-example-mixed-definition-1.png" alt="" width="50%" />

## CJK and multilingual text

Non-math text via `\text{}` supports CJK and other scripts. Font
settings from `gp` (`fontfamily`, `fontface`) apply to text only — math
rendering always uses the selected math font:

``` r
grid.newpage()
grid.latex(r"(\text{如果 } x > 0 \text{ 则 } y = x^2)",
           gp = gpar(fontsize = 24, fontfamily = "sans"))
```

<img src="man/figures/README-example-mixed-cjk-math-1.png" alt="" width="50%" />

## User-defined macros

Use `define_macro()` for zero-argument shorthands that persist across
plots in the session, and plain-TeX `\def` for parameterised macros
local to a single expression:

``` r
define_macro("RR", "\\mathbb{R}")
grid.newpage()
grid.latex(
  r"(\def\norm#1{\left\lVert #1 \right\rVert}
      \forall \vec{v} \in \RR^n,\ \norm{\vec{v}} \geq 0)",
  gp = grid::gpar(fontsize = 22)
)
```

<img src="man/figures/README-example-macros-1.png" alt="" width="60%" />

``` r
clear_macros()
```

See `?define_macro` and `vignette("getting-started")` for details.

## ggplot2 integration

Use `geom_latex()` to place LaTeX labels at data coordinates, and
`element_latex()` for LaTeX-rendered axis titles:

``` r
library(ggplot2)
# Add a LaTeX table as an annotation
tab_str <- r"(\begin{tabular}{c|c} \text{A} & B^2 \\ \hline 1 & \cellcolor{#00bde5}2 \\ 3 & 4 \end{tabular})"

df <- data.frame(x = 1:3, y = 1:3,
                 eq = c("x^2", "\\frac{a}{b}", "\\sum_{i=1}^n x_i"))
ggplot(df, aes(x, y, label = eq)) + 
  geom_latex() +
  annotate("latex", x = 1, y = 2.7, label = tab_str, size = 12) +
  labs(x = "$\\beta_1 \\cdot x + \\beta_0$") +
  theme(axis.title.x = element_latex())
```

<img src="man/figures/README-example-ggplot2-geom-1.png" alt="" width="80%" />

ggplot2 is a soft dependency — the core functions work without it. See
`vignette("ggplot2-integration")` for more examples.

## Comparison

| Approach         | LaTeX required? | Device independent? | Vector? | Math coverage |
|:-----------------|:---------------:|:-------------------:|:-------:|:-------------:|
| `tikzDevice`     |       Yes       |         No          |   Yes   |     Full      |
| `xdvir`          |       Yes       |         No          |   Yes   |     Full      |
| `latexpdf`       |       Yes       |         No          |   Yes   | Full (tables) |
| `latex2exp`      |       No        |         Yes         |   Yes   |    Limited    |
| `plotmath`       |       No        |         Yes         |   Yes   |    Limited    |
| **gridmicrotex** |     **No**      |       **Yes**       | **Yes** |   **Broad**   |

## How it works

1.  Your LaTeX string is parsed by MicroTeX’s C++ engine into a TeX box
    model
2.  A custom `Graphics2D` recorder captures every draw operation (glyph
    paths, lines, rectangles) with exact coordinates
3.  The layout crosses the C++/R boundary as a data frame
4.  R converts each record into native grid primitives (`pathGrob`,
    `segmentsGrob`, `rectGrob`)
5.  The result is a `gTree` that renders on any device at any resolution

By default, math glyphs are rendered in typeface mode as native text
using the selected math font, which keeps PDF/SVG output selectable and
searchable on devices with font embedding support.

When `render_mode = "path"` is used (or when automatic fallback is
triggered on unsupported devices), glyphs are drawn as filled vector
paths for consistent rendering everywhere.

Make sure to use `ragg::agg_png()`, `svglite::svglite()` or
`grDevices::cairo_pdf()` for best results, as some older devices may not
support the full range of path operations.

## Graphics backend

The default graphics device on Windows (`windows()`) and macOS
(`quartz()`) may not find the bundled math fonts, producing warnings
like:

    font family not found in Windows font database

To avoid this, switch to a modern graphics backend that uses
[systemfonts](https://CRAN.R-project.org/package=systemfonts) for font
resolution:

``` r
# For knitr / R Markdown — add to your setup chunk:
knitr::opts_chunk$set(dev = "ragg_png")

# For interactive use:
options(device = function(...) ragg::agg_png(tempfile(fileext = ".png"), ...))
```

Recommended backends:

| Backend | Format | Package |
|:---|:---|:---|
| `ragg::agg_png()` | PNG | [ragg](https://CRAN.R-project.org/package=ragg) |
| `svglite::svglite()` | SVG | [svglite](https://CRAN.R-project.org/package=svglite) |
| `grDevices::cairo_pdf()` | PDF | Base R (Cairo build) |

Alternatively, use `render_mode = "path"` to bypass font lookup entirely
— glyphs are drawn as vector paths, which works on all devices but
produces non-selectable text in PDF/SVG.

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dpi = 300,
  dev = "ragg_png"
)
library(gridmicrotex)

## ----basic, fig.height=0.8, fig.width=2, out.width="50%"----------------------
g <- latex_grob("\\frac{-b \\pm \\sqrt{b^2 - 4ac}}{2a}", gp = grid::gpar(fontsize = 24))
grid::grid.newpage()
grid::grid.draw(g)

## ----quick, fig.height=0.8, fig.width=2, out.width="50%"----------------------
grid::grid.newpage()
grid.latex("\\sum_{i=1}^{n} x_i^2", gp = grid::gpar(fontsize = 28))

## ----positioning, fig.height=1.2, fig.width=2, out.width="50%"----------------
grid::grid.newpage()
grid.latex("Famous $E = mc^2$", x = 0.2, y = 0.7, hjust = 0, gp = grid::gpar(fontsize = 24))
grid.latex("F = ma", x = 0.2, y = 0.3, hjust = 0, gp = grid::gpar(fontsize = 24), input_mode = "math")

## ----colors, fig.height=0.6, fig.width=2, out.width="40%"---------------------
latex_options(input_mode = "math") # Set math mode globally
grid::grid.newpage()
grid.latex(
  r"(\textcolor{red}{\alpha} + \textcolor{blue}{\beta} = \gamma)",
  gp = grid::gpar(fontsize = 28)
)

## ----fonts-list---------------------------------------------------------------
available_math_fonts()

## ----fonts-default, fig.height=0.8, fig.width=2, out.width="50%"--------------
latex_options(math_font = "stix")
grid::grid.newpage()
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24))

# Switch back to the default (Lete Sans Math)
latex_options(math_font = "lete")

## ----fonts, fig.height=1.5, fig.width=2, out.width="50%"----------------------
grid::grid.newpage()
grid::pushViewport(grid::viewport(layout = grid::grid.layout(2, 1)))
grid::pushViewport(grid::viewport(layout.pos.row = 1))
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24))
grid::upViewport()
grid::pushViewport(grid::viewport(layout.pos.row = 2))
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24), math_font = "stix")
grid::upViewport(2)

## ----dims---------------------------------------------------------------------
dims <- latex_dims("\\frac{a}{b}", gp = grid::gpar(fontsize = 20))
dims

## ----cjk, fig.height = 0.8, fig.width = 2.5, out.width="50%"------------------
grid::grid.newpage()
grid.latex("x^2 + \\text{你好}", gp = grid::gpar(fontsize = 24, fontfamily = "sans"))

## ----font-pairing, fig.height = 1, fig.width = 6, out.width="70%"-------------
grid::grid.newpage()
grid.latex(
  "\\text{Theorem: } \\forall x \\in \\mathbb{R},\\; x^2 \\geq 0",
  math_font = "stix",
  gp = grid::gpar(fontfamily = "serif", fontsize = 12)
)

## ----complex-formula, fig.height = 3, fig.width=6, out.width="60%"------------
grid::grid.newpage()
grid.latex(paste0(
      "\\begin{array}{l}",
      "  \\forall\\varepsilon\\in\\mathbb{R}_+^*\\ \\exists\\eta>0",
      "\\ |x-x_0|\\leq\\eta\\Longrightarrow|f(x)-f(x_0)|\\leq\\varepsilon\\\\",
      "  \\det",
      "  \\begin{bmatrix}",
      "      a_{11}&a_{12}&\\cdots&a_{1n}\\\\",
      "      a_{21}&\\ddots&&\\vdots\\\\",
      "      \\vdots&&\\ddots&\\vdots\\\\",
      "      a_{n1}&\\cdots&\\cdots&a_{nn}",
      "  \\end{bmatrix}",
      "  \\overset{\\mathrm{def}}{=}\\sum_{\\sigma\\in\\mathfrak{S}_n}",
      "\\varepsilon(\\sigma)\\prod_{k=1}^n a_{k\\sigma(k)}\\\\",
      "  \\int_0^\\infty{x^{2n} e^{-a x^2}\\,dx} = \\frac{2n-1}{2a}",
      " \\int_0^\\infty{x^{2(n-1)} e^{-a x^2}\\,dx}",
      " = \\frac{(2n-1)!!}{2^{n+1}} \\sqrt{\\frac{\\pi}{a^{2n+1}}}\\\\",
      "\\end{array}"
), gp = grid::gpar(fontsize = 16))

## ----table-multicolumn, fig.height = 3, fig.width=8, out.width="60%"----------
grid::grid.newpage()

grid.latex(
  "
  \\newcolumntype{s}{>{\\color{#1234B6}}c}
\\begin{array}{|c|c|c|s|}
  \\hline
  \\rowcolor{Tan}\\multicolumn{4}{|c|}{\\textcolor{white}{\\bold{\\text{Table Head}}}}\\\\
  \\hline
  \\text{Matrix}&\\multicolumn{2}{|c|}{\\text{Multicolumns}}&\\text{Font size commands}\\\\
  \\hline
  \\begin{pmatrix}
      \\alpha_{11}&\\cdots&\\alpha_{1n}\\\\
      \\hdotsfor{3}\\\\
      \\alpha_{n1}&\\cdots&\\alpha_{nn}
  \\end{pmatrix}
  &\\large \\text{Left}&\\cellcolor{#00bde5}\\small \\textcolor{white}{\\text{\\bold{Right}}}
  &\\small \\text{small Small}\\\\
  \\hline
  \\multicolumn{4}{|c|}{\\text{Table Foot}}\\\\
  \\hline
\\end{array}
  ",
  gp = grid::gpar(fontsize = 22)
)

## ----complicated-equation, fig.height = 9, fig.width=9, out.width="60%"-------
grid::grid.newpage()
grid.latex(
  "\\definecolor{gris}{gray}{0.9}
\\definecolor{noir}{rgb}{0,0,0}
\\definecolor{bleu}{rgb}{0,0,1}
\\fatalIfCmdConflict{false}
\\newcommand{\\pa}{\\left|}
\\begin{array}{c}
  \\LaTeX\\\\
  \\begin{split}
      |I_2| &= \\pa\\int_0^T\\psi(t)\\left\\{ u(a,t)-\\int_{\\gamma(t)}^a \\frac{d\\theta}{k} (\\theta,t) \\int_a^\\theta c(\\xi)
          u_t (\\xi,t)\\,d\\xi\\right\\}dt\\right|\\\\
      &\\le C_6 \\Bigg|\\pa f \\int_\\Omega \\pa\\widetilde{S}^{-1,0}_{a,-}
          W_2(\\Omega, \\Gamma_1)\\right|\\ \\right|\\left| |u|\\overset{\\circ}{\\to} W_2^{\\widetilde{A}}(\\Omega\\Gamma_r,T)\\right|\\Bigg|\\\\
      &\\\\
      &\\begin{pmatrix}
          \\alpha&\\beta&\\gamma&\\delta\\\\
          \\aleph&\\beth&\\gimel&\\daleth\\\\
          \\mathfrak{A}&\\mathfrak{B}&\\mathfrak{C}&\\mathfrak{D}\\\\
          \\boldsymbol{\\mathfrak{a}}&\\boldsymbol{\\mathfrak{b}}&\\boldsymbol{\\mathfrak{c}}&\\boldsymbol{\\mathfrak{d}}
      \\end{pmatrix}
      \\quad{(a+b)}^{\\frac{n}{2}}=\\sqrt{\\sum_{k=0}^n\\tbinom{n}{k}a^kb^{n-k}}\\quad
          \\Biggl(\\biggl(\\Bigl(\\bigl(()\\bigr)\\Bigr)\\biggr)\\Biggr)\\\\
      &\\forall\\varepsilon\\in\\mathbb{R}_+^*\\ \\exists\\eta>0\\ |x-x_0|\\leq\\eta\\Longrightarrow|f(x)-f(x_0)|\\leq\\varepsilon\\\\
      &\\det
      \\begin{bmatrix}
          a_{11}&a_{12}&\\cdots&a_{1n}\\\\
          a_{21}&\\ddots&&\\vdots\\\\
          \\vdots&&\\ddots&\\vdots\\\\
          a_{n1}&\\cdots&\\cdots&a_{nn}
      \\end{bmatrix}
      \\overset{\\mathrm{def}}{=}\\sum_{\\sigma\\in\\mathfrak{S}_n}\\varepsilon(\\sigma)\\prod_{k=1}^n a_{k\\sigma(k)}\\\\
      &\\Delta f(x,y)=\\frac{\\partial^2f}{\\partial x^2}+\\frac{\\partial^2f}{\\partial y^2}\\qquad\\qquad \\fcolorbox{noir}{gris}
          {n!\\underset{n\\rightarrow+\\infty}{\\sim} {\\left(\\frac{n}{e}\\right)}^n\\sqrt{2\\pi n}}\\\\
      &\\sideset{_\\alpha^\\beta}{_\\gamma^\\delta}{
      \\begin{pmatrix}
          a&b\\\\
          c&d
      \\end{pmatrix}}
      \\xrightarrow[T]{n\\pm i-j}\\sideset{^t}{}A\\xleftarrow{\\overrightarrow{u}\\wedge\\overrightarrow{v}}
          \\underleftrightarrow{\\iint_{\\mathds{R}^2}e^{-\\left(x^2+y^2\\right)}\\,\\mathrm{d}x\\mathrm{d}y}
  \\end{split}\\\\
  \\rotatebox{30}{\\sum_{n=1}^{+\\infty}}\\quad\\mbox{Mirror rorriM}\\reflectbox{\\mbox{Mirror rorriM}}
\\end{array}",
  gp = grid::gpar(fontsize = 22),
  render_mode = "path"
)

## ----options, eval=FALSE------------------------------------------------------
# latex_options(math_font = "stix", render_mode = "typeface")
# 
# # Later calls pick these up automatically
# grid.latex("\\sum_{i=1}^{n} i^{2}", gp = grid::gpar(fontsize = 14))
# 
# # Query current settings
# latex_options()
# 
# # Reset to built-in defaults
# reset_latex_options()

## ----macros, fig.height=0.7, fig.width=3, out.width="50%"---------------------
define_macro("RR", "\\mathbb{R}")
define_macro("eps", "\\varepsilon")

grid::grid.newpage()
grid.latex("\\forall \\eps > 0, \\eps \\in \\RR", gp = grid::gpar(fontsize = 24))

clear_macros()

## ----cache, eval=FALSE--------------------------------------------------------
# latex_cache_info()       # size / max_size / hits / misses
# latex_cache_limit(1024)  # raise or lower the LRU capacity
# latex_cache_clear()      # wipe the cache (e.g. after re-loading fonts)

## ----tree---------------------------------------------------------------------
tr <- latex_tree("\\frac{a}{b}")
tr
head(tr$records, 3)

## ----debug, fig.height=1, fig.width=3, out.width="60%"------------------------
grid::grid.newpage()
grid.latex("x^{2} + y_{i}", gp = grid::gpar(fontsize = 30), debug = TRUE)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4,
  dpi = 300,
  dev = "png"
)
library(gridmicrotex)
library(ggplot2)

## ----geom-basic, out.width="70%"----------------------------------------------
df <- data.frame(
  x = 1:3,
  y = 1:3,
  eq = c(r"($x^2$)", r"(\frac{a}{b})", r"($\sum_{i=1}^n x_i$)"),
  col = c("red", "blue", "green")
)

ggplot(df, aes(x, y, 
               label = eq, 
               colour = col, 
               size = c(14, 18, 14))) +
  geom_latex() +
  scale_colour_identity() +
  scale_size_identity() +
  labs(
    x = r"($\beta_1 \cdot x + \beta_0$)",
    y = r"($\mathrm{mpg}$)"
  ) +
  theme(
    axis.title.x = element_latex(fontsize = 14),
    axis.title.y = element_latex(fontsize = 14)
  )

## ----regression-annotation, out.width="70%"-----------------------------------
fit <- lm(mpg ~ wt, data = mtcars)
b0 <- round(coef(fit)[1], 1)
b1 <- round(coef(fit)[2], 1)
r2 <- round(summary(fit)$r.squared, 3)

eq_label <- sprintf(r"($\hat{y} = %s %s x, \quad R^2 = %s$)", b0, b1, r2)

ggplot(mtcars, aes(wt, mpg)) +
  geom_point() +
  geom_smooth(method = "lm", se = FALSE) +
  annotate("latex", x = 4, y = 30, label = eq_label, size = 12) +
  theme_minimal()

## ----markdown, out.width="70%"------------------------------------------------
df <- data.frame(
  x   = 1:3,
  y   = c(2, 3, 1),
  lab = c("**bold**", r"(*slope* $\beta_1$)", "`code` and $x^2$")
)

ggplot(df, aes(x, y, label = lab)) +
  geom_point() +
  geom_markdown(fontsize = 14, vjust = -0.6) +
  ylim(0.5, 3.6) +
  labs(
    title = r"(*Fitted* model: $\hat{y} = \beta_0 + \beta_1 x$)",
    x     = "**weight** in $10^3$ lbs",
    y     = r"(*efficiency* $\eta$)"
  ) +
  theme(
    plot.title   = element_markdown(fontsize = 14),
    axis.title.x = element_markdown(),
    axis.title.y = element_markdown()
  )

## ----markdown-annotation, out.width="70%"-------------------------------------
fit <- lm(mpg ~ wt, data = mtcars)

note <- sprintf(
  r"(**Linear fit**<br>$\hat{y} = %s %s x$<br>$R^2 = %s$, *p* < 0.001)",
  round(coef(fit)[1], 1),
  round(coef(fit)[2], 1),
  round(summary(fit)$r.squared, 3)
)

ggplot(mtcars, aes(wt, mpg)) +
  geom_point(colour = "grey65") +
  geom_smooth(method = "lm", se = FALSE, colour = "#1F6FB2") +
  annotate("markdown", x = 4.1, y = 32, label = note, size = 11,
           style = "strong { color: #B22222 }") +
  theme_minimal()

## ----style, out.width="70%"---------------------------------------------------
css <- "
  body   { color: #33475B }
  strong { color: #B22222 }
  code   { color: #1F6FB2 }
  .unit  { color: grey55; font-size: smaller }
"

df <- data.frame(
  x = 1:3, y = c(2, 3, 1),
  lab = c("**bold** is red", "`code` is blue",
          'plain <span class="unit">with a note</span>')
)

ggplot(df, aes(x, y, label = lab)) +
  geom_point() +
  geom_markdown(style = css, fontsize = 14, vjust = -0.8) +
  ylim(0.5, 3.8) +
  labs(title = r"(**Styled** labels and $\beta_1$)",
       x = 'weight <span class="unit">(1000 lbs)</span>') +
  theme(
    plot.title   = element_markdown(style = css, fontsize = 14),
    axis.title.x = element_markdown(style = css)
  )

## ----blocktitle, out.width="70%"----------------------------------------------
ggplot(mtcars, aes(wt, mpg)) +
  geom_point(colour = "grey40") +
  labs(title = r"(## Fuel economy falls with weight

- slope $\beta_1 = -5.34$, *p* < 0.001
- $R^2 = 0.75$ over $n = 32$ cars)") +
  theme(plot.title = element_markdown(style = "
    body { background: #F4F7FB; padding: 10px;
           border: 1px solid #C7D6E5; border-radius: 4px }
  "))

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dpi = 300,
  dev = "png"
)

## ----basic, fig.height=1, fig.width=2.4, out.width="45%"----------------------
library(gridmicrotex)
library(grid)

grid.newpage()
grid.latex(r"($\frac{\textcolor{red}{-b} \pm \sqrt{b^2 - 4ac}}{2a}$)")

## ----modes, fig.height=1.2, fig.width=3.2, out.width="45%"--------------------
grid.newpage()
grid.latex(r"(Famous: $E = mc^2$)",
           x = 0.05, y = 0.7, hjust = 0, )
grid.latex(r"(\text{Famous: } E = mc^2)", input_mode = "math",
           x = 0.05, y = 0.3, hjust = 0, )

## ----showcase-table, fig.height = 3, fig.width = 8, out.width="70%"-----------
grid.newpage()
grid.latex(r"(
\newcolumntype{s}{>{\color{#1234B6}}c}
\begin{array}{|c|c|c|s|}
  \hline
  \rowcolor{Tan}\multicolumn{4}{|c|}{\textcolor{white}{\bold{\text{Table Head}}}}\\
  \hline
  \text{Matrix}&\multicolumn{2}{|c|}{\text{Multicolumns}}&\text{Font size commands}\\
  \hline
  \begin{pmatrix}
      \alpha_{11}&\cdots&\alpha_{1n}\\
      \hdotsfor{3}\\
      \alpha_{n1}&\cdots&\alpha_{nn}
  \end{pmatrix}
  &\large \text{Left}&\cellcolor{#00bde5}\small \textcolor{white}{\text{\bold{Right}}}
  &\small \text{small Small}\\
  \hline
  \multicolumn{4}{|c|}{\text{Table Foot}}\\
  \hline
\end{array}
)", )

## ----showcase-equation, fig.height = 9, fig.width = 9, out.width="70%"--------
grid.newpage()
grid.latex(r"(
\definecolor{gris}{gray}{0.9}
\definecolor{noir}{rgb}{0,0,0}
\fatalIfCmdConflict{false}
\newcommand{\pa}{\left|}
\begin{array}{c}
  \LaTeX\\
  \begin{split}
      |I_2| &= \pa\int_0^T\psi(t)\left\{ u(a,t)-\int_{\gamma(t)}^a \frac{d\theta}{k} (\theta,t) \int_a^\theta c(\xi)
          u_t (\xi,t)\,d\xi\right\}dt\right|\\
      &\le C_6 \Bigg|\pa f \int_\Omega \pa\widetilde{S}^{-1,0}_{a,-}
          W_2(\Omega, \Gamma_1)\right|\ \right|\left| |u|\overset{\circ}{\to} W_2^{\widetilde{A}}(\Omega\Gamma_r,T)\right|\Bigg|\\
      &\\
      &\begin{pmatrix}
          \alpha&\beta&\gamma&\delta\\
          \aleph&\beth&\gimel&\daleth\\
          \mathfrak{A}&\mathfrak{B}&\mathfrak{C}&\mathfrak{D}\\
          \boldsymbol{\mathfrak{a}}&\boldsymbol{\mathfrak{b}}&\boldsymbol{\mathfrak{c}}&\boldsymbol{\mathfrak{d}}
      \end{pmatrix}
      \quad{(a+b)}^{\frac{n}{2}}=\sqrt{\sum_{k=0}^n\tbinom{n}{k}a^kb^{n-k}}\quad
          \Biggl(\biggl(\Bigl(\bigl(()\bigr)\Bigr)\biggr)\Biggr)\\
      &\forall\varepsilon\in\mathbb{R}_+^*\ \exists\eta>0\ |x-x_0|\leq\eta\Longrightarrow|f(x)-f(x_0)|\leq\varepsilon\\
      &\det
      \begin{bmatrix}
          a_{11}&a_{12}&\cdots&a_{1n}\\
          a_{21}&\ddots&&\vdots\\
          \vdots&&\ddots&\vdots\\
          a_{n1}&\cdots&\cdots&a_{nn}
      \end{bmatrix}
      \overset{\mathrm{def}}{=}\sum_{\sigma\in\mathfrak{S}_n}\varepsilon(\sigma)\prod_{k=1}^n a_{k\sigma(k)}\\
      &\Delta f(x,y)=\frac{\partial^2f}{\partial x^2}+\frac{\partial^2f}{\partial y^2}\qquad\qquad \fcolorbox{noir}{gris}
          {n!\underset{n\rightarrow+\infty}{\sim} {\left(\frac{n}{e}\right)}^n\sqrt{2\pi n}}\\
      &\sideset{_\alpha^\beta}{_\gamma^\delta}{
      \begin{pmatrix}
          a&b\\
          c&d
      \end{pmatrix}}
      \xrightarrow[T]{n\pm i-j}\sideset{^t}{}A\xleftarrow{\overrightarrow{u}\wedge\overrightarrow{v}}
          \underleftrightarrow{\iint_{\mathds{R}^2}e^{-\left(x^2+y^2\right)}\,\mathrm{d}x\mathrm{d}y}
  \end{split}\\
  \rotatebox{30}{\sum_{n=1}^{+\infty}}\quad\mbox{Mirror rorriM}\reflectbox{\mbox{Mirror rorriM}}
\end{array}
)", render_mode = "path")

## ----baseline-align, fig.height=1.1, fig.width=6, out.width="70%"-------------
grid.newpage()
y <- 0.5
grid.segments(unit(0, "npc"), unit(y, "npc"),
              unit(1, "npc"), unit(y, "npc"), gp = gpar(col = "grey80"))
grid.text("if ", x = 0.10, y = y, just = c(0, 0.5), gp = gpar(fontsize = 20))
grid.latex(r"($x \geq \sqrt{2\pi}$)",
           x = 0.22, y = y, hjust = "left", vjust = "baseline",
           gp = gpar(fontsize = 20))
grid.text(", then proceed.", x = 0.62, y = y, just = c(0, 0.5),
          gp = gpar(fontsize = 20))

## ----mark, fig.height=2.4, fig.width=5, out.width="70%"-----------------------
g <- latex_grob(r"($a^2 + b\mark{term}^2 \mark{equals}= c^2$)",
                x = 0.5, y = 0.4)
grid.newpage()
grid.draw(g)

# The "=" sign, pointed at from above.
mk_eq <- grobMark(g, "equals")
grid.segments(mk_eq$x, mk_eq$y + unit(15, "mm"),
              mk_eq$x, mk_eq$y + unit(3, "mm"),
              arrow = arrow(length = unit(2, "mm"), type = "closed"),
              gp = gpar(col = "red"))
grid.text("equals", x = mk_eq$x, y = mk_eq$y + unit(18, "mm"),
          gp = gpar(col = "red"))

# The b^2 term, from below -- the mark sits at the end of the term,
# including the superscript's smaller scale.
mk_bsq <- grobMark(g, "term")
grid.segments(mk_bsq$x - unit(6, "mm"), mk_bsq$y - unit(15, "mm"),
              mk_bsq$x - unit(2, "mm"), mk_bsq$y - unit(3, "mm"),
              arrow = arrow(length = unit(2, "mm"), type = "closed"),
              gp = gpar(col = "blue"))
grid.text("b² term", x = mk_bsq$x - unit(7, "mm"),
          y = mk_bsq$y - unit(18, "mm"), just = "right",
          gp = gpar(col = "blue"))

## ----style-options, fig.height=1.9, fig.width=6.5, out.width="100%"-----------
sum_expr <- r"(\sum_{i=1}^{n} \frac{x_i}{n})"
styles <- c("display", "text", "script", "scriptscript")
labels <- c('"display"  ($$...$$)', '"text"  ($...$)',
            '"script"', '"scriptscript"')

grid.newpage()
for (i in seq_along(styles)) {
  pushViewport(viewport(x = (i - 0.5) / 4, width = 1 / 4))
  grid.text(labels[i], y = 0.88, gp = gpar(cex = 0.75, fontface = "bold"))
  grid.latex(sum_expr, y = 0.42, input_mode = "math",
             tex_style = styles[i], gp = gpar(fontsize = 20))
  grid.rect(gp = gpar(col = "grey85", fill = NA))
  popViewport()
}

## ----wrap, fig.height = 2, fig.width = 4, out.width = "65%"-------------------
prose <- paste(rep(
  r"(The quick brown fox jumps over the lazy dog, and $x^2$ too.)", 3),
  collapse = " ")

grid.newpage()
pushViewport(viewport(layout = grid.layout(2, 1)))
pushViewport(viewport(layout.pos.row = 1))
grid.text("ragged (default)", x = 0.02, y = 0.98, hjust = 0, vjust = 1,
          gp = gpar(col = "grey40"))
grid.latex(prose, x = 0.02, y = 0.78, hjust = 0, vjust = 1,
           max_width = 3.6 * 72, gp = gpar(fontsize = 11))
popViewport()
pushViewport(viewport(layout.pos.row = 2))
grid.text("justified + optimal", x = 0.02, y = 0.98, hjust = 0, vjust = 1,
          gp = gpar(col = "grey40"))
grid.latex(prose, x = 0.02, y = 0.78, hjust = 0, vjust = 1,
           max_width = 3.6 * 72, justify = TRUE, line_break = "optimal",
           gp = gpar(fontsize = 11))
popViewport(2)

## ----img-basic, fig.height = 1.6, fig.width = 4, out.width = "60%"------------
fig <- tempfile(fileext = ".svg")
svglite::svglite(fig, width = 2, height = 1.2)
grid.newpage()
grid.circle(r = 0.35, gp = gpar(fill = "steelblue", col = NA))
dev.off()

grid.newpage()
grid.latex(sprintf(r"(\text{before }\includegraphics[width=1in]{%s}\text{ after})", fig),
           input_mode = "math", gp = gpar(fontsize = 20))

## ----img-raise, fig.height = 1.5, fig.width = 4.5, out.width = "70%"----------
icon <- sprintf(r"(\includegraphics[width=14bp]{%s})", fig)
grid.newpage()
grid.latex(sprintf(r"(\text{baseline }%s\text{, raised }\raisebox{4bp}{%s})",
                   icon, icon),
           input_mode = "math", gp = gpar(fontsize = 20))

## ----img-caption, fig.height = 1.4, fig.width = 3, out.width = "45%"----------
logo <- system.file("img", "Rlogo.png", package = "png")
grid.newpage()
grid.latex(sprintf(r"(\begin{array}{c}
  \includegraphics[width=0.6in]{%s}\\
  \caption{Figure 1: the R logo}
\end{array})", logo), gp = gpar(fontsize = 11))

## ----fonts-list---------------------------------------------------------------
available_math_fonts()

## ----fonts, fig.height=1.5, fig.width=3.4, out.width="60%"--------------------
formula <- r"(Theorem: $\int_0^1 f(x)\,dx \geq 0$)"

grid.newpage()
pushViewport(viewport(layout = grid.layout(2, 1)))
pushViewport(viewport(layout.pos.row = 1))
grid.latex(formula, gp = gpar(fontfamily = "sans"))
upViewport()
pushViewport(viewport(layout.pos.row = 2))
grid.latex(formula, math_font = "stix",
           gp = gpar(fontfamily = "serif"))
upViewport(2)

## ----gmfontfamily, fig.height=0.8, fig.width=4.5, out.width="85%", dev = "ragg_png", dev.args = list()----
grid.newpage()
grid.latex(
  r"(\textsf{sans \textrm{body} sans} \quad \gmfontfamily{mono}{mono})",
  gp = gpar(fontfamily = "serif")
)

## ----dims---------------------------------------------------------------------
latex_dims(r"(\frac{a}{b})", gp = gpar(fontsize = 20))

## ----options, eval=FALSE------------------------------------------------------
# latex_options(math_font = "stix", render_mode = "typeface")
# latex_options()        # query
# reset_latex_options()  # back to built-in defaults

## ----macros, fig.height=0.7, fig.width=3, out.width="50%"---------------------
define_macro("RR", r"(\mathbb{R})")
define_macro("eps", r"(\varepsilon)")

grid.newpage()
grid.latex(r"(\forall \eps > 0, \eps \in \RR)")

clear_macros()

## ----def-inline, fig.height=0.7, fig.width=4, out.width="60%"-----------------
grid.newpage()
grid.latex(
  r"(\def\norm#1{\left\lVert #1 \right\rVert}
      \norm{\vec{v}} = \sqrt{\langle \vec{v}, \vec{v} \rangle})"
)

## ----cache, eval=FALSE--------------------------------------------------------
# latex_cache_info()       # size / max_size / hits / misses
# latex_cache_limit(1024)  # LRU capacity; 0 disables caching
# latex_cache_clear()      # wipe (e.g. after re-loading fonts)

## ----debug, fig.height=1, fig.width=3, out.width="60%"------------------------
grid.newpage()
grid.latex(r"($x^{2} + y_{i}$)", debug = TRUE)

## ----lists, fig.height=1.4, fig.width=3, out.width="45%"----------------------
grid.newpage()
grid.latex(r"(\begin{enumerate}
  \item e^{i\pi} + 1 = 0
  \item \begin{itemize}
          \item \alpha \item \beta
        \end{itemize}
\end{enumerate})", gp = gpar(fontsize = 20))

## ----pasted, fig.height = 1.1, fig.width = 3.2, out.width = "55%"-------------
snippet <- r"(
% latex table generated by kable()
\begin{table}[ht]
\centering
\caption{Model coefficients}
\begin{tabular}{lrr}
\toprule
Term & Estimate & \emph{p} \\
\midrule
Intercept & 2.14 & 0.003 \\
Slope & 0.42 & 0.001 \\
\bottomrule
\end{tabular}
\end{table}
)"

grid.newpage()
grid.latex(snippet, input_mode = "mixed", gp = gpar(fontsize = 11))

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dpi = 300,
  dev = "png"
)
library(gridmicrotex)
library(grid)

## ----inline, fig.height = 0.8, fig.width = 6, out.width = "90%"---------------
grid.newpage()
grid.markdown(
  r"(The **fitted** $\hat{\beta} = (X^\top X)^{-1} X^\top y$ has ~~no~~ `se` = *0.42*.)",
  x = 0.02, hjust = 0, gp = gpar(fontsize = 13)
)

## ----html-css, fig.height = 1.3, fig.width = 6, out.width = "90%", dev = "ragg_png", dev.args = list()----
grid.newpage()
grid.markdown(
  r"(<span style="font-size:22pt;color:#2E5E8E">Model fit</span>,
     <span style="font-family:serif">serif</span> and
     <span style="font-family:monospace">mono</span>.<br><br>
     The <span style="color:#B22222">**residual**</span> for H<sub>0</sub>
     is <u>within</u> <mark>$2\sigma$</mark>.)",
  x = 0.02, y = 0.9, hjust = 0, vjust = 1, gp = gpar(fontsize = 13)
)

## ----block, fig.height = 4, fig.width = 4.6, out.width = "80%"----------------
md <- r"(
# Model summary

The slope is $\beta_1$ with *p* < 0.001, and this paragraph is long
enough that it wraps inside the column.

## Diagnostics

- residuals look **fine**
- $R^2 = 0.87$
- no influential points

~~~r
fit <- lm(y ~ x, data = d)  # refit
if (anyNA(d)) {
    stop("missing values")
}
~~~

> Assumptions were checked and hold.
)"

grid.newpage()
grid.draw(markdown_box_grob(
  md,
  width   = unit(4.6, "in"),
  padding = unit(10, "pt"),
  box_gp  = gpar(fill = "grey97", col = "grey40"),
  gp      = gpar(fontsize = 13),
  style = markdown_style(css = "pre { background: #ece2f0 }") # Code background
))

## ----lists-tables, fig.height = 2, fig.width = 4.4, out.width = "80%"---------
md <- r"(
### Checklist

- [x] fit the model
- [ ] write it up

Coefficients:

| term | $\beta$  | *p* |
|:-----|------------:|:---:|
| intercept | 30.1 | *** |
| slope | -5.3 | *** |
)"

grid.newpage()
grid.draw(markdown_box_grob(
  md,
  width   = unit(4.4, "in"),
  padding = unit(8, "pt"),
  gp      = gpar(fontsize = 13)
))

## ----image, fig.height = 1.8, fig.width = 4, out.width = "70%"----------------
# Any .png / .jpeg path works; this one ships with the png package.
img <- system.file("img", "Rlogo.png", package = "png")

md <- sprintf("
A figure with a caption below it.

![the R logo](%s)

The caption explains the figure.
", img)

grid.newpage()
grid.draw(markdown_box_grob(
  md, width = unit(4, "in"), padding = unit(8, "pt"),
  gp = gpar(fontsize = 13)
))

## ----style-css, fig.height = 1.2, fig.width = 5, out.width = "85%"------------
doc <- r"(
# Results

The slope is $\beta_1$ with *p* < 0.001.

> Worth a second look.
)"

grid.newpage()
grid.draw(markdown_box_grob(
  doc,
  width = unit(4.5, "in"), padding = unit(10, "pt"),
  style = "
    h1         { color: steelblue; font-size: 1.8rem }
    blockquote { color: grey40; border-left: 3px solid steelblue }
  ",
  gp = gpar(fontsize = 13)
))

## ----style-r, eval = FALSE----------------------------------------------------
# markdown_style(
#   h1         = md_style(color = "steelblue", font_size = 1.8),
#   blockquote = md_style(color = "grey40",
#                         border_left = "3px solid steelblue")
# )

## ----style-preset, eval = FALSE-----------------------------------------------
# markdown_style("github", h1 = md_style(color = "firebrick"))
# file.show(system.file("css", "github.css", package = "gridmicrotex"))

## ----style-div, fig.height = 1, fig.width = 5, out.width = "85%"--------------
doc <- '
Ordinary text.

<div class="note">

**Note.** This chunk is indented and set apart.

</div>

Ordinary text again.
'

grid.newpage()
grid.draw(markdown_box_grob(
  doc,
  width = unit(4.5, "in"), padding = unit(10, "pt"),
  style = ".note { color: grey35; padding-left: 1.5rem }",
  gp = gpar(fontsize = 13)
))

## ----style-table, fig.height = 2, fig.width = 5, out.width = "85%"------------
tbl <- r"(
| Term | Meaning |
|:-----|:--------|
| $\beta_1$ | the slope, which needs a good deal of room to explain |
| $\sigma$ | the residual standard deviation |
)"

grid.newpage()
grid.draw(markdown_box_grob(
  tbl,
  width = unit(4, "in"), padding = unit(8, "pt"),
  style = "
    table { table-layout: fixed; border-color: #D1D9E0 }
    th    { background: #F6F8FA }
    tr    { border-bottom: 1px solid #D1D9E0 }
  ",
  gp = gpar(fontsize = 13)
))

## ----latex-escape, fig.height = 2.4, fig.width = 4.6, out.width = "80%"-------
tbl <- r"($$\begin{array}{|l|c|c|}\hline
\rowcolor{#F6F8FA}\multicolumn{3}{|c|}{\textbf{Model comparison}}\\\hline
\textbf{Model}&\textbf{AIC}&\textbf{R}^2\\\hline
\text{linear}&214.3&0.71\\
\cellcolor{#DDF0DD}\text{quadratic}&\cellcolor{#DDF0DD}201.8&\cellcolor{#DDF0DD}0.87\\\hline
\end{array}$$)"

grid.newpage()
grid.draw(markdown_box_grob(
  paste0("Compare the two fits.\n\n", tbl, "\n\nThe quadratic wins."),
  width = unit(4.6, "in"), padding = unit(10, "pt"),
  style = "math { color: #1F3864; font-size: 1.1rem }",
  gp = gpar(fontsize = 13)
))

## ----degrade, fig.height = 1.5, fig.width = 4.4, out.width = "80%"------------
doc <- r"(A [link](https://example.org) keeps its text, an
![inline diagram](missing.png) becomes its alt text, and an
<unknown>unknown tag</unknown> keeps its content.

Footnotes[^1] land at the foot.

<table><tr><td>an HTML table is dropped</td></tr></table>

[^1]: Like this one.)"

grid.newpage()
grid.draw(markdown_box_grob(
  doc,
  width = unit(4.4, "in"), padding = unit(8, "pt"),
  gp = gpar(fontsize = 13)
))

.latex_macros <- new.env(parent = emptyenv())
.latex_macros$defs <- list()

#' Define a user-level LaTeX macro
#'
#' Registers a zero-argument shorthand that is expanded by text
#' substitution before the expression reaches the MicroTeX parser.
#' Useful for domain-specific notation (e.g. \code{\\RR} for
#' \code{\\mathbb\{R\}}) you reuse across many plots.
#'
#' @section Choosing between this and \code{\\newcommand}:
#' MicroTeX also accepts \code{\\newcommand} and plain-TeX \code{\\def}
#' written inside the expression itself, and those are the more capable
#' form: they take up to nine arguments, which \code{define_macro()} does
#' not --- it substitutes text and nothing else.
#'
#' \preformatted{
#'   # parameterised, but local to this one expression
#'   grid.latex(r"(\\def\\norm#1{\\left\\lVert #1 \\right\\rVert}
#'                 \\norm{\\vec{v}})")
#' }
#'
#' What they cannot do is persist: the user-macro table is cleared at the
#' start of every parse, so a \code{\\newcommand} written in one call is
#' gone by the next. That is the one thing \code{define_macro()} is for.
#' Use \code{\\newcommand} / \code{\\def} for an abbreviation local to a
#' single label, and \code{define_macro()} for notation you want available
#' to every label in a script.
#'
#' @param name Macro name \strong{without} the leading backslash. For
#'   \code{clear_macros}, the macro name to drop, or \code{NULL}
#'   (default) to clear all.
#' @param definition LaTeX source the macro expands to.
#' @return
#' \itemize{
#'   \item \code{define_macro}: Invisibly returns \code{NULL}.
#'   \item \code{clear_macros}: Invisibly returns \code{NULL}.
#'   \item \code{list_macros}: A named character vector mapping
#'     macro names to their expansions. Empty if no macros are defined.
#' }
#' @seealso \code{\link{latex_grob}}, \code{\link{latex_options}}
#' @export
#'
#' @examples
#' \donttest{
#'   define_macro("RR", "\\mathbb{R}")
#'   define_macro("eps", "\\varepsilon")
#'   grid::grid.newpage()
#'   grid.latex("\\forall \\eps > 0, \\eps \\in \\RR")
#'   clear_macros()
#' }
define_macro <- function(name, definition) {
  stopifnot(is.character(name), length(name) == 1L, nzchar(name))
  stopifnot(is.character(definition), length(definition) == 1L)
  if (grepl("[^A-Za-z]", name)) {
    stop("Macro name must contain only letters (ASCII a-z, A-Z).",
         call. = FALSE)
  }
  .latex_macros$defs[[name]] <- definition
  invisible(NULL)
}

#' @rdname define_macro
#' @export
clear_macros <- function(name = NULL) {
  if (is.null(name)) {
    .latex_macros$defs <- list()
  } else {
    .latex_macros$defs[[name]] <- NULL
  }
  invisible(NULL)
}


#' @rdname define_macro
#' @export
list_macros <- function() {
  defs <- .latex_macros$defs
  if (length(defs) == 0L) return(character(0))
  unlist(defs)
}

# Apply macro expansion to a tex string. Matches \name where the next
# character is not a letter (so `\RR` matches but `\RRend` does not),
# in iteration until a fixed point so nested macros expand.
.expand_macros <- function(tex) {
  defs <- .latex_macros$defs
  if (length(defs) == 0L) return(tex)
  patterns <- paste0("\\\\", names(defs), "(?![A-Za-z])")
  for (depth in seq_len(8L)) {
    before <- tex
    for (i in seq_along(defs)) {
      # Escape backslashes in the replacement so gsub treats them
      # literally rather than as backreference escapes.
      replacement <- gsub("\\\\", "\\\\\\\\", defs[[i]])
      tex <- gsub(patterns[i], replacement, tex, perl = TRUE)
    }
    if (identical(tex, before)) break
  }
  # A defined macro token still present means expansion never resolved --
  # either a circular definition (\a -> \b -> \a) or one too deep for the
  # 8-iteration cap. Either way the output is wrong, so warn.
  if (any(vapply(patterns, grepl, logical(1), x = tex, perl = TRUE))) {
    warning(
      "Macro expansion did not resolve; check for circular macro ",
      "definitions.",
      call. = FALSE
    )
  }
  tex
}

test_that(".expand_macros expands nested macros to a fixed point", {
  on.exit(clear_macros(), add = TRUE)
  define_macro("RR", "\\mathbb{R}")
  define_macro("dom", "\\RR \\times \\RR")
  expect_equal(.expand_macros("\\dom"), "\\mathbb{R} \\times \\mathbb{R}")
})

test_that(".expand_macros warns on circular definitions", {
  on.exit(clear_macros(), add = TRUE)
  define_macro("a", "\\b")
  define_macro("b", "\\a")
  expect_warning(.expand_macros("\\a"), "circular macro")
})

# Null-coalescing operator. Defined here so it's available to every R
# file in the package (R loads files alphabetically by default, and
# `o` sorts ahead of the files that use it).
`%||%` <- function(x, y) if (is.null(x)) y else x

.latex_options <- new.env(parent = emptyenv())
.latex_options$values <- list(
  math_font   = NULL,
  render_mode = NULL,
  tex_style   = NULL,
  input_mode  = NULL,
  justify     = NULL,
  line_break  = NULL,
  markdown_style = NULL
)

# Validate the `justify` argument. Kept here so latex_grob(),
# latex_dims() and latex_options() all reject the same things.
.check_justify <- function(justify) {
  if (!is.logical(justify) || length(justify) != 1L || is.na(justify)) {
    stop("`justify` must be TRUE or FALSE.", call. = FALSE)
  }
  invisible(TRUE)
}

#' Set or query package-wide LaTeX rendering defaults
#'
#' A single entry point for project-wide defaults used by
#' \code{\link{latex_grob}}, \code{\link{grid.latex}},
#' \code{\link{latex_dims}}, and \code{\link{latex_tree}}. Options set
#' here are applied only when the corresponding argument is \emph{not}
#' supplied at the call site, so explicit arguments always win.
#'
#' Calling \code{latex_options()} with no arguments returns the current
#' settings (a list whose \code{NULL} entries mean "use the built-in
#' default"). Supply one or more named arguments to update them.
#'
#' Font size and line spacing are controlled via \code{gp} parameters
#' (\code{fontsize}, \code{cex}, \code{lineheight}) at the grob level
#' --- see \code{\link{latex_grob}}.
#'
#' @param math_font Math font name or alias (see
#'   \code{\link{available_math_fonts}}).
#' @param render_mode Either \code{"typeface"} or \code{"path"}.
#' @param tex_style TeX style override. One of \code{""} (let the parser
#'   decide), \code{"display"}, \code{"text"}, \code{"script"}, or
#'   \code{"scriptscript"}. \code{"display"} forces large operators with
#'   limits placed over/under, useful for inline labels that should still
#'   look like display equations.
#' @param input_mode How the input string is interpreted before being
#'   handed to MicroTeX. \code{"mixed"} (default) wraps the string in
#'   \code{\\text{...}} so it reads as ordinary text, with \code{$...$}
#'   (and \code{\\(...\\)}) opening math mode --- the document-level
#'   LaTeX convention. Useful when consuming labels from other packages
#'   that mix prose and math without explicit \code{\\text{}} markers.
#'   \code{"math"} treats the whole string as math --- the classic
#'   MicroTeX behaviour, where letters render as math italics and
#'   unwrapped prose looks wrong.
#' @param justify Logical. When \code{TRUE}, wrapped text is stretched at
#'   its interword spaces so every line but the last fills
#'   \code{max_width} exactly. Has no effect without \code{max_width},
#'   since it acts on the lines the wrapper produces. \code{FALSE}
#'   (default) leaves the right edge ragged, matching R's own text
#'   drawing. In a narrow column, justifying alone opens noticeably wide
#'   word spaces; mark the words that may break with \code{\\-}.
#' @param line_break How lines are chosen when wrapping.
#'   \code{"greedy"} (default) fills each line as far as it will go and
#'   never reconsiders. \code{"optimal"} chooses the breaks together so
#'   the paragraph as a whole reads best, in the spirit of Knuth-Plass:
#'   pulling one word down early can improve every later line, which a
#'   greedy pass cannot see. Requires \code{max_width}, and costs a
#'   little more layout time.
#' @param markdown_style Default style for \code{\link{markdown_grob}} and
#'   \code{\link{markdown_box_grob}}: a \code{\link{markdown_style}}
#'   object, CSS text, or a path to a \code{.css} file.
#' @return Invisibly returns the previous settings (a list). With no
#'   arguments, returns the current settings visibly.
#' @seealso \code{\link{available_math_fonts}}, \code{\link{latex_grob}}
#' @export
#'
#' @examples
#' \donttest{
#'   latex_options(math_font = "stix", render_mode = "typeface")
#'   grid.latex("\\sum_{i=1}^{n} i^{2}", gp = grid::gpar(fontsize = 14))
#'   reset_latex_options()
#' }
latex_options <- function(math_font = NULL, render_mode = NULL,
                          tex_style = NULL, input_mode = NULL,
                          justify = NULL, line_break = NULL,
                          markdown_style = NULL) {
  if (nargs() == 0L) {
    return(as.list(.latex_options$values))
  }

  old <- as.list(.latex_options$values)

  if (!is.null(math_font)) {
    stopifnot(is.character(math_font), length(math_font) == 1L)
    .set_math_font(math_font)
    .latex_options$values$math_font <- math_font
  }
  if (!is.null(render_mode)) {
    render_mode <- match.arg(render_mode, c("typeface", "path"))
    .latex_options$values$render_mode <- render_mode
  }
  if (!is.null(tex_style)) {
    .check_tex_style(tex_style)
    .latex_options$values$tex_style <- tex_style
  }
  if (!is.null(input_mode)) {
    input_mode <- match.arg(input_mode, c("math", "mixed"))
    .latex_options$values$input_mode <- input_mode
  }
  if (!is.null(justify)) {
    .check_justify(justify)
    .latex_options$values$justify <- justify
  }
  if (!is.null(line_break)) {
    line_break <- match.arg(line_break, c("greedy", "optimal"))
    .latex_options$values$line_break <- line_break
  }
  if (!is.null(markdown_style)) {
    # Coerce here rather than at each use, so a bad value is rejected by
    # the call that set it.
    .latex_options$values$markdown_style <- .md_as_style(markdown_style)
  }
  invisible(old)
}

#' @rdname latex_options
#'
#' @export
reset_latex_options <- function() {
  .latex_options$values <- list(
    math_font   = NULL,
    render_mode = NULL,
    tex_style   = NULL,
    input_mode  = NULL,
    justify     = NULL,
    line_break  = NULL,
    markdown_style = NULL
  )
  invisible(NULL)
}

# Internal: resolve an argument against latex_options().
.opt <- function(name) {
  .latex_options$values[[name]]
}

# Resolve named formal args of the calling function against
# latex_options(): any arg the caller did not supply explicitly is
# replaced (in the caller's frame) by .opt(<name>) when that option is
# set. Used by latex_grob(), latex_dims(), latex_tree() so the
# missing-arg-then-fallback pattern lives in one place.
.apply_opts <- function(...) {
  env <- parent.frame()
  for (n in c(...)) {
    if (eval(call("missing", as.name(n)), env)) {
      opt <- .opt(n)
      if (!is.null(opt)) assign(n, opt, envir = env)
    }
  }
}

# Internal: validate a tex_style value. Use exact matching (not match.arg)
# because "" is one of the valid choices and partial matching against an
# empty string is ambiguous.
.tex_style_choices <- c("", "display", "text", "script", "scriptscript")
.check_tex_style <- function(x) {
  stopifnot(is.character(x), length(x) == 1L)
  if (!x %in% .tex_style_choices) {
    stop(
      "tex_style must be one of: ",
      paste(sprintf("'%s'", .tex_style_choices), collapse = ", "),
      call. = FALSE
    )
  }
  x
}

# Line breaking in the vendored MicroTeX splitter.
#
# BoxSplitter::split() used to accept only an HBox and return anything
# else untouched, so the moment a formula carried an explicit line break
# its top box became a VBox and max_width was silently ignored -- the
# content just overflowed. These tests pin the fix.

long_text <- paste0("\\text{", paste(rep("longword", 12), collapse = " "), "}")
short_text <- "\\text{short}"

dims <- function(tex, max_width = 0) {
  d <- latex_dims(tex, max_width = max_width)
  list(w = as.numeric(d$width), h = as.numeric(d$height), split = d$is_split)
}

test_that("content separated by \\\\ honours max_width", {
  # Each of these previously came back at full natural width with
  # is_split = FALSE no matter how narrow max_width was.
  for (tex in c(paste0(short_text, "\\\\", long_text),
                paste0(long_text, "\\\\", long_text))) {
    wide <- dims(tex)
    narrow <- dims(tex, 200)
    expect_true(narrow$split)
    expect_lt(narrow$w, wide$w)
    expect_lte(narrow$w, 200)
    # Wrapping trades width for height.
    expect_gt(narrow$h, wide$h)
  }
})

test_that("a single flowing paragraph still wraps as it always did", {
  wide <- dims(long_text)
  narrow <- dims(long_text, 200)
  expect_true(narrow$split)
  expect_lte(narrow$w, 200)
})

test_that("content that already fits is returned untouched", {
  # The splitter must not rebuild a box it does not need to change:
  # identical width and height at any max_width.
  for (tex in c(short_text,
                "\\frac{a}{b}",
                "\\begin{matrix}a&b\\\\c&d\\end{matrix}",
                "\\begin{cases}a&b\\\\c&d\\end{cases}",
                "\\begin{pmatrix}1\\\\2\\end{pmatrix}")) {
    a <- dims(tex)
    b <- dims(tex, 200)
    expect_equal(b$w, a$w, tolerance = 1e-6)
    expect_equal(b$h, a$h, tolerance = 1e-6)
    expect_false(b$split)
  }
})

test_that("vertical centring on the math axis survives a rebuilt VBox", {
  # MatrixAtom overwrites the VBox height/depth to centre it on the math
  # axis. Rebuilding the box without restoring that offset shifts the
  # baseline of every multi-row formula, which no width assertion would
  # catch.
  for (tex in c("\\begin{matrix}a&b\\\\c&d\\end{matrix}",
                "\\begin{cases}x\\\\y\\end{cases}",
                "\\begin{pmatrix}1\\\\2\\end{pmatrix}")) {
    plain <- latex_dims(tex)
    limited <- latex_dims(tex, max_width = 200)
    expect_equal(limited$baseline, plain$baseline, tolerance = 1e-6)
    expect_equal(as.numeric(limited$depth), as.numeric(plain$depth),
                 tolerance = 1e-6)
  }
})

test_that("splitting a wrapped paragraph keeps the text on the page", {
  # Guards the rebuild: every drawn record must sit inside the reported
  # bounding box. A botched height/depth fix-up shows up here as content
  # drawn outside its own box.
  g <- latex_grob(paste0(short_text, "\\\\", long_text), max_width = 200)
  expect_true(g$is_split)
  expect_lte(max(g$layout_df$x, na.rm = TRUE), g$bbox_w + 1)
  expect_gte(min(g$layout_df$x, na.rm = TRUE), -1)
})

test_that("list and matrix cells do not wrap (documented limitation)", {
  # Long items inside itemize/enumerate/align live in WrapperBoxes whose
  # metrics are the row's, precomputed by MatrixAtom. The splitter
  # deliberately does not reach into them. If this ever starts passing,
  # the cell-splitting work landed and R/markdown.R can stop stacking
  # list items itself.
  for (tex in c(paste0("\\begin{itemize}\\item ", long_text, "\\end{itemize}"),
                paste0("\\begin{enumerate}\\item ", long_text, "\\end{enumerate}"))) {
    narrow <- dims(tex, 200)
    expect_false(narrow$split)
  }
})

# --- justification -----------------------------------------------------
#
# Interword spaces used to be rigid StrutBoxes, so a line had nothing to
# stretch. They are GlueBoxes now, carrying TeX's half-space stretch;
# nothing reads that unless justify = TRUE, so ragged output is
# unchanged (the visual snapshots are the wider guard on that).

just_para <- paste0(
  "\\text{",
  paste(rep("The quick brown fox jumps over the lazy dog.", 4), collapse = " "),
  "}"
)

# Right edge of each rendered line.
#
# A text record carries no width of its own. That used to make its origin
# a fair proxy for the line's right edge, because every record held a
# single character. It no longer is: consecutive characters are drawn as
# one word-level record (RowAtom::processTextRun), so the last origin on
# a line sits a whole word short of the edge, by a different amount on
# every line. Measure the glyphs the record actually holds instead.
.rec_width <- function(txt, size) {
  grid::pushViewport(grid::viewport(gp = grid::gpar(fontsize = size)))
  on.exit(grid::popViewport())
  grid::convertWidth(grid::stringWidth(txt), "bigpts", valueOnly = TRUE)
}

line_edges <- function(g) {
  df <- g$layout_df
  ys <- sort(unique(round(df$y, 1)))
  vapply(ys, function(yy) {
    d <- df[abs(round(df$y, 1) - yy) < 0.01, , drop = FALSE]
    w <- vapply(seq_len(nrow(d)), function(i) {
      if (!is.na(d$width[i])) return(d$width[i])
      if (!identical(d$type[i], "text") || is.na(d$text[i])) return(0)
      .rec_width(d$text[i], d$font_size[i])
    }, numeric(1))
    max(d$x + w)
  }, numeric(1))
}

test_that("justified text fills the measure exactly", {
  for (w in c(200, 300, 400)) {
    d <- latex_dims(just_para, max_width = w, justify = TRUE)
    expect_equal(as.numeric(d$width), w, tolerance = 1e-6)
  }
})

test_that("justification evens the lines out", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g_just <- latex_grob(just_para, max_width = 300, justify = TRUE)
  g_rag  <- latex_grob(just_para, max_width = 300, justify = FALSE)
  # Drop the last line: it is deliberately left ragged.
  e_just <- head(line_edges(g_just), -1)
  e_rag  <- head(line_edges(g_rag), -1)
  expect_gt(length(e_just), 2)
  # Justified lines end at nearly the same place; ragged ones do not.
  expect_lt(diff(range(e_just)), 0.25 * diff(range(e_rag)))
})

test_that("the last line stays ragged", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g <- latex_grob(just_para, max_width = 300, justify = TRUE)
  e <- line_edges(g)
  expect_lt(e[length(e)], min(head(e, -1)) - 10)
})

test_that("justification does not change how tall the text is", {
  for (w in c(200, 300)) {
    r <- latex_dims(just_para, max_width = w, justify = FALSE)
    j <- latex_dims(just_para, max_width = w, justify = TRUE)
    expect_equal(as.numeric(j$height), as.numeric(r$height), tolerance = 1e-6)
    expect_equal(j$is_split, r$is_split)
  }
})

test_that("justify does nothing without max_width", {
  # It acts on the lines the wrapper produces, so with nothing to wrap
  # there is nothing to justify.
  a <- latex_dims(just_para, justify = FALSE)
  b <- latex_dims(just_para, justify = TRUE)
  expect_equal(as.numeric(b$width), as.numeric(a$width), tolerance = 1e-6)
})

test_that("the justify flag does not leak into later parses", {
  # It is a global on the splitter, guarded in C++; a leak would justify
  # every subsequent render in the session.
  before <- as.numeric(latex_dims(just_para, max_width = 300)$width)
  invisible(latex_dims(just_para, max_width = 300, justify = TRUE))
  after <- as.numeric(latex_dims(just_para, max_width = 300)$width)
  expect_equal(after, before, tolerance = 1e-6)
  expect_lt(after, 300)  # still ragged
})

test_that("justified and ragged layouts are cached separately", {
  # The cache key must include justify, or the second call returns the
  # first one's layout.
  latex_cache_clear()
  r <- as.numeric(latex_dims(just_para, max_width = 300, justify = FALSE)$width)
  j <- as.numeric(latex_dims(just_para, max_width = 300, justify = TRUE)$width)
  expect_lt(r, j)
  expect_equal(j, 300, tolerance = 1e-6)
})

test_that("justify is validated and settable as an option", {
  expect_error(latex_dims("x", justify = "yes"), "TRUE or FALSE")
  expect_error(latex_dims("x", justify = NA), "TRUE or FALSE")
  expect_error(latex_dims("x", justify = c(TRUE, FALSE)), "TRUE or FALSE")

  latex_options(justify = TRUE)
  expect_true(latex_options()$justify)
  expect_equal(as.numeric(latex_dims(just_para, max_width = 300)$width), 300,
               tolerance = 1e-6)
  reset_latex_options()
  expect_null(latex_options()$justify)
})

test_that("a line of one long word is left alone", {
  solid <- paste0("\\text{", strrep("x", 80), "}")
  plain <- as.numeric(latex_dims(solid, max_width = 100)$width)
  just  <- as.numeric(latex_dims(solid, max_width = 100, justify = TRUE)$width)
  # There is no interword glue to stretch, so justification must leave the
  # line exactly as it was -- and it overruns the measure rather than
  # being padded out to it.
  expect_equal(just, plain)
  expect_gt(just, 100)
})

# --- optimal (total-fit) line breaking ---------------------------------
#
# The default splitter is greedy: it fills each line as far as it can and
# never reconsiders. "optimal" chooses the breaks together, minimising
# summed TeX-style demerits, so pulling a word down early can improve
# every later line.

opt_para <- paste0(
  "\\text{",
  paste(rep("The quick brown fox jumps over the lazy dog.", 4), collapse = " "),
  "}"
)

opt_edges <- function(g) {
  df <- g$layout_df
  ys <- sort(unique(round(df$y, 1)))
  vapply(ys, function(yy) {
    s <- abs(round(df$y, 1) - yy) < 0.01
    max(df$x[s] + ifelse(is.na(df$width[s]), 0, df$width[s]))
  }, numeric(1))
}

test_that("greedy remains the default and is untouched", {
  for (w in c(200, 260, 300)) {
    a <- latex_dims(opt_para, max_width = w)
    b <- latex_dims(opt_para, max_width = w, line_break = "greedy")
    expect_equal(as.numeric(a$width), as.numeric(b$width), tolerance = 1e-6)
    expect_equal(as.numeric(a$height), as.numeric(b$height), tolerance = 1e-6)
  }
})

test_that("optimal never overflows more than greedy does", {
  # canBreak() can settle on a first line wider than the measure for text
  # spread over several runs. Total fit picks among the breaks canBreak
  # offers, so it inherits that but must never make it worse.
  for (w in seq(140, 360, by = 20)) {
    g <- as.numeric(latex_dims(opt_para, max_width = w,
                               line_break = "greedy")$width)
    o <- as.numeric(latex_dims(opt_para, max_width = w,
                               line_break = "optimal")$width)
    expect_lte(o, max(g, w) + 0.51)
  }
})

test_that("optimal does not make the paragraph taller", {
  # The per-line penalty in the demerits stops it buying evenness by
  # adding a line.
  for (w in c(180, 220, 260, 300, 340)) {
    g <- as.numeric(latex_dims(opt_para, max_width = w,
                               line_break = "greedy")$height)
    o <- as.numeric(latex_dims(opt_para, max_width = w,
                               line_break = "optimal")$height)
    expect_lte(o, g + 0.51)
  }
})

test_that("optimal is at least as even as greedy", {
  spread <- function(g) {
    e <- head(opt_edges(g), -1)   # last line is deliberately short
    if (length(e) < 2) return(0)
    stats::sd(e)
  }
  for (w in c(220, 260, 300, 340)) {
    sg <- spread(latex_grob(opt_para, max_width = w, line_break = "greedy"))
    so <- spread(latex_grob(opt_para, max_width = w, line_break = "optimal"))
    expect_lte(so, sg + 1e-6)
  }
})

test_that("optimal measurably improves at least one width", {
  # Pins the feature actually doing something, so a regression that
  # silently disabled it would be caught.
  g <- latex_grob(opt_para, max_width = 300, line_break = "greedy")
  o <- latex_grob(opt_para, max_width = 300, line_break = "optimal")
  expect_lt(stats::sd(head(opt_edges(o), -1)),
            stats::sd(head(opt_edges(g), -1)))
})

test_that("optimal composes with justify", {
  g <- latex_grob(opt_para, max_width = 300,
                  line_break = "optimal", justify = TRUE)
  expect_equal(g$bbox_w, 300, tolerance = 1e-6)
})

test_that("line_break is validated, settable and does not leak", {
  expect_error(latex_dims("x", line_break = "nope"))

  before <- as.numeric(latex_dims(opt_para, max_width = 300)$width)
  invisible(latex_dims(opt_para, max_width = 300, line_break = "optimal"))
  expect_equal(as.numeric(latex_dims(opt_para, max_width = 300)$width),
               before, tolerance = 1e-6)

  latex_options(line_break = "optimal")
  expect_equal(latex_options()$line_break, "optimal")
  reset_latex_options()
  expect_null(latex_options()$line_break)
})

test_that("greedy and optimal layouts are cached separately", {
  latex_cache_clear()
  a <- latex_dims(opt_para, max_width = 300, line_break = "greedy")
  b <- latex_dims(opt_para, max_width = 300, line_break = "optimal")
  # Same measure, but the two must not share a cache entry: differing
  # line shape at equal width is exactly the collision to avoid.
  expect_false(identical(a$is_split, NA))
  expect_equal(latex_cache_info()$size, 2L)
})

test_that("content with nothing to break is unaffected", {
  solid <- paste0("\\text{", strrep("x", 60), "}")
  expect_equal(
    as.numeric(latex_dims(solid, max_width = 100, line_break = "optimal")$width),
    as.numeric(latex_dims(solid, max_width = 100, line_break = "greedy")$width),
    tolerance = 1e-6
  )
})

.onLoad <- function(libname, pkgname) {
  # Force-load systemfonts here: our C++ OTF parser calls its
  # get_cached_face via R_GetCCallable at init time, and that lookup
  # only succeeds once systemfonts' DLL init has run. Relying on the
  # Imports declaration alone isn't sufficient on all R versions.
  requireNamespace("systemfonts", quietly = TRUE)

  # Initialize MicroTeX with Lete Sans Math as the default font --- it pairs
  # naturally with R's sans-serif default for plot text. The CLM metrics
  # are synthesised in C++ from the OTF's OpenType MATH table at load
  # time; no companion .clm2 file is shipped.
  otf_path <- system.file("fonts", "LeteSansMath.otf", package = pkgname)

  if (nchar(otf_path) == 0) {
    warning("gridmicrotex: Lete Sans Math font file not found, LaTeX rendering will not work")
    return()
  }

  microtex_init_from_otf(otf_path)

  # Add the bundled STIX Two Math font to MicroTeX's registry. The
  # systemfonts registration (so gp$fontfamily = "lete"/"stix" resolves
  # to the bundled OTF) is deferred to first render — see
  # .ensure_bundled_fonts_registered() in R/fonts.R for why.
  stix_path <- system.file("fonts", "STIXTwoMath-Regular.otf", package = pkgname)
  if (nzchar(stix_path)) {
    microtex_add_font_from_otf(stix_path, 0L)
  }

  # Initialize ggplot2 integration if ggplot2 is available
  .onLoad_ggplot2()
}

.onAttach <- function(libname, pkgname) {
  # Warn only if the bundled default (Lete) is missing.
  if (!"Lete Sans Math" %in% microtex_math_font_names()) {
    packageStartupMessage(
      "gridmicrotex: bundled default math font (Lete Sans Math) failed ",
      "to register. Run check_math_fonts() for details."
    )
  }
}

.onUnload <- function(libpath) {
  microtex_release()
  # Hand the shared object back as well. R does not do this for us, and
  # R_unload_gridmicrotex() (src/init.cpp) -- which frees the macro
  # registries and releases the preserved text-measurer callback -- runs
  # from nowhere else.
  library.dynam.unload("gridmicrotex", libpath)
}

#' Convert TeX points to big points
#'
#' TeX points (1/72.27 inch) differ from PostScript/big points (1/72 inch).
#' R's grid "bigpts" unit uses PostScript points.
#'
#' @param tex_pt Numeric value in TeX points.
#' @return Numeric value in big (PostScript) points.
#' @noRd
tex_pt_to_bigpt <- function(tex_pt) {
  tex_pt * (72 / 72.27)
}

#' Convert big points to TeX points
#'
#' @param big_pt Numeric value in PostScript points.
#' @return Numeric value in TeX points.
#' @noRd
bigpt_to_tex_pt <- function(big_pt) {
  big_pt * (72.27 / 72)
}

# Convert a grid-style lineheight multiplier to MicroTeX's line_space
# (extra gap between lines, in big points). Mirrors grid semantics:
# total line height = fontsize * lineheight.
.line_space_from_lineheight <- function(lineheight, fontsize) {
  if (is.null(lineheight)) lineheight <- 1.2
  if (is.null(fontsize) || fontsize <= 0) return(0)
  max(0, (lineheight - 1) * fontsize)
}

test_that("unit conversions are correct, vectorized, and invertible", {
  # Forward conversions
  expect_equal(gridmicrotex:::tex_pt_to_bigpt(1), 72 / 72.27, tolerance = 1e-6)
  expect_equal(gridmicrotex:::tex_pt_to_bigpt(0), 0)

  # Vectorized
  result <- gridmicrotex:::tex_pt_to_bigpt(c(72.27, 144.54))
  expect_equal(result, c(72, 144), tolerance = 1e-4)

  # Inverse round-trip
  expect_equal(
    gridmicrotex:::bigpt_to_tex_pt(gridmicrotex:::tex_pt_to_bigpt(100)),
    100, tolerance = 1e-9
  )
})

test_that("tex_pt_to_bigpt scales proportionally", {
  expect_equal(
    gridmicrotex:::tex_pt_to_bigpt(20),
    2 * gridmicrotex:::tex_pt_to_bigpt(10),
    tolerance = 1e-9
  )
})

# TrueType Collection (.ttc) → single-face sfnt extractor.
#
# systemfonts::match_fonts() on macOS can return a TTC (e.g. Helvetica.ttc)
# plus a face index. The C++ A3 font loader and the grid glyph renderer
# both want a single-face .otf/.ttf path; this helper splits out the
# requested face into a standalone sfnt in the cache dir on first use
# and returns the new path. Subsequent calls short-circuit to the cached
# file so the split cost is paid once per (ttc, mtime, size, index).
#
# All OpenType values are big-endian.

# Detect a TrueType Collection by its 'ttcf' magic. Cheap — reads 4 bytes.
.is_ttc_file <- function(path) {
  if (!file.exists(path)) return(FALSE)
  magic <- tryCatch(readBin(path, what = "raw", n = 4L), error = function(e) raw(0))
  length(magic) == 4L && identical(magic, charToRaw("ttcf"))
}

# Extract face `index` from a TrueType Collection into a freestanding
# single-face sfnt, written to the cache dir, and return the new path.
# Result is cached across sessions keyed on (path, mtime, size, index).
# The output is a valid single-face OTF/TTF: sfnt header + rewritten
# table directory (new local offsets) + each selected table's bytes
# copied verbatim.
.extract_ttc_face <- function(ttc_path, index = 0L) {
  if (!file.exists(ttc_path)) {
    stop("TTC file not found: ", ttc_path, call. = FALSE)
  }
  index <- as.integer(index)
  if (is.na(index) || index < 0L) index <- 0L

  cache_dir <- .ttc_cache_dir()
  if (!dir.exists(cache_dir)) {
    dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  }

  key <- .ttc_face_cache_key(ttc_path, index)
  base <- tools::file_path_sans_ext(basename(ttc_path))
  out_path <- file.path(cache_dir, paste0(base, "-face", index, "-", key, ".otf"))
  if (file.exists(out_path) && file.info(out_path)$size > 0L) {
    return(out_path)
  }

  bytes <- readBin(ttc_path, what = "raw", n = file.info(ttc_path)$size)
  if (length(bytes) < 16L || !identical(bytes[1:4], charToRaw("ttcf"))) {
    stop("Not a TrueType Collection: ", ttc_path, call. = FALSE)
  }

  # TTC header: tag(4) + version(4) + numFonts(u32) + offsets[numFonts](u32).
  num_fonts <- .ru32(bytes, 9L)
  if (index >= num_fonts) {
    stop("TTC face index ", index, " out of range (collection has ",
         num_fonts, " face", if (num_fonts == 1) "" else "s", "): ",
         ttc_path, call. = FALSE)
  }
  face_off <- as.integer(.ru32(bytes, 13L + index * 4L)) + 1L  # R 1-based

  # sfnt header at face_off: sfntVersion(4), numTables(u16), then three u16
  # search fields we recompute.
  sfnt_version <- bytes[face_off:(face_off + 3L)]
  num_tables <- .ru16(bytes, face_off + 4L)
  if (num_tables == 0L) {
    stop("TTC face ", index, " has no tables: ", ttc_path, call. = FALSE)
  }

  records <- vector("list", num_tables)
  rec_pos <- face_off + 12L
  for (i in seq_len(num_tables)) {
    records[[i]] <- list(
      tag      = bytes[rec_pos:(rec_pos + 3L)],
      checksum = bytes[(rec_pos + 4L):(rec_pos + 7L)],
      offset   = .ru32(bytes, rec_pos + 8L),
      length   = as.integer(.ru32(bytes, rec_pos + 12L))
    )
    rec_pos <- rec_pos + 16L
  }

  # Lay out the new file: sfnt header (12) + table directory (numTables*16)
  # + table bodies, each padded to a 4-byte boundary.
  pad4 <- function(n) (4L - (n %% 4L)) %% 4L
  header_size <- 12L + num_tables * 16L
  body_offsets <- integer(num_tables)
  cursor <- header_size
  for (i in seq_len(num_tables)) {
    body_offsets[i] <- cursor
    cursor <- cursor + records[[i]]$length + pad4(records[[i]]$length)
  }
  total_size <- cursor

  out <- raw(total_size)
  out[1:4] <- sfnt_version
  out[5:6] <- .w_u16(num_tables)
  es <- as.integer(floor(log2(num_tables)))
  sr <- bitwShiftL(1L, es) * 16L
  rs <- num_tables * 16L - sr
  out[7:8]   <- .w_u16(sr)
  out[9:10]  <- .w_u16(es)
  out[11:12] <- .w_u16(rs)

  dir_pos <- 13L
  for (i in seq_len(num_tables)) {
    rec <- records[[i]]
    off <- body_offsets[i]
    len <- rec$length

    out[dir_pos:(dir_pos + 3L)]         <- rec$tag
    out[(dir_pos + 4L):(dir_pos + 7L)]  <- rec$checksum
    out[(dir_pos + 8L):(dir_pos + 11L)] <- .w_u32(off)
    out[(dir_pos + 12L):(dir_pos + 15L)] <- .w_u32(len)
    dir_pos <- dir_pos + 16L

    if (len > 0L) {
      src_from <- as.integer(rec$offset) + 1L
      out[(off + 1L):(off + len)] <- bytes[src_from:(src_from + len - 1L)]
    }
  }

  tmp <- paste0(out_path, ".tmp")
  writeBin(out, tmp)
  file.rename(tmp, out_path)
  out_path
}

.ttc_face_cache_key <- function(ttc_path, index) {
  info <- file.info(ttc_path)
  hash_input <- paste(
    normalizePath(ttc_path, winslash = "/", mustWork = FALSE),
    sprintf("%.0f", as.numeric(info$mtime)),
    sprintf("%.0f", as.numeric(info$size)),
    as.integer(index),
    "ttcface-v1",
    sep = "|"
  )
  substr(.short_hash(hash_input), 1L, 12L)
}

.ttc_cache_dir <- function() {
  tools::R_user_dir("gridmicrotex", which = "cache")
}

# --- Low-level big-endian readers/writers used only by the TTC splitter. ---
.ru16 <- function(b, off) {
  bitwShiftL(as.integer(b[off]), 8L) + as.integer(b[off + 1L])
}
.ru32 <- function(b, off) {
  # R integers are 32-bit signed; use double to avoid overflow.
  (as.double(b[off])       * 16777216) +
  (as.double(b[off + 1L])  * 65536) +
  (as.double(b[off + 2L])  * 256) +
   as.double(b[off + 3L])
}
.w_u16 <- function(n) {
  n <- as.integer(n)
  as.raw(c(bitwAnd(bitwShiftR(n, 8L), 255L), bitwAnd(n, 255L)))
}
.w_u32 <- function(n) {
  n <- as.double(n)
  as.raw(c(
    as.integer(floor(n / 16777216) %% 256),
    as.integer(floor(n / 65536)    %% 256),
    as.integer(floor(n / 256)      %% 256),
    as.integer(n %% 256)
  ))
}

# Small platform-independent hash without pulling in a new dep.
.short_hash <- function(s) {
  tmp <- tempfile()
  on.exit(unlink(tmp), add = TRUE)
  writeBin(charToRaw(s), tmp)
  unname(tools::md5sum(tmp))
}

# Automatic text-font registration for MicroTeX's `main_font`.
#
# When a user passes gp = gpar(fontfamily = "..."), we want MicroTeX to
# lay out non-math text using the same font that R will draw with. We
# look up the OTF via the systemfonts package and hand it to the C++
# A3 path (microtex_add_font_from_otf), which parses metrics in-memory
# and registers the font with MicroTeX in one step.
#
# TrueType Collections (.ttc) are split into a single-face sfnt on
# first use so systemfonts' glyph-rendering path can also resolve the
# font_file for drawing — see R/ttc-splitter.R::.extract_ttc_face.
#
# If the font can't be resolved we fall back to an empty main_font;
# MicroTeX then uses the math font's text fallback. Layout and rendering
# may then drift slightly, but nothing errors.

# In-process registry of OTFs we've already fed to MicroTeX, keyed by
# the on-disk path. Value is the family name MicroTeX registered under.
.text_font_registered <- new.env(parent = emptyenv())

# Lightweight in-process cache of family-string → resolved family name
# so that repeated grobs in the same session avoid the systemfonts +
# file-stat round trip. Cleared with .clear_text_font_cache().
.text_font_lookup <- new.env(parent = emptyenv())

# Resolve an R fontfamily string to a MicroTeX main_font name, loading
# the OTF on demand. Returns "" if anything goes wrong (caller should
# treat "" as "use default"). Never throws.
.resolve_text_font <- function(family) {
  if (!is.character(family) || length(family) != 1L || !nzchar(family)) {
    family <- "sans"
  }

  cached <- .text_font_lookup[[family]]
  if (!is.null(cached)) return(cached)

  resolved <- tryCatch(
    .do_resolve_text_font(family),
    error = function(e) {
      warning(
        "Could not prepare text font for '", family, "': ", conditionMessage(e),
        "\nFalling back to MicroTeX's built-in metrics.",
        call. = FALSE
      )
      ""
    }
  )
  .text_font_lookup[[family]] <- resolved
  resolved
}

.do_resolve_text_font <- function(family) {
  # match_fonts() always returns *something* (a system fallback) so we
  # don't need an extra existence check. `index` identifies the face
  # within a TrueType Collection (.ttc); for a single-face file it's 0.
  match <- systemfonts::match_fonts(family)
  source_path <- match$path
  if (!is.character(source_path) || !nzchar(source_path) || !file.exists(source_path)) {
    return("")
  }
  face_index <- if (is.numeric(match$index)) as.integer(match$index) else 0L
  if (is.na(face_index) || face_index < 0L) face_index <- 0L

  # TTCs (e.g. macOS Helvetica.ttc) hold multiple faces in one file;
  # MicroTeX takes a single-face sfnt, so extract the requested face
  # into a freestanding OTF the first time we see it.
  otf_path <- if (.is_ttc_file(source_path)) {
    .extract_ttc_face(source_path, face_index)
  } else {
    source_path
  }

  # Already registered? Re-use the family name we recorded then.
  cached_fam <- .text_font_registered[[otf_path]]
  if (!is.null(cached_fam)) return(cached_fam)

  # C++ parses the OTF, synthesises CLM bytes in memory, and registers
  # with MicroTeX in one call. Empty string on failure.
  fam_name <- microtex_add_font_from_otf(otf_path, 0L)
  if (!is.character(fam_name) || !nzchar(fam_name)) return("")

  # If systemfonts handed us a font that happens to carry an OT MATH
  # table (some Linux CI runners alias "sans" to such a font), MicroTeX
  # registers it as a math font, not a main font — it would never be
  # found via the main-font lookup. Treat this as "no matching text
  # font" so the caller falls back to MicroTeX's default text metrics.
  if (!fam_name %in% microtex_main_font_families()) return("")

  .text_font_registered[[otf_path]] <- fam_name
  fam_name
}

# For tests and user-facing "clear everything" helpers.
.clear_text_font_cache <- function() {
  rm(list = ls(.text_font_lookup), envir = .text_font_lookup)
  rm(list = ls(.text_font_registered), envir = .text_font_registered)
  invisible(NULL)
}

# Auto text-font resolution: gp$fontfamily drives MicroTeX's main_font
# so layout metrics match what grid will draw.

test_that(".resolve_text_font registers the system font for a family", {
  gridmicrotex:::.clear_text_font_cache()
  fam <- gridmicrotex:::.resolve_text_font("sans")
  # Some CI images (notably minimal Ubuntu runners) resolve fontconfig's
  # "sans" alias to a font that happens to carry an OT MATH table, in
  # which case MicroTeX registers it as a *math* font and .resolve_text_font
  # returns "" so the caller falls back to default text metrics. Skip
  # the registry check there; the invariant we care about is: if we DID
  # resolve a text font, it's discoverable via main_font_families().
  skip_if(!nzchar(fam), "No non-math system font resolvable for 'sans' here.")
  expect_true(fam %in% microtex_main_font_families())
})

test_that("second resolve of the same family hits the in-process cache", {
  gridmicrotex:::.clear_text_font_cache()
  gridmicrotex:::.resolve_text_font("sans")
  t <- system.time(gridmicrotex:::.resolve_text_font("sans"))["elapsed"]
  expect_lt(t, 0.05)
})

test_that("unknown family falls back to a registered font, or to no font", {
  expect_silent(fam <- gridmicrotex:::.resolve_text_font("TotallyNotAFontFamily123"))
  expect_length(fam, 1L)
  # Only two answers are usable: a family MicroTeX has actually registered,
  # or "" meaning "fall back to the default text metrics". Anything else is
  # a name the layout engine will not find, and metrics silently go wrong.
  if (nzchar(fam)) {
    expect_true(fam %in% microtex_main_font_families())
  } else {
    expect_identical(fam, "")
  }
})

test_that("latex_grob renders with gp$fontfamily and registers main_font", {
  g1 <- latex_grob("\\text{Hello} $x^2$",
                   gp = grid::gpar(fontsize = 16, fontfamily = "sans"))
  expect_s3_class(g1, "latexgrob")
  expect_gt(g1$bbox_w, 0)

  # Passing a different family should also work
  g2 <- latex_grob("\\text{Hello} $x^2$",
                   gp = grid::gpar(fontsize = 16, fontfamily = "serif"))
  expect_s3_class(g2, "latexgrob")
})

test_that("latex_dims accepts a fontfamily via gp and returns finite dims", {
  d <- latex_dims("\\text{Width test}", gp = grid::gpar(fontsize = 12, fontfamily = "sans"))
  expect_true(is.list(d))
  expect_true(is.finite(as.numeric(d$width)))
  expect_gt(as.numeric(d$width), 0)
})

.latex_cache <- new.env(parent = emptyenv())
.latex_cache$entries <- list()
.latex_cache$order <- character(0)
.latex_cache$max_size <- 512L
.latex_cache$hits <- 0L
.latex_cache$misses <- 0L

.cache_get <- function(key) {
  if (!nzchar(key) || is.null(.latex_cache$entries[[key]])) {
    .latex_cache$misses <- .latex_cache$misses + 1L
    return(NULL)
  }
  .latex_cache$hits <- .latex_cache$hits + 1L
  # touch: move key to the most-recent end
  .latex_cache$order <- c(setdiff(.latex_cache$order, key), key)
  .latex_cache$entries[[key]]
}

.cache_put <- function(key, value) {
  if (!nzchar(key) || .latex_cache$max_size <= 0L) return(invisible(NULL))
  .latex_cache$entries[[key]] <- value
  .latex_cache$order <- c(setdiff(.latex_cache$order, key), key)
  excess <- length(.latex_cache$order) - .latex_cache$max_size
  if (excess > 0L) {
    drop <- .latex_cache$order[seq_len(excess)]
    for (k in drop) .latex_cache$entries[[k]] <- NULL
    .latex_cache$order <- .latex_cache$order[-seq_len(excess)]
  }
  invisible(NULL)
}


#' Set the maximum number of entries kept in the LaTeX layout cache
#'
#' The cache stores parsed layout information for recently rendered
#' LaTeX expressions, keyed by the expression and relevant rendering
#' parameters (font, size, macros, etc.). This speeds up repeated
#' rendering of the same expressions, especially in loops or
#' interactive sessions. The default limit is 512 entries, which
#' should be sufficient for most use cases. When the limit is
#' exceeded, the least recently used entries are automatically
#' evicted.
#'
#' @param n Non-negative integer cache capacity. Default is 512. Set
#'   to \code{0} to disable caching.
#' @return
#' \itemize{
#'   \item \code{latex_cache_limit}: Invisibly returns the previous limit.
#'   \item \code{latex_cache_clear}: Invisibly returns \code{NULL}.
#'   \item \code{latex_cache_info}: A list with elements \code{size}
#'     (entries currently stored), \code{max_size}, \code{hits}, and
#'     \code{misses}.
#' }
#' @seealso \code{\link{latex_grob}}, \code{\link{latex_options}}
#' @export
#'
#' @examples
#' \donttest{
#'   latex_cache_limit(256)
#'   grid.latex("e^{i\\pi} + 1 = 0")
#'   latex_cache_info()
#'   latex_cache_clear()
#' }
latex_cache_limit <- function(n = 512L) {
  n <- as.integer(n)
  if (is.na(n) || n < 0L) {
    stop("n must be a non-negative integer.", call. = FALSE)
  }
  old <- .latex_cache$max_size
  .latex_cache$max_size <- n
  if (n == 0L) latex_cache_clear()
  excess <- length(.latex_cache$order) - n
  if (excess > 0L) {
    drop <- .latex_cache$order[seq_len(excess)]
    for (k in drop) .latex_cache$entries[[k]] <- NULL
    .latex_cache$order <- .latex_cache$order[-seq_len(excess)]
  }
  invisible(old)
}

#' @rdname latex_cache_limit
#' @export
latex_cache_clear <- function() {
  .latex_cache$entries <- list()
  .latex_cache$order <- character(0)
  .latex_cache$hits <- 0L
  .latex_cache$misses <- 0L
  # Decoded rasters and parsed pictures are held per file, and each one is
  # far larger than a layout. They have no size limit of their own, so this
  # is the only way to give that memory back.
  .image_cache_clear()
  invisible(NULL)
}


#' @rdname latex_cache_limit
#' @export
latex_cache_info <- function() {
  list(
    size = length(.latex_cache$order),
    max_size = .latex_cache$max_size,
    hits = .latex_cache$hits,
    misses = .latex_cache$misses
  )
}

# The graphics device the layout was measured on. `\text{}` runs are sized
# by the R text-measurer callback, which measures through grid on the
# current device -- and devices disagree: the same phrase came out 99bp on
# pdf() and 98bp on ragg, and much further apart when one device cannot
# resolve the requested family and silently substitutes another. Without
# this in the key, a layout measured on the screen device was reused
# verbatim by a later ggsave(), placing text using the wrong widths.
#
# With no device open the measurer opens a pdf(NULL) of its own, so that
# is the device the measurements will come from.
.cache_device <- function() {
  if (grDevices::dev.cur() == 1L) "pdf" else names(grDevices::dev.cur())
}

# Cache key for a parse_latex_cpp call. Concatenation is fine because the
# tex string is included and the remaining inputs are short numerics/strings.
# `text_family` is the *requested* gp$fontfamily, not the resolved
# main_font: two families that both fail MicroTeX resolution (main_font ==
# "") are still measured with the requested family by the R text-measurer
# callback, so they can produce different layouts and must not share a key.
.parse_cache_key <- function(tex, text_size, line_space, fg_color, max_width,
                             math_font, main_font, text_family, use_path,
                             tex_style, justify, optimal_break,
                             device = "") {
  paste(
    tex, "|", text_size, "|", line_space, "|", fg_color, "|",
    max_width, "|", math_font, "|", main_font, "|", text_family, "|",
    as.integer(use_path), "|", tex_style, "|", as.integer(justify),
    "|", as.integer(optimal_break),
    "|", device,
    sep = ""
  )
}

# Cached wrapper around parse_latex_cpp. Same signature plus `text_family`
# (key-only, not forwarded to C++), transparent cache.
.parse_latex_cached <- function(tex, text_size, line_space, fg_color,
                                max_width, math_font, main_font, use_path,
                                tex_style = "", text_family = "",
                                justify = FALSE, optimal_break = FALSE) {
  key <- .parse_cache_key(tex, text_size, line_space, fg_color, max_width,
                          math_font, main_font, text_family, use_path,
                          tex_style, justify, optimal_break,
                          device = .cache_device())
  hit <- .cache_get(key)
  if (!is.null(hit)) return(hit)
  layout <- parse_latex_cpp(
    tex = tex, text_size = text_size, line_space = line_space,
    fg_color = fg_color, max_width = max_width, math_font = math_font,
    main_font = main_font, use_path = use_path, tex_style = tex_style,
    justify = justify, optimal_break = optimal_break
  )
  .cache_put(key, layout)
  layout
}

# Authoritative list: every environment MicroTeX registers in
# src/MicroTeX/lib/macro/macro_def.cpp env(...) calls, plus the
# starred variants (amsmath's no-equation-numbering forms) that users
# commonly type and that MicroTeX accepts without separate registration.
# Hand-synced to the C++ registrations -- a gap here corrupts mixed-mode
# input, so update both together.
.MATH_ENVS <- c(
  # array family
  "array", "tabular", "tabular*",
  # matrix family
  "matrix", "smallmatrix",
  "pmatrix", "bmatrix", "Bmatrix", "vmatrix", "Vmatrix",
  # equation / display
  "equation", "equation*", "math", "displaymath",
  # amsmath alignments
  "align", "align*", "flalign", "flalign*",
  "alignat", "alignat*", "aligned",
  "alignedat", "alignedat*",
  "eqnarray", "eqnarray*",
  "multline", "multline*",
  # paragraph-like
  "gather", "gather*", "gathered",
  "split", "cases", "rcases",
  # list environments
  "itemize", "enumerate"
)

# Find \end{env} matching \begin{env}, honoring nesting of the same env.
# `from` is the index just past the opening \begin{env}; the return value
# is the index of the backslash that starts \end{env}, or NA_integer_.
.find_env_close <- function(chars, n, from, env) {
  open_tag  <- paste0("\\begin{", env, "}")
  close_tag <- paste0("\\end{",   env, "}")
  ol <- nchar(open_tag); cl <- nchar(close_tag)
  depth <- 1L; k <- from
  while (k <= n) {
    if (chars[k] == "\\") {
      if (k + cl - 1L <= n &&
          paste(chars[k:(k + cl - 1L)], collapse = "") == close_tag) {
        depth <- depth - 1L
        if (depth == 0L) return(k)
        k <- k + cl; next
      }
      if (k + ol - 1L <= n &&
          paste(chars[k:(k + ol - 1L)], collapse = "") == open_tag) {
        depth <- depth + 1L
        k <- k + ol; next
      }
      k <- k + 2L; next   # skip any other \x escape
    }
    k <- k + 1L
  }
  NA_integer_
}

# Single source of truth for "where is the math in this string".
#
# Walks `tex` once and returns its maximal math regions: `$...$`,
# `$$...$$`, `\(...\)`, `\[...\]`, and `\begin{env}...\end{env}` for
# every environment in .MATH_ENVS. Any character not covered by a
# returned span is prose.
#
# Both latex_wrap() (which wraps the prose in \text{}) and
# .md_mask_math() (which hides the math from the markdown parser) are
# built on this, so the two can never disagree about what counts as math.
#
# Returns a list of records:
#   outer_start, outer_end  full span, delimiters included
#   inner_start, inner_end  content between the delimiters. For "env"
#                           spans this equals the outer range, because
#                           the environment is passed through verbatim.
#                           inner_end < inner_start means empty content.
#   display                 TRUE for $$...$$ and \[...\]
#   kind                    "dollar1"|"dollar2"|"paren"|"bracket"|"env"
#   closed                  FALSE when the delimiter ran off the end
.scan_math_spans <- function(tex) {
  chars <- strsplit(tex, "", fixed = TRUE)[[1]]
  n <- length(chars)
  spans <- list()
  i <- 1L

  add_span <- function(outer_start, outer_end, inner_start, inner_end,
                       display, kind, closed) {
    spans[[length(spans) + 1L]] <<- list(
      outer_start = outer_start, outer_end = outer_end,
      inner_start = inner_start, inner_end = inner_end,
      display = display, kind = kind, closed = closed
    )
  }

  # Consume a delimited math run starting at `from` (first content
  # character). `close_fn` reports the length of the closing delimiter at
  # a position, or 0L when it does not close there.
  scan_to_close <- function(from, close_fn) {
    k <- from
    while (k <= n) {
      # An escaped delimiter never closes the run.
      if (chars[k] == "\\" && k < n && chars[k + 1L] %in% c("$", "\\")) {
        k <- k + 2L; next
      }
      cl <- close_fn(k)
      if (cl > 0L) return(c(k, cl))
      k <- k + 1L
    }
    c(NA_integer_, 0L)
  }

  while (i <= n) {
    ch <- chars[i]
    c2 <- if (i < n) chars[i + 1L] else ""

    # Escaped literal in prose -- never opens math.
    if (ch == "\\" && c2 %in% c("$", "\\", "{", "}", "%", "#", "&", "_")) {
      i <- i + 2L; next
    }

    # \[ ... \]  (display)
    if (ch == "\\" && c2 == "[") {
      r <- scan_to_close(i + 2L, function(k) {
        if (chars[k] == "\\" && k < n && chars[k + 1L] == "]") 2L else 0L
      })
      if (is.na(r[1])) {
        add_span(i, n, i + 2L, n, TRUE, "bracket", FALSE); break
      }
      add_span(i, r[1] + r[2] - 1L, i + 2L, r[1] - 1L, TRUE, "bracket", TRUE)
      i <- r[1] + r[2]; next
    }

    # \( ... \)  (inline)
    if (ch == "\\" && c2 == "(") {
      r <- scan_to_close(i + 2L, function(k) {
        if (chars[k] == "\\" && k < n && chars[k + 1L] == ")") 2L else 0L
      })
      if (is.na(r[1])) {
        add_span(i, n, i + 2L, n, FALSE, "paren", FALSE); break
      }
      add_span(i, r[1] + r[2] - 1L, i + 2L, r[1] - 1L, FALSE, "paren", TRUE)
      i <- r[1] + r[2]; next
    }

    # \begin{env} ... \end{env} for known math environments
    if (ch == "\\" && i + 5L <= n &&
        paste(chars[i:(i + 5L)], collapse = "") == "\\begin") {
      rest <- paste(chars[i:min(i + 64L, n)], collapse = "")
      m <- regmatches(rest, regexec("^\\\\begin\\{([^}]+)\\}", rest))[[1]]
      if (length(m) == 2 && m[2] %in% .MATH_ENVS) {
        env <- m[2]
        start_inner <- i + nchar(m[1])
        j <- .find_env_close(chars, n, start_inner, env)
        if (!is.na(j)) {
          close_len <- nchar(paste0("\\end{", env, "}"))
          e <- j + close_len - 1L
          # Environments pass through verbatim: inner == outer.
          add_span(i, e, i, e, FALSE, "env", TRUE)
          i <- e + 1L; next
        }
      }
    }

    # $$ ... $$  (display)
    if (ch == "$" && c2 == "$") {
      r <- scan_to_close(i + 2L, function(k) {
        if (chars[k] == "$" && k < n && chars[k + 1L] == "$") 2L else 0L
      })
      if (is.na(r[1])) {
        add_span(i, n, i + 2L, n, TRUE, "dollar2", FALSE); break
      }
      add_span(i, r[1] + r[2] - 1L, i + 2L, r[1] - 1L, TRUE, "dollar2", TRUE)
      i <- r[1] + r[2]; next
    }

    # $ ... $  (inline)
    if (ch == "$") {
      r <- scan_to_close(i + 1L, function(k) if (chars[k] == "$") 1L else 0L)
      if (is.na(r[1])) {
        add_span(i, n, i + 1L, n, FALSE, "dollar1", FALSE); break
      }
      add_span(i, r[1], i + 1L, r[1] - 1L, FALSE, "dollar1", TRUE)
      i <- r[1] + 1L; next
    }

    i <- i + 1L
  }

  spans
}

#' Wrap standard text for math-first LaTeX renderers
#'
#' @description
#' Parses character strings to safely isolate standard natural language from
#' LaTeX math environments. Standard text is wrapped in `\text{}` blocks, while
#' equations, display math, and specific LaTeX environments are preserved verbatim.
#' This is heavily optimized for passing mixed-content strings (like plot titles
#' or axis labels) to pure-math typesetting engines like MicroTex. The conversion
#' is not perfect, but it should handle most common cases without user intervention.
#'
#' @param tex `character`. The string or vector of strings to be processed.
#' @param input_mode `character`. A length-one character vector dictating the
#'   parsing strategy. If `"mixed"` (default), the string is tokenized and text
#'   is wrapped. If `"math"`, the parser is bypassed and the string is returned
#'   unmodified, assuming the user has provided a pure math equation.
#'
#' @details
#' `latex_wrap()` operates as a state-machine tokenizer to ensure that valid LaTeX
#' math is not corrupted by the text-wrapping process. It features:
#' * **Delimiter Preservation**: Standard inline (`$`, `\(`) and block (`$$`, `\[`)
#'   math delimiters are recognized and preserved.
#' * **Environment Tracking**: Complex nested environments (e.g., `\begin{matrix}`)
#'   are safely extracted and bypassed.
#' * **Newline Conversion**: R newline characters (`\n`) occurring outside of math
#'   environments are automatically converted to LaTeX line breaks (`\\`) inside
#'   the `\text{}` wrapper.
#' * **Literal Escapes**: Escaped LaTeX literals (e.g., `\$`, `\%`, `\#`) are
#'   safely passed into the `\text{}` block without triggering math modes. The
#'   escape character for `\$` is automatically resolved for MicroTex compatibility.
#'
#' @return A `character` vector of the same length as `tex`, formatted for
#'   math-mode LaTeX rendering.
#'
#' @export
#'
#' @examples
#' # "mixed" mode (default) safely wraps text and preserves inline math
#' latex_wrap(r"(The equation \(E=mc^2\) is famous)")
#'
#' # "mixed" mode handles user-escaped characters seamlessly
#' latex_wrap(r"(Cost: \$100 for $x$ items)")
#'
#' # "mixed" mode converts R newlines to stacked text blocks
#' latex_wrap(r"(Line 1\nLine 2)")
#'
#' # "math" mode returns the string completely unmodified
#' latex_wrap(r"(\frac{\alpha}{\beta})", input_mode = "math")

latex_wrap <- function(tex, input_mode = c("mixed", "math")) {
  input_mode <- match.arg(input_mode)
  if (anyNA(tex)) {
    stop("`tex` must not contain NA values.", call. = FALSE)
  }
  if (input_mode == "math") return(tex)
  # The scanner below operates on a single string; recurse so the
  # documented "vector in, vector of the same length out" contract holds.
  if (length(tex) != 1L) {
    return(vapply(tex, latex_wrap, character(1),
                  input_mode = input_mode, USE.NAMES = FALSE))
  }
  if (!nzchar(tex)) return(tex)

  # Math regions come from the shared scanner; everything between them is
  # prose. See .scan_math_spans() for the delimiter rules.
  spans <- .scan_math_spans(tex)
  nch <- nchar(tex)
  out <- character(0)
  pos <- 1L
  unclosed <- FALSE

  # A break is a newline or a literal `\\`, and every one of them belongs
  # at the formula level rather than inside the \text{}.
  #
  # A break kept inside the text made a multi-line *text box*, and a box is
  # one item in the enclosing row: anything after it was then set beside
  # the whole box, vertically centred between its lines, instead of on the
  # line it was written on. That is why "Title\n$x^2$" came out on one
  # line, and why a pasted \caption landed to the left of its table.
  #
  # For prose with nothing beside it the two forms lay out identically, so
  # splitting always is free.
  brk <- "(?:[ \t\r]*(?:\n|\\\\\\\\)[ \t\r]*)+"

  emit_text <- function(s) {
    if (!nzchar(s)) return()
    parts <- .split_prose(s, brk)
    # An empty segment at either end means the chunk began or ended with a
    # break: a separator between neighbours, not a line of its own. A run
    # of breaks collapses to one (the `+` in `brk`), the way consecutive
    # blank lines make a single paragraph break in LaTeX -- stripping
    # document wrappers leaves blank lines behind, and each would
    # otherwise be an empty row.
    lead  <- length(parts) > 0L && !nzchar(parts[1L])
    trail <- length(parts) > 1L && !nzchar(parts[length(parts)])
    parts <- parts[nzchar(parts)]
    # Nothing but breaks: a separator between its neighbours, not text.
    if (length(parts) == 0L) {
      out <<- c(out, "\\\\")
      return()
    }
    if (lead) out <<- c(out, "\\\\")
    for (i in seq_along(parts)) {
      if (i > 1L) out <<- c(out, "\\\\")
      out <<- c(out, paste0("\\text{", .escape_prose_amp(parts[i]), "}"))
    }
    if (trail) out <<- c(out, "\\\\")
  }

  for (sp in spans) {
    if (sp$outer_start > pos) {
      emit_text(substr(tex, pos, sp$outer_start - 1L))
    }
    if (identical(sp$kind, "env")) {
      # Environments pass through verbatim, delimiters included.
      out <- c(out, substr(tex, sp$outer_start, sp$outer_end))
    } else if (sp$inner_end >= sp$inner_start) {
      inner <- substr(tex, sp$inner_start, sp$inner_end)
      if (nzchar(inner)) {
        out <- c(out,
                 if (sp$display) paste0("\\displaystyle ", inner) else inner)
      }
    }
    if (!sp$closed) unclosed <- TRUE
    pos <- sp$outer_end + 1L
  }
  if (pos <= nch) emit_text(substr(tex, pos, nch))

  if (unclosed) {
    warning("Unclosed math delimiter detected and auto-closed at end of string.")
  }

  # A break at either end has nothing to separate -- it would only add a
  # blank first or last row.
  while (length(out) && identical(out[1L], "\\\\")) out <- out[-1L]
  while (length(out) && identical(out[length(out)], "\\\\")) {
    out <- out[-length(out)]
  }

  paste(out, collapse = "")   # <-- no space
}

# An `&` in prose is a literal ampersand -- "R&D", "Treatment & Control".
# MicroTeX reads it as an alignment tab even inside \text{}, and drops
# everything after it, so the label lost its second half with no error.
# Escape only the ones the user did not escape themselves.
#
# Prose is the only place this is safe, and it is the only place it is
# applied: math spans and `\begin{tabular}` environments are passed
# through verbatim by .scan_math_spans(), so a real alignment tab never
# reaches here. (The markdown path escapes `&` too, in .md_escape_tex();
# mixed-mode LaTeX was the one input that did not.)
.escape_prose_amp <- function(s) {
  gsub("(?<!\\\\)&", "\\\\&", s, perl = TRUE)
}

# Split a prose chunk at its line breaks, keeping every segment's braces
# balanced.
#
# Each segment becomes its own `\text{}`, so a break inside a group --
# `\textbf{one\\two}` -- used to leave the group open at the end of one
# segment and unopened at the start of the next. The braces balanced out
# across the whole string, so nothing errored; the second line just came
# out unstyled, and any text after the group escaped the \text{} wrapper.
#
# Every group still open at a break is therefore closed before it and
# re-opened after it, which is what LaTeX itself does across a line break
# inside a text command. `\textbf{one\\two}` becomes
# `\text{\textbf{one}}` + `\\` + `\text{\textbf{two}}`.
#
# Returns the segment bodies, without the `\text{}` wrapper. An empty
# first or last element means the chunk began or ended with a break.
.split_prose <- function(s, brk) {
  m <- gregexpr(brk, s, perl = TRUE)[[1]]
  starts <- if (m[1L] == -1L) integer(0) else as.integer(m)
  lens <- if (length(starts)) attr(m, "match.length") else integer(0)

  chars <- strsplit(s, "", fixed = TRUE)[[1]]
  n <- length(chars)
  # Segments are cut out of `s` by position rather than accumulated a
  # character at a time: growing a vector per character made this
  # quadratic, which showed at a few thousand characters.
  segs <- character(0)
  opens <- character(0)   # stack of opener strings, e.g. "\\textbf{"
  reopen <- ""            # openers inherited from the previous segment
  seg_start <- 1L

  flush <- function(upto) {
    body <- if (upto >= seg_start) substr(s, seg_start, upto) else ""
    segs <<- c(segs, paste0(reopen, body, strrep("}", length(opens))))
    reopen <<- paste(opens, collapse = "")
  }

  i <- 1L
  while (i <= n) {
    k <- match(i, starts)
    if (!is.na(k)) {
      flush(i - 1L)
      i <- i + lens[k]
      seg_start <- i
      next
    }
    ch <- chars[i]
    # An escaped literal (`\{`, `\}`, `\&`, ...) is two characters and
    # never opens or closes a group.
    if (ch == "\\" && i < n) {
      i <- i + 2L
      next
    }
    if (ch == "{") {
      # Re-opening needs the command the brace belongs to, not a bare
      # `{`: the group is `\textbf{`. Arguments already given are part of
      # it, both optional (`\parbox[t]{`) and mandatory
      # (`\textcolor{red}{`) -- matching only the command name reopened
      # `\textcolor{red}{two}` as a plain `{two}` and dropped the colour.
      # A command name and its arguments are short, so only the run just
      # before the brace is examined.
      sofar <- substr(s, max(1L, i - 64L), i - 1L)
      cmd <- regmatches(
        sofar,
        regexpr("\\\\[A-Za-z]+(?:\\[[^]]*\\]|\\{[^{}]*\\})*$", sofar))
      opens <- c(opens, paste0(if (length(cmd)) cmd else "", "{"))
    } else if (ch == "}" && length(opens)) {
      opens <- opens[-length(opens)]
    }
    i <- i + 1L
  }
  flush(n)
  segs
}

# Find the position of the matching `}` for an opening `{` immediately
# before `start`. Returns the 1-based index of the close brace or
# NA_integer_ when unbalanced. Backslash-escaped braces (`\{`, `\}`) are
# treated as literal and skipped.
.find_close_brace <- function(s, start) {
  chars <- strsplit(s, "", fixed = TRUE)[[1]]
  n <- length(chars)
  depth <- 1L
  i <- start
  while (i <= n) {
    ch <- chars[i]
    if (identical(ch, "\\") && i < n) { i <- i + 2L; next }
    if (identical(ch, "{")) {
      depth <- depth + 1L
    } else if (identical(ch, "}")) {
      depth <- depth - 1L
      if (depth == 0L) return(i)
    }
    i <- i + 1L
  }
  NA_integer_
}

# Replace every occurrence of `\command[opt]?(paren)?{...}` with
# `replacement`. Brace nesting is balanced via .find_close_brace.
# When `keep_inner = TRUE`, the matched braced content is re-attached
# after the replacement (so `\emph{X}` → `\textit{X}` via
# replacement = "\\textit", keep_inner = TRUE). When FALSE (default),
# the whole construct is dropped or substituted with a literal.
.replace_command_braced <- function(tex, command, replacement = "",
                                    keep_inner = FALSE) {
  pat <- paste0("\\\\", command, "(?:\\[[^]]*\\])?(?:\\([^)]*\\))?\\{")
  repeat {
    m <- regexpr(pat, tex, perl = TRUE)
    if (m == -1L) return(tex)
    brace_open <- m + attr(m, "match.length") - 1L   # index of `{`
    close <- .find_close_brace(tex, brace_open + 1L)
    if (is.na(close)) return(tex)
    if (keep_inner) {
      inner <- substr(tex, brace_open + 1L, close - 1L)
      rep <- paste0(replacement, "{", inner, "}")
    } else {
      rep <- replacement
    }
    tex <- paste0(substr(tex, 1L, m - 1L),
                  rep,
                  substr(tex, close + 1L, nchar(tex)))
  }
}

# Rewrite `\url{X}` and `\href{U}{X}` to styled text. There is no link in a
# grob, so only the appearance is reproduced -- and it follows LaTeX's
# convention rather than HTML's: hyperref with `colorlinks=true` (which is
# what almost every document sets) colours the text and does not underline
# it, and the `url` package sets a URL in monospace. The markdown side uses
# the HTML convention instead; see the `a` rule in .md_default_rules().
#
# A URL is verbatim in real LaTeX, so its LaTeX-special characters are
# escaped here -- an unescaped `_` in `example.com/a_b` would otherwise open
# a subscript.
.MD_LINK_COLOR <- "#0969DA"

.replace_links <- function(tex) {
  esc <- function(s) gsub("([{}$&#_%~^])", "\\\\\\1", s)

  # \href{destination}{text}: drop the destination, style the text. The
  # text is *not* escaped -- unlike a URL it is ordinary LaTeX and may
  # legitimately contain markup.
  repeat {
    m <- regexpr("\\\\href\\{", tex, perl = TRUE)
    if (m == -1L) break
    o1 <- m + attr(m, "match.length") - 1L
    c1 <- .find_close_brace(tex, o1 + 1L)
    if (is.na(c1)) break
    # The second group must follow immediately for this to be an \href.
    rest <- substr(tex, c1 + 1L, nchar(tex))
    if (!startsWith(rest, "{")) break
    c2 <- .find_close_brace(tex, c1 + 2L)
    if (is.na(c2)) break
    txt <- substr(tex, c1 + 2L, c2 - 1L)
    tex <- paste0(substr(tex, 1L, m - 1L),
                  "\\textcolor{", .MD_LINK_COLOR, "}{", txt, "}",
                  substr(tex, c2 + 1L, nchar(tex)))
  }

  # \url{X}: monospace and coloured, with X escaped.
  repeat {
    m <- regexpr("\\\\url\\{", tex, perl = TRUE)
    if (m == -1L) break
    o1 <- m + attr(m, "match.length") - 1L
    c1 <- .find_close_brace(tex, o1 + 1L)
    if (is.na(c1)) break
    txt <- esc(substr(tex, o1 + 1L, c1 - 1L))
    tex <- paste0(substr(tex, 1L, m - 1L),
                  "\\textcolor{", .MD_LINK_COLOR, "}{\\texttt{", txt, "}}",
                  substr(tex, c1 + 1L, nchar(tex)))
  }
  tex
}

# Rewrite `\caption[opt]?{X}` → `\text{X}\\` inline at source position.
# Position-preserving: where the caption is written in the source is where
# it renders, separated from neighbouring content by a line break.
.replace_caption <- function(tex) {
  pat <- "\\\\caption(?:\\[[^]]*\\])?\\{"
  repeat {
    m <- regexpr(pat, tex, perl = TRUE)
    if (m == -1L) return(tex)
    brace_open <- m + attr(m, "match.length") - 1L
    close <- .find_close_brace(tex, brace_open + 1L)
    if (is.na(close)) return(tex)
    inner <- substr(tex, brace_open + 1L, close - 1L)
    tex <- paste0(substr(tex, 1L, m - 1L),
                  "\\text{", inner, "}\\\\",
                  substr(tex, close + 1L, nchar(tex)))
  }
}

# Strip / rewrite LaTeX document-layer wrappers that MicroTeX doesn't
# model, so raw `print.xtable()`, `knitr::kable()`, and similar
# `tabular`-bearing strings render directly. See `?latex_grob` (section
# "LaTeX document-level wrappers") for the user-facing list.
#
# Strip-list contract: every command here MUST be absent from MicroTeX's
# command tables. Verified against src/MicroTeX/lib/macro/macro_def.cpp
# and lib/core/formula_def.cpp; in particular `\multirow` and the
# `\tiny`..`\Huge` size family ARE valid MicroTeX macros and must
# not be added.
.strip_document_wrappers <- function(tex) {
  if (!nzchar(tex)) return(tex)

  # 0. Links, before the comment stripper: a `%` inside a URL is a real
  # character, and .replace_links() escapes it to `\%`, which step 1 then
  # leaves alone. Stripping comments first would eat the rest of the URL.
  tex <- .replace_links(tex)

  # 1. `%`-to-EOL comments, preserving `\%`.
  tex <- gsub("(?<!\\\\)%[^\n]*\n?", "", tex, perl = TRUE)

  # 2. Preamble + document boundary.
  tex <- .replace_command_braced(tex, "documentclass")
  tex <- .replace_command_braced(tex, "usepackage")
  tex <- gsub("\\\\(?:begin|end)\\{document\\}", "", tex, perl = TRUE)

  # 3. Float environments: keep the contents.
  tex <- gsub("\\\\begin\\{(?:table|figure)\\*?\\}(\\[[^]]*\\])?",
              "", tex, perl = TRUE)
  tex <- gsub("\\\\end\\{(?:table|figure)\\*?\\}", "", tex, perl = TRUE)

  # 3a-i. `tabular*` takes a target width before its column spec, which
  # `tabular` does not. Dropping only the star left the width where the
  # alignment belonged and MicroTeX rejected it outright ("Invalid
  # alignment in array environment"), so drop the width with it. A grob
  # is sized by its content, so there is no width to honour.
  tex <- gsub("\\\\begin\\{tabular\\*\\}\\s*\\{[^{}]*\\}", "\\\\begin{tabular}",
              tex, perl = TRUE)

  # 3a. Strip the trailing `*` on env names. In amsmath, `align*` means
  # "no equation numbering" — MicroTeX doesn't number equations, so the
  # star is semantically a no-op. Normalising here lets the engine
  # recognise the env and enter array mode (without it, `&` errors out).
  tex <- gsub("\\\\(begin|end)\\{([A-Za-z]+)\\*\\}",
              "\\\\\\1{\\2}", tex, perl = TRUE)

  # 4. Title-page metadata (no body output in LaTeX either).
  tex <- gsub("\\\\maketitle\\b", "", tex, perl = TRUE)
  tex <- .replace_command_braced(tex, "title")
  tex <- .replace_command_braced(tex, "author")

  # 5. Cross-reference metadata.
  tex <- .replace_command_braced(tex, "label")

  # 5a. Graphics declarations. `\graphicspath` is normally consumed by
  # .resolve_graphics(), which needs the directories; this catches one that
  # survived (no \includegraphics in the string, or a macro produced it),
  # so its argument is not typeset as visible text.
  tex <- .replace_command_braced(tex, "graphicspath")
  tex <- .replace_command_braced(tex, "DeclareGraphicsExtensions")

  # 6. Layout / alignment scope declarations + content-free declarations
  # that have no analog in a fixed-size grob.
  tex <- gsub("\\\\centering\\b", "", tex, perl = TRUE)
  tex <- gsub("\\\\(?:raggedright|raggedleft|flushleft|flushright)\\b",
              "", tex, perl = TRUE)
  tex <- gsub("\\\\(?:noindent|relax)\\b", "", tex, perl = TRUE)

  # Spacing primitives: map to MicroTeX's \vspace / \quad. Values are
  # em-relative so they scale with the grob's fontsize. The ratios
  # follow LaTeX's small/med/big proportion (1:2:4) rescaled so that
  # \bigskip is one full line at the current size. \hfill and \vfill
  # are rubber lengths in LaTeX (which expand to fill the surrounding
  # glue); a fixed-size grob has nothing to fill, so we substitute a
  # static 1em horizontal / 1em vertical gap.
  tex <- gsub("\\\\smallskip\\b", "\\\\vspace{0.25em}", tex, perl = TRUE)
  tex <- gsub("\\\\medskip\\b",   "\\\\vspace{0.5em}",  tex, perl = TRUE)
  tex <- gsub("\\\\bigskip\\b",   "\\\\vspace{1em}",    tex, perl = TRUE)
  tex <- gsub("\\\\hfill\\b",     "\\\\quad",           tex, perl = TRUE)
  tex <- gsub("\\\\vfill\\b",     "\\\\vspace{1em}",    tex, perl = TRUE)

  # 7. Body-text aliases that map cleanly onto MicroTeX-supported commands.
  tex <- .replace_command_braced(tex, "emph",       "\\textit", keep_inner = TRUE)
  tex <- .replace_command_braced(tex, "textnormal", "\\text",   keep_inner = TRUE)
  tex <- gsub("\\\\(?:par|newline)\\b", "\\\\\\\\", tex, perl = TRUE)

  # 8. Booktabs rules. Top/bottom map to \thickhline (a thicker rule
  # added in MicroTeX C++ alongside this layer); middle rules stay
  # \hline. `\cmidrule[trim]?(parenarg)?{a-b}` keeps its column range
  # via \cline{a-b}, which is also new in C++.
  tex <- gsub("\\\\(?:toprule|bottomrule)\\b",
              "\\\\thickhline", tex, perl = TRUE)
  tex <- gsub("\\\\midrule\\b", "\\\\hline", tex, perl = TRUE)
  tex <- .replace_command_braced(tex, "cmidrule", "\\cline", keep_inner = TRUE)

  # 9. Caption: extract content as inline `\text{...}\\`.
  tex <- .replace_caption(tex)

  tex
}

# --- itemize / enumerate list environments ---

# latex_tree() keeps the x/y of every record, which is what makes the
# claims below checkable. In path mode the coordinates live inside the
# path itself and every row reads as NA, so "it produced a grob" is all
# you can assert there -- which is why these used to assert only that.
items <- function(tex) latex_tree(tex)$records

test_that("itemize puts each item on its own row, marker then content", {
  for (n in 1:3) {
    r <- items(paste0("\\begin{itemize}",
                      paste(paste0("\\item ", letters[seq_len(n)]),
                            collapse = " "),
                      "\\end{itemize}"))
    expect_equal(length(unique(round(r$y, 2))), n)  # one row per item
    expect_equal(nrow(r), 2L * n)                   # a marker and its letter
  }
  # Markers share a column and every letter starts to the right of it.
  r <- items("\\begin{itemize}\\item a \\item b \\item c\\end{itemize}")
  xs <- sort(unique(round(r$x, 2)))
  expect_length(xs, 2L)
  expect_gt(xs[2], xs[1])
})

test_that("math inside an item is still set as math", {
  # Flattened to text these would come out as the literal characters, so
  # assert the two things only a math setter produces: a radical rule,
  # and a superscript set smaller than its base.
  g <- latex_grob("\\begin{enumerate}\\item x^2 \\item \\sqrt{y}\\end{enumerate}",
                  render_mode = "path")
  expect_true("line" %in% g$layout_df$type)

  r <- items("\\begin{enumerate}\\item x^2\\end{enumerate}")
  expect_length(unique(r$font_size), 2L)
  expect_lt(min(r$font_size), max(r$font_size))
})

test_that("the optional [label] changes the marker", {
  right_edge <- function(lab) {
    max(items(paste0("\\begin{enumerate}", lab,
                     "\\item a \\item b\\end{enumerate}"))$x)
  }
  # "1-" is wider than the default "1.", so it pushes the text right.
  expect_gt(right_edge("[\\arabic*-]"), right_edge(""))

  # Each documented counter still yields two rows with a marker on each.
  for (lab in c("[\\alph*)]", "[\\Alph*.]", "[\\roman*)]", "[\\Roman*.]")) {
    r <- items(paste0("\\begin{enumerate}", lab,
                      "\\item a \\item b\\end{enumerate}"))
    expect_equal(length(unique(round(r$y, 2))), 2L, info = lab)
    expect_gt(nrow(r), 2L, label = lab)  # markers as well as the letters
  }
  # A roman counter reaches "ii", which is two glyphs where "b" is one.
  expect_gt(nrow(items("\\begin{enumerate}[\\roman*)]\\item a \\item b\\end{enumerate}")),
            nrow(items("\\begin{enumerate}[\\alph*)]\\item a \\item b\\end{enumerate}")))

  # An itemize marker is customisable too, and a different marker moves
  # the text.
  star  <- items("\\begin{itemize}[\\star]\\item a \\item b\\end{itemize}")
  plain <- items("\\begin{itemize}\\item a \\item b\\end{itemize}")
  expect_gt(max(star$x), max(plain$x))
})

test_that("a nested list is indented past its parent", {
  r <- items(paste0("\\begin{itemize}\\item a \\item ",
                    "\\begin{enumerate}\\item p \\item q\\end{enumerate}",
                    "\\end{itemize}"))
  left <- tapply(r$x, round(r$y, 2), min)
  expect_length(left, 4L)                       # two outer, two inner
  # The two outer bullets share the left margin; the inner rows start
  # further in. Without the indent a nested list is unreadable.
  expect_equal(sum(left == min(left)), 2L)
  expect_gt(max(left), min(left))
})

test_that("empty list does not error", {
  g <- latex_grob("\\begin{itemize}\\end{itemize}", render_mode = "path")
  expect_s3_class(g, "latexgrob")
  expect_equal(nrow(g$layout_df), 0)
})

test_that("lists survive default (mixed) input mode", {
  # latex_wrap must treat itemize/enumerate as math envs, otherwise the
  # body is wrapped in \text{} and split around nested environments.
  wrapped <- latex_wrap("\\begin{itemize}\\item a\\end{itemize}")
  expect_false(any(grepl("\\\\text\\{", wrapped)))
  src <- paste0("\\begin{enumerate}\\item x^2 \\item \\text{T:}\\ ",
                "\\begin{array}{|c|c|}a&b\\\\c&d\\end{array}\\end{enumerate}")
  g_mixed <- latex_grob(src, render_mode = "path")
  g_math <- latex_grob(src, render_mode = "path", input_mode = "math")
  # mixed mode must not mangle the body: identical layout to math mode
  expect_equal(g_mixed$bbox_h, g_math$bbox_h)
  expect_equal(nrow(g_mixed$layout_df), nrow(g_math$layout_df))
})

# Styling for markdown rendering: a small CSS cascade.
#
# Everything that can style a block or a run compiles to one internal
# representation -- an ordered list of rules, each a selector, its
# specificity, and a named list of CSS properties:
#
#   markdown_style(h1 = md_style(...))   R constructor
#   markdown_style(css = "...")          a stylesheet, text or a .css file
#   <div style=|class=>                  one chunk, in the markdown
#   <span style=|class=>                 one inline run
#
# Resolution is CSS's own: highest specificity wins, ties broken by
# document order, and inheritable properties fall through into nested
# containers while box properties do not.
#
# The property set is deliberately small -- only what MicroTeX and grid
# can actually honour. Anything else parses and is then ignored, which is
# what CSS itself does and what makes pasting a real stylesheet safe.

# Properties md_style() accepts, in CSS spelling. R arguments use
# underscores (font_size) and are translated on the way in.
.MD_STYLE_PROPS <- c(
  # inline-capable: compiled to LaTeX commands by .md_css_inline_latex()
  "color", "font-size", "font-family", "font-weight", "font-style",
  "text-decoration", "border", "border-style", "border-radius",
  "box-shadow", "visibility", "vertical-align", "transform",
  # block-only: read by the layout. `padding` and `margin` are the CSS
  # shorthands, expanded into the four longhands by .md_expand_shorthand().
  "line-height", "margin", "margin-top", "margin-bottom", "margin-left",
  "margin-right", "padding", "padding-left", "padding-right", "padding-top",
  "padding-bottom", "text-align", "border-left", "border-top", "height",
  # tables: background is read on tr/td/th, the rest on table/tr/td
  "background", "border-bottom", "border-color", "table-layout",
  # no sensible CSS spelling here
  "bullet", "marker-gap"
)

# CSS spellings accepted for a property stored under a different name.
#
# `background` is the shorthand; `background-color` is the real property,
# and it is what anyone writing a stylesheet reaches for. Storing one and
# accepting both keeps the layout reading a single name while the input
# stays valid CSS. Applied everywhere a declaration arrives -- md_style(),
# a stylesheet, and a `style` attribute -- so the two spellings can never
# behave differently.
.MD_STYLE_ALIASES <- c("background-color" = "background")

.md_canonical_props <- function(nm) {
  hit <- match(nm, names(.MD_STYLE_ALIASES))
  ok <- !is.na(hit)
  nm[ok] <- unname(.MD_STYLE_ALIASES[hit[ok]])
  nm
}

# The properties CSS says are inherited. Box properties are absent on
# purpose: a margin on a blockquote must not become a margin on every
# paragraph inside it.
.MD_INHERITED_PROPS <- c(
  "color", "font-size", "font-family", "font-weight", "font-style",
  "line-height", "text-align"
)

# Built-in defaults, specificity 0. Every value here is the constant it
# replaces in .md_layout(), so the default style reproduces the rendering
# that predates this file.
#
# Absent on purpose: the blockquote bar colour and the rule's colour and
# thickness. Those resolve from gp at draw time (gp$col %||% "grey60" and
# gp$col %||% "black"), a quirk older than this file which a default here
# would silently change.
.md_default_rules <- function() {
  # The heading ladder, measured from the \Huge..\normalsize macros it
  # replaces rather than guessed.
  h <- c(2.5, 2, 1.75, 17 / 12, 7 / 6, 1)
  d <- list(
    p          = list("margin-top" = 0.55),
    # A `$$...$$` paragraph. Centred, as display math is everywhere else.
    math       = list("margin-top" = 0.55, "text-align" = "center"),
    # A link cannot be clicked in a grob, but it should still look like
    # one. Blue and underlined is what a browser does; the LaTeX side
    # follows hyperref's convention instead -- see .strip_document_wrappers().
    a          = list(color = "#0969DA", "text-decoration" = "underline"),
    # The footnote section at the foot of the document: smaller, and set
    # off from the body above it.
    footnote   = list("margin-top" = 0.3, "font-size" = 0.85),
    blockquote = list("margin-top" = 0.55, "padding-left" = 0.75,
                      "border-left" = 0.125),
    pre        = list("margin-top" = 0.55, "line-height" = 1.15,
                      "font-family" = "mono"),
    table      = list("margin-top" = 0.55),
    # A header row is bold everywhere else markdown is rendered. It could
    # not be before: \textbf{\text{x}} reports plain, so the emphasis has
    # to be applied when the cell content is generated -- see
    # .md_table_tex(), which is why this is a `th` rule and not a wrapper.
    th         = list("font-weight" = "bold"),
    img        = list("margin-top" = 0.55),
    hr         = list("margin-top" = 0.55, "height" = 0.5),
    ul         = list("margin-top" = 0.55, "marker-gap" = 0.4,
                      "bullet" = "\\bullet"),
    ol         = list("margin-top" = 0.55, "marker-gap" = 0.4),
    li         = list("margin-top" = 0.55)
  )
  for (i in seq_along(h)) {
    d[[paste0("h", i)]] <- list("margin-top" = 0.95, "font-size" = h[i],
                                "font-weight" = "bold")
  }
  # Syntax highlighting. The selectors are the short class names
  # skylighting emits, which is what knitr already writes into an R
  # user's HTML output -- so a Pandoc syntax theme can be pasted in as
  # ordinary CSS. The colours are GitHub's light palette; names and
  # values are independent, and either can be replaced on its own.
  #
  # Only the classes the bundled grammars use are given a default. Every
  # other skylighting class still resolves through the cascade, it just
  # inherits the block's colour until a stylesheet says otherwise.
  for (nm in names(.MD_CODE_COLORS)) {
    d[[paste0(".", nm)]] <- list(color = unname(.MD_CODE_COLORS[nm]))
  }
  unname(mapply(.md_rule, names(d), d, SIMPLIFY = FALSE))
}

# A selector's specificity: a class beats a tag, an inline style beats
# both. The numbers are CSS's, scaled down to the two levels we support.
.md_specificity <- function(selector) {
  if (startsWith(selector, ".")) 10L else 1L
}

.md_rule <- function(selector, props) {
  list(selector = selector, specificity = .md_specificity(selector),
       props = props)
}

# --- md_style() ------------------------------------------------------

#' Declarations for one markdown tag
#'
#' @description
#' A set of CSS declarations, for use as a named argument to
#' \code{\link{markdown_style}}. Argument names are the CSS property
#' names with underscores in place of hyphens, so \code{font_size} sets
#' \code{font-size}.
#'
#' @details
#' Lengths accept three forms: a bare number is \code{rem}, a multiple of
#' the body font size (\code{font_size = 2.5}); a string is whatever CSS
#' says it is (\code{"2.5em"}, \code{"12pt"}, \code{"150\%"}); and a
#' \code{\link[grid]{unit}} is absolute.
#'
#' These are the supported properties, and where each one has an effect.
#' \emph{Inline} means it also works on a \code{<span>} and in
#' \code{\link{markdown_grob}}, which has no block layout; \emph{block}
#' means it needs \code{\link{markdown_box_grob}}.
#'
#' \tabular{lll}{
#'   \strong{property} \tab \strong{scope} \tab \strong{notes} \cr
#'   \code{color} \tab inline + block \tab \cr
#'   \code{font_size} \tab inline + block \tab \cr
#'   \code{font_family} \tab inline + block \tab \cr
#'   \code{font_weight} \tab inline + block \tab prose blocks only, see below \cr
#'   \code{font_style} \tab inline + block \tab prose blocks only, see below \cr
#'   \code{text_decoration} \tab inline + block \tab \code{underline},
#'     \code{overline}, \code{line-through} \cr
#'   \code{background} \tab inline + block \tab a fill behind the text \cr
#'   \code{border} \tab inline \tab a frame; the inset is fixed \cr
#'   \code{border_style} \tab inline \tab only \code{double} \cr
#'   \code{border_radius} \tab inline \tab rounds the frame \cr
#'   \code{box_shadow} \tab inline \tab \cr
#'   \code{visibility} \tab inline \tab \code{hidden} keeps the space \cr
#'   \code{vertical_align} \tab inline \tab \code{super}, \code{sub},
#'     or a length \cr
#'   \code{transform} \tab inline \tab \code{rotate()}, \code{scale()},
#'     \code{scaleX(-1)} \cr
#'   \code{line_height} \tab block \tab unitless, as \code{gpar()} wants it \cr
#'   \code{margin_top}, \code{margin_bottom} \tab block \tab margins do not
#'     collapse \cr
#'   \code{margin_left}, \code{margin_right} \tab block \tab \cr
#'   \code{padding_left}, \code{padding_right} \tab block \tab \cr
#'   \code{padding_top}, \code{padding_bottom} \tab block \tab \cr
#'   \code{margin}, \code{padding} \tab block \tab the CSS shorthand: one
#'     to four lengths, in CSS's order \cr
#'   \code{text_align} \tab block \tab \code{left}, \code{center},
#'     \code{right} \cr
#'   \code{border_left} \tab block \tab the \code{blockquote} bar \cr
#'   \code{border_top} \tab block \tab the \code{hr} rule \cr
#'   \code{border_bottom} \tab \code{tr} \tab a rule under each table row \cr
#'   \code{border_color} \tab \code{table} \tab colour of the table's rules \cr
#'   \code{table_layout} \tab \code{table} \tab \code{fixed} divides the
#'     width between the columns so a wide table wraps \cr
#'   \code{height} \tab block \tab the band an \code{hr} sits in \cr
#'   \code{bullet} \tab \code{ul} \tab raw LaTeX for the marker glyph \cr
#'   \code{marker_gap} \tab \code{ul}, \code{ol} \tab marker to text \cr
#' }
#'
#' \code{font_size} also accepts CSS's keywords --- \code{xx-small} through
#' \code{xx-large}, plus \code{smaller} and \code{larger} --- taken from the
#' \code{\\tiny}..\code{\\Huge} ladder MicroTeX implements.
#'
#' \strong{The \code{body} rule styles the box itself.} On any other tag,
#' \code{background}, \code{border}, \code{border_radius}, \code{padding}
#' and \code{margin} apply to that block. On \code{body} they apply to the
#' whole \code{\link{markdown_box_grob}} --- its fill, its frame, its
#' corner radius, and the space inside and outside it. That is the only
#' way to give a \code{\link{element_markdown}} title a background, since
#' the theme element takes no box arguments of its own:
#'
#' \preformatted{body \{ background: grey95; padding: 8px;
#'         border: 1px solid grey60; border-radius: 4px \}}
#'
#' An explicit \code{box_gp}, \code{padding}, \code{margin} or \code{r}
#' argument to \code{markdown_box_grob()} wins over the rule, the way an
#' inline style wins in CSS.
#'
#' Anything else is an error --- unlike a pasted stylesheet, where an
#' unknown property is ignored the way a browser ignores it.
#'
#' \strong{One limitation worth knowing.} \code{font_weight} and
#' \code{font_style} apply to blocks whose content is prose --- paragraphs,
#' headings, list items, block quotes, table cells and \code{<div>}s. They
#' do \emph{not} apply to \code{pre} or an image's alt text, which build
#' their own LaTeX and impose their own font handling. This is a MicroTeX
#' constraint rather than a choice: \code{\\text\{\}} resets the font
#' style, so emphasis has to be decided when the content is generated,
#' not wrapped around it afterwards.
#'
#' \strong{What cannot be styled at all.} There is no small-caps
#' (\code{\\textsc} is not a MicroTeX command), no
#' \code{font-variant-numeric}, no right-to-left or bidirectional text,
#' and no padding inside an inline \code{border} --- MicroTeX has no
#' \code{\\fboxsep}, so that inset is fixed.
#'
#' @param ... Named declarations.
#' @return An object of class \code{gridmicrotex_md_style}.
#' @seealso \code{\link{markdown_style}}, \code{\link{markdown_box_grob}}
#' @export
#'
#' @examples
#' md_style(color = "steelblue", font_size = 2.5, margin_top = 1.2)
md_style <- function(...) {
  props <- list(...)
  if (length(props) == 0L) return(structure(list(), class = "gridmicrotex_md_style"))
  nm <- names(props)
  if (is.null(nm) || any(!nzchar(nm))) {
    stop("All arguments to `md_style()` must be named.", call. = FALSE)
  }
  # R spells them with underscores; CSS with hyphens. Accept either, then
  # fold the accepted aliases (background_color -> background) so the
  # stored name is the one the layout reads.
  nm <- .md_canonical_props(gsub("_", "-", tolower(nm), fixed = TRUE))
  bad <- setdiff(nm, .MD_STYLE_PROPS)
  if (length(bad)) {
    stop("Unsupported ", if (length(bad) > 1L) "properties: " else "property: ",
         paste(sQuote(gsub("-", "_", bad, fixed = TRUE)), collapse = ", "),
         ".\nSupported: ",
         paste(gsub("-", "_", .MD_STYLE_PROPS, fixed = TRUE), collapse = ", "),
         call. = FALSE)
  }
  names(props) <- nm
  # A NULL means "say nothing", which is how a tag opts out of a default.
  props <- props[!vapply(props, is.null, logical(1))]
  structure(.md_expand_shorthand(props), class = "gridmicrotex_md_style")
}

#' @export
print.gridmicrotex_md_style <- function(x, ...) {
  if (length(x) == 0L) {
    cat("<md_style: empty>\n")
  } else {
    cat("<md_style>\n")
    for (p in names(x)) cat(sprintf("  %-16s %s\n", p, .md_fmt_value(x[[p]])))
  }
  invisible(x)
}

.md_fmt_value <- function(v) {
  if (grid::is.unit(v)) return(paste(format(v), collapse = ", "))
  paste(format(v), collapse = ", ")
}

# --- stylesheet parsing ----------------------------------------------

# Read `css` as a file when it names one, as CSS text otherwise. A
# stylesheet always contains a brace, so the two are never ambiguous in
# practice, but the file test comes first and settles it either way.
.md_read_css <- function(css) {
  if (!is.character(css)) {
    stop("`css` must be a character string of CSS, or a path to a .css file.",
         call. = FALSE)
  }
  if (length(css) == 1L && !grepl("{", css, fixed = TRUE) && file.exists(css)) {
    return(paste(readLines(css, warn = FALSE), collapse = "\n"))
  }
  paste(css, collapse = "\n")
}

# Parse a stylesheet into rules. Selectors we cannot honour -- anything
# with a combinator, pseudo-class, attribute or id -- are skipped rather
# than raised, so a real-world stylesheet can be pasted in and the parts
# that do apply still work.
.md_parse_stylesheet <- function(text) {
  text <- gsub("/[*].*?[*]/", "", text)          # strip comments
  chunks <- regmatches(text, gregexpr("[^{}]+[{][^{}]*[}]", text))[[1]]
  rules <- list()
  for (ch in chunks) {
    i <- regexpr("{", ch, fixed = TRUE)
    if (i < 1L) next
    sels <- trimws(strsplit(substr(ch, 1L, i - 1L), ",", fixed = TRUE)[[1]])
    props <- .md_parse_css(substr(ch, i + 1L, nchar(ch) - 1L))
    if (length(props) == 0L) next
    for (s in sels) {
      s <- tolower(trimws(s))
      # A bare tag or a single class. At-rules and everything more
      # elaborate than that are silently ignored.
      if (!grepl("^[.]?[a-z][a-z0-9_-]*$", s)) next
      rules[[length(rules) + 1L]] <- .md_rule(s, props)
    }
  }
  rules
}

# --- markdown_style() ------------------------------------------------

#' A style for markdown rendering
#'
#' @description
#' Builds the style used by \code{\link{markdown_box_grob}} and
#' \code{\link{markdown_grob}}: a small CSS cascade over the markdown
#' tags. One constructor covers creating a style, starting from a preset,
#' and extending an existing one. For the properties themselves, and
#' which are honoured where, see \code{\link{md_style}}.
#'
#' @details
#' Tags are named as in HTML, so a stylesheet reads the way a CSS author
#' expects: \code{body} (the document root, which every other tag
#' inherits from --- and which also styles the box itself, see
#' \code{\link{md_style}}), \code{p}, \code{h1} ... \code{h6}, \code{ul},
#' \code{ol}, \code{li}, \code{blockquote}, \code{pre} (a code block),
#' \code{code} (an inline code span), \code{strong} and \code{em} (what
#' markdown's \code{**} and \code{*} produce), \code{table}, \code{tr},
#' \code{td}, \code{th}, \code{hr}, \code{img}, \code{a} (a link),
#' \code{math} (a paragraph that is nothing but \code{$$...$$}),
#' \code{footnote}, \code{div} and \code{span}.
#'
#' The table tags nest as they do in HTML: \code{tr}, \code{td} and
#' \code{th} inherit through \code{table}, so \code{table \{ color: \}}
#' reaches the cells. \code{background} on \code{tr} fills the row, on
#' \code{td}/\code{th} the individual cell.
#'
#' Four things can style a document, and they resolve in CSS's own order:
#' the built-in defaults, then a type selector (\code{h1}), then a class
#' selector (\code{.note}), then an inline \code{style} attribute. Ties
#' are broken by document order, so a later rule wins. Inheritable
#' properties (\code{color}, the \code{font-*} family,
#' \code{line-height}, \code{text-align}) fall through into nested
#' containers, so \code{blockquote \{ color: grey40 \}} greys everything
#' quoted; box properties (margins, padding, borders) do not.
#'
#' The supported CSS is a deliberately small subset: type selectors,
#' class selectors and selector lists (\code{h1, h2}). Combinators,
#' pseudo-classes, attribute selectors, \code{#id} and at-rules are
#' skipped rather than raised, so an existing stylesheet can be handed
#' over and the parts that apply still take effect.
#'
#' @param base \code{NULL} (default) for the built-in defaults, the name
#'   of a bundled preset such as \code{"github"}, or an existing
#'   \code{markdown_style} to extend.
#' @param css A stylesheet: CSS text, or a path to a \code{.css} file.
#'   Read as a file when it names one.
#' @param ... Named tag overrides, each an \code{\link{md_style}}. Use a
#'   leading dot for a class, e.g. \code{.note = md_style(...)}.
#' @return An object of class \code{gridmicrotex_markdown_style}.
#' @seealso \code{\link{md_style}}, \code{\link{markdown_box_grob}}
#' @export
#'
#' @examples
#' markdown_style()
#' markdown_style("github", h1 = md_style(color = "firebrick"))
#' markdown_style(css = "h1 { color: steelblue } .note { padding-left: 2em }")
markdown_style <- function(base = NULL, css = NULL, ...) {
  rules <- .md_default_rules()

  if (!is.null(base)) {
    if (inherits(base, "gridmicrotex_markdown_style")) {
      rules <- base$rules
    } else if (is.character(base) && length(base) == 1L) {
      rules <- c(rules, .md_parse_stylesheet(.md_read_css(.md_preset_path(base))))
    } else {
      stop("`base` must be NULL, a preset name, or a markdown_style object.",
           call. = FALSE)
    }
  }

  if (!is.null(css)) {
    rules <- c(rules, .md_parse_stylesheet(.md_read_css(css)))
  }

  dots <- list(...)
  if (length(dots)) {
    nm <- names(dots)
    if (is.null(nm) || any(!nzchar(nm))) {
      stop("Tag overrides must be named, e.g. `h1 = md_style(...)`.",
           call. = FALSE)
    }
    for (i in seq_along(dots)) {
      st <- dots[[i]]
      if (!inherits(st, "gridmicrotex_md_style")) {
        stop("`", nm[i], "` must be an `md_style()` object.", call. = FALSE)
      }
      if (length(st)) rules <- c(rules, list(.md_rule(tolower(nm[i]), unclass(st))))
    }
  }

  structure(list(rules = rules), class = "gridmicrotex_markdown_style")
}

# Locate a bundled preset. Presets are CSS files rather than R code, so
# each one doubles as a worked example a user can copy.
.md_preset_path <- function(name) {
  if (!grepl("^[a-z0-9_-]+$", name)) {
    stop("Unknown preset ", sQuote(name), ".", call. = FALSE)
  }
  f <- system.file("css", paste0(name, ".css"), package = "gridmicrotex")
  if (!nzchar(f) || !file.exists(f)) {
    avail <- .md_preset_names()
    stop("Unknown preset ", sQuote(name),
         if (length(avail)) paste0(". Available: ",
                                   paste(sQuote(avail), collapse = ", ")) else "",
         call. = FALSE)
  }
  f
}

.md_preset_names <- function() {
  d <- system.file("css", package = "gridmicrotex")
  if (!nzchar(d)) return(character(0))
  sub("[.]css$", "", list.files(d, pattern = "[.]css$"))
}

#' @export
print.gridmicrotex_markdown_style <- function(x, ...) {
  sels <- vapply(x$rules, function(r) r$selector, character(1))
  cat("<markdown_style>", length(x$rules), "rules,",
      length(unique(sels)), "selectors\n")
  for (s in unique(sels)) {
    props <- character(0)
    for (r in x$rules[sels == s]) {
      for (p in names(r$props)) props[[p]] <- .md_fmt_value(r$props[[p]])
    }
    cat(sprintf("  %-12s %s\n", s,
                paste(names(props), unlist(props), sep = ": ", collapse = "; ")))
  }
  invisible(x)
}

# Coerce whatever `style =` was given into a style object.
#
# A bare string is accepted so the short form needs no constructor:
# style = "h1 { color: red }" or style = "house.css". A preset name would
# be ambiguous with a one-word file path, so presets stay explicit --
# markdown_style("github") -- and are not accepted here.
.md_as_style <- function(style) {
  if (is.null(style)) style <- .opt("markdown_style")
  if (is.null(style)) return(markdown_style())
  if (inherits(style, "gridmicrotex_markdown_style")) return(style)
  if (is.character(style)) return(markdown_style(css = style))
  stop("`style` must be a `markdown_style()` object, CSS text, or a path ",
       "to a .css file.", call. = FALSE)
}

# --- the cascade -----------------------------------------------------

# Resolve the properties in force for one element.
#
#   tag       the HTML-style tag name ("h1", "blockquote", ...)
#   classes   class names in scope, from an enclosing div or a span
#   inline    properties from a `style` attribute, highest priority
#   inherited already-resolved properties of the enclosing container
#
# Inherited values come first so any declared value beats them, which is
# what CSS does; among declared values, specificity then document order.
.md_cascade <- function(style, tag, classes = character(0), inline = NULL,
                        inherited = list()) {
  out <- inherited[intersect(names(inherited), .MD_INHERITED_PROPS)]

  rules <- style$rules
  if (length(rules)) {
    want <- c(tag, paste0(".", classes))
    hit <- which(vapply(rules, function(r) r$selector %in% want, logical(1)))
    if (length(hit)) {
      spec <- vapply(rules[hit], function(r) r$specificity, integer(1))
      # order() is stable, so equal specificity keeps document order.
      for (i in hit[order(spec)]) {
        for (p in names(rules[[i]]$props)) out[[p]] <- rules[[i]]$props[[p]]
      }
    }
  }

  if (length(inline)) for (p in names(inline)) out[[p]] <- inline[[p]]
  out
}

# --- value resolution ------------------------------------------------

# A CSS length in big points.
#
#   body  the document's font size, what `rem` and a bare number mean
#   own   this element's own font size, what `em` and `%` mean
#
# A bare number is `rem` rather than `em` so that the built-in defaults
# -- every one of which was a multiple of the body size before it lived
# here -- carry over unchanged while `em` keeps its proper CSS meaning.
.md_css_length <- function(v, body, own = body, horizontal = FALSE) {
  if (is.null(v)) return(NULL)
  if (grid::is.unit(v)) {
    return(if (horizontal) grid::convertWidth(v, "bigpts", valueOnly = TRUE)
           else grid::convertHeight(v, "bigpts", valueOnly = TRUE))
  }
  if (is.numeric(v)) return(if (is.finite(v)) v * body else NULL)
  x <- trimws(tolower(as.character(v)))
  m <- regmatches(x, regexec("^(-?[0-9]*[.]?[0-9]+)[[:space:]]*([a-z%]*)$", x))[[1]]
  if (length(m) != 3L) return(NULL)
  n <- as.numeric(m[2])
  if (!is.finite(n)) return(NULL)
  # A bare number carries no unit, which switch() cannot express as a
  # name, so it is settled before the switch alongside its synonym.
  u <- m[3]
  if (!nzchar(u) || u == "rem") return(n * body)
  # A big point and a CSS pt are both 1/72in, so absolute units need no
  # reference size.
  switch(u,
    em = n * own, "%" = n / 100 * own,
    pt = n, px = n * 72 / 96, "in" = n * 72,
    cm = n * 72 / 2.54, mm = n * 72 / 25.4,
    NULL)
}

# The `border-<side>` shorthand, of which only the width and the colour
# mean anything here -- every rule this package draws is solid.
# Returns list(width, color); either may be NULL, meaning "not stated".
.md_css_border <- function(v, body) {
  if (is.null(v)) return(NULL)
  if (is.numeric(v) || grid::is.unit(v)) {
    return(list(width = .md_css_length(v, body, horizontal = TRUE), color = NULL))
  }
  width <- NULL
  color <- NULL
  for (p in strsplit(trimws(as.character(v)), "[[:space:]]+")[[1]]) {
    if (tolower(p) %in% c("none", "hidden")) return(list(width = 0, color = NULL))
    if (tolower(p) %in% c("solid", "dotted", "dashed", "double", "groove",
                          "ridge", "inset", "outset")) next
    w <- .md_css_length(p, body, horizontal = TRUE)
    if (!is.null(w)) {
      width <- w
      next
    }
    cc <- .md_resolve_color(p)
    if (!is.null(cc)) color <- cc
  }
  list(width = width, color = color)
}

# The text-mode LaTeX commands a resolved style implies.
#
# These are the ones whose content must be emitted *bare*: MicroTeX
# builds a non-nested FontStyleAtom for \text{}, so \textbf{\text{x}}
# reports plain and the weight is silently lost. Measured, not assumed --
# \textbf{\text{hello}} and \text{hello} come out the same width.
.md_emphasis_cmds <- function(res) {
  cmds <- character(0)
  w <- res[["font-weight"]]
  if (!is.null(w) && .md_css_is_bold(tolower(as.character(w)))) {
    cmds <- c(cmds, "\\textbf{")
  }
  s <- res[["font-style"]]
  if (!is.null(s) && tolower(as.character(s)) %in% c("italic", "oblique")) {
    cmds <- c(cmds, "\\textit{")
  }
  cmds
}

# text-align to the fraction .md_layout()/makeContent() shift by.
.md_css_align <- function(v) {
  if (is.null(v)) return(NULL)
  switch(tolower(trimws(as.character(v))),
         left = 0, start = 0, center = 0.5, centre = 0.5,
         right = 1, end = 1, NULL)
}

# Styling: the cascade, its two constructors, and the two markup routes.

# Lay a document out with a given style and hand back the items, so a
# test can assert on positions rather than pixels.
lay_md <- function(md, style = NULL, size = 12, width = 300) {
  gp <- grid::gpar(fontsize = size)
  .md_layout(.md_parse_blocks(md, size, .md_as_style(style)),
             width, gp, size, style = .md_as_style(style))
}

# --- md_style() ----------------------------------------------------------

test_that("md_style() translates R names and rejects unknown properties", {
  s <- md_style(font_size = 2, margin_top = 1, color = "red")
  expect_named(unclass(s), c("font-size", "margin-top", "color"))
  # The CSS spelling works too, for anyone copying a declaration across.
  expect_named(unclass(md_style(`font-size` = 2)), "font-size")

  # A constructor can afford to be strict where a pasted stylesheet
  # cannot: a typo here is a mistake, not a property we do not implement.
  expect_error(md_style(colour = "red"), "colour")
  expect_error(md_style(bakground = "red"), "bakground")
  expect_error(md_style("red"), "must be named")

  # NULL is how a tag opts out of a default rather than setting one.
  expect_length(unclass(md_style(color = NULL)), 0L)
})

# --- the stylesheet parser ----------------------------------------------

test_that("the stylesheet parser reads what it supports and skips the rest", {
  r <- .md_parse_stylesheet("
    /* a comment */
    h1, h2 { color: steelblue; font-size: 2.5rem }
    .note  { padding-left: 2em }
    div > p  { color: red }
    a:hover  { color: red }
    #id      { color: red }
    @media print { }
  ")
  # A selector list becomes one rule per selector.
  expect_equal(vapply(r, function(x) x$selector, character(1)),
               c("h1", "h2", ".note"))
  expect_equal(r[[1]]$props[["font-size"]], "2.5rem")
  # A class outranks a tag.
  expect_equal(vapply(r, function(x) x$specificity, integer(1)),
               c(1L, 1L, 10L))
})

test_that("malformed CSS yields no rules rather than an error", {
  for (bad in c("", "h1 { ", "} h1 {}", "h1 {}", "not css at all")) {
    expect_silent(r <- .md_parse_stylesheet(bad))
    expect_length(r, 0L)
  }
})

test_that("css= reads a file when it names one, and text otherwise", {
  f <- tempfile(fileext = ".css")
  writeLines("h1 { color: #123456 }", f)
  on.exit(unlink(f), add = TRUE)

  from_file <- .md_cascade(markdown_style(css = f), "h1")
  from_text <- .md_cascade(markdown_style(css = "h1 { color: #123456 }"), "h1")
  expect_equal(from_file$color, "#123456")
  expect_equal(from_text$color, from_file$color)
})

# --- markdown_style() ----------------------------------------------------

test_that("markdown_style() resolves base, then css, then explicit tags", {
  s <- markdown_style(
    css = "h1 { color: #111111; font-size: 3rem }",
    h1 = md_style(color = "#222222")
  )
  res <- .md_cascade(s, "h1")
  expect_equal(res$color, "#222222")        # the explicit argument wins
  expect_equal(res[["font-size"]], "3rem")  # ...without dropping the rest

  # An existing style can be extended.
  s2 <- markdown_style(s, h1 = md_style(color = "#333333"))
  expect_equal(.md_cascade(s2, "h1")$color, "#333333")

  expect_error(markdown_style(base = 42), "must be NULL")
  expect_error(markdown_style(h1 = "not a style"), "md_style")
  expect_error(markdown_style(md_style(color = "red")), "must be NULL")
})

test_that("every bundled preset parses to a usable style", {
  presets <- .md_preset_names()
  expect_true("github" %in% presets)
  for (p in presets) {
    s <- markdown_style(p)
    expect_s3_class(s, "gridmicrotex_markdown_style")
    # A preset that contributes nothing is a preset with a typo in it.
    expect_gt(length(s$rules), length(markdown_style()$rules))
    expect_false(is.null(.md_cascade(s, "h1")[["font-size"]]))
  }
  expect_error(markdown_style("no-such-preset"), "Unknown preset")
})

# --- the cascade ---------------------------------------------------------

test_that("specificity orders the four origins", {
  s <- markdown_style(css = "h1 { color: #010101 } .note { color: #020202 }")
  expect_equal(.md_cascade(s, "h1")$color, "#010101")
  expect_equal(.md_cascade(s, "h1", classes = "note")$color, "#020202")
  expect_equal(.md_cascade(s, "h1", classes = "note",
                           inline = list(color = "#030303"))$color, "#030303")
  # A later rule of equal specificity wins.
  s2 <- markdown_style(css = "h1 { color: #010101 } h1 { color: #040404 }")
  expect_equal(.md_cascade(s2, "h1")$color, "#040404")
})

test_that("inheritable properties cascade and box properties do not", {
  s <- markdown_style()
  inherited <- list(color = "grey40", "font-family" = "serif",
                    "margin-top" = 99, "padding-left" = 99,
                    "border-left" = 99)
  res <- .md_cascade(s, "p", inherited = inherited)
  expect_equal(res$color, "grey40")
  expect_equal(res[["font-family"]], "serif")
  # p has its own margin default; the container's must not have leaked in.
  expect_equal(res[["margin-top"]], 0.55)
  expect_null(res[["padding-left"]])
  expect_null(res[["border-left"]])
})

# --- lengths -------------------------------------------------------------

test_that("lengths accept a bare number, a CSS string and a unit", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # A bare number is rem: a multiple of the body size, not the element's.
  expect_equal(.md_css_length(2.5, body = 10, own = 20), 25)
  expect_equal(.md_css_length("2.5rem", body = 10, own = 20), 25)
  expect_equal(.md_css_length("2em", body = 10, own = 20), 40)
  expect_equal(.md_css_length("150%", body = 10, own = 20), 30)
  expect_equal(.md_css_length("12pt", body = 10), 12)
  expect_equal(.md_css_length("16px", body = 10), 12)
  expect_equal(.md_css_length(grid::unit(1, "in"), body = 10), 72)
  # An absolute length must not drift with the base.
  for (b in c(5, 10, 40)) expect_equal(.md_css_length("10pt", body = b), 10)
  for (bad in list("nonsense", "", "10furlongs")) {
    expect_null(.md_css_length(bad, body = 10))
  }
})

test_that("a unitless CSS string stays invalid, as it is in CSS", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # `font-size: 12` is ignored by a browser; only the R spelling
  # md_style(font_size = 12) means a multiple.
  expect_null(.md_css_size("12", 20))
  expect_equal(.md_css_size(2.5, 20), 2.5)
  expect_equal(.md_css_size(grid::unit(10, "bigpts"), 20), 0.5)
})

test_that("the border shorthand yields a width and a colour", {
  b <- .md_css_border("3px solid steelblue", body = 10)
  expect_equal(b$width, 2.25)
  expect_equal(b$color, "#4682B4")
  # Order does not matter, and the style keyword is ignored.
  expect_equal(.md_css_border("#D1D9E0 0.25rem", body = 10)$width, 2.5)
  expect_equal(.md_css_border("none", body = 10)$width, 0)
  # Unstated parts come back NULL so the caller can fall back.
  expect_null(.md_css_border("2pt", body = 10)$color)
  expect_null(.md_css_border(NULL, body = 10))
})

# --- the default style reproduces the previous rendering -----------------

test_that("the built-in defaults carry over the constants they replaced", {
  s <- markdown_style()
  expect_equal(.md_cascade(s, "p")[["margin-top"]], 0.55)
  expect_equal(.md_cascade(s, "h1")[["margin-top"]], 0.95)
  expect_equal(.md_cascade(s, "pre")[["line-height"]], 1.15)
  expect_equal(.md_cascade(s, "blockquote")[["padding-left"]], 0.75)
  expect_equal(.md_cascade(s, "ul")[["marker-gap"]], 0.4)
  # The heading ladder, as measured from the \Huge..\normalsize macros.
  sizes <- vapply(paste0("h", 1:6),
                  function(h) .md_cascade(s, h)[["font-size"]], numeric(1))
  expect_equal(unname(sizes), c(2.5, 2, 1.75, 17 / 12, 7 / 6, 1))
})

# --- the style reaches the layout ----------------------------------------

test_that("each documented knob moves the thing it names", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  md <- "# Head\n\nA paragraph.\n\n> quoted\n\n- one\n- two\n\n---"

  expect_gt(lay_md(md, "p { margin-top: 3rem }")$height, lay_md(md)$height)
  expect_lt(lay_md(md, "p { margin-top: 0 }")$height, lay_md(md)$height)
  expect_gt(lay_md(md, "h1 { font-size: 4rem }")$items[[1]]$h,
            lay_md(md)$items[[1]]$h)

  # text-align is recorded per block, for makeContent() to apply.
  al <- lay_md(md, "h1 { text-align: center } p { text-align: right }")
  expect_equal(al$items[[1]]$align, 0.5)
  expect_equal(al$items[[2]]$align, 1)
  expect_null(lay_md(md)$items[[1]]$align)

  # marker-gap pushes the item body right; the marker stays put.
  wide <- lay_md("- one", "ul { marker-gap: 3rem }")
  expect_equal(wide$items[[1]]$x, lay_md("- one")$items[[1]]$x)
  expect_gt(wide$items[[2]]$x, lay_md("- one")$items[[2]]$x)

  # bullet is the escape hatch for a marker glyph.
  expect_equal(lay_md("- one", "ul { bullet: Z }")$items[[1]]$grob$tex, "Z")

  # border-left is the quote bar.
  bar <- function(l) Filter(function(it) inherits(it$grob, "rect"), l$items)[[1]]
  expect_equal(bar(lay_md(md, "blockquote { border-left: 6pt solid red }"))$w, 6)
})

test_that("colour reaches the drawn grob and inherits into a quote", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  l <- lay_md("> quoted text", "blockquote { color: #FF0000 }")
  txt <- Filter(function(it) inherits(it$grob, "latexgrob"), l$items)
  expect_true("#FF0000" %in% unlist(lapply(txt, function(it)
    it$grob$layout_df$color)))
})

test_that("markdown_box_grob() accepts an object, CSS text or a file path", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- tempfile(fileext = ".css")
  writeLines("h1 { color: #654321 }", f)
  on.exit(unlink(f), add = TRUE)

  for (st in list(markdown_style(css = "h1 { color: #654321 }"),
                  "h1 { color: #654321 }", f)) {
    g <- markdown_box_grob("# H", width = grid::unit(2, "in"), style = st)
    cols <- unlist(lapply(.md_box_layout(g)$items,
                          function(it) it$grob$layout_df$color))
    expect_true("#654321" %in% cols)
  }
  expect_error(markdown_box_grob("x", style = 42), "markdown_style")
})

# --- the <div> route -----------------------------------------------------

test_that("a div with blank lines around it becomes a container block", {
  b <- .md_parse_blocks(
    "Before\n\n<div class=\"note\">\n\n## In\n\nText.\n\n</div>\n\nAfter")
  expect_equal(vapply(b, function(x) x$type, character(1)),
               c("paragraph", "div", "paragraph"))
  d <- b[[2]]
  expect_equal(d$class, "note")
  expect_equal(vapply(d$blocks, function(x) x$type, character(1)),
               c("heading", "paragraph"))

  # Without blank lines cmark hands over one opaque block, which is
  # dropped exactly as it was before divs meant anything.
  expect_length(.md_parse_blocks("<div class=\"n\">\n**b**\n</div>"), 0L)
})

test_that("divs nest, close themselves, and ignore stray markup", {
  outer <- .md_parse_blocks(
    "<div class=\"a\">\n\n<div class=\"b\">\n\nx\n\n</div>\n\n</div>")[[1]]
  expect_equal(outer$class, "a")
  expect_equal(outer$blocks[[1]]$type, "div")
  expect_equal(outer$blocks[[1]]$class, "b")

  # An unclosed div still lays its contents out rather than losing them.
  u <- .md_parse_blocks("<div class=\"a\">\n\nx\n\ny")
  expect_equal(u[[1]]$type, "div")
  expect_length(u[[1]]$blocks, 2L)

  # A stray close, and any other raw block, is dropped.
  expect_equal(vapply(.md_parse_blocks("x\n\n</div>\n\ny"),
                      function(b) b$type, character(1)),
               c("paragraph", "paragraph"))
  expect_equal(vapply(.md_parse_blocks("x\n\n<table><tr></tr></table>\n\ny"),
                      function(b) b$type, character(1)),
               c("paragraph", "paragraph"))
})

test_that("a div styles the chunk inside it", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  plain <- lay_md("Text.")
  padded <- lay_md("<div style=\"padding-left: 3rem\">\n\nText.\n\n</div>")
  expect_equal(padded$items[[1]]$x - plain$items[[1]]$x, 36)

  # A class matches the element carrying it and nothing else, as in CSS:
  # the paragraph inside must not be indented a second time.
  once <- lay_md("<div class=\"note\">\n\nT.\n\n</div>",
                 ".note { padding-left: 2rem }")
  expect_equal(once$items[[1]]$x, 24)

  # Inheritable properties still reach the contents.
  col <- lay_md("<div class=\"note\">\n\nT.\n\n</div>",
                ".note { color: #00FF00 }")
  expect_true("#00FF00" %in% col$items[[1]]$grob$layout_df$color)
})

# --- the <span> route ----------------------------------------------------

test_that("a span resolves its class from the stylesheet", {
  s <- markdown_style(css = ".hot { color: #FF0000; font-weight: bold }")
  expect_match(.md_to_tex('a <span class="hot">b</span>', 20, s),
               "\\textcolor{#FF0000}{\\textbf{b}}", fixed = TRUE)
  # The style attribute outranks the class.
  expect_match(
    .md_to_tex('<span class="hot" style="color: #0000FF">b</span>', 20, s),
    "#0000FF", fixed = TRUE)
  # An unknown class is not an error, it simply matches nothing.
  expect_match(.md_to_tex('<span class="nope">b</span>', 20, s),
               "\\text{b}", fixed = TRUE)
  # Without a stylesheet a class means nothing at all.
  expect_match(.md_to_tex('<span class="hot">b</span>'), "\\text{b}",
               fixed = TRUE)
})

test_that("font-weight and font-style now work on a span", {
  # Previously parsed and dropped: .md_parse_style() knew only colour,
  # size, family and decoration.
  expect_match(.md_to_tex('<span style="font-weight: bold">b</span>'),
               "\\textbf{", fixed = TRUE)
  expect_match(.md_to_tex('<span style="font-weight: 700">b</span>'),
               "\\textbf{", fixed = TRUE)
  expect_match(.md_to_tex('<span style="font-style: italic">b</span>'),
               "\\textit{", fixed = TRUE)
  # Weights below 600 are not bold, and `normal` is not italic.
  expect_false(grepl("textbf", .md_to_tex('<span style="font-weight: 400">b</span>'),
                     fixed = TRUE))
  expect_false(grepl("textit", .md_to_tex('<span style="font-style: normal">b</span>'),
                     fixed = TRUE))
})

# --- markdown_grob() and the global option -------------------------------

test_that("markdown_grob() applies what compiles to LaTeX and ignores layout", {
  # A heading had no size at all in the flattened path before this.
  expect_match(.md_to_tex("# H", 20, markdown_style()), "\\scalebox{",
               fixed = TRUE)
  expect_match(.md_to_tex("# H", 20, .md_as_style("h1 { color: #FF0000 }")),
               "\\textcolor{#FF0000}", fixed = TRUE)
  # Layout properties have nothing to act on here and are dropped.
  expect_equal(.md_to_tex("# H", 20, .md_as_style("h1 { margin-top: 9rem }")),
               .md_to_tex("# H", 20, markdown_style()))
})

test_that("latex_options(markdown_style=) supplies a default both grobs use", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  old <- latex_options(markdown_style = "h1 { color: #0000FF }")
  on.exit(reset_latex_options(), add = TRUE)

  g <- markdown_grob("# H", gp = grid::gpar(fontsize = 12))
  expect_true("#0000FF" %in% g$layout_df$color)

  b <- markdown_box_grob("# H", width = grid::unit(2, "in"),
                         gp = grid::gpar(fontsize = 12))
  expect_true("#0000FF" %in% unlist(lapply(.md_box_layout(b)$items,
                                           function(it) it$grob$layout_df$color)))

  # An argument at the call site still wins over the option.
  g2 <- markdown_grob("# H", style = "h1 { color: #FF00FF }",
                      gp = grid::gpar(fontsize = 12))
  expect_true("#FF00FF" %in% g2$layout_df$color)

  reset_latex_options()
  expect_null(latex_options()$markdown_style)
})

# --- emphasis actually renders (not just emitted) ------------------------

# The font styles MicroTeX reports for a drawn document. 1 = plain,
# 2 = bold, 4 = italic, 6 = both.
styles_of <- function(md, css = NULL, size = 12) {
  g <- markdown_box_grob(md, width = grid::unit(3, "in"), style = css,
                         gp = grid::gpar(fontsize = size))
  d <- unlist(lapply(.md_box_layout(g)$items,
                     function(it) it$grob$layout_df$font_style))
  sort(unique(d[!is.na(d)]))
}

test_that("block font-weight and font-style reach the glyphs", {
  # Emitting \textbf{\text{x}} is not enough: \text{} builds a non-nested
  # FontStyleAtom, so the weight is silently dropped and the run reports
  # plain. The content has to be generated bare instead.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  expect_equal(styles_of("hello"), 1)
  expect_equal(styles_of("hello", "p { font-weight: bold }"), 2)
  expect_equal(styles_of("hello", "p { font-style: italic }"), 4)
  expect_equal(styles_of("hello", "p { font-weight: bold; font-style: italic }"), 6)
})

test_that("headings are bold by default, and can be told not to be", {
  # They were not, before: the old \Huge{}\textbf{\text{H}} had the same
  # reset problem, so no heading this package ever drew was actually bold.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  expect_equal(styles_of("# Head"), 2)
  expect_equal(styles_of("# Head", "h1 { font-weight: normal }"), 1)
})

test_that("emphasis inherits into quotes, list items and divs", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  expect_equal(styles_of("> quoted", "blockquote { font-weight: bold }"), 2)
  expect_equal(styles_of("- item", "li { font-weight: bold }"), 2)
  expect_equal(styles_of("<div class=\"b\">\n\nx\n\n</div>",
                         ".b { font-weight: bold }"), 2)
})

test_that("emphasis does not apply to pre or alt text", {
  # A documented limitation, tested so the documentation stays true: these
  # build their own LaTeX and impose their own font handling.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  expect_equal(styles_of("```\ncode\n```", "pre { font-weight: bold }"), 1)
  expect_equal(styles_of("![alt](nope.png)", "img { font-weight: bold }"), 1)

  # Tables used to be in this list. They are not any more: cell content is
  # generated with its emphasis already applied, so font-weight inherits
  # from `table` into the cells the way CSS says it should.
  expect_equal(styles_of("| a |\n|---|\n| 1 |", "table { font-weight: bold }"), 2)
})

# --- tags and properties that had no effect at all -----------------------

test_that("body seeds the inheritance every block starts from", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g <- markdown_box_grob("text\n\n# head", width = grid::unit(3, "in"),
                         style = "body { color: #FF0000 }")
  cols <- unlist(lapply(.md_box_layout(g)$items,
                        function(it) it$grob$layout_df$color))
  expect_equal(unique(cols), "#FF0000")
  # ...and a tag rule still beats what it inherits.
  g2 <- markdown_box_grob("text\n\n# head", width = grid::unit(3, "in"),
                          style = "body { color: #FF0000 } h1 { color: #0000FF }")
  cols2 <- unlist(lapply(.md_box_layout(g2)$items,
                         function(it) it$grob$layout_df$color))
  expect_setequal(cols2, c("#FF0000", "#0000FF"))
})

test_that("code styles an inline code span, on top of its monospace default", {
  s <- .md_as_style("code { color: #FF0000 }")
  tex <- .md_to_tex("a `x` b", 20, s)
  expect_match(tex, "\\textcolor{#FF0000}", fixed = TRUE)
  # The \texttt default is added to, not replaced.
  expect_match(tex, "\\texttt{", fixed = TRUE)
})

test_that("margin-bottom adds space below a block", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  h <- function(css) {
    s <- .md_as_style(css)
    .md_layout(.md_parse_blocks("one\n\ntwo", 12, s), 300,
               grid::gpar(fontsize = 12), 12, style = s)$height
  }
  expect_gt(h("p { margin-bottom: 4rem }"), h(NULL))
  # Every built-in default leaves it unset, so nothing moves by itself.
  expect_equal(h("p { margin-bottom: 0 }"), h(NULL))
})

test_that("visual: a styled markdown document", {
  skip_if_not_installed("vdiffr")
  skip_on_os("mac")
  md <- paste(
    "# Styled title", "",
    "A paragraph with $x^2$ and a <span class=\"hot\">warning</span>.", "",
    "> A quoted remark.", "",
    "<div class=\"note\">", "",
    "**Note.** An indented chunk.", "",
    "</div>", "",
    "- alpha", "- beta", "",
    "---",
    sep = "\n")
  vdiffr::expect_doppelganger("markdown-styled", function() {
    grid::grid.draw(markdown_box_grob(
      md,
      width = grid::unit(4.4, "in"),
      padding = grid::unit(10, "pt"),
      box_gp = grid::gpar(fill = "grey98", col = "grey40"),
      style = "
        h1         { color: #2C5F8A; font-size: 1.8rem }
        blockquote { color: #59636E; border-left: 3px solid #2C5F8A }
        .note      { color: #59636E; padding-left: 1.5rem }
        .hot       { color: #B22222; font-weight: bold }
        hr         { border-top: 2px solid #D1D9E0 }
      ",
      gp = grid::gpar(fontsize = 13)
    ))
  })
})

# --- display math blocks -------------------------------------------------

test_that("a lone $$...$$ paragraph becomes a centred mathblock", {
  types <- function(md) vapply(.md_parse_blocks(md), function(b) b$type,
                               character(1))

  # A paragraph that is nothing but display math is its own block.
  expect_identical(types("$$x^2 + y^2 = z^2$$"), "mathblock")
  expect_identical(types("Before\n\n$$\\int_0^1 f(x)dx$$\n\nAfter"),
                   c("paragraph", "mathblock", "paragraph"))

  # Inline math is untouched, and so is display math with prose beside it:
  # the block form is only for a paragraph the span fills completely.
  expect_identical(types("Inline $x^2$ here."), "paragraph")
  expect_identical(types("$$x$$ and text after"), "paragraph")
  expect_identical(types("Text $$x$$ before"), "paragraph")

  # It is centred by default, where a paragraph states no alignment and
  # inherits the box-wide halign instead.
  items <- lay_md("Before\n\n$$x^2$$\n\nAfter")$items
  expect_null(items[[1]]$align)
  expect_equal(items[[2]]$align, 0.5)

  # And it is an ordinary tag, so a stylesheet can move it.
  expect_equal(lay_md("$$x^2$$", "math { text-align: left }")$items[[1]]$align, 0)
})

test_that("a markdown link is styled through the `a` tag", {
  tex <- function(md, style = NULL) .md_to_tex(md, style = .md_as_style(style))
  col <- gridmicrotex:::.MD_LINK_COLOR

  # Blue and underlined by default, as a browser renders <a>.
  expect_equal(tex("[x](u)"),
               paste0("\\textcolor{", col, "}{\\underline{\\text{x}}}"))

  # It is an ordinary tag, so a stylesheet overrides it.
  expect_match(tex("[x](u)", "a { color: red }"), "textcolor\\{#FF0000\\}",
               fixed = FALSE)

  # \underline typesets a fresh sub-formula and would drop the bold around
  # it, so the enclosing style is re-opened inside -- the rule ~~strike~~
  # already follows.
  expect_match(tex("**[x](u)**"), "underline\\{\\\\textbf\\{")

  # The destination is dropped; only the text is kept.
  expect_false(grepl("u}", tex("[x](u)"), fixed = TRUE))
})

test_that("footnotes render markers inline and notes at the foot", {
  types <- function(md) vapply(.md_parse_blocks(md), function(b) b$type,
                               character(1))
  md <- "Body[^one] more[^two].\n\n[^one]: First.\n\n[^two]: Second."

  # CommonMark gathers the definitions at the end already, so the only
  # thing added here is a single separator before the first one.
  expect_identical(types(md),
                   c("paragraph", "thematic_break", "footnote", "footnote"))

  # The marker is a superscript, numbered by reference order.
  expect_match(.md_to_tex("a[^x] b[^y]\n\n[^x]: f\n\n[^y]: s"),
               "textsuperscript\\{1\\}.*textsuperscript\\{2\\}")

  # An inline label has no foot to put notes at: the marker stays, the
  # definition is dropped.
  expect_match(.md_to_tex("Body[^one].\n\n[^one]: note"),
               "textsuperscript\\{1\\}")
  expect_false(grepl("note", .md_to_tex("Body[^one].\n\n[^one]: note"),
                     fixed = TRUE))

  # A document without footnotes is completely unaffected.
  expect_identical(types("Just text."), "paragraph")

  # A reference with no definition is left as the literal text CommonMark
  # gives us, rather than erroring or emitting an empty superscript.
  expect_silent(t <- .md_to_tex("dangling[^nope]"))
  expect_false(grepl("textsuperscript", t, fixed = TRUE))

  # The notes are set smaller, and the tag is styleable like any other.
  expect_equal(.md_cascade(markdown_style(), "footnote")[["font-size"]], 0.85)
  small <- lay_md(md)$items
  big <- lay_md(md, "footnote { font-size: 3 }")$items
  expect_gt(big[[length(big)]]$h, small[[length(small)]]$h)
})

# --- table styling -------------------------------------------------------

# The generated tabular for a two-column table under a given stylesheet.
tbl_tex <- function(style = NULL, avail = 0,
                    md = "| Name | Value |\n|:-----|------:|\n| a | 1 |\n| b | 2 |") {
  st <- .md_as_style(style)
  b <- Filter(function(x) x$type == "table", .md_parse_blocks(md, 12, st))[[1]]
  .md_table_tex(b$nd, b$spans, b$base, st, avail = avail,
                inherited = b$ctx %||% list())
}

test_that("table CSS compiles to the tabular primitives MicroTeX has", {
  # Row and cell fills are different primitives, as in HTML: tr is a row,
  # td/th are cells.
  expect_match(tbl_tex("tr { background: #F6F8FA }"), "rowcolor{#F6F8FA}",
               fixed = TRUE)
  expect_match(tbl_tex("td { background: #DDDDDD }"), "cellcolor{#DDDDDD}",
               fixed = TRUE)
  # A th fill is per-cell, and must not also emit a redundant row fill.
  th <- tbl_tex("th { background: #EEEEEE }")
  expect_match(th, "cellcolor{#EEEEEE}", fixed = TRUE)
  expect_false(grepl("rowcolor", th, fixed = TRUE))

  expect_match(tbl_tex("table { border-color: #D1D9E0 }"),
               "arrayrulecolor{#D1D9E0}", fixed = TRUE)
  # A cell border becomes vertical rules in the column spec.
  expect_match(tbl_tex("td { border-left: 1px solid black }"),
               "{|l|r|}", fixed = TRUE)
  # Cell padding becomes the inter-column @{} material.
  expect_match(tbl_tex("td { padding-left: 6pt }"),
               "@{\\hspace{6.00pt}}", fixed = TRUE)
})

test_that("tr border-bottom rules rows without doubling the bottom rule", {
  t <- tbl_tex("tr { border-bottom: 1px solid black }")
  # One rule between the two body rows, and none before the closing
  # \thickhline, which would otherwise draw two lines on top of each other.
  expect_false(grepl("\\hline \\thickhline", t, fixed = TRUE))
  expect_match(t, "hline", fixed = TRUE)
})

test_that("header cells are bold, which needs parse-time emphasis", {
  # \textbf{\text{x}} reports plain -- \text{} builds a non-nested
  # FontStyleAtom -- so wrapping the finished table could never work. The
  # emphasis has to be applied as each cell is generated.
  expect_match(tbl_tex(), "\\textbf{Name}", fixed = TRUE)
  # And it is a plain rule, so it can be turned off.
  expect_false(grepl("textbf", tbl_tex("th { font-weight: normal }"),
                     fixed = TRUE))
  # font-weight on the table inherits into the cells, as CSS says.
  expect_match(tbl_tex("table { font-weight: bold }"), "\\textbf{a}",
               fixed = TRUE)
})

test_that("table-layout: fixed divides the measure into p{} columns", {
  # Content-sized by default: no p{} anywhere.
  expect_false(grepl("p{", tbl_tex(), fixed = TRUE))
  # Without a known width there is nothing to divide, so it stays content-sized.
  expect_false(grepl("p{", tbl_tex("table { table-layout: fixed }", avail = 0),
                     fixed = TRUE))
  expect_match(tbl_tex("table { table-layout: fixed }", avail = 200),
               "p{", fixed = TRUE)
})

test_that("a fixed-layout table fits the box instead of overflowing", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  wide <- paste0(
    "| Column one | Column two |\n|---|---|\n",
    "| a long sentence that will not fit in a narrow column at all | ",
    "another long sentence that also will not fit anywhere |")
  meas <- function(style) {
    g <- markdown_box_grob(wide, width = grid::unit(3, "in"), style = style,
                           gp = grid::gpar(fontsize = 11))
    lay <- .md_box_layout(g)
    it <- lay$items[[length(lay$items)]]
    c(content = lay$content_w, w = it$w, h = it$h)
  }
  free <- meas(NULL)
  fixed <- meas("table { table-layout: fixed }")

  # The default still overflows -- unchanged behaviour.
  expect_gt(free[["w"]], free[["content"]])
  # Fixed fits, and is taller because the cells now wrap.
  expect_lte(fixed[["w"]], fixed[["content"]] + 1)
  expect_gt(fixed[["h"]], free[["h"]])
})

# --- box properties ------------------------------------------------------

# The LaTeX a class-styled span compiles to.
span_tex <- function(css) {
  .md_to_tex('<span class="x">hi</span>',
             style = .md_as_style(paste0(".x {", css, "}")))
}

test_that("inline box properties compile to MicroTeX primitives", {
  expect_equal(span_tex("background: #FFFF00"),
               "\\bgcolor{#FFFF00}{\\text{hi}}")
  expect_equal(span_tex("text-decoration: overline"),
               "\\overline{\\text{hi}}")
  expect_equal(span_tex("border: 1px solid red"), "\\fbox{\\text{hi}}")
  expect_equal(span_tex("border-style: double"), "\\doublebox{\\text{hi}}")
  expect_equal(span_tex("border-radius: 4pt"), "\\ovalbox{\\text{hi}}")
  expect_equal(span_tex("box-shadow: 2pt 2pt"), "\\shadowbox{\\text{hi}}")
  expect_equal(span_tex("visibility: hidden"), "\\phantom{\\text{hi}}")
  expect_equal(span_tex("vertical-align: super"),
               "\\textsuperscript{\\text{hi}}")
  expect_equal(span_tex("vertical-align: sub"), "\\textsubscript{\\text{hi}}")
  expect_equal(span_tex("vertical-align: 3pt"),
               "\\raisebox{3.00pt}{\\text{hi}}")
  expect_equal(span_tex("transform: rotate(45deg)"),
               "\\rotatebox{45}{\\text{hi}}")
  expect_equal(span_tex("transform: scaleX(-1)"),
               "\\reflectbox{\\text{hi}}")

  # A border takes the background as \fcolorbox's fill, so \bgcolor must
  # not paint it a second time.
  expect_equal(span_tex("border: 1px solid red; background: #EEEEEE"),
               "\\fcolorbox{#FF0000}{#EEEEEE}{\\text{hi}}")
})

test_that("values MicroTeX has no primitive for are ignored, not errors", {
  for (css in c("transform: skew(3deg)", "visibility: visible",
                "box-shadow: none", "vertical-align: nonsense")) {
    expect_equal(span_tex(css), "\\text{hi}", info = css)
  }
})

test_that("inline box properties render", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  for (css in c("background: #FFFF00", "border: 1px solid red",
                "border-style: double", "border-radius: 4pt",
                "box-shadow: 2pt 2pt", "vertical-align: super",
                "transform: rotate(45deg)", "text-decoration: overline")) {
    expect_gt(nrow(latex_tree(span_tex(css), input_mode = "math",
                              gp = grid::gpar(fontsize = 16))$records), 0,
              label = css)
  }
  # \phantom is the exception: it takes space and draws nothing.
  expect_equal(nrow(latex_tree(span_tex("visibility: hidden"),
                               input_mode = "math")$records), 0)
})

test_that("blocks take padding and margins on all four sides", {
  md <- "Alpha\n\nBravo"
  base <- lay_md(md)
  # Left padding and left margin both shift the block right.
  expect_equal(lay_md(md, "p { padding-left: 20pt }")$items[[2]]$x, 20)
  expect_equal(lay_md(md, "p { margin-left: 15pt }")$items[[2]]$x, 15)
  # Vertical padding grows the document.
  expect_gt(lay_md(md, "p { padding-top: 10pt }")$height, base$height)
  expect_gt(lay_md(md, "p { padding-bottom: 10pt }")$height, base$height)
  # Right padding narrows the measure, so long text wraps sooner.
  long <- paste(rep("word", 40), collapse = " ")
  expect_gt(lay_md(long, "p { padding-right: 120pt }")$height,
            lay_md(long)$height)
})

test_that("a block background is a rect drawn behind the block", {
  md <- "Alpha\n\nBravo"
  plain <- lay_md(md)
  bg <- lay_md(md, "p { background: #EEEEEE }")

  # One extra item per styled block, and it comes first so it draws behind.
  expect_equal(length(bg$items), length(plain$items) + 2L)
  expect_s3_class(bg$items[[1]]$grob, "rect")

  # It spans the column -- the thing no MicroTeX box command can do, since
  # every one of them hugs its content.
  expect_equal(bg$items[[1]]$w, 300)

  # And it covers the padding, not just the text.
  padded <- lay_md(md, "p { background: #EEE; padding-top: 10pt }")
  expect_gt(padded$items[[1]]$h, bg$items[[1]]$h)
})

test_that("strong and em are styleable tags, like code", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  cols <- function(md, style) unique(stats::na.omit(
    markdown_grob(md, style = .md_as_style(style))$layout_df$color))

  # The tag vocabulary is HTML's names, and ** / * produce strong / em,
  # so a rule on those names has to reach them -- <code> always did.
  expect_equal(cols("**b**", "strong { color: #B22222 }"), "#B22222")
  expect_equal(cols("*i*", "em { color: #1F6FB2 }"), "#1F6FB2")

  # Only the styled run changes.
  expect_setequal(cols("plain **b** plain", "strong { color: #B22222 }"),
                  c("#000000", "#B22222"))

  # The emphasis itself must survive being wrapped in a colour.
  tex <- .md_to_tex("**b**", style = .md_as_style("strong { color: #B22222 }"))
  expect_match(tex, "\\textbf{", fixed = TRUE)
  expect_match(tex, "\\textcolor{#B22222}", fixed = TRUE)

  # Unstyled output is byte-identical to before the tags were wired up.
  expect_equal(.md_to_tex("**b**"), "\\textbf{b}")
  expect_equal(.md_to_tex("*i*"), "\\textit{i}")
  expect_equal(.md_to_tex("***both***"), "\\textit{\\textbf{both}}")
})

test_that("body {} styles the box itself", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # These are box properties, so .md_cascade() strips them out of the seed
  # it hands the blocks -- a background on `body` must not become a
  # background on every paragraph. That leaves the box as the only thing
  # that can consume them.
  plain <- .md_box_layout(markdown_box_grob("a note", width = grid::unit(3, "in")))
  expect_null(plain$box_gp)
  expect_equal(plain$padding, c(0, 0, 0, 0))

  styled <- .md_box_layout(markdown_box_grob(
    "a note", width = grid::unit(3, "in"),
    style = paste("body { background: #EEF3FB; padding: 10px;",
                  "border: 2px solid steelblue; border-radius: 5px }")))
  expect_equal(styled$box_gp$fill, "#EEF3FB")
  expect_false(is.na(styled$box_gp$col))
  expect_equal(styled$padding, rep(7.5, 4))   # 10px == 7.5bp
  expect_false(is.null(styled$r))

  # And the rect is actually emitted, not merely resolved.
  kids <- grid::makeContent(markdown_box_grob(
    "a note", width = grid::unit(3, "in"),
    style = "body { background: #EEF3FB }"))$children
  expect_true(any(vapply(kids, function(z) inherits(z, "rect"), logical(1))))
})

test_that("an explicit box argument beats the body rule", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # The argument is the inline style, and inline wins.
  lay <- .md_box_layout(markdown_box_grob(
    "a note", width = grid::unit(3, "in"),
    padding = grid::unit(2, "bigpts"), style = "body { padding: 30px }"))
  expect_equal(lay$padding, rep(2, 4))
})

test_that("the padding/margin shorthand expands to the longhands", {
  one <- .md_expand_shorthand(list(padding = "4px"))
  expect_null(one[["padding"]])
  expect_equal(unlist(one[paste0("padding", c("-top", "-right", "-bottom",
                                              "-left"))]),
               c("padding-top" = "4px", "padding-right" = "4px",
                 "padding-bottom" = "4px", "padding-left" = "4px"))

  # CSS's own order: vertical/horizontal, then top/horizontal/bottom.
  two <- .md_expand_shorthand(list(margin = "1px 2px"))
  expect_equal(two[["margin-top"]], "1px")
  expect_equal(two[["margin-right"]], "2px")
  expect_equal(two[["margin-bottom"]], "1px")
  expect_equal(two[["margin-left"]], "2px")

  three <- .md_expand_shorthand(list(margin = "1px 2px 3px"))
  expect_equal(three[["margin-bottom"]], "3px")
  expect_equal(three[["margin-left"]], "2px")

  # A longhand stated alongside the shorthand is the more specific one.
  mix <- .md_expand_shorthand(list(padding = "4px", "padding-left" = "9px"))
  expect_equal(mix[["padding-left"]], "9px")
  expect_equal(mix[["padding-top"]], "4px")

  # It reaches every route a declaration can arrive by.
  expect_equal(md_style(padding = "4px")[["padding-top"]], "4px")
  expect_equal(.md_parse_css("padding: 4px")[["padding-top"]], "4px")
})

test_that("background-color is accepted as a spelling of background", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # `background` is the shorthand this package stores; `background-color`
  # is the real CSS property and what anyone writing a stylesheet types.
  # It used to be dropped in silence, so the fill simply never appeared.
  fills <- function(...) {
    d <- markdown_grob(..., gp = grid::gpar(fontsize = 14))$layout_df
    sum(d$type %in% c("fill_rect", "fill_roundrect"))
  }
  expect_equal(fills("a <span style='background:yellow'>S</span> c"), 1L)
  expect_equal(fills("a <span style='background-color:yellow'>S</span> c"), 1L)
  expect_equal(
    fills("a <span class='hi'>S</span> c",
          style = markdown_style(.hi = md_style(background_color = "yellow"))),
    1L)
  expect_equal(
    fills("a <span class='hi'>S</span> c",
          style = ".hi { background-color: yellow; }"),
    1L)

  # Both spellings land on one stored name, so nothing downstream has to
  # know there were two.
  expect_equal(names(.md_parse_css("background-color: red")), "background")
  expect_equal(names(md_style(background_color = "red")), "background")
})

test_that("an unsupported property in a hand-written style= warns", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # md_style() errors on a misspelt property; a `style` attribute used to
  # swallow it, so `colour: red` did nothing with no way to notice.
  expect_warning(
    markdown_grob("a <span style='colour:red'>S</span> c"),
    "colour")
  expect_warning(
    markdown_box_grob("<div style='nonsense:1'>\n\ntext\n\n</div>",
                      width = grid::unit(2, "in")),
    "nonsense")

  # A property that is supported must stay quiet.
  expect_no_warning(markdown_grob("a <span style='color:red'>S</span> c"))
  expect_no_warning(markdown_grob("a <span style='background-color:red'>S</span> c"))

  # A pasted stylesheet is deliberately exempt: it is expected to carry
  # properties this package cannot honour, and warning about those would
  # punish exactly the leniency that makes pasting one safe.
  expect_no_warning(
    markdown_grob("a <span class='x'>S</span> c",
                  style = ".x { float: left; display: block; color: red; }"))
})

# Font name aliases for convenience
.font_aliases <- c(
  "stix"     = "STIX Two Math",
  "stix2"    = "STIX Two Math",
  "lete"     = "Lete Sans Math",
  "letesans" = "Lete Sans Math"
)

#' Resolve a math font name
#'
#' Translates short aliases (e.g., \code{"stix"}, \code{"lete"}) to the
#' full MicroTeX font name. Validates that the font is loaded.
#'
#' @param name Font name or alias. Empty string uses the default font.
#' @return The resolved font name.
#' @noRd
resolve_math_font <- function(name) {
  if (is.null(name) || !nzchar(name)) return("")

  # Check aliases first
  lower <- tolower(name)
  if (lower %in% names(.font_aliases)) {
    return(.font_aliases[[lower]])
  }

  # Check if it matches a loaded font (case-insensitive). This also
  # covers the exact-match case.
  loaded <- microtex_math_font_names()
  idx <- match(tolower(name), tolower(loaded))
  if (!is.na(idx)) {
    return(loaded[idx])
  }

  stop(
    "Math font '", name, "' not found. Available fonts: ",
    paste(loaded, collapse = ", "),
    "\nAliases: ", paste(names(.font_aliases), collapse = ", "),
    call. = FALSE
  )
}

#' List available math fonts
#'
#' Returns the names of all math fonts currently loaded by MicroTeX.
#' These names can be passed to the \code{math_font} parameter of
#' \code{\link{latex_grob}} and \code{\link{grid.latex}}.
#'
#' @section Font pairing:
#' The bundled math fonts have different styles. For a consistent look,
#' pair them with a matching \code{fontfamily} in \code{gp}:
#'
#' \tabular{lll}{
#'   \strong{Math font}     \tab \strong{Style}  \tab \strong{Suggested text font} \cr
#'   Lete Sans Math (\code{"lete"}, default) \tab Sans-serif \tab \code{"sans"} \cr
#'   STIX Two Math (\code{"stix"})   \tab Serif  \tab \code{"serif"} \cr
#' }
#' Additional math fonts can be loaded with \code{\link{load_math_font}}.
#'
#' @return A character vector of math font names.
#' @export
#'
#' @examples
#' available_math_fonts()
available_math_fonts <- function() {
  microtex_math_font_names()
}

# Internal: set the default math font used by MicroTeX.
# Public entry point is `latex_options(math_font = ...)`.
.set_math_font <- function(name) {
  if (!microtex_is_inited()) {
    stop("MicroTeX is not initialized.", call. = FALSE)
  }

  if (is.null(name) || !nzchar(name)) {
    stop(
      "Please provide a math font name. Use available_math_fonts() to list choices.",
      call. = FALSE
    )
  }

  resolved <- resolve_math_font(name)
  ok <- microtex_set_default_math_font(resolved)
  if (!ok) {
    stop(
      "Failed to set math font '", resolved, "'. Available fonts: ",
      paste(available_math_fonts(), collapse = ", "),
      call. = FALSE
    )
  }

  invisible(TRUE)
}

#' Load a math font from an OTF file
#'
#' Loads an OTF/TTF \strong{math} font --- one carrying an OpenType MATH
#' table --- into MicroTeX's internal font registry. The MATH table is
#' parsed directly in C++ and the required metrics are synthesised on the
#' fly. You can download a free math font such as Latin Modern Math
#' (the LaTeX default) and load it for math rendering.
#'
#' The font is also registered with the \pkg{systemfonts} package so it
#' can be selected for surrounding plot text via
#' \code{gp = gpar(fontfamily = "...")} without being installed
#' system-wide.
#'
#' Plain \strong{text} fonts --- those used inside \code{\\text\{\}} blocks
#' --- need no loading at all. They are resolved automatically by
#' \pkg{systemfonts} from \code{gp$fontfamily}, or per run with
#' \code{\\gmfontfamily\{\}\{\}}.
#'
#' @param otf_path Path to the OTF/TTF font file.
#' @return Invisibly returns \code{NULL}.
#' @seealso \code{\link{available_math_fonts}}, \code{\link{check_math_fonts}},
#'   \code{\link{latex_options}}, \code{\link{latex_grob}}
#' @export
#'
#' @examples
#' \donttest{
#'   # Load a math font from a local OTF file. Here we point at the
#'   # bundled STIX font so the example is self-contained and loaded.
#'   # You don't need to load the bundled fonts to use them — they're registered
#'   # with systemfonts on first render — but this shows how to load a custom font.
#'   # in practice you would pass the path to any OTF with an OpenType MATH table.
#'   otf <- system.file("fonts", "STIXTwoMath-Regular.otf",
#'                      package = "gridmicrotex")
#'   load_math_font(otf)
#'   available_math_fonts()
#' }
load_math_font <- function(otf_path) {
  if (!file.exists(otf_path)) {
    stop("Font file not found: ", otf_path, call. = FALSE)
  }

  # Reject TrueType Collections — MicroTeX::addFont expects a single face.
  header <- tryCatch(
    readBin(otf_path, what = "raw", n = 4L),
    error = function(e) raw()
  )
  if (length(header) == 4L && identical(header, charToRaw("ttcf"))) {
    stop(
      "TrueType Collection (.ttc) files are not supported.\n",
      "Extract a single face (.otf/.ttf) and pass that instead.\n",
      "File: ", otf_path,
      call. = FALSE
    )
  }

  display <- microtex_add_font_from_otf(otf_path, 0L)
  if (!nzchar(display)) {
    stop(
      "Could not read OpenType MATH table from: ", basename(otf_path), "\n",
      "The font may not be a math font, or may have an unsupported MATH ",
      "table layout.",
      call. = FALSE
    )
  }

  # Make the font selectable via gp = gpar(fontfamily = <name>).
  .register_font_with_systemfonts(otf_path, display)

  invisible(NULL)
}

# Register a math font with systemfonts so gp$fontfamily = <name> (or any
# alias) resolves to `otf_path` for grid text drawing. Silent on failure
# — registration is best-effort; failing only means gp$fontfamily won't
# resolve to the bundled OTF.
.register_font_with_systemfonts <- function(otf_path, display_name,
                                            aliases = character(0)) {
  names <- unique(c(display_name, aliases))
  for (nm in names) {
    try(
      systemfonts::register_font(name = nm, plain = otf_path),
      silent = TRUE
    )
  }
  invisible()
}

# Session-level flag for the bundled-font systemfonts registration. Kept
# out of .onLoad so we don't touch Core Text at namespace-load time on
# macOS — older SDKs print "XType: Using static font registry." to stderr,
# which R CMD check captures and flags across many check phases. Running
# this lazily on first render keeps the check log clean.
.fonts_state <- new.env(parent = emptyenv())
.fonts_state$registered <- FALSE

.ensure_bundled_fonts_registered <- function() {
  if (isTRUE(.fonts_state$registered)) return(invisible())
  .fonts_state$registered <- TRUE  # set first so a failure isn't retried each call

  lete <- system.file("fonts", "LeteSansMath.otf", package = "gridmicrotex")
  if (nzchar(lete)) {
    .register_font_with_systemfonts(lete, "Lete Sans Math", aliases = "lete")
  }

  stix <- system.file("fonts", "STIXTwoMath-Regular.otf", package = "gridmicrotex")
  if (nzchar(stix)) {
    .register_font_with_systemfonts(stix, "STIX Two Math", aliases = "stix")
  }

  invisible()
}

#' Check math font status
#'
#' Reports which \strong{math} fonts are loaded and available for
#' rendering: the MicroTeX version, the loaded math fonts, and whether the
#' bundled font files are present.
#'
#' Text fonts are not covered, because they are not registered here ---
#' they are resolved on demand by \pkg{systemfonts} from
#' \code{gp$fontfamily}. Use \code{systemfonts::match_fonts()} to see
#' what a text family resolves to.
#'
#' @return Invisibly returns the character vector of available math font
#'   names.
#' @seealso \code{\link{available_math_fonts}}, \code{\link{load_math_font}}
#' @export
#'
#' @examples
#' check_math_fonts()
check_math_fonts <- function() {
  if (!microtex_is_inited()) {
    message("MicroTeX is not initialized.")
    return(invisible(character(0)))
  }

  fonts <- microtex_math_font_names()
  message("MicroTeX version: ", microtex_version())
  message("Loaded math fonts (", length(fonts), "):")
  for (f in fonts) {
    message("  - ", f)
  }

  pkg <- "gridmicrotex"
  message("Bundled font files:")
  for (file in c("LeteSansMath.otf", "STIXTwoMath-Regular.otf")) {
    p <- system.file("fonts", file, package = pkg)
    message("  - ", file, ": ", if (nzchar(p)) "found" else "MISSING")
  }

  invisible(fonts)
}

test_that("bundled default math font (Lete) loads and resolves", {
  fonts <- available_math_fonts()
  expect_true("Lete Sans Math" %in% fonts)

  expect_equal(gridmicrotex:::resolve_math_font("lete"), "Lete Sans Math")
  expect_equal(gridmicrotex:::resolve_math_font("letesans"), "Lete Sans Math")
  expect_error(gridmicrotex:::resolve_math_font("nonexistent"), "not found")

  g <- latex_grob("\\frac{a}{b}", math_font = "lete",
                  gp = grid::gpar(fontsize = 20))
  expect_s3_class(g, "latexgrob")
})

test_that("bundled STIX math font loads, resolves aliases, and renders", {
  fonts <- available_math_fonts()
  expect_true("STIX Two Math" %in% fonts)

  expect_equal(gridmicrotex:::resolve_math_font("stix"), "STIX Two Math")
  expect_equal(gridmicrotex:::resolve_math_font("stix2"), "STIX Two Math")

  # Switching the math font has to be undone explicitly, and neither
  # do.call(latex_options, old) nor reset_latex_options() is enough:
  #
  #  * latex_options() ignores NULL arguments -- that is how it separates
  #    "leave this alone" from "set it" -- so replaying an all-NULL `old`
  #    (the state when nothing has been set yet) restores nothing.
  #  * latex_options(math_font=) also calls .set_math_font(), which sets
  #    MicroTeX's default font in C++. reset_latex_options() only clears
  #    the R-side list, leaving the engine still on STIX.
  #
  # Left unrestored, every later test file renders in the wrong font --
  # which is exactly how the visual snapshots came to be recorded against
  # STIX rather than the default Lete.
  latex_options(math_font = "stix")
  expect_equal(latex_options()$math_font, "stix")

  latex_options(math_font = "lete")  # restores the engine default
  reset_latex_options()              # and clears the R-side record
  expect_null(latex_options()$math_font)

  g <- latex_grob("\\frac{a}{b}", math_font = "stix",
                  gp = grid::gpar(fontsize = 20))
  expect_s3_class(g, "latexgrob")
})

test_that("ot_math_table_bytes returns the OT MATH blob for Lete", {
  otf <- system.file("fonts", "LeteSansMath.otf", package = "gridmicrotex")
  expect_true(nzchar(otf))

  blob <- gridmicrotex:::ot_math_table_bytes(otf)
  expect_type(blob, "raw")
  expect_gt(length(blob), 4)

  expect_identical(blob[1:4], as.raw(c(0x00, 0x01, 0x00, 0x00)))
})

test_that("ot_math_table_bytes errors on a path that cannot be opened", {
  expect_error(
    gridmicrotex:::ot_math_table_bytes(tempfile(fileext = ".otf")),
    "failed to open font"
  )
})

test_that("otf_to_clm_bytes emits a CLM v6 blob for Lete", {
  otf <- system.file("fonts", "LeteSansMath.otf", package = "gridmicrotex")
  clm <- gridmicrotex:::otf_to_clm_bytes(otf)
  expect_type(clm, "raw")
  # "clm" magic + major=6 + minor=2 (with glyph paths)
  expect_identical(clm[1:3], charToRaw("clm"))
  expect_identical(clm[4:5], as.raw(c(0x00, 0x06)))
  expect_identical(clm[6],   as.raw(0x02))
})

test_that("load_math_font synthesises CLM from a bare OTF", {
  # Copy Lete.otf to a temp file with a unique name so it registers as a
  # distinct math font (not colliding with the bundled "Lete Sans Math"
  # already loaded at .onLoad). Exercises the MATH-table synthesis path
  # end-to-end.
  src <- system.file("fonts", "LeteSansMath.otf", package = "gridmicrotex")
  skip_if_not(nzchar(src))
  tmp <- file.path(tempdir(), "A3Probe.otf")
  file.copy(src, tmp, overwrite = TRUE)
  on.exit(unlink(tmp), add = TRUE)

  # load_math_font is silent on success; it might warn if systemfonts
  # reports a duplicate registration.
  suppressWarnings(load_math_font(tmp))
  # The font registers under its family name (Lete Sans Math) — same as
  # the bundled font because it IS the same OTF.
  expect_true("Lete Sans Math" %in% available_math_fonts())
})

test_that("the deprecated aliases still work, and say so", {
  src <- system.file("fonts", "LeteSansMath.otf", package = "gridmicrotex")
  skip_if_not(nzchar(src))
  tmp <- file.path(tempdir(), "A3ProbeDeprecated.otf")
  file.copy(src, tmp, overwrite = TRUE)
  on.exit(unlink(tmp), add = TRUE)

  expect_warning(suppressMessages(load_font(tmp)), "deprecated")
  expect_true("Lete Sans Math" %in% available_math_fonts())

  expect_warning(suppressMessages(check_fonts()), "deprecated")
  # The alias forwards rather than reimplementing, so it returns what the
  # replacement returns.
  expect_identical(
    suppressWarnings(suppressMessages(check_fonts())),
    suppressMessages(check_math_fonts())
  )
})

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dpi = 300,
  dev = "ragg_png"
)


## ----basic, fig.height=0.8, fig.width=2, out.width="50%"----------------------
library(gridmicrotex)
library(grid)

g <- latex_grob("\\frac{-b \\pm \\sqrt{b^2 - 4ac}}{2a}", gp = grid::gpar(fontsize = 24))
grid::grid.newpage()
grid::grid.draw(g)

## ----quick, fig.height=0.8, fig.width=3, out.width="40%"----------------------
grid::grid.newpage()
grid.latex("$\\sum_{i=1}^{n} x_i^2$", gp = grid::gpar(fontsize = 28))

## ----positioning, fig.height=1.2, fig.width=3.5, out.width="40%"--------------
grid::grid.newpage()
grid.latex("Famous $E = mc^2$", x = 0.1, y = 0.7, hjust = 0, gp = grid::gpar(fontsize = 24))
grid.latex("F = ma", x = 0.1, y = 0.3, hjust = 0, gp = grid::gpar(fontsize = 24), input_mode = "math")

## ----baseline-align, fig.height=0.8, fig.width=4.5, out.width="60%"-----------
grid::grid.newpage()
y <- 0.5
grid::grid.segments(unit(0, "npc"), unit(y, "npc"),
                    unit(1, "npc"), unit(y, "npc"),
                    gp = grid::gpar(col = "grey80"))
grid::grid.text("if ", x = 0.10, y = y, just = c(0, 0.5),
                gp = grid::gpar(fontsize = 16))
grid.latex("$x \\geq \\sqrt{2\\pi}$",
           x = 0.22, y = y,
           hjust = "left", vjust = "baseline",
           gp = grid::gpar(fontsize = 16))
grid::grid.text(", then proceed.", x = 0.62, y = y, just = c(0, 0.5),
                gp = grid::gpar(fontsize = 16))

## ----mark, fig.height=2.4, fig.width=5, out.width="70%"-----------------------
g <- latex_grob(
  r"($a^2 + b\mark{term}^2 \mark{equals}= c^2$)",
  x = 0.5, y = 0.4,
  gp = grid::gpar(fontsize = 28)
)
grid::grid.newpage()
grid::grid.draw(g)

# Callout 1: the "=" sign, pointed at from above.
mk_eq <- grobMark(g, "equals")
grid::grid.segments(mk_eq$x, mk_eq$y + unit(15, "mm"),
                    mk_eq$x, mk_eq$y + unit(3, "mm"),
                    arrow = grid::arrow(length = unit(2, "mm"), type = "closed"),
                    gp = grid::gpar(col = "red"))
grid::grid.text("equals", x = mk_eq$x, y = mk_eq$y + unit(18, "mm"),
                gp = grid::gpar(col = "red", fontsize = 11))

# Callout 2: the b^2 term, pointed at from below — the mark sits at the
# end of the term, including the superscript's smaller scale.
mk_bsq <- grobMark(g, "term")
grid::grid.segments(mk_bsq$x - unit(6, "mm"), mk_bsq$y - unit(15, "mm"),
                    mk_bsq$x - unit(2, "mm"), mk_bsq$y - unit(3, "mm"),
                    arrow = grid::arrow(length = unit(2, "mm"), type = "closed"),
                    gp = grid::gpar(col = "blue"))
grid::grid.text("b² term",
                x = mk_bsq$x - unit(7, "mm"),
                y = mk_bsq$y - unit(18, "mm"),
                just = "right",
                gp = grid::gpar(col = "blue", fontsize = 11))

## ----colors, fig.height=0.6, fig.width=2, out.width="40%"---------------------
latex_options(input_mode = "math") # Set math mode globally
grid::grid.newpage()
grid.latex(
  r"(\textcolor{red}{\alpha} + \textcolor{blue}{\beta} = \gamma)",
  gp = grid::gpar(fontsize = 28)
)

## ----fonts-list---------------------------------------------------------------
available_math_fonts()

## ----fonts-default, fig.height=0.8, fig.width=2, out.width="50%"--------------
latex_options(math_font = "stix")
grid::grid.newpage()
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24))

# Switch back to the default (Lete Sans Math)
latex_options(math_font = "lete")

## ----fonts, fig.height=1.5, fig.width=2, out.width="50%"----------------------
grid::grid.newpage()
grid::pushViewport(grid::viewport(layout = grid::grid.layout(2, 1)))
grid::pushViewport(grid::viewport(layout.pos.row = 1))
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24))
grid::upViewport()
grid::pushViewport(grid::viewport(layout.pos.row = 2))
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24), math_font = "stix")
grid::upViewport(2)

## ----dims---------------------------------------------------------------------
dims <- latex_dims("\\frac{a}{b}", gp = grid::gpar(fontsize = 20))
dims

## ----cjk, fig.height = 0.8, fig.width = 2.5, out.width="50%"------------------
grid::grid.newpage()
grid.latex("x^2 + \\text{你好}", gp = grid::gpar(fontsize = 24, fontfamily = "sans"))

## ----font-pairing, fig.height = 0.4, fig.width = 2, out.width="70%"-----------
grid::grid.newpage()
grid.latex(
  "\\text{Theorem: } \\forall x \\in \\mathbb{R},\\; x^2 \\geq 0",
  math_font = "stix",
  gp = grid::gpar(fontfamily = "serif", fontsize = 12)
)

## ----complex-formula, fig.height = 3, fig.width=6, out.width="60%"------------
grid::grid.newpage()
grid.latex(paste0(
      "\\begin{array}{l}",
      "  \\forall\\varepsilon\\in\\mathbb{R}_+^*\\ \\exists\\eta>0",
      "\\ |x-x_0|\\leq\\eta\\Longrightarrow|f(x)-f(x_0)|\\leq\\varepsilon\\\\",
      "  \\det",
      "  \\begin{bmatrix}",
      "      a_{11}&a_{12}&\\cdots&a_{1n}\\\\",
      "      a_{21}&\\ddots&&\\vdots\\\\",
      "      \\vdots&&\\ddots&\\vdots\\\\",
      "      a_{n1}&\\cdots&\\cdots&a_{nn}",
      "  \\end{bmatrix}",
      "  \\overset{\\mathrm{def}}{=}\\sum_{\\sigma\\in\\mathfrak{S}_n}",
      "\\varepsilon(\\sigma)\\prod_{k=1}^n a_{k\\sigma(k)}\\\\",
      "  \\int_0^\\infty{x^{2n} e^{-a x^2}\\,dx} = \\frac{2n-1}{2a}",
      " \\int_0^\\infty{x^{2(n-1)} e^{-a x^2}\\,dx}",
      " = \\frac{(2n-1)!!}{2^{n+1}} \\sqrt{\\frac{\\pi}{a^{2n+1}}}\\\\",
      "\\end{array}"
), gp = grid::gpar(fontsize = 16))

## ----table-multicolumn, fig.height = 3, fig.width=8, out.width="60%"----------
grid::grid.newpage()

grid.latex(
  "
  \\newcolumntype{s}{>{\\color{#1234B6}}c}
\\begin{array}{|c|c|c|s|}
  \\hline
  \\rowcolor{Tan}\\multicolumn{4}{|c|}{\\textcolor{white}{\\bold{\\text{Table Head}}}}\\\\
  \\hline
  \\text{Matrix}&\\multicolumn{2}{|c|}{\\text{Multicolumns}}&\\text{Font size commands}\\\\
  \\hline
  \\begin{pmatrix}
      \\alpha_{11}&\\cdots&\\alpha_{1n}\\\\
      \\hdotsfor{3}\\\\
      \\alpha_{n1}&\\cdots&\\alpha_{nn}
  \\end{pmatrix}
  &\\large \\text{Left}&\\cellcolor{#00bde5}\\small \\textcolor{white}{\\text{\\bold{Right}}}
  &\\small \\text{small Small}\\\\
  \\hline
  \\multicolumn{4}{|c|}{\\text{Table Foot}}\\\\
  \\hline
\\end{array}
  ",
  gp = grid::gpar(fontsize = 22)
)

## ----complicated-equation, fig.height = 9, fig.width=9, out.width="60%"-------
grid::grid.newpage()
grid.latex(
  "\\definecolor{gris}{gray}{0.9}
\\definecolor{noir}{rgb}{0,0,0}
\\definecolor{bleu}{rgb}{0,0,1}
\\fatalIfCmdConflict{false}
\\newcommand{\\pa}{\\left|}
\\begin{array}{c}
  \\LaTeX\\\\
  \\begin{split}
      |I_2| &= \\pa\\int_0^T\\psi(t)\\left\\{ u(a,t)-\\int_{\\gamma(t)}^a \\frac{d\\theta}{k} (\\theta,t) \\int_a^\\theta c(\\xi)
          u_t (\\xi,t)\\,d\\xi\\right\\}dt\\right|\\\\
      &\\le C_6 \\Bigg|\\pa f \\int_\\Omega \\pa\\widetilde{S}^{-1,0}_{a,-}
          W_2(\\Omega, \\Gamma_1)\\right|\\ \\right|\\left| |u|\\overset{\\circ}{\\to} W_2^{\\widetilde{A}}(\\Omega\\Gamma_r,T)\\right|\\Bigg|\\\\
      &\\\\
      &\\begin{pmatrix}
          \\alpha&\\beta&\\gamma&\\delta\\\\
          \\aleph&\\beth&\\gimel&\\daleth\\\\
          \\mathfrak{A}&\\mathfrak{B}&\\mathfrak{C}&\\mathfrak{D}\\\\
          \\boldsymbol{\\mathfrak{a}}&\\boldsymbol{\\mathfrak{b}}&\\boldsymbol{\\mathfrak{c}}&\\boldsymbol{\\mathfrak{d}}
      \\end{pmatrix}
      \\quad{(a+b)}^{\\frac{n}{2}}=\\sqrt{\\sum_{k=0}^n\\tbinom{n}{k}a^kb^{n-k}}\\quad
          \\Biggl(\\biggl(\\Bigl(\\bigl(()\\bigr)\\Bigr)\\biggr)\\Biggr)\\\\
      &\\forall\\varepsilon\\in\\mathbb{R}_+^*\\ \\exists\\eta>0\\ |x-x_0|\\leq\\eta\\Longrightarrow|f(x)-f(x_0)|\\leq\\varepsilon\\\\
      &\\det
      \\begin{bmatrix}
          a_{11}&a_{12}&\\cdots&a_{1n}\\\\
          a_{21}&\\ddots&&\\vdots\\\\
          \\vdots&&\\ddots&\\vdots\\\\
          a_{n1}&\\cdots&\\cdots&a_{nn}
      \\end{bmatrix}
      \\overset{\\mathrm{def}}{=}\\sum_{\\sigma\\in\\mathfrak{S}_n}\\varepsilon(\\sigma)\\prod_{k=1}^n a_{k\\sigma(k)}\\\\
      &\\Delta f(x,y)=\\frac{\\partial^2f}{\\partial x^2}+\\frac{\\partial^2f}{\\partial y^2}\\qquad\\qquad \\fcolorbox{noir}{gris}
          {n!\\underset{n\\rightarrow+\\infty}{\\sim} {\\left(\\frac{n}{e}\\right)}^n\\sqrt{2\\pi n}}\\\\
      &\\sideset{_\\alpha^\\beta}{_\\gamma^\\delta}{
      \\begin{pmatrix}
          a&b\\\\
          c&d
      \\end{pmatrix}}
      \\xrightarrow[T]{n\\pm i-j}\\sideset{^t}{}A\\xleftarrow{\\overrightarrow{u}\\wedge\\overrightarrow{v}}
          \\underleftrightarrow{\\iint_{\\mathds{R}^2}e^{-\\left(x^2+y^2\\right)}\\,\\mathrm{d}x\\mathrm{d}y}
  \\end{split}\\\\
  \\rotatebox{30}{\\sum_{n=1}^{+\\infty}}\\quad\\mbox{Mirror rorriM}\\reflectbox{\\mbox{Mirror rorriM}}
\\end{array}",
  gp = grid::gpar(fontsize = 22),
  render_mode = "path"
)

## ----options, eval=FALSE------------------------------------------------------
# latex_options(math_font = "stix", render_mode = "typeface")
# 
# # Later calls pick these up automatically
# grid.latex("\\sum_{i=1}^{n} i^{2}", gp = grid::gpar(fontsize = 14))
# 
# # Query current settings
# latex_options()
# 
# # Reset to built-in defaults
# reset_latex_options()

## ----macros, fig.height=0.7, fig.width=3, out.width="50%"---------------------
define_macro("RR", "\\mathbb{R}")
define_macro("eps", "\\varepsilon")

grid::grid.newpage()
grid.latex("\\forall \\eps > 0, \\eps \\in \\RR", gp = grid::gpar(fontsize = 24))

clear_macros()

## ----def-inline, fig.height=0.7, fig.width=4, out.width="60%"-----------------
grid::grid.newpage()
grid.latex(
  r"(\def\norm#1{\left\lVert #1 \right\rVert}
      \def\inner#1#2{\langle #1, #2 \rangle}
      \norm{\vec{v}} = \sqrt{\inner{\vec{v}}{\vec{v}}})",
  gp = grid::gpar(fontsize = 24)
)

## ----cache, eval=FALSE--------------------------------------------------------
# latex_cache_info()       # size / max_size / hits / misses
# latex_cache_limit(1024)  # raise or lower the LRU capacity
# latex_cache_clear()      # wipe the cache (e.g. after re-loading fonts)

## ----tree---------------------------------------------------------------------
tr <- latex_tree("\\frac{a}{b}")
tr
head(tr$records, 3)

## ----debug, fig.height=1, fig.width=3, out.width="60%"------------------------
grid::grid.newpage()
grid.latex("x^{2} + y_{i}", gp = grid::gpar(fontsize = 30), debug = TRUE)

# Markdown -> LaTeX -> grid: the inline half.
#
# One run of text, no block layout -- what markdown_grob() produces.
# Block documents are in markdown-box.R, which builds on this file for
# both the inline walker and the CSS value resolvers below.
#
# Pipeline:
#   md -> .md_mask_math()          hide math from the markdown parser
#      -> commonmark::markdown_xml()  CommonMark AST
#      -> xml2::read_xml()
#      -> .md_inline_to_tex()      walk the AST, emit LaTeX
#      -> latex_grob(input_mode = "math")
#
# Why mask first: CommonMark treats a backslash before ASCII punctuation
# as an escape, so `$\begin{matrix}a\\b\end{matrix}$` loses its row
# separator (`\\` collapses to `\`) before we ever see the AST. Masking
# each math span behind a sentinel keeps the LaTeX byte-exact. It also
# stops `*` and `_` inside formulas from being read as emphasis.

# Sentinels use Unicode private-use codepoints, which cannot appear in
# real input and carry no markdown meaning, so cmark passes them through
# as ordinary text. The index lets us restore spans in any order.
# Written as \u escapes so the source stays pure ASCII (R CMD check
# flags non-ASCII bytes in R files as a portability problem).
.MD_SENTINEL_OPEN  <- "\uE000"
.MD_SENTINEL_CLOSE <- "\uE001"
# Placeholder used inside .md_escape_tex() while backslashes are parked.
.MD_ESC_BS <- "\uE002"
# U+00A0, built from its codepoint: a literal one here would be a
# non-ASCII byte in an R source file, which R CMD check flags.
.MD_NBSP <- intToUtf8(160L)

.md_sentinel <- function(i) {
  paste0(.MD_SENTINEL_OPEN, i, .MD_SENTINEL_CLOSE)
}

# Replace every math span with a sentinel. Returns the masked text plus
# the original spans, indexed so .md_unmask_math() can splice them back.
# Math regions come from the same scanner latex_wrap() uses, so the two
# can never disagree about what counts as math (see .scan_math_spans).
#
# Unclosed spans are deliberately NOT masked. latex_wrap() auto-closes
# them at end of string, but in prose a lone `$` is far more often a
# price than an opening delimiter, and swallowing the rest of the line
# into math would also skip escaping it. Leaving it unmasked lets it fall
# through to .md_escape_tex() as the literal character the user typed.
.md_mask_math <- function(text) {
  # The sentinels below are private-use codepoints, which is not the same
  # as impossible: icon fonts (Nerd Fonts and friends) put real glyphs in
  # this block, and a pasted one would be spliced with a math span on the
  # way back out, silently duplicating a formula. Drop them from the
  # input rather than try to tell them apart later.
  text <- gsub(
    paste0("[", .MD_SENTINEL_OPEN, .MD_SENTINEL_CLOSE, .MD_ESC_BS, "]"),
    "", text)
  spans <- .scan_math_spans(text)
  spans <- Filter(function(sp) isTRUE(sp$closed), spans)
  if (length(spans) == 0L) {
    return(list(text = text, spans = character(0)))
  }
  out <- character(0)
  keep <- character(0)
  disp <- logical(0)
  pos <- 1L
  for (sp in spans) {
    if (sp$outer_start > pos) {
      out <- c(out, substr(text, pos, sp$outer_start - 1L))
    }
    keep <- c(keep, substr(text, sp$outer_start, sp$outer_end))
    # `$$...$$` / `\[...\]` asked for display math. Carried as an
    # attribute rather than a new list element so `spans` stays the plain
    # character vector every caller (and .md_unmask_math) indexes into.
    disp <- c(disp, isTRUE(sp$display))
    out <- c(out, .md_sentinel(length(keep)))
    pos <- sp$outer_end + 1L
  }
  if (pos <= nchar(text)) out <- c(out, substr(text, pos, nchar(text)))
  attr(keep, "display") <- disp
  list(text = paste(out, collapse = ""), spans = keep)
}

# Index of the math span a node consists *entirely* of, or NA. Used to
# promote a paragraph that is nothing but `$$...$$` to its own centred
# block, the way every other markdown dialect lays out display math.
.md_lone_math_span <- function(nd, spans) {
  kids <- xml2::xml_children(nd)
  if (length(kids) != 1L || !identical(xml2::xml_name(kids[[1]]), "text")) {
    return(NA_integer_)
  }
  txt <- trimws(xml2::xml_text(kids[[1]]))
  m <- regmatches(txt, regexec(
    paste0("^", .MD_SENTINEL_OPEN, "([0-9]+)", .MD_SENTINEL_CLOSE, "$"), txt))[[1]]
  if (length(m) != 2L) return(NA_integer_)
  i <- as.integer(m[2])
  disp <- attr(spans, "display")
  if (is.null(disp) || is.na(i) || i > length(disp) || !isTRUE(disp[i])) {
    return(NA_integer_)
  }
  i
}

# Splice masked math back into a string, restoring the original bytes.
.md_unmask_math <- function(s, spans) {
  if (length(spans) == 0L) return(s)
  for (i in seq_along(spans)) {
    s <- gsub(.md_sentinel(i), spans[i], s, fixed = TRUE)
  }
  s
}

# Characters that must not reach the MicroTeX parser as-is. Prose from a
# markdown AST is literal text: a stray `_` would open a subscript and
# `%` would start a comment.
.md_escape_tex <- function(s) {
  # Park backslashes behind a placeholder first. Their replacement text
  # contains braces, which the brace rules below would otherwise escape
  # in turn, yielding \backslash\{\}.
  s <- gsub("\\", .MD_ESC_BS, s, fixed = TRUE)
  # A leading backslash is all these need.
  s <- gsub("([{}$&#_%])", "\\\\\\1", s)
  # `~` and `^` are NOT escaped to \~{} and \^{}: those are the tilde and
  # circumflex *accents*, which float above the line the way a diacritic
  # does -- so `lm(y ~ x)` in a code block drew its tilde up at cap
  # height, and prose `~5` did the same.
  #
  # `^` needs nothing at all: inside \text{}, and inside a text command
  # such as \textbf{}, MicroTeX already draws it flat on the baseline.
  # `~` cannot be left bare, because LaTeX reads it as a non-breaking
  # space and MicroTeX duly draws a space, losing the character. Neither
  # \textasciitilde nor \textasciicircum exists -- like \textbackslash
  # they typeset their own letters. \char126 is the spelling that works,
  # and the empty group is required: \char126b swallows the `b`.
  s <- gsub("~", "\\\\char126{}", s)
  # MicroTeX has no \textbackslash -- it would typeset those 13 letters
  # literally. \backslash is the spelling that works; the empty group
  # stops it gluing onto a following letter (\backslashx).
  gsub(.MD_ESC_BS, "\\backslash{}", s, fixed = TRUE)
}

# Render one markdown text node.
#
# Escape the prose, restore the math spans raw, then -- unless `bare` --
# hand the result to latex_wrap(). The output of this file is fed to
# latex_grob() in "math" mode, so prose normally has to arrive already
# wrapped in \text{} or it would be typeset as spaced math italics.
# latex_wrap() also strips the `$` delimiters and adds \displaystyle for
# block math, which is exactly the translation needed.
#
# `bare` is for the inside of a text command such as \textbf{}. Those
# already put their argument in text mode, and a nested \text{} *resets
# the style*: \textbf{bold} reports font style 2 (bold) but
# \textbf{\text{bold}} reports 1 (plain), so wrapping there silently
# throws the emphasis away. Bare content still handles math, because
# `$...$` opens math mode inside a text command too.
#
# Order matters twice over: escape before unmasking, so the escaper never
# touches a formula; unmask before latex_wrap(), so it can see the real
# delimiters while escaped prose dollars stay literal.
.md_text_node <- function(s, spans, bare = FALSE) {
  one <- function(t) {
    raw <- .md_unmask_math(.md_escape_tex(t), spans)
    if (bare) raw else latex_wrap(raw, input_mode = "mixed")
  }
  # CommonMark decodes &nbsp; to a real U+00A0. MicroTeX spells it \nbsp,
  # but only *between* \text{} runs -- inside one it typesets the letters
  # (measured: \text{a}\nbsp\text{b} is 21bp, matching a real space, while
  # \text{a\nbsp b} is 25). So the run is split at each one and rejoined.
  # Left alone when bare, where there is no \text{} to sit between and the
  # character already renders as a space.
  if (bare || !grepl(.MD_NBSP, s, fixed = TRUE)) return(one(s))
  parts <- strsplit(s, .MD_NBSP, fixed = TRUE)[[1]]
  paste(vapply(parts, one, character(1)), collapse = "\\nbsp ")
}

# ---------------------------------------------------------------------
# Inline HTML
#
# GFM defines no markdown syntax for colour, underline or super/subscript
# -- verified: `_x_` is italic, `__x__` bold, `^x^` and `~x~` literal.
# CommonMark's raw HTML, which GFM keeps, is therefore not a fallback for
# these; it is the only conformant way to express them, and it is what
# ggtext uses. cmark hands us each tag as its own `html_inline` node with
# the raw text, so the walker can read them.
#
# Only the presentational subset below is interpreted. Everything else --
# `<abbr>`, `<div>`, anything unrecognised -- keeps its current
# behaviour: the markup is dropped and the text between it is kept.
# GFM's tagfilter (on, via extensions = TRUE) has already stripped
# `<script>`, `<iframe>` and friends before we see them.
# ---------------------------------------------------------------------

# The nine CSS named colours R's 657-name palette does not have. Every
# other CSS name is an R name too, so col2rgb() covers the rest.
#
# The names R and CSS *share* but disagree about -- green, gray, grey,
# maroon, purple -- are deliberately left to R. This package documents
# "any R colour name" and gp$col resolves them R's way everywhere else;
# quietly giving `color: green` a different green inside a style
# attribute than beside it would be worse than the inconsistency.
.MD_CSS_COLORS <- c(
  aqua = "#00FFFF", crimson = "#DC143C", fuchsia = "#FF00FF",
  indigo = "#4B0082", lime = "#00FF00", olive = "#808000",
  rebeccapurple = "#663399", silver = "#C0C0C0", teal = "#008080"
)

# Resolve a CSS colour to "#RRGGBB", or NULL when it is not a colour.
# Feeding hex to \textcolor is the robust path: MicroTeX accepts hex, so
# this frees us from its smaller table of named colours and lets any R
# colour name work.
.md_resolve_color <- function(x) {
  x <- trimws(tolower(x %||% ""))
  if (!nzchar(x)) return(NULL)
  if (grepl("^#[0-9a-f]{6}$", x)) return(toupper(x))
  # #abc is shorthand for #aabbcc.
  if (grepl("^#[0-9a-f]{3}$", x)) {
    d <- strsplit(substring(x, 2L), "")[[1]]
    return(toupper(paste0("#", paste0(d, d, collapse = ""))))
  }
  m <- regmatches(x, regexec("^rgba?\\(\\s*([0-9]+)\\D+([0-9]+)\\D+([0-9]+)", x))[[1]]
  if (length(m) == 4L) {
    v <- pmin(pmax(as.integer(m[2:4]), 0L), 255L)
    return(sprintf("#%02X%02X%02X", v[1], v[2], v[3]))
  }
  if (x %in% names(.MD_CSS_COLORS)) return(unname(.MD_CSS_COLORS[x]))
  rgb <- tryCatch(grDevices::col2rgb(x), error = function(e) NULL)
  if (is.null(rgb)) return(NULL)
  sprintf("#%02X%02X%02X", rgb[1], rgb[2], rgb[3])
}

# The size a CSS length is relative to, resolved the same way latex_grob()
# resolves it so `font-size: 10pt` means 10pt on the page.
.md_base_size <- function(gp) {
  size <- gp$fontsize %||% 20
  if (!is.null(gp$cex)) size <- size * gp$cex
  size
}

# Fold gp$cex into gp$fontsize and drop it.
#
# markdown_box_grob() builds a gTree whose children are latex_grobs, and
# each of those has already scaled itself by cex. grid multiplies a
# parent's cex into every child that does not set its own, so leaving cex
# on the gTree applied it a second time -- gpar(fontsize = 10, cex = 2)
# measured at 20 and drew at 40, overflowing the box. latex_grob() bakes
# and strips the same pair, for the same reason.
.md_fold_cex <- function(gp) {
  if (is.null(gp$cex)) return(gp)
  gp$fontsize <- .md_base_size(gp)
  gp$cex <- NULL
  gp
}

# A CSS length as a scale factor relative to `base` (big points). The
# absolute units are the ones gridtext accepts; em / rem / % are already
# relative, so they need no base at all. NULL for anything unparseable,
# which leaves the size alone rather than guessing.
.md_css_size <- function(x, base) {
  # A style object may hold a bare number or a unit rather than a CSS
  # string, since md_style(font_size = 2.5) is the R spelling of
  # "2.5rem". Both mean a multiple of the base, which is what this
  # returns, so they need no unit handling.
  if (grid::is.unit(x)) {
    if (base <= 0) return(NULL)
    return(grid::convertHeight(x, "bigpts", valueOnly = TRUE) / base)
  }
  if (is.numeric(x)) return(if (is.finite(x) && x > 0) x else NULL)
  x <- trimws(tolower(x))
  if (identical(x, "smaller")) return(1 / 1.2)
  if (identical(x, "larger")) return(1.2)
  # CSS's absolute keywords, taken from the \tiny..\Huge ladder MicroTeX
  # already implements: measured at 9, 13, 15, 16, 18, 22, 26, 33, 37, 47
  # bigpts against an 18 bigpt normalsize, which is the ratios below.
  kw <- c("xx-small" = 0.5, "x-small" = 0.72, small = 0.83, medium = 1,
          large = 1.2, "x-large" = 1.44, "xx-large" = 1.83)
  if (x %in% names(kw)) return(unname(kw[[x]]))
  # A unitless *string* stays invalid, because that is what it is in CSS:
  # `font-size: 12` is ignored by a browser and is ignored here. Only an
  # R value -- md_style(font_size = 2.5) -- means a bare multiple, and it
  # arrives as a number, not as text.
  m <- regmatches(
    x, regexec("^([0-9]*\\.?[0-9]+)\\s*(pt|px|in|cm|mm|em|rem|%)$", x))[[1]]
  if (length(m) != 3L) return(NULL)
  v <- as.numeric(m[2])
  if (!is.finite(v) || v <= 0) return(NULL)
  pt <- switch(m[3],
    pt = v,
    px = v * 72 / 96,
    "in" = v * 72,
    cm = v * 72 / 2.54,
    mm = v * 72 / 25.4,
    em = , rem = return(v),
    "%" = return(v / 100),
    return(NULL))
  if (base <= 0) return(NULL)
  pt / base
}

# Split a declaration list ("color: red; font-size: 2em") into a named
# list of raw values, in declaration order. Property names are lowercased
# -- CSS property names are case-insensitive -- but values keep their
# case, because a font family is a *name* and some font matchers care.
#
# A repeated property keeps its first position and takes its last value,
# which is what CSS says. This is the one place the split changed
# behaviour: the old loop emitted a command per declaration, so
# "color:red;color:blue" wrapped the text twice.
#
# Shared by every route into the style system -- the `style` attribute of
# a span or a div, a rule in a stylesheet, and md_style() -- so they can
# never disagree about what a declaration means.
# `warn = TRUE` reports declarations that will be ignored. Only the
# callers reading a hand-written `style` attribute pass it: a pasted
# stylesheet is *expected* to carry properties this package cannot
# honour (`display`, `float`, ...), and warning about those would punish
# the thing the parser is deliberately lenient for. A `style=` a user
# typed into their own markdown is different -- there, `colour: red` or
# `font-wieght: bold` silently doing nothing is a typo they want told
# about, and md_style() already errors on the same mistake.
.md_parse_css <- function(style, warn = FALSE) {
  out <- list()
  for (decl in strsplit(style %||% "", ";", fixed = TRUE)[[1]]) {
    kv <- strsplit(decl, ":", fixed = TRUE)[[1]]
    if (length(kv) < 2L) next
    prop <- .md_canonical_props(trimws(tolower(kv[1])))
    # A value can legitimately contain a colon, so rejoin what was split.
    if (nzchar(prop)) out[[prop]] <- trimws(paste(kv[-1], collapse = ":"))
  }
  if (warn && length(out)) {
    bad <- setdiff(names(out), .MD_STYLE_PROPS)
    if (length(bad)) {
      warning("Ignoring unsupported CSS ",
              if (length(bad) > 1L) "properties: " else "property: ",
              paste(sQuote(bad), collapse = ", "),
              ". See ?md_style for the supported set.", call. = FALSE)
    }
  }
  .md_expand_shorthand(out)
}

# CSS's `padding` / `margin` shorthand, expanded into the longhands the
# layout actually reads. One to four values, in CSS's order: all; then
# vertical/horizontal; then top/horizontal/bottom; then top/right/bottom/left.
#
# Done here rather than at each read so the shorthand works everywhere a
# declaration can appear -- a stylesheet, a `<div style=>`, and md_style().
.md_expand_shorthand <- function(props) {
  sides <- c("-top", "-right", "-bottom", "-left")
  for (p in c("padding", "margin")) {
    v <- props[[p]]
    if (is.null(v)) next
    props[[p]] <- NULL
    parts <- if (grid::is.unit(v)) {
      lapply(seq_along(v), function(i) v[i])
    } else if (is.numeric(v)) {
      as.list(v)
    } else {
      as.list(strsplit(trimws(as.character(v)), "[[:space:]]+")[[1]])
    }
    n <- length(parts)
    if (n == 0L) next
    pick <- switch(as.character(min(n, 4L)),
      "1" = c(1, 1, 1, 1), "2" = c(1, 2, 1, 2),
      "3" = c(1, 2, 3, 2), "4" = c(1, 2, 3, 4))
    for (i in seq_along(sides)) {
      nm <- paste0(p, sides[i])
      # A longhand alongside the shorthand wins. CSS settles that by
      # source order; these declarations are unordered, so the more
      # specific one always does.
      if (is.null(props[[nm]])) props[[nm]] <- parts[[pick[i]]]
    }
  }
  props
}

# font-family is a fallback list, so take the first entry. Map the CSS
# generics onto the aliases grid already understands and pass anything
# else through: what the name resolves to is the device's business, same
# as gpar(fontfamily=). Returns "" when there is no usable name.
.md_css_family <- function(val_raw) {
  fam <- gsub("^[\"']|[\"']$", "", trimws(strsplit(val_raw %||% "", ",")[[1]]))[1]
  fam <- switch(tolower(fam %||% ""), monospace = "mono",
                "sans-serif" = "sans", fam)
  # The name is about to be spliced into LaTeX, so it must not carry
  # anything the parser reads as syntax.
  gsub("[{}\\\\]", "", fam %||% "")
}

# CSS counts 600 and up as bold, alongside the two keywords.
.md_css_is_bold <- function(val) {
  if (val %in% c("bold", "bolder")) return(TRUE)
  n <- suppressWarnings(as.numeric(val))
  isTRUE(n >= 600)
}

# Turn parsed properties into wrapping commands. Several properties nest,
# so the closer has to match the number opened. `style` names the subset
# that switches to text mode -- the caller has to emit content inside
# those bare, and put them back inside a rule (see .md_inline_to_tex).
#
# Properties with no inline meaning (margin, padding, text-align ...) are
# skipped here and read by the block layout instead.
.md_css_inline_latex <- function(props, base = 20) {
  open <- character(0)
  text_mode <- character(0)
  # Emitted in declaration order, so the nesting matches what the author
  # wrote rather than an order of our choosing.
  for (prop in names(props)) {
    val_raw <- props[[prop]]
    # Only text gets case-folded. tolower() would coerce a number or a
    # unit -- which is how md_style() states a length -- into a string,
    # and a unitless string is not a valid CSS length.
    val <- if (is.character(val_raw)) tolower(val_raw) else val_raw
    if (prop == "color") {
      hex <- .md_resolve_color(val)
      if (!is.null(hex)) open <- c(open, paste0("\\textcolor{", hex, "}{"))
    } else if (prop == "text-decoration") {
      # The three decorations MicroTeX can draw.
      if (grepl("underline", val, fixed = TRUE)) open <- c(open, "\\underline{")
      if (grepl("overline", val, fixed = TRUE)) open <- c(open, "\\overline{")
      if (grepl("line-through", val, fixed = TRUE)) open <- c(open, "\\sout{")
    } else if (prop == "background") {
      # \bgcolor fills behind the glyphs with no padding, which is what a
      # background is; \colorbox would frame them instead. This is how
      # <mark> has always been drawn.
      #
      # A border alongside it takes the fill as \fcolorbox's second
      # argument, so emitting \bgcolor as well would paint it twice.
      hex <- if (is.null(props$border)) .md_resolve_color(val) else NULL
      if (!is.null(hex)) open <- c(open, paste0("\\bgcolor{", hex, "}{"))
    } else if (prop == "border") {
      # \fcolorbox needs both colours, \fbox neither. MicroTeX has no
      # \fboxsep, so the inset is fixed -- see ?md_style.
      bd <- .md_css_border(val_raw, base)
      col <- .md_resolve_color(bd$color %||% "")
      fill <- .md_resolve_color(props$background %||% "")
      open <- c(open, if (!is.null(col) && !is.null(fill)) {
        paste0("\\fcolorbox{", col, "}{", fill, "}{")
      } else "\\fbox{")
    } else if (prop == "border-style") {
      if (identical(val, "double")) open <- c(open, "\\doublebox{")
    } else if (prop == "border-radius") {
      # \ovalbox rounds the frame; \cornersize takes the radius as a
      # fraction of the box's smaller side.
      open <- c(open, "\\ovalbox{")
    } else if (prop == "box-shadow") {
      if (!identical(val, "none")) open <- c(open, "\\shadowbox{")
    } else if (prop == "visibility") {
      # \phantom keeps the space and draws nothing, which is exactly what
      # visibility: hidden means (display: none would remove the space).
      if (identical(val, "hidden")) open <- c(open, "\\phantom{")
    } else if (prop == "vertical-align") {
      cmd <- if (identical(val, "super")) "\\textsuperscript{"
             else if (identical(val, "sub")) "\\textsubscript{"
             else {
               d <- .md_css_length(val_raw, base, base)
               if (!is.null(d)) sprintf("\\raisebox{%.2fpt}{", d) else NULL
             }
      if (!is.null(cmd)) open <- c(open, cmd)
    } else if (prop == "transform") {
      # rotate(Ndeg) / scale(N) / scaleX(-1), the three MicroTeX has.
      m <- regmatches(val, regexec("rotate\\(\\s*(-?[0-9.]+)", val))[[1]]
      if (length(m) == 2L) open <- c(open, paste0("\\rotatebox{", m[2], "}{"))
      m <- regmatches(val, regexec("(?<![a-z])scale\\(\\s*([0-9.]+)", val,
                                   perl = TRUE))[[1]]
      if (length(m) == 2L) open <- c(open, paste0("\\scalebox{", m[2], "}{"))
      if (grepl("scalex(-1", gsub("[[:space:]]", "", val), fixed = TRUE)) {
        open <- c(open, "\\reflectbox{")
      }
    } else if (prop == "font-size") {
      # \scalebox scales the whole box, which is what font-size means, and
      # unlike \underline it keeps the font style around it.
      f <- .md_css_size(val, base)
      # Fixed notation, not format(): that honours getOption("OutDec"), so
      # a comma locale emitted \scalebox{0,75}, and it switches to
      # e-notation for a small factor, which the parser cannot read either.
      if (!is.null(f)) open <- c(open, sprintf("\\scalebox{%.6f}{", f))
    } else if (prop == "font-weight") {
      # \textbf switches to text mode itself, so it joins `style` for the
      # same reason \gmfontfamily does.
      if (.md_css_is_bold(val)) {
        open <- c(open, "\\textbf{")
        text_mode <- c(text_mode, "\\textbf{")
      }
    } else if (prop == "font-style") {
      if (val %in% c("italic", "oblique")) {
        open <- c(open, "\\textit{")
        text_mode <- c(text_mode, "\\textit{")
      }
    } else if (prop == "font-family") {
      fam <- .md_css_family(val_raw)
      if (nzchar(fam)) {
        cmd <- paste0("\\gmfontfamily{", fam, "}{")
        open <- c(open, cmd)
        # \text{} replaces the whole text font style (macro_fonts.h:12
        # builds a non-nested FontStyleAtom), which would drop the family
        # along with any bold. So content goes in bare, and the command
        # joins the set re-opened inside a rule.
        text_mode <- c(text_mode, cmd)
      }
    }
  }
  list(open = paste(open, collapse = ""),
       close = strrep("}", length(open)),
       style = text_mode)
}

# A `style` attribute straight to wrapping commands.
.md_parse_style <- function(style, base = 20) {
  .md_css_inline_latex(.md_parse_css(style), base)
}

# Tags we interpret, and the LaTeX that reproduces the rendering HTML
# itself prescribes for them (HTML Living Standard, "Rendering"). Nothing
# here is invented: every entry is some element's default presentation.
# Elements whose default is *no* visual change -- <a>, <abbr>, <span>
# without a style, <bdi>, <data>, <time>, <wbr> ... -- are absent on
# purpose and fall through to the drop-the-markup-keep-the-text path,
# which is already the right rendering for them.
#
# `bare` says what the command does to the text/math mode:
#   TRUE -- it switches to text mode itself, so its content must NOT be
#           wrapped in \text{} or the style is reset and thrown away
#   NA   -- it leaves the mode alone; content inherits the caller's
#
# `reset` marks the commands that typeset their argument as a fresh
# sub-formula and so lose the bold/italic/mono they sit inside. Measured,
# not assumed: \textbf{\underline{Hg}} reports no font style at all, while
# \textbf{\textcolor{red}{Hg}}, \textbf{{\small Hg}} and \textbf{\texttt{Hg}}
# all keep theirs. Size and colour survive; only the font style is lost.
.md_tag <- function(open, close = "}", bare = NA, reset = FALSE) {
  list(open = open, close = close, bare = bare, reset = reset)
}

.MD_HTML_TAGS <- list(
  # font-weight: bold
  b       = .md_tag("\\textbf{", bare = TRUE),
  strong  = .md_tag("\\textbf{", bare = TRUE),
  # font-style: italic
  i       = .md_tag("\\textit{", bare = TRUE),
  em      = .md_tag("\\textit{", bare = TRUE),
  cite    = .md_tag("\\textit{", bare = TRUE),
  dfn     = .md_tag("\\textit{", bare = TRUE),
  var     = .md_tag("\\textit{", bare = TRUE),
  address = .md_tag("\\textit{", bare = TRUE),
  # font-family: monospace
  code    = .md_tag("\\texttt{", bare = TRUE),
  kbd     = .md_tag("\\texttt{", bare = TRUE),
  samp    = .md_tag("\\texttt{", bare = TRUE),
  tt      = .md_tag("\\texttt{", bare = TRUE),
  # text-decoration
  u       = .md_tag("\\underline{", reset = TRUE),
  ins     = .md_tag("\\underline{", reset = TRUE),
  s       = .md_tag("\\sout{", reset = TRUE),
  del     = .md_tag("\\sout{", reset = TRUE),
  strike  = .md_tag("\\sout{", reset = TRUE),
  # vertical-align
  sub     = .md_tag("\\textsubscript{", reset = TRUE),
  sup     = .md_tag("\\textsuperscript{", reset = TRUE),
  # background: yellow. \bgcolor draws the fill behind the glyphs with no
  # padding, unlike \colorbox, which frames them.
  mark    = .md_tag("\\bgcolor{#FFFF00}{"),
  # font-size: smaller / larger
  small   = .md_tag("{\\small "),
  big     = .md_tag("{\\large ")
)

# Read one attribute out of a tag's attribute text, or NULL if absent.
#
# Match to the *closing* quote of the same kind, not to whichever quote
# comes first: a value may legitimately contain the other one, as
# font-family: 'Courier New', monospace does.
.md_html_attr <- function(attrs, name) {
  for (q in c('"', "'")) {
    m <- regmatches(attrs, regexec(
      paste0(name, "\\s*=\\s*", q, "([^", q, "]*)", q), attrs))[[1]]
    if (length(m) == 2L) return(m[2])
  }
  NULL
}

# The class names on a tag, lowercased -- CSS class matching here is
# case-insensitive because every selector is lowercased on the way in.
.md_html_classes <- function(attrs) {
  v <- .md_html_attr(attrs, "class")
  if (is.null(v)) return(character(0))
  tolower(Filter(nzchar, strsplit(trimws(v), "[[:space:]]+")[[1]]))
}

# Tags whose styling comes from the cascade rather than a fixed command,
# so they honour a stylesheet, a class, and a `style` attribute. `span` is
# the generic one; `a` is here so an inline link is styled by the same `a`
# rule as markdown's own `[text](url)`.
.MD_CASCADE_TAGS <- c("span", "a")

# Classify one html_inline node. `bare` is the mode the tag appears in,
# which <q> needs: its quotation marks are glyphs, not a command, so they
# have to be wrapped or not like any other text.
#
# Returns a list with `kind`:
#   "open"  -- emit `open`, push `close` and `bare`
#   "close" -- pop the matching tag, emit its close
#   "break" -- <br>, a line break; nothing to push
#   "text"  -- emit `text` as-is; a void element, so nothing to push
#   "drop"  -- emit nothing (unrecognised markup)
.md_html_tag <- function(txt, bare = FALSE, base = 20, style = NULL) {
  m <- regmatches(txt, regexec("^<\\s*(/?)\\s*([A-Za-z][A-Za-z0-9]*)([^>]*)>$",
                               trimws(txt)))[[1]]
  if (length(m) != 4L) return(list(kind = "drop"))
  closing <- nzchar(m[2])
  tag <- tolower(m[3])
  attrs <- m[4]

  if (closing) {
    if (tag %in% .MD_CASCADE_TAGS || tag == "q" ||
        !is.null(.MD_HTML_TAGS[[tag]])) {
      return(list(kind = "close", tag = tag))
    }
    return(list(kind = "drop"))
  }
  if (tag == "br") return(list(kind = "break"))
  # An image cannot be drawn by a text grob, so keep its alt text -- which
  # is what markdown's own `![alt](src)` already does. Dropping the tag
  # whole meant the two spellings of one thing disagreed, and `<img>` lost
  # the only part of itself that could be rendered.
  if (tag == "img") {
    src <- .md_html_attr(attrs, "src")
    if (.image_drawable(src)) {
      # HTML sizes an <img> in pixels, which is exactly what `px` means to
      # the resolver. A command, so it is emitted bare either way.
      opt <- character(0)
      for (a in c("width", "height")) {
        v <- .md_html_attr(attrs, a)
        if (!is.null(v) && grepl("^[0-9.]+$", trimws(v))) {
          opt <- c(opt, paste0(a, "=", trimws(v), "px"))
        }
      }
      return(list(kind = "text", text = paste0(
        "\\includegraphics",
        if (length(opt)) paste0("[", paste(opt, collapse = ","), "]") else "",
        "{", src, "}")))
    }
    alt <- .md_html_attr(attrs, "alt")
    if (is.null(alt) || !nzchar(trimws(alt))) return(list(kind = "drop"))
    alt <- .md_escape_tex(alt)
    return(list(kind = "text",
                text = if (bare) alt else paste0("\\text{", alt, "}")))
  }
  if (tag %in% .MD_CASCADE_TAGS) {
    # Only a *link* gets the link rendering. The HTML standard styles
    # `a:link`, which is an <a> carrying an href; a bare <a> is an anchor
    # and renders as ordinary text, exactly like <span>.
    if (tag == "a" && is.null(.md_html_attr(attrs, "href"))) {
      return(list(kind = "drop"))
    }
    # A class names rules in the stylesheet; the style attribute is the
    # highest-priority origin. .md_cascade() settles the two, so a span
    # and a div read their attributes exactly the same way.
    #
    # `<a href>` resolves through the same cascade, under its own `a`
    # selector, so it lands on the rule markdown's `[text](url)` already
    # uses -- and so a user restyling `a` restyles both spellings at once.
    props <- .md_cascade(style %||% markdown_style(), tag,
                         classes = .md_html_classes(attrs),
                         inline = .md_parse_css(.md_html_attr(attrs, "style"),
                                                warn = TRUE))
    st <- .md_css_inline_latex(props, base)
    # A span with no usable style still pushes, so </span> pairs cleanly.
    # font-family opens a text-mode command, so the span then behaves like
    # <code> and its content must be emitted bare.
    return(c(list(kind = "open", tag = tag, style = st$style),
             .md_tag(st$open, st$close,
                     bare = if (length(st$style)) TRUE else NA)))
  }
  # <q> is the one element whose default rendering adds characters rather
  # than a style: content: open-quote / close-quote.
  if (tag == "q") {
    q <- if (bare) c("\u201c", "\u201d") else c("\\text{\u201c}", "\\text{\u201d}")
    return(c(list(kind = "open", tag = "q"), .md_tag(q[1], q[2])))
  }
  cmd <- .MD_HTML_TAGS[[tag]]
  if (is.null(cmd)) return(list(kind = "drop"))
  c(list(kind = "open", tag = tag), cmd)
}

# Walk the inline children of a cmark node and return a LaTeX string.
#
# `spans` carries the masked math so text nodes can restore it. Node
# types cmark can emit that MicroTeX has no equivalent for degrade to
# their text content rather than emitting an unknown command -- MicroTeX
# does not error on an unknown command, it typesets the command name as
# red glyphs, so a stray \href would print a red "href" in the label.
.md_inline_to_tex <- function(node, spans, bare = FALSE,
                              styles = character(0), base = 20,
                              style = NULL) {
  kids <- xml2::xml_contents(node)
  if (length(kids) == 0L) return("")

  # An HTML span's opening and closing tags are *siblings*, with the
  # content it styles as the siblings between them, so the walk needs
  # state across the sequence rather than a per-node map.
  out <- character(0)
  open_tags <- character(0)   # tag names, innermost last
  open_close <- character(0)  # their closing LaTeX, same order
  open_bare <- logical(0)     # the mode each one imposes, NA to inherit
  open_style <- list()        # the font-style commands it adds, if any

  # The mode content is currently in: the innermost tag that imposes one,
  # or the caller's if none does. This is what lets <b> work at all --
  # its content is a *sibling* of the tag, so the only way to emit that
  # sibling bare is to remember that a text-mode tag is open.
  cur_bare <- function() {
    set <- which(!is.na(open_bare))
    if (length(set)) open_bare[set[length(set)]] else bare
  }
  # Font-style commands in force here, outermost first. `styles` carries
  # the ones opened by an enclosing walk (markdown's own ** and *), so a
  # rule nested inside them can put them back.
  cur_styles <- function() c(styles, unlist(open_style))

  # MicroTeX gives a row holding nothing a height of zero, so consecutive
  # breaks collapse into one and `<br><br>` loses the blank line HTML
  # shows. A zero-width strut with the height and depth of a text line
  # puts that row back at exactly the height of a real one.
  row_empty <- TRUE
  emit_break <- function() {
    b <- if (row_empty) "\\vphantom{\\text{Ag}}\\\\" else "\\\\"
    row_empty <<- TRUE
    b
  }

  emit_one <- function(k, nm, bare, sty) {
    switch(nm,
      text = .md_text_node(xml2::xml_text(k), spans, bare = bare),
      # \textbf/\textit/\texttt switch to text mode themselves, so their
      # content is emitted bare -- a nested \text{} would reset the style
      # and lose the emphasis entirely.
      # Routed through the cascade like <code> is, so `strong { color: }`
      # and `em { color: }` work -- the tag vocabulary is HTML's names,
      # and these two are the tags markdown's own ** and * produce.
      strong = {
        cd <- .md_css_inline_latex(
          .md_cascade(style %||% markdown_style(), "strong"), base)
        paste0(cd$open, "\\textbf{",
               .md_inline_to_tex(k, spans, TRUE,
                                 c(sty, cd$style, "\\textbf{"), base, style),
               "}", cd$close)
      },
      emph = {
        cd <- .md_css_inline_latex(
          .md_cascade(style %||% markdown_style(), "em"), base)
        paste0(cd$open, "\\textit{",
               .md_inline_to_tex(k, spans, TRUE,
                                 c(sty, cd$style, "\\textit{"), base, style),
               "}", cd$close)
      },
      # \sout, \cancel, \bcancel and \xcancel are all registered in
      # MicroTeX (macro_def.cpp); \sout is the horizontal rule that
      # matches markdown's ~~strike~~. It typesets its argument as a fresh
      # sub-formula, which drops any font style around it -- so **~~x~~**
      # came out unbolded until the enclosing commands were re-opened
      # inside it.
      strikethrough = paste0("\\sout{", paste(sty, collapse = ""),
                             .md_inline_to_tex(
                               k, spans, if (length(sty)) TRUE else bare,
                               sty, base, style),
                             strrep("}", length(sty)), "}"),
      # Code spans are literal. Restore any masked math first so the
      # sentinel never leaks, then escape the lot -- `$x$` in backticks
      # is meant to be shown as characters, not typeset as math.
      # A `code` rule in the stylesheet wraps the result; \texttt is the
      # built-in default that a rule adds to rather than replaces.
      code = {
        cd <- .md_css_inline_latex(
          .md_cascade(style %||% markdown_style(), "code"), base)
        paste0(cd$open, "\\texttt{",
               .md_escape_tex(.md_unmask_math(xml2::xml_text(k), spans)),
               "}", cd$close)
      },
      # Grid has no clickable link, so the destination is dropped and only
      # the text is kept -- but it is styled through the `a` tag, so it
      # still reads as a link. \underline / \textcolor typeset their
      # argument as a fresh sub-formula and drop the font style around
      # them, so any enclosing bold/italic is re-opened inside, exactly as
      # ~~strike~~ does above.
      link = {
        ln <- .md_css_inline_latex(
          .md_cascade(style %||% markdown_style(), "a"), base)
        if (!nzchar(ln$open)) {
          .md_inline_to_tex(k, spans, bare, sty, base, style)
        } else {
          paste0(ln$open, paste(sty, collapse = ""),
                 .md_inline_to_tex(k, spans,
                                   if (length(sty)) TRUE else bare,
                                   sty, base, style),
                 strrep("}", length(sty)), ln$close)
        }
      },
      # A drawable file becomes a real inline image; anything else -- a web
      # URL, a missing file, an unsupported format -- keeps its alt text,
      # silently, as it always has.
      image         = {
        dest <- xml2::xml_attr(k, "destination")
        if (.image_drawable(dest)) paste0("\\includegraphics{", dest, "}")
        else .md_inline_to_tex(k, spans, bare, sty, base, style)
      },
      # A footnote reference. CommonMark gives the target's id, not a
      # number, so the number is the position of the matching <fn> in the
      # document -- computed once in .md_parse_doc() and carried on
      # `spans`, which is already threaded everywhere this walker goes.
      fnref = {
        idx <- attr(spans, "fn_index")
        dest <- xml2::xml_attr(k, "destination")
        n <- if (!is.null(idx) && !is.na(dest) && dest %in% names(idx)) {
          idx[[dest]]
        } else NA_integer_
        if (is.na(n)) "" else paste0("\\textsuperscript{", n, "}")
      },
      # A soft line break is a word space. In math mode it needs to be an
      # explicit \text{ } box, because a bare space between two \text{}
      # runs is discarded there and the words either side would run
      # together ("islong"). Inside a text command a plain space is
      # already a space, and \text{ } would reset the style.
      softbreak     = if (bare) " " else "\\text{ }",
      # Anything else unknown: drop the markup, keep the text.
      .md_text_node(xml2::xml_text(k), spans, bare = bare)
    )
  }

  # <ruby>base<rt>gloss</rt></ruby> is the one construct whose pieces
  # arrive as siblings but must come out in the *other* order, because
  # \overset takes the annotation first. So it cannot be an open/close
  # pair like every other tag: the base is captured until <rt>, the gloss
  # until </rt>, and both are emitted at </ruby>.
  ruby <- NULL  # NULL = not in a ruby; otherwise list(base=, gloss=, in_rt=)
  ruby_emit <- function() {
    r <- ruby
    ruby <<- NULL
    if (!nzchar(r$base)) return("")
    if (!nzchar(r$gloss)) return(r$base)
    paste0("\\overset{\\scalebox{0.5}{", r$gloss, "}}{", r$base, "}")
  }

  for (k in kids) {
    nm <- xml2::xml_name(k)
    # Route everything through the ruby buffers while one is open.
    if (!is.null(ruby) && identical(nm, "html_inline")) {
      t <- tolower(trimws(xml2::xml_text(k)))
      if (t == "<rt>") { ruby$in_rt <- TRUE; next }
      if (t == "</rt>") { ruby$in_rt <- FALSE; next }
      if (t == "</ruby>") { out <- c(out, ruby_emit()); next }
    }
    if (!is.null(ruby) && !identical(nm, "html_inline")) {
      # Captured exactly as it would render outside a ruby, because a
      # <ruby> with no <rt> falls back to emitting the base alone.
      s <- emit_one(k, nm, cur_bare(), cur_styles())
      if (isTRUE(ruby$in_rt)) ruby$gloss <- paste0(ruby$gloss, s)
      else ruby$base <- paste0(ruby$base, s)
      next
    }
    if (identical(nm, "html_inline") &&
        identical(tolower(trimws(xml2::xml_text(k))), "<ruby>")) {
      ruby <- list(base = "", gloss = "", in_rt = FALSE)
      next
    }
    if (identical(nm, "linebreak")) {
      out <- c(out, emit_break())
      next
    }
    if (!identical(nm, "html_inline")) {
      # A row holding only spaces is still an empty row, so neither a soft
      # break nor whitespace-only text counts as content. It is the common
      # way to write a blank line: "one<br>\n<br>\ntwo".
      blank <- identical(nm, "softbreak") ||
        (identical(nm, "text") && !nzchar(trimws(xml2::xml_text(k))))
      s <- emit_one(k, nm, cur_bare(), cur_styles())
      if (!blank && nzchar(s)) row_empty <- FALSE
      out <- c(out, s)
      next
    }
    tg <- .md_html_tag(xml2::xml_text(k), cur_bare(), base, style)
    if (identical(tg$kind, "open")) {
      sty <- cur_styles()
      # The font-style commands this tag contributes: \textbf, \textit,
      # \texttt and the like. A <span> can bring several (font-family);
      # everything else brings its own open string or none. Captured
      # before `bare` is rewritten below.
      adds <- if (!is.null(tg$style)) tg$style
              else if (isTRUE(tg$bare)) tg$open
              else character(0)
      if (isTRUE(tg$reset) && length(sty)) {
        # Re-open the font-style commands inside, so the rule (or script)
        # does not strip the bold/italic/mono it is nested in.
        out <- c(out, tg$open, sty)
        tg$close <- paste0(strrep("}", length(sty)), tg$close)
        tg$bare <- TRUE
      } else {
        out <- c(out, tg$open)
      }
      open_tags <- c(open_tags, tg$tag)
      open_close <- c(open_close, tg$close)
      open_bare <- c(open_bare, tg$bare)
      open_style <- c(open_style, list(adds))
    } else if (identical(tg$kind, "close")) {
      # Close the innermost matching tag. Anything opened inside it that
      # was never closed is closed here too, so the braces stay balanced
      # however malformed the input was.
      hit <- which(open_tags == tg$tag)
      if (length(hit)) {
        i <- hit[length(hit)]
        out <- c(out, rev(open_close[seq(i, length(open_close))]))
        open_tags <- open_tags[seq_len(i - 1L)]
        open_close <- open_close[seq_len(i - 1L)]
        open_bare <- open_bare[seq_len(i - 1L)]
        open_style <- open_style[seq_len(i - 1L)]
      }
      # An unmatched closing tag is just stray markup: drop it.
    } else if (identical(tg$kind, "break")) {
      out <- c(out, emit_break())
    } else if (identical(tg$kind, "text")) {
      # A void element that renders as characters -- <img>'s alt text.
      # Nothing to push: there is no closing tag to pair with.
      out <- c(out, tg$text)
      row_empty <- FALSE
    }
    # "drop": unrecognised markup contributes nothing.
  }
  # An unclosed <ruby> still renders what it captured, rather than losing
  # the text entirely -- the same forgiveness unclosed spans get below.
  if (!is.null(ruby)) out <- c(out, ruby_emit())
  # Unclosed spans at the end of the run.
  if (length(open_close)) out <- c(out, rev(open_close))

  paste(out, collapse = "")
}

# Mask the math, then parse to a namespace-stripped CommonMark AST.
# Both entry points below need exactly this and must agree about it: a
# difference here would make markdown_grob() and markdown_box_grob()
# read the same document two different ways.
#
# commonmark and xml2 are Imports, so they cannot be missing at run time
# and are not guarded.
.md_parse_doc <- function(md) {
  masked <- .md_mask_math(md)
  doc <- xml2::read_xml(
    # `footnotes` is a separate argument, not one of the five extensions.
    # With it, `[^1]` arrives as <fnref> and its definition as a <fn>
    # block; without it both stay literal text.
    commonmark::markdown_xml(masked$text, extensions = TRUE, footnotes = TRUE)
  )
  # Strip the CommonMark namespace so plain node names match.
  xml2::xml_ns_strip(doc)
  # Number the footnotes by the order their definitions appear, which is
  # the order CommonMark emits them (first reference). Carried on `spans`
  # so the inline walker can reach it without a new parameter.
  ids <- xml2::xml_attr(xml2::xml_find_all(doc, ".//fn"), "id")
  spans <- masked$spans
  if (length(ids)) {
    idx <- seq_along(ids)
    names(idx) <- ids
    attr(spans, "fn_index") <- idx
  }
  list(doc = doc, spans = spans)
}

# A single character string, the only thing either entry point accepts.
.md_check_string <- function(md) {
  if (!is.character(md) || length(md) != 1L) {
    stop("`md` must be a single character string.", call. = FALSE)
  }
  if (is.na(md)) {
    stop("`md` must not be NA.", call. = FALSE)
  }
  invisible(md)
}

# Convert markdown to a single LaTeX string for input_mode = "math",
# flattening block structure: paragraphs are joined with `\\` line
# breaks and headings/lists/quotes lose their block formatting. This is
# what markdown_grob() uses. markdown_box_grob() keeps block structure
# by stacking each block as its own grob instead.
.md_to_tex <- function(md, base = 20, style = NULL) {
  parsed <- .md_parse_doc(md)
  blocks <- xml2::xml_children(parsed$doc)
  if (length(blocks) == 0L) return("")
  parts <- vapply(blocks, function(b) {
    nm <- xml2::xml_name(b)
    # An html_block holds raw markup as its text, so walking into it
    # would typeset the tags -- and the markdown inside them, unparsed.
    # Drop it, matching .md_blocks() on the block path, which would
    # otherwise disagree about the same document. Only *inline* HTML
    # (.md_html_tag) is interpreted.
    if (identical(nm, "html_block")) return("")
    # A footnote definition needs a foot to sit at, and an inline label has
    # none: the marker still renders, the note itself is dropped. Only
    # markdown_box_grob() lays them out.
    if (identical(nm, "fn")) return("")

    # There is no layout here to hang a margin or an indent on, so only
    # the properties that compile to a LaTeX command apply. Everything
    # else is ignored, the way CSS ignores what a medium cannot do.
    tag <- switch(nm,
      paragraph = "p", heading = paste0("h", .md_heading_level(b)),
      block_quote = "blockquote", code_block = "pre", table = "table",
      thematic_break = "hr", item = "li", nm)
    # `body` seeds the inheritance here exactly as it does for a block
    # document (.md_layout), so `body { color: }` reaches an inline label
    # too. Without this seed a stylesheet written against the box grob
    # silently did nothing when handed to markdown_grob().
    st <- style %||% markdown_style()
    ilat <- .md_css_inline_latex(
      .md_cascade(st, tag, inherited = .md_cascade(st, "body")), base)
    # A text-mode command resets the style of a nested \text{}, so the
    # content goes in bare when one is open -- the same rule the span
    # path follows.
    inner <- .md_inline_to_tex(b, parsed$spans,
                               bare = length(ilat$style) > 0L,
                               styles = ilat$style, base = base,
                               style = style)
    if (!nzchar(inner)) return("")
    paste0(ilat$open, inner, ilat$close)
  }, character(1))
  parts <- parts[nzchar(parts)]
  paste(parts, collapse = "\\\\")
}

#' Render markdown as a grid grob
#'
#' @description
#' Parses a markdown string and returns a grid grob, so plot labels can
#' mix ordinary prose formatting with real LaTeX math. Inline markdown
#' (\code{**bold**}, \code{*italic*}, \code{`code`}, \code{~~strike~~})
#' is translated to the equivalent LaTeX commands, while \code{$...$}
#' (and \code{\\(...\\)}, \code{$$...$$}, \code{\\[...\\]}) math spans
#' are passed through to MicroTeX byte-for-byte.
#'
#' @details
#' Markdown and LaTeX disagree about several characters --- most
#' importantly \code{\\}, which CommonMark treats as an escape. Math
#' spans are therefore hidden from the markdown parser before it runs and
#' restored afterwards, so constructs like
#' \code{$\\begin{matrix}a\\\\b\\end{matrix}$} survive intact.
#'
#' That hiding uses three private-use codepoints (\code{U+E000},
#' \code{U+E001}, \code{U+E002}) as markers, so those three characters
#' are \emph{removed} from the input. They are unassigned in Unicode, but
#' icon fonts such as Nerd Fonts do put real glyphs there: if your text
#' contains one it will be dropped rather than drawn. The alternative is
#' worse --- a pasted marker would be spliced together with a math span
#' on the way back out and silently duplicate a formula.
#'
#' GFM has no markdown syntax for colour, underline, super/subscript,
#' highlight or size, so --- as in CommonMark, and as \pkg{ggtext} does
#' --- these come from inline HTML. Each tag renders as HTML's own
#' default rendering prescribes:
#'
#' \tabular{ll}{
#'   \strong{tag} \tab \strong{effect} \cr
#'   \code{<b>}, \code{<strong>} \tab bold \cr
#'   \code{<i>}, \code{<em>}, \code{<cite>}, \code{<dfn>}, \code{<var>},
#'     \code{<address>} \tab italic \cr
#'   \code{<code>}, \code{<kbd>}, \code{<samp>}, \code{<tt>} \tab monospace \cr
#'   \code{<u>}, \code{<ins>} \tab underline \cr
#'   \code{<s>}, \code{<del>}, \code{<strike>} \tab strikethrough \cr
#'   \code{<sub>}, \code{<sup>} \tab sub / superscript \cr
#'   \code{<mark>} \tab yellow highlight \cr
#'   \code{<small>}, \code{<big>} \tab smaller / larger \cr
#'   \code{<q>} \tab wrapped in quotation marks \cr
#'   \code{<br>} \tab line break \cr
#'   \code{<span style="...">} \tab see below
#' }
#'
#' A \code{style} attribute is read for \code{color} (any R colour name,
#' the nine CSS names R lacks --- \code{crimson}, \code{teal},
#' \code{rebeccapurple} and friends --- \code{#rgb}, \code{#rrggbb} or
#' \code{rgb()}; note that \code{green}, \code{gray}, \code{grey},
#' \code{maroon} and \code{purple} keep their R values, not their CSS
#' ones),
#' \code{text-decoration} (\code{underline}, \code{line-through}),
#' \code{font-size} (\code{pt}, \code{px}, \code{in}, \code{cm},
#' \code{mm}, \code{em}, \code{rem}, \code{\%}, \code{smaller},
#' \code{larger}) and \code{font-family}. Any other property is ignored.
#'
#' \code{font-family} takes the CSS generics \code{monospace},
#' \code{sans-serif} and \code{serif}, or any font name; a fallback list
#' resolves to its first entry. The name is handed to
#' \code{gp$fontfamily}, so \emph{the device resolves it}: \pkg{ragg} and
#' \pkg{svglite} see any installed family plus anything registered with
#' \code{systemfonts::register_font()}, cairo devices see installed
#' families, and base \code{pdf()} sees only what \code{pdfFonts()}
#' declares --- a named family will not resolve there. An unavailable
#' font falls back silently, as it does for \code{gpar(fontfamily=)}.
#' A font file that is not installed system-wide is used by registering
#' it first:
#'
#' \preformatted{systemfonts::register_font(name = "MyFont", plain = "MyFont.otf")}
#'
#' \code{\link{load_math_font}} is \emph{not} the function for this ---
#' it registers \emph{math} fonts with MicroTeX, which is a different
#' mechanism.
#'
#' A span's own family wins over \code{gp$fontfamily}, but the width of
#' the spaces \emph{between} its words still comes from
#' \code{gp$fontfamily}; set both to the same family if that shows.
#'
#' Tags nest and combine freely with markdown. Any other tag --- and all
#' block-level HTML --- is dropped, keeping the text inside it, which is
#' also what a browser shows for the ones (\code{<a>}, \code{<abbr>},
#' \code{<span>} without a style) that have no default rendering.
#'
#' Not every markdown feature has a MicroTeX equivalent. Links keep their
#' text and drop the destination, and images keep their alt text.
#'
#' Everything is flattened into a single run here, with paragraphs joined
#' by line breaks: there is no block layout, so indentation, list markers
#' and block-quote rules need \code{\link{markdown_box_grob}}. A heading
#' still takes the size and weight its \code{style} gives it, since those
#' compile to LaTeX commands rather than to layout.
#'
#' @param md Character string of markdown.
#' @param style Appearance of the text: a \code{\link{markdown_style}}
#'   object, CSS text, or a path to a \code{.css} file. \code{NULL}
#'   (default) uses \code{latex_options("markdown_style")} if set, and
#'   the built-in defaults otherwise. Only the properties \code{md_style}
#'   marks as \emph{inline} apply here --- there is no block layout in a
#'   single run for a margin, an indent or an alignment to act on, so
#'   those are ignored. \code{\link{markdown_box_grob}} honours them all.
#' @param ... Passed to \code{\link{latex_grob}} --- e.g. \code{x},
#'   \code{y}, \code{hjust}, \code{vjust}, \code{rot}, \code{max_width},
#'   \code{gp}.
#' @return A \code{latexgrob}, as returned by \code{\link{latex_grob}}.
#' @seealso \code{\link{latex_grob}}, \code{\link{latex_wrap}}
#' @export
#'
#' @examples
#' \donttest{
#'   grid::grid.newpage()
#'   grid.markdown("The **fitted** slope is $\\beta_1$, *p* < 0.001")
#' }
markdown_grob <- function(md, style = NULL, ...) {
  .md_check_string(md)
  # input_mode is fixed below, so letting it through `...` would fail with
  # R's "matched by multiple actual arguments" rather than saying why.
  if ("input_mode" %in% ...names()) {
    stop("`input_mode` is not accepted by markdown_grob(): markdown ",
         "decides for itself what is prose and what is math.", call. = FALSE)
  }
  # The walker has already produced \text{}-wrapped LaTeX, so bypass
  # latex_wrap() -- running it again would double-wrap the prose.
  latex_grob(.md_to_tex(md, .md_base_size(list(...)$gp), .md_as_style(style)),
             input_mode = "math", ...)
}

#' @rdname markdown_grob
#' @export
grid.markdown <- function(md, ...) {
  g <- markdown_grob(md, ...)
  grid::grid.draw(g)
  invisible(g)
}

#' Deprecated functions
#'
#' These functions still work but will be removed in a future release.
#' Both were renamed to say what they actually operate on: only
#' \strong{math} fonts (those carrying an OpenType MATH table) are ever
#' registered with MicroTeX. Text fonts need no loading --- they are
#' resolved on demand by \pkg{systemfonts} from \code{gp$fontfamily}.
#'
#' \tabular{ll}{
#'   \strong{Deprecated}  \tab \strong{Use instead} \cr
#'   \code{load_font()}   \tab \code{\link{load_math_font}()}  \cr
#'   \code{check_fonts()} \tab \code{\link{check_math_fonts}()} \cr
#' }
#'
#' @param otf_path Path to the OTF/TTF font file.
#' @return As the replacement function.
#' @name gridmicrotex-deprecated
#' @keywords internal
NULL

#' @rdname gridmicrotex-deprecated
#' @export
load_font <- function(otf_path) {
  .Deprecated("load_math_font")
  load_math_font(otf_path)
}

#' @rdname gridmicrotex-deprecated
#' @export
check_fonts <- function() {
  .Deprecated("check_math_fonts")
  check_math_fonts()
}

#' @keywords internal
#' @useDynLib gridmicrotex, .registration = TRUE
#' @importFrom Rcpp sourceCpp
#' @importFrom grid makeContent widthDetails heightDetails xDetails yDetails editDetails ascentDetails descentDetails
"_PACKAGE"

# Column names used in ggplot2 aes() calls
utils::globalVariables(c(
  "x", "y", "label", "size", "colour", "hjust", "vjust", "angle", "alpha"
))

#' Inspect the parsed layout of a LaTeX expression
#'
#' Returns the raw draw-record table produced by MicroTeX's layout pass
#' together with the bounding-box metadata. Useful for debugging
#' alignment issues, building custom grobs on top of the layout, or
#' counting glyphs/paths/rules in a formula.
#'
#' @inheritParams latex_grob
#' @return A list with class \code{"latex_tree"} containing:
#'   \describe{
#'     \item{\code{records}}{Data frame of draw records (one row per
#'       glyph, path, line, rect, or text block). Columns include
#'       \code{type}, \code{x}, \code{y}, \code{glyph}, \code{font_size},
#'       \code{color}, \code{text}, \code{codepoint}, \code{font_file}.}
#'     \item{\code{bbox}}{Named numeric vector with \code{width},
#'       \code{height}, \code{depth}, \code{baseline} (all in big points).}
#'     \item{\code{tex}}{The (macro-expanded) input string.}
#'     \item{\code{render_mode}}{Rendering mode used for the layout.}
#'   }
#' @seealso \code{\link{latex_grob}}, \code{\link{latex_dims}}
#' @export
#'
#' @examples
#' \donttest{
#'   tree <- latex_tree("\\frac{a}{b}")
#'   print(tree)
#'   head(tree$records)
#' }
latex_tree <- function(tex, math_font = "", max_width = 0,
                       tex_style = "",
                       input_mode = c("mixed", "math"),
                       render_mode = c("typeface", "path"),
                       gp = grid::gpar()) {
  .apply_opts("math_font", "render_mode", "tex_style", "input_mode")
  render_mode <- match.arg(render_mode)
  input_mode <- match.arg(input_mode)

  parsed <- .parse_from_gp(
    tex = tex, gp = gp, math_font = math_font, max_width = max_width,
    tex_style = tex_style, render_mode = render_mode,
    input_mode = input_mode
  )
  layout <- parsed$layout

  bbox <- c(
    width    = as.numeric(attr(layout, "bbox_width")),
    height   = as.numeric(attr(layout, "bbox_height")),
    depth    = as.numeric(attr(layout, "bbox_depth")),
    baseline = as.numeric(attr(layout, "bbox_baseline"))
  )

  structure(
    list(
      records = layout,
      bbox = bbox,
      tex = parsed$tex,
      render_mode = parsed$render_mode
    ),
    class = "latex_tree"
  )
}

#' @export
print.latex_tree <- function(x, ...) {
  cat("<latex_tree>\n")
  cat("  tex:         ", x$tex, "\n", sep = "")
  cat("  render_mode: ", x$render_mode, "\n", sep = "")
  cat(sprintf(
    "  bbox:        width=%.2f  height=%.2f  depth=%.2f  baseline=%.2f (bigpts)\n",
    x$bbox[["width"]], x$bbox[["height"]],
    x$bbox[["depth"]], x$bbox[["baseline"]]
  ))
  n <- nrow(x$records)
  cat("  records:     ", n, "\n", sep = "")
  if (n > 0) {
    tbl <- table(x$records$type)
    for (nm in names(tbl)) {
      cat(sprintf("    %-10s %d\n", nm, tbl[[nm]]))
    }
  }
  invisible(x)
}

# ggplot2 integration: geom_latex() and element_latex()
# All ggplot2-dependent code is guarded with requireNamespace() checks
# so the core package works without ggplot2 installed.
#
# ggplot2 >= 4.0 uses S7 for theme elements, so element_latex is defined
# as an S7 subclass of ggplot2::element_text. Both S7 and ggplot2 are
# soft dependencies — everything is created dynamically in .onLoad_ggplot2().

# --------------------------------------------------------------------------
# geom_latex() — A ggplot2 geom for rendering LaTeX labels
# --------------------------------------------------------------------------

#' A ggplot2 geom for LaTeX math labels
#'
#' Renders LaTeX math expressions as native grid grobs within a ggplot2 plot.
#' Each label is parsed and laid out by MicroTeX, producing resolution-independent
#' vector output.
#'
#' @section Aesthetics:
#' \code{geom_latex()} understands the following aesthetics (required aesthetics
#' are in bold):
#' \itemize{
#'   \item \strong{\code{x}}
#'   \item \strong{\code{y}}
#'   \item \strong{\code{label}} — LaTeX math string
#'   \item \code{size} — font size in points (default: 11)
#'   \item \code{colour} — text colour (default: \code{"black"})
#'   \item \code{angle} — rotation angle in degrees (default: 0)
#'   \item \code{hjust} — horizontal justification, 0–1 (default: 0.5)
#'   \item \code{vjust} — vertical justification, 0–1 (default: 0.5)
#'   \item \code{alpha} — transparency (default: 1)
#' }
#'
#' @inheritParams ggplot2::layer
#' @inheritParams latex_grob
#' @param fontsize Default font size in points. Overridden by the \code{size}
#'   aesthetic if mapped.
#' @param math_font Name of the math font to use (e.g., \code{"stix"}).
#' @param lineheight Multi-line height multiplier (default 1.2), matching
#'   \code{grid::gpar()} semantics.
#' @param max_width Maximum width in big points for automatic line
#'   wrapping (default: 0, no wrapping).
#' @param na.rm If \code{FALSE}, the default, missing values are removed with
#'   a warning. If \code{TRUE}, missing values are silently removed.
#' @param ... Other arguments passed to \code{\link[ggplot2]{layer}}.
#'
#' @return A ggplot2 layer.
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'   df <- data.frame(
#'     x = 1:3, y = 1:3,
#'     eq = c("x^2", "\\frac{a}{b}", "\\sum_{i=1}^n x_i")
#'   )
#'   ggplot(df, aes(x, y, label = eq)) + geom_latex()
#'
#'   # Use annotate() for single annotations (no legend, no data frame needed)
#'   ggplot(mtcars, aes(wt, mpg)) + geom_point() +
#'     annotate("latex", x = 4, y = 30,
#'              label = r"($\hat{y} = \beta_0 + \beta_1 x$)")
#' }
#' }
geom_latex <- function(mapping = NULL, data = NULL, stat = "identity",
                       position = "identity", ...,
                       fontsize = 11, math_font = "",
                       lineheight = 1.2, max_width = 0,
                       input_mode = c("mixed", "math"),
                       render_mode = c("typeface", "path"),
                       na.rm = FALSE, show.legend = NA,
                       inherit.aes = TRUE) {
  .apply_opts("math_font", "render_mode", "input_mode")
  render_mode <- match.arg(render_mode)
  input_mode <- match.arg(input_mode)
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for geom_latex(). ",
         "Please install it with install.packages('ggplot2').",
         call. = FALSE)
  }

  ggplot2::layer(
    geom = GeomLatex,
    mapping = mapping,
    data = data,
    stat = stat,
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = list(
      fontsize = fontsize,
      math_font = math_font,
      lineheight = lineheight,
      max_width = max_width,
      input_mode = input_mode,
      render_mode = render_mode,
      na.rm = na.rm,
      ...
    )
  )
}


#' @rdname geom_latex
#' @format NULL
#' @usage NULL
#' @export
GeomLatex <- NULL

# --------------------------------------------------------------------------
# element_latex() — A custom ggplot2 theme element (S7)
# --------------------------------------------------------------------------

#' A ggplot2 theme element for LaTeX text
#'
#' Use this as a theme element for axis titles, axis labels, plot titles,
#' or any other text element in a ggplot2 theme. The text string is parsed
#' as LaTeX math and rendered via MicroTeX.
#'
#' Dollar signs (\code{$...$}) in the label text are stripped automatically
#' so that both \code{"\\frac{a}{b}"} and \code{"$\\frac{a}{b}$"} work.
#'
#' This element is an S7 subclass of \code{ggplot2::element_text}, so it
#' inherits all standard text properties (size, colour, hjust, etc.) from
#' the theme and supports \code{merge_element()} correctly.
#'
#' @inheritParams latex_grob
#' @param fontsize Convenience alias for \code{size}; when supplied,
#'   it is forwarded to \code{ggplot2::element_text()} as the text size
#'   in points. If \code{NULL} (default), the theme's inherited size is
#'   used.
#' @param lineheight Multi-line height multiplier (default 1.2), matching
#'   \code{grid::gpar()} semantics.
#' @param ... Additional arguments passed to \code{ggplot2::element_text()}
#'   (e.g., \code{size}, \code{colour}, \code{hjust}).
#'
#' @return An S7 object of class \code{element_latex}, inheriting from
#'   \code{ggplot2::element_text}.
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'   ggplot(mtcars, aes(wt, mpg)) + geom_point() +
#'     labs(x = "$\\beta_1 \\cdot x + \\beta_0$") +
#'     theme(axis.title.x = element_latex())
#' }
#' }
element_latex <- function(math_font = "", fontsize = NULL,
                         lineheight = 1.2, max_width = 0,
                         input_mode = c("mixed", "math"),
                         render_mode = c("typeface", "path"), ...) {
  .apply_opts("math_font", "render_mode", "input_mode")
  render_mode <- match.arg(render_mode)
  input_mode <- match.arg(input_mode)
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for element_latex(). ",
         "Please install it with install.packages('ggplot2').",
         call. = FALSE)
  }
  # Map the convenience 'fontsize' alias to 'size' (element_text param)
  dots <- list(...)
  if (!is.null(fontsize)) {
    dots$size <- fontsize
  }
  obj <- do.call(.element_latex_class, c(list(math_font = math_font, lineheight = lineheight, max_width = max_width, input_mode = input_mode, render_mode = render_mode), dots))
  # ggplot2's element_text constructor injects legacy "element_text" and
  # "element" S3 strings into the class vector; S7 inheritance loses them,
  # which makes combine_elements treat us as an unrelated sibling and drop
  # us when resolving inherited theme entries (e.g. axis.title.y inheriting
  # from a user-set axis.title). Re-inject them so subclass detection works.
  class(obj) <- union(
    union(c("gridmicrotex::element_latex", "ggplot2::element_text", "element_text"), class(obj)),
    "element"
  )
  obj
}

# Placeholder — replaced in .onLoad_ggplot2() with the real S7 constructor
.element_latex_class <- NULL

# --------------------------------------------------------------------------
# .onLoad_ggplot2() — called from .onLoad in zzz.R
# --------------------------------------------------------------------------
.onLoad_ggplot2 <- function() {
  if (!requireNamespace("ggplot2", quietly = TRUE)) return()

  # --- GeomLatex ggproto object ---
  GeomLatex <<- ggplot2::ggproto("GeomLatex", ggplot2::Geom,

    required_aes = c("x", "y", "label"),

    default_aes = ggplot2::aes(
      size = 11,
      colour = "black",
      angle = 0,
      hjust = 0.5,
      vjust = 0.5,
      alpha = 1
    ),

    draw_key = ggplot2::draw_key_text,

    # default_aes is applied after setup_data, so a missing `size` column
    # here means the aesthetic was not mapped — fill it from the fontsize
    # parameter. (In draw_panel the column is always populated, so the
    # parameter could never win there.)
    setup_data = function(data, params) {
      if (is.null(data$size)) {
        data$size <- params$fontsize %||% 11
      }
      data
    },

    draw_panel = function(data, panel_params, coord, fontsize = 11,
                          math_font = "", lineheight = 1.2, max_width = 0,
                          input_mode = "mixed",
                          render_mode = "typeface",
                          na.rm = FALSE) {
      coords <- coord$transform(data, panel_params)

      grobs <- lapply(seq_len(nrow(coords)), function(i) {
        row <- coords[i, ]

        if (is.na(row$label) || !nzchar(row$label)) return(grid::nullGrob())

        fs <- row$size
        col <- if (!is.null(row$alpha) && row$alpha < 1) {
          grDevices::adjustcolor(row$colour, alpha.f = row$alpha)
        } else {
          row$colour
        }

        latex_grob(
          tex = row$label,
          x = grid::unit(row$x, "npc"),
          y = grid::unit(row$y, "npc"),
          hjust = row$hjust,
          vjust = row$vjust,
          rot = row$angle %||% 0,
          math_font = math_font,
          max_width = max_width,
          input_mode = input_mode,
          render_mode = render_mode,
          gp = grid::gpar(col = col, fontsize = fs, lineheight = lineheight)
        )
      })

      do.call(grid::gList, grobs)
    }
  )

  # --- element_latex S7 class ---
  .element_latex_class <<- S7::new_class(
    "element_latex",
    parent = ggplot2::element_text,
    properties = list(
      math_font = S7::new_property(S7::class_character, default = ""),
      lineheight = S7::new_property(S7::class_numeric, default = 1.2),
      max_width = S7::new_property(S7::class_numeric, default = 0),
      input_mode = S7::new_property(S7::class_character, default = "mixed"),
      render_mode = S7::new_property(S7::class_character, default = "typeface")
    )
  )

  # --- element_grob S3 method for element_latex ---
  # element_grob is still an S3 generic in ggplot2, so we register via S3.
  # S7 objects dispatch correctly on S3 generics thanks to their class vector.
  registerS3method(
    "element_grob", "gridmicrotex::element_latex",
    .element_grob_latex,
    envir = asNamespace("ggplot2")
  )

  # The markdown geom and theme element are built the same way, from the
  # same ggplot2/S7 soft dependencies.
  .onLoad_ggplot2_markdown()
}

# --------------------------------------------------------------------------
# Internal helpers
# --------------------------------------------------------------------------

# element_grob method implementation for element_latex.
# Defined outside .onLoad_ggplot2 so it can be referenced by name.
# Signature mirrors ggplot2:::element_grob.element_text so callers (axes,
# guides) can override element properties per-call.
.element_grob_latex <- function(element, label = "", x = NULL, y = NULL,
                                family = NULL, face = NULL, colour = NULL,
                                size = NULL, hjust = NULL, vjust = NULL,
                                angle = NULL, lineheight = NULL,
                                margin = NULL, margin_x = FALSE,
                                margin_y = FALSE, ...) {
  if (is.null(label) || (length(label) == 1L && (!nzchar(label) || is.na(label)))) {
    return(grid::nullGrob())
  }

  # Per-call overrides win, then element values, then a final fallback.
  fontsize    <- size       %||% element@size       %||% 11
  colour      <- colour     %||% element@colour     %||% "black"
  hjust       <- hjust      %||% element@hjust      %||% 0.5
  vjust       <- vjust      %||% element@vjust      %||% 0.5
  angle       <- angle      %||% element@angle      %||% 0
  family      <- family     %||% element@family
  face        <- face       %||% element@face
  lineheight  <- lineheight %||% element@lineheight %||% 1.2
  math_font   <- element@math_font   %||% ""
  max_width   <- element@max_width   %||% 0
  input_mode  <- element@input_mode  %||% "mixed"
  render_mode <- element@render_mode %||% "typeface"

  gp <- grid::gpar(col = colour, fontsize = fontsize, lineheight = lineheight)
  if (!is.null(family) && nzchar(family)) gp$fontfamily <- family
  if (!is.null(face)   && nzchar(face))   gp$fontface   <- face

  # In math mode, strip enclosing $...$ that users add by analogy with
  # plotmath-style labels. In text mode, $ toggles math sub-spans, so
  # leave them intact.
  strip_dollars <- function(s) {
    if (input_mode == "mixed") s else gsub("^\\$|\\$$", "", s)
  }

  # When x/y aren't supplied, anchor at the rotation-adjusted just so the
  # rotated text lands where ggplot2:::titleGrob would put it (e.g. for
  # axis.title.y with angle=90, vjust=1: anchor at left-middle, text reads
  # bottom-to-top across the cell).
  default_just <- .rotate_just(angle, hjust, vjust)

  if (length(label) == 1L) {
    label <- strip_dollars(label)
    if (is.null(x)) x <- grid::unit(default_just$hjust, "npc")
    if (is.null(y)) y <- grid::unit(default_just$vjust, "npc")
    return(latex_grob(
      tex = label, x = x, y = y,
      hjust = hjust, vjust = vjust, rot = angle,
      math_font = math_font, max_width = max_width,
      input_mode = input_mode, render_mode = render_mode,
      gp = gp
    ))
  }

  # Multiple labels (axis tick labels) — render a gTree of grobs.
  label <- strip_dollars(label)
  n <- length(label)
  if (is.null(x)) x <- grid::unit(rep(default_just$hjust, n), "npc")
  if (is.null(y)) y <- grid::unit(rep(default_just$vjust, n), "npc")

  grobs <- grid::gList()
  for (i in seq_len(n)) {
    lab <- label[i]
    if (is.na(lab) || !nzchar(lab)) next
    grobs <- grid::gList(grobs, latex_grob(
      tex = lab, x = .pick_unit(x, i), y = .pick_unit(y, i),
      hjust = hjust, vjust = vjust, rot = angle,
      math_font = math_font, max_width = max_width,
      input_mode = input_mode, render_mode = render_mode,
      gp = gp,
      name = paste0("ticklabel.", i)
    ))
  }
  grid::gTree(children = grobs, name = "axis.latex.labels",
              cl = "gridmicrotex_labels")
}

# Report the extent of a bundle of tick-label grobs.
#
# Without these, the gTree measures 0 x 0: its children sit at npc
# positions and grid has nothing to derive an extent from. ggplot2 sizes
# the axis strip from this grob, so a zero height means no room is
# reserved and the labels are drawn over the axis title.
.labels_extent <- function(x, fn) {
  kids <- x$children
  if (length(kids) == 0L) return(grid::unit(0, "bigpts"))
  max(do.call(grid::unit.c, lapply(kids, fn)))
}

#' @method widthDetails gridmicrotex_labels
#' @export
widthDetails.gridmicrotex_labels <- function(x) {
  .labels_extent(x, grid::grobWidth)
}

#' @method heightDetails gridmicrotex_labels
#' @export
heightDetails.gridmicrotex_labels <- function(x) {
  .labels_extent(x, grid::grobHeight)
}

# ggplot2 passes per-tick positions for one axis and a scalar unit for the
# orthogonal one (e.g. axis.text.x: x is length-n, y is length-1). Recycle
# the scalar so `unit[i]` doesn't error past length 1.
.pick_unit <- function(u, i) {
  if (inherits(u, "unit")) {
    if (length(u) == 1L) u else u[i]
  } else {
    grid::unit(u[((i - 1L) %% length(u)) + 1L], "npc")
  }
}

# Mirror of ggplot2:::rotate_just. Picks the anchor point so that a
# rotated grob with raw (hjust, vjust) lands inside the cell rather than
# overflowing one of its edges.
.rotate_just <- function(angle, hjust, vjust) {
  angle <- (angle %||% 0) %% 360
  if (angle < 90) {
    list(hjust = hjust,     vjust = vjust)
  } else if (angle < 180) {
    list(hjust = 1 - vjust, vjust = hjust)
  } else if (angle < 270) {
    list(hjust = 1 - hjust, vjust = 1 - vjust)
  } else {
    list(hjust = vjust,     vjust = 1 - hjust)
  }
}

# geom_latex(), annotate("latex") and element_latex() are adapters: they
# hand ggplot2 a latex_grob(). So the thing worth asserting is that a grob
# of ours, carrying what the caller asked for, actually lands in the built
# plot. Saving a png and checking it did not error says nothing -- it
# passes just as happily when the layer draws nothing at all.

skip_if_not_installed("ggplot2")

# The grobs a layer contributes to the panel.
layer_grobs <- function(p, i = 1L) ggplot2::layer_grob(p, i)[[1]]

# --- geom_latex() ---

test_that("geom_latex contributes one rendered grob per row", {
  df <- data.frame(x = 1:3, y = 1:3,
                   eq = c("x^2", "\\frac{a}{b}", "\\sum_{i=1}^n x_i"))
  gs <- layer_grobs(ggplot2::ggplot(df, ggplot2::aes(x, y, label = eq)) +
                      geom_latex())
  expect_length(gs, 3L)
  expect_true(all(vapply(gs, inherits, logical(1), "latexgrob")))
  # Each one has something to draw. A layer that silently produced empty
  # grobs would still be a gList of the right length.
  expect_true(all(vapply(gs, function(g) nrow(g$layout_df) > 0, logical(1))))
})

test_that("geom_latex passes colour and size down to the grob", {
  df <- data.frame(x = 1, y = 1, eq = "x^2")
  base <- ggplot2::ggplot(df, ggplot2::aes(x, y, label = eq))

  red <- layer_grobs(base + geom_latex(colour = "red"))[[1]]
  expect_equal(red$gp$col, "red")

  big   <- layer_grobs(base + geom_latex(fontsize = 20))[[1]]
  small <- layer_grobs(base + geom_latex(fontsize = 8))[[1]]
  expect_equal(unique(big$layout_df$font_size), 20)
  expect_equal(unique(small$layout_df$font_size), 8)
  expect_gt(big$bbox_w, small$bbox_w)
})

test_that("geom_latex draws nothing for an empty or missing label", {
  # Without the guard in draw_panel these render the literal "NA", or an
  # empty formula that still reserves space.
  df <- data.frame(x = 1:2, y = 1:2, eq = c("x^2", ""))
  gs <- layer_grobs(ggplot2::ggplot(df, ggplot2::aes(x, y, label = eq)) +
                      geom_latex())
  expect_s3_class(gs[[1]], "latexgrob")
  expect_s3_class(gs[[2]], "null")

  # na.rm drops the row before it ever reaches us.
  df_na <- data.frame(x = 1:2, y = 1:2, eq = c("x^2", NA))
  gs_na <- layer_grobs(ggplot2::ggplot(df_na, ggplot2::aes(x, y, label = eq)) +
                         geom_latex(na.rm = TRUE))
  expect_length(gs_na, 1L)
})

test_that("geom_latex fontsize parameter sets size when the aesthetic is unmapped", {
  df <- data.frame(x = 1, y = 1, eq = "x^2")

  # fontsize parameter fills the size column
  p20 <- ggplot2::ggplot(df, ggplot2::aes(x, y, label = eq)) +
    geom_latex(fontsize = 20)
  expect_equal(ggplot2::ggplot_build(p20)$data[[1]]$size, 20)

  # a mapped size aesthetic wins over the parameter
  df$s <- 30
  p_mapped <- ggplot2::ggplot(df, ggplot2::aes(x, y, label = eq, size = s)) +
    geom_latex(fontsize = 20) +
    ggplot2::scale_size_identity()
  expect_equal(ggplot2::ggplot_build(p_mapped)$data[[1]]$size, 30)
})

# --- annotate('latex') ---

test_that("annotate('latex') adds a rendered layer of its own", {
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
    ggplot2::geom_point() +
    ggplot2::annotate("latex", x = 4, y = 30,
                      label = "\\hat{y} = \\beta_0 + \\beta_1 x",
                      size = 12, colour = "red")
  # Layer 2 is the annotation; the geom has to be reachable by name for
  # annotate() to find it at all.
  g <- layer_grobs(p, 2L)[[1]]
  expect_s3_class(g, "latexgrob")
  expect_gt(nrow(g$layout_df), 0)
  expect_equal(g$gp$col, "red")
})

# --- element_latex() ---

test_that("element_latex is an element_text subclass that merges", {
  el <- element_latex(fontsize = 18, render_mode = "path")
  expect_equal(el@render_mode, "path")
  # `fontsize` is our alias for element_text's `size`; it is not a
  # property of its own, so it has to land there or it is silently lost.
  expect_equal(el@size, 18)

  # Properties the caller did not set come from the parent element.
  merged <- ggplot2::merge_element(el, ggplot2::element_text(colour = "blue"))
  expect_equal(merged@colour, "blue")
  expect_equal(merged@render_mode, "path")
})

test_that("element_latex installs our grob as the axis title", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  p <- ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
    ggplot2::geom_point() +
    ggplot2::labs(x = "$\\beta_1 \\cdot x + \\beta_0$") +
    ggplot2::theme(axis.title.x = element_latex())
  gt <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(p))

  g <- gt$grobs[[which(gt$layout$name == "xlab-b")]]
  expect_s3_class(g, "latexgrob")
  # Set as math, not as the literal dollar-delimited string.
  expect_true("glyph" %in% g$layout_df$type)
})

# Tick labels go through element_grob() with a vector label; that path and
# its reported size are covered in test-ggplot2-markdown.R, which runs it
# for element_latex() as well as element_markdown().

# --- .element_grob_latex() internals ---

test_that(".element_grob_latex handles edge cases and multiple labels", {
  # NULL/empty → nullGrob
  expect_s3_class(
    gridmicrotex:::.element_grob_latex(element_latex(), label = NULL), "null")
  expect_s3_class(
    gridmicrotex:::.element_grob_latex(element_latex(), label = ""), "null")

  # Single label + dollar stripping
  result <- gridmicrotex:::.element_grob_latex(element_latex(), label = "$x^2$")
  expect_s3_class(result, "latexgrob")

  # Multiple labels with NA/empty skipped
  result_multi <- gridmicrotex:::.element_grob_latex(
    element_latex(), label = c("x_1", NA, "", "x_4"))
  expect_equal(length(result_multi$children), 2)
})

# Images: resolving \includegraphics, and drawing what it refers to.
#
# The split is deliberate. R answers two questions at two different times:
#
#   parse time  .image_dims()  -- how big is this file, so the engine can
#                                 reserve a box of the right size
#   draw  time  .image_grob()  -- what grob actually goes in that box
#
# Keeping them apart is what lets an SVG be drawn as true vector: the box
# is fixed early, but the choice of rasterGrob vs grImport2 pictureGrob is
# made late, and a new reader never needs an engine change.
#
# .resolve_graphics() rewrites
#     \includegraphics[width=2cm]{fig.png}
# into the private, fully-resolved
#     \gmgraphics{<hex>}{<width_bp>}{<height_bp>}
# which src/MicroTeX/lib/atom/image_atom.cpp turns into a box and an IMAGE draw record.

# --- reference encoding ------------------------------------------------
#
# The payload is hex because a real path is hostile to everything between
# here and MicroTeX: Windows paths hold backslashes, paths hold spaces,
# .strip_document_wrappers() eats `%`-to-end-of-line, and .expand_macros()
# would rewrite `\Users` or `\Temp` mid-path for anyone who had defined a
# macro by that name. Hex is [0-9a-f]+ and survives all of it.
#
# mtime and size ride along so the layout cache -- which keys on the tex
# string and holds no file metadata -- misses when a figure is edited in
# place. Without them an edited figure would serve a stale layout.
.image_ref_encode <- function(path) {
  info <- file.info(path)
  payload <- paste(path, as.numeric(info$mtime), info$size, sep = "\x1f")
  paste(sprintf("%02x", as.integer(charToRaw(payload))), collapse = "")
}

.image_ref_decode <- function(hex) {
  n <- nchar(hex)
  if (n == 0L || n %% 2L != 0L) return(NULL)
  bytes <- strtoi(substring(hex, seq(1L, n, 2L), seq(2L, n, 2L)), 16L)
  parts <- strsplit(rawToChar(as.raw(bytes)), "\x1f", fixed = TRUE)[[1]]
  if (!length(parts)) return(NULL)
  parts[1]
}

# --- units -------------------------------------------------------------

# One LaTeX length -> big points. `\textwidth` / `\linewidth` have no page
# to relate to in a grob, so they resolve against max_width; without one
# they read as NA and the caller falls back to the file's own size, which
# keeps the figure on the page. Returns NA when the length cannot be read.
.REL_LEN <- "\\\\(?:text|line|column)width"

.tex_len_bp <- function(s, fontsize, max_width) {
  s <- trimws(s)
  if (!nzchar(s)) return(NA_real_)
  # A leading multiplier on \textwidth / \linewidth, e.g. 0.8\textwidth.
  # Two capture groups, so a match is length 3: whole, multiplier, name.
  m <- regmatches(s, regexec("^([0-9.eE+-]*)\\\\(textwidth|linewidth|columnwidth)$", s))[[1]]
  if (length(m) == 3L) {
    if (!isTRUE(max_width > 0)) return(NA_real_)
    mult <- if (nzchar(m[2])) suppressWarnings(as.numeric(m[2])) else 1
    return(if (is.finite(mult)) mult * max_width else NA_real_)
  }
  # The exponent must be followed by digits, or a greedy [0-9.eE+-]+ would
  # read "2em" as the number "2e" and the unit "m".
  m <- regmatches(s, regexec(
    "^([+-]?(?:[0-9]*\\.?[0-9]+(?:[eE][+-]?[0-9]+)?))\\s*([A-Za-z]*)$", s))[[1]]
  if (length(m) != 3L) return(NA_real_)
  v <- suppressWarnings(as.numeric(m[2]))
  if (!is.finite(v)) return(NA_real_)
  # Big points per unit. `pt` is TeX's 1/72.27in, `bp` the 1/72in this
  # package works in; `px` follows the 96dpi screen convention used for
  # intrinsic sizes. `ex` is approximated as half an em, as elsewhere.
  # A bare number means big points. (Handled before switch(): an empty
  # name is not legal there.)
  unit <- tolower(m[3])
  per <- if (!nzchar(unit)) 1 else switch(unit,
    bp   = 1,
    pt   = 72 / 72.27,
    pc   = 12 * 72 / 72.27,
    `in` = 72,
    cm   = 72 / 2.54,
    mm   = 72 / 25.4,
    px   = 72 / 96,
    em   = fontsize,
    ex   = fontsize / 2,
    NA_real_
  )
  if (is.na(per)) return(NA_real_)
  v * per
}

# --- intrinsic size ----------------------------------------------------

.image_ext <- function(path) tolower(tools::file_ext(path))

# The extensions graphicx tries when the argument has none, vector first,
# mirroring the driver's own `\DeclareGraphicsExtensions{.pdf,.png,...}`.
.IMAGE_EXTS <- c("svg", "png", "jpg", "jpeg")

# Find the file `\includegraphics{...}` names. LaTeX does two things here
# that a bare file.exists() does not: it appends a known extension when the
# argument has none (`{plots/fig}` is the idiomatic form), and it looks in
# the directories `\graphicspath` declared. Returns the path unchanged when
# nothing matches, so the caller reports the name the user wrote.
.image_find <- function(path, dirs = character(0)) {
  if (!nzchar(path)) return(path)
  bases <- path
  # An absolute path ignores \graphicspath, as in LaTeX.
  if (length(dirs) && !grepl("^(?:[A-Za-z]:[/\\\\]|[/\\\\~])", path)) {
    bases <- c(path, file.path(sub("[/\\\\]+$", "", dirs), path))
  }
  for (b in bases) {
    if (file.exists(b) && !dir.exists(b)) return(b)
    for (e in .IMAGE_EXTS) {
      cand <- paste0(b, ".", e)
      if (file.exists(cand)) return(cand)
    }
  }
  path
}

# `\graphicspath{{figs/}{../shared/}}` -> the directories, and the tex with
# the declaration removed. Removed rather than left alone because MicroTeX
# has no such command and would typeset the argument as visible text.
.extract_graphicspath <- function(tex) {
  dirs <- character(0)
  repeat {
    m <- regexpr("\\\\graphicspath\\s*\\{", tex, perl = TRUE)
    if (m == -1L) break
    open <- m + attr(m, "match.length") - 1L
    close <- .find_close_brace(tex, open + 1L)
    if (is.na(close)) break
    inner <- substr(tex, open + 1L, close - 1L)
    dirs <- c(dirs, trimws(regmatches(inner,
      gregexpr("(?<=\\{)[^{}]*(?=\\})", inner, perl = TRUE))[[1]]))
    tex <- paste0(substr(tex, 1L, m - 1L), substr(tex, close + 1L, nchar(tex)))
  }
  list(tex = tex, dirs = dirs[nzchar(dirs)])
}

# Intrinsic size in big points, or NULL when the file cannot be read.
# `px` is a list(w, h) of pixels for bitmaps and NULL for vector sources,
# so the caller knows whether an effective-dpi warning makes sense.
.image_dims <- function(path) {
  if (!nzchar(path) || !file.exists(path)) return(NULL)
  ext <- .image_ext(path)
  if (ext == "svg") {
    # Both draw-time routes go through rsvg -- grImport2 needs it to
    # normalise to Cairo SVG, and the raster fallback *is* it. Without it
    # nothing can be drawn, so refuse the size too: reserving a box we
    # cannot fill would leave a silent hole where the figure belongs.
    if (!requireNamespace("rsvg", quietly = TRUE)) return(NULL)
    d <- .svg_dims_bp(path)
    if (is.null(d)) return(NULL)
    return(list(w = d$w, h = d$h, px = NULL))
  }
  ras <- .image_raster(path)
  if (is.null(ras)) return(NULL)
  # 96 dpi, the screen convention this package already uses for block
  # images in markdown.
  list(w = ras$w_px * 72 / 96, h = ras$h_px * 72 / 96,
       px = list(w = ras$w_px, h = ras$h_px))
}

# An SVG's intrinsic size, from the root element. Read from the file the
# user gave us, not from the rsvg-normalised copy: svglite writes
# `width='216.00pt'` (3in), while the normalised form writes a bare `216`,
# which is user units (px) and would be 2.25in.
.svg_dims_bp <- function(path) {
  x <- tryCatch(xml2::read_xml(path), error = function(e) NULL)
  if (is.null(x)) return(NULL)
  # Well-formed XML is not an SVG. Without this a `.svg` holding anything
  # else with width/height attributes would size a box that no reader can
  # then fill.
  if (!identical(xml2::xml_name(x), "svg")) return(NULL)
  one <- function(a) {
    v <- xml2::xml_attr(x, a)
    if (is.na(v) || !nzchar(v)) return(NA_real_)
    # SVG's own unit set; a bare number is user units, i.e. px.
    m <- regmatches(v, regexec("^\\s*([0-9.eE+-]+)\\s*([a-z%]*)\\s*$", v))[[1]]
    if (length(m) != 3L) return(NA_real_)
    n <- suppressWarnings(as.numeric(m[2]))
    if (!is.finite(n)) return(NA_real_)
    # A bare number is SVG user units, i.e. px at 96dpi.
    per <- if (!nzchar(m[3])) 72 / 96 else switch(m[3],
      px = 72 / 96, pt = 1, pc = 12,
      `in` = 72, cm = 72 / 2.54, mm = 72 / 25.4, NA_real_)
    if (is.na(per)) return(NA_real_)
    n * per
  }
  w <- one("width"); h <- one("height")
  if (is.na(w) || is.na(h)) {
    vb <- xml2::xml_attr(x, "viewBox")
    if (is.na(vb)) return(NULL)
    p <- suppressWarnings(as.numeric(strsplit(trimws(vb), "[ ,]+")[[1]]))
    if (length(p) != 4L || anyNA(p)) return(NULL)
    # viewBox is in user units.
    w <- p[3] * 72 / 96; h <- p[4] * 72 / 96
  }
  if (!is.finite(w) || !is.finite(h) || w <= 0 || h <= 0) return(NULL)
  list(w = w, h = h)
}

# --- loading -----------------------------------------------------------

.image_cache <- new.env(parent = emptyenv())

.image_cache_clear <- function() {
  rm(list = ls(.image_cache, all.names = TRUE), envir = .image_cache)
  invisible(NULL)
}

.image_stamp <- function(path) {
  i <- file.info(path)
  paste(path, as.numeric(i$mtime), i$size, sep = "\x1f")
}

# Read a bitmap. Moved here from R/markdown-box.R unchanged so the inline
# and block paths share one loader; png and jpeg stay Suggests, so a
# missing reader degrades rather than becoming a hard dependency.
.image_raster <- function(path) {
  if (is.null(path) || is.na(path) || !nzchar(path)) return(NULL)
  if (!file.exists(path)) return(NULL)
  key <- paste0("ras\x1f", .image_stamp(path))
  hit <- .image_cache[[key]]
  if (!is.null(hit)) return(hit)
  reader <- switch(.image_ext(path),
    png = if (requireNamespace("png", quietly = TRUE)) png::readPNG else NULL,
    jpg = ,
    jpeg = if (requireNamespace("jpeg", quietly = TRUE)) jpeg::readJPEG else NULL,
    NULL
  )
  if (is.null(reader)) return(NULL)
  ras <- tryCatch(reader(path), error = function(e) NULL)
  if (is.null(ras) || length(dim(ras)) < 2L) return(NULL)
  out <- list(raster = grDevices::as.raster(ras),
              w_px = dim(ras)[2], h_px = dim(ras)[1])
  .image_cache[[key]] <- out
  out
}

# An SVG as real grid primitives. grImport2 needs Cairo SVG; svglite's
# output is not (verified: readPicture() fails with "color intensity NA"),
# so rsvg normalises it first. checkValidSVG() warns even on input it then
# reads correctly, so its noise is suppressed rather than shown to users.
.image_picture <- function(path) {
  if (!requireNamespace("grImport2", quietly = TRUE) ||
      !requireNamespace("rsvg", quietly = TRUE)) {
    return(NULL)
  }
  key <- paste0("pic\x1f", .image_stamp(path))
  hit <- .image_cache[[key]]
  if (!is.null(hit)) return(hit)
  out <- tryCatch({
    cairo <- tempfile(fileext = ".svg")
    on.exit(unlink(cairo), add = TRUE)
    rsvg::rsvg_svg(path, cairo)
    suppressWarnings(grImport2::readPicture(cairo))
  }, error = function(e) NULL)
  if (is.null(out)) return(NULL)
  .image_cache[[key]] <- out
  out
}

# The device's resolution, for choosing how finely to rasterise a vector
# source. Falls back to 96 when no device can answer.
.device_dpi <- function() {
  dpi <- tryCatch({
    px <- grDevices::dev.size("px"); inch <- grDevices::dev.size("in")
    if (length(px) && length(inch) && inch[1] > 0) px[1] / inch[1] else NA_real_
  }, error = function(e) NA_real_)
  if (!is.finite(dpi) || dpi <= 0) 96 else dpi
}

# The grob to place in the reserved box, chosen at draw time.
#
#   1. SVG as true vector, via grImport2
#   2. SVG rasterised by rsvg at the *output device's* resolution -- the
#      one thing that cannot be decided when the file is written
#   3. PNG / JPEG
#
# NULL when nothing can read it; the resolver has already drawn the name.
.image_grob <- function(path, w_bp, h_bp,
                        x = grid::unit(0, "bigpts"),
                        y = grid::unit(0, "bigpts"), name = NULL,
                        angle = 0) {
  # `x`/`y` are the top-left corner as grid units, so a caller can pass an
  # expression -- the block-markdown layout positions from the top with
  # `unit(1,"npc") - unit(y,"bigpts")`, which a bare number cannot express.
  #
  # Under `angle` they are the box's CENTRE instead, because that is the only
  # anchor a rotation leaves put. The unrotated shapes below are deliberately
  # untouched by this branch: a rotated image is rare, and wrapping every
  # image in a viewport would change what `.md_shift_grob()` and the block
  # layout see.
  if (!isTRUE(all.equal(angle %||% 0, 0))) {
    inner <- .image_grob(path, w_bp, h_bp,
                         x = grid::unit(0, "npc"), y = grid::unit(1, "npc"),
                         name = name)
    if (is.null(inner)) return(NULL)
    return(grid::grobTree(inner, vp = grid::viewport(
      x = x, y = y,
      width = grid::unit(w_bp, "bigpts"), height = grid::unit(h_bp, "bigpts"),
      just = "centre", angle = angle), name = name %||% "image"))
  }
  place <- list(x = x, y = y,
                width = grid::unit(w_bp, "bigpts"),
                height = grid::unit(h_bp, "bigpts"))
  if (.image_ext(path) == "svg") {
    pic <- .image_picture(path)
    if (!is.null(pic)) {
      # Wrapped in a gTree carrying a plain viewport rather than positioned
      # directly. pictureGrob's own `vp` is a vpStack, whose `x` is NULL, so
      # .md_shift_grob() -- which nudges `vp$x` to realign a block -- would
      # do `NULL + unit(...)` and error. With this wrapper both it and the
      # inline builder see an ordinary viewport they can move.
      return(grid::grobTree(
        grImport2::pictureGrob(
          pic, x = grid::unit(0.5, "npc"), y = grid::unit(0.5, "npc"),
          width = grid::unit(1, "npc"), height = grid::unit(1, "npc"),
          just = "centre", expansion = 0, distort = TRUE),
        vp = grid::viewport(x = place$x, y = place$y,
                            width = place$width, height = place$height,
                            just = c("left", "top")),
        name = name %||% "image"))
    }
    if (requireNamespace("rsvg", quietly = TRUE)) {
      px <- max(1L, as.integer(round(w_bp / 72 * .device_dpi())))
      arr <- tryCatch(rsvg::rsvg(path, width = px), error = function(e) NULL)
      if (!is.null(arr)) {
        return(grid::rasterGrob(
          grDevices::as.raster(arr), x = place$x, y = place$y,
          width = place$width, height = place$height,
          just = c("left", "top"), interpolate = TRUE, name = name))
      }
    }
    return(NULL)
  }
  ras <- .image_raster(path)
  if (is.null(ras)) return(NULL)
  grid::rasterGrob(ras$raster, x = place$x, y = place$y,
                   width = place$width, height = place$height,
                   just = c("left", "top"), interpolate = TRUE, name = name)
}

# Can this reference actually be drawn? Used by the markdown paths, which
# must decide *before* emitting: a drawable file becomes
# `\includegraphics{}`, and anything else -- a web URL, a missing file, an
# unsupported format -- keeps its alt text and stays silent, which is what
# markdown has always done and what stops a remote image warning.
.image_drawable <- function(path) {
  !is.null(path) && !is.na(path) && nzchar(path) && !is.null(.image_dims(path))
}

# --- the resolver ------------------------------------------------------

# One warning per file per *state*, not per parse.
#
# Resetting this per parse would defeat it entirely: .md_measure() calls
# latex_dims() up to seven times and editDetails() re-runs the pipeline,
# and each of those is its own parse. So the key carries the file's stamp
# instead and the record is kept for the session -- a file that is later
# created or edited has a different stamp and is allowed to speak again,
# while a persistently missing one is reported exactly once.
.image_warned <- new.env(parent = emptyenv())

.image_warn_once <- function(key, msg) {
  if (!is.null(.image_warned[[key]])) return(invisible(FALSE))
  .image_warned[[key]] <- TRUE
  warning(msg, call. = FALSE)
  invisible(TRUE)
}

.image_reset_warnings <- function() {
  rm(list = ls(.image_warned, all.names = TRUE), envir = .image_warned)
}

# Split an option list on commas that are not inside braces.
.split_opts <- function(s) {
  chars <- strsplit(s, "", fixed = TRUE)[[1]]
  out <- character(0); buf <- character(0); depth <- 0L
  for (ch in chars) {
    if (ch == "{") depth <- depth + 1L
    if (ch == "}") depth <- max(0L, depth - 1L)
    if (ch == "," && depth == 0L) {
      out <- c(out, paste(buf, collapse = "")); buf <- character(0)
    } else {
      buf <- c(buf, ch)
    }
  }
  c(out, paste(buf, collapse = ""))
}

# `[width=2cm,keepaspectratio]` -> list(width = "2cm", keepaspectratio =
# "true"). graphicx keys may be valueless (keepaspectratio, clip, draft),
# which a `key=value` pattern alone would drop.
.parse_graphics_opts <- function(opt) {
  out <- list()
  for (kv in .split_opts(opt)) {
    kv <- trimws(kv)
    if (!nzchar(kv)) next
    p <- regmatches(kv, regexec("^\\s*([A-Za-z]+)\\s*=\\s*(.*?)\\s*$", kv))[[1]]
    if (length(p) == 3L) {
      out[[tolower(p[2])]] <- gsub("^\\{|\\}$", "", p[3])
    } else if (grepl("^[A-Za-z]+$", kv)) {
      out[[tolower(kv)]] <- "true"
    }
  }
  out
}

.opt_flag <- function(v) !is.null(v) && !tolower(v) %in% c("false", "0", "off")

# The final width/height in big points for one image.
.image_size_bp <- function(dims, opts, fontsize, max_width) {
  w <- .tex_len_bp(opts[["width"]] %||% "", fontsize, max_width)
  h <- .tex_len_bp(opts[["height"]] %||% "", fontsize, max_width)
  aspect <- dims$h / dims$w
  if (is.na(w) && is.na(h)) {
    w <- dims$w; h <- dims$h
  } else if (is.na(h)) {
    h <- w * aspect
  } else if (is.na(w)) {
    w <- h / aspect
  } else if (.opt_flag(opts[["keepaspectratio"]])) {
    # graphicx fits the picture *inside* width x height rather than
    # stretching it to fill: the smaller of the two scale factors wins.
    s <- min(w / dims$w, h / dims$h)
    w <- dims$w * s; h <- dims$h * s
  }
  # `scale` multiplies whatever width/height settled on, as in graphicx --
  # it is not an alternative to them.
  sc <- suppressWarnings(as.numeric(opts[["scale"]] %||% NA))
  if (is.finite(sc) && sc > 0) { w <- w * sc; h <- h * sc }
  # An absolutely sized image cannot shrink, so .md_measure()'s retry loop
  # would never converge on one wider than its column: clamp here instead.
  if (isTRUE(max_width > 0) && w > max_width) {
    h <- h * max_width / w
    w <- max_width
  }
  list(w = w, h = h)
}

# Rewrite every \includegraphics in `tex`. Idempotent: the output is
# \gmgraphics, which this never matches.
#
# Runs twice in .parse_from_gp() -- once at the very top, before
# .strip_document_wrappers() and .expand_macros() can mangle a path, and
# once after macro expansion to catch an \includegraphics a user macro
# produced. A third case, \newcommand expanded inside MicroTeX's own
# parser, is out of reach of both and is handled by the C++ override in
# src/MicroTeX/lib/atom/image_atom.cpp, which draws the file's name.
.resolve_graphics <- function(tex, fontsize = 20, max_width = 0) {
  gp <- .extract_graphicspath(tex)
  out <- gp$tex
  if (!grepl("\\includegraphics", out, fixed = TRUE)) return(out)
  from <- 1L
  repeat {
    # The starred form clips to the bounding box; there is nothing outside
    # the box to clip here, so it differs from the plain form only in being
    # spelled with a `*`.
    m <- regexpr("\\\\includegraphics\\*?", substr(out, from, nchar(out)), perl = TRUE)
    if (m == -1L) break
    start <- from + m - 1L
    i <- start + attr(m, "match.length")
    n <- nchar(out)
    skip_ws <- function(k) { while (k <= n && grepl("^[ \t\r\n]$", substr(out, k, k))) k <- k + 1L; k }
    i <- skip_ws(i)
    opt <- character(0)
    bad <- FALSE
    # Up to two bracket groups: the graphicx `[key=val]` and the older
    # `[llx,lly][urx,ury]` spelling, whose bare numbers simply parse as no
    # recognised key.
    while (i <= n && substr(out, i, i) == "[" && length(opt) < 2L) {
      # Scan to the matching `]`, ignoring one inside braces so
      # `[width={0.5\textwidth}]` survives.
      j <- i + 1L; depth <- 0L
      while (j <= n) {
        ch <- substr(out, j, j)
        if (ch == "{") depth <- depth + 1L
        else if (ch == "}") depth <- max(0L, depth - 1L)
        else if (ch == "]" && depth == 0L) break
        j <- j + 1L
      }
      if (j > n) { bad <- TRUE; break }
      opt <- c(opt, substr(out, i + 1L, j - 1L))
      i <- skip_ws(j + 1L)
    }
    if (bad) break
    if (i > n || substr(out, i, i) != "{") { from <- start + 1L; next }
    close <- .find_close_brace(out, i + 1L)
    if (is.na(close)) break
    path <- trimws(substr(out, i + 1L, close - 1L))

    rep <- .resolve_one_graphic(path, paste(opt, collapse = ","),
                                fontsize, max_width, gp$dirs)
    out <- paste0(substr(out, 1L, start - 1L), rep,
                  substr(out, close + 1L, nchar(out)))
    from <- start + nchar(rep)
  }
  out
}

# graphicx keys that change what is drawn but that a fixed box cannot
# honour. Ignoring them silently would draw a figure that is wrong in a way
# nothing in the output hints at. `angle` is NOT among them: it is turned
# into a \rotatebox below, which the recorder and the grid builder carry
# through to a rotated viewport.
.IMAGE_IGNORED_OPTS <- c("origin", "trim", "clip", "viewport", "bb")

.resolve_one_graphic <- function(path, opt, fontsize, max_width,
                                 dirs = character(0)) {
  named <- path
  path <- .image_find(path, dirs)
  fallback <- function() paste0("\\text{", .md_escape_tex(basename(named)), "}")
  dims <- .image_dims(path)
  if (is.null(dims)) {
    ext <- .image_ext(path)
    why <- if (!file.exists(path)) {
      "file not found"
    } else if (dir.exists(path)) {
      "it is a directory, not a file"
    } else if (ext %in% c("pdf", "eps", "ps")) {
      paste0("'", ext, "' is not supported; save the figure as SVG instead")
    } else if (ext %in% c("png", "jpg", "jpeg")) {
      # "no dimensions" has two causes and they need different answers:
      # the reader is absent, or the reader is present and the bytes are
      # not what the extension claims.
      pkg <- if (ext == "png") "png" else "jpeg"
      if (!requireNamespace(pkg, quietly = TRUE)) {
        paste0("the '", pkg, "' package is needed to read it")
      } else {
        paste0("the '", pkg, "' package could not read it -- the file may be ",
               "empty, truncated, or not really ", toupper(ext))
      }
    } else if (!nzchar(ext)) {
      "it has no extension, and no .svg, .png, .jpg or .jpeg of that name exists"
    } else if (ext == "svg") {
      if (!requireNamespace("rsvg", quietly = TRUE)) {
        "drawing an SVG needs the 'rsvg' package"
      } else {
        "it has no <svg> root element, or no readable size on it"
      }
    } else {
      paste0("'", ext, "' is not a supported image format (PNG, JPEG, SVG)")
    }
    .image_warn_once(paste0("miss\x1f", .image_stamp(path)),
                     paste0("Cannot draw '", named, "': ", why,
                            ". Drawing the file name instead."))
    return(fallback())
  }

  opts <- .parse_graphics_opts(opt)

  ignored <- intersect(names(opts), .IMAGE_IGNORED_OPTS)
  if (length(ignored)) {
    .image_warn_once(
      paste0("opt\x1f", .image_stamp(path), "\x1f", paste(ignored, collapse = ",")),
      paste0("Ignoring \\includegraphics option",
             if (length(ignored) > 1L) "s " else " ",
             paste0("`", ignored, "`", collapse = ", "),
             " for '", basename(named), "': the image is drawn whole, ",
             "uncropped, and about its own centre."))
  }

  # A length that cannot be read is silently ignored by the arithmetic
  # below, so `[width=3 inches]` or a typo would come out at the file's own
  # size with nothing to say why. `\textwidth` without a max_width is the
  # one unreadable case with its own message, just below.
  for (k in c("width", "height")) {
    v <- opts[[k]]
    if (is.null(v) || !nzchar(trimws(v))) next
    if (grepl(.REL_LEN, v, perl = TRUE)) next
    if (is.na(.tex_len_bp(v, fontsize, max_width))) {
      .image_warn_once(
        paste0("len\x1f", .image_stamp(path), "\x1f", k, "\x1f", v),
        paste0("Cannot read `", k, "=", v, "` for '", basename(named),
               "'; drawing it at its own size."))
    }
  }
  if (!is.null(opts[["scale"]])) {
    sc <- suppressWarnings(as.numeric(opts[["scale"]]))
    if (!is.finite(sc) || sc <= 0) {
      .image_warn_once(
        paste0("scl\x1f", .image_stamp(path), "\x1f", opts[["scale"]]),
        paste0("Ignoring `scale=", opts[["scale"]], "` for '", basename(named),
               "': a scale must be a positive number."))
    }
  }

  # `\textwidth` has no page to measure in a grob. Falling back to the
  # file's own size keeps the figure visible, which is nearer to what the
  # document meant than dropping it, but it is not what was asked for.
  if (!isTRUE(max_width > 0) &&
      any(grepl(.REL_LEN, unlist(opts[c("width", "height")]), perl = TRUE))) {
    .image_warn_once(
      paste0("relw\x1f", .image_stamp(path)),
      paste0("'", basename(named), "' is sized against `\\textwidth`, which ",
             "has no meaning without `max_width`; drawing it at its own size."))
  }

  size <- tryCatch(.image_size_bp(dims, opts, fontsize, max_width),
                   error = function(e) { .image_warn_once(
                     paste0("size\x1f", .image_stamp(path), "\x1f", opt),
                     paste0("Cannot size '", named, "': ", conditionMessage(e))); NULL })
  # The floor is where `sprintf("%.4f")` below stops being able to say a
  # number: anything smaller writes "0.0000" and reserves a box of nothing,
  # which draws as an invisible gap rather than as a very small picture.
  if (is.null(size) || !is.finite(size$w) || !is.finite(size$h) ||
      size$w < 5e-4 || size$h < 5e-4) {
    if (!is.null(size) && is.finite(size$w) && is.finite(size$h)) {
      .image_warn_once(
        paste0("zero\x1f", .image_stamp(path), "\x1f", opt),
        sprintf(paste0("`%s` sizes '%s' to %.4g x %.4g bp, which is nothing ",
                       "to draw. Drawing the file name instead."),
                opt, basename(named), size$w, size$h))
    }
    return(fallback())
  }

  # Effective resolution, for bitmaps drawn at a size the author chose.
  #
  # Only then: at intrinsic size the answer is 96 dpi by construction --
  # that is the convention pixels are read with -- so warning there would
  # fire for every unsized bitmap and say nothing actionable. The useful
  # case is "you asked for 3in and there are only 96 pixels". The size
  # check, not just the option check, because a `\textwidth` that could not
  # resolve lands back on the intrinsic size with `width` still named.
  #
  # A vector source has no effective resolution at all, which is precisely
  # the reason to prefer SVG.
  sized <- any(c("width", "height", "scale") %in% names(opts)) &&
    !isTRUE(all.equal(size$w, dims$w))
  if (sized && !is.null(dims$px)) {
    dpi <- dims$px$w * 72 / size$w
    if (is.finite(dpi) && dpi < 150) {
      # %.0f, not %d: a preposterous width -- `[width=1e7in]` -- makes a
      # pixel count no integer can hold, and sprintf("%d") errors on it
      # rather than degrading.
      want <- ceiling(size$w / 72 * 300)
      want_h <- ceiling(size$h / 72 * 300)
      .image_warn_once(
        paste0("dpi\x1f", .image_stamp(path), "\x1f", round(size$w)),
        sprintf(paste0("Image '%s' is %.0f dpi at this size; save it at ",
                       "%.0f x %.0f px for 300 dpi, or use SVG."),
                basename(path), dpi, want, want_h))
    }
  }

  # sprintf, never as.character: Units::getUnit() falls back to "pixel" for
  # an unrecognised suffix with no error path, so "1e-05bp" would silently
  # parse as 1 pixel.
  out <- sprintf("\\gmgraphics{%s}{%.4f}{%.4f}",
                 .image_ref_encode(path), size$w, size$h)

  # `angle` becomes a real rotation by handing it to \rotatebox, which the
  # engine already knows: it rotates the transform, the recorder reads the
  # angle off it, and the grid builder places the grob in a rotated
  # viewport. \rotatebox also grows the surrounding box to the rotated
  # bounds, which is what LaTeX does and what a fixed-size record could not
  # have expressed on its own.
  ang <- suppressWarnings(as.numeric(opts[["angle"]] %||% NA))
  if (is.finite(ang) && ang %% 360 != 0) {
    out <- sprintf("\\rotatebox{%.4f}{%s}", ang, out)
  }
  out
}

# \includegraphics: resolving a file reference into a sized box, and
# drawing whatever that file deserves.
#
# The resolver runs before every other rewriting step, so most of what is
# under test here is that a real path survives contact with the pipeline.

# A PNG of a known pixel size. `width`/`height` are PIXELS for agg_png,
# which is easy to get wrong -- 1 means one pixel, not one inch.
mk_png <- function(w_px = 300, h_px = 200) {
  f <- tempfile(fileext = ".png")
  ragg::agg_png(f, width = w_px, height = h_px)
  grid::grid.rect(gp = grid::gpar(fill = "tomato", col = NA))
  grDevices::dev.off()
  f
}

mk_svg <- function(w_in = 3, h_in = 2) {
  f <- tempfile(fileext = ".svg")
  svglite::svglite(f, width = w_in, height = h_in)
  grid::grid.circle(r = 0.4, gp = grid::gpar(fill = "steelblue", col = NA))
  grDevices::dev.off()
  f
}

# Warnings are suppressed here on purpose: sizing a small test image to a
# few inches legitimately trips the low-resolution notice, and that notice
# has its own test below. These helpers are about geometry.
wid <- function(tex, ...) {
  suppressWarnings(as.numeric(latex_dims(tex, input_mode = "math", ...)$width))
}
hei <- function(tex, ...) {
  suppressWarnings(as.numeric(latex_dims(tex, input_mode = "math", ...)$height))
}

test_that("a length is read in every unit LaTeX writes one in", {
  bp <- gridmicrotex:::.tex_len_bp
  expect_equal(bp("3in", 20, 0), 216)
  expect_equal(bp("216bp", 20, 0), 216)
  expect_equal(bp("72pt", 20, 0), 72 * 72 / 72.27)
  expect_equal(bp("2.54cm", 20, 0), 72, tolerance = 1e-6)
  expect_equal(bp("25.4mm", 20, 0), 72, tolerance = 1e-6)
  expect_equal(bp("96px", 20, 0), 72)
  expect_equal(bp("2em", 20, 0), 40)
  expect_equal(bp("12", 20, 0), 12)      # bare number is big points

  # \textwidth has no page in a grob, so it resolves against max_width;
  # without one it is unreadable and the caller falls back to the file's
  # own size rather than dropping the figure.
  expect_equal(bp("\\textwidth", 20, 400), 400)
  expect_equal(bp("0.5\\textwidth", 20, 400), 200)
  expect_equal(bp("0.5\\linewidth", 20, 400), 200)
  expect_true(is.na(bp("\\textwidth", 20, 0)))

  # Unreadable rather than guessed.
  expect_true(is.na(bp("3furlongs", 20, 0)))
  expect_true(is.na(bp("", 20, 0)))
})

test_that("a file reference survives the pipeline that would mangle a path", {
  enc <- gridmicrotex:::.image_ref_encode
  dec <- gridmicrotex:::.image_ref_decode
  # Hex, because every one of these breaks a raw path: backslashes are
  # LaTeX escapes, `%` starts a comment that .strip_document_wrappers()
  # would run to end of line, and .expand_macros() would rewrite `\Users`
  # for anyone who had defined a macro of that name.
  for (p in c("C:\\Users\\a\\my fig.png", "a/b/100%plot.png",
              "with space_and_under.png", "plain.png")) {
    expect_equal(dec(enc(p)), p, info = p)
  }
  # Hex is safe in a LaTeX argument: no backslash, brace or percent.
  expect_match(enc("C:\\Users\\a b%c.png"), "^[0-9a-f]+$")
})

test_that("an image reserves a box of exactly the requested size", {
  skip_if_not_installed("ragg")
  skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_png(300, 200)

  # Intrinsic: pixels at 96 dpi, the convention the block path already used.
  expect_equal(wid(sprintf("\\includegraphics{%s}", f)), 300 * 72 / 96)
  expect_equal(hei(sprintf("\\includegraphics{%s}", f)), 200 * 72 / 96)

  # Absolute, and independent of font size -- it is a picture, not type.
  for (fs in c(10, 40)) {
    expect_equal(
      wid(sprintf("\\includegraphics[width=3in]{%s}", f),
          gp = grid::gpar(fontsize = fs)), 216, info = fs)
  }

  # Aspect is preserved when only one side is given, and deliberately not
  # when both are, as in LaTeX.
  expect_equal(hei(sprintf("\\includegraphics[width=3in]{%s}", f)), 144)
  expect_equal(wid(sprintf("\\includegraphics[height=1in]{%s}", f)), 108)
  # 225 * 0.5 = 112.5; the bbox is reported in whole big points.
  expect_equal(wid(sprintf("\\includegraphics[scale=0.5]{%s}", f)), 112)
  expect_equal(hei(sprintf("\\includegraphics[width=3in,height=1in]{%s}", f)), 72)

  # It sits on the baseline: no depth, as \includegraphics does in LaTeX.
  d <- latex_dims(sprintf("\\includegraphics[width=1in]{%s}", f),
                  input_mode = "math")
  expect_equal(as.numeric(d$depth), 0)
})

test_that("an oversized image is clamped so a markdown column can converge", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_png(300, 200)
  # .md_measure() shrinks max_width and retries up to six times; an
  # absolutely sized image never shrinks, so without a clamp the loop
  # burns six parses and still overflows.
  expect_equal(wid(sprintf("\\includegraphics[width=10in]{%s}", f),
                   max_width = 200), 200)
  expect_lte(wid(sprintf("\\includegraphics{%s}", f), max_width = 100), 100)
})

test_that("the record and the grob carry the file through", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_png()
  g <- latex_grob(sprintf("\\includegraphics[width=1in]{%s}", f),
                  input_mode = "math")
  d <- g$layout_df
  expect_true("image" %in% d$type)
  row <- d[d$type == "image", ]
  expect_equal(nrow(row), 1L)
  expect_equal(row$width, 72)
  expect_equal(gridmicrotex:::.image_ref_decode(row$image_ref), f)

  kids <- grid::makeContent(g)$children
  expect_true(any(vapply(kids, inherits, logical(1), "rastergrob")))
})

test_that("an SVG is drawn as real vector, not a raster", {
  skip_if_not_installed("svglite")
  skip_if_not_installed("rsvg")
  skip_if_not_installed("grImport2")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_svg(3, 2)

  # Intrinsic size comes from the SVG header, which svglite writes in pt.
  expect_equal(wid(sprintf("\\includegraphics{%s}", f)), 216)
  expect_equal(hei(sprintf("\\includegraphics{%s}", f)), 144)

  # Installed is not the same as working: a container can carry rsvg and
  # grImport2 while librsvg cannot actually turn the file into a Picture,
  # and the loader then correctly falls back to a raster. Probe the real
  # conversion before asserting a vector came out of it, or this fails on
  # the environment rather than on the package. Seen on R-hub's nold
  # container, where all three packages are installed.
  # A Picture object is not enough: it has to carry drawable content.
  # librsvg on R-hub's nold container returns an empty one, and the
  # loader then correctly falls back to a raster.
  can_vector <- tryCatch(suppressWarnings({
    cairo <- tempfile(fileext = ".svg")
    rsvg::rsvg_svg(f, cairo)
    pic <- grImport2::readPicture(cairo)
    inherits(pic, "Picture") && length(pic@content) > 0
  }), error = function(e) FALSE)
  skip_if(!isTRUE(can_vector),
          "rsvg/grImport2 cannot turn an SVG into a drawable Picture here")

  kids <- grid::makeContent(
    latex_grob(sprintf("\\includegraphics[width=1in]{%s}", f),
               input_mode = "math"))$children
  expect_length(kids, 1L)
  # grImport2 turns the file into grid drawing primitives; a raster would
  # be a rastergrob, and would not scale with the device. How deep the
  # primitives sit depends on the SVG, so walk the whole subtree rather
  # than assuming a fixed nesting, and report the tree when it is absent.
  expect_false(inherits(kids[[1]], "rastergrob"))
  grob_classes <- function(g) {
    c(class(g)[1], unlist(lapply(g$children, grob_classes), use.names = FALSE))
  }
  prims <- grob_classes(kids[[1]])
  expect_true(any(grepl("^pic", prims)),
              info = paste("grob tree:", paste(prims, collapse = " / ")))

  # The wrapper must carry a plain viewport, or .md_shift_grob() -- which
  # realigns a block by nudging vp$x -- would hit pictureGrob's vpStack,
  # whose x is NULL.
  expect_false(is.null(kids[[1]]$vp))
  expect_false(is.null(kids[[1]]$vp$x))
})

test_that("an SVG falls back to a raster when there is no picture reader", {
  skip_if_not_installed("svglite")
  skip_if_not_installed("rsvg")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_svg(2, 1)

  # This branch is for machines that have rsvg but not grImport2. Every
  # other SVG test skips without grImport2, so with it installed the
  # fallback would never execute and would rot unnoticed -- mock the
  # picture reader away to force it.
  # `.package` is given explicitly: without it testthat infers the target
  # from a pkgload context, so this block errored with "No packages
  # loaded with pkgload" under a plain test_dir() / test_file() run --
  # green under devtools::test() and R CMD check, which both supply one.
  testthat::local_mocked_bindings(.image_picture = function(...) NULL,
                                  .package = "gridmicrotex")
  g <- .image_grob(f, 144, 72)
  expect_s3_class(g, "rastergrob")
  # Rasterised at the device's resolution rather than a size baked in when
  # the file was written -- the one thing a vector source makes possible.
  expect_gt(nrow(g$raster), 1L)
  expect_gt(ncol(g$raster), 1L)

  # And with neither reader there is nothing to draw, so the resolver's
  # fallback text is what the user sees.
  testthat::local_mocked_bindings(.image_raster = function(...) NULL,
                                  .package = "gridmicrotex")
  expect_null(.image_grob(tempfile(fileext = ".png"), 72, 72))
})

test_that("an image that cannot be drawn warns once and shows its name", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  drawn <- function(tex) {
    d <- suppressWarnings(
      latex_grob(tex, input_mode = "math")$layout_df)
    paste(d$text[d$type == "text" & !is.na(d$text)], collapse = "")
  }
  expect_warning(latex_dims("\\includegraphics{no-such-file.png}",
                            input_mode = "math"), "file not found")
  expect_equal(drawn("\\includegraphics{no-such-file.png}"), "no-such-file.png")

  # PDF and EPS are the formats real LaTeX points at, so the message says
  # what to do instead rather than only that it failed.
  p <- tempfile(fileext = ".pdf")
  grDevices::pdf(p, width = 2, height = 1); grid::grid.rect(); grDevices::dev.off()
  expect_warning(latex_dims(sprintf("\\includegraphics{%s}", p),
                            input_mode = "math"), "save the figure as SVG")

  t <- tempfile(fileext = ".tiff"); writeBin(as.raw(1:20), t)
  expect_warning(latex_dims(sprintf("\\includegraphics{%s}", t),
                            input_mode = "math"), "not a supported image format")
})

test_that("one warning per file, however many times the layout is measured", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # .md_measure() calls latex_dims() up to seven times for a single render
  # and editDetails() re-runs the pipeline, so deduplicating per parse
  # would not deduplicate at all.
  f <- tempfile(fileext = ".png")   # never created
  seen <- character(0)
  withCallingHandlers(
    for (i in 1:5) latex_dims(sprintf("\\includegraphics{%s}", f),
                              input_mode = "math"),
    warning = function(w) {
      seen <<- c(seen, conditionMessage(w)); invokeRestart("muffleWarning")
    })
  expect_length(seen, 1L)
})

test_that("effective resolution is reported only when the author chose a size", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  warns <- function(expr) {
    ws <- character(0)
    withCallingHandlers(expr, warning = function(w) {
      ws <<- c(ws, conditionMessage(w)); invokeRestart("muffleWarning") })
    ws
  }
  small <- mk_png(96, 64)
  # Asked for 3in from 96px: 32 dpi, and the fix is more pixels.
  expect_match(warns(latex_dims(sprintf("\\includegraphics[width=3in]{%s}", small),
                                input_mode = "math"))[1], "dpi")
  # At intrinsic size the answer is 96 dpi by construction, so there is
  # nothing to act on and nothing to say.
  expect_length(warns(latex_dims(sprintf("\\includegraphics{%s}", mk_png(96, 64)),
                                 input_mode = "math")), 0L)
  # A vector source has no effective resolution at all.
  skip_if_not_installed("svglite"); skip_if_not_installed("rsvg")
  expect_length(warns(latex_dims(sprintf("\\includegraphics[width=9in]{%s}", mk_svg()),
                                 input_mode = "math")), 0L)
})

test_that("an \\includegraphics inside a macro definition still resolves", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # The resolver sees this one after all: the command sits literally in the
  # \newcommand's replacement text, which is just characters in the string.
  # So it becomes a real image, not a fallback.
  f <- mk_png()
  d <- latex_grob(sprintf("\\newcommand{\\fig}{\\includegraphics[width=1in]{%s}}x\\fig y", f),
                  input_mode = "math")$layout_df
  expect_true("image" %in% d$type)
})

test_that("an \\includegraphics the resolver declines is still not silent", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # Malformed input -- no braces, or unbalanced ones -- is the case the R
  # scanner deliberately leaves alone rather than guessing at. It then
  # reaches MicroTeX, whose vendored stub returns nullptr and draws
  # nothing at all. The C++ override draws the argument instead, so a
  # broken \includegraphics is visible rather than an invisible gap.
  drawn <- function(tex) {
    d <- suppressWarnings(latex_grob(tex, input_mode = "math")$layout_df)
    paste(d$text[d$type == "text" & !is.na(d$text)], collapse = "")
  }
  expect_equal(drawn("x\\includegraphics{unbalanced"), "unbalanced")
  expect_equal(drawn("x\\includegraphics y"), "y")

  # And the resolver really did leave them: these are not rewritten forms.
  for (tex in c("x\\includegraphics{unbalanced", "x\\includegraphics y")) {
    expect_match(gridmicrotex:::.resolve_graphics(tex, 20, 0),
                 "includegraphics", fixed = TRUE)
  }
})

test_that("the layout cache notices a file that changed on disk", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- tempfile(fileext = ".png")
  ragg::agg_png(f, width = 300, height = 200); grid::grid.rect(); dev.off()
  latex_cache_clear()
  w1 <- wid(sprintf("\\includegraphics{%s}", f))
  before <- latex_cache_info()$hits
  wid(sprintf("\\includegraphics{%s}", f))
  expect_equal(latex_cache_info()$hits - before, 1L)   # same file: a hit

  # The cache key is the tex string, which holds no file metadata of its
  # own -- so mtime and size ride along in the encoded reference. Without
  # them an edited figure would keep its old layout forever.
  Sys.sleep(1.1)
  ragg::agg_png(f, width = 600, height = 200); grid::grid.rect(); dev.off()
  expect_false(identical(wid(sprintf("\\includegraphics{%s}", f)), w1))
})

test_that("markdown draws a real image, and keeps alt text when it cannot", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_png()
  tex <- gridmicrotex:::.md_to_tex(sprintf("a ![ALT](%s) b", f))
  expect_match(tex, "includegraphics", fixed = TRUE)

  # A web URL or a missing file is not an error in markdown -- it keeps the
  # alt text and stays silent, which is what it has always done.
  expect_match(gridmicrotex:::.md_to_tex("a ![ALT](https://x/y.png) b"), "ALT")
  expect_no_warning(markdown_grob("a ![ALT](https://x/y.png) b"))
  expect_no_warning(markdown_grob("a ![ALT](nope.png) b"))

  # <img> reaches the same place, and HTML sizes it in pixels.
  expect_match(gridmicrotex:::.md_to_tex(sprintf("a <img src='%s' width='48'> b", f)),
               "width=48px", fixed = TRUE)
})

test_that("keepaspectratio fits inside the box instead of stretching it", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_png(300, 200)                     # 3:2, intrinsic 225 x 150 bp
  # Without the key, both lengths are obeyed and the picture distorts --
  # that is what LaTeX does too.
  expect_equal(wid(sprintf("\\includegraphics[width=3in,height=3in]{%s}", f)), 216)
  expect_equal(hei(sprintf("\\includegraphics[width=3in,height=3in]{%s}", f)), 216)
  # With it, the smaller scale factor wins and the whole picture fits.
  ka <- sprintf("\\includegraphics[width=3in,height=3in,keepaspectratio]{%s}", f)
  expect_equal(wid(ka), 216)
  expect_equal(hei(ka), 144)
  # It is a graphicx boolean, so `=false` is the plain behaviour again.
  expect_equal(hei(sprintf(
    "\\includegraphics[width=3in,height=3in,keepaspectratio=false]{%s}", f)), 216)
})

test_that("scale multiplies width and height rather than replacing them", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_png(300, 200)
  # graphicx applies scale to whatever width/height settled on.
  expect_equal(wid(sprintf("\\includegraphics[width=4in,scale=0.5]{%s}", f)), 144)
  expect_equal(hei(sprintf("\\includegraphics[width=4in,scale=0.5]{%s}", f)), 96)
})

test_that("a path is found the way graphicx finds one", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  dir <- tempfile("gfx"); dir.create(file.path(dir, "sub"), recursive = TRUE)
  file.copy(mk_png(300, 200), file.path(dir, "sub", "fig.png"))
  old <- setwd(dir); on.exit(setwd(old), add = TRUE)

  # No extension: the idiomatic LaTeX form, and the one that used to warn.
  expect_match(gridmicrotex:::.resolve_graphics("\\includegraphics{sub/fig}", 20, 0),
               "gmgraphics", fixed = TRUE)
  # \graphicspath supplies the directory, and is consumed rather than
  # typeset -- MicroTeX has no such command and would draw its argument.
  out <- gridmicrotex:::.resolve_graphics(
    "\\graphicspath{{sub/}}a\\includegraphics{fig}b", 20, 0)
  expect_match(out, "gmgraphics", fixed = TRUE)
  expect_false(grepl("graphicspath", out, fixed = TRUE))
  expect_false(grepl("sub/", out, fixed = TRUE))
  # ...including when there is no image at all to trigger the scan.
  expect_equal(gridmicrotex:::.resolve_graphics("\\graphicspath{{sub/}}text", 20, 0),
               "text")
})

test_that("the starred and two-argument spellings are the same command", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_png(300, 200)
  # Unmatched, these rendered the option list and the raw path as text.
  expect_equal(wid(sprintf("\\includegraphics*[width=3in]{%s}", f)), 216)
  # The older `[llx,lly][urx,ury]` spelling: two groups, no recognised key,
  # so the picture comes out at its own size rather than as literal text.
  expect_equal(wid(sprintf("\\includegraphics[0,0][10,10]{%s}", f)), 225)
  expect_equal(wid(sprintf("\\includegraphics[0,0][width=3in]{%s}", f)), 216)
})

test_that("an option that changes the picture warns rather than being dropped", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  warns <- function(expr) {
    ws <- character(0)
    withCallingHandlers(expr, warning = function(w) {
      ws <<- c(ws, conditionMessage(w)); invokeRestart("muffleWarning") })
    ws
  }
  # A cropped figure drawn whole is wrong in a way nothing in the output
  # hints at, so it has to say so. (`angle` is not in this set: it is a real
  # rotation now -- see the rotation test above.)
  w <- warns(latex_dims(sprintf("\\includegraphics[origin=c,width=1in]{%s}",
                                mk_png(300, 200)), input_mode = "math"))
  expect_match(paste(w, collapse = " "), "origin")
  w <- warns(latex_dims(sprintf("\\includegraphics[trim=1 2 3 4,clip,width=1in]{%s}",
                                mk_png(300, 200)), input_mode = "math"))
  expect_match(paste(w, collapse = " "), "trim")
  expect_match(paste(w, collapse = " "), "clip")
})

test_that("\\textwidth without max_width keeps the figure at its own size", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_png(300, 200)
  tex <- sprintf("\\includegraphics[width=\\textwidth]{%s}", f)
  ws <- character(0)
  w <- withCallingHandlers(
    suppressMessages(as.numeric(latex_dims(tex, input_mode = "math")$width)),
    warning = function(cnd) {
      ws <<- c(ws, conditionMessage(cnd)); invokeRestart("muffleWarning") })
  # Dropping the figure entirely was the old behaviour; the intrinsic size
  # is nearer to what the document meant, and the warning says so.
  expect_equal(w, 300 * 72 / 96)
  expect_match(paste(ws, collapse = " "), "textwidth")
  # ...and it is not also reported as low-resolution: at intrinsic size the
  # answer is 96 dpi by construction.
  expect_false(any(grepl("dpi", ws)))
})

test_that("an SVG no reader can draw is refused at parse time, not at draw time", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # Well-formed XML with a readable width/height, but not an SVG. This is
  # the shape of every "the header parsed but nothing can draw it" case,
  # including a user with no rsvg installed. Sizing a box for it would
  # reserve space and then draw nothing into it -- a silent hole, which is
  # the one outcome this whole path exists to prevent.
  dir <- tempfile("gfx"); dir.create(dir)
  f <- file.path(dir, "notanimage.svg")
  writeLines("<notsvg width='72pt' height='36pt'><x/></notsvg>", f)
  expect_null(gridmicrotex:::.image_dims(f))
  expect_false(gridmicrotex:::.image_drawable(f))

  d <- suppressWarnings(latex_grob(sprintf("\\includegraphics{%s}", f),
                                   input_mode = "math")$layout_df)
  expect_false("image" %in% d$type)
  expect_match(paste(d$text[!is.na(d$text)], collapse = ""), "notanimage.svg",
               fixed = TRUE)

  # Markdown keeps its alt text for the same reason.
  expect_match(gridmicrotex:::.md_to_tex(sprintf("a ![ALT](%s) b", f)), "ALT")
})

test_that("a rotated image is drawn rotated, not dropped", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_png(300, 200)
  rec <- function(tex) {
    d <- suppressWarnings(latex_tree(tex, input_mode = "math")$records)
    d[d$type == "image", ]
  }
  # \rotatebox used to delete the figure outright: no record, no warning,
  # nothing on the page. Everything else survives a rotation -- rects become
  # paths, rules become lines -- so the image was the sole exception.
  r <- rec(sprintf("\\rotatebox{30}{\\includegraphics[width=1in]{%s}}", f))
  expect_equal(nrow(r), 1L)
  expect_equal(r$rotation, -30, tolerance = 1e-3)   # y-down, so negated

  # `angle=` is the same thing spelled the graphicx way, and is routed to
  # \rotatebox rather than warned about.
  expect_equal(rec(sprintf("\\includegraphics[angle=30,width=1in]{%s}", f))$rotation,
               -30, tolerance = 1e-3)
  expect_equal(rec(sprintf("\\includegraphics[width=1in]{%s}", f))$rotation, 0)
  expect_match(gridmicrotex:::.resolve_graphics(
    sprintf("\\includegraphics[angle=45]{%s}", f), 20, 0), "rotatebox", fixed = TRUE)
  # A whole turn is not a rotation.
  expect_false(grepl("rotatebox", gridmicrotex:::.resolve_graphics(
    sprintf("\\includegraphics[angle=360]{%s}", f), 20, 0), fixed = TRUE))

  # The surrounding box grows to the rotated bounds, as in LaTeX.
  flat <- suppressWarnings(latex_dims(sprintf("\\includegraphics[width=1in]{%s}", f),
                                      input_mode = "math"))
  turned <- suppressWarnings(latex_dims(sprintf("\\includegraphics[angle=30,width=1in]{%s}", f),
                                        input_mode = "math"))
  expect_gt(as.numeric(turned$height), as.numeric(flat$height))

  # And it reaches the page as a grob in a rotated viewport.
  g <- suppressWarnings(latex_grob(sprintf("\\includegraphics[angle=30,width=1in]{%s}", f),
                                   input_mode = "math"))
  kid <- grid::makeContent(g)$children[[1]]
  expect_equal(kid$vp$angle, 30, tolerance = 1e-3)
})

test_that("a size that cannot be drawn says why instead of vanishing quietly", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  warns <- function(tex) {
    ws <- character(0)
    withCallingHandlers(latex_dims(tex, input_mode = "math"),
      warning = function(w) { ws <<- c(ws, conditionMessage(w)); invokeRestart("muffleWarning") })
    paste(ws, collapse = " ")
  }
  G <- function(o) sprintf("\\includegraphics%s{%s}", o, mk_png(300, 200))
  # An unreadable length was the dangerous one: the arithmetic ignores it,
  # so a typo came out at the file's own size with nothing said.
  expect_match(warns(G("[width=abc]")), "Cannot read")
  expect_match(warns(G("[width=3 inches]")), "Cannot read")
  expect_match(warns(G("[width=0]")), "nothing to draw")
  expect_match(warns(G("[width=-1in]")), "nothing to draw")
  # 1e-5bp is positive but writes as "0.0000" at the four decimal places the
  # reference carries, so it reserved a box of nothing.
  expect_match(warns(G("[width=0.00001bp]")), "nothing to draw")
  expect_match(warns(G("[scale=-2]")), "positive number")

  # A width no pixel count can express must degrade, not error: the dpi
  # advice used sprintf("%d") on it.
  expect_no_error(suppressWarnings(latex_dims(G("[width=1e7in]"), input_mode = "math")))
})

test_that("an unreadable file says which of the two things went wrong", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  skip_if_not_installed("png")
  why <- function(path) {
    ws <- character(0)
    withCallingHandlers(
      gridmicrotex:::.resolve_graphics(sprintf("\\includegraphics{%s}", path), 20, 0),
      warning = function(w) { ws <<- c(ws, conditionMessage(w)); invokeRestart("muffleWarning") })
    paste(ws, collapse = " ")
  }
  dir <- tempfile("gfx"); dir.create(dir)
  # "no dimensions" has two causes and they need different answers. Blaming
  # a missing package for a truncated file sends the reader to install
  # something they already have.
  empty <- file.path(dir, "empty.png"); file.create(empty)
  expect_match(why(empty), "could not read it")
  expect_false(grepl("is needed", why(empty)))
  expect_match(why(dir), "directory")
  noext <- file.path(dir, "noext"); writeLines("x", noext)
  expect_match(why(noext), "no extension")
})

test_that("latex_cache_clear() gives back the decoded images too", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  suppressWarnings(latex_dims(sprintf("\\includegraphics{%s}", mk_png(300, 200)),
                              input_mode = "math"))
  expect_gt(length(ls(gridmicrotex:::.image_cache)), 0L)
  # Rasters and pictures are far larger than a layout and have no size
  # limit of their own, so this is the only way to release them.
  latex_cache_clear()
  expect_equal(length(ls(gridmicrotex:::.image_cache)), 0L)
})

test_that("a file that stops being readable after measuring is reported", {
  skip_if_not_installed("ragg"); skip_if_not_installed("png")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  f <- mk_png(300, 200)
  g <- latex_grob(sprintf("\\includegraphics[width=1in]{%s}", f),
                  input_mode = "math")
  unlink(f)
  # The box is already in the layout and cannot be given back, so the only
  # honest thing left is to say the gap is there.
  expect_warning(grid::makeContent(g), "was readable when")
})

#' Create a grid grob from a LaTeX expression
#'
#' Parses a LaTeX math expression and returns a grid grob object
#' that renders the formula using native grid graphics primitives.
#' The grob supports standard grid queries such as \code{grobWidth()},
#' \code{grobHeight()}, \code{grobX()}, and \code{grobY()}.
#'
#' @param tex Character string of LaTeX math code.
#' @param x,y Position in grid coordinates.
#' @param default.units Units for x, y if given as numeric.
#' @param hjust,vjust Horizontal/vertical justification. Accepts the
#'   usual numeric values in `[0, 1]`. As a convenience, `hjust` also
#'   accepts the strings `"left"`/`"bbleft"`, `"center"`/`"centre"`/
#'   `"middle"`/`"bbcentre"`, and `"right"`/`"bbright"`; `vjust` accepts
#'   `"bottom"`, `"center"`/`"centre"`/`"middle"`, `"top"`, and
#'   `"baseline"`. `"baseline"` aligns the formula's math baseline with
#'   the anchor point — handy for placing a formula in flowing text.
#' @param rot Rotation angle in degrees, counter-clockwise (default: 0).
#'   Matches the \code{rot} parameter of \code{\link[grid]{textGrob}}.
#' @param math_font Name of the math font to use (e.g., \code{"stix"}).
#'   Use \code{""} (default) for Lete Sans Math, which pairs with R's
#'   default sans-serif text font.
#'   See \code{\link{available_math_fonts}} for loaded fonts.
#' @param max_width Numeric maximum width in big points for automatic
#'   line wrapping.  Use \code{0} (default) for no wrapping.
#' @param tex_style Character: TeX style override. One of \code{""}
#'   (default; let the parser decide), \code{"display"}, \code{"text"},
#'   \code{"script"}, or \code{"scriptscript"}. See
#'   \code{\link{latex_grob}} for the semantics of each value.
#' @param input_mode How \code{tex} is interpreted before being parsed.
#'   \code{"mixed"} (default) wraps the input in \code{\\text{...}} so the
#'   string reads as ordinary text and \code{$...$} (or \code{\\(...\\)})
#'   opens math mode, matching document-level LaTeX semantics. Useful for
#'   labels that arrive from external sources mixing prose and math without
#'   explicit \code{\\text{}} markers. \code{"math"} is the classic
#'   MicroTeX behaviour --- the whole string is treated as math, so unwrapped
#'   prose renders as spaced math italics. The default can be changed globally via
#'   \code{\link{latex_options}(input_mode = "math")}. See \code{\link{latex_wrap}}
#'  for details on the wrapping process.
#' @param render_mode Character string: \code{"typeface"} (default) renders
#'   glyphs as native text using the math font, producing
#'   selectable/accessible text in PDF and SVG output.
#'   Bundled math fonts and any registered via \code{\link{load_math_font}}
#'   are read directly from their OTF files --- no system-wide font
#'   install is required.
#'   Falls back to path mode automatically on devices that lack the
#'   R \eqn{\geq} 4.3 glyph engine (e.g., the base \code{pdf()} device).
#'   For selectable PDF output, prefer \code{\link[grDevices]{cairo_pdf}}.
#'   \code{"path"} renders math symbols as filled vector paths (works on
#'   all devices but text is not selectable in PDF/SVG).
#' @param justify Logical. When \code{TRUE}, wrapped text is stretched at
#'   its interword spaces so every line but the last fills
#'   \code{max_width} exactly. Requires \code{max_width}: it acts on the
#'   lines the wrapper produces, so it does nothing on its own.
#'   \code{FALSE} (default) leaves the right edge ragged, matching R's own
#'   text drawing. In a narrow column, expect wide word spaces; mark the
#'   words that may break with \code{\\-} to tighten them.
#' @param line_break How lines are chosen when wrapping.
#'   \code{"greedy"} (default) fills each line as far as it will go and
#'   never reconsiders. \code{"optimal"} chooses the breaks together so
#'   the paragraph as a whole reads best, in the spirit of Knuth-Plass:
#'   pulling one word down early can improve every later line, which a
#'   greedy pass cannot see. Requires \code{max_width}, and costs a
#'   little more layout time.
#' @param debug Logical; if \code{TRUE}, draws diagnostic overlays on the
#'   grob --- the full bounding box (dashed gray), the baseline (solid
#'   red), the depth line (dashed gray), and a small dot at each
#'   MicroTeX draw record's origin. Useful for checking positioning and
#'   diagnosing vertical alignment.
#' @param name Optional grob name.
#' @param gp Graphical parameters (see \code{\link[grid]{gpar}}).
#'   Common entries: \code{col} (formula foreground), \code{fontfamily}
#'   (text font), \code{fontsize} / \code{cex}
#'   (formula size), and \code{lineheight} (multi-line spacing). See
#'   \code{\link{latex_grob}} for how each of these flows through
#'   MicroTeX.
#'
#' @details
#' ## Controlling TeX style with `tex_style`
#'
#' `tex_style` selects the size-and-spacing regime MicroTeX applies to
#' the whole expression. It changes the *style* (display vs. text), not
#' the font size --- size is always set via `gp$fontsize` / `gp$cex`;
#' style-dependent shrinking (for `"script"` and `"scriptscript"`) is
#' applied on top of that size.
#'
#' - `""` (default): let the parser choose based on the delimiters in
#'   `tex`. Inline delimiters (single `$`, or `\(...\)`) produce
#'   `"text"` style; display delimiters (double `$$`, or `\[...\]`)
#'   produce `"display"` style. If the string has no delimiters,
#'   MicroTeX defaults to `"text"` style.
#' - `"display"`: force display style. Large operators (`\sum`,
#'   `\int`, `\prod`) render at their full size, limits are placed
#'   above/below rather than as subscripts/superscripts, and fractions
#'   use full-size numerators and denominators. Useful when you want a
#'   display-style equation inline in a label, legend, or
#'   `element_latex()` title.
#' - `"text"`: force text (inline) style. Big operators shrink to their
#'   inline size and limits attach as scripts. The right choice for
#'   formulas embedded in a line of prose.
#' - `"script"`: force script style --- the size normally used for
#'   first-level subscripts and superscripts. Produces a smaller,
#'   tighter layout; mainly useful for callouts or sub-labels where a
#'   compact equation is wanted.
#' - `"scriptscript"`: force scriptscript style --- the smallest style,
#'   used by TeX for doubly-nested scripts. Rarely needed on its own;
#'   primarily for very dense annotations.
#'
#' `tex_style` applies to the entire expression. To override the style
#' of a sub-expression from within `tex`, use the inline TeX commands
#' `\displaystyle`, `\textstyle`, `\scriptstyle`, or
#' `\scriptscriptstyle`.
#'
#' ## Graphical parameters (`gp`)
#'
#' - `col`: default foreground color for the formula. Individual
#'   elements can still be overridden with an inline `\textcolor`
#'   command in the LaTeX string.
#' - `fontfamily`: controls the font of text inside `\text` and `\mbox`
#'   blocks. For example, `gpar(fontfamily = "serif")` renders `\text`
#'   content in R's serif family. Any font available to R's graphics
#'   system works --- base families (`"sans"`, `"serif"`, `"mono"`) as
#'   well as fonts registered via \pkg{showtext} or \pkg{systemfonts}.
#'   Math symbols always use the selected math font (see `math_font`).
#'   Bold/italic text is controlled from within the LaTeX source
#'   (`\textbf{}`, `\textit{}`, `\bf`, ...), not via `gp$fontface` ---
#'   MicroTeX needs the style at layout time to size each run correctly,
#'   so a `gpar()`-level face is not consulted.
#'
#'   `fontfamily` *also* drives MicroTeX's layout metrics for non-math
#'   text: the matching system font is resolved via \pkg{systemfonts},
#'   a minimal metrics file is generated on first use and cached under
#'   `tools::R_user_dir("gridmicrotex", "cache")`, so MicroTeX's
#'   spacing of `\text` blocks stays in sync with what \pkg{grid}
#'   actually draws. When `fontfamily` is unset, the R default
#'   (`"sans"`) is used. No manual font loading is required for text
#'   fonts; [load_math_font()] remains only for adding custom **math**
#'   fonts.
#' - `fontsize` / `cex`: formula size is `fontsize * cex` big points
#'   (default 20 * 1). Both math and text scale together. The effective
#'   size is baked into the parsed layout, so downstream viewports that
#'   inherit `cex` will not re-scale the grob (matching `textGrob`
#'   semantics when `gp` is set explicitly).
#' - `lineheight`: controls multi-line spacing (default 1.2). The
#'   inter-line gap is `(lineheight - 1) * fontsize` big points.
#'
#' ## LaTeX document-level wrappers
#'
#' The parser accepts raw output from \code{print.xtable()},
#' \code{knitr::kable()}, and similar functions that emit complete
#' `tabular` LaTeX. The following document-level constructs are
#' recognized and rewritten silently before the input reaches MicroTeX:
#'
#' **Removed (no visual effect):**
#' * `%`-to-end-of-line comments (escaped `\%` is preserved)
#' * preamble: `\documentclass[...]{...}`, `\usepackage[...]{...}`,
#'   `\begin{document}` / `\end{document}`
#' * title metadata: `\maketitle`, `\title{...}`, `\author{...}`
#' * cross-reference labels: `\label{...}`
#' * float wrappers: `\begin{table}` / `\end{table}`, `\begin{figure}` /
#'   `\end{figure}` (and starred variants)
#' * layout scopes: `\centering`, `\raggedright`, `\raggedleft`,
#'   `\flushleft`, `\flushright`
#'
#' **Rewritten:**
#' * booktabs rules: `\toprule`, `\midrule`, `\bottomrule`, `\cmidrule`
#'   are mapped to `\hline`. The optional column-range and trim
#'   arguments of `\cmidrule` are discarded (MicroTeX has no concept of
#'   partial-column rules).
#' * `\caption[short]{X}` is extracted as `\text{X}\\` at its source
#'   position, so a caption written after `\includegraphics` renders below
#'   the figure and one written before a `tabular` renders above the table.
#'   Full LaTeX instead positions the caption by float type regardless of
#'   source order, and numbers it from a counter; there is no counter here.
#'   Wrap the figure and its caption in `\begin{array}{c}...\end{array}` to
#'   centre them on each other (`\centering` is dropped --- a grob has no
#'   page to centre against).
#' * `\graphicspath{{dir/}}` and `\DeclareGraphicsExtensions{...}` are
#'   consumed rather than typeset; the former's directories are searched.
#'
#' **Images:**
#' * `\includegraphics[opts]{file}` draws a PNG, JPEG or SVG inline. The
#'   starred form is accepted and behaves identically. `width`, `height`
#'   and `scale` take any LaTeX length (`\textwidth` resolves against
#'   `max_width`, and without one falls back to the file's own size with a
#'   warning); `scale` multiplies whatever `width`/`height` settled on, and
#'   `keepaspectratio` fits inside them instead of stretching to fill.
#'   `angle` rotates the figure and grows the surrounding box to the rotated
#'   bounds, as `\rotatebox` does. `origin`, `trim`, `clip` and `viewport`
#'   are parsed but not applied, and warn once so the difference is not
#'   silent; so does a length that cannot be read, or one that sizes the
#'   figure to nothing.
#' * The extension may be omitted, as in LaTeX: `{plots/fig}` finds
#'   `plots/fig.svg`, then `.png`, `.jpg`, `.jpeg`.
#' * An SVG is drawn as real vector and stays sharp at any output
#'   resolution; a bitmap does not, and warns when it would be shown below
#'   150 dpi. PDF and EPS are not supported --- save the figure as SVG
#'   instead. A file that cannot be read --- missing, unsupported, or an
#'   SVG with no `rsvg` installed --- warns and draws its name rather than
#'   disappearing.
#'
#' Anything not in this list is passed to MicroTeX unchanged. An unknown
#' command is not an error: MicroTeX typesets its name in red, which
#' makes unsupported markup easy to spot in the output.
#'
#' ## Parallelism
#' The MicroTeX engine keeps mutable C++ state for font caching and text
#' measurement. Rendering is safe single-threaded and under separate-process
#' backends such as \code{future::plan(multisession)}. It is \emph{not} safe
#' under forked backends (\code{parallel::mclapply()},
#' \code{future::plan(multicore)}) on Unix, because forked workers share that
#' state without synchronisation. Use a socket/multisession backend instead.
#'
#' @return A \code{grid} grob of class \code{"latexgrob"}.
#' @seealso \code{\link{grid.latex}}, \code{\link{latex_dims}},
#'   \code{\link{geom_latex}}, \code{\link{available_math_fonts}},
#' \code{\link{latex_wrap}}, \code{\link{latex_options}}
#' @export
#'
#' @examples
#' \donttest{
#'   g <- latex_grob(r"($\fcolorbox{red}{yellow}{\frac{a}{b}}$)",
#'                   x = grid::unit(0.3, "npc"),
#'                   y = grid::unit(0.3, "npc"),
#'                   gp = grid::gpar(fontsize = 30))
#'   grid::grid.draw(g)
#'   # Red formula
#'   grid::grid.draw(latex_grob("$x^{2}$",
#'                              x = grid::unit(0.3, "npc"),
#'                              y = grid::unit(0.8, "npc"),
#'                              gp = grid::gpar(col = "red")))
#'
#'                              # Rotated formula
#'   grid::grid.draw(latex_grob(r"($\colorbox{BurntOrange}{x^{2}} + y^{2}$)",
#'                              x = grid::unit(0.6, "npc"),
#'                              y = grid::unit(0.3, "npc"),
#'                              gp = grid::gpar(fontsize = 24),
#'                              rot = 45))
#'
#'   grid.latex(r"($\textcolor{red}{x^{2}} + y^{2} = z^{2}$)",
#'              x = grid::unit(0.6, "npc"),
#'              y = grid::unit(0.8, "npc"),)
#' }
latex_grob <- function(tex,
                       x = grid::unit(0.5, "npc"),
                       y = grid::unit(0.5, "npc"),
                       default.units = "npc",
                       hjust = 0.5,
                       vjust = 0.5,
                       rot = 0,
                       math_font = "",
                       max_width = 0,
                       tex_style = "",
                       input_mode = c("mixed", "math"),
                       render_mode = c("typeface", "path"),
                       justify = FALSE,
                       line_break = c("greedy", "optimal"),
                       debug = FALSE,
                       name = NULL,
                       gp = grid::gpar()) {

  # Whether typeface was actually asked for, rather than inherited as the
  # default. Captured before .apply_opts(), which assigns the option value
  # into `render_mode` and would make missing() FALSE from then on.
  render_mode_explicit <- !missing(render_mode) || !is.null(.opt("render_mode"))

  .apply_opts("math_font", "render_mode", "tex_style", "input_mode",
              "justify", "line_break")
  render_mode <- match.arg(render_mode)
  input_mode <- match.arg(input_mode)
  line_break <- match.arg(line_break)
  .check_justify(justify)

  parsed <- .parse_from_gp(
    tex = tex, gp = gp, math_font = math_font, max_width = max_width,
    tex_style = tex_style, render_mode = render_mode,
    input_mode = input_mode,
    with_path_fallback = TRUE, justify = justify, line_break = line_break
  )

  # Convert numeric x/y to units
  if (is.numeric(x)) x <- grid::unit(x, default.units)
  if (is.numeric(y)) y <- grid::unit(y, default.units)

  layout <- parsed$layout
  bbox_w <- attr(layout, "bbox_width")
  bbox_h <- attr(layout, "bbox_height")
  bbox_d <- attr(layout, "bbox_depth")
  # Baseline as bigpts from the bottom edge of the bounding box —
  # exposed read-only on the gTree for advanced grob-to-grob alignment.
  bbox_bl_bp <- bbox_h * (1 - attr(layout, "bbox_baseline"))
  is_split <- isTRUE(attr(layout, "bbox_is_split"))

  just <- .resolve_just(hjust, vjust, bbox_bl_bp = bbox_bl_bp, bbox_h = bbox_h)
  marks <- .extract_marks(layout, bbox_h = bbox_h)

  grid::gTree(
    tex = parsed$tex,
    layout_df = layout,
    bbox_w = bbox_w,
    bbox_h = bbox_h,
    bbox_d = bbox_d,
    bbox_bl_bp = bbox_bl_bp,
    is_split = is_split,
    marks = marks,
    fontsize = parsed$fontsize,
    hjust = just$hjust,
    vjust = just$vjust,
    hjust_input = hjust,
    vjust_input = vjust,
    # Input parameters kept on the grob so editGrob() can re-parse when
    # any of them change. Resolved/baked values live in the parsed fields
    # above; these fields hold the user-facing inputs.
    math_font = math_font,
    max_width = max_width,
    tex_style = tex_style,
    input_mode = input_mode,
    justify = justify,
    line_break = line_break,
    text_gp = parsed$text_gp,
    render_mode = parsed$render_mode,
    render_mode_explicit = render_mode_explicit,
    path_layout_df = parsed$path_layout,
    debug = isTRUE(debug),
    cl = "latexgrob",
    name = name,
    gp = parsed$gp,
    vp = grid::viewport(
      x = x, y = y,
      width = grid::unit(bbox_w, "bigpts"),
      height = grid::unit(bbox_h, "bigpts"),
      just = c(just$hjust, just$vjust),
      angle = rot
    )
  )
}

# Extract \mark{name} anchors from a parsed layout. MicroTeX's y axis is
# top-down; flip to grid's bottom-up so the values can be added directly
# to a bigpts-from-bbox-bottom-left reference like the children grobs use.
# Returns a data.frame with columns (name, x, y) in bigpts, or NULL when
# no marks were emitted.
.extract_marks <- function(layout, bbox_h) {
  m <- attr(layout, "marks")
  if (is.null(m) || nrow(m) == 0L) return(NULL)
  data.frame(
    name = as.character(m$name),
    x    = as.numeric(m$x),
    y    = bbox_h - as.numeric(m$y),
    stringsAsFactors = FALSE
  )
}

#' Look up a named anchor inside a LaTeX grob
#'
#' Resolves a \code{\\mark\{name\}} that was placed inside the LaTeX
#' source to a pair of \pkg{grid} units in the grob's parent viewport.
#' The returned units already account for the grob's viewport position
#' and \code{hjust}/\code{vjust}, so you can pass them directly to grid
#' drawing functions to anchor other graphics on parts of the formula.
#'
#' @param grob A \code{latexgrob} returned by \code{\link{latex_grob}}.
#' @param name The mark name (the argument to \code{\\mark\{...\}}).
#' @return A list with elements \code{x} and \code{y}, each a
#'   \code{\link[grid]{unit}}. Mark coordinates are evaluated in the
#'   grob's parent viewport. They assume an unrotated grob: with
#'   \code{rot != 0} the returned position does not account for the
#'   rotation.
#' @seealso \code{\link{latex_grob}}
#' @export
#'
#' @examples
#' \donttest{
#'   g <- latex_grob(r"($a\mark{eq}^2 = b + c^2$)",
#'                   x = grid::unit(0.5, "npc"),
#'                   y = grid::unit(0.5, "npc"))
#'   grid::grid.newpage(); grid::grid.draw(g)
#'   mk <- grobMark(g, "eq")
#'   grid::grid.points(mk$x, mk$y, pch = 19,
#'                     gp = grid::gpar(col = "red"))
#' }
grobMark <- function(grob, name) {
  if (!inherits(grob, "latexgrob")) {
    stop("grob must be a latexgrob (returned by latex_grob()).", call. = FALSE)
  }
  marks <- grob$marks
  if (is.null(marks) || nrow(marks) == 0L) {
    stop("This grob has no marks. Place \\mark{name} inside the LaTeX source.",
         call. = FALSE)
  }
  idx <- match(name, marks$name)
  if (is.na(idx)) {
    stop(
      "Mark '", name, "' not found. Available: ",
      paste(sprintf("'%s'", marks$name), collapse = ", "),
      call. = FALSE
    )
  }
  vp <- grob$vp
  bbox_w <- grob$bbox_w
  bbox_h <- grob$bbox_h
  # bbox bottom-left in the parent viewport, expressed as a unit
  # expression so it resolves lazily at draw time.
  left   <- vp$x - grid::unit(grob$hjust * bbox_w, "bigpts")
  bottom <- vp$y - grid::unit(grob$vjust * bbox_h, "bigpts")
  list(
    x = left   + grid::unit(marks$x[idx], "bigpts"),
    y = bottom + grid::unit(marks$y[idx], "bigpts")
  )
}

# Translate string-valued hjust/vjust into the [0,1] viewport just values
# grid expects. Numeric inputs pass through unchanged. "baseline" (vjust
# only) places the formula's math baseline at the anchor point — using
# bbox_bl_bp / bbox_h, the same baseline that grobs query via
# `ascentDetails()`/`descentDetails()`.
.resolve_just <- function(hjust, vjust, bbox_bl_bp, bbox_h) {
  hj <- .resolve_hjust(hjust)
  vj <- .resolve_vjust(vjust, bbox_bl_bp = bbox_bl_bp, bbox_h = bbox_h)
  list(hjust = hj, vjust = vj)
}

.hjust_strings <- c(
  left     = 0,
  bbleft   = 0,
  center   = 0.5,
  centre   = 0.5,
  middle   = 0.5,
  bbcentre = 0.5,
  right    = 1,
  bbright  = 1
)

.resolve_hjust <- function(hjust) {
  if (is.numeric(hjust)) return(hjust)
  if (!is.character(hjust) || length(hjust) != 1L) {
    stop("hjust must be a numeric or a single string.", call. = FALSE)
  }
  v <- .hjust_strings[hjust]
  if (is.na(v)) {
    stop(
      "hjust must be numeric or one of: ",
      paste(sprintf("'%s'", names(.hjust_strings)), collapse = ", "),
      call. = FALSE
    )
  }
  unname(v)
}

.resolve_vjust <- function(vjust, bbox_bl_bp, bbox_h) {
  if (is.numeric(vjust)) return(vjust)
  if (!is.character(vjust) || length(vjust) != 1L) {
    stop("vjust must be a numeric or a single string.", call. = FALSE)
  }
  switch(
    vjust,
    bottom = 0,
    center = ,
    centre = ,
    middle = 0.5,
    top = 1,
    baseline = if (bbox_h > 0) bbox_bl_bp / bbox_h else 0.5,
    stop(
      "vjust must be numeric or one of: 'bottom', 'center'/'centre'/'middle', ",
      "'top', 'baseline'.",
      call. = FALSE
    )
  )
}

# Shared parse pipeline used by latex_grob(), latex_dims(), latex_tree().
# Resolves fontsize/cex/lineheight/fontfamily/col out of `gp`,
# runs MicroTeX parse via the cache, optionally also runs a path-mode
# parse for device-fallback. Returns the layout and the stripped-down
# `gp` safe to attach to child grobs (fontsize/cex/lineheight removed
# so they don't re-scale at draw time).
.parse_from_gp <- function(tex, gp, math_font, max_width, tex_style,
                           render_mode, input_mode = "mixed",
                           with_path_fallback = FALSE, justify = FALSE,
                           line_break = "greedy") {
  # Patterns are registered on first use, not at load.
  .ensure_bundled_fonts_registered()
  .check_tex_style(tex_style)
  input_mode <- match.arg(input_mode, c("math", "mixed"))
  if (max_width < 0) stop("max_width must be non-negative.", call. = FALSE)

  # Font size is needed before anything else now, because `em`/`ex` in an
  # \includegraphics option resolve against it.
  fontsize <- gp$fontsize %||% 20
  if (!is.null(gp$cex)) fontsize <- fontsize * gp$cex

  # Images are resolved first, before any other rewriting, because both of
  # the steps below would corrupt a file path: .strip_document_wrappers()
  # eats `%`-to-end-of-line, and .expand_macros() would rewrite `\Users` or
  # `\Temp` mid-path for anyone who had defined a macro by that name. The
  # second pass catches an \includegraphics that a macro produced; it is a
  # no-op when the first pass already consumed them all.
  tex <- .resolve_graphics(tex, fontsize = fontsize, max_width = max_width)
  tex <- .strip_document_wrappers(tex)
  tex <- .expand_macros(tex)
  tex <- .resolve_graphics(tex, fontsize = fontsize, max_width = max_width)
  # The user-facing `tex` stays as the macro-expanded source so that
  # editDetails() can re-parse without doubling up the \text{} wrap.
  # `parse_input` is the actual string handed to the MicroTeX parser.
  parse_input <- latex_wrap(tex, input_mode = input_mode)
  math_font <- resolve_math_font(math_font)

  fg_color <- if (!is.null(gp$col)) {
    # parse_latex_cpp takes a single colour; use the first if a vector
    # slipped through gpar().
    rgba <- grDevices::col2rgb(gp$col[[1]], alpha = TRUE)[, 1]
    if (rgba[["alpha"]] >= 255L) {
      sprintf("#%02X%02X%02X", rgba[["red"]], rgba[["green"]], rgba[["blue"]])
    } else {
      # MicroTeX's decodeColor() reads 9-char hex as #AARRGGBB, not the
      # #RRGGBBAA that grDevices::rgb() would emit.
      sprintf("#%02X%02X%02X%02X", rgba[["alpha"]],
              rgba[["red"]], rgba[["green"]], rgba[["blue"]])
    }
  } else {
    "#000000"
  }

  # Grid semantics: gp$fontsize is in points, gp$cex multiplies it,
  # gp$lineheight is total-line-height multiplier. Bake these into the
  # parse call (layout depends on them), then strip from gp so they
  # don't re-apply at draw time. `fontsize` itself is resolved at the top
  # of this function, since the image resolver needs it.
  line_space <- .line_space_from_lineheight(gp$lineheight, fontsize)
  gp$fontsize <- NULL
  gp$cex <- NULL
  gp$lineheight <- NULL

  # Only fontfamily matters for \text{} blocks: bold/italic runs come from
  # the LaTeX source (\textbf, \textit, ...) as per-record font_style, so a
  # gpar()-level fontface is not consulted.
  text_gp <- grid::gpar()
  if (!is.null(gp$fontfamily)) text_gp$fontfamily <- gp$fontfamily

  main_font <- .resolve_text_font(text_gp$fontfamily %||% "sans")

  measurer <- .make_text_measurer(text_gp)
  register_text_measurer(measurer)
  on.exit(clear_text_measurer(), add = TRUE)

  text_family <- text_gp$fontfamily %||% ""

  layout <- .parse_latex_cached(
    tex = parse_input, text_size = fontsize, line_space = line_space,
    fg_color = fg_color, max_width = max_width, math_font = math_font,
    main_font = main_font, use_path = (render_mode == "path"),
    tex_style = tex_style, text_family = text_family, justify = justify,
    optimal_break = identical(line_break, "optimal")
  )

  path_layout <- NULL
  if (with_path_fallback && render_mode == "typeface") {
    path_layout <- .parse_latex_cached(
      tex = parse_input, text_size = fontsize, line_space = line_space,
      fg_color = fg_color, max_width = max_width, math_font = math_font,
      main_font = main_font, use_path = TRUE, tex_style = tex_style,
      text_family = text_family, justify = justify,
      optimal_break = identical(line_break, "optimal")
    )
  }

  list(
    tex = tex,
    layout = layout,
    path_layout = path_layout,
    fontsize = fontsize,
    fg_color = fg_color,
    text_gp = text_gp,
    gp = gp,
    render_mode = render_mode
  )
}


# Check whether the current graphics device supports rendering glyphGrob
# objects via the dev->glyph() graphics engine interface (R >= 4.3).
# Uses dev.capabilities()$glyphs when a device is open; returns TRUE
# when no device is open (layout-only / measurement context).
.device_supports_typeface_glyphs <- function() {
  cur <- grDevices::dev.cur()
  if (cur == 1L) return(TRUE)  # null device (no drawing)

  caps <- grDevices::dev.capabilities()
  isTRUE(caps[["glyphs"]])
}

# Devices already told about the typeface fallback, keyed by device number
# and name. Whether glyphs can be set is a property of the *device*, not of
# the grob, but makeContent() runs per grob and on every redraw -- warning
# there flooded a figure holding several labels with the same message.
.typeface_noted <- new.env(parent = emptyenv())

.clear_typeface_noted <- function() {
  rm(list = ls(.typeface_noted), envir = .typeface_noted)
  invisible(NULL)
}

# Tell the user at most once per device. Returns TRUE if it did.
.note_typeface_fallback_once <- function(explicit) {
  # Only tell someone who asked for typeface and did not get it. Reporting
  # the default would reach every user on every unsupported device, and
  # they never expressed a preference. This also needs no list of which
  # devices are vector -- there is no capability to test for that.
  if (!isTRUE(explicit)) return(invisible(FALSE))
  cur <- grDevices::dev.cur()
  key <- paste0(cur, ":", names(cur))
  if (!is.null(.typeface_noted[[key]])) return(invisible(FALSE))
  .typeface_noted[[key]] <- TRUE
  # A message, not a warning: nothing is wrong and the package has already
  # done the sensible thing. A warning would also be escalated to an error
  # under options(warn = 2), which some CI setups use.
  message(
    "Current graphics device does not support glyph rendering; falling ",
    "back to path mode, so math is drawn as outlines rather than text. ",
    "On a vector device -- svglite::svglite() or grDevices::cairo_pdf() ",
    "-- it stays selectable."
  )
  invisible(TRUE)
}

#' @method makeContent latexgrob
#' @export
makeContent.latexgrob <- function(x) {
  render_mode <- x$render_mode %||% "typeface"
  layout_df <- x$layout_df

  if (identical(render_mode, "typeface") && !.device_supports_typeface_glyphs()) {
    if (!is.null(x$path_layout_df)) {
      layout_df <- x$path_layout_df
      render_mode <- "path"
      .note_typeface_fallback_once(isTRUE(x$render_mode_explicit))
    }
  }

  children <- build_latex_children(
    layout_df, x$bbox_h,
    depth = x$bbox_d %||% 0,
    text_gp = x$text_gp,
    render_mode = render_mode
  )

  if (isTRUE(x$debug)) {
    children <- .add_debug_overlay(
      children, layout_df,
      total_h = x$bbox_h,
      bbox_w = x$bbox_w,
      depth = x$bbox_d %||% 0
    )
  }

  grid::setChildren(x, children)
}

# Fields whose values feed .parse_from_gp(); editing any of them forces a
# re-parse so the layout/bbox/text metrics stay in sync with the inputs.
.latex_parse_fields <- c("tex", "math_font", "max_width", "tex_style",
                         "input_mode", "render_mode", "justify",
                         "line_break", "gp")

#' @method editDetails latexgrob
#' @export
editDetails.latexgrob <- function(x, specs) {
  if (length(specs) == 0L) return(x)

  parse_changed <- any(.latex_parse_fields %in% names(specs))
  just_changed  <- any(c("hjust", "vjust") %in% names(specs))

  if (parse_changed) {
    parsed <- .parse_from_gp(
      tex = x$tex, gp = x$gp, math_font = x$math_font,
      max_width = x$max_width, tex_style = x$tex_style,
      input_mode = x$input_mode %||% "math",
      render_mode = x$render_mode, with_path_fallback = TRUE,
      justify = isTRUE(x$justify),
      line_break = x$line_break %||% "greedy"
    )
    layout <- parsed$layout
    x$tex            <- parsed$tex
    x$layout_df      <- layout
    x$bbox_w         <- attr(layout, "bbox_width")
    x$bbox_h         <- attr(layout, "bbox_height")
    x$bbox_d         <- attr(layout, "bbox_depth")
    x$bbox_bl_bp     <- x$bbox_h * (1 - attr(layout, "bbox_baseline"))
    x$is_split       <- isTRUE(attr(layout, "bbox_is_split"))
    x$marks          <- .extract_marks(layout, bbox_h = x$bbox_h)
    x$fontsize       <- parsed$fontsize
    x$text_gp        <- parsed$text_gp
    x$render_mode    <- parsed$render_mode
    x$path_layout_df <- parsed$path_layout
    x$gp             <- parsed$gp
  }

  # When the user edits hjust/vjust, the spec value is the new raw input
  # (potentially a string like "baseline"). Stash it so a later parse-only
  # edit can re-resolve correctly against the new bbox.
  if ("hjust" %in% names(specs)) x$hjust_input <- specs$hjust
  if ("vjust" %in% names(specs)) x$vjust_input <- specs$vjust

  if ((parse_changed || just_changed) && !is.null(x$vp)) {
    just <- .resolve_just(
      x$hjust_input %||% x$hjust,
      x$vjust_input %||% x$vjust,
      bbox_bl_bp = x$bbox_bl_bp, bbox_h = x$bbox_h
    )
    x$hjust <- just$hjust
    x$vjust <- just$vjust
    old_vp <- x$vp
    x$vp <- grid::viewport(
      x = old_vp$x, y = old_vp$y,
      width  = grid::unit(x$bbox_w, "bigpts"),
      height = grid::unit(x$bbox_h, "bigpts"),
      just   = c(just$hjust, just$vjust),
      angle  = old_vp$angle
    )
  }

  x
}

.add_debug_overlay <- function(children, layout_df, total_h, bbox_w, depth) {
  bbox <- grid::rectGrob(
    x = grid::unit(0, "bigpts"),
    y = grid::unit(0, "bigpts"),
    width = grid::unit(bbox_w, "bigpts"),
    height = grid::unit(total_h, "bigpts"),
    just = c("left", "bottom"),
    gp = grid::gpar(col = "gray60", fill = NA, lty = "dashed", lwd = 0.5),
    name = "debug.bbox"
  )

  baseline_y <- depth
  baseline <- grid::segmentsGrob(
    x0 = grid::unit(0, "bigpts"),
    y0 = grid::unit(baseline_y, "bigpts"),
    x1 = grid::unit(bbox_w, "bigpts"),
    y1 = grid::unit(baseline_y, "bigpts"),
    gp = grid::gpar(col = "red", lwd = 0.75),
    name = "debug.baseline"
  )

  depth_line <- grid::segmentsGrob(
    x0 = grid::unit(0, "bigpts"),
    y0 = grid::unit(0, "bigpts"),
    x1 = grid::unit(bbox_w, "bigpts"),
    y1 = grid::unit(0, "bigpts"),
    gp = grid::gpar(col = "gray60", lwd = 0.5, lty = "dashed"),
    name = "debug.depth"
  )

  overlays <- grid::gList(bbox, depth_line, baseline)

  n <- nrow(layout_df)
  if (!is.null(n) && n > 0) {
    ox <- layout_df$x
    oy <- total_h - layout_df$y
    keep <- is.finite(ox) & is.finite(oy)
    if (any(keep)) {
      dots <- grid::pointsGrob(
        x = grid::unit(ox[keep], "bigpts"),
        y = grid::unit(oy[keep], "bigpts"),
        pch = 20,
        size = grid::unit(0.6, "mm"),
        gp = grid::gpar(col = "blue"),
        name = "debug.origins"
      )
      overlays <- grid::gList(overlays, dots)
    }
  }

  grid::gList(children, overlays)
}

#' @method widthDetails latexgrob
#' @export
widthDetails.latexgrob <- function(x) {
  grid::unit(x$bbox_w, "bigpts")
}

#' @method heightDetails latexgrob
#' @export
heightDetails.latexgrob <- function(x) {
  grid::unit(x$bbox_h, "bigpts")
}

#' @method ascentDetails latexgrob
#' @export
ascentDetails.latexgrob <- function(x) {
  grid::unit(x$bbox_h - x$bbox_d, "bigpts")
}

#' @method descentDetails latexgrob
#' @export
descentDetails.latexgrob <- function(x) {
  grid::unit(x$bbox_d, "bigpts")
}

# grid resolves grobx/groby units by pushing this grob's viewport, calling
# xDetails() AND yDetails() with the same theta, and transforming the
# resulting (x, y) location back through the viewport (position, just,
# rotation). The formula's bounding box *is* the viewport, so the boundary
# point is the boundary of the unit square in the pushed context — exactly
# what grid's own rect method computes (ray from the centre at angle theta,
# intersected with the edges). Delegating keeps us identical to a
# rectGrob drawn in the same viewport; note this means theta is measured
# in the formula's own frame, so for rot != 0 it rotates with the grob.

#' @method xDetails latexgrob
#' @export
xDetails.latexgrob <- function(x, theta) {
  grid::xDetails(grid::rectGrob(), theta)
}

#' @method yDetails latexgrob
#' @export
yDetails.latexgrob <- function(x, theta) {
  grid::yDetails(grid::rectGrob(), theta)
}


#' Draw LaTeX directly to the current device
#'
#' A convenience wrapper that creates a \code{\link{latex_grob}} and
#' immediately draws it on the current device via
#' \code{\link[grid]{grid.draw}}.
#'
#' @param tex Character string of LaTeX math code.
#' @param ... Additional arguments passed to \code{\link{latex_grob}}.
#' @return Invisibly returns the grob.
#' @rdname latex_grob
#' @export
#'
grid.latex <- function(tex, ...) {
  g <- latex_grob(tex, ...)
  grid::grid.draw(g)
  invisible(g)
}

#' Get dimensions of a LaTeX expression
#'
#' @inheritParams latex_grob
#' @return A list with the following elements:
#' \itemize{
#'   \item \code{width}, \code{height}, \code{depth}: grid unit objects
#'     in big points. \code{height} is total height (ascent + descent).
#'   \item \code{baseline}: grid unit object giving the baseline position
#'     measured in big points from the \emph{bottom} of the bounding box.
#'     Equivalent to \code{height - depth} for single-line formulas. Useful
#'     for aligning a formula's baseline with surrounding text.
#'   \item \code{is_split}: logical; \code{TRUE} if the formula was wrapped
#'     across multiple lines (only possible when \code{max_width > 0}).
#' }
#' @export
#'
#' @examples
#' latex_dims("\\frac{a}{b}")
latex_dims <- function(tex, math_font = "", max_width = 0,
                       tex_style = "",
                       input_mode = c("mixed", "math"),
                       render_mode = c("typeface", "path"),
                       justify = FALSE,
                       line_break = c("greedy", "optimal"),
                       gp = grid::gpar()) {
  .apply_opts("math_font", "render_mode", "tex_style", "input_mode",
              "justify", "line_break")
  render_mode <- match.arg(render_mode)
  input_mode <- match.arg(input_mode)
  line_break <- match.arg(line_break)
  .check_justify(justify)

  parsed <- .parse_from_gp(
    tex = tex, gp = gp, math_font = math_font, max_width = max_width,
    tex_style = tex_style, render_mode = render_mode,
    input_mode = input_mode, justify = justify, line_break = line_break
  )
  layout <- parsed$layout
  bbox_h <- attr(layout, "bbox_height")
  bbox_bl_frac <- attr(layout, "bbox_baseline")
  list(
    width    = grid::unit(attr(layout, "bbox_width"), "bigpts"),
    height   = grid::unit(bbox_h, "bigpts"),
    depth    = grid::unit(attr(layout, "bbox_depth"), "bigpts"),
    baseline = grid::unit(bbox_h * (1 - bbox_bl_frac), "bigpts"),
    is_split = isTRUE(attr(layout, "bbox_is_split"))
  )
}


# On some Windows locale/font combinations, measuring can error for CJK text.
# Keep layout flowing with a simple width fallback instead of failing hard.
# `tg` is a textGrob carrying the measurement gp; `em` is the font size the
# measurement runs at (the measurer's ref_size).
.measure_text_bigpts <- function(tg, text, em = 72) {
  out <- tryCatch(
    grid::convertWidth(grid::grobWidth(tg), "bigpts", valueOnly = TRUE),
    error = function(e) {
      w <- tryCatch(base::nchar(text, type = "width"), error = function(...) NA_real_)
      if (is.na(w)) {
        w <- base::nchar(text, type = "chars")
      }
      # Half an em per terminal width cell: narrow chars ~0.5 em, CJK
      # (2 cells) ~1 em — matching the C++ heuristic in src/init.cpp.
      as.numeric(w) * 0.5 * em
    }
  )
  as.numeric(out)
}


#' Create a text measurement closure for MicroTeX layout
#'
#' Returns a function that measures text using R's grid graphics system.
#' The closure is called from C++ during \code{parse_latex_cpp()} to get
#' accurate font metrics for \code{\\text\{\}} blocks.
#'
#' @param text_gp A \code{\link[grid]{gpar}} object whose
#'   \code{fontfamily} is used for measurement. The face comes from
#'   MicroTeX's per-run \code{font_style}, not from \code{text_gp}.
#' @return A function taking \code{(text, font_style)} that returns
#'   \code{c(width_ratio, ascent_ratio, height_ratio)} where ratios
#'   are relative to the font size.
#' @noRd
.make_text_measurer <- function(text_gp) {
  ref_size <- 72  # reference size in points for measurement precision

  # Cache the R version check
  has_ascent_fn <- getRversion() >= "4.4.0"

  # Per-closure cache keyed on (font_style, font_family, text). Lifetime =
  # one parse (the closure is created fresh per parse in
  # latex_grob/latex_dims), so graphics state can't drift between calls.
  # Hits avoid a textGrob construction + grid::convertHeight round trip per
  # repeated span.
  cache <- new.env(parent = emptyenv())

  # `font_family` is the family named by a \gmfontfamily span, passed in by
  # TextLayout_R. Defaulted, so the two-argument calls that predate it --
  # including register_text_measurer() users -- keep working.
  function(text, font_style, font_family = "") {
    key <- paste0(as.integer(font_style), "\x1f", font_family, "\x1f", text)
    hit <- cache[[key]]
    if (!is.null(hit)) return(hit)

    # MicroTeX probes C-style escapes while tokenising \text{} content
    # (e.g. the leading "\f" of \frac arrives here as a form feed before
    # the parser backtracks to the real command). Control characters have
    # no metrics on some devices (grid warns, e.g. cp1252 pdf()) and the
    # probe never reaches the final layout — answer without touching the
    # device.
    if (grepl("^[[:cntrl:]]*$", text)) {
      result <- c(0, 0.8, 1)
      cache[[key]] <- result
      return(result)
    }

    face <- .resolve_text_face(as.integer(font_style))

    gp <- grid::gpar(fontsize = ref_size, fontface = face)
    fam <- .resolve_text_family(as.integer(font_style), text_gp$fontfamily,
                                font_family)
    if (!is.null(fam)) {
      gp$fontfamily <- fam
    }

    # Ensure a graphics device is available for measurement
    needs_dev <- grDevices::dev.cur() == 1L
    if (needs_dev) {
      grDevices::pdf(NULL)
      on.exit(grDevices::dev.off(), add = TRUE)
    }

    # Carry the font settings on a throwaway textGrob rather than pushing a
    # viewport. pushViewport() writes to the device's display list (6 records
    # per push/pop), which makes a caller's device look like it holds a plot:
    # knitr then snapshots that page as a spurious blank figure before the
    # real plot's grid.newpage(). grob* queries below only read metrics.
    tg <- grid::textGrob(text, gp = gp)

    # Measuring is per *character*, so a device that cannot resolve the
    # family -- base pdf() has no named families, only what pdfFonts()
    # declares -- would warn dozens of times for one label. Stay quiet
    # here: the same device warns again when the text is actually drawn,
    # which is the once-per-run, user-actionable copy of the message.
    w <- suppressWarnings(.measure_text_bigpts(tg, text, em = ref_size))

    # Measure ascent and descent (with Windows/CJK locale fallback)
    ad <- suppressWarnings(tryCatch({
      if (has_ascent_fn) {
        asc <- grid::convertHeight(
          get("grobAscent", envir = asNamespace("grid"))(tg),
          "bigpts", valueOnly = TRUE
        )
        desc <- grid::convertHeight(
          get("grobDescent", envir = asNamespace("grid"))(tg),
          "bigpts", valueOnly = TRUE
        )
      } else {
        h <- grid::convertHeight(grid::grobHeight(tg), "bigpts", valueOnly = TRUE)
        asc <- h * 0.8
        desc <- h - asc
      }
      c(asc, desc)
    }, error = function(e) {
      # Approximate: 80% of font size for ascent, 20% for descent
      c(ref_size * 0.8, ref_size * 0.2)
    }))
    asc  <- ad[1]
    desc <- ad[2]

    result <- c(w / ref_size, asc / ref_size, (asc + desc) / ref_size)
    cache[[key]] <- result
    result
  }
}

# Fixed-width, wrapping table columns: the `p{len}` column type added to
# the vendored MicroTeX (atom_matrix.cpp). Before it, the spec parser knew
# only `l r c | @ * >` and `p{}` was a hard parse error, so a wide table
# could only overflow.

test_that("p{} wraps a cell to a fixed measure", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  long <- "\\text{the quick brown fox jumps over the lazy dog again and again}"
  dims <- function(spec) {
    d <- latex_dims(paste0("\\begin{tabular}{", spec, "}", long,
                           "\\end{tabular}"),
                    input_mode = "math", gp = grid::gpar(fontsize = 16))
    c(w = as.numeric(d$width), h = as.numeric(d$height))
  }

  free <- dims("l")
  fixed <- dims("p{3cm}")
  # Narrower, and taller because it now occupies several lines.
  expect_lt(fixed[["w"]], free[["w"]] / 2)
  expect_gt(fixed[["h"]], free[["h"]] * 2)

  # The width tracks the request rather than the content.
  w2 <- dims("p{2cm}")[["w"]]
  w4 <- dims("p{4cm}")[["w"]]
  w6 <- dims("p{6cm}")[["w"]]
  expect_lt(w2, w4)
  expect_lt(w4, w6)
  # Equal steps in the request give equal steps in the result.
  expect_equal(w4 - w2, w6 - w4, tolerance = 2)

  # m{} and b{} differ from p{} only in vertical alignment, which the row
  # builder already handles, so they parse to the same measure.
  expect_equal(dims("m{3cm}")[["w"]], dims("p{3cm}")[["w"]])
  expect_equal(dims("b{3cm}")[["w"]], dims("p{3cm}")[["w"]])
})

test_that("a malformed p{} width leaves the column content-sized", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  ref <- as.numeric(latex_dims(
    "\\begin{tabular}{l}\\text{a}\\end{tabular}",
    input_mode = "math", gp = grid::gpar(fontsize = 16))$width)
  for (spec in c("p{}", "p{nonsense}", "p", "p{0pt}")) {
    w <- as.numeric(latex_dims(
      paste0("\\begin{tabular}{", spec, "}\\text{a}\\end{tabular}"),
      input_mode = "math", gp = grid::gpar(fontsize = 16))$width)
    expect_equal(w, ref, info = spec)
  }
})

test_that("p{} does not disturb ordinary tables or leak between parses", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  plain <- function() as.numeric(latex_dims(
    "\\begin{tabular}{|c|c|}\\hline \\text{a}&\\text{b}\\\\\\hline\\end{tabular}",
    input_mode = "math", gp = grid::gpar(fontsize = 16))$width)

  before <- plain()
  invisible(latex_dims(
    "\\begin{tabular}{p{3cm}}\\text{wrap me please}\\end{tabular}",
    input_mode = "math", gp = grid::gpar(fontsize = 16)))
  # A fixed width is per-column state; it must not survive into the next
  # parse the way a stray static would.
  expect_equal(plain(), before)
})

# The release / re-init half of this lives in test-zz-release-cycle.R:
# tearing MicroTeX down mid-suite strands the font registry that
# test-text-font-auto.R depends on.

# Text is drawn as runs, not one record per letter. How far a run
# stretches depends on whether anything will wrap:
#
#   no width set  -> the whole phrase, spaces included, is one run, so the
#                    device can order it (bidi) and kern across spaces
#   width set     -> one run per word, because the spaces are where the
#                    line breaks
#
# See RowAtom::processTextRun in src/MicroTeX/lib/atom/atom_row.cpp.

texts <- function(tex, ...) {
  r <- latex_tree(tex, input_mode = "math", ...)$records
  r$text[r$type == "text"]
}
# latex_tree() takes no max_width-dependent options, so the wrapped cases
# go through latex_grob().
wrapped <- function(tex, mw, ...) {
  d <- latex_grob(tex, input_mode = "math", max_width = mw, ...)$layout_df
  d$text[d$type == "text"]
}

test_that("an unwrapped phrase is a single record", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  expect_equal(texts("\\text{Hello}"), "Hello")
  expect_equal(texts("\\text{Hello, world!}"), "Hello, world!")
  # Digits come along too: the break position after each one only matters
  # when a long number might have to wrap.
  expect_equal(texts("\\text{page 42 of 99}"), "page 42 of 99")
})

test_that("a wrapped phrase keeps one run per word", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # This is the guard that matters: the spaces have to stay separate
  # boxes, because they are what the splitter breaks at.
  expect_equal(wrapped("\\text{one two three}", 40), c("one", "two", "three"))
  expect_equal(wrapped("\\text{ab12cd}", 40), c("ab", "1", "2", "cd"))
})

test_that("kerning is applied, because the device sees a whole word", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # Laid out per character the width is exactly the sum of the advances,
  # which for a kern-heavy string is visibly loose -- AVATAR came out
  # 80.00 against R's 72.22, about 11% too wide.
  gp <- grid::gpar(fontsize = 20, fontfamily = "sans")
  for (s in c("AVATAR", "Wave", "To Vary")) {
    w <- as.numeric(latex_dims(paste0("\\text{", s, "}"),
                               input_mode = "math", gp = gp)$width)
    grid::pushViewport(grid::viewport(gp = gp))
    kerned <- grid::convertWidth(grid::stringWidth(s), "bigpts",
                                 valueOnly = TRUE)
    grid::popViewport()
    # Within a big point: latex_dims() reports whole big points.
    expect_lt(abs(w - kerned), 1)
  }
})

test_that("the output holds the word, so a viewer can find it", {
  skip_if_not_installed("svglite")
  f <- tempfile(fileext = ".svg")
  on.exit(unlink(f), add = TRUE)
  svglite::svglite(f, width = 5, height = 1)
  grid.latex("\\text{Hello world}", input_mode = "math",
             gp = grid::gpar(fontsize = 20))
  dev.off()
  svg <- paste(readLines(f, warn = FALSE), collapse = "\n")
  # The whole phrase in one element, so a viewer can search for it. Per
  # character this was ten elements holding one letter each, and even the
  # space was missing, so no search could match anything.
  expect_equal(length(gregexpr("<text", svg, fixed = TRUE)[[1]]), 1L)
  expect_true(grepl("Hello world", svg, fixed = TRUE))
})

test_that("math is laid out per glyph, untouched", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # Math positions every glyph itself; merging there would destroy the
  # layout. These counts must not move.
  expect_equal(nrow(latex_tree("$\\frac{a}{b}$")$records), 3L)
  expect_equal(nrow(latex_tree("$\\sum_{i=1}^n x_i^2$")$records), 8L)
  expect_equal(nrow(latex_tree("$\\alpha\\beta\\gamma$")$records), 3L)
})

test_that("a run stops at anything that is not plain text", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # Each of these is load-bearing. A font scope must not be merged
  # across, or the styling is lost; math and a measured skip keep their
  # own boxes. Only the ordinary word space is folded in.
  expect_equal(texts("\\textbf{bold}\\text{plain}"), c("bold", "plain"))
  expect_equal(texts("\\textsf{ab\\textrm{cd}ef}"), c("ab", "cd", "ef"))
  expect_equal(texts("\\text{before $x$ after}"), c("before ", " after"))
  expect_length(texts("\\text{a \\quad b}"), 2L)
})

test_that("an unwrapped right-to-left phrase comes out in the right order", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # One run means the device orders it, and the device implements the
  # bidirectional algorithm that MicroTeX does not. Positioned per word
  # it came out reversed.
  # Written as \u escapes so the file itself stays ASCII, which is what
  # R CMD check wants of package sources.
  ar <- paste("\u0645\u0631\u062d\u0628\u0627", "\u0628\u0643")
  expect_equal(texts(paste0("\\text{", ar, "}")), ar)
  # Mixed scripts too -- the device resolves the whole line at once.
  mixed <- paste("\u0645\u0631\u062d\u0628\u0627", "abc", "\u0628\u0643")
  expect_equal(texts(paste0("\\text{", mixed, "}")), mixed)
})

test_that("a wrapped right-to-left paragraph is reordered per line", {
  skip_if_not(microtex_bidi_available(),
              "built without fribidi; wrapped RTL keeps logical order")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # Lines are broken in logical order and only then put into visual
  # order, which is what the Unicode algorithm requires. Reading each
  # line right to left must give the source order back.
  words <- c("\u0645\u0631\u062d\u0628\u0627", "\u0628\u0643",
             "\u0641\u064a", "\u0627\u0644\u0639\u0627\u0644\u0645")
  src <- rep(words, 3)
  ar <- paste(src, collapse = " ")
  d <- latex_grob(paste0("\\text{", ar, "}"), input_mode = "math",
                  max_width = 120, gp = grid::gpar(fontsize = 16))$layout_df
  d <- d[d$type == "text", ]
  expect_gt(length(unique(round(d$y, 1))), 1L)   # it really wrapped
  got <- unlist(lapply(sort(unique(round(d$y, 1))), function(yy) {
    s <- d[abs(round(d$y, 1) - yy) < 0.01, ]
    s$text[order(-s$x)]                          # right to left
  }))
  expect_equal(got, src)
})

# Reading a laid-out line from the right, across every line in order.
# Right to left is the direction the source is written in, so for a
# right-to-left paragraph this must give the source words back.
right_to_left <- function(tex, mw = 110, fn = latex_grob, ...) {
  d <- fn(tex, max_width = mw, gp = grid::gpar(fontsize = 14), ...)$layout_df
  d <- d[d$type == "text" & !is.na(d$text), ]
  unlist(lapply(sort(unique(round(d$y, 1))), function(yy) {
    s <- d[abs(round(d$y, 1) - yy) < 0.01, ]
    s$text[order(-s$x)]
  }))
}

test_that("right-to-left text is ordered across font groups, not only within one", {
  skip_if_not(microtex_bidi_available(), "built without fribidi")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # A row used to collect only the characters that were its *direct*
  # children, so text living in sibling groups -- which is what every
  # emphasis compiles to -- left the row with an empty string, no levels
  # and no reordering at all. See RowAtom::collectBidiText.
  a <- c("\u0645\u0631\u062d\u0628\u0627", "\u0628\u0643",
         "\u0641\u064a", "\u0627\u0644\u0639\u0627\u0644\u0645")
  txt <- function(s) paste0("\\text{", s, "}")

  # Two plain groups, and an emphasised group between two plain ones.
  expect_equal(
    right_to_left(paste0(txt(paste(a[1], a[2], "")), txt(paste(a[3], a[4]))),
                  input_mode = "math"),
    a)
  expect_equal(
    right_to_left(paste0(txt(paste(a[1], a[2], "")), "\\textbf{", txt(a[3]), "}",
                         txt(paste0(" ", a[4]))),
                  input_mode = "math"),
    a)
  # A colour and a named family are wrappers of their own.
  expect_equal(
    right_to_left(paste0(txt(paste0(a[1], " ")), "\\textcolor{red}{", txt(a[2]), "}",
                         txt(paste0(" ", a[3]))),
                  input_mode = "math"),
    a[1:3])
  expect_equal(
    right_to_left(paste0(txt(paste0(a[1], " ")), "\\gmfontfamily{serif}{", txt(a[2]), "}",
                         txt(paste0(" ", a[3]))),
                  input_mode = "math"),
    a[1:3])

  # Still correct where it already was: one group, and emphasis nested
  # inside a group rather than beside it.
  expect_equal(right_to_left(txt(paste(a, collapse = " ")), input_mode = "math"), a)
  expect_equal(
    right_to_left(txt(paste0(a[1], " \\textbf{", a[2], "} ", a[3])),
                  input_mode = "math"),
    a[1:3])
})

test_that("a left-to-right run inside right-to-left keeps its own direction", {
  skip_if_not(microtex_bidi_available(), "built without fribidi")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  a1 <- "\u0645\u0631\u062d\u0628\u0627"; a2 <- "\u0628\u0643"
  txt <- function(s) paste0("\\text{", s, "}")

  # A Latin word in its own group sits between the two Arabic words.
  expect_equal(
    right_to_left(paste0(txt(paste0(a1, " ")), "\\textbf{", txt("Latin"), "}",
                         txt(paste0(" ", a2))),
                  input_mode = "math"),
    c(a1, "Latin", a2))

  # Digits are a left-to-right run inside the line: each digit is its own
  # record (they carry break positions), so read from the right they come
  # back reversed -- which is exactly what "drawn left to right" means
  # inside a right-to-left line.
  got <- right_to_left(paste0(txt(paste0(a1, " ")), txt("2026"),
                              txt(paste0(" ", a2))), input_mode = "math")
  expect_equal(got[1], a1)                       # Arabic still rightmost
  expect_equal(got[length(got)], a2)
  expect_equal(rev(got[2:(length(got) - 1)]), c("2", "0", "2", "6"))
})

test_that("a left-to-right group that breaks across lines stays in the flow", {
  skip_if_not(microtex_bidi_available(), "built without fribidi")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # A group with no right-to-left text of its own resolves no levels, so
  # when the splitter breaks inside it the piece was given level 0. Rule
  # L2 reverses runs at or above the lowest *odd* level, so a 0 among 1s
  # split the reversal in two and scrambled the line. It has to inherit
  # the level of the neighbour it is placed against instead.
  u <- c("\u062e\u06c7\u0634", "\u0643\u06d5\u0644\u062f\u0649\u06ad\u0649\u0632",
         "\u0628\u06c7", "\u062f\u06c7\u0646\u064a\u0627\u063a\u0627")
  latin <- c("alpha", "beta", "gamma", "delta", "epsilon", "zeta")
  tex <- paste0("\\text{", u[1], " ", u[2], " }",
                "\\textbf{\\text{", paste(latin, collapse = " "), "}}",
                "\\text{ ", u[3], " ", u[4], "}")
  d <- latex_grob(tex, input_mode = "math", max_width = 130,
                  gp = grid::gpar(fontsize = 14))$layout_df
  d <- d[d$type == "text" & !is.na(d$text), ]
  lines <- sort(unique(round(d$y, 1)))
  expect_gt(length(lines), 1L)                    # it really wrapped

  by_line <- function(f) {
    unlist(lapply(lines, function(yy) f(d[abs(round(d$y, 1) - yy) < 0.01, ])))
  }
  # Right-to-left text reads from the right; the Latin run reads from the
  # left. Both must come back in source order.
  got_u <- by_line(function(s) s$text[order(-s$x)])
  expect_equal(got_u[got_u %in% u], u)
  got_l <- by_line(function(s) s$text[order(s$x)])
  expect_equal(got_l[got_l %in% latin], latin)
})

test_that("right-to-left markdown is ordered through its emphasis", {
  skip_if_not(microtex_bidi_available(), "built without fribidi")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # Markdown compiles emphasis to sibling groups, so this is the case the
  # gap actually bit: bold, italic and code all lost the ordering.
  a <- c("\u0645\u0631\u062d\u0628\u0627", "\u0628\u0643",
         "\u0641\u064a", "\u0627\u0644\u0639\u0627\u0644\u0645")
  expect_equal(
    right_to_left(paste(a[1], a[2], paste0("**", a[3], "**"), a[4]),
                  fn = markdown_grob),
    a)
  expect_equal(
    right_to_left(paste(a[1], paste0("*", a[2], "*"), paste0("`", a[3], "`"), a[4]),
                  fn = markdown_grob),
    a)
})

test_that("an explicit right-to-left mark counts as right-to-left", {
  skip_if_not(microtex_bidi_available(), "built without fribidi")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # U+200F carries no script of its own, so a scan for Hebrew/Arabic
  # letters missed it -- yet it is exactly how a right-to-left run is
  # marked when the characters themselves are direction-neutral.
  rlm <- "\u200f"
  words <- c("(a)", "(b)", "(c)")
  order_of <- function(tex) {
    d <- latex_grob(tex, input_mode = "math", max_width = 60,
                    gp = grid::gpar(fontsize = 14))$layout_df
    d <- d[d$type == "text", ]
    d$text[order(-d$x)]                       # right to left
  }
  # With the mark the line reads right to left, so the source order comes
  # back when read from the right. Without it, it does not.
  marked <- order_of(paste0("\\text{", rlm, paste(words, collapse = " "), "}"))
  expect_equal(length(marked), 3L)
  expect_match(marked[1], "a", fixed = TRUE)
  expect_equal(order_of(paste0("\\text{", paste(words, collapse = " "), "}"))[1],
               "(c)")
})

test_that("a line ending at a drawn hyphen is still reordered", {
  skip_if_not(microtex_bidi_available(), "built without fribidi")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # The hyphen `\-` draws is appended to the finished line, after the
  # levels were assigned. Appending the box alone left one more child than
  # levels, and bidi_reorder does nothing when the two disagree -- so the
  # whole line silently stayed in logical order, but only when a break
  # that draws something was taken.
  ar <- "\u0645\u0631\u062d\u0628\u0627"
  first_line <- function(tex) {
    d <- latex_grob(tex, input_mode = "math", max_width = 130,
                    gp = grid::gpar(fontsize = 14))$layout_df
    d <- d[d$type == "text", ]
    d <- d[abs(round(d$y, 1) - min(round(d$y, 1))) < 0.01, ]
    d$text[order(-d$x)]
  }
  marked <- paste0("\\text{", ar, " inter\\-nation\\-alization \u0628\u0643}")
  plain <- paste0("\\text{", ar, " internationalization \u0628\u0643}")

  hy <- first_line(marked)
  expect_true("-" %in% hy)              # the break really was taken
  # Base direction is right-to-left, so the Arabic word is rightmost and
  # the Latin run sits to its left. Unreordered it comes out the other
  # way round, with the Arabic first.
  expect_equal(hy[1], ar)
  expect_equal(first_line(plain)[1], ar)
})

test_that("a p{} cell wraps even when the formula as a whole does not", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # A fixed-width column is broken to its own measure whatever the global
  # width is, so its cells must keep word runs. Folding them into one
  # phrase left the cell unbreakable and the column silently sized to
  # content -- caught only because the p{} tests measure the result.
  cell <- "\\text{a fairly long cell that ought to wrap inside its column}"
  mk <- function(spec) paste0("\\begin{tabular}{", spec,
                              "}\\hline ", cell, " & \\text{b}\\\\ \\hline\\end{tabular}")
  free <- as.numeric(latex_dims(mk("ll"), input_mode = "math")$width)
  fixed <- as.numeric(latex_dims(mk("p{3cm}l"), input_mode = "math")$width)
  expect_lt(fixed, free / 2)
})

test_that("wrapping still happens, and still respects the measure", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  prose <- paste0("\\text{", paste(rep("The quick brown fox jumps over the lazy dog.", 3),
                                   collapse = " "), "}")
  for (mw in c(150, 250, 400)) {
    d <- latex_dims(prose, max_width = mw, input_mode = "math")
    expect_lte(as.numeric(d$width), mw)
    expect_gt(as.numeric(d$height), 18)  # it did wrap
  }
})

test_that("right-to-left runs are ordered without a max_width", {
  skip_if_not(microtex_bidi_available(), "built without fribidi")
  pdf(NULL); on.exit(dev.off(), add = TRUE)

  # Reordering used to live only on the line-splitting path, which runs
  # only when max_width > 0. With the default max_width = 0 a right-to-left
  # label came out in logical order the moment it held a second child --
  # any emphasis, colour or font change -- because the single merged run
  # that the backend orders for us was no longer the whole line.
  # \u escapes so the source stays ASCII, as elsewhere in this file.
  heb <- "\u05e9\u05dc\u05d5\u05dd \u05e2\u05d5\u05dc\u05dd"
  order_of <- function(tex, mw = 0) {
    d <- latex_grob(tex, input_mode = "math", max_width = mw,
                    gp = grid::gpar(fontsize = 14))$layout_df
    d <- d[d$type == "text" & !is.na(d$text), ]
    d$text[order(d$x)]
  }

  # Base direction is right-to-left (first strong character is Hebrew), so
  # the Latin run belongs to the left of the Hebrew, not the right.
  for (wrap in list(c("\\textbf{", "}"), c("\\textcolor{red}{", "}"),
                    c("\\gmfontfamily{serif}{", "}"))) {
    tex <- paste0("\\text{", heb, " }", wrap[1], "\\text{alpha}", wrap[2])
    got <- order_of(tex)
    expect_equal(length(got), 2L, info = wrap[1])
    expect_equal(got[[1]], "alpha", info = wrap[1])
    expect_match(got[[2]], "^\u05e9", info = wrap[1])
  }

  # Setting a width must not change the reading order, only where it breaks.
  expect_equal(order_of(paste0("\\text{", heb, " }\\textbf{\\text{alpha}}"))[[1]],
               "alpha")
  expect_equal(order_of(paste0("\\text{", heb, " }\\textbf{\\text{alpha}}"), 400)[[1]],
               "alpha")

  # The merged run survives: the Hebrew is still one record, so the text
  # stays selectable and kerned rather than being split into words.
  expect_equal(length(order_of(paste0("\\text{", heb, "}"))), 1L)

  # A left-to-right formula must be untouched by any of this.
  expect_equal(order_of("\\text{alpha }\\textbf{\\text{beta}}"),
               c("alpha ", "beta"))
})

# Visual samples.
#
# Two figures, not seven. Each sample used to have a snapshot of its own,
# which meant a change to shared layout code arrived as seven separate
# diffs to open and compare -- more work to review, not less, and easy to
# accept in bulk without looking. Grouping them puts every sample in one
# image, so one look answers "did anything move?".
#
# Every sample is drawn at the size it had when it was its own file, so
# what is under test is unchanged; only the composition is.
#
# These do not run on CRAN, so they cannot stand in for unit coverage --
# see test-text-runs.R for the assertions behind the direction figure.

# Draws one labelled sample, positioned in points from the top-left of the
# page so the samples cannot collide as they would under a null layout.
place <- function(x, y, label, tex, fontsize, ...) {
  top <- function(dy) grid::unit(1, "npc") - grid::unit(dy, "pt")
  grid::grid.text(label, x = grid::unit(x, "pt"), y = top(y),
                  hjust = 0, vjust = 1,
                  gp = grid::gpar(fontsize = 8, col = "grey45"))
  grid.latex(tex, x = grid::unit(x, "pt"), y = top(y + 13),
             hjust = 0, vjust = 1, input_mode = "math",
             render_mode = "path", gp = grid::gpar(fontsize = fontsize), ...)
}

test_that("visual: math gallery", {
  skip_if_not_installed("vdiffr")
  skip_on_os("mac")
  vdiffr::expect_doppelganger("math-gallery", function() {
    # Left column: the wide samples.
    place(24, 20, "complex formula", paste0(
      "\\begin{array}{l}",
      "  \\forall\\varepsilon\\in\\mathbb{R}_+^*\\ \\exists\\eta>0",
      "\\ |x-x_0|\\leq\\eta\\Longrightarrow|f(x)-f(x_0)|\\leq\\varepsilon\\\\",
      "  \\det",
      "  \\begin{bmatrix}",
      "      a_{11}&a_{12}&\\cdots&a_{1n}\\\\",
      "      a_{21}&\\ddots&&\\vdots\\\\",
      "      \\vdots&&\\ddots&\\vdots\\\\",
      "      a_{n1}&\\cdots&\\cdots&a_{nn}",
      "  \\end{bmatrix}",
      "  \\overset{\\mathrm{def}}{=}\\sum_{\\sigma\\in\\mathfrak{S}_n}",
      "\\varepsilon(\\sigma)\\prod_{k=1}^n a_{k\\sigma(k)}\\\\",
      "  \\int_0^\\infty{x^{2n} e^{-a x^2}\\,dx} = \\frac{2n-1}{2a}",
      " \\int_0^\\infty{x^{2(n-1)} e^{-a x^2}\\,dx}",
      " = \\frac{(2n-1)!!}{2^{n+1}} \\sqrt{\\frac{\\pi}{a^{2n+1}}}\\\\",
      "\\end{array}"
    ), 12)

    place(24, 150, "multicolumn and borders", paste0(
      "\\begin{array}{|c|c|c|c|}",
      "  \\hline",
      "  \\multicolumn{4}{|c|}{\\text{Table Head}}\\\\",
      "  \\hline",
      "  \\text{Matrix}&\\multicolumn{2}{|c|}{\\text{Multicolumns}}",
      "&\\text{Font size commands}\\\\",
      "  \\hline",
      "  \\begin{pmatrix}",
      "      \\alpha_{11}&\\cdots&\\alpha_{1n}\\\\",
      "      \\hdotsfor{3}\\\\",
      "      \\alpha_{n1}&\\cdots&\\alpha_{nn}",
      "  \\end{pmatrix}",
      "  &\\large \\text{Left}&\\small \\text{Right}",
      "  &\\small \\text{small Small}\\\\",
      "  \\hline",
      "  \\multicolumn{4}{|c|}{\\text{Table Foot}}\\\\",
      "  \\hline",
      "\\end{array}"
    ), 12)

    # One enumerate exercising a \Roman* counter, a nested itemize, a
    # nested enumerate with an \alph* counter, and an item whose content
    # is an array using the \thickhline and \cline rules.
    place(24, 275, "lists and table rules", paste0(
      "\\begin{enumerate}[\\Roman*.]",
      "  \\item \\text{Limit: }\\forall\\varepsilon>0\\ \\exists\\eta>0",
      "  \\item \\text{Bullets:}\\ ",
      "        \\begin{itemize}",
      "          \\item e^{i\\pi}+1=0",
      "          \\item \\sum_{k=1}^n k=\\tfrac{n(n+1)}{2}",
      "        \\end{itemize}",
      "  \\item \\text{Lettered:}\\ ",
      "        \\begin{enumerate}[\\alph*)]",
      "          \\item \\alpha^2 \\item \\sqrt{\\beta} \\item \\gamma_0",
      "        \\end{enumerate}",
      "  \\item \\text{Ruled table:}\\ ",
      "        \\begin{array}{|c|c|c|}",
      "          \\thickhline x^2&y^2&z^2\\\\",
      "          \\cline{1-2} a&b&c\\\\\\thickhline",
      "        \\end{array}",
      "\\end{enumerate}"
    ), 15)

    # Right column: the narrow ones.
    place(380, 20, "continued fraction",
          "\\cfrac{1}{\\sqrt{2}+\n\\cfrac{1}{\\sqrt{2}+\n\\cfrac{1}{\\sqrt{2}+\\dotsb\n}}}",
          20)
    place(380, 145, "overbrace and underbrace",
          "\\rlap{\\overbrace{\\phantom{1 + a + b + \\cdots + z}}^{\\text{total + 1}}}\n1 + \\underbrace{a + b + \\cdots + z}_{\\text{total}}",
          20)
    place(380, 235, "cancel variants",
          "\\frac{a\\cancel{b}}{\\cancel{b}} = a;\n\\frac{a\\bcancel{b}}{\\bcancel{b}} = a;\n\\frac{a\\xcancel{b}}{\\xcancel{b}} = a;",
          20)
    place(380, 290, "cases", paste0(
      "P_{r-j}=\\begin{cases}",
      "0& \\text{if $r-j$ is odd},\\\\",
      "r!\\,(-1)^{(r-j)/2}& \\text{if $r-j$ is even}.",
      "\\end{cases}"
    ), 12)
  })
})

test_that("visual: text direction", {
  skip_if_not_installed("vdiffr")
  skip_on_os("mac")
  # Unlike the gallery, this figure is drawn with system fonts rather than
  # bundled outlines, so it needs the machine to have a Perso-Arabic and a
  # Hebrew face. A CI image that lacks one would fail on the missing font
  # rather than on anything we changed. Drop this line if the runners are
  # known to have them.
  skip_on_ci()

  # Escaped so the source file stays ASCII, which is what R CMD check
  # wants.
  #
  # Uyghur, in the Perso-Arabic script: "xush keldingiz" (welcome) and
  # "bu dunyagha" (to this world). It is written right to left and its
  # letters join, like Arabic, but it reaches further into the Arabic
  # block for the vowels Arabic does not write -- U+06C7, U+06D5, U+06AD
  # here -- so the shaping is a little more demanding than Arabic alone.
  # Hebrew, which does not join, stays as the second script:
  # "shalom olam".
  hello <- "\u062e\u06c7\u0634 \u0643\u06d5\u0644\u062f\u0649\u06ad\u0649\u0632"
  more  <- "\u0628\u06c7 \u062f\u06c7\u0646\u064a\u0627\u063a\u0627"
  heb   <- "\u05e9\u05dc\u05d5\u05dd \u05e2\u05d5\u05dc\u05dd"

  # Only unwrapped rows here. Wrapped right-to-left is correct, but this
  # figure cannot show it: vdiffr's device measures a Perso-Arabic run
  # about twice as wide as the truth, so the words land at meaningless
  # offsets
  # and the picture records the device's font handling rather than ours.
  # The wrapped case is asserted on coordinates in test-text-runs.R,
  # which is unaffected by how wide the device thinks the words are.
  vdiffr::expect_doppelganger("text-direction", function() {
    # Every row is drawn twice: ours on the left, grid.text() on the
    # right. The device implements the bidirectional algorithm and we do
    # not -- we hand it a whole run and let it order the run. So the two
    # columns must read alike, and that is visible here rather than
    # inferred from coordinates.
    ref <- function(y, s) {
      grid::grid.text(s, x = grid::unit(380, "pt"),
                      y = grid::unit(1, "npc") - grid::unit(y + 13, "pt"),
                      hjust = 0, vjust = 1,
                      gp = grid::gpar(fontsize = 16))
    }
    colhead <- function(x, s) {
      grid::grid.text(s, x = grid::unit(x, "pt"),
                      y = grid::unit(1, "npc") - grid::unit(8, "pt"),
                      hjust = 0, vjust = 1,
                      gp = grid::gpar(fontsize = 8, col = "grey45"))
    }
    colhead(24, "gridmicrotex::grid.latex()")
    colhead(380, "grid::grid.text() reference")

    rows <- list(
      list(24,  "left-to-right (control)", "Hello world"),
      list(74,  "right-to-left, Uyghur", hello),
      list(124, "right-to-left, Hebrew", heb),
      # A right-to-left run inside a left-to-right sentence and the other
      # way round: the paragraph direction is resolved from the first
      # strong character, so these two must not come out alike.
      list(174, "right-to-left inside left-to-right",
           paste("The greeting", hello, "means welcome")),
      list(224, "left-to-right inside right-to-left",
           paste(hello, "and", more)),
      # Digits are their own class in the bidirectional algorithm -- they
      # run left to right inside a right-to-left line.
      list(274, "digits inside right-to-left",
           paste(hello, "2026", more))
    )
    for (r in rows) {
      place(24, r[[1]], r[[2]], paste0("\\text{", r[[3]], "}"), 16)
      ref(r[[1]], r[[3]])
    }
  })
})

test_that("string hjust/vjust resolve to the expected viewport just values", {
  g <- latex_grob("x^2", hjust = "left", vjust = "top")
  expect_equal(g$hjust, 0)
  expect_equal(g$vjust, 1)

  g <- latex_grob("x^2", hjust = "centre", vjust = "middle")
  expect_equal(g$hjust, 0.5)
  expect_equal(g$vjust, 0.5)

  g <- latex_grob("x^2", hjust = "bbright", vjust = "bottom")
  expect_equal(g$hjust, 1)
  expect_equal(g$vjust, 0)
})

test_that("vjust = 'baseline' aligns at bbox_bl_bp / bbox_h", {
  g <- latex_grob("x^2", vjust = "baseline")
  expect_equal(g$vjust, g$bbox_bl_bp / g$bbox_h)
  # baseline should sit below center for an expression with descent (e.g. \sum)
  g2 <- latex_grob(r"(\sum_{i=1}^n i)", vjust = "baseline")
  expect_gt(g2$vjust, 0)
  expect_lt(g2$vjust, 1)
})

test_that("invalid string just values error clearly", {
  expect_error(latex_grob("x", hjust = "wonky"), "hjust must be")
  expect_error(latex_grob("x", vjust = "wonky"), "vjust must be")
})

test_that("\\mark{name} records a queryable anchor", {
  g <- latex_grob(r"(a + \mark{eq} b = c)", x = grid::unit(0.5, "npc"),
                  y = grid::unit(0.5, "npc"))
  expect_s3_class(g$marks, "data.frame")
  expect_equal(nrow(g$marks), 1L)
  expect_equal(g$marks$name, "eq")
  mk <- grobMark(g, "eq")
  expect_true(grid::is.unit(mk$x))
  expect_true(grid::is.unit(mk$y))
})

test_that("grobMark errors when the grob has no marks", {
  g <- latex_grob("x + y")
  expect_error(grobMark(g, "eq"), "no marks")
})

test_that("grobMark errors with a clear message when the name is unknown", {
  g <- latex_grob(r"(\mark{foo} x)")
  expect_error(grobMark(g, "bar"), "not found.*foo")
})

test_that("multiple marks are stored in source order", {
  g <- latex_grob(r"(\mark{a} x + \mark{b} y)")
  expect_equal(nrow(g$marks), 2L)
  expect_equal(g$marks$name, c("a", "b"))
  expect_lt(g$marks$x[1], g$marks$x[2])
})

test_that("editGrob() preserves string-valued vjust across reparse", {
  g <- latex_grob("x^2", vjust = "baseline", gp = grid::gpar(fontsize = 12))
  bl1 <- g$vjust
  g2 <- grid::editGrob(g, gp = grid::gpar(fontsize = 24))
  # bbox changes, but vjust should still resolve "baseline" against the new bbox
  expect_equal(g2$vjust, g2$bbox_bl_bp / g2$bbox_h)
})

# Regression: \def with backslash control sequences in the body used to fail
# on subsequent parses because MicroTeX's static macro table leaked the name
# into the next parse's preprocess pass, which then mis-tokenised the body.
# A macro is only doing its job if the call lays out exactly as the body
# written out longhand. Asserting the class alone catches a parse error
# and nothing else -- an expansion that silently dropped its argument
# would still return a latexgrob.
expands_to <- function(macro, longhand, ...) {
  expect_equal(latex_grob(macro, ...)$layout_df,
               latex_grob(longhand, ...)$layout_df)
}

test_that("plain-TeX \\def with parameterised body expands to its body", {
  expands_to(r"(\def\norm#1{\left\lVert #1 \right\rVert} \norm{v})",
             r"(\left\lVert v \right\rVert)", input_mode = "math")
  expands_to(r"(\def\inner#1#2{\langle #1, #2 \rangle} \inner{u}{v})",
             r"(\langle u, v \rangle)", input_mode = "math")
  # The typeface path triggers a second internal parse for the path-mode
  # fallback layout — exercise it explicitly.
  expands_to(r"(\def\frob#1{|#1|} \frob{M})", r"(|M|)",
             render_mode = "typeface", input_mode = "math")
})

test_that("\\newcommand survives the typeface mode double-parse", {
  # The typeface mode parses twice (once for glyph layout, once for path
  # fallback). Before clearUserMacros() landed in parse_latex_cpp, the
  # second internal parse hit "Command already exists!" because
  # MicroTeX's static _codes map persisted the registration from the
  # first parse.
  expands_to(r"(\newcommand{\xyznorm}[1]{\lVert #1 \rVert} \xyznorm{v})",
             r"(\lVert v \rVert)", render_mode = "typeface",
             input_mode = "math")
})

test_that("\\textcolor body inherits the surrounding math/text mode", {
  # Regression: \textcolor used to force its body into text mode, so
  # `\textcolor{red}{c^2}` rendered "c^2" literally with a caret glyph
  # instead of a superscript. Now the body inherits math mode and the
  # ^ produces a real superscript record.
  g <- latex_grob(r"($\textcolor{blue}{c^2}$)", input_mode = "math")
  # The literal '^' should not appear as a TEXT record; if it did, the
  # body was parsed in text mode.
  layout <- g$layout_df
  has_caret_glyph <- any(layout$type == "text" &
                         vapply(layout$text, function(s) {
                           !is.na(s) && grepl("\\^", s)
                         }, logical(1)))
  expect_false(has_caret_glyph,
               info = "Caret '^' rendered as text — \\textcolor body lost math mode")
})

test_that("\\color declaration colours the rest of the enclosing group", {
  # Regression: \color outside array mode was a no-op, so
  # `\color{blue} E = mc^2` rendered black. Now it consumes the rest of
  # the enclosing group and wraps it in a ColorAtom.
  g <- latex_grob(r"($\color{blue} E = mc^2$)", input_mode = "math")
  cols <- unique(g$layout_df$color[g$layout_df$type %in% c("glyph", "path")])
  cols <- cols[!is.na(cols)]
  # At least one ink record must be blue-ish (#0000FF or its alpha variant).
  expect_true(any(grepl("^#0000FF", cols, ignore.case = TRUE)),
              info = paste("Expected blue ink, got:", paste(cols, collapse = ", ")))
})

test_that("\\newcommand and \\def do not leak between independent latex_grob calls", {
  src <- r"(\newcommand{\xyzleak}{Q} \xyzleak)"
  first <- latex_grob(src, input_mode = "math")$layout_df
  # A second call with the same definition must succeed -- it errored
  # "already exists" when state leaked between parses -- and must lay out
  # the same, which a re-registration that shadowed the body would not.
  expect_equal(latex_grob(src, input_mode = "math")$layout_df, first)
  expands_to(src, "Q", input_mode = "math")

  # And built-in environments must still be available after the leak guard
  # runs (it cleared _codes wholesale in an earlier iteration of the fix).
  pmat <- latex_grob(r"(\begin{pmatrix} 1 & 2 \\ 3 & 4 \end{pmatrix})",
                     input_mode = "math")
  # Four cells and a pair of delimiters, not the literal source text.
  expect_gte(nrow(pmat$layout_df), 4L)
  expect_false(any(pmat$layout_df$type == "text" &
                     grepl("pmatrix", pmat$layout_df$text, fixed = TRUE)))
})

# --- grobX / grobY boundary points ---

test_that("grobX/grobY cardinal points hit the bbox edges (rot = 0)", {
  grDevices::pdf(NULL, width = 8, height = 6)
  on.exit(grDevices::dev.off(), add = TRUE)
  grid::grid.newpage()

  g <- latex_grob("x^2", x = grid::unit(2, "in"), y = grid::unit(1.5, "in"),
                  hjust = 0, vjust = 0, gp = grid::gpar(fontsize = 20))
  w <- grid::convertWidth(grid::grobWidth(g), "inches", valueOnly = TRUE)
  h <- grid::convertHeight(grid::grobHeight(g), "inches", valueOnly = TRUE)

  bx <- function(th) grid::convertX(grid::grobX(g, th), "inches", valueOnly = TRUE)
  by <- function(th) grid::convertY(grid::grobY(g, th), "inches", valueOnly = TRUE)

  # Boundary point = ray from the bbox centre at angle theta.
  expect_equal(bx(0),   2 + w,     tolerance = 1e-6)   # east: right edge
  expect_equal(by(0),   1.5 + h/2, tolerance = 1e-6)   #       centre y
  expect_equal(bx(90),  2 + w/2,   tolerance = 1e-6)   # north: centre x
  expect_equal(by(90),  1.5 + h,   tolerance = 1e-6)   #        top edge
  expect_equal(bx(180), 2,         tolerance = 1e-6)   # west: left edge
  expect_equal(by(180), 1.5 + h/2, tolerance = 1e-6)
  expect_equal(bx(270), 2 + w/2,   tolerance = 1e-6)   # south: centre x
  expect_equal(by(270), 1.5,       tolerance = 1e-6)   #        bottom edge

  # Character theta as accepted by grobX/grobY
  expect_equal(grid::convertX(grid::grobX(g, "east"), "inches", valueOnly = TRUE),
               2 + w, tolerance = 1e-6)
  expect_equal(grid::convertY(grid::grobY(g, "north"), "inches", valueOnly = TRUE),
               1.5 + h, tolerance = 1e-6)

  # 45-degree ray leaves through the top edge (bbox is wider than tall)
  expect_equal(by(45), 1.5 + h, tolerance = 1e-6)
  expect_equal(bx(45), 2 + w/2 + h/2, tolerance = 1e-6)
})

test_that("grobX/grobY match a rectGrob with identical geometry across theta", {
  grDevices::pdf(NULL, width = 8, height = 6)
  on.exit(grDevices::dev.off(), add = TRUE)
  grid::grid.newpage()

  dims <- latex_dims("\frac{a}{b}", gp = grid::gpar(fontsize = 20))
  thetas <- c(0, 30, 45, 90, 135, 180, 225, 270, 315)

  for (just in list(c(0, 0), c(0.5, 0.5), c(1, 0.25))) {
    g <- latex_grob("\frac{a}{b}",
                    x = grid::unit(0.25, "npc"), y = grid::unit(0.4, "npc"),
                    hjust = just[1], vjust = just[2],
                    gp = grid::gpar(fontsize = 20))
    r <- grid::rectGrob(x = grid::unit(0.25, "npc"), y = grid::unit(0.4, "npc"),
                        width = dims$width, height = dims$height, just = just)
    for (th in thetas) {
      expect_equal(grid::convertX(grid::grobX(g, th), "in", valueOnly = TRUE),
                   grid::convertX(grid::grobX(r, th), "in", valueOnly = TRUE),
                   tolerance = 1e-6)
      expect_equal(grid::convertY(grid::grobY(g, th), "in", valueOnly = TRUE),
                   grid::convertY(grid::grobY(r, th), "in", valueOnly = TRUE),
                   tolerance = 1e-6)
    }
  }
})

test_that("grobX/grobY on a rotated grob match a rectGrob in the same viewport", {
  grDevices::pdf(NULL, width = 8, height = 6)
  on.exit(grDevices::dev.off(), add = TRUE)
  grid::grid.newpage()

  dims <- latex_dims("x^2", gp = grid::gpar(fontsize = 20))

  for (rot in c(37, 90, 180)) {
    g <- latex_grob("x^2", x = grid::unit(0.25, "npc"), y = grid::unit(0.4, "npc"),
                    hjust = 0, vjust = 0, rot = rot, gp = grid::gpar(fontsize = 20))
    # grid's reference behaviour for a grob whose rotation comes from its
    # viewport: theta is measured in the grob's own (rotated) frame.
    r <- grid::rectGrob(vp = grid::viewport(
      x = grid::unit(0.25, "npc"), y = grid::unit(0.4, "npc"),
      width = dims$width, height = dims$height,
      just = c(0, 0), angle = rot
    ))
    for (th in c(0, 45, 90, 180, 270)) {
      expect_equal(grid::convertX(grid::grobX(g, th), "in", valueOnly = TRUE),
                   grid::convertX(grid::grobX(r, th), "in", valueOnly = TRUE),
                   tolerance = 1e-6)
      expect_equal(grid::convertY(grid::grobY(g, th), "in", valueOnly = TRUE),
                   grid::convertY(grid::grobY(r, th), "in", valueOnly = TRUE),
                   tolerance = 1e-6)
    }
  }
})

# --- latex_grob creation and structure ---

test_that("latex_grob creates valid grob and returns correct dimensions", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g <- latex_grob("\\frac{x^{2}+1}{\\sqrt{y}}")
  expect_s3_class(g, "latexgrob")
  expect_true(nrow(g$layout_df) > 0)
  expect_true(g$bbox_w > 0)

  # render_mode stored correctly
  g_path <- latex_grob("x^2", render_mode = "path")
  g_type <- latex_grob("x^2", render_mode = "typeface")
  expect_equal(g_path$render_mode, "path")
  expect_null(g_path$path_layout_df)
  expect_s3_class(g_type$path_layout_df, "data.frame")

  # latex_dims with fontsize scaling
  dims <- latex_dims("\\frac{a}{b}", render_mode = "path")
  expect_true(grid::convertWidth(dims$width, "points", valueOnly = TRUE) > 0)
  w_small <- grid::convertWidth(latex_dims("x^2", gp = grid::gpar(fontsize = 10))$width,
                                "bigpts", valueOnly = TRUE)
  w_large <- grid::convertWidth(latex_dims("x^2", gp = grid::gpar(fontsize = 40))$width,
                                "bigpts", valueOnly = TRUE)
  expect_true(w_large > w_small)
})

# --- latex_grob parameters ---

test_that("latex_grob parameters work correctly", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # Rotation
  expect_equal(latex_grob("x^2", rot = 45)$vp$angle, 45)

  # max_width
  g_mw <- latex_grob("x^2 + y^2 = z^2", max_width = 50, render_mode = "path")
  expect_true(grid::convertWidth(grid::grobWidth(g_mw), "bigpts", valueOnly = TRUE) > 0)

  # makeContent builds children
  g_mc <- grid::makeContent(latex_grob("\\frac{a}{b}", render_mode = "path"))
  expect_true(length(g_mc$children) > 0)

  # width/height details positive
  g <- latex_grob("\\frac{a}{b}", render_mode = "path")
  expect_true(grid::convertWidth(grid::widthDetails(g), "bigpts", valueOnly = TRUE) > 0)
  expect_true(grid::convertHeight(grid::heightDetails(g), "bigpts", valueOnly = TRUE) > 0)
})

# --- device support and typeface rendering ---

test_that("device support detection and typeface fallback work", {
  # pdf reports glyphs=TRUE
  tf_pdf <- tempfile(fileext = ".pdf")
  grDevices::pdf(tf_pdf)
  expect_true(gridmicrotex:::.device_supports_typeface_glyphs())

  # PDF: renders without fallback warning
  expect_no_warning({
    g <- latex_grob("\\frac{a}{b}", render_mode = "typeface", gp = grid::gpar(fontsize = 20))
    grid::grid.newpage()
    grid::grid.draw(g)
  })
  grDevices::dev.off()
  unlink(tf_pdf)

  # Postscript: falls back with warning
  tf_ps <- tempfile(fileext = ".ps")
  grDevices::postscript(tf_ps)
  gridmicrotex:::.clear_typeface_noted()
  on.exit({ grDevices::dev.off(); unlink(tf_ps) }, add = TRUE)
  expect_false(gridmicrotex:::.device_supports_typeface_glyphs())
  expect_message({
    g <- latex_grob("\\frac{a}{b}", render_mode = "typeface",
                    gp = grid::gpar(fontsize = 20))
    grid::grid.newpage()
    grid::grid.draw(g)
  }, "falling back to path mode")
  # The fallback has to substitute the path layout, not merely warn: the
  # children it draws must be outlines, not glyphs the device cannot set.
  kids <- suppressMessages(grid::makeContent(g))$children
  expect_true("pathgrob" %in% vapply(kids, function(k) class(k)[1], ""))
})

# --- edge cases: empty and invalid input ---

test_that("latex_grob handles empty input as a zero-size grob", {
  g <- latex_grob("")
  expect_s3_class(g, "latexgrob")
  expect_equal(g$bbox_w, 0)
  expect_equal(g$bbox_h, 0)
  expect_equal(nrow(g$layout_df), 0L)
})

test_that("an unknown command is set as its own name, not dropped", {
  # MicroTeX is lenient rather than strict: \notavalidcommand comes out as
  # the letters of its name followed by its argument. Rendering nothing
  # would hide a typo completely, so assert the name is actually drawn.
  g <- latex_grob("\\notavalidcommand{x}", input_mode = "math")
  expect_gte(nrow(g$layout_df), nchar("notavalidcommand"))
  expect_gt(g$bbox_w, 0)
})

# --- editGrob: re-parse on parse-affecting fields ---

test_that("editGrob re-parses when tex changes", {
  g <- latex_grob("x", render_mode = "path")
  g2 <- grid::editGrob(g, tex = "x^{2} + y^{2} + z^{2}")
  expect_equal(g2$tex, "x^{2} + y^{2} + z^{2}")
  expect_true(g2$bbox_w > g$bbox_w)
  # That the layout is genuinely rebuilt, without assuming how many
  # records a given string produces -- consecutive text is drawn as one
  # run when nothing wraps, so record count is not a proxy for length.
  expect_false(identical(g2$layout_df, g$layout_df))
  # viewport width/height tracked bbox
  expect_equal(
    as.numeric(g2$vp$width),
    as.numeric(grid::unit(g2$bbox_w, "bigpts"))
  )
})

test_that("editGrob re-parses when gp (fontsize) changes", {
  g20 <- latex_grob("x^2", render_mode = "path", gp = grid::gpar(fontsize = 20))
  g40 <- grid::editGrob(g20, gp = grid::gpar(fontsize = 40))
  expect_equal(g40$fontsize, 40)
  # 2x font -> ~2x bbox
  expect_equal(g40$bbox_w / g20$bbox_w, 2, tolerance = 0.05)
  # gp is re-stripped after parse (fontsize not carried on the gTree)
  expect_null(g40$gp$fontsize)
})

test_that("editGrob re-parses when math_font / tex_style / render_mode change", {
  g <- latex_grob("\\frac{a}{b}", render_mode = "path", math_font = "lete")
  g2 <- grid::editGrob(g, math_font = "stix")
  expect_false(identical(g$layout_df, g2$layout_df))

  g3 <- latex_grob("\\sum_{i=1}^{n} i", render_mode = "path", tex_style = "text")
  g4 <- grid::editGrob(g3, tex_style = "display")
  expect_true(g4$bbox_h > g3$bbox_h)  # display makes \sum taller

  g5 <- latex_grob("x^2", render_mode = "path")
  g6 <- grid::editGrob(g5, render_mode = "typeface")
  expect_equal(g6$render_mode, "typeface")
  expect_s3_class(g6$path_layout_df, "data.frame")  # fallback layout generated
})

test_that("editGrob on non-parse fields does not re-parse", {
  g <- latex_grob("\\frac{a}{b}", render_mode = "path")
  orig_layout <- g$layout_df
  g2 <- grid::editGrob(g, debug = TRUE)
  expect_true(isTRUE(g2$debug))
  expect_identical(g2$layout_df, orig_layout)
})

test_that("ascentDetails + descentDetails sum to heightDetails", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g <- latex_grob("\\frac{a}{b}", render_mode = "path", gp = grid::gpar(fontsize = 24))
  asc  <- grid::convertHeight(grid::ascentDetails(g),  "bigpts", valueOnly = TRUE)
  desc <- grid::convertHeight(grid::descentDetails(g), "bigpts", valueOnly = TRUE)
  h    <- grid::convertHeight(grid::heightDetails(g),  "bigpts", valueOnly = TRUE)
  expect_equal(asc + desc, h, tolerance = 1e-6)
  expect_true(asc > 0)
  expect_true(desc >= 0)
  # Descent matches the bbox_d field exposed for grob-to-grob alignment
  expect_equal(desc, g$bbox_d, tolerance = 1e-6)
})

test_that("editGrob keeps viewport just in sync with hjust/vjust", {
  g <- latex_grob("x", render_mode = "path", hjust = 0.5, vjust = 0.5)
  g2 <- grid::editGrob(g, hjust = 0, vjust = 1)
  expect_equal(g2$hjust, 0)
  expect_equal(g2$vjust, 1)
  expect_equal(as.numeric(g2$vp$valid.just), c(0, 1))
})

test_that("latex_dims respects math_font parameter", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  expr <- "$\\int_0^1 f(x)\\,dx + x + y$"
  dims_lete <- latex_dims(expr, math_font = "lete", gp = grid::gpar(fontsize = 20))
  dims_stix <- latex_dims(expr, math_font = "stix", gp = grid::gpar(fontsize = 20))
  w_lete <- grid::convertWidth(dims_lete$width, "bigpts", valueOnly = TRUE)
  w_stix <- grid::convertWidth(dims_stix$width, "bigpts", valueOnly = TRUE)
  expect_true(w_lete > 0)
  expect_true(w_stix > 0)
  expect_false(w_lete == w_stix)
})

# --- \def command ---

test_that("\\def defines a zero-argument macro and renders identically to \\newcommand", {
  # \def\mymacroA{x^2} should produce the same layout as \newcommand{\mymacroB}{x^2}
  layout_nc  <- parse_latex_cpp("\\newcommand{\\mymacroB}{x^2} \\mymacroB", text_size = 20)
  layout_def <- parse_latex_cpp("\\def\\mymacroA{x^2} \\mymacroA",           text_size = 20)
  expect_equal(nrow(layout_def), nrow(layout_nc))
})

test_that("\\def silently overwrites an existing macro, and the last wins", {
  # \newcommand errors on redefinition; \def replaces. Asserting only that
  # it does not error would pass if the *first* body were kept.
  got <- parse_latex_cpp(
    "\\def\\myoverwrite{x} \\def\\myoverwrite{y} \\myoverwrite", text_size = 20)
  expect_equal(got, parse_latex_cpp("y", text_size = 20))
  expect_false(isTRUE(all.equal(got, parse_latex_cpp("x", text_size = 20))))
})

test_that("\\def with invalid control sequence name throws a parse error", {
  expect_error(
    parse_latex_cpp("\\def{notacontrolseq}{body}", text_size = 20)
  )
})

test_that("\\def with sequential #1..#N parameters expands like \\newcommand[N]", {
  # Single-arg form: \def\sq#1{#1^2} should match \newcommand{\sq}[1]{#1^2}
  layout_nc1  <- parse_latex_cpp("\\newcommand{\\sqB}[1]{#1^2} \\sqB{a}", text_size = 20)
  layout_def1 <- parse_latex_cpp("\\def\\sqA#1{#1^2} \\sqA{a}",           text_size = 20)
  expect_equal(nrow(layout_def1), nrow(layout_nc1))

  # Two-arg form: \def\pair#1#2{#1+#2}
  layout_nc2  <- parse_latex_cpp("\\newcommand{\\pairB}[2]{#1+#2} \\pairB{a}{b}", text_size = 20)
  layout_def2 <- parse_latex_cpp("\\def\\pairA#1#2{#1+#2} \\pairA{a}{b}",         text_size = 20)
  expect_equal(nrow(layout_def2), nrow(layout_nc2))
})

test_that("\\def rejects non-sequential or malformed parameter patterns", {
  # Out-of-order parameters: #2 before #1
  expect_error(
    parse_latex_cpp("\\def\\bad#2#1{#1#2}", text_size = 20)
  )
  # Skipping a parameter: #1 then #3
  expect_error(
    parse_latex_cpp("\\def\\skip#1#3{#1#3}", text_size = 20)
  )
  # '#' not followed by a digit
  expect_error(
    parse_latex_cpp("\\def\\noarg#x{x}", text_size = 20)
  )
})

test_that("the typeface fallback warns once per device, not once per grob", {
  # The condition belongs to the device, but makeContent() runs per grob and
  # on every redraw, so a figure with several labels used to raise the same
  # warning once per label.
  tf <- tempfile(fileext = ".ps")
  grDevices::postscript(tf)
  on.exit({ grDevices::dev.off(); unlink(tf) }, add = TRUE)
  gridmicrotex:::.clear_typeface_noted()

  draw <- function() {
    g <- latex_grob("\frac{a}{b}", render_mode = "typeface",
                    gp = grid::gpar(fontsize = 20))
    grid::grid.newpage()
    grid::grid.draw(g)
  }
  w <- character(0)
  withCallingHandlers(
    for (i in 1:5) draw(),
    message = function(cond) {
      w <<- c(w, conditionMessage(cond))
      invokeRestart("muffleMessage")
    }
  )
  fallback <- grep("falling back to path mode", w, value = TRUE)
  expect_length(fallback, 1L)

  # A different device is a different answer, so it is told too.
  tf2 <- tempfile(fileext = ".ps")
  grDevices::postscript(tf2)
  w2 <- character(0)
  withCallingHandlers(draw(), message = function(cond) {
    w2 <<- c(w2, conditionMessage(cond)); invokeRestart("muffleMessage")
  })
  grDevices::dev.off(); unlink(tf2)
  expect_length(grep("falling back to path mode", w2, value = TRUE), 1L)
})

test_that("the fallback is reported only when typeface was asked for", {
  # render_mode defaults to "typeface", so everyone lands in the fallback on
  # a device without glyphs. Reporting that would reach users who never
  # expressed a preference; only an unmet *explicit* request is worth a word.
  msgs <- function(expr) {
    got <- character(0)
    withCallingHandlers(expr, message = function(cond) {
      got <<- c(got, conditionMessage(cond)); invokeRestart("muffleMessage")
    })
    grep("falling back to path mode", got, value = TRUE)
  }
  on_ps <- function(expr) {
    f <- tempfile(fileext = ".ps")
    grDevices::postscript(f)
    on.exit({ grDevices::dev.off(); unlink(f) }, add = TRUE)
    gridmicrotex:::.clear_typeface_noted()
    msgs({ grid::grid.newpage(); expr() })
  }

  # asked for it explicitly -> told once
  expect_length(on_ps(function()
    grid.latex("x^2", render_mode = "typeface", gp = grid::gpar(fontsize = 20))), 1L)

  # inherited the default -> silent
  expect_length(on_ps(function()
    grid.latex("x^2", gp = grid::gpar(fontsize = 20))), 0L)

  # set globally via latex_options() counts as asking
  old <- latex_options(render_mode = "typeface")
  on.exit(reset_latex_options(), add = TRUE)
  expect_length(on_ps(function()
    grid.latex("x^2", gp = grid::gpar(fontsize = 20))), 1L)
})

# Named "zz" so testthat runs it last: it tears MicroTeX down and lets it
# re-initialise, which is not something the other files should inherit
# halfway through.

test_that("a release / re-init cycle leaves the macro registry usable", {
  # MicroTeX::release() is MacroInfo::_free_() + NewCommandMacro::_free_(),
  # i.e. final teardown: it empties the macro registries, which are only
  # ever repopulated from inside MicroTeX::init(). microtex_release() is
  # not paired with a re-init, so it must leave them alone and drop only
  # per-session state -- see src/init.cpp. (Historically _free_() deleted
  # every value in the static _commands map without erasing it, so the
  # next MacroInfo::add() double-freed a built-in; that is fixed at the
  # source now, but calling it from here would still strand the parser
  # without \frac and friends.)
  pdf(NULL)
  on.exit(dev.off(), add = TRUE)

  big <- paste0(rep("\\text{x}", 254), collapse = "")
  before <- nrow(latex_grob(big, input_mode = "math")$layout_df)
  expect_gt(before, 0)

  gridmicrotex:::microtex_release()
  # Force a genuine re-parse; a cache hit would prove nothing.
  latex_cache_clear()

  # Our own macros still resolve...
  expect_equal(
    nrow(latex_grob("\\gmfontfamily{A}{x}", input_mode = "math")$layout_df),
    1L
  )
  # \mark is ours, registered at init. After a release it must still be
  # recognised: the anchor is recorded and the marker itself draws
  # nothing, so the layout is exactly that of the text without it.
  marked <- latex_grob("a\\mark{m}b", input_mode = "math")
  expect_equal(marked$marks$name, "m")
  expect_equal(nrow(marked$layout_df),
               nrow(latex_grob("ab", input_mode = "math")$layout_df))
  expect_equal(
    .resolve_text_family(
      latex_grob("\\textsf{\\textrm{a}}", input_mode = "math")$layout_df$font_style[1],
      default = "BODY", family = "gridmicrotex.default"
    ),
    "BODY"
  )

  # ...and the formula that used to blow up still parses identically.
  expect_equal(nrow(latex_grob(big, input_mode = "math")$layout_df), before)

  # The p{} column type is registered by the vendored spec parser, not by
  # a macro, so it should be unaffected by a release -- assert it rather
  # than assume it.
  wide <- "\\begin{tabular}{p{3cm}}\\text{wrap me over several lines}\\end{tabular}"
  free <- "\\begin{tabular}{l}\\text{wrap me over several lines}\\end{tabular}"
  d <- latex_dims(wide, input_mode = "math", gp = grid::gpar(fontsize = 16))
  # p{3cm} still constrains the column: narrower than the same cell in a
  # free column, and tall enough to have wrapped onto more than one line.
  expect_lt(as.numeric(d$width),
            as.numeric(latex_dims(free, input_mode = "math",
                                  gp = grid::gpar(fontsize = 16))$width))
  expect_gt(as.numeric(d$height), 20)
})

test_that("unloading the namespace unloads the DLL, and a reload still parses", {
  # This has to run in a subprocess: the package under test cannot unload
  # itself out from under testthat. Two things are asserted, and the first
  # is the one that silently regressed before -- .onUnload() used to call
  # microtex_release() only, so R kept the shared object mapped and
  # R_unload_gridmicrotex() never ran.
  skip_on_cran()
  rscript <- file.path(R.home("bin"), "Rscript")
  skip_if(
    !any(file.exists(rscript, paste0(rscript, ".exe"))),
    "Rscript not found next to this R"
  )

  probe <- tempfile(fileext = ".R")
  writeLines(c(
    'pdf(NULL)',
    'library(gridmicrotex)',
    'tex <- "\\\\frac{1}{2} + x^2"',
    'before <- nrow(gridmicrotex:::latex_grob(tex, input_mode = "math")$layout_df)',
    'unloadNamespace("gridmicrotex")',
    'cat("GM-DLL-MAPPED:", "gridmicrotex" %in% names(getLoadedDLLs()), "\\n")',
    'library(gridmicrotex)',
    'gridmicrotex::latex_cache_clear()',
    'after <- nrow(gridmicrotex:::latex_grob(tex, input_mode = "math")$layout_df)',
    'cat("GM-ROWS:", before, after, "\\n")',
    # Our own macros are registered behind s_registered guards in
    # atom/font_family_atom.cpp / atom/mark_atom.cpp, while the built-ins come back
    # via MicroTeX::init(). If a teardown ever clears the macro table
    # without also resetting those guards, these are what break.
    'ff <- nrow(gridmicrotex:::latex_grob("\\\\gmfontfamily{mono}{x}",',
    '                                     input_mode = "math")$layout_df)',
    'mk <- gridmicrotex:::latex_grob("a\\\\mark{m}b", input_mode = "math")',
    'cat("GM-OURS:", ff, if (is.null(mk$marks)) "NA" else mk$marks$name, "\\n")'
  ), probe)
  on.exit(unlink(probe), add = TRUE)

  # system2(env=) is a no-op on Windows, so set it here and inherit.
  old <- Sys.getenv("R_LIBS", unset = NA)
  Sys.setenv(R_LIBS = paste(.libPaths(), collapse = .Platform$path.sep))
  on.exit(
    if (is.na(old)) Sys.unsetenv("R_LIBS") else Sys.setenv(R_LIBS = old),
    add = TRUE
  )

  out <- suppressWarnings(
    system2(rscript, c("--vanilla", shQuote(probe)), stdout = TRUE, stderr = TRUE)
  )
  info <- paste(out, collapse = "\n")

  mapped <- grep("^GM-DLL-MAPPED:", out, value = TRUE)
  rows <- grep("^GM-ROWS:", out, value = TRUE)
  expect_length(mapped, 1L)
  expect_length(rows, 1L)

  # The DLL is gone after unloadNamespace(), which is what makes
  # R_unload_gridmicrotex() reachable at all.
  expect_equal(trimws(sub("^GM-DLL-MAPPED:", "", mapped)), "FALSE", info = info)

  # And the reload re-registers the built-ins: \frac parses to the same
  # number of draw records as it did before the teardown.
  n <- as.integer(strsplit(trimws(sub("^GM-ROWS:", "", rows)), " +")[[1]])
  expect_gt(n[1], 0L)
  expect_equal(n[2], n[1], info = info)

  # ...and so do ours: \gmfontfamily lays out its one glyph, and \mark
  # still records its anchor rather than being parsed as literal text.
  ours <- grep("^GM-OURS:", out, value = TRUE)
  expect_length(ours, 1L)
  parts <- strsplit(trimws(sub("^GM-OURS:", "", ours)), " +")[[1]]
  expect_equal(as.integer(parts[1]), 1L, info = info)
  expect_equal(parts[2], "m", info = info)
})

test_that("latex_wrap converts correctly", {
  expect_equal(latex_wrap("It is $x^2$"),
               "\\text{It is }x^2")
  expect_equal(latex_wrap("\\(E=mc^2\\) is famous"),
               "E=mc^2\\text{ is famous}")
  expect_equal(latex_wrap("plain text"),
               "\\text{plain text}")
  expect_equal(latex_wrap("cost \\$5 for $x$"),
               "\\text{cost \\$5 for }x")
  expect_equal(latex_wrap("Line 1\nLine 2"),
               "\\text{Line 1}\\\\\\text{Line 2}")
})

test_that("latex_wrap is vectorized over its input", {
  # Documented contract: vector in, vector of the same length out.
  expect_equal(
    latex_wrap(c("plain text", "It is $x^2$", "")),
    c("\\text{plain text}", "\\text{It is }x^2", "")
  )
  # math mode bypasses every element unmodified
  expect_equal(
    latex_wrap(c("\\frac{a}{b}", "x^2"), input_mode = "math"),
    c("\\frac{a}{b}", "x^2")
  )
  expect_equal(latex_wrap(character(0)), character(0))
})

test_that("latex_wrap rejects NA input", {
  expect_error(latex_wrap(NA_character_), "must not contain NA")
  expect_error(latex_wrap(c("a", NA, "b")), "must not contain NA")
  expect_error(latex_wrap(NA_character_, input_mode = "math"),
               "must not contain NA")
})

test_that(".find_close_brace balances nested and escaped braces", {
  fcb <- gridmicrotex:::.find_close_brace
  expect_equal(fcb("a}b", 1L), 2L)
  expect_equal(fcb("a{b}c}d", 1L), 6L)         # nested {} skipped
  expect_equal(fcb("a\\}b}c", 1L), 5L)         # escaped \} skipped
  expect_true(is.na(fcb("a{b}c", 1L)))         # unbalanced
})

test_that(".strip_document_wrappers handles comments and floats", {
  strip <- gridmicrotex:::.strip_document_wrappers

  expect_equal(strip("% leading comment\n$x^2$"), "$x^2$")
  expect_equal(strip("$50\\% + x^2$"), "$50\\% + x^2$")

  src <- "\\begin{table}[ht]\n\\centering\n\\begin{tabular}{rr} a & b \\\\ \\end{tabular}\n\\end{table}"
  out <- strip(src)
  expect_false(grepl("\\\\begin\\{table",  out))
  expect_false(grepl("\\\\end\\{table",    out))
  expect_false(grepl("\\\\centering",      out))
  expect_true( grepl("\\\\begin\\{tabular\\}", out))

  # figure floats + starred variants
  expect_false(grepl("\\\\begin\\{figure",
                     strip("\\begin{figure*}[ht]x\\end{figure*}")))
  expect_false(grepl("\\\\begin\\{table",
                     strip("\\begin{table*}[ht]x\\end{table*}")))
})

test_that(".strip_document_wrappers removes preamble and title metadata", {
  strip <- gridmicrotex:::.strip_document_wrappers

  src <- "\\documentclass[12pt]{article}\\usepackage[utf8]{inputenc}\\usepackage{amsmath}\\begin{document}x^2\\end{document}"
  out <- strip(src)
  expect_false(grepl("documentclass", out))
  expect_false(grepl("usepackage",    out))
  expect_false(grepl("begin\\{document",  out))
  expect_false(grepl("end\\{document",    out))
  expect_true( grepl("x\\^2", out))

  out2 <- strip("\\maketitle\\title{Foo}\\author{Bar}\\label{eq:1}content")
  expect_equal(out2, "content")

  # alignment scope declarations
  for (cmd in c("centering", "raggedright", "raggedleft",
                "flushleft", "flushright")) {
    expect_false(grepl(cmd, strip(paste0("\\", cmd, " x"))))
  }
})

test_that("booktabs rules alias correctly", {
  strip <- gridmicrotex:::.strip_document_wrappers

  # top/bottom rules render thicker; middle stays normal
  expect_equal(strip("\\toprule a\\midrule b\\bottomrule"),
               "\\thickhline a\\hline b\\thickhline")

  # \cmidrule keeps its column range via \cline (no longer lossy)
  expect_equal(strip("\\cmidrule{2-3}"),          "\\cline{2-3}")
  expect_equal(strip("\\cmidrule(lr){2-3}"),      "\\cline{2-3}")
  expect_equal(strip("\\cmidrule[2pt]{2-3}"),     "\\cline{2-3}")
  expect_equal(strip("\\cmidrule[2pt](lr){2-3}"), "\\cline{2-3}")
})

test_that("Tier 1 aliases rewrite to MicroTeX-supported commands", {
  strip <- gridmicrotex:::.strip_document_wrappers

  expect_equal(strip("\\emph{stress}"),       "\\textit{stress}")
  expect_equal(strip("\\textnormal{plain}"),  "\\text{plain}")
  expect_equal(strip("a\\par b"),             "a\\\\ b")
  expect_equal(strip("a\\newline b"),         "a\\\\ b")

  # nested braces in argument
  expect_equal(strip("\\emph{a $x_{i}$ b}"),
               "\\textit{a $x_{i}$ b}")
})

test_that("content-free declarations are stripped", {
  strip <- gridmicrotex:::.strip_document_wrappers
  for (cmd in c("noindent", "relax")) {
    expect_false(grepl(cmd, strip(paste0("\\", cmd, " x"))))
  }
})

test_that("skip and fill commands map to em-relative \\vspace / \\quad", {
  strip <- gridmicrotex:::.strip_document_wrappers

  expect_equal(strip("a\\smallskip b"), "a\\vspace{0.25em} b")
  expect_equal(strip("a\\medskip b"),   "a\\vspace{0.5em} b")
  expect_equal(strip("a\\bigskip b"),   "a\\vspace{1em} b")
  expect_equal(strip("a\\hfill b"),     "a\\quad b")
  expect_equal(strip("a\\vfill b"),     "a\\vspace{1em} b")

  # mapped commands render end-to-end (MicroTeX has \vspace and \quad)
  d <- latex_dims("a\\bigskip b")
  expect_gt(as.numeric(d$width),  0)
  expect_gt(as.numeric(d$height), 0)
})

test_that("\\caption{X} is extracted as inline \\text{X}\\\\", {
  strip <- gridmicrotex:::.strip_document_wrappers

  expect_equal(strip("\\caption{Hello}"),     "\\text{Hello}\\\\")
  expect_equal(strip("a \\caption{Hi} b"),    "a \\text{Hi}\\\\ b")
  expect_equal(strip("\\caption[short]{Long}"), "\\text{Long}\\\\")

  # nested braces in caption content (e.g. inline subscripts)
  expect_equal(strip("\\caption{Foo $x_{i}$}"),
               "\\text{Foo $x_{i}$}\\\\")
})

test_that("every break lands at the formula level, not inside the text", {
  # A break kept inside the \text{} makes a multi-line text *box*, and a
  # box is one item in the enclosing row -- so anything after it was set
  # beside the whole box rather than on the line it was written on.
  expect_equal(latex_wrap("Title\n$x^2$"), "\\text{Title}\\\\x^2")
  expect_equal(latex_wrap("Caption\n\\begin{array}{l}a\\end{array}"),
               "\\text{Caption}\\\\\\begin{array}{l}a\\end{array}")
  # Interior breaks too, which is what puts trailing math on the right
  # line instead of centring it between the two.
  expect_equal(latex_wrap("Line 1\nLine 2"), "\\text{Line 1}\\\\\\text{Line 2}")
  expect_equal(latex_wrap("a\nb $x$"), "\\text{a}\\\\\\text{b }x")
  # A literal \\ is a break like any other.
  expect_equal(latex_wrap("a\\\\b"), "\\text{a}\\\\\\text{b}")
  # Nothing but breaks between two math spans is a separator, not text.
  expect_equal(latex_wrap("$a$\n$b$"), "a\\\\b")
  # A break with nothing on one side of it would only add a blank first
  # or last row.
  expect_equal(latex_wrap("\n\nTitle\n"), "\\text{Title}")
  # A run collapses to one, the way consecutive blank lines do in LaTeX.
  expect_equal(latex_wrap("a\n\n\n$x$"), "\\text{a}\\\\x")
  # No break, no change.
  expect_equal(latex_wrap("It is $x^2$"), "\\text{It is }x^2")
  expect_equal(latex_wrap(" "), "\\text{ }")
})

test_that("splitting a prose run does not move it", {
  # The split is only free if the two forms lay out identically.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  xy <- function(tex) {
    d <- latex_grob(tex, input_mode = "math",
                    gp = grid::gpar(fontsize = 14))$layout_df
    d[, c("x", "y")]
  }
  expect_equal(xy("\\text{line one}\\\\\\text{line two}"),
               xy("\\text{line one \\\\line two}"))
})

test_that("a pasted caption sits above its table, not beside it", {
  # End to end, because the regression was invisible in the wrapped string
  # alone: \caption becomes \text{X}\\, and in mixed mode that trailing
  # break used to be swallowed into the surrounding \text{}, leaving the
  # caption as a box to the LEFT of the tabular.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  snippet <- paste(
    "\\begin{table}[ht]", "\\centering", "\\caption{Model coefficients}",
    "\\begin{tabular}{lr}", "\\toprule", "Term & Estimate \\\\",
    "\\midrule", "Intercept & 2.14 \\\\", "\\bottomrule",
    "\\end{tabular}", "\\end{table}", sep = "\n")

  d <- latex_grob(snippet, input_mode = "mixed",
                  gp = grid::gpar(fontsize = 11))$layout_df
  cap <- d[!is.na(d$text) & grepl("Model", d$text), ]
  rules <- d[d$type == "line", ]
  expect_equal(nrow(cap), 1L)
  # Above: the caption's baseline is higher up the page than the top rule
  # (y grows downward), and it starts at the same left edge.
  expect_lt(cap$y, min(rules$y))
  expect_equal(cap$x, min(rules$x))
})

test_that("strip preserves legitimate MicroTeX math commands", {
  strip <- gridmicrotex:::.strip_document_wrappers

  # Size commands (mac(0, macro_sizes, ...)) and \multirow stay intact.
  passthrough <- c(
    "\\sum_{i=1}^n x_i",
    "\\int_0^\\infty e^{-x} dx",
    "\\alpha + \\beta",
    "\\frac{a}{b}",
    "\\small \\text{tiny note}",
    "\\Large \\text{title-ish}",
    "\\multirow{2}{*}{a}",
    "\\hline\\hline"
  )
  for (s in passthrough) expect_equal(strip(s), s)
})

test_that("latex_grob renders raw xtable output end-to-end", {
  src <- paste(
    "% latex table generated by xtable",
    "\\begin{table}[ht]",
    "\\centering",
    "\\begin{tabular}{rrr} \\hline a & b & c \\\\ \\hline 1 & 2 & 3 \\\\ \\hline \\end{tabular}",
    "\\caption{Cox fit}",
    "\\label{tab:cox}",
    "\\end{table}",
    sep = "\n"
  )
  d <- latex_dims(src)
  expect_gt(as.numeric(d$width),  0)
  expect_gt(as.numeric(d$height), 0)
})

test_that("latex_grob renders kable booktabs-style tabular", {
  src <- "\\begin{tabular}{lr}\\toprule x & y \\\\ \\midrule 1 & 2 \\\\ 3 & 4 \\\\ \\bottomrule \\end{tabular}"
  d <- latex_dims(src)
  expect_gt(as.numeric(d$width),  0)
  expect_gt(as.numeric(d$height), 0)
})

test_that("starred env variants are normalized to non-starred", {
  strip <- gridmicrotex:::.strip_document_wrappers
  expect_equal(strip("\\begin{align*}a&=b\\end{align*}"),
               "\\begin{align}a&=b\\end{align}")
  expect_equal(strip("\\begin{alignat*}{2}a&=b\\end{alignat*}"),
               "\\begin{alignat}{2}a&=b\\end{alignat}")

  # End-to-end: starred align/eqnarray previously errored on '&' because
  # MicroTeX didn't recognise the env and never entered array mode.
  alignment_cases <- list(
    "\\begin{align*} a &= b \\end{align*}",
    "\\begin{eqnarray*} a & = & b \\end{eqnarray*}",
    "\\begin{flalign*} a &= b \\end{flalign*}",
    "\\begin{alignat*}{2} a & = b & c & = d \\end{alignat*}",
    "\\begin{alignedat*}{2} a & = b & c & = d \\end{alignedat*}"
  )
  for (src in alignment_cases) {
    d <- latex_dims(src)
    expect_gt(as.numeric(d$width), 0)
  }
})

test_that("latex_wrap passes every MicroTeX-registered env verbatim", {
  # These are the environments registered in
  # src/MicroTeX/lib/macro/macro_def.cpp; mixed mode must not wrap them
  # in \text{} or split them at $-delimiters.
  envs <- c("array", "tabular", "matrix", "smallmatrix",
            "pmatrix", "bmatrix", "Bmatrix", "vmatrix", "Vmatrix",
            "eqnarray", "align", "flalign", "alignat", "aligned",
            "alignedat", "multline", "cases", "rcases", "split",
            "gather", "gathered", "math", "displaymath", "equation")
  for (e in envs) {
    src <- sprintf("\\begin{%s}a & b\\end{%s}", e, e)
    out <- latex_wrap(src, input_mode = "mixed")
    expect_false(grepl("\\\\text\\{", out),
                 info = sprintf("env '%s' got wrapped in \\text{}", e))
    expect_equal(out, src, info = sprintf("env '%s' was modified", e))
  }
})

test_that("tabular is treated as a math environment in mixed mode", {
  # Regression: `$...$` inside cells used to chop the tabular into text
  # chunks because latex_wrap didn't know `tabular` was an array env.
  src <- "\\begin{tabular}{rr}a & Pr($>$F) \\\\ 1 & 2 \\end{tabular}"
  out <- latex_wrap(src, input_mode = "mixed")
  # The whole tabular block should pass through verbatim, not be wrapped
  # in \text{} or split at the $ toggles.
  expect_false(grepl("\\\\text\\{", out))
  expect_equal(out, src)

  # End-to-end: an xtable-style table with $-delimited math in a header
  # renders without collapsing into a single column.
  d <- latex_dims(src)
  expect_gt(as.numeric(d$width),  50)
  expect_gt(as.numeric(d$height), 0)
})

test_that("MicroTeX renders \\thickhline and \\cline natively", {
  d1 <- latex_dims("\\begin{array}{cc}\\thickhline a & b \\\\ \\thickhline\\end{array}")
  expect_gt(as.numeric(d1$width),  0)
  expect_gt(as.numeric(d1$height), 0)

  d2 <- latex_dims("\\begin{array}{cccc}a & b & c & d \\\\ \\cline{2-3} 1 & 2 & 3 & 4\\end{array}")
  expect_gt(as.numeric(d2$width),  0)
  expect_gt(as.numeric(d2$height), 0)

  # single-column form: \cline{2} == \cline{2-2}
  d3 <- latex_dims("\\begin{array}{cc}a & b \\\\ \\cline{2}\\end{array}")
  expect_gt(as.numeric(d3$width), 0)
})

test_that("\\cmidrule renders end-to-end via \\cline", {
  src <- "\\begin{tabular}{lrr}\\toprule x & y & z \\\\ \\cmidrule(lr){2-3} 1 & 2 & 3 \\\\ \\bottomrule \\end{tabular}"
  d <- latex_dims(src)
  expect_gt(as.numeric(d$width),  0)
  expect_gt(as.numeric(d$height), 0)
})

test_that("strip emits no messages — all transforms are silent", {
  msgs <- testthat::capture_messages(
    latex_dims("\\caption{X}\\usepackage{amsmath}\\toprule x \\bottomrule")
  )
  expect_equal(length(msgs), 0L)
})

test_that("links are styled, following LaTeX's convention not HTML's", {
  strip <- gridmicrotex:::.strip_document_wrappers
  col <- gridmicrotex:::.MD_LINK_COLOR

  # hyperref with colorlinks=true colours a link and does not underline it;
  # the `url` package sets a URL in monospace. The markdown side uses the
  # HTML convention (blue AND underlined) instead -- see the `a` rule.
  expect_equal(strip("\\href{https://ex.org}{the text}"),
               paste0("\\textcolor{", col, "}{the text}"))
  expect_equal(strip("\\url{https://ex.org}"),
               paste0("\\textcolor{", col, "}{\\texttt{https://ex.org}}"))

  # A URL is verbatim in real LaTeX, so its specials are escaped: an
  # unescaped _ would open a subscript.
  expect_equal(strip("\\url{a.io/x_y}"),
               paste0("\\textcolor{", col, "}{\\texttt{a.io/x\\_y}}"))

  # Links are rewritten before the comment stripper, so a % inside a URL
  # survives as \% instead of eating the rest of the line.
  expect_equal(strip("\\url{a.io/x%20y} tail"),
               paste0("\\textcolor{", col, "}{\\texttt{a.io/x\\%20y}} tail"))

  # \href text is ordinary LaTeX and keeps its markup.
  expect_equal(strip("\\href{u}{\\textbf{b}}"),
               paste0("\\textcolor{", col, "}{\\textbf{b}}"))

  # Malformed input is left alone rather than half-rewritten.
  expect_equal(strip("\\href{only-one-group}"), "\\href{only-one-group}")
})

test_that("an ampersand in prose is a literal, not an alignment tab", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # MicroTeX reads a bare `&` as an alignment tab even inside \text{} and
  # drops everything after it, so "Treatment & Control" rendered as
  # "Treatment " -- half the label gone, with no error. `&` cannot mean
  # alignment in prose: math spans and tabular environments are passed
  # through verbatim by the scanner and never reach the prose escaper.
  drawn <- function(s) {
    d <- latex_grob(s, gp = grid::gpar(fontsize = 14))$layout_df
    paste(d$text[d$type == "text" & !is.na(d$text)], collapse = "")
  }
  expect_equal(drawn("Treatment & Control"), "Treatment  Control")
  expect_equal(drawn("Cats & Dogs & Mice"), "Cats  Dogs  Mice")

  # An ampersand the user already escaped must not be escaped twice.
  expect_equal(latex_wrap("a \\& b"), "\\text{a \\& b}")
  expect_equal(latex_wrap("a & b"), "\\text{a \\& b}")

  # Verbatim regions keep their real alignment tabs.
  expect_equal(latex_wrap("\\begin{tabular}{ll}a&b\\end{tabular}"),
               "\\begin{tabular}{ll}a&b\\end{tabular}")
  expect_equal(latex_wrap("$\\begin{matrix}a&b\\end{matrix}$"),
               "\\begin{matrix}a&b\\end{matrix}")
})

test_that("a line break inside a group re-opens the group on the next line", {
  # Each segment becomes its own \text{}, so a break inside \textbf{}
  # used to leave the group open at the end of one segment and unopened
  # at the start of the next: the second line came out unstyled and any
  # text after the group escaped the \text{} wrapper entirely.
  expect_equal(latex_wrap("Hello \\textbf{big\nbold} world"),
               "\\text{Hello \\textbf{big}}\\\\\\text{\\textbf{bold} world}")

  # The re-opener carries the command's arguments, not just its name --
  # matching only `\textcolor` reopened a plain `{two}` and lost the colour.
  expect_equal(latex_wrap("\\textcolor{red}{one\ntwo}"),
               "\\text{\\textcolor{red}{one}}\\\\\\text{\\textcolor{red}{two}}")

  # Nesting re-opens every level, outermost first.
  expect_equal(latex_wrap("a \\textbf{\\textit{p\nq}} b"),
               "\\text{a \\textbf{\\textit{p}}}\\\\\\text{\\textbf{\\textit{q}} b}")

  # A literal \\ inside the group behaves the same as a newline.
  expect_equal(latex_wrap("A \\textbf{x \\\\ y} B"),
               "\\text{A \\textbf{x}}\\\\\\text{\\textbf{y} B}")

  # Escaped braces are literals and must not move the group depth.
  expect_equal(latex_wrap("a \\{b\nc\\} d"),
               "\\text{a \\{b}\\\\\\text{c\\} d}")

  # Breaks outside any group are unchanged.
  expect_equal(latex_wrap("plain\ntext"), "\\text{plain}\\\\\\text{text}")
})

test_that("the styling really survives the break, not just the source", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  face <- function(s) {
    d <- latex_grob(s, gp = grid::gpar(fontsize = 14))$layout_df
    d <- d[d$type == "text" & !is.na(d$text), ]
    stats::setNames(d$font_style, d$text)
  }
  # FontStyle is a bitmask: 1 = rm, +2 = bold, +4 = italic.
  f <- face("Hello \\textbf{big\nbold} world")
  expect_equal(unname(f[["big"]]), 3L)
  expect_equal(unname(f[["bold"]]), 3L)   # was 1L: the emphasis was lost
  expect_equal(unname(f[["Hello "]]), 1L)

  f <- face("a \\textbf{\\textit{p\nq}} b")
  expect_equal(unname(f[["p"]]), 7L)
  expect_equal(unname(f[["q"]]), 7L)

  col <- function(s) {
    d <- latex_grob(s, gp = grid::gpar(fontsize = 14))$layout_df
    d <- d[d$type == "text" & !is.na(d$text), ]
    stats::setNames(d$color, d$text)
  }
  cc <- col("\\textcolor{red}{one\ntwo}")
  expect_equal(unname(cc[["one"]]), "#FF0000")
  expect_equal(unname(cc[["two"]]), "#FF0000")
})

test_that("tabular* renders instead of erroring on its width argument", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # `tabular*` takes a target width before the column spec. Stripping only
  # the star left the width where the alignment belonged, and MicroTeX
  # rejected it outright with "Invalid alignment in array environment".
  strip <- gridmicrotex:::.strip_document_wrappers
  expect_equal(strip("\\begin{tabular*}{\\textwidth}{lcr}a\\end{tabular*}"),
               "\\begin{tabular}{lcr}a\\end{tabular}")

  starred <- latex_dims("\\begin{tabular*}{\\textwidth}{lcr}a&b&c\\end{tabular*}")
  plain <- latex_dims("\\begin{tabular}{lcr}a&b&c\\end{tabular}")
  expect_equal(as.numeric(starred$width), as.numeric(plain$width))
})

# Generated bidirectional-ordering matrix.
#
# Two bidi defects this cycle survived a green suite and surfaced only
# when a specific shape was built by hand -- each time the gap was testing
# the case just written for, not the one a step past it. So this file
# crosses the factors rather than picking cases.
#
# The oracle is textshaping::shape_text(), which shapes and orders a
# string independently of us. Only *ordering* is compared: the oracle
# never sees a measure, so our line breaking is not under test here.

# --- vocabulary -------------------------------------------------------
#
# Uyghur joins its letters and reaches into the Arabic block for vowels
# Arabic does not write (U+06C7, U+06D5, U+06AD); Hebrew is right to left
# but does not join. Both, so joining is not the only RTL case covered.
UY <- c("\u062e\u06c7\u0634", "\u0643\u06d5\u0644\u062f\u0649\u06ad\u0649\u0632",
        "\u0628\u06c7", "\u062f\u06c7\u0646\u064a\u0627\u063a\u0627")
HE <- c("\u05e9\u05dc\u05d5\u05dd", "\u05e2\u05d5\u05dc\u05dd",
        "\u05e1\u05e4\u05e8", "\u05d2\u05d3\u05d5\u05dc")
LA <- c("alpha", "beta", "gamma", "delta")

# --- the oracle -------------------------------------------------------

# The paragraph's base direction: the direction of its first strong
# character (UAX #9 rules P2/P3). Resolved once for the paragraph and
# inherited by every line -- a line that happens to start with a Latin
# word does *not* become left-to-right on its own.
base_dir <- function(s) {
  for (cp in utf8ToInt(s)) {
    if ((cp >= 0x0590 && cp <= 0x08FF) ||
        (cp >= 0xFB1D && cp <= 0xFDFF) ||
        (cp >= 0xFE70 && cp <= 0xFEFF)) return("rtl")
    if ((cp >= 0x41 && cp <= 0x5A) || (cp >= 0x61 && cp <= 0x7A)) return("ltr")
  }
  "ltr"
}

# Visual order of the words of `s`, from textshaping.
#
# shape_text() returns one row per glyph. Its `glyph` column is the
# 1-based *character* index (`index` is the font's glyph id -- the names
# read the other way round), and `x_offset` is the visual position. So a
# word's visual position is the smallest x of any character in it.
#
# `dir` must be the *paragraph's* base direction, not the line's. Left to
# resolve it per line, the oracle re-runs P2/P3 on each line and a line
# beginning with a Latin word comes back left-to-right -- which is what
# made this disagree with a correct layout the first time it was written.
#
# Returns NULL when the shape contract does not hold, so a textshaping
# change makes these tests skip rather than fail for the wrong reason.
oracle_order <- function(s, dir, size = 14) {
  sh <- textshaping::shape_text(s, family = "sans", size = size,
                                direction = dir)$shape
  if (is.null(sh$glyph) || is.null(sh$x_offset)) return(NULL)
  if (max(sh$glyph) != nchar(s)) return(NULL)
  words <- strsplit(s, " ", fixed = TRUE)[[1]]
  starts <- cumsum(c(1, head(nchar(words), -1) + 1))
  ends <- starts + nchar(words) - 1
  owner <- vapply(sh$glyph, function(i) {
    w <- which(i >= starts & i <= ends)
    if (length(w)) w[1] else NA_integer_
  }, integer(1))
  keep <- !is.na(owner)
  if (!any(keep)) return(NULL)
  xmin <- tapply(sh$x_offset[keep], owner[keep], min)
  words[as.integer(names(sort(xmin)))]
}

# Our own layout, as a list of lines, each a vector of record texts in
# visual (left to right) order.
our_lines <- function(tex, max_width, ...) {
  d <- latex_grob(tex, input_mode = "math", max_width = max_width,
                  gp = grid::gpar(fontsize = 14), ...)$layout_df
  d <- d[d$type == "text" & !is.na(d$text), ]
  lapply(sort(unique(round(d$y, 1))), function(yy) {
    s <- d[abs(round(d$y, 1) - yy) < 0.01, ]
    s$text[order(s$x)]
  })
}

# --- case generation --------------------------------------------------

# Wrap the i-th word of a sentence in a group, so the row has siblings.
WRAPPERS <- list(
  none   = function(w) w,
  bold   = function(w) paste0("}\\textbf{\\text{", w, "}}\\text{"),
  colour = function(w) paste0("}\\textcolor{red}{\\text{", w, "}}\\text{"),
  family = function(w) paste0("}\\gmfontfamily{serif}{\\text{", w, "}}\\text{"),
  nested = function(w) paste0("}\\textbf{\\textit{\\text{", w, "}}}\\text{")
)

# A sentence, and the LaTeX that renders it with `wrap` applied to every
# third word. The words themselves never change, so the same expected
# order serves every structure -- which is the point: structure must not
# change ordering.
make_case <- function(words, wrap) {
  f <- WRAPPERS[[wrap]]
  parts <- vapply(seq_along(words), function(i) {
    if (i %% 3L == 0L) f(words[i]) else words[i]
  }, character(1))
  list(sentence = paste(words, collapse = " "),
       tex = paste0("\\text{", paste(parts, collapse = " "), "}"))
}

VOCAB <- list(
  uyghur      = rep(UY, 3),
  hebrew      = rep(HE, 3),
  latin       = rep(LA, 3),
  uy_latin    = rep(c(UY[1], LA[1], UY[2], LA[2]), 3),
  he_latin    = rep(c(HE[1], LA[1], HE[2], LA[2]), 3),
  uy_hebrew   = rep(c(UY[1], HE[1], UY[2], HE[2]), 3)
)

# --- the matrix -------------------------------------------------------

test_that("ordering matches an independent shaper, across scripts and structures", {
  skip_on_cran()
  skip_if_not_installed("textshaping")
  skip_if_not(microtex_bidi_available(), "built without fribidi")
  pdf(NULL); on.exit(dev.off(), add = TRUE)

  checked <- 0L
  for (vocab in names(VOCAB)) {
    words <- VOCAB[[vocab]]
    # Mixed-script text inside a *wrapper* is the known gap below: a
    # group is reordered as one unit, which cannot express a group whose
    # own contents span two embedding levels. Covered there, not here.
    wraps <- if (grepl("_", vocab)) "none" else names(WRAPPERS)
    for (wrap in wraps) {
      case <- make_case(words, wrap)
      for (mw in c(140, 220, 320)) {
        for (just in c(FALSE, TRUE)) {
          lines <- our_lines(case$tex, mw, justify = just)
          # Lines are broken in logical order, so line k holds the next
          # n_k source words. That is what lets each line be handed to
          # the oracle on its own -- carrying the paragraph's direction,
          # which the line does not get to re-decide.
          dir <- base_dir(case$sentence)
          taken <- 0L
          for (ln in lines) {
            logical <- words[taken + seq_along(ln)]
            taken <- taken + length(ln)
            want <- oracle_order(paste(logical, collapse = " "), dir)
            if (is.null(want)) skip("textshaping's shape contract changed")
            expect_equal(
              ln, want,
              info = sprintf("%s / %s / width %d / justify %s",
                             vocab, wrap, mw, just))
            checked <- checked + 1L
          }
        }
      }
    }
  }
  expect_gt(checked, 200L)   # the matrix really did generate cases
})

test_that("every layout is well formed, whatever the direction", {
  # No oracle needed, so this half also runs on CRAN and without FriBidi:
  # it is about the layout being sane, not about which way it reads.
  pdf(NULL); on.exit(dev.off(), add = TRUE)

  for (vocab in names(VOCAB)) {
    words <- VOCAB[[vocab]]
    for (wrap in names(WRAPPERS)) {
      case <- make_case(words, wrap)
      for (mw in c(140, 320)) {
        for (opt in list(list(), list(justify = TRUE))) {
          tag <- sprintf("%s / %s / %d / %s", vocab, wrap, mw,
                         paste(names(opt), collapse = ","))
          g <- do.call(latex_grob,
                       c(list(case$tex, input_mode = "math", max_width = mw,
                              gp = grid::gpar(fontsize = 14)), opt))
          d <- g$layout_df
          d <- d[d$type == "text" & !is.na(d$text), ]

          # Every word is drawn exactly once. Reordering is a permutation;
          # dropping or duplicating one would be the worst kind of bug.
          drawn <- d$text[d$text %in% words]
          expect_equal(sort(drawn), sort(words), info = tag)

          # Nothing overlaps its neighbour on a line, and nothing runs
          # past the measure.
          for (yy in unique(round(d$y, 1))) {
            s <- d[abs(round(d$y, 1) - yy) < 0.01, ]
            s <- s[order(s$x), ]
            expect_false(any(diff(s$x) < 0), info = tag)
          }
          # Only for the flat structure. A font group is not a break
          # opportunity, so a paragraph built from groups can overrun its
          # measure by a word -- pre-existing, unrelated to direction
          # (it happens for all-Latin text too, where no level is ever
          # resolved), and not something this file is about.
          if (identical(wrap, "none")) expect_lte(g$bbox_w, mw + 1, label = tag)
        }
      }
    }
  }
})

test_that("a group holding both directions still lays out every word", {
  skip_on_cran()
  skip_if_not_installed("textshaping")
  skip_if_not(microtex_bidi_available(), "built without fribidi")
  pdf(NULL); on.exit(dev.off(), add = TRUE)

  # Levels are resolved for the whole paragraph, so every *character* has
  # the level UAX #9 gives it. What a line cannot yet express is a group
  # whose own contents span two levels: the group is one child of the
  # line and carries one level, so rule L2 either moves all of it or none
  # of it. `\text{alpha SHALOM }` holds level 2 and level 1 at once.
  #
  # Whether a given paragraph *meets* that shape depends on where its
  # lines break, which depends on font metrics -- so this asserts the
  # ordering is sane, not that the gap is present. An earlier version
  # asserted the gap and failed under R CMD check, where different
  # metrics broke the lines elsewhere and the shape never arose.
  #
  # Closing it means reordering the line's *leaves* rather than its
  # children. That is possible -- a font group's box is its inner row's
  # HBox, since FontStyleAtom applies the style through Env rather than
  # wrapping a box -- but a ColorAtom really does wrap its child in a
  # ColorBox, so flattening has to stop there. Left deliberately.
  words <- rep(c("\u05e9\u05dc\u05d5\u05dd", "alpha",
                 "\u05e2\u05d5\u05dc\u05dd", "beta"), 3)
  case <- make_case(words, "bold")
  lines <- our_lines(case$tex, 220)

  # Whatever the ordering, the words are all still there, once each, and
  # each line reads in one consistent direction.
  expect_equal(sort(unlist(lines)), sort(words))
  expect_gt(length(lines), 1L)
})

test_that("a left-to-right formula is untouched by any of this", {
  # Levels are only resolved when the text holds something right-to-left,
  # so an all-Latin formula must lay out identically however it is built.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  words <- rep(LA, 3)
  for (wrap in names(WRAPPERS)) {
    for (mw in c(140, 200, 320)) {
      # Read left to right, across the lines in order: for text with no
      # right-to-left in it that is simply the order it was written in.
      # A wrapper changes where the lines break -- it is not a break
      # opportunity -- so positions legitimately move; the order does not.
      got <- unlist(our_lines(make_case(words, wrap)$tex, mw))
      expect_equal(got, words, info = sprintf("%s / %d", wrap, mw))
    }
  }
})

# --- text/CJK parsing ---

test_that("text and CJK parsing produces correct layout records", {
  # Latin \text{} now renders via text records (not math font paths)
  layout_latin <- parse_latex_cpp("\\text{Hello}", text_size = 20)
  expect_true("text" %in% layout_latin$type)
  expect_false("path" %in% layout_latin$type)

  # CJK uses text records; mixed math+CJK produces both types
  layout_mix <- parse_latex_cpp("x^2 + \\text{\u4F60\u597D}", text_size = 20)
  expect_true("text" %in% layout_mix$type)
  expect_true("path" %in% layout_mix$type)
  text_rows <- layout_mix[layout_mix$type == "text", ]
  expect_false(is.na(text_rows$font_size[1]))
})

# --- latex_grob with CJK ---

test_that("latex_grob with CJK text renders and stores fontfamily", {
  g <- latex_grob("x + \\text{\u4F60\u597D}",
                  gp = grid::gpar(fontfamily = "sans", fontsize = 20))
  expect_s3_class(g, "latexgrob")
  expect_equal(g$text_gp$fontfamily, "sans")
})

# --- text rotation / reflection ---

test_that("rotatebox populates rotation column on text records", {
  layout <- parse_latex_cpp("\\rotatebox{30}{\\mbox{Ab}}", text_size = 20)
  expect_true("rotation" %in% names(layout))
  txt <- layout[layout$type == "text", ]
  expect_gt(nrow(txt), 0L)
  # MicroTeX rotates CCW in a y-down canvas, so \rotatebox{30} stores -30
  # in the data frame (R flips the sign when building the grob).
  expect_true(all(abs(txt$rotation - -30) < 1e-3))
})

test_that("reflectbox leaves rotation at 0", {
  layout <- parse_latex_cpp("\\reflectbox{\\mbox{Ab}}", text_size = 20)
  txt <- layout[layout$type == "text", ]
  expect_gt(nrow(txt), 0L)
  expect_true(all(abs(txt$rotation) < 1e-3))
})

test_that("rotation past a quarter turn is not read as a reflection", {
  # The mirror test used to be `a < 0`, and `a` is cos(theta): true of a
  # \reflectbox, but equally true of every rotation past 90 degrees. Those
  # angles came out 180 degrees away from what was asked -- a vertical
  # label upside down -- and took a reflection's x-shift with them. The
  # sign of the determinant is the test that separates the two.
  for (deg in c(30, 89, 90, 91, 135, 180, 270, -90)) {
    layout <- parse_latex_cpp(sprintf("\\rotatebox{%d}{\\mbox{Ab}}", deg),
                              text_size = 20)
    txt <- layout[layout$type == "text", ]
    expect_gt(nrow(txt), 0L)
    # Stored CCW in a y-down canvas, i.e. negated, and only up to a turn --
    # so compare in [0, 360), where -180 and 180 are the same rotation.
    wrapped <- function(a) a %% 360
    expect_equal(wrapped(txt$rotation[1]), wrapped(-deg),
                 tolerance = 1e-3, info = as.character(deg))
  }
})

test_that("plain mbox leaves rotation at 0", {
  layout <- parse_latex_cpp("\\mbox{Ab}", text_size = 20)
  txt <- layout[layout$type == "text", ]
  expect_gt(nrow(txt), 0L)
  expect_true(all(abs(txt$rotation) < 1e-3))
})

# --- rect under rotation ---

test_that("rotated fbox emits 4 line records (not a single rect)", {
  plain <- parse_latex_cpp("\\fbox{A}", text_size = 20, use_path = FALSE)
  rot   <- parse_latex_cpp("\\rotatebox{30}{\\fbox{A}}",
                           text_size = 20, use_path = FALSE)
  # Plain fbox contains rect record(s); rotated version has none.
  expect_true(any(plain$type == "rect"))
  expect_false(any(rot$type == "rect"))
  # Rotated version gets extra line records (the 4 box edges).
  plain_lines <- sum(plain$type == "line")
  rot_lines   <- sum(rot$type == "line")
  expect_equal(rot_lines, plain_lines + 4L)
})

test_that("rotated colorbox emits a filled path (not a fill_rect)", {
  plain <- parse_latex_cpp("\\colorbox{yellow}{A}",
                           text_size = 20, use_path = FALSE)
  rot   <- parse_latex_cpp("\\rotatebox{30}{\\colorbox{yellow}{A}}",
                           text_size = 20, use_path = FALSE)
  expect_true(any(plain$type == "fill_rect"))
  expect_false(any(rot$type == "fill_rect"))
  expect_true(any(rot$type == "path"))
})

test_that("text measurer creates, measures, and handles styles", {
  measurer <- gridmicrotex:::.make_text_measurer(grid::gpar())
  expect_type(measurer, "closure")

  result <- measurer("Hello", 0L)
  expect_length(result, 3)
  expect_true(all(result > 0))

  # Width scales with text length
  expect_true(measurer("Hello World", 0L)[1] > measurer("Hi", 0L)[1])

  # Bold text wider than plain
  expect_true(measurer("Hello", 2L)[1] >= measurer("Hello", 0L)[1])

  # .resolve_text_face maps style codes
  expect_equal(gridmicrotex:::.resolve_text_face(0L), "plain")
  expect_equal(gridmicrotex:::.resolve_text_face(2L), "bold")
  expect_equal(gridmicrotex:::.resolve_text_face(6L), "bold.italic")
  expect_equal(gridmicrotex:::.resolve_text_face(NA_integer_), "plain")
})

test_that("\\texttt renders and measures in a monospace family", {
  # Bit 128 is MicroTeX's \texttt. It has to reach the *measurer* as well
  # as the renderer: measuring in one family and drawing in another puts
  # the glyphs where they do not fit.
  fam <- gridmicrotex:::.resolve_text_family
  expect_equal(fam(128L), "mono")
  expect_equal(fam(130L), "mono")                 # bold monospace
  expect_null(fam(2L))                            # plain bold: caller's choice
  expect_null(fam(NA_integer_))
  expect_equal(fam(2L, "serif"), "serif")         # default passes through
  expect_equal(fam(128L, "serif"), "mono")        # \texttt still wins

  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # The measurer must answer with monospace metrics, which are the same
  # for every character -- that is what makes the font monospace.
  m <- gridmicrotex:::.make_text_measurer(grid::gpar())
  expect_equal(m("i", 128L)[1], m("W", 128L)[1])
  expect_true(m("i", 1L)[1] < m("W", 1L)[1])

  # End to end: the drawn textGrob carries the family too.
  g <- grid::makeContent(latex_grob("\\texttt{Hg}", input_mode = "math"))
  fams <- vapply(g$children, function(k) k$gp$fontfamily %||% "", character(1))
  expect_true("mono" %in% fams)
})

test_that("a named font family travels from LaTeX to the layout", {
  # MicroTeX has no channel for a font *name* -- a text run carries only
  # FontStyle, a bitfield. \gmfontfamily packs an index into its unused
  # high byte, which C++ resolves back to the name on the record.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  runs <- function(tex) {
    df <- latex_grob(tex, input_mode = "math",
                     gp = grid::gpar(fontsize = 20))$layout_df
    df <- df[df$type == "text", c("text", "font_family", "font_style")]
    rownames(df) <- NULL
    df
  }
  one <- runs("\\gmfontfamily{Georgia}{ab}")
  expect_true(all(one$font_family == "Georgia"))
  # Ordinary text names no family, so gp$fontfamily still decides.
  expect_true(all(is.na(runs("\\text{ab}")$font_family)))

  # Nesting must *replace*: two indices OR'd together would address a
  # third, unrelated family, so "c" would come back as neither A nor B.
  nested <- runs("\\gmfontfamily{A}{a\\gmfontfamily{B}{b}c}")
  expect_equal(nested$font_family, c("A", "B", "A"))

  # The family composes with emphasis instead of replacing it: the low
  # byte keeps the bold bit, the high byte carries the index.
  bold <- runs("\\textbf{\\gmfontfamily{Georgia}{ab}}")
  expect_true(all(bitwAnd(bold$font_style, 2L) != 0L))
  expect_true(all(bold$font_family == "Georgia"))
  # Either order gives the same answer.
  expect_equal(bold, runs("\\gmfontfamily{Georgia}{\\textbf{ab}}"))

  # An empty name is a no-op rather than an error.
  expect_silent(runs("\\gmfontfamily{}{ab}"))

  # Two families in one expression must be measured apart. The C++ cache
  # used to key on the low byte of the style, which is where the family
  # index is *not*, so they shared one set of metrics -- whichever was
  # asked for first. Needs a device that resolves families.
  skip_if_not_installed("ragg")
  f <- tempfile(fileext = ".png")
  ragg::agg_png(f, width = 400, height = 120)
  on.exit({ dev.off(); unlink(f) }, add = TRUE)
  # Measured large on purpose. Layout widths are whole big points, so at a
  # normal size that quantum is several percent of a three-letter word: two
  # genuinely different faces whose "Wig" differs by less than one point land
  # on the same number, and the families look identical although nothing is
  # wrong. That is what failed on a CRAN Debian box. It reproduces here at
  # fontsize 10, where stock DejaVu mono and sans both measure 18; at 200 the
  # same pair differs by 14.
  w <- function(tex) as.numeric(latex_dims(tex, input_mode = "math",
                                           gp = grid::gpar(fontsize = 200))$width)
  # A font set with no distinct monospace face resolves both names to one
  # file, which no font size can separate. Probe with \texttt, which takes its
  # family from the FontStyle bit rather than the \gmfontfamily registry, so
  # this cannot mask a regression in the code under test: that would leave
  # \texttt working and fail the assertions below.
  #
  # The message carries what the layout recorded, so a machine we cannot reach
  # reports itself. Built lazily: skip_if() only forces it when it skips.
  tt <- w("\\texttt{Wig}")
  body <- w("\\text{Wig}")
  skip_if(isTRUE(all.equal(tt, body)), {
    d <- latex_grob("\\texttt{Wig}", input_mode = "math",
                    gp = grid::gpar(fontsize = 20))$layout_df
    d <- d[d$type == "text", ]
    paste0("text measurement does not distinguish font families: ",
           "texttt=", tt, " text=", body,
           " font_style=", paste(d$font_style, collapse = "/"),
           " font_family=", paste(d$font_family, collapse = "/"),
           " mono=", basename(systemfonts::match_fonts("mono")$path),
           " sans=", basename(systemfonts::match_fonts("sans")$path))
  })
  mono <- w("\\gmfontfamily{mono}{Wig}")
  sans <- w("\\gmfontfamily{sans}{Wig}")
  expect_false(isTRUE(all.equal(mono, sans)))
  expect_equal(w("\\gmfontfamily{mono}{Wig}\\gmfontfamily{sans}{Wig}"),
               mono + sans, tolerance = 1)
})

test_that("\\textrm returns to the caller's font", {
  # MicroTeX's own \textrm only ORs in FontStyle::rm -- the same bit a plain
  # \text{} run already carries -- so it could never express "reset the
  # family", and did nothing at all. The override names a reserved family
  # instead; see src/MicroTeX/lib/atom/font_family_atom.h.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  fam <- function(tex) {
    df <- latex_grob(tex, input_mode = "math",
                     gp = grid::gpar(fontsize = 20))$layout_df
    df <- df[df$type == "text", ]
    vapply(seq_len(nrow(df)),
           function(i) .resolve_text_family(df$font_style[i], default = "BODY",
                                            family = df$font_family[i]),
           character(1))
  }

  expect_equal(fam("\\textsf{a\\textrm{b}c}"), c("sans", "BODY", "sans"))
  expect_equal(fam("\\texttt{a\\textrm{b}c}"), c("mono", "BODY", "mono"))
  expect_equal(fam("\\gmfontfamily{Georgia}{a\\textrm{b}c}"),
               c("Georgia", "BODY", "Georgia"))

  # On its own it is still ordinary body text, as it always was. One
  # record, not one per letter: consecutive text characters are drawn as
  # a word (see RowAtom::processTextRun). The three cases above stay
  # one-per-record because each letter sits in a different font scope.
  expect_equal(fam("\\textrm{ab}"), "BODY")

  # Emphasis survives: the override keeps upstream's *nested* FontStyleAtom,
  # so only the family is replaced, not the bold/italic bits.
  df <- latex_grob("\\textsf{\\textbf{a\\textrm{b}c}}", input_mode = "math",
                   gp = grid::gpar(fontsize = 20))$layout_df
  expect_true(all(bitwAnd(df$font_style[df$type == "text"], 2L) != 0L))
})

test_that("\\textrm is measured in the font it is drawn in", {
  # Measuring in mono while drawing in the body font puts glyphs where they
  # do not fit, so the reset has to reach the measurer too.
  skip_if_not_installed("ragg")
  f <- tempfile(fileext = ".png")
  ragg::agg_png(f, width = 400, height = 100)
  on.exit({ dev.off(); unlink(f) }, add = TRUE)
  w <- function(tex) as.numeric(latex_dims(tex, input_mode = "math",
                                           gp = grid::gpar(fontsize = 20))$width)

  body <- w("\\text{Wig}")
  skip_if(isTRUE(all.equal(w("\\texttt{Wig}"), body)),
          "device does not distinguish mono from the body font")
  expect_equal(w("\\texttt{\\textrm{Wig}}"), body)
  expect_equal(w("\\textrm{Wig}"), body)
})

test_that("register/clear measurer lifecycle and integration", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  m <- gridmicrotex:::.make_text_measurer(grid::gpar())
  m2 <- gridmicrotex:::.make_text_measurer(grid::gpar(fontfamily = "mono"))
  # The two closures must actually disagree, or "the second registration
  # replaced the first" is unfalsifiable below.
  expect_false(isTRUE(all.equal(m("Wig", 0L), m2("Wig", 0L))))

  register_text_measurer(m)
  clear_text_measurer()
  # Double-register replaces the previous one rather than stacking.
  register_text_measurer(m)
  register_text_measurer(m2)
  clear_text_measurer()

  # CJK layout uses measurer for dimensions
  if (.Platform$OS.type == "windows") {
    expect_no_error(dims <- latex_dims("\\text{\u4F60\u597D\u4E16\u754C}", gp = grid::gpar(fontsize = 20)))
  } else {
    expect_silent(dims <- latex_dims("\\text{\u4F60\u597D\u4E16\u754C}", gp = grid::gpar(fontsize = 20)))
  }
  expect_true(grid::convertWidth(dims$width, "bigpts", valueOnly = TRUE) > 0)
})

test_that("measurer cache returns identical values to a fresh measurement", {
  # Within one closure, repeat calls hit the cache; they must equal the
  # first (un-cached) call bit-for-bit, and must also match a separate
  # fresh closure's first (un-cached) call.
  txt <- "The quick brown fox jumps over the lazy dog"
  m1 <- gridmicrotex:::.make_text_measurer(grid::gpar())
  first  <- m1(txt, 0L)
  second <- m1(txt, 0L)  # cache hit
  expect_identical(first, second)

  m2 <- gridmicrotex:::.make_text_measurer(grid::gpar())
  fresh  <- m2(txt, 0L)  # un-cached, separate closure
  expect_identical(first, fresh)

  # Different font_style must not collide with a cached entry.
  bold_cached <- m1(txt, 2L)  # first time for style=2 on m1
  bold_fresh  <- m2(txt, 2L)
  expect_identical(bold_cached, bold_fresh)
  expect_false(identical(first, bold_cached))
})

test_that("measuring leaves the caller's display list untouched", {
  # Measuring used to pushViewport() on whatever device the caller had open.
  # Viewport pushes are recorded on the graphics engine display list, so the
  # device then looks like it holds a plot and knitr snapshots a spurious
  # blank figure before the real plot's grid.newpage().
  dl_len <- function() length(grDevices::recordPlot()[[1]])

  pdf(NULL)
  on.exit(dev.off(), add = TRUE)
  grDevices::dev.control("enable")
  expect_identical(dl_len(), 0L)

  m <- gridmicrotex:::.make_text_measurer(grid::gpar())
  m("Heterogeneity", 0L)
  expect_identical(dl_len(), 0L)
  # The three-argument form, used by \gmfontfamily and \textrm spans.
  m("Heterogeneity", 0L, "Georgia")
  expect_identical(dl_len(), 0L)

  # Same for a full grob construction, which is what callers actually do.
  latex_grob("\\text{This is study A}\\\\\\text{This is study B}",
             input_mode = "math", render_mode = "path",
             gp = grid::gpar(fontsize = 8))
  expect_identical(dl_len(), 0L)
  latex_dims("\\text{measure me}", input_mode = "math")
  expect_identical(dl_len(), 0L)

  # Markdown measures far more text than a formula does, and arrived after
  # the fix above, so it needs its own guard.
  markdown_grob("**bold** and $x^2$")
  expect_identical(dl_len(), 0L)
  markdown_box_grob("# Title\n\nProse with $x^2$.\n\n- one\n- two",
                    width = grid::unit(3, "in"))
  expect_identical(dl_len(), 0L)

  # grobWidth() forces the layout, which is where the measuring happens.
  grid::convertWidth(grid::grobWidth(markdown_grob("hello $x$")),
                     "bigpts", valueOnly = TRUE)
  expect_identical(dl_len(), 0L)

  # A drawing call *must* still record, or the assertions above are vacuous.
  grid::grid.newpage()
  grid::grid.draw(latex_grob("\\text{drawn}", input_mode = "math",
                             render_mode = "path"))
  expect_gt(dl_len(), 0L)
})

test_that("a cached layout is not reused across graphics devices", {
  skip_if_not_installed("ragg")
  # \text{} runs are sized by the R text-measurer, which measures through
  # grid on the *current* device -- and devices disagree. Without the
  # device in the cache key, a layout measured on the screen device was
  # handed back verbatim to a later ggsave(), placing text at widths the
  # output device never agreed to.
  key <- gridmicrotex:::.parse_cache_key
  args <- list("\\text{x}", 20, 10, "#000000", 0, "", "", "", FALSE, "",
               FALSE, FALSE)
  expect_false(identical(do.call(key, c(args, device = "pdf")),
                         do.call(key, c(args, device = "agg_png"))))

  # End to end: the width from a device must not depend on which device
  # happened to parse the same expression first.
  f <- tempfile(fileext = ".png")
  on.exit(unlink(f), add = TRUE)
  tex <- "\\text{Hello world}"
  on_agg <- function() {
    ragg::agg_png(f)
    on.exit(grDevices::dev.off(), add = TRUE)
    as.numeric(latex_dims(tex, input_mode = "math")$width)
  }

  latex_cache_clear()
  alone <- on_agg()

  latex_cache_clear()
  grDevices::pdf(NULL)
  latex_dims(tex, input_mode = "math")
  grDevices::dev.off()
  expect_equal(on_agg(), alone)
})

#' Build grid children from a MicroTeX layout data.frame
#'
#' Converts each row of the layout data.frame into the appropriate
#' grid grob (pathGrob, segmentsGrob, rectGrob, textGrob, glyphGrob).
#'
#' @param layout_df Data.frame returned by \code{parse_latex_cpp()}.
#' @param total_h Total height of the formula (height + depth) in bigpts.
#' @param depth Depth below the baseline in bigpts (default 0).
#' @param text_gp Optional \code{\link[grid]{gpar}} for text grobs
#'   (from \code{\\text\{\}} blocks). Controls fontfamily; the face comes
#'   from each record's MicroTeX \code{font_style}.
#' @param render_mode Character string: \code{"path"} or \code{"typeface"}.
#'   In typeface mode, glyph records are rendered via \code{glyphGrob}.
#' @return A \code{grid::gList} of child grobs.
#' @noRd
build_latex_children <- function(layout_df, total_h, depth = 0,
                                 text_gp = NULL,
                                 render_mode = "typeface") {
  n <- nrow(layout_df)
  if (n == 0) return(grid::gList())

  # MicroTeX line widths are in bigpts (1/72 inch) but R's lwd unit is
  # 1/96 inch, so scale by 96/72 = 4/3 to get correct stroke widths.
  lwd_scale <- 96 / 72

  # Pre-extract columns. data.frame row indexing (`layout_df[i, ]`)
  # is much slower than column access (`layout_df$x[i]`).
  type   <- layout_df$type
  x      <- layout_df$x;      y      <- layout_df$y
  x2     <- layout_df$x2;     y2     <- layout_df$y2
  width  <- layout_df$width;  height <- layout_df$height
  rx     <- layout_df$rx;     ry     <- layout_df$ry
  color  <- layout_df$color;  lwd    <- layout_df$lwd

  # Pre-allocated parts slot. parts[i] holds the grob for layout row i
  # (NULL for empty/skipped rows and for glyph rows, which are batched
  # below). Reserve one extra slot at the end for the batched glyphGrob
  # so the final assembly is one do.call instead of an O(n) gList walk.
  parts <- vector("list", n + 1L)

  for (i in seq_len(n)) {
    parts[[i]] <- switch(type[i],
      "path" = {
        path_data <- layout_df$path[[i]]
        if (is.null(path_data)) NULL
        else build_path_grob(path_data, color[i], i, total_h)
      },
      "line" = if (abs(y[i] - y2[i]) < 0.001) {
        # Horizontal rule (fraction bars, \hline, vertical borders) →
        # filled rect; keeps dimensions in bigpts and avoids lwd issues.
        grid::rectGrob(
          x = grid::unit(x[i], "bigpts"),
          y = grid::unit(total_h - y[i], "bigpts"),
          width = grid::unit(x2[i] - x[i], "bigpts"),
          height = grid::unit(lwd[i], "bigpts"),
          just = c("left", "centre"),
          gp = grid::gpar(fill = color[i], col = NA),
          name = paste0("rule.", i)
        )
      } else {
        grid::segmentsGrob(
          x0 = grid::unit(x[i], "bigpts"),
          y0 = grid::unit(total_h - y[i], "bigpts"),
          x1 = grid::unit(x2[i], "bigpts"),
          y1 = grid::unit(total_h - y2[i], "bigpts"),
          gp = grid::gpar(col = color[i], lwd = lwd[i] * lwd_scale, lineend = "butt"),
          name = paste0("line.", i)
        )
      },
      "fill_rect" = grid::rectGrob(
        x = grid::unit(x[i], "bigpts"),
        y = grid::unit(total_h - y[i], "bigpts"),
        width = grid::unit(width[i], "bigpts"),
        height = grid::unit(height[i], "bigpts"),
        just = c("left", "top"),
        gp = grid::gpar(fill = color[i], col = NA),
        name = paste0("fillrect.", i)
      ),
      "rect" = grid::rectGrob(
        x = grid::unit(x[i], "bigpts"),
        y = grid::unit(total_h - y[i], "bigpts"),
        width = grid::unit(width[i], "bigpts"),
        height = grid::unit(height[i], "bigpts"),
        just = c("left", "top"),
        gp = grid::gpar(col = color[i], fill = NA, lwd = lwd[i] * lwd_scale),
        name = paste0("rect.", i)
      ),
      "fill_roundrect" = {
        # MicroTeX supplies rx/ry separately; grid only takes one radius,
        # so use the smaller axis to avoid clipping outside the bounds.
        r <- min(rx[i], ry[i], na.rm = TRUE)
        grid::roundrectGrob(
          x = grid::unit(x[i], "bigpts"),
          y = grid::unit(total_h - y[i], "bigpts"),
          width = grid::unit(width[i], "bigpts"),
          height = grid::unit(height[i], "bigpts"),
          r = grid::unit(r, "bigpts"),
          just = c("left", "top"),
          gp = grid::gpar(fill = color[i], col = NA),
          name = paste0("fillroundrect.", i)
        )
      },
      "roundrect" = {
        r <- min(rx[i], ry[i], na.rm = TRUE)
        grid::roundrectGrob(
          x = grid::unit(x[i], "bigpts"),
          y = grid::unit(total_h - y[i], "bigpts"),
          width = grid::unit(width[i], "bigpts"),
          height = grid::unit(height[i], "bigpts"),
          r = grid::unit(r, "bigpts"),
          just = c("left", "top"),
          gp = grid::gpar(col = color[i], fill = NA, lwd = lwd[i] * lwd_scale),
          name = paste0("roundrect.", i)
        )
      },
      "text" = {
        txt <- layout_df$text[i]
        if (is.na(txt) || !nzchar(txt)) NULL
        else {
          tgp <- grid::gpar(
            fontsize = layout_df$font_size[i],
            col = color[i],
            fontface = .resolve_text_face(layout_df$font_style[i])
          )
          fam <- .resolve_text_family(layout_df$font_style[i],
                                      text_gp$fontfamily,
                                      layout_df$font_family[i])
          if (!is.null(fam)) tgp$fontfamily <- fam
          grid::textGrob(
            label = txt,
            x = grid::unit(x[i], "bigpts"),
            y = grid::unit(total_h - y[i], "bigpts"),
            just = c("left", "bottom"),
            # y is flipped to grid's y-up (total_h - y), so rotation sign
            # flips too: MicroTeX y-down ccw → grid y-up ccw.
            rot = -layout_df$rotation[i],
            gp = tgp,
            name = paste0("text.", i)
          )
        }
      },
      "image" = {
        # The reference is opaque to the engine; decode it here and let
        # .image_grob() decide what kind of grob the file deserves -- a
        # true-vector pictureGrob for an SVG, a raster otherwise. The
        # record carries its top-left corner, so it flips like fill_rect.
        p <- .image_ref_decode(layout_df$image_ref[i])
        # y is flipped to grid's y-up, so the rotation sign flips with it,
        # exactly as for text. A rotated record carries its centre rather
        # than its top-left corner; .image_grob() reads x/y that way when
        # angle is non-zero.
        g <- if (is.null(p)) NULL
        else .image_grob(p, width[i], height[i],
                         x = grid::unit(x[i], "bigpts"),
                         y = grid::unit(total_h - y[i], "bigpts"),
                         name = paste0("image.", i),
                         angle = -layout_df$rotation[i])
        # The box was reserved at parse time, when the file did read. If it
        # no longer does -- deleted since, or a reader that parsed the header
        # and then choked on the body -- say so rather than leaving a gap
        # nothing in the output accounts for.
        if (is.null(g) && !is.null(p)) {
          .image_warn_once(paste0("draw\x1f", .image_stamp(p)),
                           paste0("Cannot draw '", p,
                                  "' now, though it was readable when the ",
                                  "layout was measured; leaving it blank."))
        }
        g
      },
      NULL  # glyph rows handled in bulk below; everything else → NULL
    )
  }

  # --- Glyphs: collected in bulk and rendered as a single batched
  # glyphGrob at the end of the gList (matching original behavior, so
  # math symbols overlay rules/fills). Path mode renders glyphs as
  # path records and emits no "glyph" rows. ---
  if (render_mode == "typeface") {
    g <- which(type == "glyph")
    if (length(g) > 0L) {
      ff  <- layout_df$font_file[g]
      gid <- layout_df$glyph[g]
      keep <- !is.na(ff) & nzchar(ff) & !is.na(gid)
      if (any(keep)) {
        k <- g[keep]
        parts[[n + 1L]] <- .build_glyph_grob(
          ids        = layout_df$glyph[k],
          x          = layout_df$x[k],
          y          = total_h - layout_df$y[k],
          sizes      = layout_df$font_size[k],
          cols       = layout_df$color[k],
          font_files = layout_df$font_file[k],
          depth      = depth
        )
      }
    }
  }

  # Images go behind everything else. Glyphs are batched into the final
  # slot, so leaving images in record order would put them *under* the text
  # in typeface mode and *over* it in path mode -- the same input would
  # z-order differently in the two render modes.
  img <- which(type == "image")
  if (length(img)) parts <- c(parts[img], parts[-img])
  do.call(grid::gList, Filter(Negate(is.null), parts))
}

#' Convert path segments to a grid pathGrob
#'
#' @param path_data List with \code{cmd} and \code{coords} elements.
#' @param col Fill color.
#' @param idx Index for naming.
#' @param total_h Total height for y-axis flipping.
#'
#' @return A \code{grid::pathGrob} or \code{NULL}.
#' @noRd
build_path_grob <- function(path_data, col, idx, total_h) {
  cmds <- path_data$cmd
  coords <- path_data$coords
  n <- length(cmds)

  if (n == 0) return(NULL)

  all_x <- numeric(0)
  all_y <- numeric(0)
  all_id <- integer(0)
  current_id <- 1L
  sub_x <- numeric(0)
  sub_y <- numeric(0)

  flush_subpath <- function() {
    if (length(sub_x) > 2) {
      all_x <<- c(all_x, sub_x)
      all_y <<- c(all_y, sub_y)
      all_id <<- c(all_id, rep(current_id, length(sub_x)))
      current_id <<- current_id + 1L
    }
    sub_x <<- numeric(0)
    sub_y <<- numeric(0)
  }

  for (j in seq_len(n)) {
    cmd <- cmds[j]
    cr <- coords[j, ]

    switch(cmd,
      "M" = {
        flush_subpath()
        sub_x <- cr[1]
        sub_y <- total_h - cr[2]
      },
      "L" = {
        sub_x <- c(sub_x, cr[1])
        sub_y <- c(sub_y, total_h - cr[2])
      },
      "C" = {
        if (length(sub_x) == 0) next
        p0x <- sub_x[length(sub_x)]
        p0y <- sub_y[length(sub_y)]
        pts <- cubic_bezier(
          p0x, p0y,
          cr[1], total_h - cr[2],
          cr[3], total_h - cr[4],
          cr[5], total_h - cr[6]
        )
        sub_x <- c(sub_x, pts$x[-1])
        sub_y <- c(sub_y, pts$y[-1])
      },
      "Q" = {
        if (length(sub_x) == 0) next
        p0x <- sub_x[length(sub_x)]
        p0y <- sub_y[length(sub_y)]
        pts <- quad_bezier(
          p0x, p0y,
          cr[1], total_h - cr[2],
          cr[3], total_h - cr[4]
        )
        sub_x <- c(sub_x, pts$x[-1])
        sub_y <- c(sub_y, pts$y[-1])
      },
      "Z" = {
        flush_subpath()
      }
    )
  }
  flush_subpath()

  if (length(all_x) == 0) return(NULL)

  grid::pathGrob(
    x = grid::unit(all_x, "bigpts"),
    y = grid::unit(all_y, "bigpts"),
    id = all_id,
    rule = "evenodd",
    gp = grid::gpar(fill = col, col = NA),
    name = paste0("path.", idx)
  )
}

#' Approximate a cubic bezier curve with line segments
#' @noRd
cubic_bezier <- function(x0, y0, x1, y1, x2, y2, x3, y3, n = 16) {
  t <- seq(0, 1, length.out = n)
  mt <- 1 - t
  x <- mt^3 * x0 + 3 * mt^2 * t * x1 + 3 * mt * t^2 * x2 + t^3 * x3
  y <- mt^3 * y0 + 3 * mt^2 * t * y1 + 3 * mt * t^2 * y2 + t^3 * y3
  list(x = x, y = y)
}

#' Approximate a quadratic bezier curve with line segments
#' @noRd
quad_bezier <- function(x0, y0, x1, y1, x2, y2, n = 12) {
  t <- seq(0, 1, length.out = n)
  mt <- 1 - t
  x <- mt^2 * x0 + 2 * mt * t * x1 + t^2 * x2
  y <- mt^2 * y0 + 2 * mt * t * y1 + t^2 * y2
  list(x = x, y = y)
}

#' Resolve MicroTeX FontStyle bitmask to R font face
#'
#' MicroTeX FontStyle: rm=1, bf=2, it=4, etc. (bitmask).
#'
#' @param style Integer font style bitmask.
#' @return Character: \code{"plain"}, \code{"bold"}, \code{"italic"},
#'   or \code{"bold.italic"}.
#' @noRd
.resolve_text_face <- function(style) {
  if (is.na(style)) return("plain")
  is_bold   <- bitwAnd(style, 2L) != 0L
  is_italic <- bitwAnd(style, 4L) != 0L
  if (is_bold && is_italic) "bold.italic"
  else if (is_bold) "bold"
  else if (is_italic) "italic"
  else "plain"
}

# Which font family a text record should be drawn in. The single place
# the precedence lives, because the renderer and the *measurer* both ask:
# measuring in one font and drawing in another puts the glyphs where they
# do not fit.
#
#   \gmfontfamily  >  \texttt / \textsf  >  gp$fontfamily
#
# Local beats global. `family` is the name carried by the record, from a
# CSS font-family; it can say "Georgia", which the font_style bits never
# can. Those bits still cover \texttt (128) and \textsf (64), which is how
# markdown's `code` and <code> get monospace -- without this they reported
# monospace but drew in the body font.
#
# There is deliberately no serif case for the bits: \textrm and a plain
# \text{} both report bit 1, so mapping it would restyle every ordinary
# run. font-family: serif goes through `family` instead -- and so does
# \textrm, which names the reserved family below rather than relying on
# its (indistinguishable) style bit. See src/MicroTeX/lib/atom/font_family_atom.h.
.GM_DEFAULT_FAMILY <- "gridmicrotex.default"

.resolve_text_family <- function(style, default = NULL, family = NULL) {
  if (!is.null(family) && !is.na(family) && nzchar(family)) {
    # \textrm: back to the caller's font, overriding any enclosing
    # \textsf / \texttt rather than inheriting it.
    if (identical(family, .GM_DEFAULT_FAMILY)) return(default)
    return(family)
  }
  if (is.na(style)) return(default)
  if (bitwAnd(style, 128L) != 0L) return("mono")
  if (bitwAnd(style, 64L) != 0L) return("sans")
  default
}

# Cache of font file -> glyphFont objects
.glyph_font_cache <- new.env(parent = emptyenv())

#' Get or create a glyphFont object for a font file
#'
#' Caches \code{grDevices::glyphFont()} objects keyed by file path
#' so that repeated calls for the same font file reuse the same object.
#'
#' @param font_file Absolute path to the OTF/TTF font file.
#' @return A \code{glyphFont} object.
#' @noRd
.get_glyph_font <- function(font_file) {
  cached <- .glyph_font_cache[[font_file]]
  if (!is.null(cached)) return(cached)

  gf <- grDevices::glyphFont(
    file = font_file,
    index = 0L,
    family = tools::file_path_sans_ext(basename(font_file)),
    weight = 400,
    style = "normal"
  )
  .glyph_font_cache[[font_file]] <- gf
  gf
}

#' Build a single batched glyphGrob from collected glyph data
#'
#' Creates a \code{grid::glyphGrob} containing all math glyphs,
#' using \code{grDevices::glyphInfo()} to describe glyph IDs, positions,
#' sizes, colors, and fonts. Each unique font file becomes an entry
#' in the \code{glyphFontList}.
#'
#' @param ids Integer vector of glyph IDs.
#' @param x,y Numeric vectors of glyph positions (already y-flipped, in bigpts).
#' @param sizes Numeric vector of font sizes.
#' @param cols Character vector of colors.
#' @param font_files Character vector of font file paths.
#' @param depth Depth below the baseline in bigpts (default 0).
#' @return A \code{grid::glyphGrob} or \code{NULL}.
#' @noRd
.build_glyph_grob <- function(ids, x, y, sizes, cols, font_files, depth = 0) {
  n <- length(ids)
  if (n == 0) return(NULL)

  # Build font list from unique font files
  unique_fonts <- unique(font_files)
  font_list_args <- lapply(unique_fonts, .get_glyph_font)
  font_list <- do.call(grDevices::glyphFontList, font_list_args)

  # Map each glyph to its font index (1-based)
  font_idx <- match(font_files, unique_fonts)

  # Glyph positions are in bigpts within the formula's coordinate system.
  # Set anchors at origin so positions map 1:1 to viewport coordinates.
  w <- if (n > 1) max(x) - min(x) else max(sizes)
  h <- if (n > 1) max(y) - min(y) else max(sizes)

  # Anchor points for positioning (see Paul Murrell, 2023,
  # "Rendering Typeset Glyphs in R Graphics").
  # "bottom" = 0 (the lowest glyph y-offset), "baseline" = depth
  # above the bottom (the TeX baseline), "centre" half-way up.
  baseline_y <- depth  # depth is distance from baseline to bottom
  centre_y <- h / 2

  info <- grDevices::glyphInfo(
    id = as.integer(ids),
    x = x,
    y = y,
    font = as.integer(font_idx),
    size = sizes,
    fontList = font_list,
    width = w,
    height = h,
    hAnchor = grDevices::glyphAnchor(0, label = "left"),
    vAnchor = grDevices::glyphAnchor(
      c(0, baseline_y, centre_y),
      label = c("bottom", "baseline", "centre")
    ),
    col = cols
  )

  grid::glyphGrob(
    info,
    x = grid::unit(0, "bigpts"),
    y = grid::unit(0, "bigpts"),
    hjust = "left",
    vjust = "bottom",
    name = "glyphs"
  )
}

# ---- bezier curves ----

test_that("bezier curves produce correct output", {
  # cubic: return type, endpoints, default n
  cubic <- gridmicrotex:::cubic_bezier(0, 0, 1, 2, 2, 2, 3, 0)
  expect_named(cubic, c("x", "y"))
  expect_equal(cubic$x[1], 0)
  expect_equal(cubic$x[length(cubic$x)], 3)
  expect_length(cubic$x, 16)

  # quad: return type, endpoints, default n
  quad <- gridmicrotex:::quad_bezier(1, 2, 3, 4, 5, 6)
  expect_equal(quad$x[1], 1)
  expect_equal(quad$x[length(quad$x)], 5)
  expect_length(quad$x, 12)
})

# ---- build_path_grob ----

test_that("build_path_grob handles empty, simple, and complex paths", {
  # Empty → NULL
  pd_empty <- list(cmd = character(0),
                   coords = matrix(numeric(0), nrow = 0, ncol = 6))
  expect_null(gridmicrotex:::build_path_grob(pd_empty, "#000000", 1, 100))

  # Triangle → pathgrob with y-flip
  pd_tri <- list(
    cmd    = c("M", "L", "L", "Z"),
    coords = matrix(c(
      0, 10, 0, 0, 0, 0,
      10, 20, 0, 0, 0, 0,
      5,  5, 0, 0, 0, 0,
      0,  0, 0, 0, 0, 0
    ), nrow = 4, ncol = 6, byrow = TRUE)
  )
  tri_grob <- gridmicrotex:::build_path_grob(pd_tri, "#FF0000", 1, 50)
  expect_s3_class(tri_grob, "pathgrob")

  # Multiple subpaths
  pd_multi <- list(
    cmd    = c("M", "L", "L", "Z", "M", "L", "L", "Z"),
    coords = matrix(c(
      0, 0, 0, 0, 0, 0,  10, 0, 0, 0, 0, 0,  5, 10, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0,
      20, 0, 0, 0, 0, 0,  30, 0, 0, 0, 0, 0,  25, 10, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0
    ), nrow = 8, ncol = 6, byrow = TRUE)
  )
  multi_grob <- gridmicrotex:::build_path_grob(pd_multi, "#000000", 1, 100)
  expect_true(max(multi_grob$id) == 2)
})

# ---- build_latex_children ----

test_that("build_latex_children processes all layout types", {
  # Empty layout
  empty_df <- data.frame(
    type = character(0), x = numeric(0), y = numeric(0),
    stringsAsFactors = FALSE
  )
  empty_df$path <- list()
  expect_length(gridmicrotex:::build_latex_children(empty_df, 100), 0)

  # Full layout with path + line + fill_rect from a real expression
  layout <- gridmicrotex:::parse_latex_cpp(
    "\\begin{array}{|c|} \\hline \\frac{a}{\\cancel{b}} \\\\  \\hline \\end{array}",
    use_path = TRUE
  )
  children <- gridmicrotex:::build_latex_children(
    layout, attr(layout, "bbox_height"), render_mode = "path"
  )
  # One grob type per record type: glyph outlines as paths, the array's
  # rules as segments, and \cancel's strike as a rect. "more than zero"
  # would pass with the rules silently dropped.
  expect_setequal(vapply(children, function(k) class(k)[1], character(1)),
                  c("pathgrob", "segments", "rect"))

  # Glyph rows in typeface mode
  layout2 <- gridmicrotex:::parse_latex_cpp("a+b", use_path = FALSE)
  glyph_rows <- layout2[layout2$type == "glyph" &
                          !is.na(layout2$font_file) & nzchar(layout2$font_file), ]
  skip_if(nrow(glyph_rows) == 0, "No glyph rows with font_file in layout")
  children2 <- gridmicrotex:::build_latex_children(
    glyph_rows, attr(layout2, "bbox_height"), render_mode = "typeface"
  )
  expect_true(length(children2) > 0)

  # NA text rows are skipped
  na_row <- layout[1, ]
  na_row$type <- "text"
  na_row$text <- NA_character_
  expect_length(gridmicrotex:::build_latex_children(na_row, 100, render_mode = "path"), 0)
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

register_text_measurer <- function(fn) {
    invisible(.Call(`_gridmicrotex_register_text_measurer`, fn))
}

clear_text_measurer <- function() {
    invisible(.Call(`_gridmicrotex_clear_text_measurer`))
}

microtex_init <- function(clm_path, otf_path) {
    invisible(.Call(`_gridmicrotex_microtex_init`, clm_path, otf_path))
}

microtex_init_from_otf <- function(otf_path, index = 0L) {
    invisible(.Call(`_gridmicrotex_microtex_init_from_otf`, otf_path, index))
}

microtex_add_font <- function(clm_path, otf_path) {
    invisible(.Call(`_gridmicrotex_microtex_add_font`, clm_path, otf_path))
}

microtex_math_font_names <- function() {
    .Call(`_gridmicrotex_microtex_math_font_names`)
}

microtex_set_default_math_font <- function(name) {
    .Call(`_gridmicrotex_microtex_set_default_math_font`, name)
}

microtex_bidi_available <- function() {
    .Call(`_gridmicrotex_microtex_bidi_available`)
}

microtex_release <- function() {
    invisible(.Call(`_gridmicrotex_microtex_release`))
}

microtex_is_inited <- function() {
    .Call(`_gridmicrotex_microtex_is_inited`)
}

microtex_version <- function() {
    .Call(`_gridmicrotex_microtex_version`)
}

microtex_set_default_main_font <- function(family) {
    .Call(`_gridmicrotex_microtex_set_default_main_font`, family)
}

microtex_main_font_families <- function() {
    .Call(`_gridmicrotex_microtex_main_font_families`)
}

ot_math_table_bytes <- function(path, index = 0L) {
    .Call(`_gridmicrotex_ot_math_table_bytes`, path, index)
}

otf_to_clm_bytes <- function(path, index = 0L) {
    .Call(`_gridmicrotex_otf_to_clm_bytes`, path, index)
}

microtex_add_font_from_otf <- function(otf_path, index = 0L) {
    .Call(`_gridmicrotex_microtex_add_font_from_otf`, otf_path, index)
}

parse_latex_cpp <- function(tex, text_size = 20.0, line_space = 10.0, fg_color = "#000000", max_width = 0, math_font = "", main_font = "", use_path = TRUE, tex_style = "", justify = FALSE, optimal_break = FALSE) {
    .Call(`_gridmicrotex_parse_latex_cpp`, tex, text_size, line_space, fg_color, max_width, math_font, main_font, use_path, tex_style, justify, optimal_break)
}

# Syntax highlighting for fenced code blocks.
#
# Grammars are KDE/Kate syntax XML -- the format Kate, KDevelop and
# (through skylighting) Pandoc all read, so a user can point at a file
# from that ecosystem rather than one only this package understands.
# None of KDE's own files are shipped: most are GPL or LGPL and this
# package is MIT. A file *format* is not copyrightable, so the grammars
# in inst/highlight are written here, in KDE's format.
#
# Two backends produce the same thing -- one class per character of a
# line, NA meaning "leave it plain", which is skylighting's own rule that
# a NormalTok is not marked up at all:
#
#   R          utils::getParseData(), i.e. R's own parser. Exact.
#   everything else   the grammar engine below.
#
# The per-character vector is the whole reason the emitter cannot corrupt
# a line: it indexes the original string with substring(), so characters
# can be neither lost nor duplicated no matter what a grammar does.

# --- the class vocabulary --------------------------------------------

# KDE's defStyleNum enum maps 1:1 onto skylighting's short CSS class
# names, which is why this file has no naming decisions in it: the names
# below are what Pandoc -- and therefore knitr -- already emits, taken
# from skylighting's own `short` function. dsNormal is deliberately NA.
.HL_DEF_STYLE <- c(
  dsNormal         = NA_character_,
  dsKeyword        = "kw", dsFunction       = "fu", dsVariable    = "va",
  dsControlFlow    = "cf", dsOperator       = "op", dsBuiltIn     = "bu",
  dsExtension      = "ex", dsPreprocessor   = "pp", dsAttribute   = "at",
  dsChar           = "ch", dsSpecialChar    = "sc", dsString      = "st",
  dsVerbatimString = "vs", dsSpecialString  = "ss", dsImport      = "im",
  dsDataType       = "dt", dsDecVal         = "dv", dsBaseN       = "bn",
  dsFloat          = "fl", dsConstant       = "cn", dsComment     = "co",
  dsDocumentation  = "do", dsAnnotation     = "an", dsCommentVar  = "cv",
  dsInformation    = "in", dsWarning        = "wa", dsAlert       = "al",
  dsError          = "er", dsOthers         = "ot", dsRegionMarker = "re"
)

# Default colour per class, consumed by .md_default_rules(). GitHub's
# light palette: comment grey, string dark blue, keyword red, constant
# and number blue, function and built-in purple, type and variable
# brown. Only the classes the bundled grammars actually emit appear
# here, so nothing in this table is speculative.
.MD_CODE_COLORS <- c(
  co = "#59636E", ot = "#59636E",
  st = "#0A3069", ch = "#0A3069",
  kw = "#CF222E", cf = "#CF222E", pp = "#CF222E",
  cn = "#0550AE", dv = "#0550AE", bn = "#0550AE", fl = "#0550AE",
  at = "#0550AE", sc = "#0550AE",
  fu = "#8250DF", bu = "#8250DF",
  dt = "#953800", va = "#953800"
)

# --- language names --------------------------------------------------

# GFM spellings that should reach a grammar filed under another name.
# A grammar's own `alternativeNames` is registered on top of this when it
# loads, so a downloaded file brings its aliases with it.
.HL_ALIASES <- c(
  rscript = "r",
  py = "python", python3 = "python",
  sh = "bash", shell = "bash", zsh = "bash",
  "c++" = "cpp", cc = "cpp", cxx = "cpp", c = "cpp", hpp = "cpp",
  yml = "yaml",
  jl = "julia",
  tex = "latex",
  postgresql = "sql", mysql = "sql", sqlite = "sql"
)

# An info string carries more than a language name: knitr writes
# ```{r, echo=FALSE} and other tools append attributes. Take the first
# word and lowercase it, which is also what every markdown renderer does.
.hl_lang <- function(info) {
  # xml_attr() gives NA, not NULL, for a fence with no info string.
  if (length(info) != 1L || is.na(info)) return("")
  s <- trimws(info)
  if (!nzchar(s)) return("")
  s <- sub("^\\{+", "", s)
  s <- strsplit(s, "[[:space:],;}]+")[[1]][1]
  if (is.na(s) || !nzchar(s)) return("")
  tolower(s)
}

.hl_canonical <- function(lang) {
  if (!nzchar(lang)) return("")
  a <- .hl_state$aliases[[lang]] %||% unname(.HL_ALIASES[lang])
  if (is.null(a) || is.na(a)) lang else a
}

# --- registry --------------------------------------------------------

# Compiled grammars, keyed by canonical name, plus the alias map that
# grammars contribute as they load. Populated lazily on first use rather
# than in .onLoad: startup stays free, and nothing touches the filesystem
# until a code block actually asks for it.
.hl_state <- new.env(parent = emptyenv())
.hl_state$grammars <- list()
.hl_state$aliases <- list()
.hl_state$user <- character(0)
# Memoised classification, keyed on language + source. See .hl_classes().
.hl_state$cache <- list()

# Locate a bundled grammar. Mirrors .md_preset_path() -- grammars are
# data files rather than R code for the same reason the CSS presets are:
# each one doubles as a worked example a user can copy.
.hl_grammar_path <- function(name) {
  if (!grepl("^[a-z0-9_+-]+$", name)) return(NULL)
  f <- system.file("highlight", paste0(name, ".xml"), package = "gridmicrotex")
  if (!nzchar(f) || !file.exists(f)) NULL else f
}

.hl_grammar_names <- function() {
  d <- system.file("highlight", package = "gridmicrotex")
  if (!nzchar(d)) return(character(0))
  sub("[.]xml$", "", list.files(d, pattern = "[.]xml$"))
}

# Resolve a canonical name to a compiled grammar, compiling and caching a
# bundled file on first use. NULL when there is no such grammar, which is
# how an unrecognised fence ends up rendering plain.
.hl_grammar <- function(lang) {
  if (!nzchar(lang)) return(NULL)
  g <- .hl_state$grammars[[lang]]
  if (!is.null(g)) return(g)
  f <- .hl_grammar_path(lang)
  if (is.null(f)) return(NULL)
  .hl_store(lang, .hl_compile(f))
}

.hl_store <- function(lang, g, own = TRUE) {
  .hl_state$grammars[[lang]] <- g
  # `own` says the grammar is filed under the name it belongs to: a
  # bundled file, or a user registration that used the grammar's own
  # name. Otherwise its alternativeNames are ignored, because
  # `register_highlighter("mybash", bash.xml)` would quietly repoint
  # ```sh, ```zsh and ```shell at "mybash" -- hijacking three fence
  # languages the caller never named. (The bundled names cannot simply
  # be compared against g$name: cpp.xml declares "C++".)
  if (isTRUE(own)) {
    for (a in g$aliases) if (nzchar(a)) .hl_state$aliases[[a]] <- lang
  }
  g
}

# --- KDE XML -> an ordered pattern list -------------------------------

# Every PCRE metacharacter, backslash included -- a literal string from a
# grammar (StringDetect, a keyword item) must not be read as a pattern.
.hl_esc_re <- function(x) {
  gsub("([\\\\^$.|?*+()\\[\\]{}])", "\\\\\\1", x, perl = TRUE)
}

# Rule elements this engine implements. Anything else makes the grammar
# refuse to register rather than render approximately -- a half-supported
# grammar that silently colours the wrong things is worse than one that
# says it is not supported.
.HL_RULES <- c("keyword", "RegExpr", "DetectChar", "Detect2Chars",
               "StringDetect", "WordDetect", "AnyChar", "RangeDetect",
               "Int", "Float", "HlCOct", "HlCHex", "HlCChar",
               "HlCStringChar", "LineContinue",
               "DetectSpaces", "DetectIdentifier")

.hl_compile <- function(file) {
  # Internal DTD entities are expanded by xml2 without asking, which the
  # KDE files lean on heavily (python.xml defines a dozen).
  doc <- tryCatch(xml2::read_xml(file), error = function(e)
    stop("Could not parse ", sQuote(basename(file)), " as XML: ",
         conditionMessage(e), call. = FALSE))
  lang_nd <- if (identical(xml2::xml_name(doc), "language")) doc else
    xml2::xml_find_first(doc, "//language")
  if (inherits(lang_nd, "xml_missing")) {
    stop("Not a syntax grammar: no <language> element in ", sQuote(basename(file)),
         ".", call. = FALSE)
  }

  name <- tolower(xml2::xml_attr(lang_nd, "name") %|NA% "")
  alt <- xml2::xml_attr(lang_nd, "alternativeNames") %|NA% ""
  aliases <- tolower(trimws(strsplit(alt, "[;,]")[[1]]))
  aliases <- aliases[nzchar(aliases)]
  csens <- .hl_bool(xml2::xml_attr(lang_nd, "casesensitive"), default = TRUE)

  # attribute name -> CSS class, via defStyleNum.
  items <- xml2::xml_find_all(lang_nd, ".//itemDatas/itemData")
  attr_cls <- unname(.HL_DEF_STYLE[xml2::xml_attr(items, "defStyleNum")])
  names(attr_cls) <- xml2::xml_attr(items, "name")

  ctxs <- xml2::xml_find_all(lang_nd, ".//contexts/context")
  if (!length(ctxs)) {
    stop("Grammar ", sQuote(basename(file)), " has no <context>.", call. = FALSE)
  }
  # Keyword lists, by name.
  lists <- list()
  for (l in xml2::xml_find_all(lang_nd, ".//list")) {
    lists[[xml2::xml_attr(l, "name")]] <-
      trimws(xml2::xml_text(xml2::xml_find_all(l, "./item")))
  }

  bad <- character(0)
  note <- function(x) bad <<- c(bad, x)

  ctx_names <- xml2::xml_attr(ctxs, "name")

  # One rule element -> the compiled form the engine runs, or NULL.
  #
  # A rule this engine cannot represent is *skipped* when it would have
  # stayed in the current context, and *refuses the whole grammar* when
  # it would have switched. The asymmetry is the point: a dropped
  # in-context rule costs colour on the text it would have matched, which
  # is a visible but local loss. A dropped context switch leaves the
  # machine in the wrong state for the rest of the file -- an unclosed
  # string painting every following line -- which is the runaway
  # mis-colouring this design refuses everywhere.
  one_rule <- function(r, ctx_attr) {
    kind <- xml2::xml_name(r)
    target <- xml2::xml_attr(r, "context") %|NA% "#stay"
    switches <- !identical(target, "#stay")
    give_up <- function(what) {
      if (switches) note(what)
      NULL
    }
    if (!kind %in% .HL_RULES) return(give_up(paste0("<", kind, ">")))
    if (.hl_bool(xml2::xml_attr(r, "dynamic"))) {
      # %1 substitution from the match. A dynamic rule is usually the one
      # that closes a here-doc or a custom delimiter, so skipping one that
      # switches context would strand the machine.
      return(give_up(paste0("<", kind, " dynamic>")))
    }
    if (startsWith(target, "##")) {
      return(give_up(paste0("<", kind, " context=\"", target,
                            "\"> (another language)")))
    }
    re <- tryCatch(.hl_rule_regex(r, kind, lists, file),
                   error = function(e) NULL)
    if (is.null(re)) {
      # Most often a keyword list defined in another file, reached through
      # a cross-language include.
      return(give_up(paste0("<", kind, "> (unresolved ",
                            xml2::xml_attr(r, "String") %|NA% "rule", ")")))
    }
    col <- xml2::xml_attr(r, "column") %|NA% ""
    if (nzchar(col) && !identical(col, "0")) {
      return(give_up(paste0("<", kind, " column=\"", col, "\">")))
    }
    # A leading `^` is left in the pattern rather than lifted into
    # at_start. The engine matches `^.{k}\K(?:re)` against the whole
    # line, so an inner `^` still means start-of-line and simply fails
    # for k > 0 -- which is the correct reading. Lifting it applied the
    # anchor to the *whole* rule, so `^aaa|bbb` became `aaa|bbb`
    # restricted to column 0 and `bbb` stopped matching anywhere.
    #
    # column="0" and firstNonSpace are different: they are attributes of
    # the rule, not of the pattern, so the engine answers them from the
    # scan position instead of a lookbehind PCRE2 would reject.
    at_start <- identical(col, "0")
    first_non_space <- .hl_bool(xml2::xml_attr(r, "firstNonSpace"))
    if (.hl_bool(xml2::xml_attr(r, "insensitive")) ||
        (!csens && kind %in% c("keyword", "WordDetect", "StringDetect"))) {
      re <- paste0("(?i)", re)
    }
    a <- xml2::xml_attr(r, "attribute") %|NA% ctx_attr
    if (nzchar(a) && !a %in% names(attr_cls)) {
      # Always fatal: an attribute naming nothing is an authoring typo,
      # and a rule that matches but paints nothing is the silent failure
      # this package refuses to ship.
      note(paste0("attribute=\"", a, "\" (no <itemData> defines it)"))
      return(NULL)
    }
    list(re = paste0("(?:", re, ")"),
         class = if (nzchar(a)) unname(attr_cls[[a]]) else NA_character_,
         action = .hl_action(target, ctx_names, note),
         look_ahead = .hl_bool(xml2::xml_attr(r, "lookAhead")),
         at_start = at_start, first_non_space = first_non_space)
  }

  # Compile every context, keeping <IncludeRules> unresolved for now.
  ctx <- list()
  for (i in seq_along(ctxs)) {
    nd <- ctxs[[i]]
    own_attr <- xml2::xml_attr(nd, "attribute") %|NA% ""
    rules <- list()
    for (r in xml2::xml_children(nd)) {
      if (identical(xml2::xml_name(r), "IncludeRules")) {
        tgt <- xml2::xml_attr(r, "context") %|NA% ""
        # `##Language` pulls in another grammar file -- almost always
        # ##Alerts, ##Modelines or ##Doxygen, which only add TODO/FIXME
        # style marks *inside* a comment. Skipping one loses those marks
        # and nothing else: the comment is still a comment, because the
        # context that made it one is untouched. It never switches
        # context itself, so it is always safe to drop.
        if (!startsWith(tgt, "##") && tgt %in% ctx_names) {
          rules[[length(rules) + 1L]] <- list(include = tgt)
        }
        next
      }
      cr <- one_rule(r, own_attr)
      if (!is.null(cr)) rules[[length(rules) + 1L]] <- cr
    }
    ctx[[ctx_names[i]]] <- list(
      rules = rules,
      # A character no rule matches takes the context's own attribute --
      # this is how the *body* of a string or comment gets coloured at
      # all. Only the delimiters have rules of their own; everything
      # between them is unmatched text inside the pushed context.
      class = if (nzchar(own_attr) && own_attr %in% names(attr_cls))
        unname(attr_cls[[own_attr]]) else NA_character_,
      line_end = .hl_action(xml2::xml_attr(nd, "lineEndContext") %|NA% "#stay",
                            ctx_names, note),
      # fallthroughContext is frequently "#pop", not a context name, so
      # it has to go through the same action parser as everything else.
      fallthrough = {
        ft <- xml2::xml_attr(nd, "fallthroughContext") %|NA% ""
        if (nzchar(ft) && !startsWith(ft, "##"))
          .hl_action(ft, ctx_names, note) else NULL
      })
  }

  # Splice includes in, depth-first, with a visited set so a cycle cannot
  # loop forever.
  resolve <- function(nm, seen = character(0)) {
    if (nm %in% seen) return(list())
    out <- list()
    for (r in ctx[[nm]]$rules) {
      if (!is.null(r$include)) {
        out <- c(out, resolve(r$include, c(seen, nm)))
      } else {
        out[[length(out) + 1L]] <- r
      }
    }
    out
  }
  for (nm in names(ctx)) ctx[[nm]]$rules <- resolve(nm)

  if (length(bad)) {
    stop("Grammar ", sQuote(basename(file)), " uses constructs this ",
         "highlighter does not implement: ",
         paste(unique(bad), collapse = ", "), ".", call. = FALSE)
  }
  if (!length(unlist(lapply(ctx, function(c) c$rules), recursive = FALSE))) {
    stop("Grammar ", sQuote(basename(file)), " defines no usable rules.",
         call. = FALSE)
  }

  list(name = name, aliases = aliases, contexts = ctx, start = ctx_names[1])
}

# A KDE context target -> what the engine does to the stack.
#   #stay            nothing
#   #pop, #pop#pop   pop that many
#   #popN            pop N
#   #pop!Name        pop one, then push Name
#   Name             push Name
.hl_action <- function(s, ctx_names, note) {
  s <- s %|NA% "#stay"
  if (!nzchar(s) || identical(s, "#stay")) return(list(op = "stay"))
  if (startsWith(s, "#pop")) {
    push <- NA_character_
    bang <- regmatches(s, regexpr("![^#!]+$", s))
    if (length(bang) && nzchar(bang)) {
      push <- substring(bang, 2L)
      s <- sub("![^#!]+$", "", s)
      # Validated exactly as a plain push is. Without this an unknown
      # target compiles, the engine then finds no such context, and
      # highlighting dies for the rest of the line *and every line
      # after it* -- the stack never recovers.
      if (!push %in% ctx_names) {
        note(paste0("context=\"#pop!", push, "\" (no such <context>)"))
        push <- NA_character_
      }
    }
    num <- regmatches(s, regexpr("(?<=#pop)[0-9]+", s, perl = TRUE))
    n <- if (length(num) && nzchar(num)) as.integer(num) else
      length(gregexpr("#pop", s, fixed = TRUE)[[1]])
    return(list(op = "pop", n = max(1L, n), push = push))
  }
  if (startsWith(s, "#")) return(list(op = "stay"))
  if (!s %in% ctx_names) {
    note(paste0("context=\"", s, "\" (no such <context>)"))
    return(list(op = "stay"))
  }
  list(op = "push", name = s)
}

.hl_push <- function(stack, a) {
  switch(a$op,
    stay = stack,
    pop = {
      stack <- stack[seq_len(max(1L, length(stack) - a$n))]
      if (!is.na(a$push)) c(stack, a$push) else stack
    },
    push = c(stack, a$name),
    stack)
}

# One KDE rule element -> one PCRE pattern.
.hl_rule_regex <- function(r, kind, lists, file) {
  at <- function(a) xml2::xml_attr(r, a) %|NA% ""
  switch(kind,
    keyword = {
      nm <- at("String")
      words <- lists[[nm]]
      if (is.null(words) || !length(words)) {
        stop("Grammar ", sQuote(basename(file)), " refers to keyword list ",
             sQuote(nm), ", which it does not define.", call. = FALSE)
      }
      # Longest first so `else if` cannot be shadowed by `else`, and so
      # `<=>` wins over `<`.
      words <- words[nzchar(words)]
      words <- words[order(nchar(words), decreasing = TRUE)]
      # A word boundary is added per item, not around the whole group: a
      # symbol keyword such as `+` or `<=>` has no word character to
      # bound, so a blanket \b(?:...)\b made it impossible to match.
      # Operator keywords are common in imported grammars.
      lead <- ifelse(grepl("^\\w", words), "\\b", "")
      trail <- ifelse(grepl("\\w$", words), "\\b", "")
      paste0("(?:",
             paste0(lead, .hl_esc_re(words), trail, collapse = "|"), ")")
    },
    RegExpr = at("String"),
    DetectChar = .hl_esc_re(at("char")),
    Detect2Chars = paste0(.hl_esc_re(at("char")), .hl_esc_re(at("char1"))),
    StringDetect = .hl_esc_re(at("String")),
    WordDetect = paste0("\\b", .hl_esc_re(at("String")), "\\b"),
    AnyChar = paste0("[", gsub("([\\\\^\\]\\[-])", "\\\\\\1", at("String"),
                               perl = TRUE), "]"),
    RangeDetect = paste0(.hl_esc_re(at("char")), "[^", .hl_esc_re(at("char1")),
                         "]*", .hl_esc_re(at("char1"))),
    Int = "\\b[0-9]+\\b",
    Float = "\\b[0-9]*\\.[0-9]+(?:[eE][-+]?[0-9]+)?\\b|\\b[0-9]+\\.(?![.0-9])",
    HlCOct = "\\b0[0-7]+\\b",
    HlCHex = "\\b0[xX][0-9a-fA-F]+\\b",
    HlCChar = "'(?:\\\\(?:[abefnrtv?\"'\\\\]|x[0-9a-fA-F]+|[0-7]{1,3})|[^\\\\'])'",
    HlCStringChar =
      "\\\\(?:[abefnrtv?\"'\\\\]|x[0-9a-fA-F]+|[0-7]{1,3}|u[0-9a-fA-F]{4}|U[0-9a-fA-F]{8})",
    LineContinue = "\\\\$",
    DetectSpaces = "[ \\t]+",
    DetectIdentifier = "[A-Za-z_][A-Za-z0-9_]*",
    NULL
  )
}

# --- the engine ------------------------------------------------------

# Kate's own algorithm: at each position, try the current context's rules
# in order and take the first that matches there; assign its class,
# apply its context switch, and resume past what it consumed. The
# context stack carries across lines via lineEndContext, which is what
# lets a string or block comment span them.
#
# Scanning by position rather than pre-finding every match is what makes
# a *context* grammar possible at all -- the set of live rules changes
# mid-line -- and it also gets these right without the grammar having to
# choose between them:
#
#   # say "hi"      the comment starts first, so it takes the whole line
#   x <- "a # b"    the string starts first, so the # inside is not a comment
.hl_patterns <- function(lines, g) {
  stack <- g$start
  out <- vector("list", length(lines))

  for (li in seq_along(lines)) {
    line <- lines[li]
    n <- nchar(line)
    cls <- rep(NA_character_, n)
    pos <- 1L
    nonblank <- attr(regexpr("^[ \t]*", line), "match.length") + 1L
    # Bounds the work done without consuming a character, so a lookAhead
    # rule that never advances, or a pair of fallthroughs that bounce
    # between two contexts, cannot hang a plot.
    spin <- 0L

    while (pos <= n) {
      cur <- g$contexts[[stack[length(stack)]]]
      if (is.null(cur)) break
      # `\K` drops the skipped prefix from the reported match, so the
      # rule is tried at exactly `pos` while the characters to its left
      # stay visible. Matching a substr() instead would hide them, and
      # \b would then fire inside a word -- colouring the `in` of
      # `begin` as a keyword -- while every lookbehind silently failed.
      # Verified that .{k} counts characters, not bytes.
      pre <- paste0("^.{", pos - 1L, "}\\K")

      hit <- NULL
      len <- 0L
      for (r in cur$rules) {
        if (r$at_start && pos != 1L) next
        if (r$first_non_space && pos != nonblank) next
        m <- tryCatch(regexpr(paste0(pre, r$re), line, perl = TRUE,
                              useBytes = FALSE),
                      error = function(e) -1L)
        if (m[1] != pos) next
        hit <- r
        len <- attr(m, "match.length")
        break
      }

      if (is.null(hit)) {
        if (!is.null(cur$fallthrough) && spin < 20L) {
          stack <- .hl_push(stack, cur$fallthrough)
          spin <- spin + 1L
          next
        }
        cls[pos] <- cur$class
        pos <- pos + 1L
        spin <- 0L
        next
      }

      if (!hit$look_ahead && len > 0L) {
        cls[pos:min(pos + len - 1L, n)] <- hit$class
      }
      stack <- .hl_push(stack, hit$action)
      if (hit$look_ahead || len <= 0L) {
        # Zero-width or look-ahead: the context change is the point, but
        # something must still give or we would sit here forever.
        spin <- spin + 1L
        if (spin > 20L) { pos <- pos + 1L; spin <- 0L }
      } else {
        pos <- pos + len
        spin <- 0L
      }
    }

    out[[li]] <- cls
    cur <- g$contexts[[stack[length(stack)]]]
    if (!is.null(cur)) stack <- .hl_push(stack, cur$line_end)
  }
  out
}

# --- the R backend ---------------------------------------------------

.hl_r_class <- function(token, text) {
  switch(token,
    COMMENT = "co",
    STR_CONST = "st",
    NULL_CONST = "cn",
    SYMBOL_FUNCTION_CALL = "fu",
    # `T` and `F` are not here on purpose: the parser reports them as
    # SYMBOL, not NUM_CONST, because they are ordinary bindings.
    NUM_CONST = if (text %in% c("TRUE", "FALSE", "NA", "Inf", "NaN"))
      "cn" else if (grepl("[.eE]", text)) "fl" else "dv",
    IF = , ELSE = , FOR = , WHILE = , REPEAT = , BREAK = , NEXT = "cf",
    FUNCTION = , IN = "kw",
    NA_character_
  )
}

# R gets its own parser rather than a grammar: it is exact, it is already
# installed, and it sees the whole block at once so a string spanning
# several lines is classified correctly. Returns NULL when the block does
# not parse -- a fenced block is often a fragment -- and the caller then
# falls back to the r.xml grammar.
.hl_r <- function(lines) {
  # Errors only. Catching warnings here would silently drop R's exact
  # backend for a block that parses perfectly well but happens to warn.
  pd <- tryCatch(
    utils::getParseData(parse(text = lines, keep.source = TRUE)),
    error = function(e) NULL)
  if (is.null(pd) || !nrow(pd)) return(NULL)
  pd <- pd[pd$terminal, , drop = FALSE]
  if (!nrow(pd)) return(NULL)

  out <- lapply(nchar(lines), function(n) rep(NA_character_, n))
  for (i in seq_len(nrow(pd))) {
    cls <- .hl_r_class(pd$token[i], pd$text[i])
    if (is.na(cls)) next
    l1 <- pd$line1[i]; l2 <- pd$line2[i]
    if (l1 < 1L || l2 > length(lines)) next
    for (ln in l1:l2) {
      n <- length(out[[ln]])
      if (!n) next
      from <- if (ln == l1) pd$col1[i] else 1L
      to <- if (ln == l2) pd$col2[i] else n
      # getParseData reports byte offsets under some encodings. Rather
      # than guess, check the span really is the token before trusting
      # it; a mismatch leaves those characters plain.
      if (ln == l1 && ln == l2 &&
          !identical(substring(lines[ln], from, to), pd$text[i])) next
      if (from < 1L || to > n || to < from) next
      out[[ln]][from:to] <- cls
    }
  }
  out
}

# --- entry point -----------------------------------------------------

# Classes for every line of a code block, or NULL for "render plain".
# Never signals: a highlighter that threw would take a user's plot with
# it, and a fence naming a language we do not have is not an error -- it
# is what GitHub does too.
.hl_classes <- function(lines, info) {
  tryCatch({
    lang <- .hl_canonical(.hl_lang(info))
    if (!nzchar(lang) || !length(lines)) return(NULL)
    # Classification runs from .md_layout(), which is makeContent() --
    # so it repeats on every single draw, and a resize or an animation
    # pays it again each frame. Measured on a 100-line python block it
    # is 0.59s of a 1.09s draw. The answer for identical input never
    # changes, so remember it.
    key <- paste0(lang, "\r", paste(lines, collapse = "\n"))
    hit <- .hl_state$cache[[key]]
    if (!is.null(hit)) return(hit$value)

    out <- if (identical(lang, "r")) .hl_r(lines) else NULL
    if (is.null(out)) {
      g <- .hl_grammar(lang)
      if (is.null(g)) return(NULL)
      out <- .hl_patterns(lines, g)
    }
    # Bounded so a document with many distinct blocks cannot grow it
    # without limit; the whole cache is dropped rather than aged, which
    # needs no bookkeeping and costs one recomputation.
    if (length(.hl_state$cache) >= 64L) .hl_state$cache <- list()
    .hl_state$cache[[key]] <- list(value = out)
    out
  }, error = function(e) NULL)
}

# xml_attr() returns NA, not "", for an absent attribute.
`%|NA%` <- function(x, y) if (length(x) != 1L || is.na(x)) y else x

# KDE writes booleans both ways and mixes them within one file: python.xml
# has lookAhead="1" ninety times and lookAhead="true" not once, while
# insensitive appears as both. Reading only "true" silently turned every
# look-ahead rule into a consuming one.
.hl_bool <- function(x, default = FALSE) {
  if (length(x) != 1L || is.na(x) || !nzchar(x)) return(default)
  tolower(x) %in% c("1", "true", "on", "yes")
}

# --- one code line -> TeX --------------------------------------------

# Split a line into the runs that will become \text{} groups: one per
# stretch of the same token class, plus one per space wherever spaces
# run two or more deep. A code block used to lose its indentation
# entirely because the whole line went into a single run and MicroTeX
# collapses a run of spaces inside one \text{}.
#
# Kept apart from the TeX building so the invariant that matters can be
# asserted directly -- paste0(.hl_segments(line, cls)$text) == line --
# with no escaping to undo first. Segments index the original line with
# substring(), so the emitted text is the input text whatever the
# classifier did.
.hl_segments <- function(line, cls) {
  n <- nchar(line)
  if (!n) return(data.frame(text = character(0), class = character(0),
                            space = logical(0), stringsAsFactors = FALSE))
  ch <- substring(line, seq_len(n), seq_len(n))
  if (length(cls) != n) cls <- rep(NA_character_, n)
  sp <- ch == " "
  key <- ifelse(is.na(cls), "", cls)

  # Only a *run* of two or more spaces collapses inside one \text{}; a
  # lone space is kept (\text{a }\text{b} and \text{a b} both measure
  # 21bp at 12pt). Splitting only where it is needed matters: MicroTeX
  # emits one draw record per run, so splitting at every space would
  # multiply the records in a code block sixfold and undo the file-size
  # win of drawing text a line at a time.
  multi <- if (n == 1L) FALSE else sp & (c(FALSE, sp[-n]) | c(sp[-1], FALSE))
  brk <- if (n == 1L) TRUE else
    c(TRUE, multi[-1] | multi[-n] | key[-1] != key[-n])
  grp <- cumsum(brk)
  ends <- c(which(diff(grp) != 0L), n)
  starts <- c(1L, utils::head(ends, -1L) + 1L)

  data.frame(text = substring(line, starts, ends),
             class = cls[starts], space = sp[starts],
             stringsAsFactors = FALSE)
}

.hl_code_tex <- function(line, cls, colour) {
  # An empty source line still occupies a row.
  if (!nchar(line)) return("\\texttt{\\text{ }}")
  seg <- .hl_segments(line, cls)
  out <- paste0("\\text{", .md_escape_tex(seg$text), "}")
  for (i in seq_len(nrow(seg))) {
    if (seg$space[i] || is.na(seg$class[i])) next
    col <- colour(seg$class[i])
    if (!is.null(col)) out[i] <- paste0("\\textcolor{", col, "}{", out[i], "}")
  }
  paste0("\\texttt{", paste0(out, collapse = ""), "}")
}

# --- user-facing -----------------------------------------------------

#' Add a syntax highlighting grammar
#'
#' Registers a KDE/Kate syntax definition so that fenced code blocks
#' tagged with \code{lang} are highlighted by
#' \code{\link{markdown_box_grob}}. Ten languages are built in --- see
#' \code{\link{available_highlighters}} --- and this is how to add
#' another.
#'
#' The grammar is an XML file in the format used by Kate, KDevelop and,
#' through \pkg{skylighting}, Pandoc. The simplest way to write one is to
#' copy a built-in grammar and edit it:
#'
#' \preformatted{
#'   file.copy(system.file("highlight", "python.xml",
#'                         package = "gridmicrotex"),
#'             "mylang.xml")
#' }
#'
#' Because the format is KDE's, the several hundred definitions upstream
#' are a useful \emph{reference} when writing your own --- one XML file
#' per language, catalogued at \url{https://kate-editor.org/syntax/},
#' repository at
#' \url{https://invent.kde.org/frameworks/syntax-highlighting}.
#'
#' Many load and work directly: of twenty sampled, thirteen registered,
#' including \code{python} (249 contexts), \code{css}, \code{yaml},
#' \code{makefile} and \code{go}. The seven that did not --- \code{bash},
#' \code{ruby}, \code{perl}, \code{rust}, \code{lua},
#' \code{javascript} and \code{markdown} --- all use \emph{dynamic}
#' rules, which substitute part of the match into a later pattern and are
#' not implemented.
#'
#' Cross-language includes (\code{##Alerts}, \code{##Doxygen} and
#' friends) are skipped rather than followed. Those only add TODO/FIXME
#' marks \emph{inside} a comment, so the comment is still a comment; the
#' marks are all that is lost.
#'
#' They also carry their own licences, mostly GPL or LGPL, which is why
#' none is bundled with this MIT-licensed package. Borrowing from one
#' locally is your own decision; redistributing it is subject to its
#' licence.
#'
#' Colours come from the \code{defStyleNum} of each \code{<itemData>},
#' which maps onto the same CSS class names \pkg{knitr} and Pandoc use
#' (\code{kw} for a keyword, \code{co} for a comment, \code{st} for a
#' string, and so on), so a grammar needs no colour information of its
#' own --- restyle with \code{\link{markdown_style}}.
#'
#' At each position the current context's rules are tried in document
#' order and the first to match wins. Contexts, \code{IncludeRules},
#' \code{lookAhead} and \code{fallthroughContext} all work, and the
#' context stack carries across lines, so a string or block comment may
#' span them.
#'
#' A rule this engine cannot represent is skipped when it would have
#' stayed in the current context --- costing colour on the text it
#' matched and nothing more --- but \emph{refuses the grammar} when it
#' would have switched, because a lost context switch leaves the machine
#' in the wrong state and paints the rest of the file wrongly. The
#' refusal names the construct.
#'
#' @param lang Language name, as written after the opening fence. Case is
#'   ignored. Registering a name that is already built in replaces it.
#' @param file Path to a KDE syntax XML grammar.
#' @return Invisibly, \code{lang}.
#' @seealso \code{\link{available_highlighters}},
#'   \code{\link{markdown_box_grob}}, \code{\link{markdown_style}}
#' @export
#'
#' @examples
#' # Registering a grammar under a name of your own.
#' f <- system.file("highlight", "python.xml", package = "gridmicrotex")
#' register_highlighter("mypython", f)
#' "mypython" %in% available_highlighters()
register_highlighter <- function(lang, file) {
  stopifnot(is.character(lang), length(lang) == 1L, !is.na(lang), nzchar(lang))
  stopifnot(is.character(file), length(file) == 1L, !is.na(file))
  if (!file.exists(file)) {
    stop("Grammar file not found: ", sQuote(file), call. = FALSE)
  }
  lang <- tolower(lang)
  # Compile now, so a broken grammar fails at this call rather than in
  # the middle of drawing a plot.
  g <- .hl_compile(file)
  .hl_store(lang, g, own = identical(lang, g$name))
  # Drop memoised classifications: editing a grammar and registering it
  # again must change what the next draw produces. Only re-registration
  # can invalidate them -- a lazy first load of a bundled grammar adds a
  # language nothing was cached under, so it must not clear the cache
  # (that would wipe it on first use of every language in a document).
  .hl_state$cache <- list()
  .hl_state$user <- union(.hl_state$user, lang)
  invisible(lang)
}

#' Syntax highlighting languages available
#'
#' Names accepted after an opening code fence in
#' \code{\link{markdown_box_grob}}. Includes both the built-in grammars
#' and any added with \code{\link{register_highlighter}}.
#'
#' Several common aliases also work and are not listed here, among them
#' \code{py}, \code{sh}, \code{c++}, \code{yml}, \code{jl} and
#' \code{tex}. A fence naming anything else renders as plain monospace
#' text, without a warning.
#'
#' @return A character vector of language names, sorted.
#' @seealso \code{\link{register_highlighter}}
#' @export
#'
#' @examples
#' available_highlighters()
available_highlighters <- function() {
  sort(union(.hl_grammar_names(), .hl_state$user))
}

# Helpers shared by the tests below.

# The class of each character of one line, as a plain vector.
hl <- function(line, lang) {
  cls <- .hl_classes(line, lang)
  if (is.null(cls)) NULL else cls[[1]]
}

# The class assigned to the first occurrence of `what` in `line`.
class_of <- function(line, lang, what) {
  cls <- hl(line, lang)
  if (is.null(cls)) return(NA_character_)
  at <- regexpr(what, line, fixed = TRUE)
  if (at < 0) stop("`", what, "` is not in `", line, "`")
  unique(cls[at:(at + nchar(what) - 1L)])
}

# Width of a code line in bigpts, as the emitter renders it.
code_w <- function(line, cls = NULL) {
  tex <- .hl_code_tex(line, cls, function(k) NULL)
  d <- latex_dims(tex, input_mode = "math",
                  gp = grid::gpar(fontsize = 12, fontfamily = "mono"))
  as.numeric(grid::convertWidth(d$width, "bigpts"))
}

# Every LaTeX string the box grob actually builds, gathered by walking
# the children makeContent() produces. Drives the real entry point.
box_tex <- function(md, style = NULL) {
  g <- markdown_box_grob(md, width = grid::unit(4, "in"), style = style)
  out <- character(0)
  walk <- function(x) {
    if (!is.null(x$tex)) out <<- c(out, x$tex)
    for (k in x$children) walk(k)
  }
  walk(grid::makeContent(g))
  out
}

# Write a grammar to a temp file and return its path.
grammar_file <- function(rules, lists = "", lang = "Toy", csens = "1",
                         contexts = "") {
  f <- tempfile(fileext = ".xml")
  writeLines(sprintf(
    '<?xml version="1.0" encoding="UTF-8"?>
     <language name="%s" version="1" kateversion="5.0" casesensitive="%s">
       <highlighting>
         %s
         <contexts>
           <context name="Normal" attribute="Normal Text" lineEndContext="#stay">
             %s
           </context>
           %s
         </contexts>
         <itemDatas>
           <itemData name="Normal Text"  defStyleNum="dsNormal"/>
           <itemData name="Comment"      defStyleNum="dsComment"/>
           <itemData name="String"       defStyleNum="dsString"/>
           <itemData name="Keyword"      defStyleNum="dsKeyword"/>
           <itemData name="Control Flow" defStyleNum="dsControlFlow"/>
           <itemData name="Data Type"    defStyleNum="dsDataType"/>
           <itemData name="Decimal"      defStyleNum="dsDecVal"/>
           <itemData name="Char"         defStyleNum="dsChar"/>
           <itemData name="BaseN"        defStyleNum="dsBaseN"/>
         </itemDatas>
       </highlighting>
     </language>', lang, csens, lists, rules, contexts), f)
  f
}


# --- indentation ------------------------------------------------------

test_that("leading whitespace in a code block survives", {
  # The defect this replaces: a whole line went into one \text{} run and
  # MicroTeX collapses a run of spaces inside one run to a single space,
  # so eight leading spaces measured exactly one space wide.
  one_char <- code_w("xx") - code_w("x")
  base <- code_w("return 1")
  for (n in c(2L, 4L, 8L, 12L)) {
    indented <- code_w(paste0(strrep(" ", n), "return 1"))
    # Within half a character of n characters wider.
    expect_equal(indented - base, n * one_char, tolerance = 0.5 * one_char,
                 info = paste(n, "spaces"))
  }
})

test_that("interior alignment is preserved, not just the left margin", {
  narrow <- code_w("a = 1")
  wide <- code_w("a     = 1")
  one_char <- code_w("xx") - code_w("x")
  expect_equal(wide - narrow, 4 * one_char, tolerance = 0.5 * one_char)
})

test_that("code with no repeated spaces renders exactly as it did before", {
  # Guards the claim that the emitter change is width-neutral for
  # ordinary code: a single space between runs measures the same as a
  # space inside one run, so nothing already in a snapshot moves.
  for (line in c("x<-foo(1)", "a b", "if (x) y else z")) {
    expect_equal(code_w(line),
                 {
                   tex <- paste0("\\texttt{\\text{", .md_escape_tex(line), "}}")
                   d <- latex_dims(tex, input_mode = "math",
                                   gp = grid::gpar(fontsize = 12,
                                                   fontfamily = "mono"))
                   as.numeric(grid::convertWidth(d$width, "bigpts"))
                 },
                 info = line)
  }
})

test_that("~ and ^ are literal characters, not accents", {
  # They used to be escaped to \~{} and \^{}, which are the tilde and
  # circumflex *accents*: `lm(y ~ x)` drew its tilde floating at cap
  # height instead of on the baseline. `^` needs no escape at all, and
  # `~` needs \char126{} -- left bare it is a non-breaking space and the
  # character disappears entirely.
  drawn <- function(line) {
    df <- parse_latex_cpp(.hl_code_tex(line, NULL, function(k) NULL),
                          text_size = 12, use_path = FALSE)
    d <- df[df$type %in% c("text", "glyph"), ]
    paste0(ifelse(is.na(d$text), "<GLYPH>", d$text), collapse = "")
  }
  for (line in c("y ~ x", "a^b", "~/.bashrc", "2^32", "x <- y ~ z")) {
    expect_identical(drawn(line), line, info = line)
  }
  # An accent is drawn as a separate glyph; a literal stays in the run.
  df <- parse_latex_cpp(.hl_code_tex("y ~ x", NULL, function(k) NULL),
                        text_size = 12, use_path = FALSE)
  expect_equal(sum(df$type == "glyph"), 0L)
  # And each occupies exactly one monospace cell, like any operator.
  expect_equal(code_w("y ~ x"), code_w("y - x"))
  expect_equal(code_w("a^b"), code_w("a-b"))
  # The escaper is shared with prose, which had the same fault.
  expect_match(.md_to_tex("approx ~5"), "char126", fixed = TRUE)
  expect_false(grepl("\\~{}", .md_to_tex("approx ~5"), fixed = TRUE))
})

test_that("nothing in a code block is interpreted as math or LaTeX", {
  # Math is masked out of the whole document before CommonMark parses it,
  # and CommonMark cannot tell it about fences. Left masked, the span's
  # *content* is lost and the sentinel's index is drawn in its place:
  # `# see $E = mc^2$ here` came out as `# see 1 here`. Restored here, and
  # then neutralised by the ordinary escaping.
  for (line in c("# see $E = mc^2$ here", "cost is $5 and $10",
                 "s <- \"$x^2$\"", "a $$ b")) {
    tex <- box_tex(paste0("~~~\n", line, "\n~~~\n"))
    # Every dollar survives, escaped, so none can open math.
    expect_equal(lengths(regmatches(tex, gregexpr("\\$", tex, fixed = TRUE))),
                 lengths(regmatches(line, gregexpr("$", line, fixed = TRUE))),
                 info = line)
    expect_false(grepl("\\text{}", tex, fixed = TRUE), info = line)
  }
  # The content itself is intact, not replaced by a sentinel index.
  expect_match(box_tex("~~~\n# see $E = mc^2$ here\n~~~\n"),
               "E = mc^2", fixed = TRUE)
  # And with a language, so the classifier sees the real source too --
  # the dollars are escaped, which is what stops them opening math.
  expect_match(box_tex("~~~r\nx <- \"$a$\"\n~~~\n"), "\\$a\\$", fixed = TRUE)
  # An inline code span was already correct; keep it that way.
  expect_match(.md_to_tex("`$x$`"), "\\$x\\$", fixed = TRUE)
  # Prose outside a fence must still render real math, not a literal.
  expect_match(.md_to_tex("value $x$ here"), "\\text{value }x", fixed = TRUE)
})

test_that("#pop!Context validates its push target", {
  # Unvalidated, an unknown target compiles, the engine then finds no
  # such context, and highlighting dies for the rest of the line *and
  # every line after it* -- the stack never recovers. Exactly the
  # runaway this design refuses everywhere else.
  f <- grammar_file('<DetectChar attribute="String" char="q" context="S"/>',
                    contexts = '<context name="S" attribute="String" lineEndContext="#stay">
                       <DetectChar attribute="String" char="z" context="#pop!Nope"/>
                     </context>')
  expect_error(register_highlighter("popbang", f), "Nope")
  expect_false("popbang" %in% available_highlighters())

  # The valid form still works.
  ok <- grammar_file('<DetectChar attribute="String" char="q" context="S"/>',
                     contexts = '<context name="S" attribute="String" lineEndContext="#stay">
                        <DetectChar attribute="Keyword" char="z" context="#pop!K"/>
                      </context>
                      <context name="K" attribute="Keyword" lineEndContext="#stay"/>')
  expect_silent(register_highlighter("popbang2", ok))
})

test_that("a leading ^ anchors only its own alternative", {
  # Lifting the ^ out into an at_start flag applied it to the whole
  # rule, so `^aaa|bbb` became `aaa|bbb` restricted to column 0 and
  # `bbb` stopped matching anywhere at all.
  f <- grammar_file('<RegExpr attribute="Keyword" String="^aaa|bbb"/>')
  register_highlighter("anch", f)
  expect_identical(class_of("aaa xx bbb", "anch", "aaa"), "kw")
  expect_identical(class_of("aaa xx bbb", "anch", "bbb"), "kw")
  expect_identical(class_of("zz bbb", "anch", "bbb"), "kw")
  # ...and the anchor still bites: `aaa` mid-line is not a match.
  expect_true(is.na(class_of("zz aaa", "anch", "aaa")))
})

test_that("registering under a custom name does not hijack aliases", {
  # `register_highlighter("mybash", bash.xml)` used to repoint ```sh,
  # ```zsh and ```shell at "mybash" -- three fence languages the caller
  # never named.
  expect_identical(.hl_canonical("sh"), "bash")
  register_highlighter("mybash", .hl_grammar_path("bash"))
  expect_identical(.hl_canonical("sh"), "bash")
  expect_true("mybash" %in% available_highlighters())
  # A grammar registered under its own name still contributes aliases.
  register_highlighter("yaml", .hl_grammar_path("yaml"))
  expect_identical(.hl_canonical("yml"), "yaml")
})

test_that("symbol keywords in a <list> match", {
  # \b around the whole alternation cannot bound `+` or `<=>`, so
  # operator keywords silently never matched.
  f <- tempfile(fileext = ".xml")
  writeLines('<?xml version="1.0" encoding="UTF-8"?>
    <language name="Sym" version="1" kateversion="5.0" casesensitive="1">
      <highlighting>
        <list name="ops"><item>+</item><item>&lt;=&gt;</item><item>mod</item></list>
        <contexts><context name="Normal" attribute="Normal Text" lineEndContext="#stay">
          <keyword attribute="Keyword" String="ops"/>
        </context></contexts>
        <itemDatas><itemData name="Normal Text" defStyleNum="dsNormal"/>
          <itemData name="Keyword" defStyleNum="dsKeyword"/>
        </itemDatas></highlighting></language>', f)
  register_highlighter("syms", f)
  expect_identical(class_of("a + b", "syms", "+"), "kw")
  expect_identical(class_of("a <=> b", "syms", "<=>"), "kw")
  expect_identical(class_of("a mod b", "syms", "mod"), "kw")
  # A word keyword still may not match inside an identifier.
  expect_true(is.na(class_of("modulo", "syms", "mod")))
})

test_that("classification is memoised, and re-registering clears it", {
  # .md_layout() runs at draw time, so without this every redraw
  # reclassifies: 0.59s of a 1.09s draw on a 100-line block.
  code <- rep("x <- foo(1)  # hi", 20)
  .hl_state$cache <- list()
  a <- .hl_classes(code, "r")
  expect_length(.hl_state$cache, 1L)
  b <- .hl_classes(code, "r")
  expect_identical(a, b)
  # A different language is a different entry, not a collision.
  .hl_classes(code, "python")
  expect_length(.hl_state$cache, 2L)
  # Registering anything drops it, so an edited grammar takes effect.
  register_highlighter("yaml", .hl_grammar_path("yaml"))
  expect_length(.hl_state$cache, 0L)
})

test_that("a tab advances to the next tab stop, not by a fixed four", {
  # Makefiles have no choice about tabs, and Go is tab-indented by
  # convention; expanding to a fixed four misaligns everything after
  # the first partial column.
  expect_identical(.md_expand_tabs("a\tb"), "a   b")
  expect_identical(.md_expand_tabs("abc\tx"), "abc x")
  expect_identical(.md_expand_tabs("abcd\tx"), "abcd    x")
  expect_identical(.md_expand_tabs("\tx"), "    x")
  expect_identical(.md_expand_tabs("a\tb\tc"), "a   b   c")
  expect_identical(.md_expand_tabs("no tabs"), "no tabs")
  # Column counting restarts per line.
  g <- .md_parse_doc("~~~\nab\tx\ncd\ty\n~~~\n")
  nd <- xml2::xml_find_first(g$doc, "//code_block")
  expect_identical(.md_code_lines(nd, g$spans), c("ab  x", "cd  y"))
})

test_that("tabs become spaces so they can be seen at all", {
  doc <- xml2::read_xml(commonmark::markdown_xml("```\n\tx\n```\n"))
  nd <- xml2::xml_find_first(doc, "//*[local-name()='code_block']")
  expect_identical(.md_code_lines(nd), "    x")
})


# --- the segmentation invariant --------------------------------------

test_that("segments reproduce the source line exactly", {
  # The reason the emitter cannot corrupt code: every segment is a
  # substring of the original line, so no classifier -- ours or a user's
  # -- can drop or duplicate a character.
  cases <- list(
    c("x <- \"a # b\"", "r"),
    c("# say \"hi\"", "python"),
    c("    deep    indent   ", "python"),
    c("aéb # café ü", "python"),
    c("f'{x!r}' + \"unterminated", "python"),
    c("SELECT 'it''s';", "sql"),
    c("<<>>&%$_^~\\{}", "cpp"),
    c("   ", "yaml"),
    c("中文 = 1  # コメント", "python")
  )
  for (cc in cases) {
    cls <- hl(cc[1], cc[2])
    seg <- .hl_segments(cc[1], cls)
    expect_identical(paste0(seg$text, collapse = ""), cc[1], info = cc[1])
    expect_identical(sum(nchar(seg$text)), nchar(cc[1]), info = cc[1])
  }
})

test_that("only runs of spaces are split, and no segment mixes classes", {
  # A run of spaces collapses inside one \text{} and must be split; a
  # lone space does not, and splitting it would multiply the draw
  # records in every code block for nothing.
  expect_identical(.hl_segments("a   b", NULL)$text, c("a", " ", " ", " ", "b"))
  expect_identical(.hl_segments("a b", NULL)$text, "a b")
  expect_identical(.hl_segments("a b c d", NULL)$text, "a b c d")
  expect_identical(.hl_segments("  a", NULL)$text, c(" ", " ", "a"))
  expect_identical(.hl_segments(" ", NULL)$text, " ")
  cls <- hl("x <- 1  # hi", "r")
  seg <- .hl_segments("x <- 1  # hi", cls)
  expect_true(all(vapply(seq_len(nrow(seg)),
                         function(i) length(unique(cls[
                           seq(sum(nchar(seg$text[seq_len(i - 1)])) + 1L,
                               length.out = nchar(seg$text[i]))])) == 1L,
                         logical(1))))
})


# --- the R backend ----------------------------------------------------

test_that("R is classified by R's own parser", {
  line <- "y <- plot(1.5, TRUE)  # note"
  expect_identical(class_of(line, "r", "plot"), "fu")
  expect_identical(class_of(line, "r", "1.5"), "fl")
  expect_identical(class_of(line, "r", "TRUE"), "cn")
  expect_identical(class_of(line, "r", "# note"), "co")
  expect_identical(class_of("if (x) 1L else 2L", "r", "if"), "cf")
  expect_identical(class_of("f <- function(x) x", "r", "function"), "kw")
  expect_identical(class_of("n <- 42L", "r", "42L"), "dv")
})

test_that("a multi-line R string is classified across both lines", {
  # The reason R parses the whole block rather than line by line.
  cls <- .hl_classes(c("s <- \"first", "second\"", "t <- 1"), "r")
  expect_true(all(cls[[1]][6:11] == "st"))
  expect_true(all(cls[[2]][1:7] == "st"))
  expect_identical(unique(cls[[3]][6]), "dv")
})

test_that("an R fragment that does not parse still gets highlighted", {
  # A fenced snippet is very often not a complete program. parse() fails,
  # and the r.xml grammar takes over rather than the block going plain.
  expect_null(tryCatch(parse(text = "for (i in x) {  # note", keep.source = TRUE),
                       error = function(e) NULL))
  expect_identical(class_of("for (i in x) {  # note", "r", "# note"), "co")
  expect_identical(class_of("for (i in x) {  # note", "r", "for"), "cf")
})

test_that("non-ASCII does not shift classes onto neighbouring characters", {
  # getParseData reports byte offsets under some encodings; a span that
  # does not match its own token text is dropped rather than trusted.
  line <- "x <- 1  # café éé"
  cls <- hl(line, "r")
  expect_length(cls, nchar(line))
  at <- regexpr("#", line, fixed = TRUE)
  expect_true(all(cls[at:nchar(line)] == "co"))
  expect_true(all(is.na(cls[1:2])))
})


# --- the grammar engine ----------------------------------------------

test_that("the earliest match wins, whatever order the rules are in", {
  # Both of these come out right without the grammar having to choose.
  # An all-or-nothing "claim if unclaimed" pass gets one of them wrong.
  expect_identical(class_of("# say \"hi\"", "python", "\"hi\""), "co")
  expect_identical(class_of("x = \"a # b\"", "python", "# b\""), "st")
})

test_that("rule order breaks a tie at the same starting position", {
  kw_first <- grammar_file(
    '<keyword attribute="Keyword" String="kw"/>
     <RegExpr attribute="Data Type" String="\\bfoo\\b"/>',
    '<list name="kw"><item>foo</item></list>')
  re_first <- grammar_file(
    '<RegExpr attribute="Data Type" String="\\bfoo\\b"/>
     <keyword attribute="Keyword" String="kw"/>',
    '<list name="kw"><item>foo</item></list>')
  register_highlighter("tie1", kw_first)
  register_highlighter("tie2", re_first)
  expect_identical(class_of("foo", "tie1", "foo"), "kw")
  expect_identical(class_of("foo", "tie2", "foo"), "dt")
})

test_that("defStyleNum maps onto the skylighting class names", {
  f <- grammar_file(
    '<RegExpr attribute="Control Flow" String="c"/>
     <RegExpr attribute="Data Type" String="d"/>
     <RegExpr attribute="Char" String="h"/>
     <RegExpr attribute="Normal Text" String="n"/>')
  register_highlighter("styles", f)
  expect_identical(class_of("c", "styles", "c"), "cf")
  expect_identical(class_of("d", "styles", "d"), "dt")
  expect_identical(class_of("h", "styles", "h"), "ch")
  # dsNormal is not a class: skylighting leaves a NormalTok unmarked.
  expect_true(is.na(class_of("n", "styles", "n")))
})

test_that("each supported rule type matches what it claims to", {
  f <- grammar_file(
    '<DetectChar attribute="Keyword" char="@"/>
     <Detect2Chars attribute="Comment" char="/" char1="/"/>
     <StringDetect attribute="String" String="&lt;&lt;"/>
     <WordDetect attribute="Data Type" String="word"/>
     <AnyChar attribute="Char" String="!?"/>
     <RangeDetect attribute="String" char="[" char1="]"/>
     <HlCHex attribute="BaseN"/>
     <Int attribute="Decimal"/>')
  register_highlighter("rules", f)
  expect_identical(class_of("@", "rules", "@"), "kw")
  expect_identical(class_of("// x", "rules", "//"), "co")
  expect_identical(class_of("a << b", "rules", "<<"), "st")
  expect_identical(class_of("a word b", "rules", "word"), "dt")
  expect_identical(class_of("a ! b", "rules", "!"), "ch")
  expect_identical(class_of("a [xy] b", "rules", "[xy]"), "st")
  expect_identical(class_of("0xFF", "rules", "0xFF"), "bn")
  expect_identical(class_of("42", "rules", "42"), "dv")
  # WordDetect must not match inside a longer identifier.
  expect_true(is.na(class_of("wordy", "rules", "word")))
})

test_that("a rule with no attribute takes its context's, as KDE does", {
  # Not a corner case: 226 of the 714 rules in KDE's own bash.xml leave
  # `attribute` off. Assigning them no class would silently under-colour
  # a third of any grammar downloaded from upstream.
  f <- tempfile(fileext = ".xml")
  writeLines('<?xml version="1.0" encoding="UTF-8"?>
    <language name="Inh" version="1" kateversion="5.0" casesensitive="1">
      <highlighting><contexts>
        <context name="N" attribute="Comment" lineEndContext="#stay">
          <RegExpr String="zap"/>
          <RegExpr attribute="Keyword" String="pow"/>
        </context></contexts>
        <itemDatas>
          <itemData name="Normal Text" defStyleNum="dsNormal"/>
          <itemData name="Comment" defStyleNum="dsComment"/>
          <itemData name="Keyword" defStyleNum="dsKeyword"/>
        </itemDatas></highlighting></language>', f)
  register_highlighter("inherit", f)
  expect_identical(class_of("a zap b", "inherit", "zap"), "co")
  # An explicit attribute still wins over the context's.
  expect_identical(class_of("a pow b", "inherit", "pow"), "kw")
})

test_that("an attribute no <itemData> defines is refused", {
  # A typo here would otherwise produce a grammar that loads happily and
  # colours nothing -- the failure mode this design refuses everywhere else.
  f <- grammar_file('<RegExpr attribute="Typoo" String="zap"/>')
  expect_error(register_highlighter("typo", f), "Typoo")
  expect_false("typo" %in% available_highlighters())
})

test_that("bundled grammar aliases are all in the static alias table", {
  # Aliases are declared twice: in each grammar's alternativeNames and in
  # .HL_ALIASES. Only the static table works before a grammar has been
  # loaded, so an alias present only in the XML resolves or not depending
  # on what else the session rendered first. Pin the invariant.
  for (g in .hl_grammar_names()) {
    for (a in .hl_compile(.hl_grammar_path(g))$aliases) {
      expect_identical(unname(.HL_ALIASES[a]), g,
                       info = paste0(g, ".xml declares alias ", sQuote(a)))
    }
  }
})

test_that("every alias resolves from a cold registry", {
  # The regression the test above guards against, exercised directly.
  old <- list(g = .hl_state$grammars, a = .hl_state$aliases)
  on.exit({ .hl_state$grammars <- old$g; .hl_state$aliases <- old$a }, add = TRUE)
  for (a in names(.HL_ALIASES)) {
    .hl_state$grammars <- list()
    .hl_state$aliases <- list()
    expect_false(is.null(.hl_classes("if x", a)), info = a)
  }
})

test_that("a literal from a grammar is not read as a pattern", {
  # StringDetect, DetectChar, WordDetect and keyword items are literals.
  # An unescaped metacharacter in one would either match the wrong thing
  # or make the whole rule an invalid regex and match nothing.
  bs <- intToUtf8(92)
  for (lit in c("a.c", "x*", "(y)", "[z]", "a+b", paste0("p", bs, "q"), "$v")) {
    esc <- .hl_esc_re(lit)
    expect_true(grepl(esc, lit, perl = TRUE), info = lit)
    # "a.c" must not match "abc": the dot is a literal dot.
    expect_false(grepl(esc, gsub("[^A-Za-z]", "Q", lit), perl = TRUE),
                 info = lit)
  }
})

test_that("a case-insensitive grammar matches either case", {
  f <- grammar_file('<keyword attribute="Keyword" String="kw"/>',
                    '<list name="kw"><item>SELECT</item></list>',
                    csens = "0")
  register_highlighter("ci", f)
  expect_identical(class_of("select 1", "ci", "select"), "kw")
  expect_identical(class_of("SELECT 1", "ci", "SELECT"), "kw")
})

test_that("internal DTD entities are expanded", {
  # The design relies on xml2 doing this: KDE grammars define entities
  # heavily, and an unexpanded &name; would be matched literally.
  f <- tempfile(fileext = ".xml")
  writeLines(
    '<?xml version="1.0" encoding="UTF-8"?>
     <!DOCTYPE language [<!ENTITY num "[0-9]+">]>
     <language name="Ent" version="1" kateversion="5.0" casesensitive="1">
       <highlighting>
         <contexts>
           <context name="Normal" attribute="Normal Text" lineEndContext="#stay">
             <RegExpr attribute="Decimal" String="&num;"/>
           </context>
         </contexts>
         <itemDatas>
           <itemData name="Normal Text" defStyleNum="dsNormal"/>
           <itemData name="Decimal" defStyleNum="dsDecVal"/>
         </itemDatas>
       </highlighting>
     </language>', f)
  register_highlighter("ent", f)
  expect_identical(class_of("x 123", "ent", "123"), "dv")
})


# --- refusing what is not implemented --------------------------------

test_that("an unrepresentable rule that switches context refuses the grammar", {
  # Losing a context switch strands the machine: an unclosed string would
  # paint every following line. These must fail loudly, by name.
  push <- grammar_file('<DetectChar attribute="String" char="&quot;" context="Nowhere"/>')
  expect_error(register_highlighter("bad1", push), "Nowhere")

  dyn <- grammar_file(
    '<RegExpr attribute="String" String="x" dynamic="true" context="Q"/>')
  expect_error(register_highlighter("bad2", dyn), "dynamic")

  cross <- grammar_file('<RegExpr attribute="String" String="x" context="##Doxygen"/>')
  expect_error(register_highlighter("bad3", cross), "another language")

  expect_false(any(c("bad1", "bad2", "bad3") %in% available_highlighters()))
})

test_that("an unrepresentable rule that stays in context is skipped", {
  # The other half of the same principle. Dropping an in-context rule
  # costs colour on the text it would have matched and nothing more, so
  # refusing the whole grammar over it would reject almost every real
  # KDE file for no benefit.
  g <- grammar_file(
    '<RegExpr attribute="String" String="zzz" dynamic="true"/>
     <keyword attribute="Keyword" String="kw"/>',
    '<list name="kw"><item>keep</item></list>')
  register_highlighter("skipped", g)
  # The rule that survived still works...
  expect_identical(class_of("keep zzz", "skipped", "keep"), "kw")
  # ...and the dropped one simply colours nothing.
  expect_true(is.na(class_of("keep zzz", "skipped", "zzz")))
})

test_that("a malformed grammar fails at registration, not at draw time", {
  f <- tempfile(fileext = ".xml")
  writeLines("<not-a-grammar/>", f)
  expect_error(register_highlighter("nope", f), "no <language>")

  writeLines("this is not xml at all", f)
  expect_error(register_highlighter("nope2", f), "as XML")

  empty <- grammar_file("")
  expect_error(register_highlighter("empty", empty), "no usable rules")

  # A keyword list defined in another file, reached through a
  # cross-language include we skipped: nothing usable is left.
  missing_list <- grammar_file('<keyword attribute="Keyword" String="absent"/>')
  expect_error(register_highlighter("ml2", missing_list), "no usable rules")

  expect_error(register_highlighter("gone", tempfile()), "not found")
})


# --- the bundled grammars --------------------------------------------

test_that("every bundled grammar compiles", {
  langs <- .hl_grammar_names()
  expect_gte(length(langs), 10L)
  for (g in langs) {
    expect_silent(gram <- .hl_compile(.hl_grammar_path(g)))
    expect_gt(length(unlist(lapply(gram$contexts, function(x) x$rules),
                            recursive = FALSE)), 0L)
  }
})

test_that("every bundled grammar classifies a keyword, string, comment and number", {
  # line, keyword, string, comment, number. The comment comes last on
  # every line: anything after it is part of the comment, not a token.
  cases <- list(
    r      = list("if (x) s <- \"a\" + 1.5  # c", "if", "\"a\"", "# c", "1.5"),
    python = list("if x: s = \"a\" + 1.5  # c", "if", "\"a\"", "# c", "1.5"),
    sql    = list("SELECT 'a', 1.5 FROM t  -- c", "SELECT", "'a'", "-- c", "1.5"),
    bash   = list("if x; then s=\"a\" 15; fi  # c", "if", "\"a\"", "# c", "15"),
    cpp    = list("if (x) s = \"a\" + 1.5;  // c", "if", "\"a\"", "// c", "1.5"),
    stan   = list("if (x) s = \"a\" + 1.5;  // c", "if", "\"a\"", "// c", "1.5"),
    julia  = list("if x; s = \"a\" + 1.5; end  # c", "if", "\"a\"", "# c", "1.5"),
    yaml   = list("k: \"a\" 1.5  # c", NA, "\"a\"", "# c", "1.5"),
    json   = list("{\"k\": \"a\", \"n\": 1.5}", NA, "\"a\"", NA, "1.5"),
    latex  = list("\\alpha{a} 1.5  % c", "\\alpha", NA, "% c", "1.5")
  )
  expect_setequal(names(cases), .hl_grammar_names())
  for (lang in names(cases)) {
    cc <- cases[[lang]]
    line <- cc[[1]]
    if (!is.na(cc[[2]])) {
      expect_true(class_of(line, lang, cc[[2]]) %in% c("kw", "cf"),
                  info = paste(lang, "keyword"))
    }
    if (!is.na(cc[[3]])) {
      expect_identical(class_of(line, lang, cc[[3]]), "st",
                       info = paste(lang, "string"))
    }
    if (!is.na(cc[[4]])) {
      expect_identical(class_of(line, lang, cc[[4]]), "co",
                       info = paste(lang, "comment"))
    }
    expect_true(class_of(line, lang, cc[[5]]) %in% c("dv", "fl"),
                info = paste(lang, "number"))
  }
})

test_that("a comment after a string containing the comment character", {
  # The case that retired the vectorised fast path: gregexpr consumed the
  # `#` inside the string, so the real trailing comment was never seen.
  # The positional engine is inside the string context there, so its
  # comment rule is not even live.
  expect_identical(class_of('s = "a # b"  # real', "python", "# real"), "co")
  expect_identical(class_of('s = "a # b"  # real', "python", '"a # b"'), "st")
  expect_identical(class_of("x <- \"a # b\"  # real", "r", "# real"), "co")
})

test_that("a multi-context grammar keeps state across lines", {
  # The whole point of contexts: a delimiter opened on one line still
  # governs the next, so a keyword inside a string is not a keyword.
  f <- tempfile(fileext = ".xml")
  writeLines('<?xml version="1.0" encoding="UTF-8"?>
    <language name="Ml" version="1" kateversion="5.0" casesensitive="1">
      <highlighting>
        <list name="kw"><item>keyword</item></list>
        <contexts>
          <context name="Normal" attribute="Normal Text" lineEndContext="#stay">
            <StringDetect attribute="String" String="&lt;&lt;" context="Str"/>
            <keyword attribute="Keyword" String="kw"/>
          </context>
          <context name="Str" attribute="String" lineEndContext="#stay">
            <StringDetect attribute="String" String="&gt;&gt;" context="#pop"/>
          </context>
        </contexts>
        <itemDatas>
          <itemData name="Normal Text" defStyleNum="dsNormal"/>
          <itemData name="String" defStyleNum="dsString"/>
          <itemData name="Keyword" defStyleNum="dsKeyword"/>
        </itemDatas></highlighting></language>', f)
  register_highlighter("ml", f)
  expect_length(.hl_compile(f)$contexts, 2L)

  cls <- .hl_classes(c("keyword << keyword", "keyword", ">> keyword"), "ml")
  # Line 1: the first keyword is a keyword, the one after << is string.
  expect_identical(unique(cls[[1]][1:7]), "kw")
  expect_identical(unique(cls[[1]][12:18]), "st")
  # Line 2 is entirely inside the string opened on line 1.
  expect_identical(unique(cls[[2]]), "st")
  # After >> pops, keywords are keywords again.
  expect_identical(unique(cls[[3]][4:10]), "kw")
})

test_that("KDE's boolean attributes are read in both spellings", {
  # KDE mixes them inside one file: python.xml has lookAhead="1" ninety
  # times and lookAhead="true" not once. Reading only "true" silently
  # turned every look-ahead rule into a consuming one, which mis-coloured
  # every string and comment in every upstream grammar.
  expect_true(.hl_bool("1"))
  expect_true(.hl_bool("true"))
  expect_true(.hl_bool("TRUE"))
  expect_false(.hl_bool("0"))
  expect_false(.hl_bool(NA))
  expect_false(.hl_bool(""))
  expect_true(.hl_bool(NA, default = TRUE))

  f <- tempfile(fileext = ".xml")
  writeLines('<?xml version="1.0" encoding="UTF-8"?>
    <language name="La" version="1" kateversion="5.0" casesensitive="1">
      <highlighting><contexts>
        <context name="Normal" attribute="Normal Text" lineEndContext="#stay">
          <DetectChar attribute="Normal Text" char="q" context="Q" lookAhead="1"/>
        </context>
        <context name="Q" attribute="String" lineEndContext="#stay">
          <DetectChar attribute="String" char="z" context="#pop"/>
        </context></contexts>
        <itemDatas>
          <itemData name="Normal Text" defStyleNum="dsNormal"/>
          <itemData name="String" defStyleNum="dsString"/>
        </itemDatas></highlighting></language>', f)
  register_highlighter("la", f)
  # lookAhead="1" must not consume the q: it belongs to the Str context.
  expect_identical(class_of("aqbz", "la", "q"), "st")
})

test_that("the github preset styles every token class the defaults do", {
  # The preset is a copyable worked example, so a class present in the
  # built-in defaults but missing from the CSS would silently render in
  # the body colour for anyone using the preset.
  st <- markdown_style("github")
  sels <- vapply(st$rules, function(r) r$selector, character(1))
  expect_true(all(paste0(".", names(.MD_CODE_COLORS)) %in% sels))
  # And the two palettes agree, so switching to the preset does not
  # quietly restyle code.
  for (k in names(.MD_CODE_COLORS)) {
    expect_identical(.md_resolve_color(.md_cascade(st, "pre", classes = k)$color),
                     unname(.MD_CODE_COLORS[k]), info = k)
  }
})

test_that("every class a bundled grammar emits has a default colour", {
  # Otherwise a token would be classified and then rendered in the body
  # colour, which looks like the highlighter silently failed.
  for (g in .hl_grammar_names()) {
    gram <- .hl_compile(.hl_grammar_path(g))
    rules <- unlist(lapply(gram$contexts, function(x) x$rules), recursive = FALSE)
    used <- vapply(rules, function(p) p$class, character(1))
    used <- unique(used[!is.na(used)])
    expect_true(all(used %in% names(.MD_CODE_COLORS)),
                info = paste(g, ":", paste(setdiff(used, names(.MD_CODE_COLORS)),
                                           collapse = " ")))
  }
})


# --- language names ---------------------------------------------------

test_that("GFM aliases and knitr's chunk header reach the right grammar", {
  expect_identical(.hl_canonical(.hl_lang("py")), "python")
  expect_identical(.hl_canonical(.hl_lang("c++")), "cpp")
  expect_identical(.hl_canonical(.hl_lang("yml")), "yaml")
  expect_identical(.hl_canonical(.hl_lang("jl")), "julia")
  expect_identical(.hl_canonical(.hl_lang("tex")), "latex")
  expect_identical(.hl_canonical(.hl_lang("sh")), "bash")
  expect_identical(.hl_canonical(.hl_lang("R")), "r")
  # knitr writes the whole chunk header into the info string.
  expect_identical(.hl_canonical(.hl_lang("{r, echo=FALSE}")), "r")
  expect_identical(.hl_canonical(.hl_lang("python title=\"x\"")), "python")
  # A grammar's own alternativeNames are honoured once it is loaded.
  expect_identical(class_of("import x", "python3", "import"), "kw")
})

test_that("an unknown or absent language is silent, not an error", {
  for (lang in list(NA_character_, "", "klingon", NULL)) {
    expect_silent(cls <- .hl_classes("x <- 1", lang))
    expect_null(cls)
  }
})


# --- registration -----------------------------------------------------

test_that("a registered grammar is used and listed", {
  f <- grammar_file('<RegExpr attribute="Keyword" String="\\bzap\\b"/>')
  before <- available_highlighters()
  expect_false("zap" %in% before)
  register_highlighter("zap", f)
  expect_true("zap" %in% available_highlighters())
  expect_identical(class_of("a zap b", "zap", "zap"), "kw")
})

test_that("registering over a bundled name replaces it", {
  f <- grammar_file('<RegExpr attribute="Data Type" String="\\bplot\\b"/>')
  on.exit({
    .hl_state$grammars[["python"]] <- NULL
    .hl_state$user <- setdiff(.hl_state$user, "python")
  }, add = TRUE)
  register_highlighter("python", f)
  expect_identical(class_of("plot(1)", "python", "plot"), "dt")
})


# --- rendering --------------------------------------------------------

test_that("token colours reach the drawn layout", {
  cls <- .hl_classes("foo(1)  # hi", "r")[[1]]
  tex <- .hl_code_tex("foo(1)  # hi", cls,
                      function(k) unname(.MD_CODE_COLORS[k]))
  df <- parse_latex_cpp(tex, text_size = 12, use_path = FALSE)
  got <- unique(df$color)
  expect_true(.MD_CODE_COLORS[["fu"]] %in% got)  # foo
  expect_true(.MD_CODE_COLORS[["dv"]] %in% got)  # 1
  expect_true(.MD_CODE_COLORS[["co"]] %in% got)  # # hi
})

test_that("highlighting survives the whole markdown_box_grob() path", {
  # Everything above works on the emitter directly; this drives the real
  # entry point, so a break in the wiring cannot pass unnoticed.
  md <- "```r\nplot(1)  # hi\n```\n"
  tex <- box_tex(md)
  expect_true(any(grepl(.MD_CODE_COLORS[["fu"]], tex, fixed = TRUE)))
  expect_true(any(grepl(.MD_CODE_COLORS[["co"]], tex, fixed = TRUE)))
  # And a fence with no language stays plain all the way through.
  expect_false(any(grepl("textcolor", box_tex("```\nplot(1)  # hi\n```\n"),
                         fixed = TRUE)))
})

test_that("a stylesheet overrides one token class and only that one", {
  md <- "```r\nplot(1)  # hi\n```\n"
  themed <- box_tex(md, style = ".co { color: #123456 }")
  expect_true(any(grepl("#123456", themed, fixed = TRUE)))
  # The rule names only .co, so the function colour is untouched.
  expect_true(any(grepl(.MD_CODE_COLORS[["fu"]], themed, fixed = TRUE)))
  expect_false(any(grepl(.MD_CODE_COLORS[["co"]], themed, fixed = TRUE)))
})

test_that("a token whose colour equals the block colour emits no markup", {
  # Why the emitter compares against the block colour rather than always
  # wrapping: colouring a token the colour it already is would bloat
  # every line with markup that changes nothing.
  md <- "```r\nplot(1)\n```\n"
  tex <- box_tex(md, style = "pre { color: #112233 } .fu { color: #112233 }")
  expect_false(any(grepl("textcolor{#112233}", tex, fixed = TRUE)))
})

test_that("visual: a highlighted code block", {
  skip_if_not_installed("vdiffr")
  skip_on_os("mac")
  # Evidence, not coverage: this block skips on CRAN, so nothing above is
  # delegated to it. vdiffr writes one family name for every family, so
  # the mono run is recorded as `font-family: sans` here -- that is the
  # snapshot device, not the layout. The colours are real.
  vdiffr::expect_doppelganger("markdown-highlight", function() {
    grid::grid.draw(markdown_box_grob(
      "```r\nfit <- lm(y ~ x)  # refit\nif (bad) {\n    stop(\"no\")\n}\n```\n",
      width = grid::unit(3.4, "in"),
      padding = grid::unit(8, "pt"),
      box_gp = grid::gpar(fill = "grey96", col = "grey40"),
      gp = grid::gpar(fontsize = 12)
    ))
  })
})

test_that("an unhighlighted block emits no colour markup at all", {
  # Keeps a plain fence byte-identical to what it rendered before, which
  # is what stops every existing snapshot from moving.
  for (lang in c(NA_character_, "klingon")) {
    cls <- .hl_classes("x <- 1", lang)
    tex <- .hl_code_tex("x <- 1", if (is.null(cls)) NULL else cls[[1]],
                        function(k) unname(.MD_CODE_COLORS[k]))
    expect_false(grepl("textcolor", tex, fixed = TRUE))
  }
})

# Block-level markdown: a document laid out in grid.
#
# The other half of markdown.R, which handles the inline pipeline -- one
# run of text, no layout. The split is the same one the API makes:
# markdown_grob() is a run, markdown_box_grob() is a document.
#
# Everything here builds on markdown.R and not the other way round: the
# block walker calls .md_inline_to_tex() for each block's prose, and the
# CSS value resolvers (.md_css_length(), .md_resolve_color(), ...) live
# over there because the inline path needs them too.
#
# Three stages, in order:
#
#   .md_parse_blocks()  CommonMark AST -> a list of typed blocks, each
#                       carrying the LaTeX for its own content
#   .md_layout()        blocks -> positioned items, measured against a
#                       width; recurses for lists and block quotes
#   .md_box_layout()    the box around them -- margin, padding, chrome --
#                       resolved at draw time, since a relative width
#                       needs a viewport and measuring needs a device
#
# Why any of this is in R rather than MicroTeX: the engine cannot lay a
# document out in one parse. A `\` puts a VBox at the top of the box tree
# and, inside matrix/array cells, max_width is not honoured at all (see
# the note in BoxSplitter::split). So block structure is assembled here
# -- one latex_grob per block, stacked in grid -- which is also where
# boxes, padding, background fills and rules have to live, none of which
# MicroTeX models.

# Heading level -> MicroTeX size macro. All ten of the \tiny..\Huge
# family are registered (macro_def.cpp); these six are the ones a
# markdown heading can reach.
.MD_HEADING_SIZE <- c("\\Huge", "\\huge", "\\LARGE",
                      "\\Large", "\\large", "\\normalsize")

# Classify an html_block as a <div> boundary, or neither.
#
# cmark hands `<div ...>` and `</div>` over as separate html_block
# siblings whenever there are blank lines around them, with the markdown
# between them parsed into real blocks -- which is what makes a div
# usable as a container here. Written without blank lines the whole thing
# arrives as one opaque block instead, and falls through to "other".
.md_div_tag <- function(txt) {
  t <- trimws(txt)
  if (grepl("^</[[:space:]]*div[[:space:]]*>$", t, ignore.case = TRUE)) {
    return(list(kind = "close"))
  }
  m <- regmatches(t, regexec("^<[[:space:]]*div([^>]*)>$", t,
                             ignore.case = TRUE))[[1]]
  if (length(m) != 2L) return(list(kind = "other"))
  list(kind = "open",
       class = .md_html_classes(m[2]),
       # Hand-written markup, so an unsupported property is worth a word
       # -- see the note on .md_parse_css().
       style = .md_parse_css(.md_html_attr(m[2], "style"), warn = TRUE))
}

# Parse markdown into a list of block descriptors. Recurses through
# containers (lists, block quotes) so nesting is preserved, and folds
# <div>...</div> runs into container blocks of their own.
#
# `ctx` is the enclosing container's resolved style. It is threaded here,
# and not only at layout time, because a block's *content* has to know
# whether an emphasis command will wrap it: \textbf{\text{x}} silently
# reports plain, so prose destined for \textbf{} must be emitted bare.
# The cascade is the same function either way, so the two passes cannot
# disagree about the rules.
.md_blocks <- function(nodes, spans, base = 20, style = NULL, ctx = list()) {
  out <- list()
  # Open <div>s, innermost last. Blocks are collected into the innermost
  # one until it closes, and each carries the context its contents see.
  open <- list()
  # Whether the separator before the footnote section has been emitted.
  seen_fn <- FALSE

  cur_ctx <- function() if (length(open)) open[[length(open)]]$ctx else ctx
  emit <- function(blk) {
    k <- length(open)
    if (k) open[[k]]$blocks <<- c(open[[k]]$blocks, list(blk))
    else out <<- c(out, list(blk))
  }
  close_div <- function() {
    d <- open[[length(open)]]
    open[[length(open)]] <<- NULL
    emit(list(type = "div", class = d$class, style = d$style,
              blocks = d$blocks))
  }
  # Prose for a block, emitted bare when emphasis will wrap it.
  prose <- function(nd, tag) {
    res <- .md_cascade(style %||% markdown_style(), tag, inherited = cur_ctx())
    emph <- .md_emphasis_cmds(res)
    .md_inline_to_tex(nd, spans, bare = length(emph) > 0L, styles = emph,
                      base = base, style = style)
  }
  # The context a container hands to its children.
  child_ctx <- function(tag) {
    .md_cascade(style %||% markdown_style(), tag, inherited = cur_ctx())
  }

  for (nd in nodes) {
    nm <- xml2::xml_name(nd)

    if (identical(nm, "html_block")) {
      tg <- .md_div_tag(xml2::xml_text(nd))
      if (identical(tg$kind, "open")) {
        open[[length(open) + 1L]] <- list(
          class = tg$class, style = tg$style, blocks = list(),
          ctx = .md_cascade(style %||% markdown_style(), "div",
                            classes = tg$class, inline = tg$style,
                            inherited = cur_ctx()))
      } else if (identical(tg$kind, "close") && length(open)) {
        close_div()
      }
      # Any other raw block, and a stray </div>, is dropped rather than
      # passed through: MicroTeX would typeset the markup.
      next
    }

    # A paragraph holding nothing but images is a block image, which is
    # how markdown renderers treat it. Handled before the switch because
    # one paragraph can yield several image blocks.
    if (identical(nm, "paragraph")) {
      imgs <- .md_image_blocks(nd, spans, base, style)
      if (!is.null(imgs)) {
        for (im in imgs) emit(im)
        next
      }
    }
    # A paragraph that is nothing but `$$...$$` is a display equation, not
    # prose: it gets its own centred block, as in every other markdown
    # dialect. Inline `$...$` is untouched.
    if (identical(nm, "paragraph") && !is.na(.md_lone_math_span(nd, spans))) {
      emit(list(type = "mathblock", tex = prose(nd, "math")))
      next
    }
    blk <- switch(nm,
      paragraph = list(type = "paragraph", tex = prose(nd, "p")),
      heading = {
        lvl <- .md_heading_level(nd)
        list(type = "heading", level = lvl,
             tex = prose(nd, paste0("h", lvl)))
      },
      block_quote = list(type = "block_quote",
                         blocks = .md_blocks(xml2::xml_children(nd), spans,
                                             base, style,
                                             child_ctx("blockquote"))),
      code_block = list(type = "code_block",
                        lines = .md_code_lines(nd, spans),
                        # The fence's info string, e.g. "python". NA when
                        # the fence names no language.
                        lang = xml2::xml_attr(nd, "info")),
      list = .md_list_block(nd, spans, base, style, cur_ctx()),
      # Built content-sized here, which is what every caller sees. The
      # node is kept as well so .md_layout() can rebuild the table when
      # `table-layout: fixed` asks for the columns to be divided out of
      # the *available* width, which is not known until the block is
      # placed. xml2 keeps the document alive through the node.
      table = list(type = "table",
                   tex = .md_table_tex(nd, spans, base, style,
                                       inherited = cur_ctx()),
                   nd = nd, spans = spans, base = base, ctx = cur_ctx()),
      thematic_break = list(type = "thematic_break"),
      # CommonMark already gathers footnote definitions at the end of the
      # document, in reference order, so they need no reordering here --
      # only a separator before the first one.
      fn = {
        # Plain <- : this runs in .md_blocks()'s own frame, where <<- would
        # skip the local and assign into the enclosing environment.
        if (!seen_fn) {
          seen_fn <- TRUE
          emit(list(type = "thematic_break"))
        }
        list(type = "footnote",
             blocks = .md_blocks(xml2::xml_children(nd), spans, base, style,
                                 child_ctx("footnote")))
      },
      # Anything unrecognised is dropped.
      NULL
    )
    if (!is.null(blk)) emit(blk)
  }

  # A div left open at the end of a container closes here, so the blocks
  # inside it are still laid out rather than lost.
  while (length(open)) close_div()
  out
}

.md_heading_level <- function(nd) {
  lvl <- suppressWarnings(as.integer(xml2::xml_attr(nd, "level")))
  if (is.na(lvl)) lvl <- 1L
  min(max(lvl, 1L), 6L)
}

# Code blocks are literal. MicroTeX has no verbatim environment, so each
# source line becomes its own \texttt{} block and the stacker puts them
# on separate rows.
#
# Indentation is *not* preserved by that split -- MicroTeX collapses a
# run of spaces inside one \text{} run, so the leading whitespace of a
# line used to measure one space wide however deep it was. See
# .hl_segments(), which breaks a line apart wherever spaces run two or
# more deep.
#
# Tabs are expanded here rather than at emit time, because the classifier
# works in character positions and would be thrown off by a later
# substitution that changes the length of a line.
.md_code_lines <- function(nd, spans = NULL) {
  # Math is masked out of the *whole* document before CommonMark parses
  # it, which cannot know about fences -- so a `$...$` written inside a
  # code block arrives here as a sentinel. Restore the original bytes:
  # in code a dollar is literal, and .md_escape_tex() neutralises it.
  # Without this the span's content is lost outright and the sentinel's
  # index is drawn in its place, turning `# see $E = mc^2$ here` into
  # `# see 1 here`. Inline code spans already do this; blocks did not.
  txt <- .md_unmask_math(xml2::xml_text(nd), spans)
  lines <- strsplit(txt, "\n", fixed = TRUE)[[1]]
  if (length(lines) == 0L) return(character(0))
  # A trailing newline yields a spurious empty final line.
  if (length(lines) > 1L && !nzchar(lines[length(lines)])) {
    lines <- lines[-length(lines)]
  }
  vapply(lines, .md_expand_tabs, character(1), USE.NAMES = FALSE)
}

# A tab advances to the next multiple-of-4 column, which is what every
# editor and terminal does -- not a fixed four spaces. The difference
# shows the moment a tab follows anything: `abc<tab>x` puts x at column
# 5, not 9, so tab-indented code (a Makefile has no choice) lines up.
# Column counting restarts on each line, so this cannot be a gsub over
# the whole block.
.md_expand_tabs <- function(s, width = 4L) {
  if (!grepl("\t", s, fixed = TRUE)) return(s)
  chars <- strsplit(s, "", fixed = TRUE)[[1]]
  out <- character(length(chars))
  col <- 0L
  for (i in seq_along(chars)) {
    if (identical(chars[i], "\t")) {
      pad <- width - (col %% width)
      out[i] <- strrep(" ", pad)
      col <- col + pad
    } else {
      out[i] <- chars[i]
      col <- col + 1L
    }
  }
  paste0(out, collapse = "")
}

.md_list_block <- function(nd, spans, base = 20, style = NULL, ctx = list()) {
  ordered <- identical(xml2::xml_attr(nd, "type"), "ordered")
  start <- suppressWarnings(as.integer(xml2::xml_attr(nd, "start")))
  if (is.na(start)) start <- 1L
  kids <- xml2::xml_children(nd)
  # An item's contents see ul/ol and then li, the same chain .md_layout()
  # walks when it resolves the same blocks for drawing.
  st <- style %||% markdown_style()
  item_ctx <- .md_cascade(st, "li",
                          inherited = .md_cascade(st, if (ordered) "ol" else "ul",
                                                  inherited = ctx))
  items <- lapply(kids, function(it) {
    .md_blocks(xml2::xml_children(it), spans, base, style, item_ctx)
  })
  # A GFM task list item arrives as <tasklist completed="..."> in place
  # of <item>; NA marks an ordinary item with no checkbox.
  checked <- vapply(kids, function(it) {
    if (!identical(xml2::xml_name(it), "tasklist")) return(NA)
    identical(xml2::xml_attr(it, "completed"), "true")
  }, logical(1))
  list(type = "list", ordered = ordered, start = start,
       # `1)` is as valid as `1.` in CommonMark, and cmark reports which
       # the author used.
       paren = identical(xml2::xml_attr(nd, "delim"), "paren"),
       tight = identical(xml2::xml_attr(nd, "tight"), "true"),
       items = items, checked = checked)
}

# Return one image block per image when a paragraph contains nothing but
# images, otherwise NULL so the paragraph is rendered normally. Images
# mixed into a sentence stay inline, where only their alt text survives.
.md_image_blocks <- function(nd, spans, base = 20, style = NULL) {
  kids <- xml2::xml_children(nd)
  if (length(kids) == 0L) return(NULL)
  nms <- xml2::xml_name(kids)
  if (!all(nms %in% c("image", "softbreak"))) return(NULL)
  lapply(which(nms == "image"), function(i) {
    im <- kids[[i]]
    list(type = "image",
         path = xml2::xml_attr(im, "destination"),
         alt = .md_inline_to_tex(im, spans, base = base, style = style))
  })
}

# Load a raster image. Returns NULL -- and the caller falls back to the
# alt text -- when the file is missing, the format is unsupported, or the
# reader package is not installed. png and jpeg are Suggests, so images
# degrade rather than becoming a hard dependency of the whole package.
.md_image_raster <- function(path) {
  if (is.null(path) || is.na(path) || !nzchar(path)) return(NULL)
  if (!file.exists(path)) return(NULL)
  ext <- tolower(tools::file_ext(path))
  reader <- switch(ext,
    png = if (requireNamespace("png", quietly = TRUE)) png::readPNG else NULL,
    jpg = ,
    jpeg = if (requireNamespace("jpeg", quietly = TRUE)) jpeg::readJPEG else NULL,
    NULL
  )
  if (is.null(reader)) return(NULL)
  ras <- tryCatch(reader(path), error = function(e) NULL)
  if (is.null(ras) || length(dim(ras)) < 2L) return(NULL)
  list(raster = grDevices::as.raster(ras),
       w_px = dim(ras)[2], h_px = dim(ras)[1])
}

# Build a `tabular` from the GFM table extension. MicroTeX supports
# tabular natively along with \hline and the \thickhline added for
# booktabs output, so a markdown table becomes a real typeset table
# rather than an image -- something marquee cannot do at all.
.md_table_tex <- function(nd, spans, base = 20, style = NULL,
                          avail = 0, inherited = list()) {
  rows <- xml2::xml_children(nd)
  if (length(rows) == 0L) return("")
  st <- style %||% markdown_style()

  # The four table tags resolve once. `tr`/`td`/`th` inherit through
  # `table`, so `table { color: }` reaches the cells as CSS says it should.
  r_table <- .md_cascade(st, "table", inherited = inherited)
  r_tr    <- .md_cascade(st, "tr", inherited = r_table)
  r_td    <- .md_cascade(st, "td", inherited = r_table)
  r_th    <- .md_cascade(st, "th", inherited = r_table)

  # A cell's content, wrapped in whatever emphasis its tag asks for. The
  # emphasis has to be applied here rather than around the finished table:
  # \text{} builds a non-nested FontStyleAtom, so \textbf{\text{x}} comes
  # out plain -- which is why header cells were never bold.
  cell_tex <- function(cell, res) {
    emph <- .md_emphasis_cmds(res)
    inner <- .md_inline_to_tex(cell, spans, bare = length(emph) > 0L,
                               styles = emph, base = base, style = style)
    fill <- .md_resolve_color(res$background)
    paste0(if (!is.null(fill)) paste0("\\cellcolor{", fill, "}"),
           paste(emph, collapse = ""), inner, strrep("}", length(emph)))
  }
  cells_of <- function(r, res) {
    vapply(xml2::xml_children(r), cell_tex, character(1), res = res)
  }

  header <- NULL
  body <- list()
  for (r in rows) {
    is_head <- identical(xml2::xml_name(r), "table_header")
    cs <- cells_of(r, if (is_head) r_th else r_td)
    if (is_head) header <- cs else body[[length(body) + 1L]] <- cs
  }
  ncol <- max(c(length(header), vapply(body, length, integer(1))), 0L)
  if (ncol == 0L) return("")
  pad <- function(cs) c(cs, rep("", ncol - length(cs)))
  line <- function(cs) paste(pad(cs), collapse = " & ")

  # Column alignment from `|:--|:-:|--:|`. cmark records it on the header
  # cells only, and MicroTeX honours l/c/r in the tabular spec.
  spec <- rep("l", ncol)
  hdr <- Filter(function(r) identical(xml2::xml_name(r), "table_header"), rows)
  if (length(hdr)) {
    a <- vapply(xml2::xml_children(hdr[[1]]), function(cell) {
      switch(xml2::xml_attr(cell, "align") %||% "left",
             center = "c", right = "r", "l")
    }, character(1))
    a <- a[!is.na(a)]
    k <- seq_len(min(ncol, length(a)))
    if (length(k)) spec[k] <- a[k]
  }

  # table-layout: fixed divides the measure between the columns and
  # renders each as a p{} column, so a wide table wraps instead of running
  # off the edge. Content-sized (l/c/r) stays the default.
  fixed <- identical(tolower(as.character(r_table[["table-layout"]] %||% "")),
                     "fixed")
  if (fixed && avail > 0 && ncol > 0) {
    # Each column costs its width plus the inter-column separation, which
    # defaults to 1em. Subtract that share or the table overshoots the
    # measure it was asked to fit into.
    gap_est <- .md_css_length(r_td[["padding-left"]], base, base,
                              horizontal = TRUE) %||% base
    per <- max(avail / ncol - gap_est, base)
    spec <- rep(sprintf("p{%.2fpt}", per), ncol)
  }

  # Vertical rules from a border on the cells, and the gap between
  # columns from their horizontal padding. `@{...}` replaces the default
  # inter-column space entirely, so it is only emitted when asked for.
  vrule <- if (!is.null(.md_css_border(r_td[["border-left"]], base))) "|" else ""
  gap <- .md_css_length(r_td[["padding-left"]], base, base, horizontal = TRUE)
  sep <- if (!is.null(gap)) sprintf("@{\\hspace{%.2fpt}}", gap) else ""
  spec_str <- if (nzchar(vrule)) {
    paste0(vrule, paste(spec, collapse = paste0(vrule, sep)), vrule)
  } else if (nzchar(sep)) {
    paste(spec, collapse = sep)
  } else {
    paste(spec, collapse = "")
  }

  # Row rules: `tr { border-bottom }` picks the weight. MicroTeX has two,
  # so anything above a hairline becomes the thick one. The outer rules
  # keep the booktabs look the default has always had.
  row_rule <- local({
    bd <- .md_css_border(r_tr[["border-bottom"]], base)
    if (is.null(bd) || (bd$width %||% 0) <= 0) "" else
      if ((bd$width %||% 0) > 1.5) "\\thickhline " else "\\hline "
  })

  arc <- .md_resolve_color(r_table[["border-color"]])
  parts <- c(if (!is.null(arc)) paste0("\\arrayrulecolor{", arc, "}"),
             "\\begin{tabular}{", spec_str, "}",
             "\\thickhline ")
  # `tr` is the row and `th`/`td` are the cells, as in HTML: a row fill is
  # a row-leading \rowcolor and applies to the header too, while a cell
  # fill is the \cellcolor already emitted by cell_tex().
  row_fill <- .md_resolve_color(r_tr$background)
  row_lead <- if (!is.null(row_fill)) paste0("\\rowcolor{", row_fill, "}")
  if (!is.null(header)) {
    parts <- c(parts, row_lead, line(header), "\\\\", "\\hline ")
  }
  for (i in seq_along(body)) {
    # The closing \thickhline already rules off the last row; adding the
    # per-row rule as well would draw two lines on top of each other.
    parts <- c(parts, row_lead, line(body[[i]]), "\\\\",
               if (i < length(body)) row_rule)
  }
  parts <- c(parts, "\\thickhline", "\\end{tabular}")
  paste(parts, collapse = "")
}

# Parse a markdown string into block descriptors.
.md_parse_blocks <- function(md, base = 20, style = NULL, ctx = list()) {
  parsed <- .md_parse_doc(md)
  .md_blocks(xml2::xml_children(parsed$doc), parsed$spans, base, style, ctx)
}

# Does this markdown need block layout?
#
# Anything that is not a single plain paragraph does: a heading, a list, a
# quote, a table, a rule, a code block, a footnote, two paragraphs -- or a
# paragraph that is nothing but display math, which .md_blocks() turns
# into a centred `math` block of its own.
#
# Reads the parsed document rather than the built blocks on purpose: only
# the shape matters, and building every block's LaTeX just to count them
# would double the work for the one-paragraph label that is the norm.
.md_needs_blocks <- function(md) {
  parsed <- .md_parse_doc(md)
  nodes <- xml2::xml_children(parsed$doc)
  if (length(nodes) == 0L) return(FALSE)
  if (length(nodes) != 1L) return(TRUE)
  if (!identical(xml2::xml_name(nodes[[1]]), "paragraph")) return(TRUE)
  !is.na(.md_lone_math_span(nodes[[1]], parsed$spans))
}

# Marker text for list item `n`.
#
# `checked` is NA for an ordinary item, TRUE/FALSE for a GFM task list
# item. The two checkbox states must be the SAME glyph size or the list
# looks ragged: \square (empty) and \blacksquare (filled) are both 14x13
# in the math font, whereas the more literal \boxtimes is 18x16 and left
# every checked row visibly larger. (\Box is not a glyph at all -- it
# typesets the three letters "Box".)
.md_list_marker <- function(ordered, start, n, paren = FALSE, checked = NA,
                            bullet = NULL) {
  if (!is.na(checked)) {
    return(if (isTRUE(checked)) "\\blacksquare" else "\\square")
  }
  if (ordered) {
    paste0("\\text{", start + n - 1L, if (paren) ")" else ".", "}")
  } else {
    # `bullet` is raw LaTeX from the style, the one escape hatch for a
    # marker glyph -- there is no CSS spelling for "\\triangleright".
    bullet %||% "\\bullet"
  }
}

# ---------------------------------------------------------------------
# Layout
#
# Each block is measured with latex_dims() at the available width and
# placed at an offset from the top of the stack. Offsets grow downward;
# makeContent turns them into viewports. Working top-down means the
# total height does not have to be known in advance.
#
# All distances are big points, matching what latex_dims() reports.
# ---------------------------------------------------------------------

# One entry in the laid-out stack. `align` is the block's own text-align
# as a fraction, or NULL to take the box-wide halign.
.md_item <- function(grob, x, y, w, h, align = NULL) {
  list(grob = grob, x = x, y = y, w = w, h = h, align = align)
}

# Move a laid-out grob by (dx, dy) big points, dy measured downward to
# match the stack's own top-down offsets.
#
# Two kinds of grob end up in the stack and they are positioned
# differently. A text block is a latex_grob, which carries its position
# in a viewport; a rule, an image and a block quote's bar are plain
# rect/raster grobs positioned by their own x/y. Assuming the viewport is
# what made a horizontal rule inside a block quote an error rather than a
# rule, and what left images pinned to the left edge under `halign`.
.md_shift_grob <- function(g, dx = 0, dy = 0) {
  if (!is.null(g$vp)) {
    if (dx != 0) g$vp$x <- g$vp$x + grid::unit(dx, "bigpts")
    if (dy != 0) g$vp$y <- g$vp$y - grid::unit(dy, "bigpts")
    return(g)
  }
  if (dx != 0 && !is.null(g$x)) g$x <- g$x + grid::unit(dx, "bigpts")
  if (dy != 0 && !is.null(g$y)) g$y <- g$y - grid::unit(dy, "bigpts")
  g
}

# Measure a run, and return the max_width that achieved that measurement
# so the caller can draw with exactly the same setting.
#
# input_mode MUST match .md_run_grob(). latex_dims() defaults to
# "mixed", which would run latex_wrap() over LaTeX the AST walker has
# already wrapped -- measuring a different (double-wrapped) string from
# the one drawn. That mismatch shows up as blocks overflowing the box.
#
# The retry works around MicroTeX's line breaker, which is greedy
# first-fit: it takes the last break position that fits and never
# reconsiders, so text spread over several \text{} runs can settle on a
# first line wider than the limit. Asking for a slightly narrower line
# finds a different, fitting set of breaks. Bounded, so genuinely
# unbreakable content (one long word, a table) still returns promptly.
.md_measure <- function(tex, width, gp) {
  d <- latex_dims(tex, max_width = width, input_mode = "math", gp = gp)
  w <- as.numeric(d$width)
  if (width > 0 && w > width) {
    target <- width
    for (i in seq_len(6L)) {
      target <- target * 0.94
      d2 <- latex_dims(tex, max_width = target, input_mode = "math", gp = gp)
      if (as.numeric(d2$width) <= width) {
        return(list(w = as.numeric(d2$width), h = as.numeric(d2$height),
                    bl = as.numeric(d2$baseline), mw = target))
      }
    }
  }
  # `bl` is the baseline as bigpts up from the bottom of the box, so
  # (h - bl) is the baseline measured down from the top -- what the list
  # layout needs to line a marker up with its text.
  list(w = w, h = as.numeric(d$height), bl = as.numeric(d$baseline),
       mw = width)
}

# Build the grob for a run of LaTeX at a given offset. `y` is the
# distance from the top of the content area; vjust = 1 (top) makes the
# offset land on the block's top edge, so stacking is just addition.
.md_run_grob <- function(tex, x, y, width, gp) {
  latex_grob(
    tex,
    x = grid::unit(x, "bigpts"),
    y = grid::unit(1, "npc") - grid::unit(y, "bigpts"),
    hjust = 0, vjust = 1,
    max_width = width,
    input_mode = "math",
    gp = gp
  )
}

# The style tag a block is matched by. HTML's names, not our internal
# ones, so that `blockquote { }` in a stylesheet reads the way a CSS
# author expects it to.
.md_block_tag <- function(blk) {
  switch(blk$type,
    paragraph      = "p",
    mathblock      = "math",
    footnote       = "footnote",
    heading        = paste0("h", blk$level),
    code_block     = "pre",
    block_quote    = "blockquote",
    thematic_break = "hr",
    image          = "img",
    list           = if (isTRUE(blk$ordered)) "ol" else "ul",
    div            = "div",
    blk$type)
}

# Turn resolved properties into the grid gp a block draws with, plus the
# LaTeX commands that wrap its content.
#
# Colour, family, size and line height ride on gp, which latex_grob()
# bakes into the layout. Weight, slant and decoration have no gp
# equivalent -- gp$fontface is deliberately not honoured, since LaTeX
# controls the face -- so they wrap the content instead, reusing the
# same commands the inline walker emits.
.md_block_style <- function(gp, res, body, inherited_size) {
  size <- .md_css_length(res[["font-size"]], body, inherited_size) %||%
    inherited_size
  gp$fontsize <- size
  # Already folded into `body` by markdown_box_grob(); leaving it would
  # scale the block a second time.
  gp$cex <- NULL

  if (!is.null(res$color)) {
    hex <- .md_resolve_color(res$color)
    if (!is.null(hex)) gp$col <- hex
  }
  if (!is.null(res[["font-family"]])) {
    fam <- .md_css_family(res[["font-family"]])
    if (nzchar(fam)) gp$fontfamily <- fam
  }
  if (!is.null(res[["line-height"]])) {
    # Unitless, as line-height almost always is, and as gpar() wants it.
    lh <- suppressWarnings(as.numeric(res[["line-height"]]))
    if (is.finite(lh)) gp$lineheight <- lh
  }

  # Decorations wrap the emphasis, not the other way round. \underline
  # and \sout typeset their argument as a fresh sub-formula and drop the
  # font style inside it, so \textbf{\underline{x}} loses the bold while
  # \underline{\textbf{x}} keeps it.
  dec <- character(0)
  td <- tolower(as.character(res[["text-decoration"]] %||% ""))
  if (grepl("underline", td, fixed = TRUE)) dec <- c(dec, "\\underline{")
  if (grepl("overline", td, fixed = TRUE)) dec <- c(dec, "\\overline{")
  if (grepl("line-through", td, fixed = TRUE)) dec <- c(dec, "\\sout{")
  emph <- .md_emphasis_cmds(res)
  open <- c(dec, emph)

  list(gp = gp, size = size, open = paste(open, collapse = ""),
       close = strrep("}", length(open)), emph = emph)
}

# Lay out a list of blocks into positioned items.
#
# Returns list(items = <list of .md_item>, height = <total bigpts>).
# `indent` shifts everything right, which is how nested lists and block
# quotes are built -- each recursion re-measures its content at the
# reduced width so text still wraps inside the indent.
#
# `style` is the resolved cascade; `inherited` the properties of the
# enclosing container, so colour and font settings fall through into a
# block quote's contents while its margins and border do not.
.md_layout <- function(blocks, width, gp, fontsize, indent = 0, first = TRUE,
                       style = NULL, inherited = list()) {
  items <- list()
  y <- 0
  style <- style %||% markdown_style()
  # The container's font size, which `em` and `%` resolve against.
  base_size <- .md_css_length(inherited[["font-size"]], fontsize, fontsize) %||%
    fontsize

  add <- function(it) items[[length(items) + 1L]] <<- it
  # Baseline of the first line laid out here, in bigpts down from the top.
  # A list marker needs it to sit on the baseline of its item's text; it
  # cannot be assumed, because a tall first line (a superscript, say)
  # pushes that baseline further down. NA if nothing text-like came first.
  first_bl <- NA_real_
  note_bl <- function(v) if (is.na(first_bl)) first_bl <<- v

  for (i in seq_along(blocks)) {
    blk <- blocks[[i]]
    # A class matches the element carrying it and nothing else, as in
    # CSS: what reaches the blocks inside a styled <div> reaches them by
    # inheritance, not by matching the div's selector again.
    res <- .md_cascade(style, .md_block_tag(blk),
                       classes = blk$class %||% character(0),
                       inline = blk$style, inherited = inherited)
    sty <- .md_block_style(gp, res, fontsize, base_size)
    gp_blk <- sty$gp
    # Wrap once, so every branch below draws what the cascade asked for.
    wrap <- function(tex) paste0(sty$open, tex, sty$close)
    align <- .md_css_align(res[["text-align"]])

    # The first block in a container opens flush; a margin only ever
    # separates one block from another.
    if (!(first && i == 1L)) {
      y <- y + (.md_css_length(res[["margin-top"]], fontsize, sty$size) %||% 0)
    }
    len <- function(p, horizontal = FALSE) {
      .md_css_length(res[[p]], fontsize, sty$size, horizontal = horizontal) %||% 0
    }
    pad_l <- len("padding-left", TRUE)
    pad_r <- len("padding-right", TRUE)
    pad_t <- len("padding-top")
    pad_b <- len("padding-bottom")
    indent_blk <- indent + len("margin-left", TRUE) + pad_l
    avail <- width - indent_blk - pad_r - len("margin-right", TRUE)

    # A block background is the one box property no MicroTeX command can
    # give: every one of them hugs its content, and a code block's fill
    # has to span the column. The rect is emitted after the block's own
    # grob is measured, and inserted before it so it draws behind.
    bg <- .md_resolve_color(res$background)
    y_box <- y
    y <- y + pad_t
    n_before <- length(items)

    if (identical(blk$type, "paragraph") || identical(blk$type, "mathblock")) {
      if (!nzchar(blk$tex)) next
      tex <- wrap(blk$tex)
      m <- .md_measure(tex, avail, gp_blk)
      # Draw at m$mw, not avail: .md_measure() may have had to narrow the
      # request to get a fitting set of line breaks, and drawing at a
      # different width would re-break the text and undo the fit.
      add(.md_item(.md_run_grob(tex, indent_blk, y, m$mw, gp_blk),
                   indent_blk, y, m$w, m$h, align))
      note_bl(y + m$h - m$bl)
      y <- y + m$h

    } else if (identical(blk$type, "heading")) {
      # The size comes from gp, not a \Huge..\normalsize macro: the macro
      # ladder is six fixed steps, and font-size has to be continuous.
      tex <- wrap(blk$tex)
      m <- .md_measure(tex, avail, gp_blk)
      add(.md_item(.md_run_grob(tex, indent_blk, y, m$mw, gp_blk),
                   indent_blk, y, m$w, m$h, align))
      note_bl(y + m$h - m$bl)
      y <- y + m$h

    } else if (identical(blk$type, "code_block")) {
      # Code must render in a monospace font. \texttt switches MicroTeX's
      # internal style, but the \text{} content is drawn with the resolved
      # system font, which is the sans body font unless told otherwise --
      # and in a proportional font "<-" comes out misshapen, the "<" set
      # as a raised math relation while the "-" is a low text hyphen.
      # The default style says so; a stylesheet may say otherwise.
      #
      # Lines advance by a fixed leading rather than by their measured
      # height: a line with no descender measures shorter, and stacking
      # on that would let the next line's ascenders collide with it.
      code_lh <- (gp_blk$lineheight %||% 1.15) * sty$size
      # NULL when the fence names no language, or one we have no grammar
      # for; every line then renders plain, exactly as it used to.
      cls <- .hl_classes(blk$lines, blk$lang)
      if (is.null(cls)) cls <- vector("list", length(blk$lines))
      # A token is wrapped in \textcolor only when its class resolves to
      # a colour *different* from the block's own. An unstyled class
      # therefore emits no markup at all, which is what keeps an
      # unhighlighted block byte-identical to the old output.
      base_col <- .md_resolve_color(res$color)
      seen <- list()
      colour <- function(k) {
        if (!is.null(seen[[k]])) return(seen[[k]]$v)
        r <- .md_cascade(style, "pre",
                         classes = c(blk$class %||% character(0), k),
                         inline = blk$style, inherited = inherited)
        v <- .md_resolve_color(r$color)
        if (!is.null(v) && identical(v, base_col)) v <- NULL
        seen[[k]] <<- list(v = v)
        v
      }
      for (li in seq_along(blk$lines)) {
        tex <- wrap(.hl_code_tex(blk$lines[[li]], cls[[li]], colour))
        m <- .md_measure(tex, 0, gp_blk)
        add(.md_item(.md_run_grob(tex, indent_blk, y, 0, gp_blk),
                     indent_blk, y, m$w, code_lh, align))
        y <- y + code_lh
      }

    } else if (identical(blk$type, "table")) {
      # Content-sized already, from .md_blocks(). Only a fixed layout
      # needs rebuilding, because only it depends on `avail`.
      blk_tex <- blk$tex
      if (identical(tolower(as.character(res[["table-layout"]] %||% "")),
                    "fixed") && !is.null(blk$nd)) {
        blk_tex <- .md_table_tex(blk$nd, blk$spans, blk$base, style,
                                 avail = avail,
                                 inherited = blk$ctx %||% list())
      }
      if (!nzchar(blk_tex)) next
      tex <- wrap(blk_tex)
      m <- .md_measure(tex, 0, gp_blk)
      add(.md_item(.md_run_grob(tex, indent_blk, y, 0, gp_blk),
                   indent_blk, y, m$w, m$h, align))
      y <- y + m$h

    } else if (identical(blk$type, "image")) {
      # Shares the inline path's loader and grob builder, so a block image
      # gains SVG-as-vector for free. The sizing arithmetic below is
      # deliberately unchanged: .image_dims() reports the same
      # `w_px * 72/96` this used to compute inline, so a bitmap lays out
      # exactly as before.
      dims <- .image_dims(blk$path)
      g <- if (is.null(dims)) NULL else {
        # Scaled down to fit the column, never blown up past natural size.
        iw <- min(dims$w, avail)
        ih <- iw * dims$h / dims$w
        gg <- .image_grob(blk$path, iw, ih,
                          x = grid::unit(indent_blk, "bigpts"),
                          y = grid::unit(1, "npc") - grid::unit(y, "bigpts"))
        if (is.null(gg)) NULL else list(g = gg, w = iw, h = ih)
      }
      if (is.null(g)) {
        # Missing file, unsupported format, or no reader installed: show
        # the alt text so the reader still learns what belongs here.
        if (nzchar(blk$alt)) {
          tex <- wrap(blk$alt)
          m <- .md_measure(tex, avail, gp_blk)
          add(.md_item(.md_run_grob(tex, indent_blk, y, m$mw, gp_blk),
                       indent_blk, y, m$w, m$h, align))
          y <- y + m$h
        }
      } else {
        add(.md_item(g$g, indent_blk, y, g$w, g$h, align))
        y <- y + g$h
      }

    } else if (identical(blk$type, "thematic_break")) {
      # The rule sits centred in a band of its own, so `height` is the
      # band and the border is the ink drawn across the middle of it.
      h <- .md_css_length(res$height, fontsize, sty$size) %||% (0.5 * fontsize)
      bd <- .md_css_border(res[["border-top"]], fontsize)
      add(.md_item(
        grid::rectGrob(
          x = grid::unit(indent_blk, "bigpts"),
          y = grid::unit(1, "npc") - grid::unit(y + h / 2, "bigpts"),
          width = grid::unit(avail, "bigpts"),
          # Unstated thickness and colour resolve as they did before the
          # cascade existed: a hairline that grows with the text, inked
          # in the text colour.
          height = grid::unit(bd$width %||% max(1, sty$size / 20), "bigpts"),
          hjust = 0, vjust = 0.5,
          gp = grid::gpar(fill = bd$color %||% gp_blk$col %||% "black", col = NA)
        ),
        indent_blk, y, avail, h, align))
      y <- y + h

    } else if (identical(blk$type, "div")) {
      # A styled chunk. It draws no ink of its own: padding moves its
      # contents right, and its inheritable properties fall through.
      #
      # Transparent to margins too -- unlike a quote or a list item, a
      # div supplies no offset of its own, so its first child keeps its
      # top margin unless the div itself opens the container.
      inner <- .md_layout(blk$blocks, width, gp, fontsize,
                          indent = indent_blk, first = first && i == 1L,
                          style = style, inherited = res)
      for (it in inner$items) {
        add(.md_item(.md_shift_grob(it$grob, dy = y),
                     it$x, y + it$y, it$w, it$h, it$align))
      }
      y <- y + inner$height

    } else if (identical(blk$type, "footnote")) {
      # Laid out like a quote without the bar: the note's blocks, indented
      # by whatever padding the `footnote` rule asks for and inheriting its
      # reduced size.
      inner <- .md_layout(blk$blocks, width, gp, fontsize,
                          indent = indent_blk, first = TRUE,
                          style = style, inherited = res)
      for (it in inner$items) {
        add(.md_item(.md_shift_grob(it$grob, dy = y),
                     it$x, y + it$y, it$w, it$h, it$align))
      }
      y <- y + inner$height

    } else if (identical(blk$type, "block_quote")) {
      bd <- .md_css_border(res[["border-left"]], fontsize)
      bar <- bd$width %||% (0.125 * fontsize)
      # padding-left has already moved indent_blk; the bar is drawn at
      # the quote's own left edge, in the space that padding opened.
      inner <- .md_layout(blk$blocks, width, gp, fontsize,
                          indent = indent_blk, first = TRUE,
                          style = style, inherited = res)
      for (it in inner$items) {
        add(.md_item(.md_shift_grob(it$grob, dy = y),
                     it$x, y + it$y, it$w, it$h, it$align))
      }
      # The bar spans the quoted content, added after so it is measured
      # against the final height.
      add(.md_item(
        grid::rectGrob(
          x = grid::unit(indent, "bigpts"),
          y = grid::unit(1, "npc") - grid::unit(y, "bigpts"),
          width = grid::unit(max(1, bar), "bigpts"),
          height = grid::unit(inner$height, "bigpts"),
          hjust = 0, vjust = 1,
          gp = grid::gpar(fill = bd$color %||% gp_blk$col %||% "grey60",
                          col = NA)
        ),
        indent, y, bar, inner$height
      ))
      y <- y + inner$height

    } else if (identical(blk$type, "list")) {
      # A tight list closes its items up; CommonMark decides which it is.
      res_li <- .md_cascade(style, "li", inherited = res)
      sty_li <- .md_block_style(gp, res_li, fontsize, sty$size)
      gap_item <- if (isTRUE(blk$tight)) 0.25 * fontsize else
        (.md_css_length(res_li[["margin-top"]], fontsize, sty_li$size) %||%
           (0.55 * fontsize))
      # Markers baseline-align to the first line of item text. Without
      # this the marker is top-aligned, so a bullet floats near the top
      # of the line instead of sitting on the text baseline. An ordinary
      # ascender/descender line stands in for a body with no baseline of
      # its own to align to.
      refd <- latex_dims("\\text{Ag}", input_mode = "math", gp = gp_blk)
      ref_bl_top <- as.numeric(refd$height) - as.numeric(refd$baseline)
      marker_gap <- .md_css_length(res[["marker-gap"]], fontsize, sty$size,
                                   horizontal = TRUE) %||% (0.4 * fontsize)
      for (j in seq_along(blk$items)) {
        if (j > 1L) y <- y + gap_item
        marker <- .md_list_marker(blk$ordered, blk$start, j,
                                  paren = isTRUE(blk$paren),
                                  checked = if (is.null(blk$checked)) NA
                                            else blk$checked[j],
                                  bullet = res$bullet)
        mm <- .md_measure(marker, 0, gp_blk)
        # Hanging indent: the marker sits at the current indent and the
        # item body is measured at the reduced width, so continuation
        # lines line up under the text rather than under the marker.
        # This is the reason lists are stacked here instead of being
        # handed to MicroTeX's itemize -- max_width does not reach into
        # its cells.
        body_indent <- indent_blk + mm$w + marker_gap
        inner <- .md_layout(blk$items[[j]], width, gp, fontsize,
                            indent = body_indent, first = TRUE,
                            style = style, inherited = res_li)
        # Shift the marker down so its own baseline lands on the text
        # baseline of the first line -- the item's own baseline, not the
        # reference one, since a tall line ($x^2$, say) sits lower in its
        # box and a marker placed at the generic offset floats above it.
        # The reference is only a fallback for a body that opens with
        # something untextual, like a code block.
        bl_top <- if (is.na(inner$first_bl)) ref_bl_top else inner$first_bl
        y_marker <- y + bl_top - (mm$h - mm$bl)
        note_bl(y_marker + mm$h - mm$bl)
        add(.md_item(.md_run_grob(marker, indent_blk, y_marker, 0, gp_blk),
                     indent_blk, y_marker, mm$w, mm$h))
        for (it in inner$items) {
          add(.md_item(.md_shift_grob(it$grob, dy = y),
                       it$x, y + it$y, it$w, it$h, it$align))
        }
        # Advance past whichever reaches lower: the body, or a marker that
        # was pushed below it.
        y <- y + max(inner$height, (y_marker - y) + mm$h)
      }
    }

    y <- y + pad_b

    # The background spans the block plus its padding. Inserted *before*
    # the grobs it sits behind, matching how the box-level background is
    # emitted first in makeContent.markdownbox().
    if (!is.null(bg) && length(items) > n_before) {
      rect <- .md_item(
        grid::rectGrob(
          x = grid::unit(indent + len("margin-left", TRUE), "bigpts"),
          y = grid::unit(1, "npc") - grid::unit(y_box, "bigpts"),
          width = grid::unit(max(avail + pad_l + pad_r, 1), "bigpts"),
          height = grid::unit(max(y - y_box, 1), "bigpts"),
          hjust = 0, vjust = 1,
          gp = grid::gpar(fill = bg, col = NA)
        ),
        indent, y_box, avail + pad_l + pad_r, y - y_box
      )
      items <- append(items, list(rect), after = n_before)
    }

    # Margins do not collapse here, unlike CSS: the space below a block
    # simply adds to the space above the next. Every built-in default
    # leaves margin-bottom unset, so this changes nothing until a
    # stylesheet asks for it.
    y <- y + (.md_css_length(res[["margin-bottom"]], fontsize, sty$size) %||% 0)
  }

  list(items = items, height = y, first_bl = first_bl)
}

# Validate a trbl unit. Split out from .md_trbl() so the constructor can
# reject a bad padding/margin immediately, rather than at draw time when
# the error surfaces far from the call that caused it.
.md_check_trbl <- function(u, what) {
  if (is.null(u)) return(invisible(NULL))
  if (!grid::is.unit(u)) {
    stop("`", what, "` must be a grid unit.", call. = FALSE)
  }
  if (!length(u) %in% c(1L, 4L)) {
    stop("`", what, "` must have length 1 or 4 (top, right, bottom, left).",
         call. = FALSE)
  }
  invisible(NULL)
}

# Resolve a length-1 or length-4 unit into trbl big points.
.md_trbl <- function(u, what) {
  if (is.null(u)) return(c(0, 0, 0, 0))
  .md_check_trbl(u, what)
  n <- length(u)
  v <- vapply(seq_len(n), function(i) {
    # Vertical and horizontal components are converted with the matching
    # function so relative units resolve against the right dimension.
    if (i %% 2L == 1L) {
      grid::convertHeight(u[i], "bigpts", valueOnly = TRUE)
    } else {
      grid::convertWidth(u[i], "bigpts", valueOnly = TRUE)
    }
  }, numeric(1))
  if (n == 1L) rep(v, 4L) else v
}

# The measure used to find a box's natural width: wide enough that no
# realistic line wraps, small enough that a `table-layout: fixed` table
# does not ask MicroTeX for absurd column widths.
.MD_PROBE_W <- 1e4

# The box's own chrome, as declared by `body {}` in the stylesheet.
#
# These are box properties, so .md_cascade() deliberately strips them out
# of the seed it hands the blocks -- a background on `body` must not
# become a background on every paragraph. That leaves the box itself as
# the only thing that can consume them, which is what this does. Each
# component is NULL when `body` says nothing, so the caller can tell
# "unset" from "set to zero" and let an explicit argument win.
.md_body_chrome <- function(body, fontsize) {
  side <- function(p, horizontal) {
    .md_css_length(body[[p]], fontsize, fontsize, horizontal = horizontal) %||% 0
  }
  trbl <- function(prefix) {
    nm <- paste0(prefix, c("-top", "-right", "-bottom", "-left"))
    if (!any(nm %in% names(body))) return(NULL)
    c(side(nm[1], FALSE), side(nm[2], TRUE), side(nm[3], FALSE), side(nm[4], TRUE))
  }

  bd <- .md_css_border(body[["border"]], fontsize)
  fill <- .md_resolve_color(body[["background"]])
  col <- bd$color %||% .md_resolve_color(body[["border-color"]])
  box_gp <- if (!is.null(fill) || !is.null(col)) {
    # A border width is a length in big points; gpar()'s lwd is 1/96in.
    lwd <- if (is.null(bd$width)) 1 else bd$width * 96 / 72
    grid::gpar(fill = fill %||% NA, col = col %||% NA, lwd = lwd)
  }
  radius <- .md_css_length(body[["border-radius"]], fontsize, fontsize,
                           horizontal = TRUE)

  list(margin = trbl("margin"), padding = trbl("padding"),
       box_gp = box_gp,
       r = if (!is.null(radius)) grid::unit(radius, "bigpts"))
}

# Do the geometry once: resolve the box, lay the blocks out, and report
# every dimension makeContent() and the *Details() methods need. Must run
# at draw time, because the width may be relative and because measuring
# text needs an open device.
.md_box_layout <- function(x) {
  gp <- x$gp %||% grid::gpar()
  # markdown_box_grob() has already folded cex in, so this is the drawn
  # size; .md_base_size() is the same rule latex_grob() applies.
  fontsize <- .md_base_size(gp)

  style <- x$style %||% markdown_style()
  # `body` is the document root. It seeds the inheritance every block
  # starts from, and -- since .md_cascade() filters that seed down to the
  # inheritable properties -- it is also the only place a box property can
  # be declared without leaking into every block inside the box.
  body <- .md_cascade(style, "body")
  chrome <- .md_body_chrome(body, fontsize)

  # An argument is the inline style: it wins over the stylesheet.
  zero <- c(0, 0, 0, 0)
  margin <- if (is.null(x$margin)) chrome$margin %||% zero else
    .md_trbl(x$margin, "margin")
  padding <- if (is.null(x$padding)) chrome$padding %||% zero else
    .md_trbl(x$padding, "padding")

  if (is.null(x$width)) {
    # Natural width. Lay out once at a width nothing realistic wraps at,
    # then take the widest item -- skipping the ones that are full-width
    # by construction (a rule, and a block background) since those just
    # hand the probe width back.
    probe <- .md_layout(x$blocks, .MD_PROBE_W, gp, fontsize, style = style,
                        inherited = body)
    ink <- vapply(probe$items, function(it) {
      if (it$w < .MD_PROBE_W * 0.99) it$x + it$w else NA_real_
    }, numeric(1))
    content_w <- if (all(is.na(ink))) fontsize * 20 else max(ink, na.rm = TRUE)
    # A point of slack, because latex_dims() reports whole big points:
    # handing a rounded-down width back as the measure is enough to make
    # MicroTeX break the very line it was measured from.
    content_w <- max(content_w, 1) + 1
    box_w <- content_w + padding[2] + padding[4]
    total_w <- box_w + margin[2] + margin[4]
  } else {
    total_w <- grid::convertWidth(x$width, "bigpts", valueOnly = TRUE)
    box_w <- total_w - margin[2] - margin[4]
    content_w <- max(box_w - padding[2] - padding[4], 1)
  }

  laid <- .md_layout(x$blocks, content_w, gp, fontsize, style = style,
                     inherited = body)

  content_h <- laid$height
  box_h <- content_h + padding[1] + padding[3]
  total_h <- if (is.null(x$height)) {
    box_h + margin[1] + margin[3]
  } else {
    grid::convertHeight(x$height, "bigpts", valueOnly = TRUE)
  }
  if (!is.null(x$height)) box_h <- total_h - margin[1] - margin[3]

  list(margin = margin, padding = padding,
       total_w = total_w, total_h = total_h,
       box_w = box_w, box_h = box_h,
       content_w = content_w, content_h = content_h,
       box_gp = x$box_gp %||% chrome$box_gp,
       r = x$r %||% chrome$r,
       items = laid$items)
}

#' @export
makeContent.markdownbox <- function(x) {
  lay <- .md_box_layout(x)
  kids <- list()

  # Background / border, inset by the margin.
  if (!is.null(lay$box_gp)) {
    rr <- lay$r %||% grid::unit(0, "pt")
    common <- list(
      x = grid::unit(lay$margin[4], "bigpts"),
      y = grid::unit(1, "npc") - grid::unit(lay$margin[1], "bigpts"),
      width = grid::unit(lay$box_w, "bigpts"),
      height = grid::unit(lay$box_h, "bigpts"),
      # `just`, not hjust/vjust: roundrectGrob() has no hjust/vjust
      # arguments, and rectGrob() honours either.
      just = c(0, 1), gp = lay$box_gp
    )
    box <- if (grid::convertWidth(rr, "bigpts", valueOnly = TRUE) > 0) {
      do.call(grid::roundrectGrob, c(common, list(r = rr)))
    } else {
      do.call(grid::rectGrob, common)
    }
    kids[[length(kids) + 1L]] <- box
  }

  # Vertical placement of the content inside a taller fixed box.
  slack <- max(lay$box_h - lay$padding[1] - lay$padding[3] - lay$content_h, 0)
  voff <- slack * (1 - (x$valign %||% 1))

  halign <- x$halign %||% 0
  content <- lapply(lay$items, function(it) {
    # A block's own text-align wins; the box-wide halign is the default
    # for blocks that state none.
    a <- it$align %||% halign
    if (a == 0) return(it$grob)
    # Slack to the right of this item, so alignment cooperates with the
    # per-block indent instead of fighting it.
    .md_shift_grob(it$grob, dx = (lay$content_w - it$x - it$w) * a)
  })

  kids <- c(kids, list(grid::gTree(
    children = do.call(grid::gList, content),
    vp = grid::viewport(
      x = grid::unit(lay$margin[4] + lay$padding[4], "bigpts"),
      y = grid::unit(1, "npc") -
        grid::unit(lay$margin[1] + lay$padding[1] + voff, "bigpts"),
      width = grid::unit(lay$content_w, "bigpts"),
      height = grid::unit(lay$content_h, "bigpts"),
      just = c(0, 1)
    )
  )))
  kids <- do.call(grid::gList, kids)

  # A natural-width box has no viewport of its own: its size is not known
  # until the blocks have been measured, which is here. Everything above
  # is positioned in big points from the box's top-left, so one viewport
  # of the measured size, placed at the requested just, anchors the lot.
  if (is.null(x$width)) {
    kids <- grid::gList(grid::gTree(
      children = kids,
      vp = grid::viewport(
        x = x$box_x, y = x$box_y,
        width = grid::unit(lay$total_w, "bigpts"),
        height = grid::unit(lay$total_h, "bigpts"),
        just = c(x$box_hjust %||% 0.5, x$box_vjust %||% 0.5)
      )
    ))
  }

  grid::setChildren(x, kids)
}

#' @export
widthDetails.markdownbox <- function(x) {
  grid::unit(.md_box_layout(x)$total_w, "bigpts")
}

#' @export
heightDetails.markdownbox <- function(x) {
  grid::unit(.md_box_layout(x)$total_h, "bigpts")
}

#' Render a markdown document as a boxed grid grob
#'
#' @description
#' Lays markdown out as a block document --- headings, paragraphs, lists
#' (including GFM task lists), block quotes, code blocks, tables,
#' horizontal rules and images --- inside an optional padded, filled and
#' bordered box. Prose wraps to the requested width, and \code{$...$}
#' math is typeset by MicroTeX as usual. All the inline formatting
#' \code{\link{markdown_grob}} understands, including the inline HTML
#' subset, works inside every block.
#'
#' @details
#' Where \code{\link{markdown_grob}} flattens everything into a single
#' run, this stacks one grob per block. That is what makes headings, list
#' indentation, block-quote rules and background fills possible:
#' MicroTeX has no concept of any of them, and its line breaking does not
#' reach inside the cells it uses for list and table layout.
#'
#' Two consequences worth knowing. Table cells and code lines are not
#' wrapped, so a wide table overflows rather than reflowing. And list
#' items are stacked here rather than handed to MicroTeX's
#' \code{itemize}, which is what gives them a proper hanging indent.
#'
#' An image on a line of its own is drawn as a raster, scaled to fit the
#' column but never enlarged past its natural size (pixels are read at
#' 96 dpi). PNG needs the \pkg{png} package and JPEG needs \pkg{jpeg},
#' both \emph{Suggests}: when the reader is not installed, the file is
#' missing, or the format is anything else, the image degrades to its alt
#' text. An image \emph{within} a sentence stays inline, where only its
#' alt text survives.
#'
#' The layout is computed at draw time, so an open device is required ---
#' which is what lets a relative \code{width} and the measured height of
#' the text resolve against the viewport the box is actually drawn in.
#'
#' @section Styling:
#' Appearance comes from a small CSS cascade. \code{style} sets the house
#' style for the whole document, by tag:
#'
#' \preformatted{markdown_box_grob(md, style = markdown_style(
#'   h1         = md_style(color = "steelblue", font_size = 2),
#'   blockquote = md_style(border_left = "3px solid grey60")
#' ))}
#'
#' The same thing written as CSS, which \code{style} also takes directly,
#' as text or as the path to a \code{.css} file:
#'
#' \preformatted{markdown_box_grob(md, style = "
#'   h1 \{ color: steelblue; font-size: 2rem \}
#'   blockquote \{ border-left: 3px solid grey60 \}
#' ")}
#'
#' To style one chunk rather than every block of a kind, wrap it in a
#' \code{<div>} carrying a \code{class} or a \code{style}. \strong{Leave
#' blank lines around the tags} --- that is what makes CommonMark parse
#' the markdown between them instead of treating the whole thing as raw
#' HTML:
#'
#' \preformatted{<div class="note">
#'
#' ## This heading only
#'
#' </div>}
#'
#' Inline runs take \code{class} as well as \code{style} on a
#' \code{<span>}. See \code{\link{markdown_style}} for the tag names, the
#' supported properties and how the cascade resolves.
#'
#' @param md Character string of markdown.
#' @param x,y Position of the box in the parent viewport.
#' @param width Width of the box, including \code{margin}. \code{NULL}
#'   sizes the box to its content, so nothing wraps --- useful where the
#'   available width is not known, as in a ggplot2 theme element.
#' @param height Fixed height, or \code{NULL} (default) to take whatever
#'   height the content needs.
#' @param hjust,vjust Justification of the whole box about \code{x} and
#'   \code{y}.
#' @param halign Horizontal alignment of blocks within the box:
#'   \code{0} left (default), \code{0.5} centred, \code{1} right.
#' @param valign Vertical alignment of the content when \code{height}
#'   leaves room to spare: \code{1} top (default), \code{0} bottom.
#' @param padding,margin A \code{\link[grid]{unit}} of length 1 or 4
#'   giving top, right, bottom and left. Padding is inside the box,
#'   margin outside it. \code{NULL} (default) takes them from the
#'   stylesheet's \code{body} rule, and is zero if that says nothing.
#' @param box_gp Graphical parameters for the box itself, e.g.
#'   \code{gpar(fill = "grey95", col = "black")}. \code{NULL} (default)
#'   takes the fill from \code{body \{ background \}} and the border from
#'   \code{body \{ border \}}, and draws no box if neither is set.
#' @param r Corner radius; a non-zero value draws a rounded box.
#'   \code{NULL} (default) takes it from \code{body \{ border-radius \}}.
#' @param style Appearance of the blocks: a \code{\link{markdown_style}}
#'   object, CSS text, or a path to a \code{.css} file. \code{NULL}
#'   (default) uses \code{latex_options("markdown_style")} if set, and
#'   the built-in defaults otherwise.
#' @param name Optional grob name.
#' @param gp Graphical parameters for the text. \code{fontsize} also sets
#'   the scale for block spacing and list indentation, and \code{cex}
#'   multiplies it as elsewhere in \pkg{grid}.
#' @param vp Optional viewport. Supplying one replaces the viewport built
#'   from \code{x}, \code{y}, \code{width}, \code{height}, \code{hjust}
#'   and \code{vjust}, so those are then ignored.
#' @return A \code{markdownbox} gTree.
#' @seealso \code{\link{markdown_grob}}, \code{\link{latex_grob}}
#' @export
#'
#' @examples
#' \donttest{
#'   md <- paste(
#'     "# Results", "",
#'     "The slope is $\\beta_1$ with *p* < 0.001.", "",
#'     "- first point", "- second point",
#'     sep = "\n"
#'   )
#'   grid::grid.newpage()
#'   grid::grid.draw(markdown_box_grob(
#'     md,
#'     width = grid::unit(4, "in"),
#'     padding = grid::unit(8, "pt"),
#'     box_gp = grid::gpar(fill = "grey95", col = "grey40")
#'   ))
#' }
markdown_box_grob <- function(md,
                              x = grid::unit(0.5, "npc"),
                              y = grid::unit(0.5, "npc"),
                              width = grid::unit(1, "npc"),
                              height = NULL,
                              hjust = 0.5, vjust = 0.5,
                              halign = 0, valign = 1,
                              padding = NULL,
                              margin = NULL,
                              box_gp = NULL,
                              r = NULL,
                              style = NULL,
                              name = NULL,
                              gp = grid::gpar(),
                              vp = NULL) {
  .md_check_string(md)
  .md_check_trbl(padding, "padding")
  .md_check_trbl(margin, "margin")
  gp <- .md_fold_cex(gp)
  style <- .md_as_style(style)
  # Parse once at construction; the layout is redone at draw time, when
  # relative widths resolve and a device is available for measuring.
  # The style has to be in hand here as well as at draw time, because a
  # <span class> compiles to LaTeX commands during the parse.
  blocks <- .md_parse_blocks(md, .md_base_size(gp), style,
                             .md_cascade(style, "body"))

  # A relative width can be resolved by a viewport built here; a natural
  # width cannot, because it is not known until the blocks are measured at
  # draw time. In that case the grob carries no viewport and
  # makeContent() builds one of the measured size instead.
  if (is.null(vp) && !is.null(width)) {
    vp <- grid::viewport(
      x = x, y = y,
      width = width,
      height = height %||% grid::unit(1, "npc"),
      just = c(hjust, vjust)
    )
  }

  grid::gTree(
    md = md, blocks = blocks, style = style,
    width = width, height = height,
    halign = halign, valign = valign,
    padding = padding, margin = margin,
    box_gp = box_gp, r = r,
    box_x = x, box_y = y, box_hjust = hjust, box_vjust = vjust,
    gp = gp, name = name, cl = "markdownbox",
    vp = vp
  )
}

test_that("math spans survive the markdown parser byte-for-byte", {
  # The regression that motivates masking: CommonMark treats `\` before
  # ASCII punctuation as an escape, so an unmasked `\\` row separator
  # collapses to `\` and the matrix loses its second row.
  for (m in c("$x^2$", "$x_i + y_i$", "$\\frac{a}{b}$",
              "$\\sum_{i=1}^n \\alpha_i$",
              "$\\begin{matrix}a\\\\b\\end{matrix}$")) {
    masked <- .md_mask_math(m)
    expect_identical(.md_unmask_math(masked$text, masked$spans), m)
  }
})

test_that("a matrix keeps both rows through markdown_grob()", {
  g <- markdown_grob("matrix $\\begin{matrix}a\\\\b\\end{matrix}$")
  flat <- markdown_grob("matrix ab")
  # Two stacked rows must be taller than the same glyphs on one line.
  expect_gt(g$bbox_h, flat$bbox_h)
  expect_match(.md_to_tex("$\\begin{matrix}a\\\\b\\end{matrix}$"),
               "\\begin{matrix}a\\\\b\\end{matrix}", fixed = TRUE)
})

test_that("inline markdown maps onto the expected LaTeX commands", {
  expect_match(.md_to_tex("**b**"), "\\textbf{", fixed = TRUE)
  expect_match(.md_to_tex("*i*"),   "\\textit{", fixed = TRUE)
  expect_match(.md_to_tex("`c`"),   "\\texttt{", fixed = TRUE)
  # \sout is the horizontal rule (ulem); \cancel is a diagonal slash and
  # is NOT the right target for ~~...~~.
  expect_match(.md_to_tex("~~s~~"), "\\sout{", fixed = TRUE)
  expect_false(grepl("\\cancel", .md_to_tex("~~s~~"), fixed = TRUE))
})

test_that("prose is wrapped in \\text{} rather than left as math italics", {
  # Output feeds latex_grob(input_mode = "math"), so bare prose would be
  # typeset as spaced math italics.
  expect_match(.md_to_tex("plain words"), "\\text{plain words}", fixed = TRUE)
  g <- markdown_grob("plain words here")
  expect_true(all(g$layout_df$type == "text"))
})

test_that("markdown-only constructs never emit unknown LaTeX commands", {
  # MicroTeX renders an unknown command as literal glyphs instead of
  # erroring, so a leaked \href would silently typeset the letters.
  tex <- .md_to_tex(paste(
    "# Heading", "", "a [link](http://x.com) and ![alt](i.png)", "",
    "> quote", "", "```", "code", "```", sep = "\n"))
  for (bad in c("\\section", "\\href", "\\includegraphics", "verbatim",
                "\\linewidth", "\\begin{quote}")) {
    expect_false(grepl(bad, tex, fixed = TRUE), label = paste("leaked", bad))
  }
  # Link and image degrade to their text, which must survive.
  expect_match(.md_to_tex("[label](http://x)"), "label", fixed = TRUE)
  expect_match(.md_to_tex("![alt text](i.png)"), "alt text", fixed = TRUE)
})

test_that("TeX special characters in prose are escaped", {
  expect_match(.md_to_tex("100% done"), "100\\% done", fixed = TRUE)
  expect_match(.md_to_tex("a_b"),       "a\\_b",       fixed = TRUE)
  expect_match(.md_to_tex("x & y"),     "x \\& y",     fixed = TRUE)
  expect_match(.md_to_tex("no #1"),     "no \\#1",     fixed = TRUE)
  # `^` is deliberately NOT escaped. \^{} is the circumflex *accent*,
  # which floats above the line; inside \text{} MicroTeX already draws a
  # bare `^` flat on the baseline. Likewise `~` becomes \char126{}, not
  # the \~{} accent -- see test-markdown-highlight.R.
  expect_match(.md_to_tex("2 ^ 3"),     "2 ^ 3",       fixed = TRUE)
  expect_false(grepl("\\^{}", .md_to_tex("2 ^ 3"), fixed = TRUE))
  expect_match(.md_to_tex("a ~ b"),     "\\char126{}", fixed = TRUE)
  # MicroTeX has no \textbackslash -- it would typeset the letters.
  expect_match(.md_to_tex("a \\ b"), "\\backslash{}", fixed = TRUE)
  expect_false(grepl("textbackslash", .md_to_tex("a \\ b"), fixed = TRUE))
})

test_that("a lone dollar sign is literal, not an unclosed math span", {
  # In prose a stray `$` is far more often a price than a delimiter.
  # Masking it would swallow the rest of the line and skip escaping.
  tex <- .md_to_tex("costs $ and & more")
  expect_match(tex, "\\$", fixed = TRUE)
  expect_match(tex, "\\&", fixed = TRUE)
  expect_no_warning(.md_to_tex("costs $ and more"))
})

test_that("code spans stay literal and never leak a sentinel", {
  tex <- .md_to_tex("`code with $x$ inside`")
  expect_match(tex, "\\$x\\$", fixed = TRUE)
  # The private-use sentinels must never reach the output.
  expect_false(grepl("\uE000", tex, fixed = TRUE))
  expect_false(grepl("\uE001", tex, fixed = TRUE))
  expect_false(grepl("\uE002", tex, fixed = TRUE))
})

test_that("wrapping still works for single-paragraph markdown", {
  # Guards against a stray `\\` sneaking into the output: MicroTeX
  # silently ignores max_width once the box tree contains a line break.
  long <- paste(rep("The quick brown fox jumps over the lazy dog.", 4),
                collapse = " ")
  g <- markdown_grob(paste("**bold**", long), max_width = 200)
  expect_true(g$is_split)
  expect_lte(g$bbox_w, 200)
})

test_that("markdown_grob() validates its input", {
  expect_error(markdown_grob(NA_character_), "must not be NA")
  expect_error(markdown_grob(c("a", "b")), "single character string")
  expect_error(markdown_grob(42), "single character string")
  expect_s3_class(markdown_grob("ok"), "latexgrob")
})

test_that("grid.markdown() draws and returns the grob invisibly", {
  pdf(NULL)
  on.exit(dev.off(), add = TRUE)
  grid::grid.newpage()
  expect_invisible(g <- grid.markdown("**hi** $x$"))
  # The returned grob is the one that was drawn, not a fresh empty shell.
  expect_s3_class(g, "latexgrob")
  expect_true(all(c("text", "glyph") %in% g$layout_df$type))
})

# --- block-level markdown (markdown_box_grob) ------------------------

md_doc <- paste(
  "# Title", "",
  "First paragraph that is long enough to need wrapping when the box",
  "is narrow.", "",
  "## Section", "",
  "- alpha", "- beta", "",
  "1. one", "2. two", "",
  "> quoted", "",
  "```", "x <- 1", "y <- 2", "```", "",
  "| a | b |", "|---|---|", "| 1 | 2 |", "",
  "---",
  sep = "\n"
)

test_that("every block type is recognised", {
  types <- vapply(.md_parse_blocks(md_doc), function(b) b$type, character(1))
  expect_setequal(
    types,
    c("heading", "paragraph", "heading", "list", "list",
      "block_quote", "code_block", "table", "thematic_break")
  )
  blks <- .md_parse_blocks(md_doc)
  expect_equal(blks[[1]]$level, 1L)
  expect_equal(blks[[3]]$level, 2L)
  expect_false(Filter(function(b) b$type == "list", blks)[[1]]$ordered)
  expect_true(Filter(function(b) b$type == "list", blks)[[2]]$ordered)
})

test_that("a markdown table becomes a real tabular", {
  tex <- Filter(function(b) b$type == "table", .md_parse_blocks(md_doc))[[1]]$tex
  expect_match(tex, "\\begin{tabular}", fixed = TRUE)
  expect_match(tex, "\\thickhline", fixed = TRUE)
  expect_match(tex, "\\hline", fixed = TRUE)
  expect_match(tex, "&", fixed = TRUE)
  # It must be renderable, not just well-formed.
  expect_gt(as.numeric(latex_dims(tex, input_mode = "math")$width), 0)
})

test_that("measuring and drawing agree, so blocks stay inside the box", {
  # latex_dims() defaults to input_mode = "mixed"; measuring a run that
  # the AST walker already wrapped would double-wrap it and report a
  # width the drawn grob does not have. Blocks then overflow the border.
  skip_if_not_installed("ragg")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  tex <- .md_parse_blocks("Some *emphasised* prose with $x^2$ inside it.")[[1]]$tex
  gp <- grid::gpar(fontsize = 13)
  for (w in c(120, 200, 280)) {
    m <- .md_measure(tex, w, gp)
    g <- latex_grob(tex, max_width = m$mw, input_mode = "math", gp = gp)
    expect_equal(m$w, g$bbox_w, tolerance = 1e-6)
    expect_lte(g$bbox_w, w + 0.5)
  }
})

test_that("a soft line break renders as a word space", {
  # In math mode a bare space between two \text{} runs is discarded, so
  # the words either side would be run together ("islong").
  tex <- .md_parse_blocks("alpha\nbeta")[[1]]$tex
  expect_match(tex, "\\text{ }", fixed = TRUE)
  spaced <- latex_grob(tex, input_mode = "math")$bbox_w
  joined <- latex_grob("\\text{alpha}\\text{beta}", input_mode = "math")$bbox_w
  expect_gt(spaced, joined)
})

test_that("the box grows taller as it is made narrower", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  h <- vapply(c(6, 4, 3), function(wi) {
    g <- markdown_box_grob(md_doc, width = grid::unit(wi, "in"))
    grid::convertHeight(grid::grobHeight(g), "in", valueOnly = TRUE)
  }, numeric(1))
  expect_true(all(diff(h) > 0))
})

test_that("padding and margin enlarge the reported size", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  bare <- markdown_box_grob("text", width = grid::unit(3, "in"))
  padded <- markdown_box_grob("text", width = grid::unit(3, "in"),
                              padding = grid::unit(12, "pt"))
  hb <- grid::convertHeight(grid::grobHeight(bare), "bigpts", valueOnly = TRUE)
  hp <- grid::convertHeight(grid::grobHeight(padded), "bigpts", valueOnly = TRUE)
  expect_gt(hp, hb)
  # Width is the requested width; padding eats into the content, not out.
  expect_equal(
    grid::convertWidth(grid::grobWidth(padded), "in", valueOnly = TRUE), 3,
    tolerance = 1e-6
  )
})

test_that("padding and margin reject bad units", {
  expect_error(markdown_box_grob("x", padding = 5), "grid unit")
  expect_error(markdown_box_grob("x", margin = grid::unit(c(1, 2), "pt")),
               "length 1 or 4")
})

test_that("makeContent produces a box plus the stacked content", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g <- markdown_box_grob(md_doc, width = grid::unit(4, "in"),
                         box_gp = grid::gpar(fill = "grey95"))
  kids <- grid::makeContent(g)$children
  expect_length(kids, 2L)                      # box rect + content tree
  expect_gt(length(kids[[2]]$children), 5L)    # one grob per block/line
  # With no box_gp there is no rect.
  g2 <- markdown_box_grob(md_doc, width = grid::unit(4, "in"))
  expect_length(grid::makeContent(g2)$children, 1L)
})

test_that("markdown_box_grob validates its input", {
  expect_error(markdown_box_grob(NA_character_), "must not be NA")
  expect_error(markdown_box_grob(c("a", "b")), "single character string")
  expect_s3_class(markdown_box_grob("ok"), "markdownbox")
})

# "it draws without error" lived here. The snapshot below draws the same
# document and compares every coordinate, so the weaker claim added
# nothing that could fail on its own.

test_that("visual: markdown document box", {
  skip_if_not_installed("vdiffr")
  skip_on_os("mac")
  # The code block looks like body text in this snapshot, and "x <- 1"
  # reads as an underscore because a proportional hyphen sits low. That
  # is the snapshot device, not the layout: vdiffr writes one family name
  # for every family, so a mono run is recorded as `font-family: sans`
  # and any viewer draws it proportionally. The grob really does carry
  # fontfamily="mono" -- see the `pre` rule in .md_default_rules().
  vdiffr::expect_doppelganger("markdown-box", function() {
    grid::grid.draw(markdown_box_grob(
      md_doc,
      width = grid::unit(4.4, "in"),
      padding = grid::unit(10, "pt"),
      box_gp = grid::gpar(fill = "grey96", col = "grey40"),
      gp = grid::gpar(fontsize = 13)
    ))
  })
})

test_that("a raw HTML block is dropped, not typeset", {
  # An html_block holds raw markup as its text. Walking into it typeset
  # the tags and the unparsed markdown inside them, and the inline path
  # and the block path disagreed about the same document.
  md <- "Before\n\n<div class='x'>raw **html** block</div>\n\nAfter"
  tex <- .md_to_tex(md)
  expect_false(grepl("<div", tex, fixed = TRUE))
  expect_false(grepl("**html**", tex, fixed = TRUE))
  expect_match(tex, "Before", fixed = TRUE)
  expect_match(tex, "After", fixed = TRUE)

  # Both entry points must reach the same conclusion.
  types <- vapply(.md_parse_blocks(md), function(b) b$type, character(1))
  expect_setequal(types, c("paragraph", "paragraph"))

  g <- markdown_grob(md)
  expect_false(grepl("<div", paste(stats::na.omit(g$layout_df$text),
                                   collapse = ""), fixed = TRUE))
})

# --- CommonMark coverage gaps ------------------------------------------

test_that("GFM task list items get a checkbox marker", {
  # cmark emits <tasklist completed="..."> in place of <item>; without
  # handling it the checkbox is silently dropped and a task list looks
  # like an ordinary bullet list.
  l <- Filter(function(b) b$type == "list",
              .md_parse_blocks("- [ ] todo\n- [x] done\n- plain"))[[1]]
  expect_equal(l$checked, c(FALSE, TRUE, NA))
  expect_equal(.md_list_marker(FALSE, 1, 1, FALSE, FALSE), "\\square")
  expect_equal(.md_list_marker(FALSE, 1, 2, FALSE, TRUE), "\\blacksquare")
  expect_equal(.md_list_marker(FALSE, 1, 3, FALSE, NA), "\\bullet")
  # The two checkbox states must be single glyphs AND the same size, or a
  # task list looks ragged. \square and \blacksquare are both 14x13;
  # \boxtimes (the more literal "checked") is 18x16 and was rejected for
  # exactly this reason.
  d_unchecked <- latex_dims("\\square", input_mode = "math")
  d_checked   <- latex_dims("\\blacksquare", input_mode = "math")
  expect_equal(nrow(latex_grob("\\square", input_mode = "math")$layout_df), 1L)
  expect_equal(nrow(latex_grob("\\blacksquare", input_mode = "math")$layout_df), 1L)
  expect_equal(as.numeric(d_checked$width), as.numeric(d_unchecked$width))
  expect_equal(as.numeric(d_checked$height), as.numeric(d_unchecked$height))
})

test_that("a list marker sits on its own item's text baseline", {
  # Top-aligning left bullets floating above the text. Aligning them all to
  # one reference baseline was not enough either: a tall first line -- a
  # superscript, a fraction -- sits lower inside its own box, so exactly
  # those items kept a floating marker.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  gp <- grid::gpar(fontsize = 12)
  blocks <- .md_parse_blocks(paste(
    "- plain bullet, no math",
    "- plain bullet with $x^2$",
    "- deep $\\frac{a}{b}$ fraction",
    "- a long bullet item that has to wrap because it is very wide indeed",
    sep = "\n"))
  items <- .md_layout(blocks, 200, gp, 12)$items
  # Items alternate marker, body. `y` is the top of each box and
  # (bbox_h - bbox_bl_bp) the first baseline measured down from it.
  bl <- function(it) it$y + it$grob$bbox_h - it$grob$bbox_bl_bp
  for (k in seq(1L, length(items), by = 2L)) {
    expect_equal(bl(items[[k]]), bl(items[[k + 1L]]),
                 label = items[[k + 1L]]$grob$tex)
    # Which means the marker is pushed *down* from the top of its line.
    expect_gt(items[[k]]$y, items[[k + 1L]]$y - 1e-9)
  }
})

test_that("table column alignment is carried into the tabular spec", {
  tb <- Filter(function(b) b$type == "table",
               .md_parse_blocks("| a | b | c |\n|:--|:-:|--:|\n| 1 | 2 | 3 |"))[[1]]
  expect_match(tb$tex, "\\begin{tabular}{lcr}", fixed = TRUE)
  # Default when the source gives no alignment.
  tb2 <- Filter(function(b) b$type == "table",
                .md_parse_blocks("| a | b |\n|---|---|\n| 1 | 2 |"))[[1]]
  expect_match(tb2$tex, "\\begin{tabular}{ll}", fixed = TRUE)
})

test_that("ordered lists keep the delimiter the author used", {
  period <- Filter(function(b) b$type == "list",
                   .md_parse_blocks("1. one\n2. two"))[[1]]
  paren <- Filter(function(b) b$type == "list",
                  .md_parse_blocks("1) one\n2) two"))[[1]]
  expect_false(period$paren)
  expect_true(paren$paren)
  expect_equal(.md_list_marker(TRUE, 1, 1, FALSE, NA), "\\text{1.}")
  expect_equal(.md_list_marker(TRUE, 1, 1, TRUE, NA), "\\text{1)}")
})

test_that("a block image is drawn as a raster and scaled to the column", {
  skip_if_not_installed("png")
  f <- tempfile(fileext = ".png")
  png::writePNG(array(0.5, c(100, 400, 3)), f)   # 400x100 px
  on.exit(unlink(f), add = TRUE)
  pdf(NULL); on.exit(dev.off(), add = TRUE)

  # Only a paragraph holding nothing but images becomes an image block;
  # one in the middle of a sentence has no raster to sit in.
  types <- vapply(.md_parse_blocks(paste0("Before\n\n![alt](", f, ")\n\nAfter")),
                  function(b) b$type, character(1))
  expect_equal(types, c("paragraph", "image", "paragraph"))
  expect_equal(vapply(.md_parse_blocks("text ![alt](x.png) more"),
                      function(b) b$type, character(1)), "paragraph")
  expect_match(.md_to_tex("text ![alt](x.png) more"), "alt", fixed = TRUE)

  r <- .md_image_raster(f)
  expect_equal(c(r$w_px, r$h_px), c(400, 100))

  g <- markdown_box_grob(paste0("![x](", f, ")"), width = grid::unit(100, "bigpts"))
  content <- grid::makeContent(g)$children
  content <- content[[length(content)]]$children
  ras <- Filter(function(k) inherits(k, "rastergrob"), content)[[1]]
  w <- grid::convertWidth(ras$width, "bigpts", valueOnly = TRUE)
  h <- grid::convertHeight(ras$height, "bigpts", valueOnly = TRUE)
  expect_lte(w, 100 + 0.5)             # scaled down to the column
  expect_equal(h / w, 100 / 400, tolerance = 0.01)   # aspect preserved
})

test_that("an unusable image degrades to its alt text", {
  # Missing file, unsupported format, or reader package absent must not
  # error -- png and jpeg are Suggests.
  for (bad in list("does-not-exist.png", "file.svg", "", NA_character_)) {
    expect_null(.md_image_raster(bad))
  }
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g <- markdown_box_grob("![the alt text](missing.png)",
                         width = grid::unit(3, "in"))
  expect_s3_class(grid::makeContent(g), "markdownbox")
})

# --- inline HTML ---------------------------------------------------------
#
# GFM defines no markdown syntax for colour, underline, super/subscript,
# highlight or size, so raw HTML -- which CommonMark includes and GFM
# keeps -- is the only conformant way to express them. What each tag is
# supposed to look like is not our invention either: it is the default
# rendering HTML itself prescribes. Everything with no default rendering
# keeps the old behaviour of dropping the markup and keeping the text.

md_colours <- function(md) {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  unique(markdown_grob(md)$layout_df$color)
}
md_types <- function(md) {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  unique(markdown_grob(md)$layout_df$type)
}
# Everything observable about a rendered string, for the tag matrix below.
md_render <- function(md, fontsize = 14) {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g <- markdown_grob(md, gp = grid::gpar(fontsize = fontsize))
  df <- g$layout_df
  fs <- df$font_style[!is.na(df$font_style)]
  list(bold   = any(bitwAnd(fs, 2L) > 0L),
       italic = any(bitwAnd(fs, 4L) > 0L),
       mono   = any(bitwAnd(fs, 128L) > 0L),
       lines  = sum(df$type == "line"),
       fills  = sum(df$type == "fill_rect"),
       colour = any(df$color != "#000000", na.rm = TRUE),
       w = g$bbox_w, h = g$bbox_h, n = nrow(df))
}

# One row per supported tag, saying what HTML's default rendering makes it
# look like. Driven off a table on purpose: the coverage check below fails
# if a tag is added to .MD_HTML_TAGS without an entry here, so a new tag
# cannot arrive without someone stating what it is meant to do.
md_tag_effect <- list(
  bold    = c("b", "strong"),
  italic  = c("i", "em", "cite", "dfn", "var", "address"),
  mono    = c("code", "kbd", "samp", "tt"),
  rule    = c("u", "ins", "s", "del", "strike"),
  script  = c("sub", "sup"),
  fill    = "mark",
  smaller = "small",
  larger  = "big",
  quotes  = "q"
)

test_that("every supported tag renders what HTML prescribes for it", {
  # Mid-sentence, because CommonMark makes some of these tag names start an
  # HTML *block* when they open a line -- <address> is one -- and a block
  # is dropped whole. Inline is the case this table is about.
  wrap <- function(tag) sprintf("i <%s>Hg</%s> j", tag, tag)
  ref <- md_render("i Hg j")

  for (effect in names(md_tag_effect)) {
    for (tag in md_tag_effect[[effect]]) {
      got <- md_render(wrap(tag))
      lab <- sprintf("<%s> (%s)", tag, effect)
      switch(effect,
        bold    = expect_true(got$bold && !got$italic && !got$mono, label = lab),
        italic  = expect_true(got$italic && !got$bold && !got$mono, label = lab),
        mono    = expect_true(got$mono && !got$bold && !got$italic, label = lab),
        rule    = expect_equal(got$lines, 1L, label = lab),
        # A script is set smaller and off the baseline, so the run is
        # narrower than the same characters at full size.
        script  = expect_lt(got$w, ref$w),
        fill    = expect_true(got$fills == 1L && got$colour, label = lab),
        smaller = expect_lt(got$w, ref$w),
        larger  = expect_gt(got$w, ref$w),
        # <q> adds quotation marks, so it gains records and width.
        quotes  = expect_true(got$w > ref$w && got$n > ref$n, label = lab)
      )
      # Nothing here should draw a rule or a fill it was not asked for.
      if (!identical(effect, "rule")) expect_equal(got$lines, 0L, label = lab)
      if (!identical(effect, "fill")) expect_equal(got$fills, 0L, label = lab)
    }
  }

  # The table above must cover the implementation. <q> and <span> are
  # handled in .md_html_tag() rather than .MD_HTML_TAGS, because their
  # LaTeX depends on the mode / the style attribute.
  expect_setequal(names(.MD_HTML_TAGS), setdiff(unlist(md_tag_effect), "q"))
  # gridtext's dispatch_tag() knows exactly these and errors on the rest.
  # <img> and <p> are the two we do not do: an inline raster cannot go
  # inside a MicroTeX line, and <p> arrives as an HTML *block*, which
  # markdown's own paragraphs already cover.
  expect_true(all(c("b", "strong", "i", "em", "sub", "sup") %in%
                    names(.MD_HTML_TAGS)))

  # Tags with no default rendering are absent on purpose: <abbr>, a bare
  # <span> and friends change nothing visually in a browser either, so
  # dropping the markup and keeping the text *is* the rendering.
  # <ruby> is not in this list any more: it annotates its content with
  # \overset, so it does have a rendering.
  #
  # A bare <a> belongs here too -- the standard styles `a:link`, so an
  # anchor with no href is ordinary text. An <a href> is the opposite
  # case and is checked below.
  for (tag in c("a", "abbr", "span", "bdi", "bdo", "data", "time", "wbr",
                "output", "nobr")) {
    expect_equal(.md_to_tex(sprintf("<%s>Hg</%s>", tag, tag)), "\\text{Hg}",
                 label = tag)
  }
})

test_that("a style attribute is read for colour, size and family", {
  sz <- function(css, base = 20) {
    pdf(NULL); on.exit(dev.off(), add = TRUE)
    markdown_grob(sprintf('<span style="font-size:%s">Hg</span>', css),
                  gp = grid::gpar(fontsize = base))$layout_df$font_size[1]
  }
  fam <- function(css) {
    pdf(NULL); on.exit(dev.off(), add = TRUE)
    df <- markdown_grob(sprintf('<span style="font-family:%s">Hg</span>',
                                css))$layout_df
    df <- df[df$type == "text", ]
    unique(vapply(seq_len(nrow(df)), function(i)
      .resolve_text_family(df$font_style[i], "-", df$font_family[i]),
      character(1)))
  }

  # colour: hex, #abc, rgb(), and any R colour name, resolved to hex here
  # rather than handed to MicroTeX's smaller named-colour table.
  for (css in c("red", "#FF0000", "#f00", "rgb(255,0,0)")) {
    expect_true("#FF0000" %in%
                  md_colours(sprintf('<span style="color:%s">x</span>', css)),
                label = css)
  }
  expect_true("#4682B4" %in% md_colours('<span style="color:steelblue">x</span>'))
  # The nine CSS names R's palette lacks; without these the span silently
  # did not colour at all.
  expect_true("#DC143C" %in% md_colours('<span style="color:crimson">x</span>'))
  for (nm in names(.MD_CSS_COLORS)) expect_false(is.null(.md_resolve_color(nm)))
  # A value may carry the other kind of quote, so the attribute has to be
  # matched to its own closing quote.
  expect_true("#FF0000" %in% md_colours("<span style='color:red'>x</span>"))

  # font-size: every unit gridtext accepts, plus the relative ones, which
  # come free because they need no base size.
  expect_equal(sz("10pt"), 10)
  expect_equal(sz("20px"), 15)            # 96 px to the inch
  expect_equal(sz("0.25in"), 18)
  expect_equal(sz("1cm"), 72 / 2.54, tolerance = 0.05)
  expect_equal(sz("5mm"), 72 / 25.4 * 5, tolerance = 0.05)
  expect_equal(sz("0.5em"), 10)
  expect_equal(sz("150%"), 30)
  expect_equal(sz("larger"), 24)
  # An absolute size is absolute: it must not drift with the base.
  for (base in c(10, 20, 40)) expect_equal(sz("10pt", base), 10)
  # Anything unparseable leaves the size alone rather than guessing.
  for (bad in c("nonsense", "-5pt", "0pt", "12", "")) {
    expect_equal(sz(bad), 20, label = bad)
  }

  # font-family: the CSS generics map onto the aliases grid understands,
  # and a name travels as itself -- serif and named fonts were previously
  # inexpressible, since MicroTeX's style bits cannot tell \textrm from a
  # plain \text{}, nor one named font from another.
  expect_equal(fam("monospace"), "mono")
  expect_equal(fam("sans-serif"), "sans")
  expect_equal(fam("serif"), "serif")
  expect_equal(fam("Georgia"), "Georgia")
  # A fallback list resolves to its first entry, and case is preserved
  # because some font matchers care.
  expect_equal(fam("'Courier New', monospace"), "Courier New")
  expect_match(.md_to_tex('<span style="font-family:Georgia">x</span>'),
               "\\gmfontfamily{Georgia}{", fixed = TRUE)
  # The name is spliced into LaTeX, so it must not carry parser syntax.
  expect_equal(.md_to_tex('<span style="font-family:a}b\\c">x</span>'),
               "\\gmfontfamily{abc}{x}")
})

test_that("styles nest, compose, and survive a rule", {
  # \underline and \sout typeset their argument as a fresh sub-formula and
  # drop the bold/italic/mono around them, so those commands are re-opened
  # inside. This bit markdown's own syntax too: **~~x~~** came out unbold.
  for (md in c("**~~Hg~~**", "<b>~~Hg~~</b>", "**<u>Hg</u>**",
               "<b><u>Hg</u></b>", "<b><s>Hg</s></b>")) {
    got <- md_render(md)
    expect_true(got$bold, label = md)
    expect_equal(got$lines, 1L, label = md)
  }
  # Both orders agree, which is the real check: nesting is symmetric.
  expect_equal(md_render("<b><u>Hg</u></b>"), md_render("<u><b>Hg</b></u>"))
  expect_equal(md_render("**~~Hg~~**"), md_render("~~**Hg**~~"))
  expect_equal(md_render("<b>**Hg**</b>"), md_render("**<b>Hg</b>**"))
  # Italic and mono survive too, two rules both draw, and a superscript --
  # set from a \text{} of its own -- needs the same treatment.
  expect_true(md_render("*~~Hg~~*")$italic)
  expect_true(md_render("~~`Hg`~~")$mono)
  expect_true(all(unlist(md_render("**_~~Hg~~_**")[c("bold", "italic")])))
  expect_equal(md_render("<u><s>Hg</s></u>")$lines, 2L)
  expect_true(md_render("<b>H<sup>2</sup></b>")$bold)

  # Tags compose with each other, with a styled span, and with markdown.
  expect_true(all(unlist(md_render("<b><i>Hg</i></b>")[c("bold", "italic")])))
  expect_true(all(unlist(md_render("<b><code>Hg</code></b>")[c("bold", "mono")])))
  expect_true(md_render('<span style="color:red"><b>Hg</b></span>')$bold)
  expect_true(md_render('<b><span style="color:red">Hg</span></b>')$colour)
  expect_true(md_render("<mark><b>Hg</b></mark>")$fills == 1L)
  # Two CSS properties on one span open two commands and close both.
  one <- '<span style="color:red;text-decoration:underline">x</span>'
  expect_true("#FF0000" %in% md_colours(one))
  expect_true("line" %in% md_types(one))
  # Markdown and math inside a span are still parsed, and inherit the style.
  mixed <- '<span style="color:red">a **b** $x^2$ c</span>'
  expect_true("#FF0000" %in% md_colours(mixed))
  expect_match(.md_to_tex(mixed), "\\textbf{", fixed = TRUE)
  expect_gt(md_render("<b>a $x^2$ b</b>")$w, md_render("<b>a b</b>")$w)
})

test_that("a span's family beats gp$fontfamily, and only inside the span", {
  # Local overrides global -- and it has to hold in the measurer too, or
  # the run is measured in one font and drawn in another.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  df <- markdown_grob('a <span style="font-family:mono">b</span> c',
                      gp = grid::gpar(fontsize = 20,
                                      fontfamily = "serif"))$layout_df
  df <- df[df$type == "text", ]
  fams <- vapply(seq_len(nrow(df)), function(i)
    .resolve_text_family(df$font_style[i], "serif", df$font_family[i]),
    character(1))
  expect_equal(fams[df$text == "b"], "mono")
  expect_true(all(fams[df$text %in% c("a", "c")] == "serif"))
})

test_that("markup we do not interpret is dropped, and stays balanced", {
  # Unrecognised tags, and CSS we cannot honour, keep the text.
  for (md in c("<abbr>x</abbr>", "<span>x</span>",
               '<span style="color:notacolor">x</span>')) {
    expect_equal(.md_to_tex(md), "\\text{x}", label = md)
  }
  # Block-level HTML is dropped whole -- walking into it would typeset the
  # tags, and the markdown inside them unparsed.
  tex <- .md_to_tex("Before\n\n<div>raw **html**</div>\n\nAfter")
  expect_false(grepl("<div", tex, fixed = TRUE))
  expect_match(tex, "Before", fixed = TRUE)
  expect_match(tex, "After", fixed = TRUE)

  # A code span is its own node type and is never scanned for tags, so
  # documenting the syntax in backticks is safe.
  tex <- .md_to_tex("`<u>x</u>`")
  expect_match(tex, "\\texttt{", fixed = TRUE)
  expect_match(tex, "<u>x</u>", fixed = TRUE)
  expect_false(grepl("\\underline", tex, fixed = TRUE))

  # However malformed the input, every command opened is closed.
  balanced <- function(tex) {
    ob <- lengths(regmatches(tex, gregexpr("(?<!\\\\)\\{", tex, perl = TRUE)))
    cb <- lengths(regmatches(tex, gregexpr("(?<!\\\\)\\}", tex, perl = TRUE)))
    ob == cb
  }
  for (md in c("<u>unclosed", "</u>stray", "<b>unclosed", "</b>stray",
               '<span style="color:red">a<u>b</span>c',
               "<b>a<u>b</b>c", "<u><b>x</u></b>", "<b><i>x</b></i>",
               "<mark>a<b>b</mark>c", "<b><q>x</b></q>")) {
    expect_true(balanced(.md_to_tex(md)), label = md)
  }
  # An unclosed tag still styles what follows it.
  expect_match(.md_to_tex("<u>unclosed"), "\\underline{", fixed = TRUE)
})

test_that("inline HTML works in the block renderer too", {
  # markdown_box_grob() walks the same inline path.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g <- markdown_box_grob(
    'Text with <span style="color:red">red</span> and <u>under</u>.',
    width = grid::unit(3, "in"))
  kids <- grid::makeContent(g)$children
  inner <- kids[[length(kids)]]$children
  cols <- unlist(lapply(inner, function(k) k$layout_df$color))
  expect_true("#FF0000" %in% cols)
})

test_that("consecutive breaks leave a blank line", {
  # MicroTeX gives a row holding nothing zero height, so `<br><br>` used to
  # collapse to a single break. A strut restores the row HTML would show.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  gp <- grid::gpar(fontsize = 15)
  h <- function(md) {
    as.numeric(latex_dims(.md_to_tex(md), input_mode = "math", gp = gp)$height)
  }
  # Measure the leading from text that has both an ascender and a
  # descender: a row is as tall as its content, so "a" alone would give a
  # short row and understate what a full blank line is worth.
  one <- h("Agp<br>Agp")
  lead <- one - h("Agp")
  # Each extra break adds exactly one more line, not zero.
  expect_equal(h("Agp<br><br>Agp"), one + lead)
  expect_equal(h("Agp<br><br><br>Agp"), one + 2 * lead)
  # The usual way of writing it -- a break, a newline, a break -- counts the
  # whitespace-only row as empty and gets the same blank line.
  expect_equal(h("Agp<br>\n<br>\nAgp"), one + lead)
  expect_equal(h("Agp<br> <br>Agp"), one + lead)
  # Markdown's own hard break (backslash at end of line) goes the same way.
  expect_equal(h("Agp\\\n\\\nAgp"), one + lead)
  # A single break is untouched, and a trailing one adds no dangling row.
  expect_match(.md_to_tex("a<br>b"), "\\text{a}\\\\\\text{b}", fixed = TRUE)
  expect_equal(h("Agp<br>"), h("Agp"))
})

test_that("a registered user font can be named by a span", {
  # The route for a font that is not installed system-wide. No download:
  # a bundled OTF stands in for the user's file.
  skip_if_not_installed("ragg")
  otf <- system.file("fonts", package = "gridmicrotex")
  files <- list.files(otf, pattern = "\\.otf$", full.names = TRUE,
                      recursive = TRUE)
  skip_if(length(files) == 0L, "no bundled OTF to register")
  systemfonts::register_font(name = "gmTestUserFont", plain = files[1])

  f <- tempfile(fileext = ".png")
  ragg::agg_png(f, width = 400, height = 120)
  on.exit(unlink(f), add = TRUE)
  w_user <- as.numeric(latex_dims("\\gmfontfamily{gmTestUserFont}{Wig}",
                                  input_mode = "math")$width)
  w_plain <- as.numeric(latex_dims("\\text{Wig}", input_mode = "math")$width)
  dev.off()
  expect_gt(w_user, 0)
  expect_false(isTRUE(all.equal(w_user, w_plain)))
})

# --- regressions: block nesting and gp handling --------------------------

test_that("an absolute font-size resolves inside a list item or image alt", {
  # `base` was referenced but never passed into .md_list_block(),
  # .md_image_blocks() or .md_table_tex(), so any CSS length that needs a
  # reference size errored with "object 'base' not found". Only absolute
  # units reach it -- em/rem/% return before base is touched, which is
  # why this went unnoticed.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # Every font size anywhere in the laid-out box.
  sizes <- function(md) {
    out <- numeric()
    walk <- function(g) {
      out <<- c(out, g$layout_df$font_size)
      for (ch in g$children) walk(ch)
    }
    walk(grid::makeContent(markdown_box_grob(md, width = grid::unit(3, "in"))))
    sort(unique(stats::na.omit(out)))
  }
  for (md in c("- <span style=\"font-size:12pt\">big</span> item",
               "![<span style=\"font-size:12pt\">alt</span>](missing.png)",
               "| <span style=\"font-size:12pt\">a</span> | b |\n|---|---|\n| 1 | 2 |")) {
    # Not just "it laid out": the 12pt has to arrive, or the length was
    # resolved against the wrong reference and nobody would notice.
    expect_true(12 %in% sizes(md), info = md)
  }
  # And it is a real override, not the default in disguise.
  expect_false(12 %in% sizes("- big item"))
})

test_that("a table cell sizes against the caller's font size", {
  # .md_table_tex() measured cells against the default 20 whatever gp said.
  blk <- .md_parse_blocks(
    "| <span style=\"font-size:40pt\">big</span> | b |\n|---|---|\n| 1 | 2 |",
    base = 40)[[1]]
  expect_match(blk$tex, "\\scalebox{1.0", fixed = TRUE)
})

test_that("a rule, image or quote nested in a container lays out", {
  # .md_layout() shifted nested children through it$grob$vp$y, but rules,
  # images and block-quote bars are rect/raster grobs with no viewport,
  # so "object is not coercible to a unit" came out of ordinary markdown.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  kinds <- function(md) {
    out <- character()
    walk <- function(g) {
      out <<- c(out, class(g)[1])
      for (ch in g$children) walk(ch)
    }
    walk(grid::makeContent(markdown_box_grob(md, width = grid::unit(3, "in"))))
    out
  }
  for (md in c("> a\n>\n> ---\n>\n> b",
               "- a\n\n  ---\n\n- b",
               "- item\n\n  > quoted",
               "> outer\n>\n> > inner")) {
    k <- kinds(md)
    # Each case pairs prose with a rect -- the rule, or the quote bar.
    # Laying out to nothing would also not error.
    expect_true("latexgrob" %in% k, info = md)
    expect_true("rect" %in% k, info = md)
  }
})

test_that("a nested rule is offset by its container, not left at the top", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  blocks <- .md_parse_blocks("intro\n\n> quoted\n>\n> ---\n>\n> more")
  lay <- .md_layout(blocks, 300, grid::gpar(fontsize = 12), 12)
  rules <- Filter(function(it) inherits(it$grob, "rect"), lay$items)
  # Two rects: the quote's own bar, and the rule inside it. The inner one
  # must sit below the quote's first line, not at the container's origin.
  expect_length(rules, 2L)
  expect_gt(max(vapply(rules, function(it) it$y, numeric(1))), 0)
})

test_that("markdown_box_grob() applies gp$cex exactly once", {
  # The gTree carried the raw gp, so grid multiplied cex into children
  # that had already scaled themselves by it: 10pt at cex 2 drew at 40.
  skip_if_not_installed("svglite")
  sizes <- function(gp) {
    f <- tempfile(fileext = ".svg")
    svglite::svglite(f, width = 6, height = 4)
    grid::grid.draw(markdown_box_grob("hello world",
                                      width = grid::unit(3, "in"), gp = gp))
    dev.off()
    on.exit(unlink(f), add = TRUE)
    s <- readLines(f, warn = FALSE)
    unique(unlist(regmatches(s, gregexpr("font-size: *[0-9.]+", s))))
  }
  expect_equal(sizes(grid::gpar(fontsize = 10, cex = 2)),
               sizes(grid::gpar(fontsize = 20)))
})

# The x of a block item, whichever way the grob carries it. A rasterGrob
# is positioned by its own `x`; the wrapper around a vector picture is
# positioned by its viewport, because grImport2's pictureGrob brings a
# vpStack whose `x` is NULL. Reading only `$x` would pass on PNG and error
# on SVG, which is exactly the gap this helper exists to close.
block_xs <- function(g) {
  kids <- grid::makeContent(g)$children[[1]]$children
  vapply(kids, function(k) {
    u <- if (!is.null(k$x)) k$x else if (!is.null(k$vp)) k$vp$x else NULL
    if (is.null(u)) NA_real_ else grid::convertX(u, "bigpts", valueOnly = TRUE)
  }, numeric(1))
}

test_that("halign moves an image that has room, and leaves a full-width rule", {
  skip_if_not_installed("png")
  f <- tempfile(fileext = ".png")
  png::writePNG(array(0.5, c(20, 40, 3)), f)   # 40x20 px, narrow
  on.exit(unlink(f), add = TRUE)
  pdf(NULL); on.exit(dev.off(), add = TRUE)

  xs <- function(ha) {
    block_xs(markdown_box_grob(paste0("---\n\n![a](", f, ")"),
                               width = grid::unit(4, "in"), halign = ha,
                               gp = grid::gpar(fontsize = 12)))
  }
  left <- xs(0)
  right <- xs(1)
  expect_equal(left[[1]], right[[1]])   # the rule spans the column
  expect_gt(right[[2]], left[[2]])      # the image has slack and uses it
})

test_that("a block image aligns the same whether it is raster or vector", {
  skip_if_not_installed("png")
  skip_if_not_installed("svglite")
  skip_if_not_installed("rsvg")
  skip_if_not_installed("grImport2")
  pdf(NULL); on.exit(dev.off(), add = TRUE)

  p <- tempfile(fileext = ".png")
  png::writePNG(array(0.5, c(20, 40, 3)), p)
  s <- tempfile(fileext = ".svg")
  svglite::svglite(s, width = 40 / 72, height = 20 / 72)
  grid::grid.rect(gp = grid::gpar(fill = "grey50", col = NA))
  grDevices::dev.off()
  on.exit(unlink(c(p, s)), add = TRUE)

  at <- function(f, ha) {
    block_xs(markdown_box_grob(sprintf("![a](%s)", f),
                               width = grid::unit(4, "in"), halign = ha,
                               gp = grid::gpar(fontsize = 12)))[[1]]
  }
  # A vector picture is positioned through its viewport rather than its
  # own x, so this is the guard that both routes still move together.
  for (f in c(p, s)) {
    expect_equal(at(f, 0), 0, info = f)
    expect_gt(at(f, 1), at(f, 0.5))
    expect_gt(at(f, 0.5), at(f, 0))
  }

  # And the cascade reaches an image block as well as the argument does.
  centred <- block_xs(markdown_box_grob(
    sprintf("![a](%s)", p), width = grid::unit(4, "in"),
    style = markdown_style(img = md_style(text_align = "center")),
    gp = grid::gpar(fontsize = 12)))[[1]]
  expect_gt(centred, 0)
})

test_that("private-use characters in the input cannot forge a math span", {
  # The mask sentinels are private-use codepoints, which icon fonts do
  # use. A pasted one used to be spliced with a real math span.
  inj <- paste0("icon ", intToUtf8(0xE000), "1", intToUtf8(0xE001),
                " and $y$")
  out <- .md_to_tex(inj)
  expect_match(out, "icon 1 and ", fixed = TRUE)
  expect_equal(lengths(regmatches(out, gregexpr("y", out))), 1L)
})

test_that("markdown_grob() rejects input_mode instead of failing obscurely", {
  expect_error(markdown_grob("x", input_mode = "mixed"), "input_mode")
})

test_that("<ruby> annotates its base with \\overset", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)

  # The one construct whose pieces arrive as siblings but come out in the
  # other order, because \overset takes the annotation first.
  expect_equal(.md_to_tex("<ruby>base<rt>gloss</rt></ruby>"),
               "\\overset{\\scalebox{0.5}{\\text{gloss}}}{\\text{base}}")
  # It composes with markdown inside the base.
  expect_match(.md_to_tex("<ruby>**b**<rt>g</rt></ruby>"),
               "{\\textbf{b}}", fixed = TRUE)
  # Surrounding text is unaffected.
  expect_match(.md_to_tex("x <ruby>a<rt>b</rt></ruby> y"), "text{x }",
               fixed = TRUE)

  # Degenerate forms keep the text rather than losing it.
  expect_equal(.md_to_tex("<ruby>no rt</ruby>"), "\\text{no rt}")
  expect_match(.md_to_tex("<ruby>unclosed<rt>g"), "overset", fixed = TRUE)
  expect_equal(.md_to_tex("<rt>orphan</rt>"), "\\text{orphan}")

  # An annotation adds height, not width -- it sits above the base.
  w <- function(t) as.numeric(latex_dims(t, input_mode = "math",
                                         gp = grid::gpar(fontsize = 16))$width)
  h <- function(t) as.numeric(latex_dims(t, input_mode = "math",
                                         gp = grid::gpar(fontsize = 16))$height)
  plain <- .md_to_tex("base")
  ruby <- .md_to_tex("<ruby>base<rt>gloss</rt></ruby>")
  expect_equal(w(ruby), w(plain))
  expect_gt(h(ruby), h(plain))
})

test_that("&nbsp; becomes a non-breaking space between text runs", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # \nbsp only works *between* \text{} runs: inside one it typesets the
  # letters. Measured, so the split is not taken on faith.
  w <- function(t) as.numeric(latex_dims(t, input_mode = "math",
                                         gp = grid::gpar(fontsize = 16))$width)
  expect_gt(w("\\text{a\\nbsp b}"), w("\\text{a}\\nbsp\\text{b}"))

  expect_equal(.md_to_tex("a&nbsp;b"), "\\text{a}\\nbsp \\text{b}")
  # It measures about the same as an ordinary space, but cannot be broken
  # at. Only about: an ordinary space is now folded into the phrase and
  # measured by the device, while \nbsp sits between two \text{} runs and
  # keeps MicroTeX's own space metric. The two differ by a fraction of a
  # point -- visually identical, no longer equal to the last digit.
  expect_equal(w(.md_to_tex("a&nbsp;b")), w(.md_to_tex("a b")),
               tolerance = 0.1)
  # Text with no nbsp takes the untouched path.
  expect_equal(.md_to_tex("plain text"), "\\text{plain text}")
})

test_that("font-size takes CSS absolute keywords", {
  # Read off the \tiny..\Huge ladder MicroTeX implements.
  expect_equal(.md_css_size("medium", 20), 1)
  expect_lt(.md_css_size("xx-small", 20), .md_css_size("x-small", 20))
  expect_lt(.md_css_size("x-small", 20), .md_css_size("small", 20))
  expect_lt(.md_css_size("small", 20), .md_css_size("medium", 20))
  expect_lt(.md_css_size("medium", 20), .md_css_size("large", 20))
  expect_lt(.md_css_size("large", 20), .md_css_size("x-large", 20))
  expect_lt(.md_css_size("x-large", 20), .md_css_size("xx-large", 20))
  # Unknown keywords stay invalid.
  expect_null(.md_css_size("enormous", 20))
})

test_that("width = NULL sizes the box to its content", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g <- markdown_box_grob(
    "a **long** paragraph that would certainly wrap in a narrow box",
    width = NULL)
  w <- vapply(c(2, 6, 9), function(wd) {
    grid::pushViewport(grid::viewport(width = grid::unit(wd, "in")))
    on.exit(grid::popViewport(), add = TRUE)
    grid::convertWidth(grid::widthDetails(g), "bigpts", valueOnly = TRUE)
  }, numeric(1))
  # The whole point: nothing about the size depends on the parent, which
  # is what makes it safe to measure before ggplot2 has placed the grob.
  expect_equal(w, rep(w[1], 3))
  expect_gt(w[1], 100)

  h <- vapply(c(2, 9), function(wd) {
    grid::pushViewport(grid::viewport(width = grid::unit(wd, "in")))
    on.exit(grid::popViewport(), add = TRUE)
    grid::convertHeight(grid::heightDetails(g), "bigpts", valueOnly = TRUE)
  }, numeric(1))
  expect_equal(h[1], h[2])
})

test_that("the natural width does not re-break the line it was measured from", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # latex_dims() reports whole big points, so handing the rounded width
  # back as the measure can be up to half a point short -- enough for
  # MicroTeX to break the very line the number came from. The symptom is
  # a heading that wraps in a box that was sized not to wrap.
  head_only <- "## Fuel economy falls with weight"
  one <- markdown_box_grob(head_only, width = NULL)
  h1 <- grid::convertHeight(grid::heightDetails(one), "bigpts",
                            valueOnly = TRUE)
  # One line of an h2, not two.
  expect_lt(h1, 40)

  with_list <- markdown_box_grob(
    paste(head_only, "", "- a bullet", sep = "\n"), width = NULL)
  h2 <- grid::convertHeight(grid::heightDetails(with_list), "bigpts",
                            valueOnly = TRUE)
  expect_lt(h2 - h1, 40)
})

test_that("a rule does not inflate the natural width", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # An <hr>, and a block background, span whatever width they are handed,
  # so a naive max() over the probe layout hands the probe width straight
  # back and the box comes out metres wide.
  g <- markdown_box_grob("short\n\n---\n\nalso short", width = NULL)
  w <- grid::convertWidth(grid::widthDetails(g), "bigpts", valueOnly = TRUE)
  expect_lt(w, .MD_PROBE_W / 10)
})

test_that("a natural-width box still draws, and honours hjust", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # With no width there is nothing to build a viewport from at
  # construction, so makeContent() has to supply one of the measured size.
  g <- markdown_box_grob("# H\n\n- a\n- b", width = NULL, hjust = 0)
  expect_null(g$vp)
  # hjust rides on the supplied viewport's justification, so read it
  # there -- the viewport's x stays at 0.5npc whatever hjust is.
  for (hj in c(0, 0.5, 1)) {
    ct <- grid::makeContent(
      markdown_box_grob("# H\n\n- a\n- b", width = NULL, hjust = hj))
    expect_s3_class(ct, "markdownbox")
    expect_equal(ct$children[[1]]$vp$justification[1], hj)
  }
})

test_that("an inline <a> is styled by the same rule as [text](url)", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # Both spellings mean "a link", so both resolve the `a` selector rather
  # than one of them carrying a hard-coded colour. The inline tag used to
  # fall through to plain body text.
  col <- function(md, ...) {
    d <- markdown_grob(md, gp = grid::gpar(fontsize = 14), ...)$layout_df
    d <- d[d$type == "text" & !is.na(d$text) & d$text == "LINK", ]
    d$color[1]
  }
  expect_equal(col("a <a href='http://x'>LINK</a> c"),
               col("a [LINK](http://x) c"))
  expect_equal(col("a [LINK](http://x) c"), .MD_LINK_COLOR)

  # Only `a:link` is styled: an anchor with no href is ordinary text, as
  # it is in a browser.
  expect_equal(.md_to_tex("<a>LINK</a>"), "\\text{LINK}")
  expect_false(col("a <a name='x'>LINK</a> c") == .MD_LINK_COLOR)

  # Restyling the rule reaches both, which is the point of routing the
  # tag through the cascade instead of giving it its own colour.
  green <- markdown_style(a = md_style(color = "green"))
  expect_equal(col("a <a href='http://x'>LINK</a> c", style = green),
               col("a [LINK](http://x) c", style = green))
  expect_false(col("a <a href='http://x'>LINK</a> c", style = green) ==
                 .MD_LINK_COLOR)
})

test_that("an <img> keeps its alt text, as markdown's own image does", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # Neither form can draw a raster into a text run, so both fall back to
  # the alt text. `<img>` used to contribute nothing at all, losing the
  # one part of itself that could be rendered.
  txt <- function(md) {
    d <- markdown_grob(md, gp = grid::gpar(fontsize = 14))$layout_df
    d$text[d$type == "text" & !is.na(d$text)]
  }
  expect_equal(txt("a <img src='x.png' alt='ALT'> c"),
               txt("a ![ALT](x.png) c"))
  expect_true("ALT" %in% txt("a <img src='x.png' alt='ALT'> c"))

  # Inside a text command the alt text must be emitted bare, or the
  # emphasis around it is thrown away.
  expect_true("ALT" %in% txt("a <b><img src='x.png' alt='ALT'></b> c"))

  # No alt is the HTML spelling of "decorative": nothing to say.
  expect_false("ALT" %in% txt("a <img src='x.png'> c"))

  # The alt text is prose, so its LaTeX specials must be escaped rather
  # than reaching the parser: a bare `_` would open a subscript and `%`
  # would comment out the rest of the line.
  #
  # Written inline on purpose. An <img> alone on its line is an *html
  # block* to cmark, not an inline node, and raw HTML blocks are dropped
  # -- a separate rule, older than this one.
  expect_equal(.md_to_tex("a <img src='x.png' alt='a_b'> c"),
               .md_to_tex("a ![a_b](x.png) c"))
  expect_match(.md_to_tex("a <img src='x.png' alt='a_b'> c"),
               "a\\\\_b", fixed = FALSE)
  expect_match(.md_to_tex("a <img src='x.png' alt='100% sure'> c"),
               "100\\\\%", fixed = FALSE)
})

skip_if_not_installed("ggplot2")

test_that("the ggplot2 objects are created at load", {
  expect_s3_class(GeomMarkdown, "Geom")
  expect_false(is.null(.element_markdown_class))
  expect_true(!is.null(getS3method(
    "element_grob", "gridmicrotex::element_markdown",
    optional = TRUE, envir = asNamespace("ggplot2")
  )))
})

test_that("element_markdown keeps the legacy element classes", {
  # S7 inheritance drops ggplot2's legacy "element_text"/"element" S3
  # strings, and without them combine_elements() treats the object as an
  # unrelated sibling and silently discards it when resolving inherited
  # theme entries.
  e <- element_markdown()
  expect_true(inherits(e, "element_text"))
  expect_true(inherits(e, "element"))
  expect_true(inherits(e, "gridmicrotex::element_markdown"))
})

test_that("element_markdown survives theme inheritance", {
  th <- ggplot2::theme_gray() +
    ggplot2::theme(axis.title = element_markdown())
  merged <- ggplot2::calc_element("axis.title.y", th)
  expect_true(inherits(merged, "gridmicrotex::element_markdown"))
})

test_that("emphasis actually reaches the rendered output", {
  # \text{} inside a text command resets the style: \textbf{bold} reports
  # font style 2, \textbf{\text{bold}} reports 1. Wrapping there would
  # silently drop every bold and italic in the document.
  expect_match(.md_to_tex("**b**"), "\\textbf{b}", fixed = TRUE)
  expect_match(.md_to_tex("*i*"), "\\textit{i}", fixed = TRUE)
  expect_false(grepl("\\textbf{\\text{", .md_to_tex("**b**"), fixed = TRUE))

  g <- markdown_grob("**bold** and *ital* and plain")
  faces <- vapply(g$layout_df$font_style, .resolve_text_face, character(1))
  expect_true("bold" %in% faces)
  expect_true("italic" %in% faces)
  expect_true("plain" %in% faces)
})

test_that("prose outside a text command is still \\text{}-wrapped", {
  # Top level is math mode, so unwrapped prose would come out as spaced
  # math italics.
  expect_match(.md_to_tex("plain words"), "\\text{plain words}", fixed = TRUE)
})

test_that("math and escapes survive inside emphasis", {
  tex <- .md_to_tex("**bold with $x^2$ inside**")
  expect_match(tex, "$x^2$", fixed = TRUE)
  g <- markdown_grob("**bold with $x^2$ inside**")
  expect_true("glyph" %in% g$layout_df$type)   # the math
  expect_true("text" %in% g$layout_df$type)    # the prose
})

test_that("geom_markdown contributes one rendered grob per row", {
  # "the plot builds" is ggplot2's business; ours is that the layer hands
  # back a grob per row with something in it.
  df <- data.frame(x = 1:3, y = 1:3,
                   lab = c("**b**", "*i* $\\beta_1$", "`c` $x^2$"))
  gs <- ggplot2::layer_grob(
    ggplot2::ggplot(df, ggplot2::aes(x, y, label = lab)) +
      geom_markdown(fontsize = 12))[[1]]
  expect_length(gs, 3L)
  expect_true(all(vapply(gs, inherits, logical(1), "latexgrob")))
  expect_true(all(vapply(gs, function(g) nrow(g$layout_df) > 0, logical(1))))
})

test_that("element_markdown installs our grob as the axis titles", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  df <- data.frame(x = 1:3, y = 1:3)
  p <- ggplot2::ggplot(df, ggplot2::aes(x, y)) + ggplot2::geom_point() +
    ggplot2::labs(x = "**w** in $10^3$", y = "*eta* $\\eta$") +
    ggplot2::theme(
      axis.title.x = element_markdown(),
      axis.title.y = element_markdown()
    )
  gt <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(p))
  named <- function(nm) gt$grobs[[which(gt$layout$name == nm)]]

  # Both titles, so the y case -- which is rotated and takes a different
  # branch -- is covered too.
  expect_s3_class(named("xlab-b"), "latexgrob")
  expect_s3_class(named("ylab-l"), "latexgrob")
  # Rendered as markdown, not as the literal asterisks.
  expect_false(any(grepl("*", named("xlab-b")$layout_df$text, fixed = TRUE)))
})

test_that("a bundle of tick labels reports a non-zero size", {
  # ggplot2 sizes the axis strip from this grob. When it measured 0 x 0
  # no room was reserved and the labels were drawn over the axis title.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  for (el in list(element_markdown(), element_latex())) {
    g <- ggplot2::element_grob(el, label = c("1.0", "1.5", "2.0"),
                               x = grid::unit(c(0.2, 0.5, 0.8), "npc"))
    expect_s3_class(g, "gridmicrotex_labels")
    expect_gt(grid::convertHeight(grid::grobHeight(g), "pt", valueOnly = TRUE), 0)
    expect_gt(grid::convertWidth(grid::grobWidth(g), "pt", valueOnly = TRUE), 0)
  }
})

test_that("an empty label gives a null grob", {
  expect_s3_class(ggplot2::element_grob(element_markdown(), label = ""),
                  "null")
  expect_s3_class(ggplot2::element_grob(element_markdown(), label = NA),
                  "null")
})

test_that("element_markdown validates justify", {
  expect_error(element_markdown(justify = "yes"), "TRUE or FALSE")
  expect_error(geom_markdown(justify = NA), "TRUE or FALSE")
})

test_that("dollar delimiters are never stripped", {
  # element_latex() strips enclosing $...$ in math mode, where users add
  # them by analogy with plotmath. In markdown a `$` pair is the math
  # delimiter, so stripping would destroy the label.
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g <- ggplot2::element_grob(element_markdown(), label = "$x^2$")
  expect_s3_class(g, "latexgrob")
  # Rendered as math (glyphs), not as the literal characters.
  expect_true("glyph" %in% g$layout_df$type)
})

test_that("style= reaches both ggplot2 entry points", {
  skip_if_not_installed("ggplot2")
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  cols <- function(g) unique(stats::na.omit(g$layout_df$color))

  # element_markdown stores it and element_grob honours it.
  e <- element_markdown(style = "body { color: #B22222 }")
  expect_equal(cols(ggplot2::element_grob(e, label = "**hi**")), "#B22222")
  expect_equal(cols(ggplot2::element_grob(element_markdown(), label = "hi")),
               "#000000")

  # The multi-label path (axis tick labels) carries it too.
  g <- ggplot2::element_grob(e, label = c("**a**", "*b*"))
  ticks <- unlist(lapply(g$children, function(k) cols(k)))
  expect_equal(unique(ticks), "#B22222")

  # geom_markdown takes it as a layer parameter.
  expect_true("style" %in% names(formals(geom_markdown)))

  # The element must still merge into a theme. A NULL default would fail
  # here with "Can't find property <ggplot2::element_text>@style", because
  # ggplot2 reads an unset property off the parent element -- which is why
  # the property defaults to NA and is mapped back to NULL on use.
  p <- ggplot2::ggplot(data.frame(x = 1, y = 1, lab = "**a**"),
                       ggplot2::aes(x, y, label = lab)) +
    ggplot2::geom_point() +
    geom_markdown(style = "body { color: #1F6FB2 }") +
    ggplot2::labs(x = "**wt**") +
    ggplot2::theme(axis.title.x = element_markdown(style = "body { color: red }"))
  expect_no_error(ggplot2::ggplot_build(p))
})

test_that("body seeds inheritance for inline markdown too", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # markdown_box_grob() has always seeded from `body`; markdown_grob() did
  # not, so a stylesheet written against the box grob silently did nothing
  # when handed to the inline one.
  cols <- function(g) unique(stats::na.omit(g$layout_df$color))
  expect_equal(cols(markdown_grob("hi", style = "body { color: #B22222 }")),
               "#B22222")
  # An explicit tag rule still wins over the body seed.
  expect_equal(
    cols(markdown_grob("hi",
                       style = "body { color: #B22222 } p { color: #1F6FB2 }")),
    "#1F6FB2")
})

test_that(".md_needs_blocks agrees with what .md_blocks() would build", {
  expect_false(.md_needs_blocks("plain **text** with $x$ math"))
  expect_false(.md_needs_blocks(""))
  # Inline $$..$$ in the middle of a sentence stays a paragraph...
  expect_false(.md_needs_blocks("a $$x$$ inline"))
  # ...but a paragraph that is nothing else becomes a centred math block.
  expect_true(.md_needs_blocks("$$x^2$$"))
  expect_true(.md_needs_blocks("# H"))
  expect_true(.md_needs_blocks("one\n\ntwo"))
  expect_true(.md_needs_blocks("- a\n- b"))
})

test_that("a block-structured label is laid out as blocks, a paragraph is not", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  eg <- function(lab, ...) {
    ggplot2::element_grob(element_markdown(...), label = lab)
  }
  # The run path flattens blocks: list markers and indents are simply
  # lost, which is what promotion exists to fix.
  expect_s3_class(eg("The **fitted** slope is $\\beta_1$"), "latexgrob")
  expect_s3_class(eg("## H\n\n- one\n- two"), "markdownbox")
  # A width asks for the box even with only a paragraph: a run cannot
  # wrap to a unit.
  expect_s3_class(eg("just a paragraph", width = grid::unit(2, "in")),
                  "markdownbox")
})

test_that("a rotated block label keeps its angle and warns", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # The box cannot rotate, so one of the two has to give. Losing the
  # angle would lay a y-axis title horizontally across the panel; losing
  # the block layout only flattens it, which is today's behaviour anyway.
  expect_warning(
    g <- ggplot2::element_grob(element_markdown(angle = 90),
                               label = "# H\n\n- a"),
    "rotated"
  )
  expect_s3_class(g, "latexgrob")

  # A width alongside an angle is not ignored either -- it becomes the
  # run's own wrapping measure.
  expect_no_warning(
    g2 <- ggplot2::element_grob(
      element_markdown(angle = 90, width = grid::unit(100, "bigpts")),
      label = "a long paragraph that will want to wrap somewhere along it")
  )
  expect_s3_class(g2, "latexgrob")
})

test_that("axis tick labels are never laid out as blocks", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  # ggplot2 asks an element for its height before placing it, so a box
  # sized in npc measures against the whole device -- every tick would
  # claim the full width.
  g <- ggplot2::element_grob(element_markdown(), label = c("# H", "# H"))
  expect_s3_class(g, "gridmicrotex_labels")
  # The heading would promote to a box on its own; in a tick bundle each
  # label has to stay a run.
  expect_true(all(vapply(g$children, inherits, logical(1), "latexgrob")))
  expect_false(any(vapply(g$children, inherits, logical(1), "markdownbox")))
})

test_that("a promoted label's height does not depend on the parent width", {
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  g <- ggplot2::element_grob(element_markdown(), label = "# H\n\n- one\n- two")
  h <- vapply(c(2, 6, 9), function(w) {
    grid::pushViewport(grid::viewport(width = grid::unit(w, "in")))
    on.exit(grid::popViewport(), add = TRUE)
    grid::convertHeight(grid::heightDetails(g), "bigpts", valueOnly = TRUE)
  }, numeric(1))
  expect_equal(h, rep(h[1], 3))
})

test_that("a block plot title is promoted to a box, and its style applies", {
  skip_on_cran()
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  mk <- function(...) {
    ggplot2::ggplot(mtcars, ggplot2::aes(wt, mpg)) +
      ggplot2::geom_point() +
      ggplot2::labs(title = "## Findings\n\n- slope $\\beta_1$\n- *p* < 0.001") +
      ggplot2::theme(plot.title = element_markdown(...))
  }
  title_grob <- function(p) {
    gt <- ggplot2::ggplot_gtable(ggplot2::ggplot_build(p))
    gt$grobs[[which(gt$layout$name == "title")]]
  }
  height <- function(g) {
    grid::convertHeight(grid::heightDetails(g), "bigpts", valueOnly = TRUE)
  }

  plain <- title_grob(mk())
  expect_s3_class(plain, "markdownbox")

  styled <- title_grob(mk(width = grid::unit(1, "npc"),
                          style = "body { background: #EEF3FB; padding: 8px }"))
  expect_s3_class(styled, "markdownbox")
  # 8px of padding above and below has to reach the box, or the style is
  # being carried but never resolved.
  expect_gt(height(styled), height(plain))
})

# ggplot2 integration for markdown: geom_markdown() and element_markdown().
#
# These mirror geom_latex() / element_latex() in ggplot2-integration.R and
# reuse the same machinery -- the S7 subclass-of-element_text pattern, the
# legacy class re-injection, .rotate_just() and .pick_unit(). Only the grob
# constructor differs: markdown_grob() instead of latex_grob().
#
# Two differences from the LaTeX versions are deliberate:
#
#   input_mode is absent. Markdown decides for itself what is prose and
#   what is math; the walker emits \text{}-wrapped LaTeX either way.
#
#   Enclosing $...$ is never stripped. element_latex() strips it in math
#   mode, where users add it by analogy with plotmath. In markdown a `$`
#   pair *is* the math delimiter, so removing it would destroy the label.

# --------------------------------------------------------------------------
# geom_markdown()
# --------------------------------------------------------------------------

#' A ggplot2 geom for markdown labels
#'
#' Renders markdown labels -- \code{**bold**}, \code{*italic*},
#' \code{`code`}, \code{~~strike~~} and \code{$math$} -- as native grid
#' grobs inside a plot. The markdown is converted to LaTeX and laid out by
#' MicroTeX, so the output is resolution-independent vector graphics.
#'
#' @section Aesthetics:
#' \code{geom_markdown()} understands the following aesthetics (required
#' aesthetics are in bold):
#' \itemize{
#'   \item \strong{\code{x}}
#'   \item \strong{\code{y}}
#'   \item \strong{\code{label}} --- markdown string
#'   \item \code{size} --- font size in points (default: 11)
#'   \item \code{colour} --- text colour (default: \code{"black"})
#'   \item \code{angle} --- rotation angle in degrees (default: 0)
#'   \item \code{hjust} --- horizontal justification, 0-1 (default: 0.5)
#'   \item \code{vjust} --- vertical justification, 0-1 (default: 0.5)
#'   \item \code{alpha} --- transparency (default: 1)
#' }
#'
#' @inheritParams ggplot2::layer
#' @inheritParams markdown_grob
#' @param fontsize Default font size in points. Overridden by the
#'   \code{size} aesthetic if mapped.
#' @param math_font Name of the math font to use (e.g. \code{"stix"}).
#' @param lineheight Multi-line height multiplier (default 1.2), matching
#'   \code{grid::gpar()} semantics.
#' @param max_width Maximum width in big points for automatic line
#'   wrapping (default: 0, no wrapping).
#' @param render_mode \code{"typeface"} (default) or \code{"path"}; see
#'   \code{\link{latex_grob}}.
#' @param justify Logical; justify wrapped lines. Requires
#'   \code{max_width}. See \code{\link{latex_grob}}.
#' @param style A \code{\link{markdown_style}} object, CSS text, or the
#'   path to a \code{.css} file, applied to every label this layer draws.
#'   \code{NULL} falls back to
#'   \code{\link{latex_options}(markdown_style = )}. Only the properties
#'   that compile to LaTeX apply here --- a label has no block layout, so
#'   margins and padding are ignored. See \code{\link{md_style}}.
#' @param na.rm If \code{FALSE}, the default, missing values are removed
#'   with a warning. If \code{TRUE}, they are removed silently.
#' @param ... Other arguments passed to \code{\link[ggplot2]{layer}}.
#'
#' @return A ggplot2 layer.
#' @seealso \code{\link{markdown_grob}}, \code{\link{geom_latex}}
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'   df <- data.frame(
#'     x = 1:3, y = 1:3,
#'     lab = c("**bold**", "*slope* $\\beta_1$", "`code` and $x^2$")
#'   )
#'   ggplot(df, aes(x, y, label = lab)) + geom_markdown()
#' }
#' }
geom_markdown <- function(mapping = NULL, data = NULL, stat = "identity",
                          position = "identity", ...,
                          fontsize = 11, math_font = "",
                          lineheight = 1.2, max_width = 0,
                          render_mode = c("typeface", "path"),
                          justify = FALSE, style = NULL,
                          na.rm = FALSE, show.legend = NA,
                          inherit.aes = TRUE) {
  .apply_opts("math_font", "render_mode", "justify")
  render_mode <- match.arg(render_mode)
  .check_justify(justify)
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for geom_markdown(). ",
         "Please install it with install.packages('ggplot2').",
         call. = FALSE)
  }

  ggplot2::layer(
    geom = GeomMarkdown,
    mapping = mapping,
    data = data,
    stat = stat,
    position = position,
    show.legend = show.legend,
    inherit.aes = inherit.aes,
    params = list(
      fontsize = fontsize,
      math_font = math_font,
      lineheight = lineheight,
      max_width = max_width,
      render_mode = render_mode,
      justify = justify,
      style = style,
      na.rm = na.rm,
      ...
    )
  )
}

#' @rdname geom_markdown
#' @format NULL
#' @usage NULL
#' @export
GeomMarkdown <- NULL

# --------------------------------------------------------------------------
# element_markdown()
# --------------------------------------------------------------------------

#' A ggplot2 theme element for markdown text
#'
#' Use as a theme element for axis titles, axis labels, plot titles or any
#' other text element. The label is parsed as markdown --- including
#' \code{$...$} math --- and rendered via MicroTeX.
#'
#' @details
#' This is an S7 subclass of \code{ggplot2::element_text}, so it inherits
#' the standard text properties (size, colour, hjust, ...) from the theme
#' and merges correctly with inherited theme entries.
#'
#' @section Block labels:
#' A label with real block structure --- a heading, a list, a table, a
#' rule, or more than one paragraph --- is laid out by
#' \code{\link{markdown_box_grob}} rather than flattened into one run, so
#' list markers, indents and block spacing survive. That makes a title
#' like this work:
#'
#' \preformatted{labs(title = "## Findings\\n\\n- slope $\\\\beta_1$\\n- *p* < 0.001")}
#'
#' The box's own background, border, padding and corner radius come from
#' the stylesheet's \code{body} rule --- \code{style = "body \{ background:
#' grey95; padding: 8px \}"} --- not from arguments here. See
#' \code{\link{markdown_style}}.
#'
#' Three details follow from how ggplot2 measures theme elements:
#'
#' \itemize{
#'   \item \strong{Wrapping is opt-in.} Without \code{width} the label is
#'     sized to its content, because ggplot2 asks an element for its
#'     height before placing it, when a relative width would resolve
#'     against the whole device rather than the element's cell.
#'   \item \strong{Axis tick labels are never laid out as blocks},
#'     whatever they contain, for the same reason.
#'   \item \strong{A rotated label is never laid out as blocks.} The box
#'     cannot rotate, so a label with both blocks and a non-zero
#'     \code{angle} keeps the angle, is rendered as a single run, and
#'     warns. A \code{width} given with an angle becomes the run's
#'     wrapping measure instead.
#' }
#'
#' \code{math_font}, \code{render_mode} and \code{justify} are not
#' forwarded to the box; a block label takes those from
#' \code{\link{latex_options}}.
#'
#' Note that \pkg{ggtext} also exports a function called
#' \code{element_markdown()}. If both packages are attached, the one loaded
#' later wins; call \code{gridmicrotex::element_markdown()} explicitly to
#' be unambiguous. The two are not interchangeable --- ggtext renders
#' HTML/CSS and images, this renders LaTeX math.
#'
#' @param math_font Name of the math font to use (e.g. \code{"stix"}).
#' @param fontsize Convenience alias for \code{size}; forwarded to
#'   \code{ggplot2::element_text()} as the text size in points.
#'   \code{NULL} (default) uses the theme's inherited size.
#' @param lineheight Multi-line height multiplier (default 1.2).
#' @param max_width Maximum width in big points for automatic line
#'   wrapping (default: 0, no wrapping).
#' @param render_mode \code{"typeface"} (default) or \code{"path"}.
#' @param justify Logical; justify wrapped lines. Requires
#'   \code{max_width}.
#' @param style A \code{\link{markdown_style}} object, CSS text, or the
#'   path to a \code{.css} file, applied to labels drawn through this
#'   theme element. \code{NA}, the default, means unset --- the global
#'   \code{\link{latex_options}(markdown_style = )} applies instead. Only
#'   the properties that compile to LaTeX have an effect, unless the label
#'   is laid out as blocks (see \emph{Block labels}). See
#'   \code{\link{md_style}}.
#' @param width Wrapping measure for the label, as a
#'   \code{\link[grid]{unit}}. \code{NA}, the default, means unset: the
#'   label is sized to its content and does not wrap.
#'   \code{unit(1, "npc")} is the useful value for a plot title, whose
#'   cell really is the full plot width. See \emph{Block labels}.
#' @param ... Additional arguments passed to
#'   \code{ggplot2::element_text()} (e.g. \code{colour}, \code{hjust}).
#'
#' @return An S7 object of class \code{element_markdown}, inheriting from
#'   \code{ggplot2::element_text}.
#' @seealso \code{\link{markdown_grob}}, \code{\link{element_latex}}
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'   ggplot(mtcars, aes(wt, mpg)) + geom_point() +
#'     labs(x = "**weight** in $10^3$ lbs") +
#'     theme(axis.title.x = gridmicrotex::element_markdown())
#' }
#' }
element_markdown <- function(math_font = "", fontsize = NULL,
                             lineheight = 1.2, max_width = 0,
                             render_mode = c("typeface", "path"),
                             justify = FALSE, style = NA, width = NA, ...) {
  .apply_opts("math_font", "render_mode", "justify")
  render_mode <- match.arg(render_mode)
  .check_justify(justify)
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for element_markdown(). ",
         "Please install it with install.packages('ggplot2').",
         call. = FALSE)
  }
  dots <- list(...)
  if (!is.null(fontsize)) dots$size <- fontsize

  obj <- do.call(.element_markdown_class, c(
    list(math_font = math_font, lineheight = lineheight,
         max_width = max_width, render_mode = render_mode,
         justify = justify, style = style, width = width),
    dots
  ))
  # ggplot2's element_text constructor injects legacy "element_text" and
  # "element" S3 strings into the class vector; S7 inheritance loses them,
  # which makes combine_elements treat us as an unrelated sibling and drop
  # us when resolving inherited theme entries (e.g. axis.title.y inheriting
  # from a user-set axis.title). Re-inject them so subclass detection works.
  class(obj) <- union(
    union(
      c("gridmicrotex::element_markdown", "ggplot2::element_text",
        "element_text"),
      class(obj)
    ),
    "element"
  )
  obj
}

# Placeholder -- replaced in .onLoad_ggplot2_markdown() with the real
# S7 constructor.
.element_markdown_class <- NULL

# --------------------------------------------------------------------------
# .onLoad_ggplot2_markdown() -- called from .onLoad_ggplot2()
# --------------------------------------------------------------------------
.onLoad_ggplot2_markdown <- function() {
  if (!requireNamespace("ggplot2", quietly = TRUE)) return()

  GeomMarkdown <<- ggplot2::ggproto("GeomMarkdown", ggplot2::Geom,

    required_aes = c("x", "y", "label"),

    default_aes = ggplot2::aes(
      size = 11,
      colour = "black",
      angle = 0,
      hjust = 0.5,
      vjust = 0.5,
      alpha = 1
    ),

    draw_key = ggplot2::draw_key_text,

    # default_aes is applied after setup_data, so a missing `size` column
    # here means the aesthetic was not mapped -- fill it from the fontsize
    # parameter. (In draw_panel the column is always populated, so the
    # parameter could never win there.)
    setup_data = function(data, params) {
      if (is.null(data$size)) {
        data$size <- params$fontsize %||% 11
      }
      data
    },

    draw_panel = function(data, panel_params, coord, fontsize = 11,
                          math_font = "", lineheight = 1.2, max_width = 0,
                          render_mode = "typeface", justify = FALSE,
                          style = NULL, na.rm = FALSE) {
      coords <- coord$transform(data, panel_params)

      grobs <- lapply(seq_len(nrow(coords)), function(i) {
        row <- coords[i, ]

        if (is.na(row$label) || !nzchar(row$label)) return(grid::nullGrob())

        col <- if (!is.null(row$alpha) && row$alpha < 1) {
          grDevices::adjustcolor(row$colour, alpha.f = row$alpha)
        } else {
          row$colour
        }

        markdown_grob(
          md = row$label,
          x = grid::unit(row$x, "npc"),
          y = grid::unit(row$y, "npc"),
          hjust = row$hjust,
          vjust = row$vjust,
          rot = row$angle %||% 0,
          math_font = math_font,
          max_width = max_width,
          render_mode = render_mode,
          justify = justify,
          style = style,
          gp = grid::gpar(col = col, fontsize = row$size,
                          lineheight = lineheight)
        )
      })

      do.call(grid::gList, grobs)
    }
  )

  .element_markdown_class <<- S7::new_class(
    "element_markdown",
    parent = ggplot2::element_text,
    properties = list(
      math_font = S7::new_property(S7::class_character, default = ""),
      lineheight = S7::new_property(S7::class_numeric, default = 1.2),
      max_width = S7::new_property(S7::class_numeric, default = 0),
      render_mode = S7::new_property(S7::class_character,
                                     default = "typeface"),
      justify = S7::new_property(S7::class_logical, default = FALSE),
      # A grid unit, or NA for "unset" -- see the note on `style` below
      # for why no property here may default to NULL.
      width = S7::new_property(S7::class_any, default = NA),
      # A markdown_style object, CSS text or a .css path -- whatever
      # markdown_grob(style =) takes, hence class_any.
      #
      # The default is NA, not NULL: ggplot2's theme merge reads an unset
      # property from the parent element, and a plain element_text has no
      # @style, so a NULL default fails with "Can't find property
      # <ggplot2::element_text>@style" as soon as the element is inherited
      # into. Every other property here has a non-NULL typed default for
      # the same reason. .element_grob_markdown() maps NA back to NULL,
      # which is what means "fall back to latex_options(markdown_style=)".
      style = S7::new_property(S7::class_any, default = NA)
    )
  )

  # element_grob is still an S3 generic in ggplot2, so register via S3.
  # S7 objects dispatch correctly thanks to their class vector.
  registerS3method(
    "element_grob", "gridmicrotex::element_markdown",
    .element_grob_markdown,
    envir = asNamespace("ggplot2")
  )
}

# element_grob method for element_markdown. Signature mirrors
# ggplot2:::element_grob.element_text so axes and guides can override
# element properties per call.
.element_grob_markdown <- function(element, label = "", x = NULL, y = NULL,
                                   family = NULL, face = NULL, colour = NULL,
                                   size = NULL, hjust = NULL, vjust = NULL,
                                   angle = NULL, lineheight = NULL,
                                   margin = NULL, margin_x = FALSE,
                                   margin_y = FALSE, ...) {
  if (is.null(label) ||
      (length(label) == 1L && (!nzchar(label) || is.na(label)))) {
    return(grid::nullGrob())
  }

  # Per-call overrides win, then element values, then a final fallback.
  fontsize    <- size       %||% element@size       %||% 11
  colour      <- colour     %||% element@colour     %||% "black"
  hjust       <- hjust      %||% element@hjust      %||% 0.5
  vjust       <- vjust      %||% element@vjust      %||% 0.5
  angle       <- angle      %||% element@angle      %||% 0
  family      <- family     %||% element@family
  face        <- face       %||% element@face
  lineheight  <- lineheight %||% element@lineheight %||% 1.2
  math_font   <- element@math_font   %||% ""
  max_width   <- element@max_width   %||% 0
  render_mode <- element@render_mode %||% "typeface"
  justify     <- element@justify     %||% FALSE
  # NA is the element's "unset" (see the property definition); markdown_grob
  # wants NULL for that, so it can fall back to the global option.
  unset <- function(v) is.atomic(v) && length(v) == 1L && is.na(v)
  style       <- element@style
  if (unset(style)) style <- NULL
  width       <- element@width
  if (unset(width)) width <- NULL
  margin      <- margin %||% element@margin

  gp <- grid::gpar(col = colour, fontsize = fontsize, lineheight = lineheight)
  if (!is.null(family) && nzchar(family)) gp$fontfamily <- family
  if (!is.null(face)   && nzchar(face))   gp$fontface   <- face

  # No $-stripping here, unlike element_latex(): in markdown a `$` pair is
  # the math delimiter, so removing it would destroy the label.

  # When x/y aren't supplied, anchor at the rotation-adjusted just so the
  # rotated text lands where ggplot2:::titleGrob would put it (e.g. for
  # axis.title.y with angle = 90, vjust = 1).
  default_just <- .rotate_just(angle, hjust, vjust)

  if (length(label) == 1L) {
    # A label with real block structure -- a heading, a list, a table, more
    # than one paragraph -- cannot be laid out as a single run: the run
    # path flattens it and the list markers and indents are lost. Fall
    # through to the box, which is the only thing here that does blocks.
    #
    # Tick labels never take this path: the multi-label branch below is
    # left alone deliberately, because a box sized in npc would measure
    # against the whole device (ggplot2 has not placed the element in its
    # cell when it asks for the height) and every tick would claim it.
    promote <- .md_needs_blocks(label)

    if (promote && angle %% 360 != 0) {
      # The box has no rotation, so one of the two has to give. Keeping
      # the angle and flattening the blocks is much the smaller loss: a
      # rotated axis title laid out horizontally would cross the panel.
      warning("A rotated `element_markdown()` label cannot use block ",
              "layout; rendering it as a single run. Set `angle = 0` to ",
              "lay the blocks out.", call. = FALSE)
      promote <- FALSE
    }
    if (!is.null(width) && angle %% 360 != 0) {
      # Still honour the requested measure, as the run path's own.
      max_width <- grid::convertWidth(width, "bigpts", valueOnly = TRUE)
      width <- NULL
    }

    if (promote || !is.null(width)) {
      return(markdown_box_grob(
        md = label,
        x = x %||% grid::unit(hjust, "npc"),
        y = y %||% grid::unit(vjust, "npc"),
        # NULL sizes the box to its content, so nothing wraps and the
        # reported height does not depend on a width that is not yet known.
        width = width,
        hjust = hjust, vjust = vjust,
        # hjust positions a content-sized box; halign is what aligns the
        # blocks when the box spans its cell. Both are wanted.
        halign = hjust,
        margin = if (grid::is.unit(margin) && length(margin) == 4L) margin,
        style = style, gp = gp
      ))
    }

    if (is.null(x)) x <- grid::unit(default_just$hjust, "npc")
    if (is.null(y)) y <- grid::unit(default_just$vjust, "npc")
    return(markdown_grob(
      md = label, x = x, y = y,
      hjust = hjust, vjust = vjust, rot = angle,
      math_font = math_font, max_width = max_width,
      render_mode = render_mode, justify = justify, style = style,
      gp = gp
    ))
  }

  # Multiple labels (axis tick labels) -- render a gTree of grobs.
  n <- length(label)
  if (is.null(x)) x <- grid::unit(rep(default_just$hjust, n), "npc")
  if (is.null(y)) y <- grid::unit(rep(default_just$vjust, n), "npc")

  grobs <- grid::gList()
  for (i in seq_len(n)) {
    lab <- label[i]
    if (is.na(lab) || !nzchar(lab)) next
    grobs <- grid::gList(grobs, markdown_grob(
      md = lab, x = .pick_unit(x, i), y = .pick_unit(y, i),
      hjust = hjust, vjust = vjust, rot = angle,
      math_font = math_font, max_width = max_width,
      render_mode = render_mode, justify = justify, style = style,
      gp = gp,
      name = paste0("ticklabel.", i)
    ))
  }
  # Same class as the LaTeX tick labels, so the width/height methods that
  # let ggplot2 reserve room for the axis strip apply here too.
  grid::gTree(children = grobs, name = "axis.markdown.labels",
              cl = "gridmicrotex_labels")
}

---
title: "Introduction to gridmicrotex"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to gridmicrotex}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dpi = 300,
  dev = "ragg_png"
)
library(gridmicrotex)
```

## What is gridmicrotex?

**gridmicrotex** renders LaTeX math equations as native R `grid` graphics
objects (grobs). It uses the [MicroTeX](https://github.com/NanoMichael/MicroTeX)
C++ library as its layout engine --- MicroTeX parses LaTeX, builds the TeX
box model, and computes exact glyph coordinates. The package intercepts
this layout data and maps it to native grid primitives (`pathGrob`,
`segmentsGrob`, `rectGrob`, `textGrob`), producing a `gTree` that works
on any R graphics device at any resolution.

**Key features:**

- No external LaTeX installation required --- MicroTeX is fully embedded
- Resolution-independent vector output on all R devices (PNG, PDF, SVG, ...)
- Full math support: fractions, roots, integrals, matrices, Greek letters,
  accents, delimiters, and more
- Two bundled math fonts: Lete Sans Math (sans-serif, default) and
  STIX Two Math (serif); additional fonts via `load_font()`
- Color support via `\textcolor{}`
- ggplot2 integration with `geom_latex()` and `element_latex()`
- CJK and multilingual text in `\text{}` blocks

## Basic usage

The core function is `latex_grob()`, which returns a grid grob:

```{r basic, fig.height=0.8, fig.width=2, out.width="50%"}
g <- latex_grob("\\frac{-b \\pm \\sqrt{b^2 - 4ac}}{2a}", gp = grid::gpar(fontsize = 24))
grid::grid.newpage()
grid::grid.draw(g)
```

For quick rendering, use `grid.latex()`:

```{r quick, fig.height=0.8, fig.width=2, out.width="50%"}
grid::grid.newpage()
grid.latex("\\sum_{i=1}^{n} x_i^2", gp = grid::gpar(fontsize = 28))
```

## Positioning and justification

Control placement with `x`, `y`, `hjust`, and `vjust`:

```{r positioning, fig.height=1.2, fig.width=2, out.width="50%"}
grid::grid.newpage()
grid.latex("Famous $E = mc^2$", x = 0.2, y = 0.7, hjust = 0, gp = grid::gpar(fontsize = 24))
grid.latex("F = ma", x = 0.2, y = 0.3, hjust = 0, gp = grid::gpar(fontsize = 24), input_mode = "math")
```

By default, the input is treated as LaTeX math mode, which treats string as text by default and use `$...$` or `\\(...\\)` delimiters to render math. The `"Famous"` in the first equation above treated as text. The `mixed` mode converts the input to math mode to reduce the user burden for typing `\\text{}` and the conversion might not be perfect, but it should handle most common cases without user intervention. Use `input_mode = "math"` to treat the whole string as math mode (the second example) or if you find a problem with the conversion and render text with `\\text{}`. You can change this with global `latex_options(input_mode = "math")` for heavy math or users who wants to the advantages LaTex macros, etc. The vignette from next example will set the input mode to `math` globally and render the whole string as math mode.


## Colors

You can use `r"()"` raw strings to write LaTeX with regular newlines and quotes without escaping. Set the formula color via `gp`, or use `\textcolor{}` within the LaTeX:

```{r colors, fig.height=0.6, fig.width=2, out.width="40%"}
latex_options(input_mode = "math") # Set math mode globally
grid::grid.newpage()
grid.latex(
  r"(\textcolor{red}{\alpha} + \textcolor{blue}{\beta} = \gamma)",
  gp = grid::gpar(fontsize = 28)
)
```

## Math fonts

The package ships with two math fonts, both loaded automatically:

| Alias | Font | License |
|-------|------|---------|
| `"lete"` (default) | Lete Sans Math | SIL Open Font License |
| `"stix"` | STIX Two Math | SIL Open Font License |

```{r fonts-list}
available_math_fonts()
```

```{r fonts-default, fig.height=0.8, fig.width=2, out.width="50%"}
latex_options(math_font = "stix")
grid::grid.newpage()
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24))

# Switch back to the default (Lete Sans Math)
latex_options(math_font = "lete")
```

You can also override the font per call via `math_font`:

```{r fonts, fig.height=1.5, fig.width=2, out.width="50%"}
grid::grid.newpage()
grid::pushViewport(grid::viewport(layout = grid::grid.layout(2, 1)))
grid::pushViewport(grid::viewport(layout.pos.row = 1))
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24))
grid::upViewport()
grid::pushViewport(grid::viewport(layout.pos.row = 2))
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24), math_font = "stix")
grid::upViewport(2)
```

Use `available_math_fonts()` to list loaded fonts and `check_fonts()`
for a diagnostic report.

### Advanced: loading custom fonts

Use `load_font()` to add any additional OpenType math font. The OpenType
MATH table is parsed directly in C++ --- no companion metrics file or
external toolchain is required:

```r
load_font("path/to/MyFont.otf")
```

### Render modes

gridmicrotex supports two rendering modes for math glyphs:
  
- **`"typeface"`** (default): Renders glyphs as native text using the
  math font's typeface. This produces selectable, searchable, and
  accessible text in PDF and SVG output. Bundled math fonts (Lete Sans
  Math, STIX Two Math) and any registered via `load_font()` are read
  directly from their OTF files --- no system-wide font install is
  required. Requires a device that supports the R \eqn{\geq} 4.3 glyph
  engine (e.g., `ragg::agg_png()`, `svglite::svglite()`,
  `grDevices::cairo_pdf()`). On devices that do not (e.g., the base
  `pdf()` device), the package automatically falls back to path mode
  with a warning.

- **`"path"`**: Renders each glyph as a filled vector path.
  This works on all R graphics devices and produces pixel-perfect output.
  However, text in PDF/SVG output is not selectable or searchable.

```r
# Default typeface mode (selectable text in PDF/SVG)
grid.latex("E = mc^2", gp = grid::gpar(fontsize = 24))

# Explicit path mode (works everywhere, but text is not selectable)
grid.latex("E = mc^2", gp = grid::gpar(fontsize = 24), render_mode = "path")
```

> **Important: Do not use `showtext::showtext_auto()` with typeface mode.**
> The [showtext](https://CRAN.R-project.org/package=showtext) package
> globally intercepts all text rendering and converts it to vector paths.
> This silently defeats typeface mode, causing all math glyphs to appear
> as paths instead of native text — even on devices like `svglite` and
> `ragg` that fully support font embedding. If you need showtext for
> other parts of your plot, disable it before drawing LaTeX formulas:
>
> ```r
> showtext::showtext_auto(FALSE)
> grid.latex("E = mc^2", gp = grid::gpar(fontsize = 24))  # typeface mode works correctly
> ```

## Querying dimensions

`latex_dims()` returns the bounding box of an expression:

```{r dims}
dims <- latex_dims("\\frac{a}{b}", gp = grid::gpar(fontsize = 20))
dims
```

This is useful for layout calculations and ensuring labels fit.

## Text rendering and CJK support

Text inside `\text{}` and `\mbox{}` is rendered using R's standard
text-rendering system. This means `gp$fontfamily` controls the font for
**all** text content --- Latin letters, CJK characters, Cyrillic, and any
other script your R graphics device supports:

```{r cjk, fig.height = 0.8, fig.width = 2.5, out.width="50%"}
grid::grid.newpage()
grid.latex("x^2 + \\text{你好}", gp = grid::gpar(fontsize = 24, fontfamily = "sans"))
```

Any font available to R works: base families like `"sans"`, `"serif"`,
`"mono"`, or fonts registered via **showtext** / **systemfonts**.

### Font pairing

The bundled math fonts have different styles. For a consistent look,
pair them with a matching `fontfamily`:

| Math font | Style | Suggested `fontfamily` |
|-----------|-------|------------------------|
| Lete Sans Math (`"lete"`, default) | Sans-serif | `"sans"` |
| STIX Two Math (`"stix"`) | Serif | `"serif"` |

```{r font-pairing, fig.height = 1, fig.width = 6, out.width="70%"}
grid::grid.newpage()
grid.latex(
  "\\text{Theorem: } \\forall x \\in \\mathbb{R},\\; x^2 \\geq 0",
  math_font = "stix",
  gp = grid::gpar(fontfamily = "serif", fontsize = 12)
)
```

## Supported LaTeX

gridmicrotex uses the MicroTeX engine, which is a **math formula renderer**,
not a full document typesetter. It covers the vast majority of math notation
you would use in plots and figures, but does not attempt to replace a full
LaTeX installation.

### Complicated examples

```{r complex-formula, fig.height = 3, fig.width=6, out.width="60%"}
grid::grid.newpage()
grid.latex(paste0(
      "\\begin{array}{l}",
      "  \\forall\\varepsilon\\in\\mathbb{R}_+^*\\ \\exists\\eta>0",
      "\\ |x-x_0|\\leq\\eta\\Longrightarrow|f(x)-f(x_0)|\\leq\\varepsilon\\\\",
      "  \\det",
      "  \\begin{bmatrix}",
      "      a_{11}&a_{12}&\\cdots&a_{1n}\\\\",
      "      a_{21}&\\ddots&&\\vdots\\\\",
      "      \\vdots&&\\ddots&\\vdots\\\\",
      "      a_{n1}&\\cdots&\\cdots&a_{nn}",
      "  \\end{bmatrix}",
      "  \\overset{\\mathrm{def}}{=}\\sum_{\\sigma\\in\\mathfrak{S}_n}",
      "\\varepsilon(\\sigma)\\prod_{k=1}^n a_{k\\sigma(k)}\\\\",
      "  \\int_0^\\infty{x^{2n} e^{-a x^2}\\,dx} = \\frac{2n-1}{2a}",
      " \\int_0^\\infty{x^{2(n-1)} e^{-a x^2}\\,dx}",
      " = \\frac{(2n-1)!!}{2^{n+1}} \\sqrt{\\frac{\\pi}{a^{2n+1}}}\\\\",
      "\\end{array}"
), gp = grid::gpar(fontsize = 16))
```


```{r table-multicolumn, fig.height = 3, fig.width=8, out.width="60%"}
grid::grid.newpage()

grid.latex(
  "
  \\newcolumntype{s}{>{\\color{#1234B6}}c}
\\begin{array}{|c|c|c|s|}
  \\hline
  \\rowcolor{Tan}\\multicolumn{4}{|c|}{\\textcolor{white}{\\bold{\\text{Table Head}}}}\\\\
  \\hline
  \\text{Matrix}&\\multicolumn{2}{|c|}{\\text{Multicolumns}}&\\text{Font size commands}\\\\
  \\hline
  \\begin{pmatrix}
      \\alpha_{11}&\\cdots&\\alpha_{1n}\\\\
      \\hdotsfor{3}\\\\
      \\alpha_{n1}&\\cdots&\\alpha_{nn}
  \\end{pmatrix}
  &\\large \\text{Left}&\\cellcolor{#00bde5}\\small \\textcolor{white}{\\text{\\bold{Right}}}
  &\\small \\text{small Small}\\\\
  \\hline
  \\multicolumn{4}{|c|}{\\text{Table Foot}}\\\\
  \\hline
\\end{array}
  ",
  gp = grid::gpar(fontsize = 22)
)
```

```{r complicated-equation, fig.height = 9, fig.width=9, out.width="60%"}
grid::grid.newpage()
grid.latex(
  "\\definecolor{gris}{gray}{0.9}
\\definecolor{noir}{rgb}{0,0,0}
\\definecolor{bleu}{rgb}{0,0,1}
\\fatalIfCmdConflict{false}
\\newcommand{\\pa}{\\left|}
\\begin{array}{c}
  \\LaTeX\\\\
  \\begin{split}
      |I_2| &= \\pa\\int_0^T\\psi(t)\\left\\{ u(a,t)-\\int_{\\gamma(t)}^a \\frac{d\\theta}{k} (\\theta,t) \\int_a^\\theta c(\\xi)
          u_t (\\xi,t)\\,d\\xi\\right\\}dt\\right|\\\\
      &\\le C_6 \\Bigg|\\pa f \\int_\\Omega \\pa\\widetilde{S}^{-1,0}_{a,-}
          W_2(\\Omega, \\Gamma_1)\\right|\\ \\right|\\left| |u|\\overset{\\circ}{\\to} W_2^{\\widetilde{A}}(\\Omega\\Gamma_r,T)\\right|\\Bigg|\\\\
      &\\\\
      &\\begin{pmatrix}
          \\alpha&\\beta&\\gamma&\\delta\\\\
          \\aleph&\\beth&\\gimel&\\daleth\\\\
          \\mathfrak{A}&\\mathfrak{B}&\\mathfrak{C}&\\mathfrak{D}\\\\
          \\boldsymbol{\\mathfrak{a}}&\\boldsymbol{\\mathfrak{b}}&\\boldsymbol{\\mathfrak{c}}&\\boldsymbol{\\mathfrak{d}}
      \\end{pmatrix}
      \\quad{(a+b)}^{\\frac{n}{2}}=\\sqrt{\\sum_{k=0}^n\\tbinom{n}{k}a^kb^{n-k}}\\quad
          \\Biggl(\\biggl(\\Bigl(\\bigl(()\\bigr)\\Bigr)\\biggr)\\Biggr)\\\\
      &\\forall\\varepsilon\\in\\mathbb{R}_+^*\\ \\exists\\eta>0\\ |x-x_0|\\leq\\eta\\Longrightarrow|f(x)-f(x_0)|\\leq\\varepsilon\\\\
      &\\det
      \\begin{bmatrix}
          a_{11}&a_{12}&\\cdots&a_{1n}\\\\
          a_{21}&\\ddots&&\\vdots\\\\
          \\vdots&&\\ddots&\\vdots\\\\
          a_{n1}&\\cdots&\\cdots&a_{nn}
      \\end{bmatrix}
      \\overset{\\mathrm{def}}{=}\\sum_{\\sigma\\in\\mathfrak{S}_n}\\varepsilon(\\sigma)\\prod_{k=1}^n a_{k\\sigma(k)}\\\\
      &\\Delta f(x,y)=\\frac{\\partial^2f}{\\partial x^2}+\\frac{\\partial^2f}{\\partial y^2}\\qquad\\qquad \\fcolorbox{noir}{gris}
          {n!\\underset{n\\rightarrow+\\infty}{\\sim} {\\left(\\frac{n}{e}\\right)}^n\\sqrt{2\\pi n}}\\\\
      &\\sideset{_\\alpha^\\beta}{_\\gamma^\\delta}{
      \\begin{pmatrix}
          a&b\\\\
          c&d
      \\end{pmatrix}}
      \\xrightarrow[T]{n\\pm i-j}\\sideset{^t}{}A\\xleftarrow{\\overrightarrow{u}\\wedge\\overrightarrow{v}}
          \\underleftrightarrow{\\iint_{\\mathds{R}^2}e^{-\\left(x^2+y^2\\right)}\\,\\mathrm{d}x\\mathrm{d}y}
  \\end{split}\\\\
  \\rotatebox{30}{\\sum_{n=1}^{+\\infty}}\\quad\\mbox{Mirror rorriM}\\reflectbox{\\mbox{Mirror rorriM}}
\\end{array}",
  gp = grid::gpar(fontsize = 22),
  render_mode = "path"
)
```



### What is not supported

MicroTeX is a **math formula renderer**, not a full LaTeX engine. The
following are outside its scope:

- **Document structure**: `\section`, `\begin{document}`, page layout,
  headers/footers, `\tableofcontents`
- **Package loading**: `\usepackage{}` --- all supported commands are
  built in
- **Paragraph text**: line breaking, hyphenation, justified paragraphs
- **TikZ / PGF** drawing commands
- **Images**: `\includegraphics`
- **Cross-references**: `\label`, `\ref`, `\cite`, bibliographies
- **Theorem environments**: `\begin{theorem}`, `\begin{proof}`
- **Lists**: `itemize`, `enumerate`, `description`
- **Some amsmath commands**: `\substack`, `\tag`, equation numbering

For most statistical graphics use cases --- axis labels, annotations,
legends, and in-plot formulas --- the supported feature set is more
than sufficient.

## Project-wide defaults

`latex_options()` sets defaults for `math_font` and `render_mode`, used
by `latex_grob()`, `grid.latex()`, `latex_dims()`, and `latex_tree()`
whenever the corresponding argument is not supplied at the call site.
Size is controlled at the grob level via `gp$fontsize` /
`gp$lineheight` (see *Basic usage*).

```{r options, eval=FALSE}
latex_options(math_font = "stix", render_mode = "typeface")

# Later calls pick these up automatically
grid.latex("\\sum_{i=1}^{n} i^{2}", gp = grid::gpar(fontsize = 14))

# Query current settings
latex_options()

# Reset to built-in defaults
reset_latex_options()
```

Explicit arguments always win. Setting `math_font` via
`latex_options()` also updates the MicroTeX engine default, so you
don't also need a separate font-setup call.

## User-defined macros

`define_macro()` registers zero-argument shorthands that are expanded
by text substitution before the expression reaches MicroTeX. Handy for
recurring notation:

```{r macros, fig.height=0.7, fig.width=3, out.width="50%"}
define_macro("RR", "\\mathbb{R}")
define_macro("eps", "\\varepsilon")

grid::grid.newpage()
grid.latex("\\forall \\eps > 0, \\eps \\in \\RR", gp = grid::gpar(fontsize = 24))

clear_macros()
```

Macro names must be ASCII letters. Expansion iterates to a fixed point,
so macros can reference other macros. Use `list_macros()` to see
currently registered ones, and `clear_macros()` (with no arguments) to
drop them all.

## Layout caching

Parsed layouts are memoised by `(tex, fontsize, math_font, render_mode,
...)`. Re-drawing the same formula --- for example, the same axis label
across many plots --- reuses the cached layout:

```{r cache, eval=FALSE}
latex_cache_info()       # size / max_size / hits / misses
latex_cache_limit(1024)  # raise or lower the LRU capacity
latex_cache_clear()      # wipe the cache (e.g. after re-loading fonts)
```

Set the limit to `0` to disable caching entirely.

## Introspecting a formula

`latex_tree()` returns the raw draw-record table plus bbox metadata,
useful for debugging alignment, counting glyphs, or building custom
grobs on top of the layout:

```{r tree}
tr <- latex_tree("\\frac{a}{b}")
tr
head(tr$records, 3)
```

## Debug overlay

Pass `debug = TRUE` to `latex_grob()` / `grid.latex()` to overlay
diagnostics on the rendered formula --- the full bounding box (dashed
gray), the baseline (solid red), and a dot at each draw record's
origin. Useful for checking vertical alignment between a formula and
surrounding grobs:

```{r debug, fig.height=1, fig.width=3, out.width="60%"}
grid::grid.newpage()
grid.latex("x^{2} + y_{i}", gp = grid::gpar(fontsize = 30), debug = TRUE)
```

## Comparison with alternatives

| Approach       | LaTeX required? | Device independent? | Vector? | Math coverage |
|:---------------|:---------------:|:-------------------:|:-------:|:-------------:|
| `tikzDevice`   | Yes             | No                  | Yes     | Full          |
| `xdvir`        | Yes             | No                  | Yes     | Full          |
| `latexpdf`     | Yes             | No                  | Yes     | Full (tables) |
| `latex2exp`    | No              | Yes                 | Yes     | Limited       |
| `plotmath`     | No              | Yes                 | Yes     | Limited       |
| **gridmicrotex** | **No**       | **Yes**             | **Yes** | **Broad**     |


## Graphics backend

The default graphics device on Windows (`windows()`) and macOS
(`quartz()`) may not find the bundled math fonts, producing warnings
like:

```
font family not found in Windows font database
```

To avoid this, switch to a modern graphics backend that uses
[systemfonts](https://CRAN.R-project.org/package=systemfonts) for font
resolution:

```r
# For knitr / R Markdown --- add to your setup chunk:
knitr::opts_chunk$set(dev = "ragg_png")

# For interactive use:
options(device = function(...) ragg::agg_png(tempfile(fileext = ".png"), ...))
```

Recommended backends:

| Backend               | Format | Package               |
|:----------------------|:-------|:----------------------|
| `ragg::agg_png()`     | PNG    | [ragg](https://CRAN.R-project.org/package=ragg)       |
| `svglite::svglite()`  | SVG    | [svglite](https://CRAN.R-project.org/package=svglite) |
| `grDevices::cairo_pdf()` | PDF | Base R (Cairo build)  |

Alternatively, use `render_mode = "path"` to bypass font lookup
entirely --- glyphs are drawn as vector paths, which works on all
devices but produces non-selectable text in PDF/SVG.

---
title: "Using LaTeX Math in ggplot2"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using LaTeX Math in ggplot2}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 6,
  fig.height = 4,
  dpi = 300,
  dev = "png"
)
library(gridmicrotex)
library(ggplot2)
```

gridmicrotex provides ggplot2 extensions for rendering LaTeX math in
plots:

- **`geom_latex()`** --- a geom layer for placing LaTeX labels at data
  coordinates.
- **`element_latex()`** --- a theme element for rendering axis titles,
  plot titles, and other text elements as LaTeX.

There are matching `geom_markdown()` and `element_markdown()` for labels
written in markdown rather than raw LaTeX --- covered at the end of this
vignette, and in more depth in `vignette("markdown")`.

## Annotating plots with `geom_latex()`

`geom_latex()` works like `geom_text()` but interprets the `label`
aesthetic as a LaTeX math string. You can also map the `size` (font size in points) 
and `colour` aesthetics as usual. `element_latex()` replaces a text theme element so that its label is
rendered as LaTeX math.

```{r geom-basic, out.width="70%"}
df <- data.frame(
  x = 1:3,
  y = 1:3,
  eq = c(r"($x^2$)", r"(\frac{a}{b})", r"($\sum_{i=1}^n x_i$)"),
  col = c("red", "blue", "green")
)

ggplot(df, aes(x, y, 
               label = eq, 
               colour = col, 
               size = c(14, 18, 14))) +
  geom_latex() +
  scale_colour_identity() +
  scale_size_identity() +
  labs(
    x = r"($\beta_1 \cdot x + \beta_0$)",
    y = r"($\mathrm{mpg}$)"
  ) +
  theme(
    axis.title.x = element_latex(fontsize = 14),
    axis.title.y = element_latex(fontsize = 14)
  )
```

Dollar-sign delimiters are stripped automatically here, so
`r"(\frac{a}{b})"` and `r"($\frac{a}{b}$)"` produce the same output.


### Adding equation annotations to a scatter plot

A common use case is annotating a regression fit with the model
equation. Use `annotate("latex", ...)` for single annotations --- it
delegates to `GeomLatex` internally but avoids creating a data frame
and automatically hides the legend.

```{r regression-annotation, out.width="70%"}
fit <- lm(mpg ~ wt, data = mtcars)
b0 <- round(coef(fit)[1], 1)
b1 <- round(coef(fit)[2], 1)
r2 <- round(summary(fit)$r.squared, 3)

eq_label <- sprintf(r"($\hat{y} = %s %s x, \quad R^2 = %s$)", b0, b1, r2)

ggplot(mtcars, aes(wt, mpg)) +
  geom_point() +
  geom_smooth(method = "lm", se = FALSE) +
  annotate("latex", x = 4, y = 30, label = eq_label, size = 12) +
  theme_minimal()
```

## Markdown labels

When a label is more prose than formula --- a bold phrase, an italic
word, a symbol in a sentence --- `geom_markdown()` and
`element_markdown()` accept markdown with inline `$math$`. They take the
same aesthetics and theme slots as their LaTeX counterparts; only the
label syntax differs.

```{r markdown, out.width="70%"}
df <- data.frame(
  x   = 1:3,
  y   = c(2, 3, 1),
  lab = c("**bold**", r"(*slope* $\beta_1$)", "`code` and $x^2$")
)

ggplot(df, aes(x, y, label = lab)) +
  geom_point() +
  geom_markdown(fontsize = 14, vjust = -0.6) +
  ylim(0.5, 3.6) +
  labs(
    title = r"(*Fitted* model: $\hat{y} = \beta_0 + \beta_1 x$)",
    x     = "**weight** in $10^3$ lbs",
    y     = r"(*efficiency* $\eta$)"
  ) +
  theme(
    plot.title   = element_markdown(fontsize = 14),
    axis.title.x = element_markdown(),
    axis.title.y = element_markdown()
  )
```

Unlike `element_latex()`, `element_markdown()` never strips `$`
delimiters --- in markdown a `$...$` pair *is* the math, so removing it
would change the label. 

### Annotating with markdown

`annotate("markdown", ...)` is the markdown counterpart of
`annotate("latex", ...)`. It suits a callout that is part prose and part
formula --- and because `<br>` starts a new line, one annotation can hold
several:

```{r markdown-annotation, out.width="70%"}
fit <- lm(mpg ~ wt, data = mtcars)

note <- sprintf(
  r"(**Linear fit**<br>$\hat{y} = %s %s x$<br>$R^2 = %s$, *p* < 0.001)",
  round(coef(fit)[1], 1),
  round(coef(fit)[2], 1),
  round(summary(fit)$r.squared, 3)
)

ggplot(mtcars, aes(wt, mpg)) +
  geom_point(colour = "grey65") +
  geom_smooth(method = "lm", se = FALSE, colour = "#1F6FB2") +
  annotate("markdown", x = 4.1, y = 32, label = note, size = 11,
           style = "strong { color: #B22222 }") +
  theme_minimal()
```

The LaTeX version of the same annotation would need `\text{}` around
every word and could not set the heading in bold on its own line. 


### Styling labels

Both take a `style`: a `markdown_style()` object, CSS text, or the path
to a `.css` file. It is the same cascade `markdown_box_grob()` uses --- so
a stylesheet can set the look of every label at once, and `<span
class=>` picks out one run inside a label.

```{r style, out.width="70%"}
css <- "
  body   { color: #33475B }
  strong { color: #B22222 }
  code   { color: #1F6FB2 }
  .unit  { color: grey55; font-size: smaller }
"

df <- data.frame(
  x = 1:3, y = c(2, 3, 1),
  lab = c("**bold** is red", "`code` is blue",
          'plain <span class="unit">with a note</span>')
)

ggplot(df, aes(x, y, label = lab)) +
  geom_point() +
  geom_markdown(style = css, fontsize = 14, vjust = -0.8) +
  ylim(0.5, 3.8) +
  labs(title = r"(**Styled** labels and $\beta_1$)",
       x = 'weight <span class="unit">(1000 lbs)</span>') +
  theme(
    plot.title   = element_markdown(style = css, fontsize = 14),
    axis.title.x = element_markdown(style = css)
  )
```

On a single-run label only the properties that compile to LaTeX apply ---
colour, the `font-*` family, decorations. Margins, padding and
backgrounds need block layout, which is the next section.
`latex_options(markdown_style = )` sets a default for a whole document,
so the argument is only needed to override it.

### Block titles

A label with real block structure --- a heading, a list, a table, or more
than one paragraph --- is laid out as blocks instead of being flattened
into a single run. There is nothing to switch on: `element_markdown()`
notices. That makes a title a small document, and the `body` rule styles
the box around it.

```{r blocktitle, out.width="70%"}
ggplot(mtcars, aes(wt, mpg)) +
  geom_point(colour = "grey40") +
  labs(title = r"(## Fuel economy falls with weight

- slope $\beta_1 = -5.34$, *p* < 0.001
- $R^2 = 0.75$ over $n = 32$ cars)") +
  theme(plot.title = element_markdown(style = "
    body { background: #F4F7FB; padding: 10px;
           border: 1px solid #C7D6E5; border-radius: 4px }
  "))
```

Two consequences of how ggplot2 measures theme elements are worth
knowing.

**Wrapping is opt-in.** ggplot2 asks an element how tall it is before it
knows how wide the element's cell will be, so a relative width would be
resolved against the whole device and the title would reserve the wrong
height. Without `width` the label is sized to its content and never
wraps. Pass one when you want wrapping --- `unit(1, "npc")` is the right
value for a title, whose cell really is the plot width.

**Tick labels and rotated labels stay single runs.** Axis tick labels for
the same measuring reason. Rotated labels because the box cannot rotate:
a rotated label with blocks in it keeps its angle, renders as one run,
and warns, since laying a `axis.title.y` out horizontally across the
panel would be the worse of the two failures.

---
title: "Introduction to gridmicrotex"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to gridmicrotex}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dpi = 300,
  dev = "png"
)
```

## What is gridmicrotex?

**gridmicrotex** renders LaTeX math as native R `grid` graphics objects.
It embeds the [MicroTeX](https://github.com/NanoMichael/MicroTeX) C++
engine: MicroTeX parses the LaTeX, builds the TeX box model and computes
exact glyph coordinates, and the package maps that layout onto grid
primitives (`pathGrob`, `segmentsGrob`, `rectGrob`, `textGrob`),
returning a `gTree`.

No LaTeX installation is required, and the result is resolution
independent on every R device.

- Full math: fractions, roots, integrals, matrices, Greek, accents,
  delimiters, and colour via `\textcolor{}`
- Two bundled math fonts, plus any OpenType math font through
  `load_math_font()`
- CJK, RTL and multilingual text inside `\text{}`
- ggplot2 integration --- `geom_latex()` and `element_latex()`, see
  `vignette("ggplot2-integration")`
- Markdown with inline math --- see `vignette("markdown")`

## Quick start

`latex_grob()` returns a grob; `grid.latex()` builds and draws one.

```{r basic, fig.height=1, fig.width=2.4, out.width="45%"}
library(gridmicrotex)
library(grid)

grid.newpage()
grid.latex(r"($\frac{\textcolor{red}{-b} \pm \sqrt{b^2 - 4ac}}{2a}$)")
```

**Write LaTeX in a raw string.** `r"(...)"` passes backslashes through
untouched, so what you paste is what renders. Every example here uses
one. In an ordinary `"..."` string each `\` has to be doubled --- and
LaTeX's own row separator `\\` becomes a bewildering `\\\\`.

### Mixing text and math

The default `input_mode = "mixed"` reads the string as prose and
typesets only what sits inside `$…$` or `\(…\)` as math, which is why
`Famous:` below needs no markup. It saves typing, but the split is
heuristic. `input_mode = "math"` treats the whole string as math, so
prose must be wrapped in `\text{}`; that suits heavy math and pasted
LaTeX. These two render identically:

```{r modes, fig.height=1.2, fig.width=3.2, out.width="45%"}
grid.newpage()
grid.latex(r"(Famous: $E = mc^2$)",
           x = 0.05, y = 0.7, hjust = 0, )
grid.latex(r"(\text{Famous: } E = mc^2)", input_mode = "math",
           x = 0.05, y = 0.3, hjust = 0, )
```

Set the mode for a whole session with `latex_options(input_mode = )`.

## What it can render

Two figures before the details, because the range is easier to see than
to describe. A coloured `array` with `\multicolumn`, `\rowcolor`,
`\cellcolor`, a custom column type and a nested matrix:

```{r showcase-table, fig.height = 3, fig.width = 8, out.width="70%"}
grid.newpage()
grid.latex(r"(
\newcolumntype{s}{>{\color{#1234B6}}c}
\begin{array}{|c|c|c|s|}
  \hline
  \rowcolor{Tan}\multicolumn{4}{|c|}{\textcolor{white}{\bold{\text{Table Head}}}}\\
  \hline
  \text{Matrix}&\multicolumn{2}{|c|}{\text{Multicolumns}}&\text{Font size commands}\\
  \hline
  \begin{pmatrix}
      \alpha_{11}&\cdots&\alpha_{1n}\\
      \hdotsfor{3}\\
      \alpha_{n1}&\cdots&\alpha_{nn}
  \end{pmatrix}
  &\large \text{Left}&\cellcolor{#00bde5}\small \textcolor{white}{\text{\bold{Right}}}
  &\small \text{small Small}\\
  \hline
  \multicolumn{4}{|c|}{\text{Table Foot}}\\
  \hline
\end{array}
)", )
```

And a page of assorted notation --- `split` alignment, fraktur, stacked
delimiters, `\sideset`, extensible arrows, `\rotatebox`, `\reflectbox`
and a boxed result:

```{r showcase-equation, fig.height = 9, fig.width = 9, out.width="70%"}
grid.newpage()
grid.latex(r"(
\definecolor{gris}{gray}{0.9}
\definecolor{noir}{rgb}{0,0,0}
\fatalIfCmdConflict{false}
\newcommand{\pa}{\left|}
\begin{array}{c}
  \LaTeX\\
  \begin{split}
      |I_2| &= \pa\int_0^T\psi(t)\left\{ u(a,t)-\int_{\gamma(t)}^a \frac{d\theta}{k} (\theta,t) \int_a^\theta c(\xi)
          u_t (\xi,t)\,d\xi\right\}dt\right|\\
      &\le C_6 \Bigg|\pa f \int_\Omega \pa\widetilde{S}^{-1,0}_{a,-}
          W_2(\Omega, \Gamma_1)\right|\ \right|\left| |u|\overset{\circ}{\to} W_2^{\widetilde{A}}(\Omega\Gamma_r,T)\right|\Bigg|\\
      &\\
      &\begin{pmatrix}
          \alpha&\beta&\gamma&\delta\\
          \aleph&\beth&\gimel&\daleth\\
          \mathfrak{A}&\mathfrak{B}&\mathfrak{C}&\mathfrak{D}\\
          \boldsymbol{\mathfrak{a}}&\boldsymbol{\mathfrak{b}}&\boldsymbol{\mathfrak{c}}&\boldsymbol{\mathfrak{d}}
      \end{pmatrix}
      \quad{(a+b)}^{\frac{n}{2}}=\sqrt{\sum_{k=0}^n\tbinom{n}{k}a^kb^{n-k}}\quad
          \Biggl(\biggl(\Bigl(\bigl(()\bigr)\Bigr)\biggr)\Biggr)\\
      &\forall\varepsilon\in\mathbb{R}_+^*\ \exists\eta>0\ |x-x_0|\leq\eta\Longrightarrow|f(x)-f(x_0)|\leq\varepsilon\\
      &\det
      \begin{bmatrix}
          a_{11}&a_{12}&\cdots&a_{1n}\\
          a_{21}&\ddots&&\vdots\\
          \vdots&&\ddots&\vdots\\
          a_{n1}&\cdots&\cdots&a_{nn}
      \end{bmatrix}
      \overset{\mathrm{def}}{=}\sum_{\sigma\in\mathfrak{S}_n}\varepsilon(\sigma)\prod_{k=1}^n a_{k\sigma(k)}\\
      &\Delta f(x,y)=\frac{\partial^2f}{\partial x^2}+\frac{\partial^2f}{\partial y^2}\qquad\qquad \fcolorbox{noir}{gris}
          {n!\underset{n\rightarrow+\infty}{\sim} {\left(\frac{n}{e}\right)}^n\sqrt{2\pi n}}\\
      &\sideset{_\alpha^\beta}{_\gamma^\delta}{
      \begin{pmatrix}
          a&b\\
          c&d
      \end{pmatrix}}
      \xrightarrow[T]{n\pm i-j}\sideset{^t}{}A\xleftarrow{\overrightarrow{u}\wedge\overrightarrow{v}}
          \underleftrightarrow{\iint_{\mathds{R}^2}e^{-\left(x^2+y^2\right)}\,\mathrm{d}x\mathrm{d}y}
  \end{split}\\
  \rotatebox{30}{\sum_{n=1}^{+\infty}}\quad\mbox{Mirror rorriM}\reflectbox{\mbox{Mirror rorriM}}
\end{array}
)", render_mode = "path")
```

Both are ordinary LaTeX, pasted unchanged. The rest of this vignette is
about placing such things, choosing fonts, and knowing where the
supported set ends.


### Aligning to the math baseline

`hjust` and `vjust` also take names. The useful one is
`vjust = "baseline"`, which puts the formula's *math baseline* --- not
the bounding-box centre --- on the anchor, so a formula sits beside
running text the way it would in a typeset document.

```{r baseline-align, fig.height=1.1, fig.width=6, out.width="70%"}
grid.newpage()
y <- 0.5
grid.segments(unit(0, "npc"), unit(y, "npc"),
              unit(1, "npc"), unit(y, "npc"), gp = gpar(col = "grey80"))
grid.text("if ", x = 0.10, y = y, just = c(0, 0.5), gp = gpar(fontsize = 20))
grid.latex(r"($x \geq \sqrt{2\pi}$)",
           x = 0.22, y = y, hjust = "left", vjust = "baseline",
           gp = gpar(fontsize = 20))
grid.text(", then proceed.", x = 0.62, y = y, just = c(0, 0.5),
          gp = gpar(fontsize = 20))
```

`hjust` accepts `"left"`/`"bbleft"`, `"center"`/`"centre"`/`"middle"`/
`"bbcentre"`, and `"right"`/`"bbright"`. `vjust` accepts `"bottom"`,
`"center"`/`"centre"`/`"middle"`, `"top"`, and `"baseline"`.

### Named anchors with `\mark{}`

`\mark{name}` records a named anchor inside the formula, and `grobMark()`
resolves it to a pair of grid units ready to drive an arrow or a callout.
Marks work at any nesting level --- even inside a superscript or a
fraction --- and inherit the surrounding transform (font shrink, scaling,
rotation), so the anchor lands on the rendered glyph.

```{r mark, fig.height=2.4, fig.width=5, out.width="70%"}
g <- latex_grob(r"($a^2 + b\mark{term}^2 \mark{equals}= c^2$)",
                x = 0.5, y = 0.4)
grid.newpage()
grid.draw(g)

# The "=" sign, pointed at from above.
mk_eq <- grobMark(g, "equals")
grid.segments(mk_eq$x, mk_eq$y + unit(15, "mm"),
              mk_eq$x, mk_eq$y + unit(3, "mm"),
              arrow = arrow(length = unit(2, "mm"), type = "closed"),
              gp = gpar(col = "red"))
grid.text("equals", x = mk_eq$x, y = mk_eq$y + unit(18, "mm"),
          gp = gpar(col = "red"))

# The b^2 term, from below -- the mark sits at the end of the term,
# including the superscript's smaller scale.
mk_bsq <- grobMark(g, "term")
grid.segments(mk_bsq$x - unit(6, "mm"), mk_bsq$y - unit(15, "mm"),
              mk_bsq$x - unit(2, "mm"), mk_bsq$y - unit(3, "mm"),
              arrow = arrow(length = unit(2, "mm"), type = "closed"),
              gp = gpar(col = "blue"))
grid.text("b² term", x = mk_bsq$x - unit(7, "mm"),
          y = mk_bsq$y - unit(18, "mm"), just = "right",
          gp = gpar(col = "blue"))
```

The returned units carry the grob's viewport position and `hjust`/`vjust`,
so they go straight into any grid drawing function with no offset
arithmetic. A mark is a single point, not a span: to centre a callout over
a multi-glyph term, use a pair (`\mark{l}…\mark{r}`) and take the midpoint.

## Display and inline style

TeX sets the same expression two ways. In *display* style a `\sum` is
drawn at full size with its limits above and below; in *text* (inline)
style it shrinks and the limits become scripts beside it. This is a
choice about spacing and layout, not about size --- size is always
`gp$fontsize`.

By default the delimiters decide, exactly as in a LaTeX document: `$…$`
gives text style and `$$…$$` gives display style. A label written without
delimiters --- a plot title, an axis title, an `element_latex()` --- has
nothing for the parser to go on, so it gets text style. `tex_style`
overrides that for the whole expression, and takes all four of TeX's
styles:

```{r style-options, fig.height=1.9, fig.width=6.5, out.width="100%"}
sum_expr <- r"(\sum_{i=1}^{n} \frac{x_i}{n})"
styles <- c("display", "text", "script", "scriptscript")
labels <- c('"display"  ($$...$$)', '"text"  ($...$)',
            '"script"', '"scriptscript"')

grid.newpage()
for (i in seq_along(styles)) {
  pushViewport(viewport(x = (i - 0.5) / 4, width = 1 / 4))
  grid.text(labels[i], y = 0.88, gp = gpar(cex = 0.75, fontface = "bold"))
  grid.latex(sum_expr, y = 0.42, input_mode = "math",
             tex_style = styles[i], gp = gpar(fontsize = 20))
  grid.rect(gp = gpar(col = "grey85", fill = NA))
  popViewport()
}
```

Every panel is `gp = gpar(fontsize = 20)`. The display version is taller
and narrower because the limits moved above and below the operator
instead of sitting beside it --- worth knowing when you are budgeting
space for a label, and `latex_dims()` reports the difference if you need
the numbers. `"script"` and `"scriptscript"` are the sizes TeX uses for
first- and second-level scripts; they are occasionally handy for a
compact sub-label, and rarely needed otherwise.

To change style for part of an expression rather than all of it, use the
inline commands `\displaystyle`, `\textstyle`, `\scriptstyle` and
`\scriptscriptstyle` instead. `latex_options(tex_style = )` sets a
session default.

## Wrapping and justification

`max_width`, in big points, wraps a label over as many lines as it needs.

```{r wrap, fig.height = 2, fig.width = 4, out.width = "65%"}
prose <- paste(rep(
  r"(The quick brown fox jumps over the lazy dog, and $x^2$ too.)", 3),
  collapse = " ")

grid.newpage()
pushViewport(viewport(layout = grid.layout(2, 1)))
pushViewport(viewport(layout.pos.row = 1))
grid.text("ragged (default)", x = 0.02, y = 0.98, hjust = 0, vjust = 1,
          gp = gpar(col = "grey40"))
grid.latex(prose, x = 0.02, y = 0.78, hjust = 0, vjust = 1,
           max_width = 3.6 * 72, gp = gpar(fontsize = 11))
popViewport()
pushViewport(viewport(layout.pos.row = 2))
grid.text("justified + optimal", x = 0.02, y = 0.98, hjust = 0, vjust = 1,
          gp = gpar(col = "grey40"))
grid.latex(prose, x = 0.02, y = 0.78, hjust = 0, vjust = 1,
           max_width = 3.6 * 72, justify = TRUE, line_break = "optimal",
           gp = gpar(fontsize = 11))
popViewport(2)
```

Both refinements are off by default. `justify = TRUE` fills every line but
the last; `line_break = "optimal"` chooses the breaks by total fit rather
than one line at a time. Justifying a narrow column opens wide word gaps,
since nothing may break inside a word --- mark the words that may with
`\-`, as in `in\-ter\-na\-tion\-al`.

## Including a figure

`\includegraphics` draws an image inside the formula. PNG, JPEG and SVG
are supported, sized with `width`, `height` or `scale` in any LaTeX
length; give one of width/height and the aspect ratio is kept, or both
plus `keepaspectratio` to fit inside them. As in LaTeX the extension may
be omitted (`{plots/fig}` finds `plots/fig.svg` or `plots/fig.png`), and
`\graphicspath{{figs/}}` adds a directory to search. `angle` rotates the
figure, like `\rotatebox`; `trim` and `clip` are recognised but not
applied, and say so.

```{r img-basic, fig.height = 1.6, fig.width = 4, out.width = "60%"}
fig <- tempfile(fileext = ".svg")
svglite::svglite(fig, width = 2, height = 1.2)
grid.newpage()
grid.circle(r = 0.35, gp = gpar(fill = "steelblue", col = NA))
dev.off()

grid.newpage()
grid.latex(sprintf(r"(\text{before }\includegraphics[width=1in]{%s}\text{ after})", fig),
           input_mode = "math", gp = gpar(fontsize = 20))
```

Inline, the image sits **on the baseline**, as in LaTeX --- `\raisebox`
moves it. A paragraph holding only an image is a **block**, flush left
unless `halign` or a `text-align` rule on `img` says otherwise.

```{r img-raise, fig.height = 1.5, fig.width = 4.5, out.width = "70%"}
icon <- sprintf(r"(\includegraphics[width=14bp]{%s})", fig)
grid.newpage()
grid.latex(sprintf(r"(\text{baseline }%s\text{, raised }\raisebox{4bp}{%s})",
                   icon, icon),
           input_mode = "math", gp = gpar(fontsize = 20))
```

### A caption

`\caption` renders where it is written. A caption after `\includegraphics`
therefore sits below the figure, and one written before a `tabular` sits
above the table --- which is what makes raw `xtable` and `kable` output
come out the right way round without any float logic.

`\centering` is dropped, because a grob has no page to centre against. A
one-column `array` does the job instead: it centres the figure and its
caption on each other.

```{r img-caption, fig.height = 1.4, fig.width = 3, out.width = "45%"}
logo <- system.file("img", "Rlogo.png", package = "png")
grid.newpage()
grid.latex(sprintf(r"(\begin{array}{c}
  \includegraphics[width=0.6in]{%s}\\
  \caption{Figure 1: the R logo}
\end{array})", logo), gp = gpar(fontsize = 11))
```

Remove the `array` and the same two lines come out flush left. LaTeX
numbers captions from a counter; here the number is yours to write.

**Prefer SVG.** An SVG is drawn as real grid primitives, so it stays sharp
at any output resolution — there is no dpi to choose. A PNG or JPEG is a
fixed grid of pixels, so if you display one much larger than it was saved
you get a blurry result; the package warns when the effective resolution
falls below 150 dpi and tells you what pixel size would reach 300. The
rule is `pixels = display_inches * output_dpi`.

PDF and EPS are **not** supported. Save the figure as SVG instead: it needs
no external tool and, unlike a rasterised PDF, its text stays text.


## Fonts

Two layers are in play. **MicroTeX** chooses the math glyphs and their
metrics; **grid** draws everything inside `\text{}` using `gp$fontfamily`,
so prose follows R's ordinary font handling --- Latin, CJK, Cyrillic and
anything else the device supports.

Two math fonts ship with the package and load automatically:

| Alias | Font | Style | Pairs with |
|-------|------|-------|------------|
| `"lete"` (default) | Lete Sans Math | Sans-serif | `fontfamily = "sans"` |
| `"stix"` | STIX Two Math | Serif | `fontfamily = "serif"` |

```{r fonts-list}
available_math_fonts()
```

Set one per call with `math_font`, or for the session with
`latex_options(math_font = )`. Both rows below are the same formula ---
only the math font and its paired text family differ:

```{r fonts, fig.height=1.5, fig.width=3.4, out.width="60%"}
formula <- r"(Theorem: $\int_0^1 f(x)\,dx \geq 0$)"

grid.newpage()
pushViewport(viewport(layout = grid.layout(2, 1)))
pushViewport(viewport(layout.pos.row = 1))
grid.latex(formula, gp = gpar(fontfamily = "sans"))
upViewport()
pushViewport(viewport(layout.pos.row = 2))
grid.latex(formula, math_font = "stix",
           gp = gpar(fontfamily = "serif"))
upViewport(2)
```

`check_math_fonts()` gives a diagnostic report. Any font available to R
works for the text half --- base families like `"sans"`, `"serif"` and
`"mono"`, or anything registered through **systemfonts**:

```{r cjk, fig.height=1, fig.width=5, out.width="55%", dev = "ragg_png", dev.args = list(), purl = FALSE}
grid.newpage()
grid.latex(r"(如果 $x > 0$ 则 $y = x^2$)",
           gp = gpar(fontfamily = "sans"))
```

Right-to-left scripts work the same way, mixed with Latin included. Two
things do not: wrapping a right-to-left paragraph across lines needs the
optional [FriBidi](https://github.com/fribidi/fribidi) library, and a
math atom between two right-to-left runs --- `\text{…}$x^2$\text{…}` ---
falls back to left-to-right order.

### Naming a font for one run

`gp$fontfamily` applies to the whole grob. For a *single run*, use
`\gmfontfamily{family}{content}` --- a gridmicrotex extension, not
standard LaTeX. `\textrm{…}` goes the other way, returning content to
`gp$fontfamily` even inside a `\textsf{…}`, `\texttt{…}` or
`\gmfontfamily{…}{…}` group, and without disturbing bold or italic:

```{r gmfontfamily, fig.height=0.8, fig.width=4.5, out.width="85%", dev = "ragg_png", dev.args = list()}
grid.newpage()
grid.latex(
  r"(\textsf{sans \textrm{body} sans} \quad \gmfontfamily{mono}{mono})",
  gp = gpar(fontfamily = "serif")
)
```

`family` is anything `gp$fontfamily` accepts: a generic (`"sans"`,
`"serif"`, `"mono"`) or a specific name such as `"Georgia"`, which
`\textsf{…}` / `\texttt{…}` cannot express. Unresolvable names fall back
silently. The content is typeset as text, so this styles prose, not math
--- math glyphs follow `math_font`. It composes with emphasis in either
nesting order, but a nested `\gmfontfamily` *replaces* the family it sits
inside.

The name is deliberately not `\fontfamily`: LaTeX's takes one argument,
does nothing until `\selectfont`, and wants an NFSS code (`ptm`) rather
than a font name. Since gridmicrotex accepts pasted LaTeX, claiming that
name would silently misparse real input. This is also what markdown's
`font-family` CSS compiles to --- see `vignette("markdown")`.

### Loading a custom math font

`load_math_font()` adds any OpenType math font. The OpenType MATH table
is parsed directly in C++, so no companion metrics file and no external
toolchain are needed:

```r
load_math_font("path/to/MyFont.otf")
```

This is only for **math** fonts. Text fonts need no loading at all --- set
`gp$fontfamily`, or name one for a run with `\gmfontfamily{}{}`.

## Devices and render modes

- **`"typeface"`** (default) draws glyphs as native text, so PDF and SVG
  output stays selectable and searchable: a line of prose is emitted as a
  single element, so a viewer finds a phrase and not merely a word. Text
  given a `max_width` is the exception --- it is emitted one word per
  element, because the spaces are where the lines break. Fonts are read
  straight from their OTF files,
  with no system-wide install --- but this needs a device with the R
  \eqn{\geq} 4.3 glyph engine (`ragg`, `svglite`, `cairo_pdf`). On
  others, such as the base `pdf()` device, it falls back to path mode
  with a warning.
- **`"path"`** draws each glyph as a filled vector path. Works on every
  device; the text is not selectable, and files are larger.

```r
grid.latex(r"($E = mc^2$)")                        # typeface
grid.latex(r"($E = mc^2$)", gp = gpar(fontsize = 24), render_mode = "path")  # path
```

Prefer `ragg::agg_png()`, `svglite::svglite()` or `grDevices::cairo_pdf()`.
The default devices on Windows and macOS may not find the bundled math
fonts and will warn `font family not found in Windows font database`; the
README covers the setup, and compares the package with `tikzDevice`,
`xdvir`, `latex2exp` and `plotmath`.

> **Do not use `showtext::showtext_auto()` with typeface mode.** showtext
> intercepts all text rendering and converts it to paths, silently
> defeating typeface mode even on `svglite` and `ragg`. Call
> `showtext::showtext_auto(FALSE)` before drawing formulas.

## Utilities

### Measuring

`latex_dims()` returns the bounding box of an expression, for layout
arithmetic and for checking that a label fits:

```{r dims}
latex_dims(r"(\frac{a}{b})", gp = gpar(fontsize = 20))
```

### Session defaults

`latex_options()` sets `math_font`, `render_mode` and `input_mode` for
calls that don't supply them; explicit arguments always win. Size stays at
the grob level, via `gp$fontsize` / `gp$lineheight`.

```{r options, eval=FALSE}
latex_options(math_font = "stix", render_mode = "typeface")
latex_options()        # query
reset_latex_options()  # back to built-in defaults
```

### User-defined macros

`define_macro()` registers zero-argument shorthands, expanded by text
substitution before the expression reaches MicroTeX:

```{r macros, fig.height=0.7, fig.width=3, out.width="50%"}
define_macro("RR", r"(\mathbb{R})")
define_macro("eps", r"(\varepsilon)")

grid.newpage()
grid.latex(r"(\forall \eps > 0, \eps \in \RR)")

clear_macros()
```

Names must be ASCII letters, and expansion iterates to a fixed point so
macros can reference each other. `list_macros()` shows what is registered;
`clear_macros()` drops everything.

For parameterised macros (0--9 arguments) MicroTeX also accepts plain-TeX
`\def`. These live only for the expression they appear in, so use them for
an abbreviation local to one label and `define_macro()` for one that
should persist:

```{r def-inline, fig.height=0.7, fig.width=4, out.width="60%"}
grid.newpage()
grid.latex(
  r"(\def\norm#1{\left\lVert #1 \right\rVert}
      \norm{\vec{v}} = \sqrt{\langle \vec{v}, \vec{v} \rangle})"
)
```

### Caching and introspection

Parsed layouts are memoised by `(tex, fontsize, math_font, render_mode,
…)`, so a repeated axis label is laid out once:

```{r cache, eval=FALSE}
latex_cache_info()       # size / max_size / hits / misses
latex_cache_limit(1024)  # LRU capacity; 0 disables caching
latex_cache_clear()      # wipe (e.g. after re-loading fonts)
```

`latex_tree()` returns the raw draw records plus bbox metadata, and
`debug = TRUE` overlays the bounding box, baseline and record origins ---
both useful when checking alignment:

```{r debug, fig.height=1, fig.width=3, out.width="60%"}
grid.newpage()
grid.latex(r"($x^{2} + y_{i}$)", debug = TRUE)
```

## LaTeX reference

MicroTeX is a **math formula renderer**, not a document typesetter. It
covers the vast majority of notation used in plots and figures, but does
not replace a LaTeX installation. This section is the boundary.

### Lists

`itemize` and `enumerate` lay their items out as a left-aligned column,
one per row --- `itemize` prefixes a bullet, `enumerate` numbers them:

```{r lists, fig.height=1.4, fig.width=3, out.width="45%"}
grid.newpage()
grid.latex(r"(\begin{enumerate}
  \item e^{i\pi} + 1 = 0
  \item \begin{itemize}
          \item \alpha \item \beta
        \end{itemize}
\end{enumerate})", gp = gpar(fontsize = 20))
```

An optional `[…]` argument customises the marker. For `itemize` it is the
literal marker (`\begin{itemize}[\star]`); for `enumerate` it is a counter
template containing one of `\arabic*`, `\alph*`, `\Alph*`, `\roman*` or
`\Roman*` (e.g. `\begin{enumerate}[\Roman*.]`). Lists nest, and an item
may contain any math, including a `\begin{array}` table.

Because MicroTeX is a math engine, each item is a **math-mode,
single-line** expression: no paragraph flow, no line wrapping, and prose
inside an item needs `\text{}` (`\item \text{First point}`). The
`description` environment is not supported.

### Pasting LaTeX from other sources

Input generated by other tools --- ready-to-compile `tabular` snippets,
fragments copied out of a `.tex` file --- usually arrives wrapped in
document-level constructs MicroTeX does not implement. Rather than
refusing it, gridmicrotex rewrites or removes a small set of well-known
wrappers before parsing, so `knitr::kable(format = "latex")` and
`xtable::print.xtable()` output can be pasted in unedited:

```{r pasted, fig.height = 1.1, fig.width = 3.2, out.width = "55%"}
snippet <- r"(
% latex table generated by kable()
\begin{table}[ht]
\centering
\caption{Model coefficients}
\begin{tabular}{lrr}
\toprule
Term & Estimate & \emph{p} \\
\midrule
Intercept & 2.14 & 0.003 \\
Slope & 0.42 & 0.001 \\
\bottomrule
\end{tabular}
\end{table}
)"

grid.newpage()
grid.latex(snippet, input_mode = "mixed", gp = gpar(fontsize = 11))
```

Six things were handled without any editing: the `%` comment and the
`table` float were dropped, `\centering` removed, `\caption` set as a line
above the table, `\toprule`/`\bottomrule` became thick rules, `\midrule` a
plain one, and `\emph` became italic.

Note that the cell text is set in math italics. A `tabular` is a math
environment, so its contents are math whichever `input_mode` you choose ---
the two modes give an identical layout for the snippet above. What
`"mixed"` buys you here is the prose *outside* the environment, such as the
caption. For upright cell text, wrap the cells in `\text{}`.

**Removed silently (no visual effect):**

| Construct | Why |
|---|---|
| `%`-to-end-of-line comments (`\%` is preserved) | comments are non-visual in LaTeX too |
| `\documentclass[…]{…}`, `\usepackage[…]{…}` | preamble metadata |
| `\begin{document}` / `\end{document}` | document boundary, structural only |
| `\maketitle`, `\title{…}`, `\author{…}` | title-page metadata, no body output |
| `\label{…}` | cross-reference target, never rendered in LaTeX either |
| `\begin{table}[…]` / `\end{table}`, `\begin{figure}[…]` / `\end{figure}` (and starred variants) | float wrappers; the contents stay |
| `\centering`, `\raggedright`, `\raggedleft`, `\flushleft`, `\flushright` | alignment scope declarations |
| `\noindent`, `\relax` | content-free declarations |

**Rewritten to a MicroTeX equivalent:**

| Construct | Becomes |
|---|---|
| `\emph{X}` | `\textit{X}` |
| `\textnormal{X}` | `\text{X}` |
| `\par`, `\newline` | `\\` (line break) |
| `\toprule`, `\bottomrule` | `\thickhline` (rendered ~2× thickness) |
| `\midrule` | `\hline` |
| `\cmidrule[trim]?(parenarg)?{a-b}` | `\cline{a-b}` --- partial-column rule |
| `\caption[short]{X}` | `\text{X}` plus a line break, at its source position --- so a caption written above the table stays above it |
| `\smallskip`, `\medskip`, `\bigskip` | `\vspace{0.25em}` / `\vspace{0.5em}` / `\vspace{1em}` --- em-relative so they scale with `gp$fontsize` |
| `\hfill`, `\vfill` | `\quad` / `\vspace{1em}` --- static proxies for rubber lengths |
| `\url{X}` | coloured monospace text --- a grob cannot be a hyperlink, so only the appearance survives |
| `\href{U}{X}` | `X`, coloured. Matches `hyperref` with `colorlinks=true`; markdown links use HTML's blue-and-underlined convention instead |

Three of those are approximations. `\caption` renders where it appears in
the source, not where LaTeX's float machinery would move it, so
caption-above or caption-below follows whatever your tool emits. The skips
are em-relative rather than LaTeX's absolute 3/6/12 pt, so they stay
visible at any `gp$fontsize`; use `\vspace{Xpt}` for an exact amount. And
`\hfill` / `\vfill` are *rubber* lengths with nothing to fill in a
fixed-size grob, so they become a static 1 em gap --- right position, no
elasticity.

**Not honored** --- rendered as literal text, which is intentional: it
makes unsupported markup easy to spot.

- Declarative font scopes: `\bfseries`, `\itshape`, `\ttfamily`,
  `\sffamily`, `\rmfamily`. These affect text within their group in LaTeX,
  which needs scope tracking we do not implement. Use the
  argument-bearing forms instead --- `\textbf{…}`, `\textit{…}`,
  `\texttt{…}`, `\textsf{…}`, `\textrm{…}` --- all of which nest. To
  choose the text font itself, set `gp$fontfamily` or use
  `\gmfontfamily{…}{…}`.
- References: `\ref{…}`, `\cite{…}` --- there is nothing to resolve
  against.
- Footnotes: `\footnote{…}` --- the positioning machinery is page-bound.
- Small caps: `\textsc{…}` --- MicroTeX has no small-caps glyphs.

### What is not supported

These need a real document compiler and are outside a formula renderer's
scope: document structure (`\section`, page layout, `\tableofcontents`);
automatic hyphenation (`\-` marks a break point yourself, and
`max_width` / `justify` do handle line breaking); TikZ/PGF;
cross-references and bibliographies; theorem
environments; the `description` list environment (`itemize` and
`enumerate` *are* supported); and `\tag` / equation numbering.
`\usepackage{…}` is accepted but loads nothing --- every supported command
is built into MicroTeX.

For axis labels, annotations, legends and in-plot formulas, the supported
set is more than sufficient.

---
title: "Rendering Markdown with Math"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Rendering Markdown with Math}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dpi = 300,
  dev = "png"
)
library(gridmicrotex)
library(grid)
```

Plot labels are rarely pure math. A title might be a bold phrase with a
symbol in it; a caption might be a short paragraph that happens to
contain an equation. `gridmicrotex` renders
[CommonMark](https://commonmark.org) markdown *and* LaTeX math together,
in one grob, with no external LaTeX installation.

**These functions are a wrapper around `latex_grob()`** --- your markdown
is turned into LaTeX and drawn by the same engine. So everything
`latex_grob()` renders works here too, with the same fonts, the same
options and the same limits (see `vignette("getting-started")`), and this
vignette covers only what markdown adds on top.

There are two entry points:

- **`markdown_grob()`** (and `grid.markdown()`) for a single flowing
  run of text --- the right choice for a title or an axis label.
- **`markdown_box_grob()`** for a whole block document --- headings,
  lists, block quotes, code, tables --- inside an optional padded,
  filled box.

## Inline markdown

`markdown_grob()` handles the inline part of markdown --- `**bold**`,
`*italic*`, `` `code` ``, `~~strikethrough~~` --- and any `$math$`.

```{r inline, fig.height = 0.8, fig.width = 6, out.width = "90%"}
grid.newpage()
grid.markdown(
  r"(The **fitted** $\hat{\beta} = (X^\top X)^{-1} X^\top y$ has ~~no~~ `se` = *0.42*.)",
  x = 0.02, hjust = 0, gp = gpar(fontsize = 13)
)
```

### Colour and inline HTML

Markdown has no syntax for colour, underline, super/subscript, highlight
or size. Use inline HTML; each tag does what it does in a browser:

| tag | effect |
|---|---|
| `<b>`, `<strong>` | bold |
| `<i>`, `<em>`, `<cite>`, `<dfn>`, `<var>` | italic |
| `<code>`, `<kbd>`, `<samp>`, `<tt>` | monospace |
| `<u>`, `<ins>` | underline |
| `<s>`, `<del>`, `<strike>` | strikethrough |
| `<sub>`, `<sup>` | sub / superscript |
| `<mark>` | yellow highlight |
| `<small>`, `<big>` | smaller / larger |
| `<q>` | wrapped in quotation marks |
| `<ruby>`, `<rt>` | annotation above the base (furigana) |
| `&nbsp;` | non-breaking space |
| `<br>` | line break |
| `<span style="…">` | any property from `?md_style` |

`<ruby>` is worth singling out: `<ruby>漢<rt>かん</rt></ruby>` sets the
annotation above its base with `\overset`, which is how furigana and
bopomofo are typeset. Neither marquee nor gridtext can do this.

In a `style` attribute: `color` takes any R colour name, the nine CSS
names R happens to lack (`crimson`, `teal`, `rebeccapurple` and friends),
`#rgb`, `#rrggbb` or `rgb()` --- but note that `green`, `gray`, `grey`,
`maroon` and `purple` keep their R values rather than their (darker) CSS
ones, so that they mean the same thing here as in `gpar(col = ...)`;
`text-decoration` takes `underline` or
`line-through`; `font-size` takes `pt`, `px`, `in`, `cm`, `mm`, `em`,
`rem`, `%`, `smaller` or `larger`; `font-family` takes the CSS generics
`monospace`, `sans-serif` and `serif`, or any font name, including via a
fallback list like `'Courier New', monospace`. Any other property is
ignored.

One figure with most of the table in it --- a sized and coloured span,
two font families, a subscript, an underline, a highlight, and the blank
line that two `<br>` leave behind:

```{r html-css, fig.height = 1.3, fig.width = 6, out.width = "90%", dev = "ragg_png", dev.args = list()}
grid.newpage()
grid.markdown(
  r"(<span style="font-size:22pt;color:#2E5E8E">Model fit</span>,
     <span style="font-family:serif">serif</span> and
     <span style="font-family:monospace">mono</span>.<br><br>
     The <span style="color:#B22222">**residual**</span> for H<sub>0</sub>
     is <u>within</u> <mark>$2\sigma$</mark>.)",
  x = 0.02, y = 0.9, hjust = 0, vjust = 1, gp = gpar(fontsize = 13)
)
```

Tags nest, and markdown and `$math$` keep working inside them, so
`<u>**bold underlined**</u>` and `**<u>the same thing</u>**` render alike.
Any other tag is dropped and its text kept --- as a browser does with
`<a>` or a bare `<span>`, which have no rendering of their own.

## Block documents

`markdown_box_grob()` lays out a full markdown document. Each block ---
heading, paragraph, list, quote, code, table --- becomes its own grob,
stacked top to bottom. The `width` you give is the box width; prose
wraps to fit it.

```{r block, fig.height = 4, fig.width = 4.6, out.width = "80%"}
md <- r"(
# Model summary

The slope is $\beta_1$ with *p* < 0.001, and this paragraph is long
enough that it wraps inside the column.

## Diagnostics

- residuals look **fine**
- $R^2 = 0.87$
- no influential points

~~~r
fit <- lm(y ~ x, data = d)  # refit
if (anyNA(d)) {
    stop("missing values")
}
~~~

> Assumptions were checked and hold.
)"

grid.newpage()
grid.draw(markdown_box_grob(
  md,
  width   = unit(4.6, "in"),
  padding = unit(10, "pt"),
  box_gp  = gpar(fill = "grey97", col = "grey40"),
  gp      = gpar(fontsize = 13),
  style = markdown_style(css = "pre { background: #ece2f0 }") # Code background
))
```

A fenced block that names a language is syntax-highlighted, and its
indentation is kept. (The example above fences with `~~~r` rather than
three backticks only because it lives inside an R chunk in this
vignette, where a backtick fence would end the chunk; both spellings are
standard CommonMark.) `available_highlighters()` lists what is built in
--- R, Python, SQL, shell, C++, YAML, JSON, Stan, Julia and LaTeX ---
and the usual GitHub aliases (`py`, `sh`, `c++`, `yml`, `jl`, `tex`)
work too. A fence naming anything else is set as plain monospace.

Colours are ordinary CSS, using the same class names knitr writes into
HTML output, so a Pandoc syntax theme can be pasted straight in:

```r
markdown_style(css = ".co { color: #59636E } .kw { color: #CF222E }")
```

Those are ordinary class selectors, and they are global: seventeen
two-letter names (`co`, `st`, `kw`, `cf`, `dv`, `fl`, `cn`, `fu`, `dt`,
`bu` and friends) carry a default colour. If you use one of them as your
own class in prose --- `<div class="dt">` --- it picks up the syntax
colour, and a tag rule cannot override it, because a class beats a tag
in CSS just as it does in a browser. Pick another name for your own
classes, or restyle the one you want.

`register_highlighter()` adds a language from a KDE syntax XML file ---
either one of your own (copy a bundled grammar from
`system.file("highlight", package = "gridmicrotex")` as a template) or
one downloaded from <https://kate-editor.org/syntax/>, most of which
work as they are.

Set `box_gp = NULL` to draw no box, or give the `r` argument a value
such as `unit(6, "pt")` for rounded corners. `padding` and `margin`
accept a single unit or a length-4 unit giving top, right, bottom, left.

### Lists, tasks, and tables

Ordered and unordered lists, GFM task lists, and tables all render.
Markdown tables become real typeset tables, with column alignment taken
from the `|:---:|` markers --- something a text renderer cannot do.

```{r lists-tables, fig.height = 2, fig.width = 4.4, out.width = "80%"}
md <- r"(
### Checklist

- [x] fit the model
- [ ] write it up

Coefficients:

| term | $\beta$  | *p* |
|:-----|------------:|:---:|
| intercept | 30.1 | *** |
| slope | -5.3 | *** |
)"

grid.newpage()
grid.draw(markdown_box_grob(
  md,
  width   = unit(4.4, "in"),
  padding = unit(8, "pt"),
  gp      = gpar(fontsize = 13)
))
```

### Images

An image on its own line is drawn as a raster, scaled to the column
width. `png` and `jpeg` are optional --- if neither is installed, or the
file is missing, the image degrades to its alt text.

```{r image, fig.height = 1.8, fig.width = 4, out.width = "70%"}
# Any .png / .jpeg path works; this one ships with the png package.
img <- system.file("img", "Rlogo.png", package = "png")

md <- sprintf("
A figure with a caption below it.

![the R logo](%s)

The caption explains the figure.
", img)

grid.newpage()
grid.draw(markdown_box_grob(
  md, width = unit(4, "in"), padding = unit(8, "pt"),
  gp = gpar(fontsize = 13)
))
```

## Styling

Everything above uses the built-in look. To change it, hand
`markdown_box_grob()` a `style`. It is a small CSS cascade over the
markdown tags, and it takes CSS directly --- as text, or as the path to
a `.css` file:

```{r style-css, fig.height = 1.2, fig.width = 5, out.width = "85%"}
doc <- r"(
# Results

The slope is $\beta_1$ with *p* < 0.001.

> Worth a second look.
)"

grid.newpage()
grid.draw(markdown_box_grob(
  doc,
  width = unit(4.5, "in"), padding = unit(10, "pt"),
  style = "
    h1         { color: steelblue; font-size: 1.8rem }
    blockquote { color: grey40; border-left: 3px solid steelblue }
  ",
  gp = gpar(fontsize = 13)
))
```

The same thing written in R, which validates property names and
autocompletes:

```{r style-r, eval = FALSE}
markdown_style(
  h1         = md_style(color = "steelblue", font_size = 1.8),
  blockquote = md_style(color = "grey40",
                        border_left = "3px solid steelblue")
)
```

Lengths take three forms: a bare number is `rem`, a multiple of the body
font size; a string is whatever CSS says (`"1.8rem"`, `"12pt"`, `"150%"`);
and a `grid::unit()` is absolute.

Tags are named as in HTML --- `body`, `p`, `h1`--`h6`, `ul`, `ol`, `li`,
`blockquote`, `pre`, `code`, `strong`, `em`, `a`, `table`, `tr`, `td`,
`th`, `hr`, `img`, `math`, `footnote`, `div`, `span`. The properties
are colour, the font family, size, weight and slant, decorations, line
height, margins, left padding, alignment and the two borders. `?md_style` has the full
table, including which of them work on an inline `<span>` as well as on
a block. Anything outside it parses and is then ignored, the way a
browser ignores what it does not implement --- so an existing stylesheet
can be pasted in and the parts that apply still work.

### Starting from a preset

`markdown_style("github")` is a GitHub-like look, shipped as a CSS file
you can read and copy:

```{r style-preset, eval = FALSE}
markdown_style("github", h1 = md_style(color = "firebrick"))
file.show(system.file("css", "github.css", package = "gridmicrotex"))
```

Pass an existing style as the first argument to extend it, and use
`latex_options(markdown_style = ...)` to set one for a whole document.

### Styling one chunk

A style set says how *every* heading looks. To style one particular
chunk, wrap it in a `<div>` with a `class` or a `style`. **Leave blank
lines around the tags** --- that is what makes CommonMark parse the
markdown between them rather than treating the whole block as raw HTML:

```{r style-div, fig.height = 1, fig.width = 5, out.width = "85%"}
doc <- '
Ordinary text.

<div class="note">

**Note.** This chunk is indented and set apart.

</div>

Ordinary text again.
'

grid.newpage()
grid.draw(markdown_box_grob(
  doc,
  width = unit(4.5, "in"), padding = unit(10, "pt"),
  style = ".note { color: grey35; padding-left: 1.5rem }",
  gp = gpar(fontsize = 13)
))
```

Inheritable properties --- colour and the `font-*` family --- fall
through to everything inside the div, exactly as in CSS; margins,
padding and borders do not. `<span class="...">` works the same way for
an inline run.

### Styling tables

Tables take row and cell fills, rule colour and weight, and column
spacing. The tags nest as
in HTML: `tr`, `td` and `th` inherit through `table`.

| CSS | effect |
|---|---|
| `table { border-color }` | colour of every rule |
| `tr { background }` | fills a whole row |
| `td`, `th ` `{ background }` | fills one cell |
| `tr { border-bottom }` | a rule under each row |
| `td { border-left }` | vertical rules between columns |
| `td { padding-left }` | the gap between columns |
| `table { table-layout: fixed }` | see below |

Header cells are **bold** by default, which needs saying because it could
not be done before: `\text{}` resets the font style, so wrapping a
finished table in `\textbf{}` has no effect. The emphasis is applied as
each cell is generated instead.

By default a table sizes to its content, so a wide one overflows the box.
`table-layout: fixed` divides the available width between the columns and
lets the cells wrap:

```{r style-table, fig.height = 2, fig.width = 5, out.width = "85%"}
tbl <- r"(
| Term | Meaning |
|:-----|:--------|
| $\beta_1$ | the slope, which needs a good deal of room to explain |
| $\sigma$ | the residual standard deviation |
)"

grid.newpage()
grid.draw(markdown_box_grob(
  tbl,
  width = unit(4, "in"), padding = unit(8, "pt"),
  style = "
    table { table-layout: fixed; border-color: #D1D9E0 }
    th    { background: #F6F8FA }
    tr    { border-bottom: 1px solid #D1D9E0 }
  ",
  gp = gpar(fontsize = 13)
))
```

## Dropping down to LaTeX

Anything markdown cannot express, write as LaTeX inside a math span.

The clearest case is a table: markdown tables have no spanning cells and
no per-cell colour, and a LaTeX `array` has both.

```{r latex-escape, fig.height = 2.4, fig.width = 4.6, out.width = "80%"}
tbl <- r"($$\begin{array}{|l|c|c|}\hline
\rowcolor{#F6F8FA}\multicolumn{3}{|c|}{\textbf{Model comparison}}\\\hline
\textbf{Model}&\textbf{AIC}&\textbf{R}^2\\\hline
\text{linear}&214.3&0.71\\
\cellcolor{#DDF0DD}\text{quadratic}&\cellcolor{#DDF0DD}201.8&\cellcolor{#DDF0DD}0.87\\\hline
\end{array}$$)"

grid.newpage()
grid.draw(markdown_box_grob(
  paste0("Compare the two fits.\n\n", tbl, "\n\nThe quadratic wins."),
  width = unit(4.6, "in"), padding = unit(10, "pt"),
  style = "math { color: #1F3864; font-size: 1.1rem }",
  gp = gpar(fontsize = 13)
))
```

That is one `\multicolumn` spanning header, a `\rowcolor` band and three
`\cellcolor` cells --- none of which GFM table syntax can express. The
same route reaches `cases`, `aligned`, `\multirow`, `\newcolumntype` and
everything else in `vignette("getting-started")`.

Note the header cell written `\textbf{R}^2` rather than `\textbf{R^2}`.
Inside `$$…$$` you are already in math mode, so `^` superscripts --- but
`\textbf{}` switches to *text* mode, where `^` is an accent rather than
a superscript operator and `R^2` comes out flat. Put the superscript
outside the text command. There is no need to open `$…$` again either:
delimiters do not nest, and an escaped `\$` would simply typeset a
dollar sign.

**Styling reaches the block, not inside it**, and that figure shows both
halves at once. The `math` rule sets the colour and size of the table's
text but leaves the prose either side black, because those properties
travel on the block's `gp`. The cell fills ignore it completely.

So the cascade cannot see *through* a math span. That `array` is math,
not a markdown table, and `table`, `tr` and `td` rules leave it untouched
--- which is why its colours had to be written in LaTeX rather than CSS.
That is the trade: a GFM table is styled by the cascade but cannot span
cells; a LaTeX `array` spans anything but is styled in LaTeX.

## What is and isn't supported

Full CommonMark plus the GitHub extensions --- tables, strikethrough,
autolinks and task lists --- is parsed, so parsing is never the
limitation.

Nothing errors: anything that cannot be drawn degrades to its text. This
document contains every degradation at once, which is easier to look at
than to read about:

```{r degrade, fig.height = 1.5, fig.width = 4.4, out.width = "80%"}
doc <- r"(A [link](https://example.org) keeps its text, an
![inline diagram](missing.png) becomes its alt text, and an
<unknown>unknown tag</unknown> keeps its content.

Footnotes[^1] land at the foot.

<table><tr><td>an HTML table is dropped</td></tr></table>

[^1]: Like this one.)"

grid.newpage()
grid.draw(markdown_box_grob(
  doc,
  width = unit(4.4, "in"), padding = unit(8, "pt"),
  gp = gpar(fontsize = 13)
))
```

The link is styled but inert, the missing image shows its alt text, the
unknown tag is stripped and its words kept, the footnote is numbered and
moved to the foot, and the HTML table is gone entirely --- no error, no
placeholder. In detail:

- **Inline images** keep their alt text only. A block-level image, alone
  on its line, is drawn.
- **Links** keep their text; the destination is dropped. They are styled
  blue and underlined through the `a` tag, which a stylesheet can
  change.
- **Footnotes** (`[^1]` with a `[^1]:` definition) put a superscript
  marker in place and the note at the foot, after a rule. Only in
  `markdown_box_grob()`; `markdown_grob()` shows the marker and drops
  the note.
- **Display math** --- a paragraph that is nothing but `$$…$$` --- gets
  its own centred line, styleable as the `math` tag. Inline `$…$` is
  unaffected.
- **Small caps** and `font-variant-numeric` are not rendered.
- **Block-level HTML** other than `<div>` is dropped, `<table>` included
  --- use GFM's table syntax.
- **Spanning cells** are not reachable from markdown table syntax. Write
  the table as LaTeX instead; see *Dropping down to LaTeX*.
- **Syntax highlighting** refuses a downloaded grammar that uses KDE's
  *dynamic* rules rather than colouring it approximately; thirteen of
  twenty sampled definitions loaded.
- **Code comes out slightly tight** on `png(type = "cairo")` and
  `svglite`, which report sans widths for the `"mono"` family. Use
  `ragg::agg_png()` if that matters.

Everything else --- emphasis, headings, lists, quotes, tables, rules,
block images, footnotes, ruby annotation, and of course `$math$` ---
renders.

---
title: "Introduction to gridmicrotex"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to gridmicrotex}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dpi = 300,
  dev = "ragg_png"
)

```

## What is gridmicrotex?

**gridmicrotex** renders LaTeX math equations as native R `grid` graphics
objects (grobs). It uses the [MicroTeX](https://github.com/NanoMichael/MicroTeX)
C++ library as its layout engine --- MicroTeX parses LaTeX, builds the TeX
box model, and computes exact glyph coordinates. The package intercepts
this layout data and maps it to native grid primitives (`pathGrob`,
`segmentsGrob`, `rectGrob`, `textGrob`), producing a `gTree` that works
on any R graphics device at any resolution.

**Key features:**


- No external LaTeX installation required --- MicroTeX is fully embedded
- Resolution-independent vector output on all R devices (PNG, PDF, SVG, ...)
- Full math support: fractions, roots, integrals, matrices, Greek letters,
  accents, delimiters, and more
- Two bundled math fonts: Lete Sans Math (sans-serif, default) and
  STIX Two Math (serif); additional fonts via `load_font()`
- Color support via `\textcolor{}`
- ggplot2 integration with `geom_latex()` and `element_latex()`
- CJK and multilingual text in `\text{}` blocks

## Basic usage

The core function is `latex_grob()`, which returns a grid grob:

```{r basic, fig.height=0.8, fig.width=2, out.width="50%"}
library(gridmicrotex)
library(grid)

g <- latex_grob("\\frac{-b \\pm \\sqrt{b^2 - 4ac}}{2a}", gp = grid::gpar(fontsize = 24))
grid::grid.newpage()
grid::grid.draw(g)
```

For quick rendering, use `grid.latex()`:

```{r quick, fig.height=0.8, fig.width=3, out.width="40%"}
grid::grid.newpage()
grid.latex("$\\sum_{i=1}^{n} x_i^2$", gp = grid::gpar(fontsize = 28))
```

## Positioning and justification

Control placement with `x`, `y`, `hjust`, and `vjust`:

```{r positioning, fig.height=1.2, fig.width=3.5, out.width="40%"}
grid::grid.newpage()
grid.latex("Famous $E = mc^2$", x = 0.1, y = 0.7, hjust = 0, gp = grid::gpar(fontsize = 24))
grid.latex("F = ma", x = 0.1, y = 0.3, hjust = 0, gp = grid::gpar(fontsize = 24), input_mode = "math")
```

By default, the input is treated as LaTeX math mode, which treats string as text by default and use `$...$` or `\\(...\\)` delimiters to render math. The `"Famous"` in the first equation above treated as text. The `mixed` mode converts the input to math mode to reduce the user burden for typing `\\text{}` and the conversion might not be perfect, but it should handle most common cases without user intervention. Use `input_mode = "math"` to treat the whole string as math mode (the second example) or if you find a problem with the conversion and render text with `\\text{}`. You can change this with global `latex_options(input_mode = "math")` for heavy math or users who wants to the advantages LaTex macros, etc. The vignette from next example will set the input mode to `math` globally and render the whole string as math mode.

### Aligning to the math baseline

In addition to numeric justifications in `[0, 1]`, `hjust` and `vjust`
also accept named values. The most useful one is `vjust = "baseline"`,
which places the formula's math baseline (not the bbox centre) on the
anchor point — so a formula sits next to surrounding text the way it
would in a typeset document.

```{r baseline-align, fig.height=0.8, fig.width=4.5, out.width="60%"}
grid::grid.newpage()
y <- 0.5
grid::grid.segments(unit(0, "npc"), unit(y, "npc"),
                    unit(1, "npc"), unit(y, "npc"),
                    gp = grid::gpar(col = "grey80"))
grid::grid.text("if ", x = 0.10, y = y, just = c(0, 0.5),
                gp = grid::gpar(fontsize = 16))
grid.latex("$x \\geq \\sqrt{2\\pi}$",
           x = 0.22, y = y,
           hjust = "left", vjust = "baseline",
           gp = grid::gpar(fontsize = 16))
grid::grid.text(", then proceed.", x = 0.62, y = y, just = c(0, 0.5),
                gp = grid::gpar(fontsize = 16))
```

`hjust` accepts `"left"`/`"bbleft"`, `"center"`/`"centre"`/`"middle"`/
`"bbcentre"`, and `"right"`/`"bbright"`. `vjust` accepts `"bottom"`,
`"center"`/`"centre"`/`"middle"`, `"top"`, and `"baseline"`.

### Named anchors with `\mark{}`

The `\mark{name}` macro records a named anchor at its position inside
the formula. `grobMark()` then resolves the anchor to a pair of grid
units, ready to drive an arrow, callout, or any other grob.

A mark works at any nesting level — between top-level tokens, on a
compound sub-expression like `b^2`, even inside a superscript or
fraction. The position inherits the surrounding transform (font shrink
for scripts, scaling, rotation), so the anchor lands on the rendered
glyph rather than a pre-layout offset.

```{r mark, fig.height=2.4, fig.width=5, out.width="70%"}
g <- latex_grob(
  r"($a^2 + b\mark{term}^2 \mark{equals}= c^2$)",
  x = 0.5, y = 0.4,
  gp = grid::gpar(fontsize = 28)
)
grid::grid.newpage()
grid::grid.draw(g)

# Callout 1: the "=" sign, pointed at from above.
mk_eq <- grobMark(g, "equals")
grid::grid.segments(mk_eq$x, mk_eq$y + unit(15, "mm"),
                    mk_eq$x, mk_eq$y + unit(3, "mm"),
                    arrow = grid::arrow(length = unit(2, "mm"), type = "closed"),
                    gp = grid::gpar(col = "red"))
grid::grid.text("equals", x = mk_eq$x, y = mk_eq$y + unit(18, "mm"),
                gp = grid::gpar(col = "red", fontsize = 11))

# Callout 2: the b^2 term, pointed at from below — the mark sits at the
# end of the term, including the superscript's smaller scale.
mk_bsq <- grobMark(g, "term")
grid::grid.segments(mk_bsq$x - unit(6, "mm"), mk_bsq$y - unit(15, "mm"),
                    mk_bsq$x - unit(2, "mm"), mk_bsq$y - unit(3, "mm"),
                    arrow = grid::arrow(length = unit(2, "mm"), type = "closed"),
                    gp = grid::gpar(col = "blue"))
grid::grid.text("b² term",
                x = mk_bsq$x - unit(7, "mm"),
                y = mk_bsq$y - unit(18, "mm"),
                just = "right",
                gp = grid::gpar(col = "blue", fontsize = 11))
```

Because the returned units carry the grob's viewport position and
`hjust`/`vjust`, you can pass them straight to `grid.points`,
`grid.segments`, or any other grid drawing function — no manual offset
arithmetic.

`\mark` records a single point, not a span. To centre a callout over a
multi-glyph term, place the mark at the term's end and shift in your
drawing code, or place a pair of marks (`\mark{l}…\mark{r}`) and use
their midpoint.

## Colors

You can use `r"()"` raw strings to write LaTeX with regular newlines and quotes without escaping. Set the formula color via `gp`, or use `\textcolor{}` within the LaTeX:

```{r colors, fig.height=0.6, fig.width=2, out.width="40%"}
latex_options(input_mode = "math") # Set math mode globally
grid::grid.newpage()
grid.latex(
  r"(\textcolor{red}{\alpha} + \textcolor{blue}{\beta} = \gamma)",
  gp = grid::gpar(fontsize = 28)
)
```

## Math fonts

The package ships with two math fonts, both loaded automatically:

| Alias | Font | License |
|-------|------|---------|
| `"lete"` (default) | Lete Sans Math | SIL Open Font License |
| `"stix"` | STIX Two Math | SIL Open Font License |

```{r fonts-list}
available_math_fonts()
```

```{r fonts-default, fig.height=0.8, fig.width=2, out.width="50%"}
latex_options(math_font = "stix")
grid::grid.newpage()
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24))

# Switch back to the default (Lete Sans Math)
latex_options(math_font = "lete")
```

You can also override the font per call via `math_font`:

```{r fonts, fig.height=1.5, fig.width=2, out.width="50%"}
grid::grid.newpage()
grid::pushViewport(grid::viewport(layout = grid::grid.layout(2, 1)))
grid::pushViewport(grid::viewport(layout.pos.row = 1))
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24))
grid::upViewport()
grid::pushViewport(grid::viewport(layout.pos.row = 2))
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24), math_font = "stix")
grid::upViewport(2)
```

Use `available_math_fonts()` to list loaded fonts and `check_fonts()`
for a diagnostic report.

### Advanced: loading custom fonts

Use `load_font()` to add any additional OpenType math font. The OpenType
MATH table is parsed directly in C++ --- no companion metrics file or
external toolchain is required:

```r
load_font("path/to/MyFont.otf")
```

### Render modes

gridmicrotex supports two rendering modes for math glyphs:
  
- **`"typeface"`** (default): Renders glyphs as native text using the
  math font's typeface. This produces selectable, searchable, and
  accessible text in PDF and SVG output. Bundled math fonts (Lete Sans
  Math, STIX Two Math) and any registered via `load_font()` are read
  directly from their OTF files --- no system-wide font install is
  required. Requires a device that supports the R \eqn{\geq} 4.3 glyph
  engine (e.g., `ragg::agg_png()`, `svglite::svglite()`,
  `grDevices::cairo_pdf()`). On devices that do not (e.g., the base
  `pdf()` device), the package automatically falls back to path mode
  with a warning.

- **`"path"`**: Renders each glyph as a filled vector path.
  This works on all R graphics devices and produces pixel-perfect output.
  However, text in PDF/SVG output is not selectable or searchable.

```r
# Default typeface mode (selectable text in PDF/SVG)
grid.latex("E = mc^2", gp = grid::gpar(fontsize = 24))

# Explicit path mode (works everywhere, but text is not selectable)
grid.latex("E = mc^2", gp = grid::gpar(fontsize = 24), render_mode = "path")
```

> **Important: Do not use `showtext::showtext_auto()` with typeface mode.**
> The [showtext](https://CRAN.R-project.org/package=showtext) package
> globally intercepts all text rendering and converts it to vector paths.
> This silently defeats typeface mode, causing all math glyphs to appear
> as paths instead of native text — even on devices like `svglite` and
> `ragg` that fully support font embedding. If you need showtext for
> other parts of your plot, disable it before drawing LaTeX formulas:
>
> ```r
> showtext::showtext_auto(FALSE)
> grid.latex("E = mc^2", gp = grid::gpar(fontsize = 24))  # typeface mode works correctly
> ```

## Querying dimensions

`latex_dims()` returns the bounding box of an expression:

```{r dims}
dims <- latex_dims("\\frac{a}{b}", gp = grid::gpar(fontsize = 20))
dims
```

This is useful for layout calculations and ensuring labels fit.

## Text rendering and CJK support

Text inside `\text{}` and `\mbox{}` is rendered using R's standard
text-rendering system. This means `gp$fontfamily` controls the font for
**all** text content --- Latin letters, CJK characters, Cyrillic, and any
other script your R graphics device supports:

```{r cjk, fig.height = 0.8, fig.width = 2.5, out.width="50%"}
grid::grid.newpage()
grid.latex("x^2 + \\text{你好}", gp = grid::gpar(fontsize = 24, fontfamily = "sans"))
```

Any font available to R works: base families like `"sans"`, `"serif"`,
`"mono"`, or fonts registered via **showtext** / **systemfonts**.

### Font pairing

The bundled math fonts have different styles. For a consistent look,
pair them with a matching `fontfamily`:

| Math font | Style | Suggested `fontfamily` |
|-----------|-------|------------------------|
| Lete Sans Math (`"lete"`, default) | Sans-serif | `"sans"` |
| STIX Two Math (`"stix"`) | Serif | `"serif"` |

```{r font-pairing, fig.height = 0.4, fig.width = 2, out.width="70%"}
grid::grid.newpage()
grid.latex(
  "\\text{Theorem: } \\forall x \\in \\mathbb{R},\\; x^2 \\geq 0",
  math_font = "stix",
  gp = grid::gpar(fontfamily = "serif", fontsize = 12)
)
```

## Supported LaTeX

gridmicrotex uses the MicroTeX engine, which is a **math formula renderer**,
not a full document typesetter. It covers the vast majority of math notation
you would use in plots and figures, but does not attempt to replace a full
LaTeX installation.

### Complicated examples

```{r complex-formula, fig.height = 3, fig.width=6, out.width="60%"}
grid::grid.newpage()
grid.latex(paste0(
      "\\begin{array}{l}",
      "  \\forall\\varepsilon\\in\\mathbb{R}_+^*\\ \\exists\\eta>0",
      "\\ |x-x_0|\\leq\\eta\\Longrightarrow|f(x)-f(x_0)|\\leq\\varepsilon\\\\",
      "  \\det",
      "  \\begin{bmatrix}",
      "      a_{11}&a_{12}&\\cdots&a_{1n}\\\\",
      "      a_{21}&\\ddots&&\\vdots\\\\",
      "      \\vdots&&\\ddots&\\vdots\\\\",
      "      a_{n1}&\\cdots&\\cdots&a_{nn}",
      "  \\end{bmatrix}",
      "  \\overset{\\mathrm{def}}{=}\\sum_{\\sigma\\in\\mathfrak{S}_n}",
      "\\varepsilon(\\sigma)\\prod_{k=1}^n a_{k\\sigma(k)}\\\\",
      "  \\int_0^\\infty{x^{2n} e^{-a x^2}\\,dx} = \\frac{2n-1}{2a}",
      " \\int_0^\\infty{x^{2(n-1)} e^{-a x^2}\\,dx}",
      " = \\frac{(2n-1)!!}{2^{n+1}} \\sqrt{\\frac{\\pi}{a^{2n+1}}}\\\\",
      "\\end{array}"
), gp = grid::gpar(fontsize = 16))
```


```{r table-multicolumn, fig.height = 3, fig.width=8, out.width="60%"}
grid::grid.newpage()

grid.latex(
  "
  \\newcolumntype{s}{>{\\color{#1234B6}}c}
\\begin{array}{|c|c|c|s|}
  \\hline
  \\rowcolor{Tan}\\multicolumn{4}{|c|}{\\textcolor{white}{\\bold{\\text{Table Head}}}}\\\\
  \\hline
  \\text{Matrix}&\\multicolumn{2}{|c|}{\\text{Multicolumns}}&\\text{Font size commands}\\\\
  \\hline
  \\begin{pmatrix}
      \\alpha_{11}&\\cdots&\\alpha_{1n}\\\\
      \\hdotsfor{3}\\\\
      \\alpha_{n1}&\\cdots&\\alpha_{nn}
  \\end{pmatrix}
  &\\large \\text{Left}&\\cellcolor{#00bde5}\\small \\textcolor{white}{\\text{\\bold{Right}}}
  &\\small \\text{small Small}\\\\
  \\hline
  \\multicolumn{4}{|c|}{\\text{Table Foot}}\\\\
  \\hline
\\end{array}
  ",
  gp = grid::gpar(fontsize = 22)
)
```

```{r complicated-equation, fig.height = 9, fig.width=9, out.width="60%"}
grid::grid.newpage()
grid.latex(
  "\\definecolor{gris}{gray}{0.9}
\\definecolor{noir}{rgb}{0,0,0}
\\definecolor{bleu}{rgb}{0,0,1}
\\fatalIfCmdConflict{false}
\\newcommand{\\pa}{\\left|}
\\begin{array}{c}
  \\LaTeX\\\\
  \\begin{split}
      |I_2| &= \\pa\\int_0^T\\psi(t)\\left\\{ u(a,t)-\\int_{\\gamma(t)}^a \\frac{d\\theta}{k} (\\theta,t) \\int_a^\\theta c(\\xi)
          u_t (\\xi,t)\\,d\\xi\\right\\}dt\\right|\\\\
      &\\le C_6 \\Bigg|\\pa f \\int_\\Omega \\pa\\widetilde{S}^{-1,0}_{a,-}
          W_2(\\Omega, \\Gamma_1)\\right|\\ \\right|\\left| |u|\\overset{\\circ}{\\to} W_2^{\\widetilde{A}}(\\Omega\\Gamma_r,T)\\right|\\Bigg|\\\\
      &\\\\
      &\\begin{pmatrix}
          \\alpha&\\beta&\\gamma&\\delta\\\\
          \\aleph&\\beth&\\gimel&\\daleth\\\\
          \\mathfrak{A}&\\mathfrak{B}&\\mathfrak{C}&\\mathfrak{D}\\\\
          \\boldsymbol{\\mathfrak{a}}&\\boldsymbol{\\mathfrak{b}}&\\boldsymbol{\\mathfrak{c}}&\\boldsymbol{\\mathfrak{d}}
      \\end{pmatrix}
      \\quad{(a+b)}^{\\frac{n}{2}}=\\sqrt{\\sum_{k=0}^n\\tbinom{n}{k}a^kb^{n-k}}\\quad
          \\Biggl(\\biggl(\\Bigl(\\bigl(()\\bigr)\\Bigr)\\biggr)\\Biggr)\\\\
      &\\forall\\varepsilon\\in\\mathbb{R}_+^*\\ \\exists\\eta>0\\ |x-x_0|\\leq\\eta\\Longrightarrow|f(x)-f(x_0)|\\leq\\varepsilon\\\\
      &\\det
      \\begin{bmatrix}
          a_{11}&a_{12}&\\cdots&a_{1n}\\\\
          a_{21}&\\ddots&&\\vdots\\\\
          \\vdots&&\\ddots&\\vdots\\\\
          a_{n1}&\\cdots&\\cdots&a_{nn}
      \\end{bmatrix}
      \\overset{\\mathrm{def}}{=}\\sum_{\\sigma\\in\\mathfrak{S}_n}\\varepsilon(\\sigma)\\prod_{k=1}^n a_{k\\sigma(k)}\\\\
      &\\Delta f(x,y)=\\frac{\\partial^2f}{\\partial x^2}+\\frac{\\partial^2f}{\\partial y^2}\\qquad\\qquad \\fcolorbox{noir}{gris}
          {n!\\underset{n\\rightarrow+\\infty}{\\sim} {\\left(\\frac{n}{e}\\right)}^n\\sqrt{2\\pi n}}\\\\
      &\\sideset{_\\alpha^\\beta}{_\\gamma^\\delta}{
      \\begin{pmatrix}
          a&b\\\\
          c&d
      \\end{pmatrix}}
      \\xrightarrow[T]{n\\pm i-j}\\sideset{^t}{}A\\xleftarrow{\\overrightarrow{u}\\wedge\\overrightarrow{v}}
          \\underleftrightarrow{\\iint_{\\mathds{R}^2}e^{-\\left(x^2+y^2\\right)}\\,\\mathrm{d}x\\mathrm{d}y}
  \\end{split}\\\\
  \\rotatebox{30}{\\sum_{n=1}^{+\\infty}}\\quad\\mbox{Mirror rorriM}\\reflectbox{\\mbox{Mirror rorriM}}
\\end{array}",
  gp = grid::gpar(fontsize = 22),
  render_mode = "path"
)
```



### Lists

The `itemize` and `enumerate` environments lay their items out as a
left-aligned column, one item per row --- `itemize` prefixes each item
with a bullet, `enumerate` numbers them:

```{r lists, fig.height=1.4, fig.width=3, out.width="45%"}
grid::grid.newpage()
grid.latex(paste0(
  "\\begin{enumerate}",
  "  \\item e^{i\\pi} + 1 = 0",
  "  \\item \\begin{itemize}",
  "           \\item \\alpha \\item \\beta",
  "         \\end{itemize}",
  "\\end{enumerate}"
), gp = grid::gpar(fontsize = 20))
```

An optional `[…]` argument customises the marker. For `itemize` it is
the literal marker (`\begin{itemize}[\star]`); for `enumerate` it is a
counter template containing one of `\arabic*`, `\alph*`, `\Alph*`,
`\roman*`, or `\Roman*` (e.g. `\begin{enumerate}[\Roman*.]`). Lists may
nest, and an item may contain any math --- including a `\begin{array}`
table.

Because MicroTeX is a math engine, each item is typeset as a
**math-mode, single-line** expression: there is no paragraph flow or
line wrapping, and prose inside an item must be wrapped in `\text{}`
(e.g. `\item \text{First point}`). The `description` environment is not
supported.

### Pasting LaTeX from other sources

When the input is generated by tools that emit complete LaTeX --- for
example R packages that produce ready-to-compile `tabular` snippets, or
LaTeX fragments copy-pasted from a `.tex` file --- the formula often
arrives wrapped in document-level constructs that MicroTeX does not
implement. Rather than refusing such input, gridmicrotex rewrites or
removes a small set of well-known wrappers before parsing.

**Removed silently (no visual effect):**

| Construct | Why |
|---|---|
| `%`-to-end-of-line comments (`\%` is preserved) | comments are non-visual in LaTeX too |
| `\documentclass[…]{…}`, `\usepackage[…]{…}` | preamble metadata |
| `\begin{document}` / `\end{document}` | document boundary, structural only |
| `\maketitle`, `\title{…}`, `\author{…}` | title-page metadata, no body output |
| `\label{…}` | cross-reference target, never rendered in LaTeX either |
| `\begin{table}[…]` / `\end{table}`, `\begin{figure}[…]` / `\end{figure}` (and starred variants) | float wrappers; the contents stay |
| `\centering`, `\raggedright`, `\raggedleft`, `\flushleft`, `\flushright` | alignment scope declarations |
| `\noindent`, `\relax` | content-free declarations |

**Rewritten to a MicroTeX equivalent:**

| Construct | Becomes |
|---|---|
| `\emph{X}` | `\textit{X}` |
| `\textnormal{X}` | `\text{X}` |
| `\par`, `\newline` | `\\` (line break) |
| `\toprule`, `\bottomrule` | `\thickhline` (rendered ~2× thickness) |
| `\midrule` | `\hline` |
| `\cmidrule[trim]?(parenarg)?{a-b}` | `\cline{a-b}` --- partial-column rule |
| `\caption[short]{X}` | `\text{X}\\` inserted inline at source position |
| `\smallskip`, `\medskip`, `\bigskip` | `\vspace{0.25em}` / `\vspace{0.5em}` / `\vspace{1em}` --- em-relative so they scale with `gp$fontsize` |
| `\hfill`, `\vfill` | `\quad` / `\vspace{1em}` --- static proxies for rubber lengths |

A note on `\caption`: in real LaTeX, captions are repositioned by the
float machinery (typically above or below the surrounding `tabular`,
independent of source order). Here the caption renders exactly where
it appears in the source. For tools that place `\caption` after
`\end{tabular}`, this gives a caption-below-table layout; for tools
that place it before, you get caption-above.

A note on the skips and fills: real LaTeX defines `\smallskip` /
`\medskip` / `\bigskip` as absolute `pt` amounts (3 / 6 / 12). The
mapping above uses em-relative values instead, so the gap stays
visible whether `gp$fontsize` is 10 or 30. If you need an exact
absolute size, use `\vspace{Xpt}` directly --- MicroTeX accepts pt,
em, ex, mm, cm.

`\hfill` and `\vfill` are *rubber* lengths in LaTeX --- they expand
to fill the surrounding glue. A fixed-size grob has nothing to
"fill," so the mapping produces a static 1em gap. The position is
right; the elasticity is gone.

**Not honored (rendered as literal text, which is intentional --- it
makes unsupported markup easy to spot):**

- Declarative font scopes: `\bfseries`, `\itshape`, `\ttfamily`,
  `\sffamily`, `\rmfamily`. These affect text within their group in
  LaTeX, which requires scope tracking we do not implement. Use the
  argument-bearing forms instead: `\textbf{…}`, `\textit{…}`,
  `\texttt{…}`, `\textsf{…}`, `\textrm{…}`, all of which MicroTeX
  supports natively.
- Hyperlinks and references: `\url{…}`, `\href{…}{…}`, `\ref{…}`,
  `\cite{…}`. There is no link concept inside a grob.
- Footnotes: `\footnote{…}` (positioning machinery is page-bound).
- Small caps: `\textsc{…}` (MicroTeX has no small-caps glyphs).

### What is not supported

MicroTeX is a **math formula renderer**, not a full LaTeX engine. The
following are outside its scope and would need a real document
compiler:

- **Document structure**: `\section`, page layout, headers/footers,
  `\tableofcontents`
- **Package loading semantics**: `\usepackage{…}` is accepted but
  loads nothing --- supported commands are all built into MicroTeX
- **Paragraph text**: line breaking, hyphenation, justified paragraphs
- **TikZ / PGF** drawing commands
- **Images**: `\includegraphics`
- **Cross-references**: targets via `\label` are silently dropped;
  `\ref`, `\cite`, bibliographies have nothing to resolve against
- **Theorem environments**: `\begin{theorem}`, `\begin{proof}`
- **The `description` list environment** (`itemize` and `enumerate`
  *are* supported --- see *Lists* above)
- **Some amsmath commands**: `\tag` and equation numbering

For most statistical graphics use cases --- axis labels, annotations,
legends, and in-plot formulas --- the supported feature set is more
than sufficient.

## Project-wide defaults

`latex_options()` sets defaults for `math_font` and `render_mode`, used
by `latex_grob()`, `grid.latex()`, `latex_dims()`, and `latex_tree()`
whenever the corresponding argument is not supplied at the call site.
Size is controlled at the grob level via `gp$fontsize` /
`gp$lineheight` (see *Basic usage*).

```{r options, eval=FALSE}
latex_options(math_font = "stix", render_mode = "typeface")

# Later calls pick these up automatically
grid.latex("\\sum_{i=1}^{n} i^{2}", gp = grid::gpar(fontsize = 14))

# Query current settings
latex_options()

# Reset to built-in defaults
reset_latex_options()
```

Explicit arguments always win. Setting `math_font` via
`latex_options()` also updates the MicroTeX engine default, so you
don't also need a separate font-setup call.

## User-defined macros

`define_macro()` registers zero-argument shorthands that are expanded
by text substitution before the expression reaches MicroTeX. Handy for
recurring notation:

```{r macros, fig.height=0.7, fig.width=3, out.width="50%"}
define_macro("RR", "\\mathbb{R}")
define_macro("eps", "\\varepsilon")

grid::grid.newpage()
grid.latex("\\forall \\eps > 0, \\eps \\in \\RR", gp = grid::gpar(fontsize = 24))

clear_macros()
```

Macro names must be ASCII letters. Expansion iterates to a fixed point,
so macros can reference other macros. Use `list_macros()` to see
currently registered ones, and `clear_macros()` (with no arguments) to
drop them all.

For parameterised macros (0--9 arguments) scoped to a single
expression, MicroTeX also accepts plain-TeX `\def`:

```{r def-inline, fig.height=0.7, fig.width=4, out.width="60%"}
grid::grid.newpage()
grid.latex(
  r"(\def\norm#1{\left\lVert #1 \right\rVert}
      \def\inner#1#2{\langle #1, #2 \rangle}
      \norm{\vec{v}} = \sqrt{\inner{\vec{v}}{\vec{v}}})",
  gp = grid::gpar(fontsize = 24)
)
```

`\def` definitions live only for the duration of the expression they
appear in, so they are the right tool for a parameterised abbreviation
local to one label. Reach for `define_macro()` instead when you want a
shorthand to persist across many plots in the same R session.

## Layout caching

Parsed layouts are memoised by `(tex, fontsize, math_font, render_mode,
...)`. Re-drawing the same formula --- for example, the same axis label
across many plots --- reuses the cached layout:

```{r cache, eval=FALSE}
latex_cache_info()       # size / max_size / hits / misses
latex_cache_limit(1024)  # raise or lower the LRU capacity
latex_cache_clear()      # wipe the cache (e.g. after re-loading fonts)
```

Set the limit to `0` to disable caching entirely.

## Introspecting a formula

`latex_tree()` returns the raw draw-record table plus bbox metadata,
useful for debugging alignment, counting glyphs, or building custom
grobs on top of the layout:

```{r tree}
tr <- latex_tree("\\frac{a}{b}")
tr
head(tr$records, 3)
```

## Debug overlay

Pass `debug = TRUE` to `latex_grob()` / `grid.latex()` to overlay
diagnostics on the rendered formula --- the full bounding box (dashed
gray), the baseline (solid red), and a dot at each draw record's
origin. Useful for checking vertical alignment between a formula and
surrounding grobs:

```{r debug, fig.height=1, fig.width=3, out.width="60%"}
grid::grid.newpage()
grid.latex("x^{2} + y_{i}", gp = grid::gpar(fontsize = 30), debug = TRUE)
```

## Comparison with alternatives

| Approach       | LaTeX required? | Device independent? | Vector? | Math coverage |
|:---------------|:---------------:|:-------------------:|:-------:|:-------------:|
| `tikzDevice`   | Yes             | No                  | Yes     | Full          |
| `xdvir`        | Yes             | No                  | Yes     | Full          |
| `latexpdf`     | Yes             | No                  | Yes     | Full (tables) |
| `latex2exp`    | No              | Yes                 | Yes     | Limited       |
| `plotmath`     | No              | Yes                 | Yes     | Limited       |
| **gridmicrotex** | **No**       | **Yes**             | **Yes** | **Broad**     |


## Graphics backend

The default graphics device on Windows (`windows()`) and macOS
(`quartz()`) may not find the bundled math fonts, producing warnings
like:

```
font family not found in Windows font database
```

To avoid this, switch to a modern graphics backend that uses
[systemfonts](https://CRAN.R-project.org/package=systemfonts) for font
resolution:

```r
# For knitr / R Markdown --- add to your setup chunk:
knitr::opts_chunk$set(dev = "ragg_png")

# For interactive use:
options(device = function(...) ragg::agg_png(tempfile(fileext = ".png"), ...))
```

Recommended backends:

| Backend               | Format | Package               |
|:----------------------|:-------|:----------------------|
| `ragg::agg_png()`     | PNG    | [ragg](https://CRAN.R-project.org/package=ragg)       |
| `svglite::svglite()`  | SVG    | [svglite](https://CRAN.R-project.org/package=svglite) |
| `grDevices::cairo_pdf()` | PDF | Base R (Cairo build)  |

Alternatively, use `render_mode = "path"` to bypass font lookup
entirely --- glyphs are drawn as vector paths, which works on all
devices but produces non-selectable text in PDF/SVG.

---
title: "Introduction to gridmicrotex"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to gridmicrotex}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  dpi = 300,
  dev = "ragg_png"
)

```

## What is gridmicrotex?

**gridmicrotex** renders LaTeX math equations as native R `grid` graphics
objects (grobs). It uses the [MicroTeX](https://github.com/NanoMichael/MicroTeX)
C++ library as its layout engine --- MicroTeX parses LaTeX, builds the TeX
box model, and computes exact glyph coordinates. The package intercepts
this layout data and maps it to native grid primitives (`pathGrob`,
`segmentsGrob`, `rectGrob`, `textGrob`), producing a `gTree` that works
on any R graphics device at any resolution.

**Key features:**


- No external LaTeX installation required --- MicroTeX is fully embedded
- Resolution-independent vector output on all R devices (PNG, PDF, SVG, ...)
- Full math support: fractions, roots, integrals, matrices, Greek letters,
  accents, delimiters, and more
- Two bundled math fonts: Lete Sans Math (sans-serif, default) and
  STIX Two Math (serif); additional fonts via `load_font()`
- Color support via `\textcolor{}`
- ggplot2 integration with `geom_latex()` and `element_latex()`
- CJK and multilingual text in `\text{}` blocks

## Basic usage

The core function is `latex_grob()`, which returns a grid grob:

```{r basic, fig.height=0.8, fig.width=2, out.width="50%"}
library(gridmicrotex)
library(grid)

g <- latex_grob("\\frac{-b \\pm \\sqrt{b^2 - 4ac}}{2a}", gp = grid::gpar(fontsize = 24))
grid::grid.newpage()
grid::grid.draw(g)
```

For quick rendering, use `grid.latex()`:

```{r quick, fig.height=0.8, fig.width=3, out.width="40%"}
grid::grid.newpage()
grid.latex("$\\sum_{i=1}^{n} x_i^2$", gp = grid::gpar(fontsize = 28))
```

## Positioning and justification

Control placement with `x`, `y`, `hjust`, and `vjust`:

```{r positioning, fig.height=1.2, fig.width=3.5, out.width="40%"}
grid::grid.newpage()
grid.latex("Famous $E = mc^2$", x = 0.1, y = 0.7, hjust = 0, gp = grid::gpar(fontsize = 24))
grid.latex("F = ma", x = 0.1, y = 0.3, hjust = 0, gp = grid::gpar(fontsize = 24), input_mode = "math")
```

By default, the input is treated as LaTeX math mode, which treats string as text by default and use `$...$` or `\\(...\\)` delimiters to render math. The `"Famous"` in the first equation above treated as text. The `mixed` mode converts the input to math mode to reduce the user burden for typing `\\text{}` and the conversion might not be perfect, but it should handle most common cases without user intervention. Use `input_mode = "math"` to treat the whole string as math mode (the second example) or if you find a problem with the conversion and render text with `\\text{}`. You can change this with global `latex_options(input_mode = "math")` for heavy math or users who wants to the advantages LaTex macros, etc. The vignette from next example will set the input mode to `math` globally and render the whole string as math mode.

### Aligning to the math baseline

In addition to numeric justifications in `[0, 1]`, `hjust` and `vjust`
also accept named values. The most useful one is `vjust = "baseline"`,
which places the formula's math baseline (not the bbox centre) on the
anchor point — so a formula sits next to surrounding text the way it
would in a typeset document.

```{r baseline-align, fig.height=0.8, fig.width=4.5, out.width="60%"}
grid::grid.newpage()
y <- 0.5
grid::grid.segments(unit(0, "npc"), unit(y, "npc"),
                    unit(1, "npc"), unit(y, "npc"),
                    gp = grid::gpar(col = "grey80"))
grid::grid.text("if ", x = 0.10, y = y, just = c(0, 0.5),
                gp = grid::gpar(fontsize = 16))
grid.latex("$x \\geq \\sqrt{2\\pi}$",
           x = 0.22, y = y,
           hjust = "left", vjust = "baseline",
           gp = grid::gpar(fontsize = 16))
grid::grid.text(", then proceed.", x = 0.62, y = y, just = c(0, 0.5),
                gp = grid::gpar(fontsize = 16))
```

`hjust` accepts `"left"`/`"bbleft"`, `"center"`/`"centre"`/`"middle"`/
`"bbcentre"`, and `"right"`/`"bbright"`. `vjust` accepts `"bottom"`,
`"center"`/`"centre"`/`"middle"`, `"top"`, and `"baseline"`.

### Named anchors with `\mark{}`

The `\mark{name}` macro records a named anchor at its position inside
the formula. `grobMark()` then resolves the anchor to a pair of grid
units, ready to drive an arrow, callout, or any other grob.

A mark works at any nesting level — between top-level tokens, on a
compound sub-expression like `b^2`, even inside a superscript or
fraction. The position inherits the surrounding transform (font shrink
for scripts, scaling, rotation), so the anchor lands on the rendered
glyph rather than a pre-layout offset.

```{r mark, fig.height=2.4, fig.width=5, out.width="70%"}
g <- latex_grob(
  r"($a^2 + b\mark{term}^2 \mark{equals}= c^2$)",
  x = 0.5, y = 0.4,
  gp = grid::gpar(fontsize = 28)
)
grid::grid.newpage()
grid::grid.draw(g)

# Callout 1: the "=" sign, pointed at from above.
mk_eq <- grobMark(g, "equals")
grid::grid.segments(mk_eq$x, mk_eq$y + unit(15, "mm"),
                    mk_eq$x, mk_eq$y + unit(3, "mm"),
                    arrow = grid::arrow(length = unit(2, "mm"), type = "closed"),
                    gp = grid::gpar(col = "red"))
grid::grid.text("equals", x = mk_eq$x, y = mk_eq$y + unit(18, "mm"),
                gp = grid::gpar(col = "red", fontsize = 11))

# Callout 2: the b^2 term, pointed at from below — the mark sits at the
# end of the term, including the superscript's smaller scale.
mk_bsq <- grobMark(g, "term")
grid::grid.segments(mk_bsq$x - unit(6, "mm"), mk_bsq$y - unit(15, "mm"),
                    mk_bsq$x - unit(2, "mm"), mk_bsq$y - unit(3, "mm"),
                    arrow = grid::arrow(length = unit(2, "mm"), type = "closed"),
                    gp = grid::gpar(col = "blue"))
grid::grid.text("b² term",
                x = mk_bsq$x - unit(7, "mm"),
                y = mk_bsq$y - unit(18, "mm"),
                just = "right",
                gp = grid::gpar(col = "blue", fontsize = 11))
```

Because the returned units carry the grob's viewport position and
`hjust`/`vjust`, you can pass them straight to `grid.points`,
`grid.segments`, or any other grid drawing function — no manual offset
arithmetic.

`\mark` records a single point, not a span. To centre a callout over a
multi-glyph term, place the mark at the term's end and shift in your
drawing code, or place a pair of marks (`\mark{l}…\mark{r}`) and use
their midpoint.

## Colors

You can use `r"()"` raw strings to write LaTeX with regular newlines and quotes without escaping. Set the formula color via `gp`, or use `\textcolor{}` within the LaTeX:

```{r colors, fig.height=0.6, fig.width=2, out.width="40%"}
latex_options(input_mode = "math") # Set math mode globally
grid::grid.newpage()
grid.latex(
  r"(\textcolor{red}{\alpha} + \textcolor{blue}{\beta} = \gamma)",
  gp = grid::gpar(fontsize = 28)
)
```

## Math fonts

The package ships with two math fonts, both loaded automatically:

| Alias | Font | License |
|-------|------|---------|
| `"lete"` (default) | Lete Sans Math | SIL Open Font License |
| `"stix"` | STIX Two Math | SIL Open Font License |

```{r fonts-list}
available_math_fonts()
```

```{r fonts-default, fig.height=0.8, fig.width=2, out.width="50%"}
latex_options(math_font = "stix")
grid::grid.newpage()
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24))

# Switch back to the default (Lete Sans Math)
latex_options(math_font = "lete")
```

You can also override the font per call via `math_font`:

```{r fonts, fig.height=1.5, fig.width=2, out.width="50%"}
grid::grid.newpage()
grid::pushViewport(grid::viewport(layout = grid::grid.layout(2, 1)))
grid::pushViewport(grid::viewport(layout.pos.row = 1))
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24))
grid::upViewport()
grid::pushViewport(grid::viewport(layout.pos.row = 2))
grid.latex(r"(\int_0^1 f(x)\,dx)", gp = grid::gpar(fontsize = 24), math_font = "stix")
grid::upViewport(2)
```

Use `available_math_fonts()` to list loaded fonts and `check_fonts()`
for a diagnostic report.

### Advanced: loading custom fonts

Use `load_font()` to add any additional OpenType math font. The OpenType
MATH table is parsed directly in C++ --- no companion metrics file or
external toolchain is required:

```r
load_font("path/to/MyFont.otf")
```

### Render modes

gridmicrotex supports two rendering modes for math glyphs:
  
- **`"typeface"`** (default): Renders glyphs as native text using the
  math font's typeface. This produces selectable, searchable, and
  accessible text in PDF and SVG output. Bundled math fonts (Lete Sans
  Math, STIX Two Math) and any registered via `load_font()` are read
  directly from their OTF files --- no system-wide font install is
  required. Requires a device that supports the R \eqn{\geq} 4.3 glyph
  engine (e.g., `ragg::agg_png()`, `svglite::svglite()`,
  `grDevices::cairo_pdf()`). On devices that do not (e.g., the base
  `pdf()` device), the package automatically falls back to path mode
  with a warning.

- **`"path"`**: Renders each glyph as a filled vector path.
  This works on all R graphics devices and produces pixel-perfect output.
  However, text in PDF/SVG output is not selectable or searchable.

```r
# Default typeface mode (selectable text in PDF/SVG)
grid.latex("E = mc^2", gp = grid::gpar(fontsize = 24))

# Explicit path mode (works everywhere, but text is not selectable)
grid.latex("E = mc^2", gp = grid::gpar(fontsize = 24), render_mode = "path")
```

> **Important: Do not use `showtext::showtext_auto()` with typeface mode.**
> The [showtext](https://CRAN.R-project.org/package=showtext) package
> globally intercepts all text rendering and converts it to vector paths.
> This silently defeats typeface mode, causing all math glyphs to appear
> as paths instead of native text — even on devices like `svglite` and
> `ragg` that fully support font embedding. If you need showtext for
> other parts of your plot, disable it before drawing LaTeX formulas:
>
> ```r
> showtext::showtext_auto(FALSE)
> grid.latex("E = mc^2", gp = grid::gpar(fontsize = 24))  # typeface mode works correctly
> ```

## Querying dimensions

`latex_dims()` returns the bounding box of an expression:

```{r dims}
dims <- latex_dims("\\frac{a}{b}", gp = grid::gpar(fontsize = 20))
dims
```

This is useful for layout calculations and ensuring labels fit.

## Text rendering and CJK support

Text inside `\text{}` and `\mbox{}` is rendered using R's standard
text-rendering system. This means `gp$fontfamily` controls the font for
**all** text content --- Latin letters, CJK characters, Cyrillic, and any
other script your R graphics device supports:

```{r cjk, fig.height = 0.8, fig.width = 2.5, out.width="50%"}
grid::grid.newpage()
grid.latex("x^2 + \\text{你好}", gp = grid::gpar(fontsize = 24, fontfamily = "sans"))
```

Any font available to R works: base families like `"sans"`, `"serif"`,
`"mono"`, or fonts registered via **showtext** / **systemfonts**.

### Font pairing

The bundled math fonts have different styles. For a consistent look,
pair them with a matching `fontfamily`:

| Math font | Style | Suggested `fontfamily` |
|-----------|-------|------------------------|
| Lete Sans Math (`"lete"`, default) | Sans-serif | `"sans"` |
| STIX Two Math (`"stix"`) | Serif | `"serif"` |

```{r font-pairing, fig.height = 0.4, fig.width = 2, out.width="70%"}
grid::grid.newpage()
grid.latex(
  "\\text{Theorem: } \\forall x \\in \\mathbb{R},\\; x^2 \\geq 0",
  math_font = "stix",
  gp = grid::gpar(fontfamily = "serif", fontsize = 12)
)
```

## Supported LaTeX

gridmicrotex uses the MicroTeX engine, which is a **math formula renderer**,
not a full document typesetter. It covers the vast majority of math notation
you would use in plots and figures, but does not attempt to replace a full
LaTeX installation.

### Complicated examples

```{r complex-formula, fig.height = 3, fig.width=6, out.width="60%"}
grid::grid.newpage()
grid.latex(paste0(
      "\\begin{array}{l}",
      "  \\forall\\varepsilon\\in\\mathbb{R}_+^*\\ \\exists\\eta>0",
      "\\ |x-x_0|\\leq\\eta\\Longrightarrow|f(x)-f(x_0)|\\leq\\varepsilon\\\\",
      "  \\det",
      "  \\begin{bmatrix}",
      "      a_{11}&a_{12}&\\cdots&a_{1n}\\\\",
      "      a_{21}&\\ddots&&\\vdots\\\\",
      "      \\vdots&&\\ddots&\\vdots\\\\",
      "      a_{n1}&\\cdots&\\cdots&a_{nn}",
      "  \\end{bmatrix}",
      "  \\overset{\\mathrm{def}}{=}\\sum_{\\sigma\\in\\mathfrak{S}_n}",
      "\\varepsilon(\\sigma)\\prod_{k=1}^n a_{k\\sigma(k)}\\\\",
      "  \\int_0^\\infty{x^{2n} e^{-a x^2}\\,dx} = \\frac{2n-1}{2a}",
      " \\int_0^\\infty{x^{2(n-1)} e^{-a x^2}\\,dx}",
      " = \\frac{(2n-1)!!}{2^{n+1}} \\sqrt{\\frac{\\pi}{a^{2n+1}}}\\\\",
      "\\end{array}"
), gp = grid::gpar(fontsize = 16))
```


```{r table-multicolumn, fig.height = 3, fig.width=8, out.width="60%"}
grid::grid.newpage()

grid.latex(
  "
  \\newcolumntype{s}{>{\\color{#1234B6}}c}
\\begin{array}{|c|c|c|s|}
  \\hline
  \\rowcolor{Tan}\\multicolumn{4}{|c|}{\\textcolor{white}{\\bold{\\text{Table Head}}}}\\\\
  \\hline
  \\text{Matrix}&\\multicolumn{2}{|c|}{\\text{Multicolumns}}&\\text{Font size commands}\\\\
  \\hline
  \\begin{pmatrix}
      \\alpha_{11}&\\cdots&\\alpha_{1n}\\\\
      \\hdotsfor{3}\\\\
      \\alpha_{n1}&\\cdots&\\alpha_{nn}
  \\end{pmatrix}
  &\\large \\text{Left}&\\cellcolor{#00bde5}\\small \\textcolor{white}{\\text{\\bold{Right}}}
  &\\small \\text{small Small}\\\\
  \\hline
  \\multicolumn{4}{|c|}{\\text{Table Foot}}\\\\
  \\hline
\\end{array}
  ",
  gp = grid::gpar(fontsize = 22)
)
```

```{r complicated-equation, fig.height = 9, fig.width=9, out.width="60%"}
grid::grid.newpage()
grid.latex(
  "\\definecolor{gris}{gray}{0.9}
\\definecolor{noir}{rgb}{0,0,0}
\\definecolor{bleu}{rgb}{0,0,1}
\\fatalIfCmdConflict{false}
\\newcommand{\\pa}{\\left|}
\\begin{array}{c}
  \\LaTeX\\\\
  \\begin{split}
      |I_2| &= \\pa\\int_0^T\\psi(t)\\left\\{ u(a,t)-\\int_{\\gamma(t)}^a \\frac{d\\theta}{k} (\\theta,t) \\int_a^\\theta c(\\xi)
          u_t (\\xi,t)\\,d\\xi\\right\\}dt\\right|\\\\
      &\\le C_6 \\Bigg|\\pa f \\int_\\Omega \\pa\\widetilde{S}^{-1,0}_{a,-}
          W_2(\\Omega, \\Gamma_1)\\right|\\ \\right|\\left| |u|\\overset{\\circ}{\\to} W_2^{\\widetilde{A}}(\\Omega\\Gamma_r,T)\\right|\\Bigg|\\\\
      &\\\\
      &\\begin{pmatrix}
          \\alpha&\\beta&\\gamma&\\delta\\\\
          \\aleph&\\beth&\\gimel&\\daleth\\\\
          \\mathfrak{A}&\\mathfrak{B}&\\mathfrak{C}&\\mathfrak{D}\\\\
          \\boldsymbol{\\mathfrak{a}}&\\boldsymbol{\\mathfrak{b}}&\\boldsymbol{\\mathfrak{c}}&\\boldsymbol{\\mathfrak{d}}
      \\end{pmatrix}
      \\quad{(a+b)}^{\\frac{n}{2}}=\\sqrt{\\sum_{k=0}^n\\tbinom{n}{k}a^kb^{n-k}}\\quad
          \\Biggl(\\biggl(\\Bigl(\\bigl(()\\bigr)\\Bigr)\\biggr)\\Biggr)\\\\
      &\\forall\\varepsilon\\in\\mathbb{R}_+^*\\ \\exists\\eta>0\\ |x-x_0|\\leq\\eta\\Longrightarrow|f(x)-f(x_0)|\\leq\\varepsilon\\\\
      &\\det
      \\begin{bmatrix}
          a_{11}&a_{12}&\\cdots&a_{1n}\\\\
          a_{21}&\\ddots&&\\vdots\\\\
          \\vdots&&\\ddots&\\vdots\\\\
          a_{n1}&\\cdots&\\cdots&a_{nn}
      \\end{bmatrix}
      \\overset{\\mathrm{def}}{=}\\sum_{\\sigma\\in\\mathfrak{S}_n}\\varepsilon(\\sigma)\\prod_{k=1}^n a_{k\\sigma(k)}\\\\
      &\\Delta f(x,y)=\\frac{\\partial^2f}{\\partial x^2}+\\frac{\\partial^2f}{\\partial y^2}\\qquad\\qquad \\fcolorbox{noir}{gris}
          {n!\\underset{n\\rightarrow+\\infty}{\\sim} {\\left(\\frac{n}{e}\\right)}^n\\sqrt{2\\pi n}}\\\\
      &\\sideset{_\\alpha^\\beta}{_\\gamma^\\delta}{
      \\begin{pmatrix}
          a&b\\\\
          c&d
      \\end{pmatrix}}
      \\xrightarrow[T]{n\\pm i-j}\\sideset{^t}{}A\\xleftarrow{\\overrightarrow{u}\\wedge\\overrightarrow{v}}
          \\underleftrightarrow{\\iint_{\\mathds{R}^2}e^{-\\left(x^2+y^2\\right)}\\,\\mathrm{d}x\\mathrm{d}y}
  \\end{split}\\\\
  \\rotatebox{30}{\\sum_{n=1}^{+\\infty}}\\quad\\mbox{Mirror rorriM}\\reflectbox{\\mbox{Mirror rorriM}}
\\end{array}",
  gp = grid::gpar(fontsize = 22),
  render_mode = "path"
)
```



### What is not supported

MicroTeX is a **math formula renderer**, not a full LaTeX engine. The
following are outside its scope:

- **Document structure**: `\section`, `\begin{document}`, page layout,
  headers/footers, `\tableofcontents`
- **Package loading**: `\usepackage{}` --- all supported commands are
  built in
- **Paragraph text**: line breaking, hyphenation, justified paragraphs
- **TikZ / PGF** drawing commands
- **Images**: `\includegraphics`
- **Cross-references**: `\label`, `\ref`, `\cite`, bibliographies
- **Theorem environments**: `\begin{theorem}`, `\begin{proof}`
- **Lists**: `itemize`, `enumerate`, `description`
- **Some amsmath commands**: `\substack`, `\tag`, equation numbering

For most statistical graphics use cases --- axis labels, annotations,
legends, and in-plot formulas --- the supported feature set is more
than sufficient.

## Project-wide defaults

`latex_options()` sets defaults for `math_font` and `render_mode`, used
by `latex_grob()`, `grid.latex()`, `latex_dims()`, and `latex_tree()`
whenever the corresponding argument is not supplied at the call site.
Size is controlled at the grob level via `gp$fontsize` /
`gp$lineheight` (see *Basic usage*).

```{r options, eval=FALSE}
latex_options(math_font = "stix", render_mode = "typeface")

# Later calls pick these up automatically
grid.latex("\\sum_{i=1}^{n} i^{2}", gp = grid::gpar(fontsize = 14))

# Query current settings
latex_options()

# Reset to built-in defaults
reset_latex_options()
```

Explicit arguments always win. Setting `math_font` via
`latex_options()` also updates the MicroTeX engine default, so you
don't also need a separate font-setup call.

## User-defined macros

`define_macro()` registers zero-argument shorthands that are expanded
by text substitution before the expression reaches MicroTeX. Handy for
recurring notation:

```{r macros, fig.height=0.7, fig.width=3, out.width="50%"}
define_macro("RR", "\\mathbb{R}")
define_macro("eps", "\\varepsilon")

grid::grid.newpage()
grid.latex("\\forall \\eps > 0, \\eps \\in \\RR", gp = grid::gpar(fontsize = 24))

clear_macros()
```

Macro names must be ASCII letters. Expansion iterates to a fixed point,
so macros can reference other macros. Use `list_macros()` to see
currently registered ones, and `clear_macros()` (with no arguments) to
drop them all.

For parameterised macros (0--9 arguments) scoped to a single
expression, MicroTeX also accepts plain-TeX `\def`:

```{r def-inline, fig.height=0.7, fig.width=4, out.width="60%"}
grid::grid.newpage()
grid.latex(
  r"(\def\norm#1{\left\lVert #1 \right\rVert}
      \def\inner#1#2{\langle #1, #2 \rangle}
      \norm{\vec{v}} = \sqrt{\inner{\vec{v}}{\vec{v}}})",
  gp = grid::gpar(fontsize = 24)
)
```

`\def` definitions live only for the duration of the expression they
appear in, so they are the right tool for a parameterised abbreviation
local to one label. Reach for `define_macro()` instead when you want a
shorthand to persist across many plots in the same R session.

## Layout caching

Parsed layouts are memoised by `(tex, fontsize, math_font, render_mode,
...)`. Re-drawing the same formula --- for example, the same axis label
across many plots --- reuses the cached layout:

```{r cache, eval=FALSE}
latex_cache_info()       # size / max_size / hits / misses
latex_cache_limit(1024)  # raise or lower the LRU capacity
latex_cache_clear()      # wipe the cache (e.g. after re-loading fonts)
```

Set the limit to `0` to disable caching entirely.

## Introspecting a formula

`latex_tree()` returns the raw draw-record table plus bbox metadata,
useful for debugging alignment, counting glyphs, or building custom
grobs on top of the layout:

```{r tree}
tr <- latex_tree("\\frac{a}{b}")
tr
head(tr$records, 3)
```

## Debug overlay

Pass `debug = TRUE` to `latex_grob()` / `grid.latex()` to overlay
diagnostics on the rendered formula --- the full bounding box (dashed
gray), the baseline (solid red), and a dot at each draw record's
origin. Useful for checking vertical alignment between a formula and
surrounding grobs:

```{r debug, fig.height=1, fig.width=3, out.width="60%"}
grid::grid.newpage()
grid.latex("x^{2} + y_{i}", gp = grid::gpar(fontsize = 30), debug = TRUE)
```

## Comparison with alternatives

| Approach       | LaTeX required? | Device independent? | Vector? | Math coverage |
|:---------------|:---------------:|:-------------------:|:-------:|:-------------:|
| `tikzDevice`   | Yes             | No                  | Yes     | Full          |
| `xdvir`        | Yes             | No                  | Yes     | Full          |
| `latexpdf`     | Yes             | No                  | Yes     | Full (tables) |
| `latex2exp`    | No              | Yes                 | Yes     | Limited       |
| `plotmath`     | No              | Yes                 | Yes     | Limited       |
| **gridmicrotex** | **No**       | **Yes**             | **Yes** | **Broad**     |


## Graphics backend

The default graphics device on Windows (`windows()`) and macOS
(`quartz()`) may not find the bundled math fonts, producing warnings
like:

```
font family not found in Windows font database
```

To avoid this, switch to a modern graphics backend that uses
[systemfonts](https://CRAN.R-project.org/package=systemfonts) for font
resolution:

```r
# For knitr / R Markdown --- add to your setup chunk:
knitr::opts_chunk$set(dev = "ragg_png")

# For interactive use:
options(device = function(...) ragg::agg_png(tempfile(fileext = ".png"), ...))
```

Recommended backends:

| Backend               | Format | Package               |
|:----------------------|:-------|:----------------------|
| `ragg::agg_png()`     | PNG    | [ragg](https://CRAN.R-project.org/package=ragg)       |
| `svglite::svglite()`  | SVG    | [svglite](https://CRAN.R-project.org/package=svglite) |
| `grDevices::cairo_pdf()` | PDF | Base R (Cairo build)  |

Alternatively, use `render_mode = "path"` to bypass font lookup
entirely --- glyphs are drawn as vector paths, which works on all
devices but produces non-selectable text in PDF/SVG.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latex-grob.R
\name{latex_grob}
\alias{latex_grob}
\alias{grid.latex}
\title{Create a grid grob from a LaTeX expression}
\usage{
latex_grob(
  tex,
  x = grid::unit(0.5, "npc"),
  y = grid::unit(0.5, "npc"),
  default.units = "npc",
  hjust = 0.5,
  vjust = 0.5,
  rot = 0,
  math_font = "",
  max_width = 0,
  tex_style = "",
  input_mode = c("mixed", "math"),
  render_mode = c("typeface", "path"),
  justify = FALSE,
  line_break = c("greedy", "optimal"),
  debug = FALSE,
  name = NULL,
  gp = grid::gpar()
)

grid.latex(tex, ...)
}
\arguments{
\item{tex}{Character string of LaTeX math code.}

\item{x, y}{Position in grid coordinates.}

\item{default.units}{Units for x, y if given as numeric.}

\item{hjust, vjust}{Horizontal/vertical justification. Accepts the
usual numeric values in \verb{[0, 1]}. As a convenience, \code{hjust} also
accepts the strings \code{"left"}/\code{"bbleft"}, \code{"center"}/\code{"centre"}/
\code{"middle"}/\code{"bbcentre"}, and \code{"right"}/\code{"bbright"}; \code{vjust} accepts
\code{"bottom"}, \code{"center"}/\code{"centre"}/\code{"middle"}, \code{"top"}, and
\code{"baseline"}. \code{"baseline"} aligns the formula's math baseline with
the anchor point — handy for placing a formula in flowing text.}

\item{rot}{Rotation angle in degrees, counter-clockwise (default: 0).
Matches the \code{rot} parameter of \code{\link[grid]{textGrob}}.}

\item{math_font}{Name of the math font to use (e.g., \code{"stix"}).
Use \code{""} (default) for Lete Sans Math, which pairs with R's
default sans-serif text font.
See \code{\link{available_math_fonts}} for loaded fonts.}

\item{max_width}{Numeric maximum width in big points for automatic
line wrapping.  Use \code{0} (default) for no wrapping.}

\item{tex_style}{Character: TeX style override. One of \code{""}
(default; let the parser decide), \code{"display"}, \code{"text"},
\code{"script"}, or \code{"scriptscript"}. See
\code{\link{latex_grob}} for the semantics of each value.}

\item{input_mode}{How \code{tex} is interpreted before being parsed.
\code{"mixed"} (default) wraps the input in \code{\\text{...}} so the
string reads as ordinary text and \code{$...$} (or \code{\\(...\\)})
opens math mode, matching document-level LaTeX semantics. Useful for
labels that arrive from external sources mixing prose and math without
explicit \code{\\text{}} markers. \code{"math"} is the classic
MicroTeX behaviour --- the whole string is treated as math, so unwrapped
prose renders as spaced math italics. The default can be changed globally via
\code{\link{latex_options}(input_mode = "math")}. See \code{\link{latex_wrap}}
for details on the wrapping process.}

\item{render_mode}{Character string: \code{"typeface"} (default) renders
glyphs as native text using the math font, producing
selectable/accessible text in PDF and SVG output.
Bundled math fonts and any registered via \code{\link{load_math_font}}
are read directly from their OTF files --- no system-wide font
install is required.
Falls back to path mode automatically on devices that lack the
R \eqn{\geq} 4.3 glyph engine (e.g., the base \code{pdf()} device).
For selectable PDF output, prefer \code{\link[grDevices]{cairo_pdf}}.
\code{"path"} renders math symbols as filled vector paths (works on
all devices but text is not selectable in PDF/SVG).}

\item{justify}{Logical. When \code{TRUE}, wrapped text is stretched at
its interword spaces so every line but the last fills
\code{max_width} exactly. Requires \code{max_width}: it acts on the
lines the wrapper produces, so it does nothing on its own.
\code{FALSE} (default) leaves the right edge ragged, matching R's own
text drawing. In a narrow column, expect wide word spaces; mark the
words that may break with \code{\\-} to tighten them.}

\item{line_break}{How lines are chosen when wrapping.
\code{"greedy"} (default) fills each line as far as it will go and
never reconsiders. \code{"optimal"} chooses the breaks together so
the paragraph as a whole reads best, in the spirit of Knuth-Plass:
pulling one word down early can improve every later line, which a
greedy pass cannot see. Requires \code{max_width}, and costs a
little more layout time.}

\item{debug}{Logical; if \code{TRUE}, draws diagnostic overlays on the
grob --- the full bounding box (dashed gray), the baseline (solid
red), the depth line (dashed gray), and a small dot at each
MicroTeX draw record's origin. Useful for checking positioning and
diagnosing vertical alignment.}

\item{name}{Optional grob name.}

\item{gp}{Graphical parameters (see \code{\link[grid]{gpar}}).
Common entries: \code{col} (formula foreground), \code{fontfamily}
(text font), \code{fontsize} / \code{cex}
(formula size), and \code{lineheight} (multi-line spacing). See
\code{\link{latex_grob}} for how each of these flows through
MicroTeX.}

\item{...}{Additional arguments passed to \code{\link{latex_grob}}.}
}
\value{
A \code{grid} grob of class \code{"latexgrob"}.

Invisibly returns the grob.
}
\description{
Parses a LaTeX math expression and returns a grid grob object
that renders the formula using native grid graphics primitives.
The grob supports standard grid queries such as \code{grobWidth()},
\code{grobHeight()}, \code{grobX()}, and \code{grobY()}.

A convenience wrapper that creates a \code{\link{latex_grob}} and
immediately draws it on the current device via
\code{\link[grid]{grid.draw}}.
}
\details{
\subsection{Controlling TeX style with \code{tex_style}}{

\code{tex_style} selects the size-and-spacing regime MicroTeX applies to
the whole expression. It changes the \emph{style} (display vs. text), not
the font size --- size is always set via \code{gp$fontsize} / \code{gp$cex};
style-dependent shrinking (for \code{"script"} and \code{"scriptscript"}) is
applied on top of that size.
\itemize{
\item \code{""} (default): let the parser choose based on the delimiters in
\code{tex}. Inline delimiters (single \code{$}, or \verb{\\(...\\)}) produce
\code{"text"} style; display delimiters (double \verb{$$}, or \verb{\[...\]})
produce \code{"display"} style. If the string has no delimiters,
MicroTeX defaults to \code{"text"} style.
\item \code{"display"}: force display style. Large operators (\verb{\\sum},
\verb{\\int}, \verb{\\prod}) render at their full size, limits are placed
above/below rather than as subscripts/superscripts, and fractions
use full-size numerators and denominators. Useful when you want a
display-style equation inline in a label, legend, or
\code{element_latex()} title.
\item \code{"text"}: force text (inline) style. Big operators shrink to their
inline size and limits attach as scripts. The right choice for
formulas embedded in a line of prose.
\item \code{"script"}: force script style --- the size normally used for
first-level subscripts and superscripts. Produces a smaller,
tighter layout; mainly useful for callouts or sub-labels where a
compact equation is wanted.
\item \code{"scriptscript"}: force scriptscript style --- the smallest style,
used by TeX for doubly-nested scripts. Rarely needed on its own;
primarily for very dense annotations.
}

\code{tex_style} applies to the entire expression. To override the style
of a sub-expression from within \code{tex}, use the inline TeX commands
\verb{\\displaystyle}, \verb{\\textstyle}, \verb{\\scriptstyle}, or
\verb{\\scriptscriptstyle}.
}

\subsection{Graphical parameters (\code{gp})}{
\itemize{
\item \code{col}: default foreground color for the formula. Individual
elements can still be overridden with an inline \verb{\\textcolor}
command in the LaTeX string.
\item \code{fontfamily}: controls the font of text inside \verb{\\text} and \verb{\\mbox}
blocks. For example, \code{gpar(fontfamily = "serif")} renders \verb{\\text}
content in R's serif family. Any font available to R's graphics
system works --- base families (\code{"sans"}, \code{"serif"}, \code{"mono"}) as
well as fonts registered via \pkg{showtext} or \pkg{systemfonts}.
Math symbols always use the selected math font (see \code{math_font}).
Bold/italic text is controlled from within the LaTeX source
(\verb{\\textbf\{\}}, \verb{\\textit\{\}}, \verb{\\bf}, ...), not via \code{gp$fontface} ---
MicroTeX needs the style at layout time to size each run correctly,
so a \code{gpar()}-level face is not consulted.

\code{fontfamily} \emph{also} drives MicroTeX's layout metrics for non-math
text: the matching system font is resolved via \pkg{systemfonts},
a minimal metrics file is generated on first use and cached under
\code{tools::R_user_dir("gridmicrotex", "cache")}, so MicroTeX's
spacing of \verb{\\text} blocks stays in sync with what \pkg{grid}
actually draws. When \code{fontfamily} is unset, the R default
(\code{"sans"}) is used. No manual font loading is required for text
fonts; \code{\link[=load_math_font]{load_math_font()}} remains only for adding custom \strong{math}
fonts.
\item \code{fontsize} / \code{cex}: formula size is \code{fontsize * cex} big points
(default 20 * 1). Both math and text scale together. The effective
size is baked into the parsed layout, so downstream viewports that
inherit \code{cex} will not re-scale the grob (matching \code{textGrob}
semantics when \code{gp} is set explicitly).
\item \code{lineheight}: controls multi-line spacing (default 1.2). The
inter-line gap is \code{(lineheight - 1) * fontsize} big points.
}
}

\subsection{LaTeX document-level wrappers}{

The parser accepts raw output from \code{print.xtable()},
\code{knitr::kable()}, and similar functions that emit complete
\code{tabular} LaTeX. The following document-level constructs are
recognized and rewritten silently before the input reaches MicroTeX:

\strong{Removed (no visual effect):}
\itemize{
\item \verb{\%}-to-end-of-line comments (escaped \verb{\\\%} is preserved)
\item preamble: \verb{\\documentclass[...]\{...\}}, \verb{\\usepackage[...]\{...\}},
\verb{\\begin\{document\}} / \verb{\\end\{document\}}
\item title metadata: \verb{\\maketitle}, \verb{\\title\{...\}}, \verb{\\author\{...\}}
\item cross-reference labels: \verb{\\label\{...\}}
\item float wrappers: \verb{\\begin\{table\}} / \verb{\\end\{table\}}, \verb{\\begin\{figure\}} /
\verb{\\end\{figure\}} (and starred variants)
\item layout scopes: \verb{\\centering}, \verb{\\raggedright}, \verb{\\raggedleft},
\verb{\\flushleft}, \verb{\\flushright}
}

\strong{Rewritten:}
\itemize{
\item booktabs rules: \verb{\\toprule}, \verb{\\midrule}, \verb{\\bottomrule}, \verb{\\cmidrule}
are mapped to \verb{\\hline}. The optional column-range and trim
arguments of \verb{\\cmidrule} are discarded (MicroTeX has no concept of
partial-column rules).
\item \verb{\\caption[short]\{X\}} is extracted as \verb{\\text\{X\}\\\\} at its source
position, so a caption written after \verb{\\includegraphics} renders below
the figure and one written before a \code{tabular} renders above the table.
Full LaTeX instead positions the caption by float type regardless of
source order, and numbers it from a counter; there is no counter here.
Wrap the figure and its caption in \verb{\\begin\{array\}\{c\}...\\end\{array\}} to
centre them on each other (\verb{\\centering} is dropped --- a grob has no
page to centre against).
\item \verb{\\graphicspath\{\{dir/\}\}} and \verb{\\DeclareGraphicsExtensions\{...\}} are
consumed rather than typeset; the former's directories are searched.
}

\strong{Images:}
\itemize{
\item \verb{\\includegraphics[opts]\{file\}} draws a PNG, JPEG or SVG inline. The
starred form is accepted and behaves identically. \code{width}, \code{height}
and \code{scale} take any LaTeX length (\verb{\\textwidth} resolves against
\code{max_width}, and without one falls back to the file's own size with a
warning); \code{scale} multiplies whatever \code{width}/\code{height} settled on, and
\code{keepaspectratio} fits inside them instead of stretching to fill.
\code{angle} rotates the figure and grows the surrounding box to the rotated
bounds, as \verb{\\rotatebox} does. \code{origin}, \code{trim}, \code{clip} and \code{viewport}
are parsed but not applied, and warn once so the difference is not
silent; so does a length that cannot be read, or one that sizes the
figure to nothing.
\item The extension may be omitted, as in LaTeX: \code{{plots/fig}} finds
\code{plots/fig.svg}, then \code{.png}, \code{.jpg}, \code{.jpeg}.
\item An SVG is drawn as real vector and stays sharp at any output
resolution; a bitmap does not, and warns when it would be shown below
150 dpi. PDF and EPS are not supported --- save the figure as SVG
instead. A file that cannot be read --- missing, unsupported, or an
SVG with no \code{rsvg} installed --- warns and draws its name rather than
disappearing.
}

Anything not in this list is passed to MicroTeX unchanged. An unknown
command is not an error: MicroTeX typesets its name in red, which
makes unsupported markup easy to spot in the output.
}

\subsection{Parallelism}{

The MicroTeX engine keeps mutable C++ state for font caching and text
measurement. Rendering is safe single-threaded and under separate-process
backends such as \code{future::plan(multisession)}. It is \emph{not} safe
under forked backends (\code{parallel::mclapply()},
\code{future::plan(multicore)}) on Unix, because forked workers share that
state without synchronisation. Use a socket/multisession backend instead.
}
}
\examples{
\donttest{
  g <- latex_grob(r"($\fcolorbox{red}{yellow}{\frac{a}{b}}$)",
                  x = grid::unit(0.3, "npc"),
                  y = grid::unit(0.3, "npc"),
                  gp = grid::gpar(fontsize = 30))
  grid::grid.draw(g)
  # Red formula
  grid::grid.draw(latex_grob("$x^{2}$",
                             x = grid::unit(0.3, "npc"),
                             y = grid::unit(0.8, "npc"),
                             gp = grid::gpar(col = "red")))

                             # Rotated formula
  grid::grid.draw(latex_grob(r"($\colorbox{BurntOrange}{x^{2}} + y^{2}$)",
                             x = grid::unit(0.6, "npc"),
                             y = grid::unit(0.3, "npc"),
                             gp = grid::gpar(fontsize = 24),
                             rot = 45))

  grid.latex(r"($\textcolor{red}{x^{2}} + y^{2} = z^{2}$)",
             x = grid::unit(0.6, "npc"),
             y = grid::unit(0.8, "npc"),)
}
}
\seealso{
\code{\link{grid.latex}}, \code{\link{latex_dims}},
\code{\link{geom_latex}}, \code{\link{available_math_fonts}},
\code{\link{latex_wrap}}, \code{\link{latex_options}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{check_math_fonts}
\alias{check_math_fonts}
\title{Check math font status}
\usage{
check_math_fonts()
}
\value{
Invisibly returns the character vector of available math font
names.
}
\description{
Reports which \strong{math} fonts are loaded and available for
rendering: the MicroTeX version, the loaded math fonts, and whether the
bundled font files are present.
}
\details{
Text fonts are not covered, because they are not registered here ---
they are resolved on demand by \pkg{systemfonts} from
\code{gp$fontfamily}. Use \code{systemfonts::match_fonts()} to see
what a text family resolves to.
}
\examples{
check_math_fonts()
}
\seealso{
\code{\link{available_math_fonts}}, \code{\link{load_math_font}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown-style.R
\name{md_style}
\alias{md_style}
\title{Declarations for one markdown tag}
\usage{
md_style(...)
}
\arguments{
\item{...}{Named declarations.}
}
\value{
An object of class \code{gridmicrotex_md_style}.
}
\description{
A set of CSS declarations, for use as a named argument to
\code{\link{markdown_style}}. Argument names are the CSS property
names with underscores in place of hyphens, so \code{font_size} sets
\code{font-size}.
}
\details{
Lengths accept three forms: a bare number is \code{rem}, a multiple of
the body font size (\code{font_size = 2.5}); a string is whatever CSS
says it is (\code{"2.5em"}, \code{"12pt"}, \code{"150\%"}); and a
\code{\link[grid]{unit}} is absolute.

These are the supported properties, and where each one has an effect.
\emph{Inline} means it also works on a \code{<span>} and in
\code{\link{markdown_grob}}, which has no block layout; \emph{block}
means it needs \code{\link{markdown_box_grob}}.

\tabular{lll}{
\strong{property} \tab \strong{scope} \tab \strong{notes} \cr
\code{color} \tab inline + block \tab \cr
\code{font_size} \tab inline + block \tab \cr
\code{font_family} \tab inline + block \tab \cr
\code{font_weight} \tab inline + block \tab prose blocks only, see below \cr
\code{font_style} \tab inline + block \tab prose blocks only, see below \cr
\code{text_decoration} \tab inline + block \tab \code{underline},
\code{overline}, \code{line-through} \cr
\code{background} \tab inline + block \tab a fill behind the text \cr
\code{border} \tab inline \tab a frame; the inset is fixed \cr
\code{border_style} \tab inline \tab only \code{double} \cr
\code{border_radius} \tab inline \tab rounds the frame \cr
\code{box_shadow} \tab inline \tab \cr
\code{visibility} \tab inline \tab \code{hidden} keeps the space \cr
\code{vertical_align} \tab inline \tab \code{super}, \code{sub},
or a length \cr
\code{transform} \tab inline \tab \code{rotate()}, \code{scale()},
\code{scaleX(-1)} \cr
\code{line_height} \tab block \tab unitless, as \code{gpar()} wants it \cr
\code{margin_top}, \code{margin_bottom} \tab block \tab margins do not
collapse \cr
\code{margin_left}, \code{margin_right} \tab block \tab \cr
\code{padding_left}, \code{padding_right} \tab block \tab \cr
\code{padding_top}, \code{padding_bottom} \tab block \tab \cr
\code{margin}, \code{padding} \tab block \tab the CSS shorthand: one
to four lengths, in CSS's order \cr
\code{text_align} \tab block \tab \code{left}, \code{center},
\code{right} \cr
\code{border_left} \tab block \tab the \code{blockquote} bar \cr
\code{border_top} \tab block \tab the \code{hr} rule \cr
\code{border_bottom} \tab \code{tr} \tab a rule under each table row \cr
\code{border_color} \tab \code{table} \tab colour of the table's rules \cr
\code{table_layout} \tab \code{table} \tab \code{fixed} divides the
width between the columns so a wide table wraps \cr
\code{height} \tab block \tab the band an \code{hr} sits in \cr
\code{bullet} \tab \code{ul} \tab raw LaTeX for the marker glyph \cr
\code{marker_gap} \tab \code{ul}, \code{ol} \tab marker to text \cr
}

\code{font_size} also accepts CSS's keywords --- \code{xx-small} through
\code{xx-large}, plus \code{smaller} and \code{larger} --- taken from the
\code{\\tiny}..\code{\\Huge} ladder MicroTeX implements.

\strong{The \code{body} rule styles the box itself.} On any other tag,
\code{background}, \code{border}, \code{border_radius}, \code{padding}
and \code{margin} apply to that block. On \code{body} they apply to the
whole \code{\link{markdown_box_grob}} --- its fill, its frame, its
corner radius, and the space inside and outside it. That is the only
way to give a \code{\link{element_markdown}} title a background, since
the theme element takes no box arguments of its own:

\preformatted{body \{ background: grey95; padding: 8px;
        border: 1px solid grey60; border-radius: 4px \}}

An explicit \code{box_gp}, \code{padding}, \code{margin} or \code{r}
argument to \code{markdown_box_grob()} wins over the rule, the way an
inline style wins in CSS.

Anything else is an error --- unlike a pasted stylesheet, where an
unknown property is ignored the way a browser ignores it.

\strong{One limitation worth knowing.} \code{font_weight} and
\code{font_style} apply to blocks whose content is prose --- paragraphs,
headings, list items, block quotes, table cells and \code{<div>}s. They
do \emph{not} apply to \code{pre} or an image's alt text, which build
their own LaTeX and impose their own font handling. This is a MicroTeX
constraint rather than a choice: \code{\\text\{\}} resets the font
style, so emphasis has to be decided when the content is generated,
not wrapped around it afterwards.

\strong{What cannot be styled at all.} There is no small-caps
(\code{\\textsc} is not a MicroTeX command), no
\code{font-variant-numeric}, no right-to-left or bidirectional text,
and no padding inside an inline \code{border} --- MicroTeX has no
\code{\\fboxsep}, so that inset is fixed.
}
\examples{
md_style(color = "steelblue", font_size = 2.5, margin_top = 1.2)
}
\seealso{
\code{\link{markdown_style}}, \code{\link{markdown_box_grob}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid-builder.R
\name{.resolve_text_face}
\alias{.resolve_text_face}
\title{Resolve MicroTeX FontStyle bitmask to R font face}
\usage{
.resolve_text_face(style)
}
\arguments{
\item{style}{Integer font style bitmask.}
}
\value{
Character: \code{"plain"}, \code{"bold"}, \code{"italic"},
or \code{"bold.italic"}.
}
\description{
MicroTeX FontStyle: rm=1, bf=2, it=4, etc. (bitmask).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latex-grob.R
\name{grobMark}
\alias{grobMark}
\title{Look up a named anchor inside a LaTeX grob}
\usage{
grobMark(grob, name)
}
\arguments{
\item{grob}{A \code{latexgrob} returned by \code{\link{latex_grob}}.}

\item{name}{The mark name (the argument to \code{\\mark\{...\}}).}
}
\value{
A list with elements \code{x} and \code{y}, each a
\code{\link[grid]{unit}}. Mark coordinates are evaluated in the
grob's parent viewport. They assume an unrotated grob: with
\code{rot != 0} the returned position does not account for the
rotation.
}
\description{
Resolves a \code{\\mark\{name\}} that was placed inside the LaTeX
source to a pair of \pkg{grid} units in the grob's parent viewport.
The returned units already account for the grob's viewport position
and \code{hjust}/\code{vjust}, so you can pass them directly to grid
drawing functions to anchor other graphics on parts of the formula.
}
\examples{
\donttest{
  g <- latex_grob(r"($a\mark{eq}^2 = b + c^2$)",
                  x = grid::unit(0.5, "npc"),
                  y = grid::unit(0.5, "npc"))
  grid::grid.newpage(); grid::grid.draw(g)
  mk <- grobMark(g, "eq")
  grid::grid.points(mk$x, mk$y, pch = 19,
                    gp = grid::gpar(col = "red"))
}
}
\seealso{
\code{\link{latex_grob}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{latex_cache_limit}
\alias{latex_cache_limit}
\alias{latex_cache_clear}
\alias{latex_cache_info}
\title{Set the maximum number of entries kept in the LaTeX layout cache}
\usage{
latex_cache_limit(n = 512L)

latex_cache_clear()

latex_cache_info()
}
\arguments{
\item{n}{Non-negative integer cache capacity. Default is 512. Set
to \code{0} to disable caching.}
}
\value{
\itemize{
\item \code{latex_cache_limit}: Invisibly returns the previous limit.
\item \code{latex_cache_clear}: Invisibly returns \code{NULL}.
\item \code{latex_cache_info}: A list with elements \code{size}
(entries currently stored), \code{max_size}, \code{hits}, and
\code{misses}.
}
}
\description{
The cache stores parsed layout information for recently rendered
LaTeX expressions, keyed by the expression and relevant rendering
parameters (font, size, macros, etc.). This speeds up repeated
rendering of the same expressions, especially in loops or
interactive sessions. The default limit is 512 entries, which
should be sufficient for most use cases. When the limit is
exceeded, the least recently used entries are automatically
evicted.
}
\examples{
\donttest{
  latex_cache_limit(256)
  grid.latex("e^{i\\\\pi} + 1 = 0")
  latex_cache_info()
  latex_cache_clear()
}
}
\seealso{
\code{\link{latex_grob}}, \code{\link{latex_options}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latex-tree.R
\name{latex_tree}
\alias{latex_tree}
\title{Inspect the parsed layout of a LaTeX expression}
\usage{
latex_tree(
  tex,
  math_font = "",
  max_width = 0,
  tex_style = "",
  input_mode = c("mixed", "math"),
  render_mode = c("typeface", "path"),
  gp = grid::gpar()
)
}
\arguments{
\item{tex}{Character string of LaTeX math code.}

\item{math_font}{Name of the math font to use (e.g., \code{"stix"}).
Use \code{""} (default) for Lete Sans Math, which pairs with R's
default sans-serif text font.
See \code{\link{available_math_fonts}} for loaded fonts.}

\item{max_width}{Numeric maximum width in big points for automatic
line wrapping.  Use \code{0} (default) for no wrapping.}

\item{tex_style}{Character: TeX style override. One of \code{""}
(default; let the parser decide), \code{"display"}, \code{"text"},
\code{"script"}, or \code{"scriptscript"}. See
\code{\link{latex_grob}} for the semantics of each value.}

\item{input_mode}{How \code{tex} is interpreted before being parsed.
\code{"mixed"} (default) wraps the input in \code{\\text{...}} so the
string reads as ordinary text and \code{$...$} (or \code{\\(...\\)})
opens math mode, matching document-level LaTeX semantics. Useful for
labels that arrive from external sources mixing prose and math without
explicit \code{\\text{}} markers. \code{"math"} is the classic
MicroTeX behaviour --- the whole string is treated as math, so unwrapped
prose renders as spaced math italics. The default can be changed globally via
\code{\link{latex_options}(input_mode = "math")}. See \code{\link{latex_wrap}}
for details on the wrapping process.}

\item{render_mode}{Character string: \code{"typeface"} (default) renders
glyphs as native text using the math font, producing
selectable/accessible text in PDF and SVG output.
Bundled math fonts and any registered via \code{\link{load_math_font}}
are read directly from their OTF files --- no system-wide font
install is required.
Falls back to path mode automatically on devices that lack the
R \eqn{\geq} 4.3 glyph engine (e.g., the base \code{pdf()} device).
For selectable PDF output, prefer \code{\link[grDevices]{cairo_pdf}}.
\code{"path"} renders math symbols as filled vector paths (works on
all devices but text is not selectable in PDF/SVG).}

\item{gp}{Graphical parameters (see \code{\link[grid]{gpar}}).
Common entries: \code{col} (formula foreground), \code{fontfamily}
(text font), \code{fontsize} / \code{cex}
(formula size), and \code{lineheight} (multi-line spacing). See
\code{\link{latex_grob}} for how each of these flows through
MicroTeX.}
}
\value{
A list with class \code{"latex_tree"} containing:
\describe{
\item{\code{records}}{Data frame of draw records (one row per
glyph, path, line, rect, or text block). Columns include
\code{type}, \code{x}, \code{y}, \code{glyph}, \code{font_size},
\code{color}, \code{text}, \code{codepoint}, \code{font_file}.}
\item{\code{bbox}}{Named numeric vector with \code{width},
\code{height}, \code{depth}, \code{baseline} (all in big points).}
\item{\code{tex}}{The (macro-expanded) input string.}
\item{\code{render_mode}}{Rendering mode used for the layout.}
}
}
\description{
Returns the raw draw-record table produced by MicroTeX's layout pass
together with the bounding-box metadata. Useful for debugging
alignment issues, building custom grobs on top of the layout, or
counting glyphs/paths/rules in a formula.
}
\examples{
\donttest{
  tree <- latex_tree("\\\\frac{a}{b}")
  print(tree)
  head(tree$records)
}
}
\seealso{
\code{\link{latex_grob}}, \code{\link{latex_dims}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{load_font}
\alias{load_font}
\title{Load a math font from an OTF file}
\usage{
load_font(otf_path)
}
\arguments{
\item{otf_path}{Path to the OTF/TTF font file.}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
Loads an OTF/TTF math font into MicroTeX's internal font registry. The
font's OpenType MATH table is parsed directly in C++ and the required
metrics are synthesised on the fly. You can download free math fonts like
Latin Modern Math (default math fonts in LaTeX) and load it with
\code{load_font()} to use it for math rendering.
}
\details{
The font is also registered with the \pkg{systemfonts} package so it
can be selected for surrounding plot text via
\code{gp = gpar(fontfamily = "...")} without being installed
system-wide.
}
\section{Text fonts}{

This function is only for \strong{math} fonts (fonts with an
OpenType MATH table). Plain text fonts used inside \code{\\text\{\}}
blocks are resolved automatically by \pkg{systemfonts} from the
\code{gp$fontfamily} argument --- no \code{load_font()} call required.
}

\examples{
\donttest{
  # Load a math font from a local OTF file. Here we point at the
  # bundled STIX font so the example is self-contained and loaded.
  # You don't need to load the bundled fonts to use them — they're registered
  # with systemfonts on first render — but this shows how to load a custom font.
  # in practice you would pass the path to any OTF with an OpenType MATH table.
  otf <- system.file("fonts", "STIXTwoMath-Regular.otf",
                     package = "gridmicrotex")
  load_font(otf)
  available_math_fonts()
}
}
\seealso{
\code{\link{available_math_fonts}}, \code{\link{latex_options}},
\code{\link{latex_grob}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown-highlight.R
\name{register_highlighter}
\alias{register_highlighter}
\title{Add a syntax highlighting grammar}
\usage{
register_highlighter(lang, file)
}
\arguments{
\item{lang}{Language name, as written after the opening fence. Case is
ignored. Registering a name that is already built in replaces it.}

\item{file}{Path to a KDE syntax XML grammar.}
}
\value{
Invisibly, \code{lang}.
}
\description{
Registers a KDE/Kate syntax definition so that fenced code blocks
tagged with \code{lang} are highlighted by
\code{\link{markdown_box_grob}}. Ten languages are built in --- see
\code{\link{available_highlighters}} --- and this is how to add
another.
}
\details{
The grammar is an XML file in the format used by Kate, KDevelop and,
through \pkg{skylighting}, Pandoc. The simplest way to write one is to
copy a built-in grammar and edit it:

\preformatted{
  file.copy(system.file("highlight", "python.xml",
                        package = "gridmicrotex"),
            "mylang.xml")
}

Because the format is KDE's, the several hundred definitions upstream
are a useful \emph{reference} when writing your own --- one XML file
per language, catalogued at \url{https://kate-editor.org/syntax/},
repository at
\url{https://invent.kde.org/frameworks/syntax-highlighting}.

Many load and work directly: of twenty sampled, thirteen registered,
including \code{python} (249 contexts), \code{css}, \code{yaml},
\code{makefile} and \code{go}. The seven that did not --- \code{bash},
\code{ruby}, \code{perl}, \code{rust}, \code{lua},
\code{javascript} and \code{markdown} --- all use \emph{dynamic}
rules, which substitute part of the match into a later pattern and are
not implemented.

Cross-language includes (\code{##Alerts}, \code{##Doxygen} and
friends) are skipped rather than followed. Those only add TODO/FIXME
marks \emph{inside} a comment, so the comment is still a comment; the
marks are all that is lost.

They also carry their own licences, mostly GPL or LGPL, which is why
none is bundled with this MIT-licensed package. Borrowing from one
locally is your own decision; redistributing it is subject to its
licence.

Colours come from the \code{defStyleNum} of each \code{<itemData>},
which maps onto the same CSS class names \pkg{knitr} and Pandoc use
(\code{kw} for a keyword, \code{co} for a comment, \code{st} for a
string, and so on), so a grammar needs no colour information of its
own --- restyle with \code{\link{markdown_style}}.

At each position the current context's rules are tried in document
order and the first to match wins. Contexts, \code{IncludeRules},
\code{lookAhead} and \code{fallthroughContext} all work, and the
context stack carries across lines, so a string or block comment may
span them.

A rule this engine cannot represent is skipped when it would have
stayed in the current context --- costing colour on the text it
matched and nothing more --- but \emph{refuses the grammar} when it
would have switched, because a lost context switch leaves the machine
in the wrong state and paints the rest of the file wrongly. The
refusal names the construct.
}
\examples{
# Registering a grammar under a name of your own.
f <- system.file("highlight", "python.xml", package = "gridmicrotex")
register_highlighter("mypython", f)
"mypython" \%in\% available_highlighters()
}
\seealso{
\code{\link{available_highlighters}},
\code{\link{markdown_box_grob}}, \code{\link{markdown_style}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown-highlight.R
\name{available_highlighters}
\alias{available_highlighters}
\title{Syntax highlighting languages available}
\usage{
available_highlighters()
}
\value{
A character vector of language names, sorted.
}
\description{
Names accepted after an opening code fence in
\code{\link{markdown_box_grob}}. Includes both the built-in grammars
and any added with \code{\link{register_highlighter}}.
}
\details{
Several common aliases also work and are not listed here, among them
\code{py}, \code{sh}, \code{c++}, \code{yml}, \code{jl} and
\code{tex}. A fence naming anything else renders as plain monospace
text, without a warning.
}
\examples{
available_highlighters()
}
\seealso{
\code{\link{register_highlighter}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2-integration.R
\docType{data}
\name{geom_latex}
\alias{geom_latex}
\alias{GeomLatex}
\title{A ggplot2 geom for LaTeX math labels}
\usage{
geom_latex(
  mapping = NULL,
  data = NULL,
  stat = "identity",
  position = "identity",
  ...,
  fontsize = 11,
  math_font = "",
  lineheight = 1.2,
  max_width = 0,
  input_mode = c("mixed", "math"),
  render_mode = c("typeface", "path"),
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}}. If specified and
\code{inherit.aes = TRUE} (the default), it is combined with the default mapping
at the top level of the plot. You must supply \code{mapping} if there is no plot
mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{stat}{The statistical transformation to use on the data for this layer.
When using a \verb{geom_*()} function to construct a layer, the \code{stat}
argument can be used to override the default coupling between geoms and
stats. The \code{stat} argument accepts the following:
\itemize{
\item A \code{Stat} ggproto subclass, for example \code{StatCount}.
\item A string naming the stat. To give the stat as a string, strip the
function name of the \code{stat_} prefix. For example, to use \code{stat_count()},
give the stat as \code{"count"}.
\item For more information and other ways to specify the stat, see the
\link[ggplot2:layer_stats]{layer stat} documentation.
}}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{...}{Other arguments passed to \code{\link[ggplot2]{layer}}.}

\item{fontsize}{Default font size in points. Overridden by the \code{size}
aesthetic if mapped.}

\item{math_font}{Name of the math font to use (e.g., \code{"stix"}).}

\item{lineheight}{Multi-line height multiplier (default 1.2), matching
\code{grid::gpar()} semantics.}

\item{max_width}{Maximum width in big points for automatic line
wrapping (default: 0, no wrapping).}

\item{input_mode}{How \code{tex} is interpreted before being parsed.
\code{"mixed"} (default) wraps the input in \code{\\text{...}} so the
string reads as ordinary text and \code{$...$} (or \code{\\(...\\)})
opens math mode, matching document-level LaTeX semantics. Useful for
labels that arrive from external sources mixing prose and math without
explicit \code{\\text{}} markers. \code{"math"} is the classic
MicroTeX behaviour --- the whole string is treated as math, so unwrapped
prose renders as spaced math italics. The default can be changed globally via
\code{\link{latex_options}(input_mode = "math")}. See \code{\link{latex_wrap}}
for details on the wrapping process.}

\item{render_mode}{Character string: \code{"typeface"} (default) renders
glyphs as native text using the math font, producing
selectable/accessible text in PDF and SVG output.
Bundled math fonts and any registered via \code{\link{load_math_font}}
are read directly from their OTF files --- no system-wide font
install is required.
Falls back to path mode automatically on devices that lack the
R \eqn{\geq} 4.3 glyph engine (e.g., the base \code{pdf()} device).
For selectable PDF output, prefer \code{\link[grDevices]{cairo_pdf}}.
\code{"path"} renders math symbols as filled vector paths (works on
all devices but text is not selectable in PDF/SVG).}

\item{na.rm}{If \code{FALSE}, the default, missing values are removed with
a warning. If \code{TRUE}, missing values are silently removed.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetics,
rather than combining with them. This is most useful for helper functions
that define both data and aesthetics and shouldn't inherit behaviour from
the default plot specification, e.g. \code{\link[ggplot2:annotation_borders]{annotation_borders()}}.}
}
\value{
A ggplot2 layer.
}
\description{
Renders LaTeX math expressions as native grid grobs within a ggplot2 plot.
Each label is parsed and laid out by MicroTeX, producing resolution-independent
vector output.
}
\section{Aesthetics}{

\code{geom_latex()} understands the following aesthetics (required aesthetics
are in bold):
\itemize{
\item \strong{\code{x}}
\item \strong{\code{y}}
\item \strong{\code{label}} — LaTeX math string
\item \code{size} — font size in points (default: 11)
\item \code{colour} — text colour (default: \code{"black"})
\item \code{angle} — rotation angle in degrees (default: 0)
\item \code{hjust} — horizontal justification, 0–1 (default: 0.5)
\item \code{vjust} — vertical justification, 0–1 (default: 0.5)
\item \code{alpha} — transparency (default: 1)
}
}

\examples{
\donttest{
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)
  df <- data.frame(
    x = 1:3, y = 1:3,
    eq = c("x^2", "\\\\frac{a}{b}", "\\\\sum_{i=1}^n x_i")
  )
  ggplot(df, aes(x, y, label = eq)) + geom_latex()

  # Use annotate() for single annotations (no legend, no data frame needed)
  ggplot(mtcars, aes(wt, mpg)) + geom_point() +
    annotate("latex", x = 4, y = 30,
             label = r"($\hat{y} = \beta_0 + \beta_1 x$)")
}
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid-builder.R
\name{.build_glyph_grob}
\alias{.build_glyph_grob}
\title{Build a single batched glyphGrob from collected glyph data}
\usage{
.build_glyph_grob(ids, x, y, sizes, cols, font_files, depth = 0)
}
\arguments{
\item{ids}{Integer vector of glyph IDs.}

\item{x, y}{Numeric vectors of glyph positions (already y-flipped, in bigpts).}

\item{sizes}{Numeric vector of font sizes.}

\item{cols}{Character vector of colors.}

\item{font_files}{Character vector of font file paths.}

\item{depth}{Depth below the baseline in bigpts (default 0).}
}
\value{
A \code{grid::glyphGrob} or \code{NULL}.
}
\description{
Creates a \code{grid::glyphGrob} containing all math glyphs,
using \code{grDevices::glyphInfo()} to describe glyph IDs, positions,
sizes, colors, and fonts. Each unique font file becomes an entry
in the \code{glyphFontList}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{load_math_font}
\alias{load_math_font}
\title{Load a math font from an OTF file}
\usage{
load_math_font(otf_path)
}
\arguments{
\item{otf_path}{Path to the OTF/TTF font file.}
}
\value{
Invisibly returns \code{NULL}.
}
\description{
Loads an OTF/TTF \strong{math} font --- one carrying an OpenType MATH
table --- into MicroTeX's internal font registry. The MATH table is
parsed directly in C++ and the required metrics are synthesised on the
fly. You can download a free math font such as Latin Modern Math
(the LaTeX default) and load it for math rendering.
}
\details{
The font is also registered with the \pkg{systemfonts} package so it
can be selected for surrounding plot text via
\code{gp = gpar(fontfamily = "...")} without being installed
system-wide.

Plain \strong{text} fonts --- those used inside \code{\\text\{\}} blocks
--- need no loading at all. They are resolved automatically by
\pkg{systemfonts} from \code{gp$fontfamily}, or per run with
\code{\\gmfontfamily\{\}\{\}}.
}
\examples{
\donttest{
  # Load a math font from a local OTF file. Here we point at the
  # bundled STIX font so the example is self-contained and loaded.
  # You don't need to load the bundled fonts to use them — they're registered
  # with systemfonts on first render — but this shows how to load a custom font.
  # in practice you would pass the path to any OTF with an OpenType MATH table.
  otf <- system.file("fonts", "STIXTwoMath-Regular.otf",
                     package = "gridmicrotex")
  load_math_font(otf)
  available_math_fonts()
}
}
\seealso{
\code{\link{available_math_fonts}}, \code{\link{check_math_fonts}},
\code{\link{latex_options}}, \code{\link{latex_grob}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown-style.R
\name{markdown_style}
\alias{markdown_style}
\title{A style for markdown rendering}
\usage{
markdown_style(base = NULL, css = NULL, ...)
}
\arguments{
\item{base}{\code{NULL} (default) for the built-in defaults, the name
of a bundled preset such as \code{"github"}, or an existing
\code{markdown_style} to extend.}

\item{css}{A stylesheet: CSS text, or a path to a \code{.css} file.
Read as a file when it names one.}

\item{...}{Named tag overrides, each an \code{\link{md_style}}. Use a
leading dot for a class, e.g. \code{.note = md_style(...)}.}
}
\value{
An object of class \code{gridmicrotex_markdown_style}.
}
\description{
Builds the style used by \code{\link{markdown_box_grob}} and
\code{\link{markdown_grob}}: a small CSS cascade over the markdown
tags. One constructor covers creating a style, starting from a preset,
and extending an existing one. For the properties themselves, and
which are honoured where, see \code{\link{md_style}}.
}
\details{
Tags are named as in HTML, so a stylesheet reads the way a CSS author
expects: \code{body} (the document root, which every other tag
inherits from --- and which also styles the box itself, see
\code{\link{md_style}}), \code{p}, \code{h1} ... \code{h6}, \code{ul},
\code{ol}, \code{li}, \code{blockquote}, \code{pre} (a code block),
\code{code} (an inline code span), \code{strong} and \code{em} (what
markdown's \code{**} and \code{*} produce), \code{table}, \code{tr},
\code{td}, \code{th}, \code{hr}, \code{img}, \code{a} (a link),
\code{math} (a paragraph that is nothing but \code{$$...$$}),
\code{footnote}, \code{div} and \code{span}.

The table tags nest as they do in HTML: \code{tr}, \code{td} and
\code{th} inherit through \code{table}, so \code{table \{ color: \}}
reaches the cells. \code{background} on \code{tr} fills the row, on
\code{td}/\code{th} the individual cell.

Four things can style a document, and they resolve in CSS's own order:
the built-in defaults, then a type selector (\code{h1}), then a class
selector (\code{.note}), then an inline \code{style} attribute. Ties
are broken by document order, so a later rule wins. Inheritable
properties (\code{color}, the \code{font-*} family,
\code{line-height}, \code{text-align}) fall through into nested
containers, so \code{blockquote \{ color: grey40 \}} greys everything
quoted; box properties (margins, padding, borders) do not.

The supported CSS is a deliberately small subset: type selectors,
class selectors and selector lists (\code{h1, h2}). Combinators,
pseudo-classes, attribute selectors, \code{#id} and at-rules are
skipped rather than raised, so an existing stylesheet can be handed
over and the parts that apply still take effect.
}
\examples{
markdown_style()
markdown_style("github", h1 = md_style(color = "firebrick"))
markdown_style(css = "h1 { color: steelblue } .note { padding-left: 2em }")
}
\seealso{
\code{\link{md_style}}, \code{\link{markdown_box_grob}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid-builder.R
\name{cubic_bezier}
\alias{cubic_bezier}
\title{Approximate a cubic bezier curve with line segments}
\usage{
cubic_bezier(x0, y0, x1, y1, x2, y2, x3, y3, n = 16)
}
\description{
Approximate a cubic bezier curve with line segments
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/macros.R
\name{define_macro}
\alias{define_macro}
\alias{clear_macros}
\alias{list_macros}
\title{Define a user-level LaTeX macro}
\usage{
define_macro(name, definition)

clear_macros(name = NULL)

list_macros()
}
\arguments{
\item{name}{Macro name \strong{without} the leading backslash. For
\code{clear_macros}, the macro name to drop, or \code{NULL}
(default) to clear all.}

\item{definition}{LaTeX source the macro expands to.}
}
\value{
\itemize{
\item \code{define_macro}: Invisibly returns \code{NULL}.
\item \code{clear_macros}: Invisibly returns \code{NULL}.
\item \code{list_macros}: A named character vector mapping
macro names to their expansions. Empty if no macros are defined.
}
}
\description{
Registers a zero-argument shorthand that is expanded by text
substitution before the expression reaches the MicroTeX parser.
Useful for domain-specific notation (e.g. \code{\\RR} for
\code{\\mathbb\{R\}}) you reuse across many plots.
}
\section{Choosing between this and \code{\\newcommand}}{

MicroTeX also accepts \code{\\newcommand} and plain-TeX \code{\\def}
written inside the expression itself, and those are the more capable
form: they take up to nine arguments, which \code{define_macro()} does
not --- it substitutes text and nothing else.

\preformatted{
  # parameterised, but local to this one expression
  grid.latex(r"(\\def\\norm#1{\\left\\lVert #1 \\right\\rVert}
                \\norm{\\vec{v}})")
}

What they cannot do is persist: the user-macro table is cleared at the
start of every parse, so a \code{\\newcommand} written in one call is
gone by the next. That is the one thing \code{define_macro()} is for.
Use \code{\\newcommand} / \code{\\def} for an abbreviation local to a
single label, and \code{define_macro()} for notation you want available
to every label in a script.
}

\examples{
\donttest{
  define_macro("RR", "\\\\mathbb{R}")
  define_macro("eps", "\\\\varepsilon")
  grid::grid.newpage()
  grid.latex("\\\\forall \\\\eps > 0, \\\\eps \\\\in \\\\RR")
  clear_macros()
}
}
\seealso{
\code{\link{latex_grob}}, \code{\link{latex_options}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2-markdown.R
\name{element_markdown}
\alias{element_markdown}
\title{A ggplot2 theme element for markdown text}
\usage{
element_markdown(
  math_font = "",
  fontsize = NULL,
  lineheight = 1.2,
  max_width = 0,
  render_mode = c("typeface", "path"),
  justify = FALSE,
  style = NA,
  width = NA,
  ...
)
}
\arguments{
\item{math_font}{Name of the math font to use (e.g. \code{"stix"}).}

\item{fontsize}{Convenience alias for \code{size}; forwarded to
\code{ggplot2::element_text()} as the text size in points.
\code{NULL} (default) uses the theme's inherited size.}

\item{lineheight}{Multi-line height multiplier (default 1.2).}

\item{max_width}{Maximum width in big points for automatic line
wrapping (default: 0, no wrapping).}

\item{render_mode}{\code{"typeface"} (default) or \code{"path"}.}

\item{justify}{Logical; justify wrapped lines. Requires
\code{max_width}.}

\item{style}{A \code{\link{markdown_style}} object, CSS text, or the
path to a \code{.css} file, applied to labels drawn through this
theme element. \code{NA}, the default, means unset --- the global
\code{\link{latex_options}(markdown_style = )} applies instead. Only
the properties that compile to LaTeX have an effect, unless the label
is laid out as blocks (see \emph{Block labels}). See
\code{\link{md_style}}.}

\item{width}{Wrapping measure for the label, as a
\code{\link[grid]{unit}}. \code{NA}, the default, means unset: the
label is sized to its content and does not wrap.
\code{unit(1, "npc")} is the useful value for a plot title, whose
cell really is the full plot width. See \emph{Block labels}.}

\item{...}{Additional arguments passed to
\code{ggplot2::element_text()} (e.g. \code{colour}, \code{hjust}).}
}
\value{
An S7 object of class \code{element_markdown}, inheriting from
\code{ggplot2::element_text}.
}
\description{
Use as a theme element for axis titles, axis labels, plot titles or any
other text element. The label is parsed as markdown --- including
\code{$...$} math --- and rendered via MicroTeX.
}
\details{
This is an S7 subclass of \code{ggplot2::element_text}, so it inherits
the standard text properties (size, colour, hjust, ...) from the theme
and merges correctly with inherited theme entries.
}
\section{Block labels}{

A label with real block structure --- a heading, a list, a table, a
rule, or more than one paragraph --- is laid out by
\code{\link{markdown_box_grob}} rather than flattened into one run, so
list markers, indents and block spacing survive. That makes a title
like this work:

\preformatted{labs(title = "## Findings\\n\\n- slope $\\\\beta_1$\\n- *p* < 0.001")}

The box's own background, border, padding and corner radius come from
the stylesheet's \code{body} rule --- \code{style = "body \{ background:
grey95; padding: 8px \}"} --- not from arguments here. See
\code{\link{markdown_style}}.

Three details follow from how ggplot2 measures theme elements:

\itemize{
\item \strong{Wrapping is opt-in.} Without \code{width} the label is
sized to its content, because ggplot2 asks an element for its
height before placing it, when a relative width would resolve
against the whole device rather than the element's cell.
\item \strong{Axis tick labels are never laid out as blocks},
whatever they contain, for the same reason.
\item \strong{A rotated label is never laid out as blocks.} The box
cannot rotate, so a label with both blocks and a non-zero
\code{angle} keeps the angle, is rendered as a single run, and
warns. A \code{width} given with an angle becomes the run's
wrapping measure instead.
}

\code{math_font}, \code{render_mode} and \code{justify} are not
forwarded to the box; a block label takes those from
\code{\link{latex_options}}.

Note that \pkg{ggtext} also exports a function called
\code{element_markdown()}. If both packages are attached, the one loaded
later wins; call \code{gridmicrotex::element_markdown()} explicitly to
be unambiguous. The two are not interchangeable --- ggtext renders
HTML/CSS and images, this renders LaTeX math.
}

\examples{
\donttest{
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)
  ggplot(mtcars, aes(wt, mpg)) + geom_point() +
    labs(x = "**weight** in $10^3$ lbs") +
    theme(axis.title.x = gridmicrotex::element_markdown())
}
}
}
\seealso{
\code{\link{markdown_grob}}, \code{\link{element_latex}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{latex_options}
\alias{latex_options}
\alias{reset_latex_options}
\title{Set or query package-wide LaTeX rendering defaults}
\usage{
latex_options(
  math_font = NULL,
  render_mode = NULL,
  tex_style = NULL,
  input_mode = NULL,
  justify = NULL,
  line_break = NULL,
  markdown_style = NULL
)

reset_latex_options()
}
\arguments{
\item{math_font}{Math font name or alias (see
\code{\link{available_math_fonts}}).}

\item{render_mode}{Either \code{"typeface"} or \code{"path"}.}

\item{tex_style}{TeX style override. One of \code{""} (let the parser
decide), \code{"display"}, \code{"text"}, \code{"script"}, or
\code{"scriptscript"}. \code{"display"} forces large operators with
limits placed over/under, useful for inline labels that should still
look like display equations.}

\item{input_mode}{How the input string is interpreted before being
handed to MicroTeX. \code{"mixed"} (default) wraps the string in
\code{\\text{...}} so it reads as ordinary text, with \code{$...$}
(and \code{\\(...\\)}) opening math mode --- the document-level
LaTeX convention. Useful when consuming labels from other packages
that mix prose and math without explicit \code{\\text{}} markers.
\code{"math"} treats the whole string as math --- the classic
MicroTeX behaviour, where letters render as math italics and
unwrapped prose looks wrong.}

\item{justify}{Logical. When \code{TRUE}, wrapped text is stretched at
its interword spaces so every line but the last fills
\code{max_width} exactly. Has no effect without \code{max_width},
since it acts on the lines the wrapper produces. \code{FALSE}
(default) leaves the right edge ragged, matching R's own text
drawing. In a narrow column, justifying alone opens noticeably wide
word spaces; mark the words that may break with \code{\\-}.}

\item{line_break}{How lines are chosen when wrapping.
\code{"greedy"} (default) fills each line as far as it will go and
never reconsiders. \code{"optimal"} chooses the breaks together so
the paragraph as a whole reads best, in the spirit of Knuth-Plass:
pulling one word down early can improve every later line, which a
greedy pass cannot see. Requires \code{max_width}, and costs a
little more layout time.}

\item{markdown_style}{Default style for \code{\link{markdown_grob}} and
\code{\link{markdown_box_grob}}: a \code{\link{markdown_style}}
object, CSS text, or a path to a \code{.css} file.}
}
\value{
Invisibly returns the previous settings (a list). With no
arguments, returns the current settings visibly.
}
\description{
A single entry point for project-wide defaults used by
\code{\link{latex_grob}}, \code{\link{grid.latex}},
\code{\link{latex_dims}}, and \code{\link{latex_tree}}. Options set
here are applied only when the corresponding argument is \emph{not}
supplied at the call site, so explicit arguments always win.
}
\details{
Calling \code{latex_options()} with no arguments returns the current
settings (a list whose \code{NULL} entries mean "use the built-in
default"). Supply one or more named arguments to update them.

Font size and line spacing are controlled via \code{gp} parameters
(\code{fontsize}, \code{cex}, \code{lineheight}) at the grob level
--- see \code{\link{latex_grob}}.
}
\examples{
\donttest{
  latex_options(math_font = "stix", render_mode = "typeface")
  grid.latex("\\\\sum_{i=1}^{n} i^{2}", gp = grid::gpar(fontsize = 14))
  reset_latex_options()
}
}
\seealso{
\code{\link{available_math_fonts}}, \code{\link{latex_grob}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{latex_wrap}
\alias{latex_wrap}
\title{Wrap standard text for math-first LaTeX renderers}
\usage{
latex_wrap(tex, input_mode = c("mixed", "math"))
}
\arguments{
\item{tex}{\code{character}. The string or vector of strings to be processed.}

\item{input_mode}{\code{character}. A length-one character vector dictating the
parsing strategy. If \code{"mixed"} (default), the string is tokenized and text
is wrapped. If \code{"math"}, the parser is bypassed and the string is returned
unmodified, assuming the user has provided a pure math equation.}
}
\value{
A \code{character} vector of the same length as \code{tex}, formatted for
math-mode LaTeX rendering.
}
\description{
Parses character strings to safely isolate standard natural language from
LaTeX math environments. Standard text is wrapped in \verb{\\text\{\}} blocks, while
equations, display math, and specific LaTeX environments are preserved verbatim.
This is heavily optimized for passing mixed-content strings (like plot titles
or axis labels) to pure-math typesetting engines like MicroTex. The conversion
is not perfect, but it should handle most common cases without user intervention.
}
\details{
\code{latex_wrap()} operates as a state-machine tokenizer to ensure that valid LaTeX
math is not corrupted by the text-wrapping process. It features:
\itemize{
\item \strong{Delimiter Preservation}: Standard inline (\code{$}, \verb{\\(}) and block (\verb{$$}, \verb{\[})
math delimiters are recognized and preserved.
\item \strong{Environment Tracking}: Complex nested environments (e.g., \verb{\\begin\{matrix\}})
are safely extracted and bypassed.
\item \strong{Newline Conversion}: R newline characters (\verb{\\n}) occurring outside of math
environments are automatically converted to LaTeX line breaks (\verb{\\\\}) inside
the \verb{\\text\{\}} wrapper.
\item \strong{Literal Escapes}: Escaped LaTeX literals (e.g., \verb{\\$}, \verb{\\\%}, \verb{\\#}) are
safely passed into the \verb{\\text\{\}} block without triggering math modes. The
escape character for \verb{\\$} is automatically resolved for MicroTex compatibility.
}
}
\examples{
# "mixed" mode (default) safely wraps text and preserves inline math
latex_wrap(r"(The equation \(E=mc^2\) is famous)")

# "mixed" mode handles user-escaped characters seamlessly
latex_wrap(r"(Cost: \$100 for $x$ items)")

# "mixed" mode converts R newlines to stacked text blocks
latex_wrap(r"(Line 1\nLine 2)")

# "math" mode returns the string completely unmodified
latex_wrap(r"(\frac{\alpha}{\beta})", input_mode = "math")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{gridmicrotex-deprecated}
\alias{gridmicrotex-deprecated}
\alias{load_font}
\alias{check_fonts}
\title{Deprecated functions}
\usage{
load_font(otf_path)

check_fonts()
}
\arguments{
\item{otf_path}{Path to the OTF/TTF font file.}
}
\value{
As the replacement function.
}
\description{
These functions still work but will be removed in a future release.
Both were renamed to say what they actually operate on: only
\strong{math} fonts (those carrying an OpenType MATH table) are ever
registered with MicroTeX. Text fonts need no loading --- they are
resolved on demand by \pkg{systemfonts} from \code{gp$fontfamily}.
}
\details{
\tabular{ll}{
\strong{Deprecated}  \tab \strong{Use instead} \cr
\code{load_font()}   \tab \code{\link{load_math_font}()}  \cr
\code{check_fonts()} \tab \code{\link{check_math_fonts}()} \cr
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/units.R
\name{bigpt_to_tex_pt}
\alias{bigpt_to_tex_pt}
\title{Convert big points to TeX points}
\usage{
bigpt_to_tex_pt(big_pt)
}
\arguments{
\item{big_pt}{Numeric value in PostScript points.}
}
\value{
Numeric value in TeX points.
}
\description{
Convert big points to TeX points
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gridmicrotex-package.R
\docType{package}
\name{gridmicrotex-package}
\alias{gridmicrotex}
\alias{gridmicrotex-package}
\title{gridmicrotex: Native 'LaTeX' Math Rendering for Grid Graphics}
\description{
Renders 'LaTeX' math equations as native R grid graphics objects (grobs) using the 'MicroTeX' 'C++' library as the layout engine. Produces resolution-independent vector output that works on any R graphics device, with no external 'LaTeX' installation required. Markdown labels and block documents that mix prose formatting with math are also rendered, for use with both 'grid' and 'ggplot2'.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/adayim/gridmicrotex}
  \item \url{https://adayim.github.io/gridmicrotex/}
  \item Report bugs at \url{https://github.com/adayim/gridmicrotex/issues}
}

}
\author{
\strong{Maintainer}: Alim Dayim \email{ad938@cam.ac.uk} (\href{https://orcid.org/0000-0001-9998-7463}{ORCID})

Other contributors:
\itemize{
  \item Nano Michael (Author of included 'MicroTeX' library) [copyright holder]
  \item  Bundled math font authors (See inst/COPYRIGHTS for the full list of authors of the bundled math fonts.) [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid-builder.R
\name{build_latex_children}
\alias{build_latex_children}
\title{Build grid children from a MicroTeX layout data.frame}
\usage{
build_latex_children(
  layout_df,
  total_h,
  depth = 0,
  text_gp = NULL,
  render_mode = "typeface"
)
}
\arguments{
\item{layout_df}{Data.frame returned by \code{parse_latex_cpp()}.}

\item{total_h}{Total height of the formula (height + depth) in bigpts.}

\item{depth}{Depth below the baseline in bigpts (default 0).}

\item{text_gp}{Optional \code{\link[grid]{gpar}} for text grobs
(from \code{\\text\{\}} blocks). Controls fontfamily and fontface.}

\item{render_mode}{Character string: \code{"path"} or \code{"typeface"}.
In typeface mode, glyph records are rendered via \code{glyphGrob}.}
}
\value{
A \code{grid::gList} of child grobs.
}
\description{
Converts each row of the layout data.frame into the appropriate
grid grob (pathGrob, segmentsGrob, rectGrob, textGrob, glyphGrob).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid-builder.R
\name{.get_glyph_font}
\alias{.get_glyph_font}
\title{Get or create a glyphFont object for a font file}
\usage{
.get_glyph_font(font_file)
}
\arguments{
\item{font_file}{Absolute path to the OTF/TTF font file.}
}
\value{
A \code{glyphFont} object.
}
\description{
Caches \code{grDevices::glyphFont()} objects keyed by file path
so that repeated calls for the same font file reuse the same object.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown-box.R
\name{markdown_box_grob}
\alias{markdown_box_grob}
\title{Render a markdown document as a boxed grid grob}
\usage{
markdown_box_grob(
  md,
  x = grid::unit(0.5, "npc"),
  y = grid::unit(0.5, "npc"),
  width = grid::unit(1, "npc"),
  height = NULL,
  hjust = 0.5,
  vjust = 0.5,
  halign = 0,
  valign = 1,
  padding = NULL,
  margin = NULL,
  box_gp = NULL,
  r = NULL,
  style = NULL,
  name = NULL,
  gp = grid::gpar(),
  vp = NULL
)
}
\arguments{
\item{md}{Character string of markdown.}

\item{x, y}{Position of the box in the parent viewport.}

\item{width}{Width of the box, including \code{margin}. \code{NULL}
sizes the box to its content, so nothing wraps --- useful where the
available width is not known, as in a ggplot2 theme element.}

\item{height}{Fixed height, or \code{NULL} (default) to take whatever
height the content needs.}

\item{hjust, vjust}{Justification of the whole box about \code{x} and
\code{y}.}

\item{halign}{Horizontal alignment of blocks within the box:
\code{0} left (default), \code{0.5} centred, \code{1} right.}

\item{valign}{Vertical alignment of the content when \code{height}
leaves room to spare: \code{1} top (default), \code{0} bottom.}

\item{padding, margin}{A \code{\link[grid]{unit}} of length 1 or 4
giving top, right, bottom and left. Padding is inside the box,
margin outside it. \code{NULL} (default) takes them from the
stylesheet's \code{body} rule, and is zero if that says nothing.}

\item{box_gp}{Graphical parameters for the box itself, e.g.
\code{gpar(fill = "grey95", col = "black")}. \code{NULL} (default)
takes the fill from \code{body \{ background \}} and the border from
\code{body \{ border \}}, and draws no box if neither is set.}

\item{r}{Corner radius; a non-zero value draws a rounded box.
\code{NULL} (default) takes it from \code{body \{ border-radius \}}.}

\item{style}{Appearance of the blocks: a \code{\link{markdown_style}}
object, CSS text, or a path to a \code{.css} file. \code{NULL}
(default) uses \code{latex_options("markdown_style")} if set, and
the built-in defaults otherwise.}

\item{name}{Optional grob name.}

\item{gp}{Graphical parameters for the text. \code{fontsize} also sets
the scale for block spacing and list indentation, and \code{cex}
multiplies it as elsewhere in \pkg{grid}.}

\item{vp}{Optional viewport. Supplying one replaces the viewport built
from \code{x}, \code{y}, \code{width}, \code{height}, \code{hjust}
and \code{vjust}, so those are then ignored.}
}
\value{
A \code{markdownbox} gTree.
}
\description{
Lays markdown out as a block document --- headings, paragraphs, lists
(including GFM task lists), block quotes, code blocks, tables,
horizontal rules and images --- inside an optional padded, filled and
bordered box. Prose wraps to the requested width, and \code{$...$}
math is typeset by MicroTeX as usual. All the inline formatting
\code{\link{markdown_grob}} understands, including the inline HTML
subset, works inside every block.
}
\details{
Where \code{\link{markdown_grob}} flattens everything into a single
run, this stacks one grob per block. That is what makes headings, list
indentation, block-quote rules and background fills possible:
MicroTeX has no concept of any of them, and its line breaking does not
reach inside the cells it uses for list and table layout.

Two consequences worth knowing. Table cells and code lines are not
wrapped, so a wide table overflows rather than reflowing. And list
items are stacked here rather than handed to MicroTeX's
\code{itemize}, which is what gives them a proper hanging indent.

An image on a line of its own is drawn as a raster, scaled to fit the
column but never enlarged past its natural size (pixels are read at
96 dpi). PNG needs the \pkg{png} package and JPEG needs \pkg{jpeg},
both \emph{Suggests}: when the reader is not installed, the file is
missing, or the format is anything else, the image degrades to its alt
text. An image \emph{within} a sentence stays inline, where only its
alt text survives.

The layout is computed at draw time, so an open device is required ---
which is what lets a relative \code{width} and the measured height of
the text resolve against the viewport the box is actually drawn in.
}
\section{Styling}{

Appearance comes from a small CSS cascade. \code{style} sets the house
style for the whole document, by tag:

\preformatted{markdown_box_grob(md, style = markdown_style(
  h1         = md_style(color = "steelblue", font_size = 2),
  blockquote = md_style(border_left = "3px solid grey60")
))}

The same thing written as CSS, which \code{style} also takes directly,
as text or as the path to a \code{.css} file:

\preformatted{markdown_box_grob(md, style = "
  h1 \{ color: steelblue; font-size: 2rem \}
  blockquote \{ border-left: 3px solid grey60 \}
")}

To style one chunk rather than every block of a kind, wrap it in a
\code{<div>} carrying a \code{class} or a \code{style}. \strong{Leave
blank lines around the tags} --- that is what makes CommonMark parse
the markdown between them instead of treating the whole thing as raw
HTML:

\preformatted{<div class="note">

## This heading only

</div>}

Inline runs take \code{class} as well as \code{style} on a
\code{<span>}. See \code{\link{markdown_style}} for the tag names, the
supported properties and how the cascade resolves.
}

\examples{
\donttest{
  md <- paste(
    "# Results", "",
    "The slope is $\\\\beta_1$ with *p* < 0.001.", "",
    "- first point", "- second point",
    sep = "\n"
  )
  grid::grid.newpage()
  grid::grid.draw(markdown_box_grob(
    md,
    width = grid::unit(4, "in"),
    padding = grid::unit(8, "pt"),
    box_gp = grid::gpar(fill = "grey95", col = "grey40")
  ))
}
}
\seealso{
\code{\link{markdown_grob}}, \code{\link{latex_grob}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/latex-grob.R
\name{.make_text_measurer}
\alias{.make_text_measurer}
\title{Create a text measurement closure for MicroTeX layout}
\usage{
.make_text_measurer(text_gp)
}
\arguments{
\item{text_gp}{A \code{\link[grid]{gpar}} object with font settings
(\code{fontfamily}, \code{fontface}) to use for measurement.}
}
\value{
A function taking \code{(text, font_style)} that returns
\code{c(width_ratio, ascent_ratio, height_ratio)} where ratios
are relative to the font size.
}
\description{
Returns a function that measures text using R's grid graphics system.
The closure is called from C++ during \code{parse_latex_cpp()} to get
accurate font metrics for \code{\\text\{\}} blocks.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/units.R
\name{tex_pt_to_bigpt}
\alias{tex_pt_to_bigpt}
\title{Convert TeX points to big points}
\usage{
tex_pt_to_bigpt(tex_pt)
}
\arguments{
\item{tex_pt}{Numeric value in TeX points.}
}
\value{
Numeric value in big (PostScript) points.
}
\description{
TeX points (1/72.27 inch) differ from PostScript/big points (1/72 inch).
R's grid "bigpts" unit uses PostScript points.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2-integration.R
\name{element_latex}
\alias{element_latex}
\title{A ggplot2 theme element for LaTeX text}
\usage{
element_latex(
  math_font = "",
  fontsize = NULL,
  lineheight = 1.2,
  max_width = 0,
  input_mode = c("mixed", "math"),
  render_mode = c("typeface", "path"),
  ...
)
}
\arguments{
\item{math_font}{Name of the math font to use (e.g., \code{"stix"}).
Use \code{""} (default) for Lete Sans Math, which pairs with R's
default sans-serif text font.
See \code{\link{available_math_fonts}} for loaded fonts.}

\item{fontsize}{Convenience alias for \code{size}; when supplied,
it is forwarded to \code{ggplot2::element_text()} as the text size
in points. If \code{NULL} (default), the theme's inherited size is
used.}

\item{lineheight}{Multi-line height multiplier (default 1.2), matching
\code{grid::gpar()} semantics.}

\item{max_width}{Numeric maximum width in big points for automatic
line wrapping.  Use \code{0} (default) for no wrapping.}

\item{input_mode}{How \code{tex} is interpreted before being parsed.
\code{"mixed"} (default) wraps the input in \code{\\text{...}} so the
string reads as ordinary text and \code{$...$} (or \code{\\(...\\)})
opens math mode, matching document-level LaTeX semantics. Useful for
labels that arrive from external sources mixing prose and math without
explicit \code{\\text{}} markers. \code{"math"} is the classic
MicroTeX behaviour --- the whole string is treated as math, so unwrapped
prose renders as spaced math italics. The default can be changed globally via
\code{\link{latex_options}(input_mode = "math")}. See \code{\link{latex_wrap}}
for details on the wrapping process.}

\item{render_mode}{Character string: \code{"typeface"} (default) renders
glyphs as native text using the math font, producing
selectable/accessible text in PDF and SVG output.
Bundled math fonts and any registered via \code{\link{load_math_font}}
are read directly from their OTF files --- no system-wide font
install is required.
Falls back to path mode automatically on devices that lack the
R \eqn{\geq} 4.3 glyph engine (e.g., the base \code{pdf()} device).
For selectable PDF output, prefer \code{\link[grDevices]{cairo_pdf}}.
\code{"path"} renders math symbols as filled vector paths (works on
all devices but text is not selectable in PDF/SVG).}

\item{...}{Additional arguments passed to \code{ggplot2::element_text()}
(e.g., \code{size}, \code{colour}, \code{hjust}).}
}
\value{
An S7 object of class \code{element_latex}, inheriting from
\code{ggplot2::element_text}.
}
\description{
Use this as a theme element for axis titles, axis labels, plot titles,
or any other text element in a ggplot2 theme. The text string is parsed
as LaTeX math and rendered via MicroTeX.
}
\details{
Dollar signs (\code{$...$}) in the label text are stripped automatically
so that both \code{"\\frac{a}{b}"} and \code{"$\\frac{a}{b}$"} work.

This element is an S7 subclass of \code{ggplot2::element_text}, so it
inherits all standard text properties (size, colour, hjust, etc.) from
the theme and supports \code{merge_element()} correctly.
}
\examples{
\donttest{
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)
  ggplot(mtcars, aes(wt, mpg)) + geom_point() +
    labs(x = "$\\\\beta_1 \\\\cdot x + \\\\beta_0$") +
    theme(axis.title.x = element_latex())
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{resolve_math_font}
\alias{resolve_math_font}
\title{Resolve a math font name}
\usage{
resolve_math_font(name)
}
\arguments{
\item{name}{Font name or alias. Empty string uses the default font.}
}
\value{
The resolved font name.
}
\description{
Translates short aliases (e.g., \code{"stix"}, \code{"lete"}) to the
full MicroTeX font name. Validates that the font is loaded.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown.R
\name{markdown_grob}
\alias{markdown_grob}
\alias{grid.markdown}
\title{Render markdown as a grid grob}
\usage{
markdown_grob(md, style = NULL, ...)

grid.markdown(md, ...)
}
\arguments{
\item{md}{Character string of markdown.}

\item{style}{Appearance of the text: a \code{\link{markdown_style}}
object, CSS text, or a path to a \code{.css} file. \code{NULL}
(default) uses \code{latex_options("markdown_style")} if set, and
the built-in defaults otherwise. Only the properties \code{md_style}
marks as \emph{inline} apply here --- there is no block layout in a
single run for a margin, an indent or an alignment to act on, so
those are ignored. \code{\link{markdown_box_grob}} honours them all.}

\item{...}{Passed to \code{\link{latex_grob}} --- e.g. \code{x},
\code{y}, \code{hjust}, \code{vjust}, \code{rot}, \code{max_width},
\code{gp}.}
}
\value{
A \code{latexgrob}, as returned by \code{\link{latex_grob}}.
}
\description{
Parses a markdown string and returns a grid grob, so plot labels can
mix ordinary prose formatting with real LaTeX math. Inline markdown
(\code{**bold**}, \code{*italic*}, \code{`code`}, \code{~~strike~~})
is translated to the equivalent LaTeX commands, while \code{$...$}
(and \code{\\(...\\)}, \code{$$...$$}, \code{\\[...\\]}) math spans
are passed through to MicroTeX byte-for-byte.
}
\details{
Markdown and LaTeX disagree about several characters --- most
importantly \code{\\}, which CommonMark treats as an escape. Math
spans are therefore hidden from the markdown parser before it runs and
restored afterwards, so constructs like
\code{$\\begin{matrix}a\\\\b\\end{matrix}$} survive intact.

That hiding uses three private-use codepoints (\code{U+E000},
\code{U+E001}, \code{U+E002}) as markers, so those three characters
are \emph{removed} from the input. They are unassigned in Unicode, but
icon fonts such as Nerd Fonts do put real glyphs there: if your text
contains one it will be dropped rather than drawn. The alternative is
worse --- a pasted marker would be spliced together with a math span
on the way back out and silently duplicate a formula.

GFM has no markdown syntax for colour, underline, super/subscript,
highlight or size, so --- as in CommonMark, and as \pkg{ggtext} does
--- these come from inline HTML. Each tag renders as HTML's own
default rendering prescribes:

\tabular{ll}{
\strong{tag} \tab \strong{effect} \cr
\code{<b>}, \code{<strong>} \tab bold \cr
\code{<i>}, \code{<em>}, \code{<cite>}, \code{<dfn>}, \code{<var>},
\code{<address>} \tab italic \cr
\code{<code>}, \code{<kbd>}, \code{<samp>}, \code{<tt>} \tab monospace \cr
\code{<u>}, \code{<ins>} \tab underline \cr
\code{<s>}, \code{<del>}, \code{<strike>} \tab strikethrough \cr
\code{<sub>}, \code{<sup>} \tab sub / superscript \cr
\code{<mark>} \tab yellow highlight \cr
\code{<small>}, \code{<big>} \tab smaller / larger \cr
\code{<q>} \tab wrapped in quotation marks \cr
\code{<br>} \tab line break \cr
\code{<span style="...">} \tab see below
}

A \code{style} attribute is read for \code{color} (any R colour name,
the nine CSS names R lacks --- \code{crimson}, \code{teal},
\code{rebeccapurple} and friends --- \code{#rgb}, \code{#rrggbb} or
\code{rgb()}; note that \code{green}, \code{gray}, \code{grey},
\code{maroon} and \code{purple} keep their R values, not their CSS
ones),
\code{text-decoration} (\code{underline}, \code{line-through}),
\code{font-size} (\code{pt}, \code{px}, \code{in}, \code{cm},
\code{mm}, \code{em}, \code{rem}, \code{\%}, \code{smaller},
\code{larger}) and \code{font-family}. Any other property is ignored.

\code{font-family} takes the CSS generics \code{monospace},
\code{sans-serif} and \code{serif}, or any font name; a fallback list
resolves to its first entry. The name is handed to
\code{gp$fontfamily}, so \emph{the device resolves it}: \pkg{ragg} and
\pkg{svglite} see any installed family plus anything registered with
\code{systemfonts::register_font()}, cairo devices see installed
families, and base \code{pdf()} sees only what \code{pdfFonts()}
declares --- a named family will not resolve there. An unavailable
font falls back silently, as it does for \code{gpar(fontfamily=)}.
A font file that is not installed system-wide is used by registering
it first:

\preformatted{systemfonts::register_font(name = "MyFont", plain = "MyFont.otf")}

\code{\link{load_math_font}} is \emph{not} the function for this ---
it registers \emph{math} fonts with MicroTeX, which is a different
mechanism.

A span's own family wins over \code{gp$fontfamily}, but the width of
the spaces \emph{between} its words still comes from
\code{gp$fontfamily}; set both to the same family if that shows.

Tags nest and combine freely with markdown. Any other tag --- and all
block-level HTML --- is dropped, keeping the text inside it, which is
also what a browser shows for the ones (\code{<a>}, \code{<abbr>},
\code{<span>} without a style) that have no default rendering.

Not every markdown feature has a MicroTeX equivalent. Links keep their
text and drop the destination, and images keep their alt text.

Everything is flattened into a single run here, with paragraphs joined
by line breaks: there is no block layout, so indentation, list markers
and block-quote rules need \code{\link{markdown_box_grob}}. A heading
still takes the size and weight its \code{style} gives it, since those
compile to LaTeX commands rather than to layout.
}
\examples{
\donttest{
  grid::grid.newpage()
  grid.markdown("The **fitted** slope is $\\\\beta_1$, *p* < 0.001")
}
}
\seealso{
\code{\link{latex_grob}}, \code{\link{latex_wrap}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2-markdown.R
\docType{data}
\name{geom_markdown}
\alias{geom_markdown}
\alias{GeomMarkdown}
\title{A ggplot2 geom for markdown labels}
\usage{
geom_markdown(
  mapping = NULL,
  data = NULL,
  stat = "identity",
  position = "identity",
  ...,
  fontsize = 11,
  math_font = "",
  lineheight = 1.2,
  max_width = 0,
  render_mode = c("typeface", "path"),
  justify = FALSE,
  style = NULL,
  na.rm = FALSE,
  show.legend = NA,
  inherit.aes = TRUE
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}}. If specified and
\code{inherit.aes = TRUE} (the default), it is combined with the default mapping
at the top level of the plot. You must supply \code{mapping} if there is no plot
mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{stat}{The statistical transformation to use on the data for this layer.
When using a \verb{geom_*()} function to construct a layer, the \code{stat}
argument can be used to override the default coupling between geoms and
stats. The \code{stat} argument accepts the following:
\itemize{
\item A \code{Stat} ggproto subclass, for example \code{StatCount}.
\item A string naming the stat. To give the stat as a string, strip the
function name of the \code{stat_} prefix. For example, to use \code{stat_count()},
give the stat as \code{"count"}.
\item For more information and other ways to specify the stat, see the
\link[ggplot2:layer_stats]{layer stat} documentation.
}}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{...}{Other arguments passed to \code{\link[ggplot2]{layer}}.}

\item{fontsize}{Default font size in points. Overridden by the
\code{size} aesthetic if mapped.}

\item{math_font}{Name of the math font to use (e.g. \code{"stix"}).}

\item{lineheight}{Multi-line height multiplier (default 1.2), matching
\code{grid::gpar()} semantics.}

\item{max_width}{Maximum width in big points for automatic line
wrapping (default: 0, no wrapping).}

\item{render_mode}{\code{"typeface"} (default) or \code{"path"}; see
\code{\link{latex_grob}}.}

\item{justify}{Logical; justify wrapped lines. Requires
\code{max_width}. See \code{\link{latex_grob}}.}

\item{style}{A \code{\link{markdown_style}} object, CSS text, or the
path to a \code{.css} file, applied to every label this layer draws.
\code{NULL} falls back to
\code{\link{latex_options}(markdown_style = )}. Only the properties
that compile to LaTeX apply here --- a label has no block layout, so
margins and padding are ignored. See \code{\link{md_style}}.}

\item{na.rm}{If \code{FALSE}, the default, missing values are removed
with a warning. If \code{TRUE}, they are removed silently.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{inherit.aes}{If \code{FALSE}, overrides the default aesthetics,
rather than combining with them. This is most useful for helper functions
that define both data and aesthetics and shouldn't inherit behaviour from
the default plot specification, e.g. \code{\link[ggplot2:annotation_borders]{annotation_borders()}}.}
}
\value{
A ggplot2 layer.
}
\description{
Renders markdown labels -- \code{**bold**}, \code{*italic*},
\code{`code`}, \code{~~strike~~} and \code{$math$} -- as native grid
grobs inside a plot. The markdown is converted to LaTeX and laid out by
MicroTeX, so the output is resolution-independent vector graphics.
}
\section{Aesthetics}{

\code{geom_markdown()} understands the following aesthetics (required
aesthetics are in bold):
\itemize{
\item \strong{\code{x}}
\item \strong{\code{y}}
\item \strong{\code{label}} --- markdown string
\item \code{size} --- font size in points (default: 11)
\item \code{colour} --- text colour (default: \code{"black"})
\item \code{angle} --- rotation angle in degrees (default: 0)
\item \code{hjust} --- horizontal justification, 0-1 (default: 0.5)
\item \code{vjust} --- vertical justification, 0-1 (default: 0.5)
\item \code{alpha} --- transparency (default: 1)
}
}

\examples{
\donttest{
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)
  df <- data.frame(
    x = 1:3, y = 1:3,
    lab = c("**bold**", "*slope* $\\\\beta_1$", "`code` and $x^2$")
  )
  ggplot(df, aes(x, y, label = lab)) + geom_markdown()
}
}
}
\seealso{
\code{\link{markdown_grob}}, \code{\link{geom_latex}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{available_math_fonts}
\alias{available_math_fonts}
\title{List available math fonts}
\usage{
available_math_fonts()
}
\value{
A character vector of math font names.
}
\description{
Returns the names of all math fonts currently loaded by MicroTeX.
These names can be passed to the \code{math_font} parameter of
\code{\link{latex_grob}} and \code{\link{grid.latex}}.
}
\section{Font pairing}{

The bundled math fonts have different styles. For a consistent look,
pair them with a matching \code{fontfamily} in \code{gp}:

\tabular{lll}{
\strong{Math font}     \tab \strong{Style}  \tab \strong{Suggested text font} \cr
Lete Sans Math (\code{"lete"}, default) \tab Sans-serif \tab \code{"sans"} \cr
STIX Two Math (\code{"stix"})   \tab Serif  \tab \code{"serif"} \cr
}
Additional math fonts can be loaded with \code{\link{load_math_font}}.
}

\examples{
available_math_fonts()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fonts.R
\name{check_fonts}
\alias{check_fonts}
\title{Check font status}
\usage{
check_fonts()
}
\value{
Invisibly returns the character vector of available font names.
}
\description{
Reports which math fonts are loaded and available for rendering.
Shows the MicroTeX version, loaded math fonts, and whether bundled
font files are present.
}
\examples{
check_fonts()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid-builder.R
\name{build_path_grob}
\alias{build_path_grob}
\title{Convert path segments to a grid pathGrob}
\usage{
build_path_grob(path_data, col, idx, total_h)
}
\arguments{
\item{path_data}{List with \code{cmd} and \code{coords} elements.}

\item{col}{Fill color.}

\item{idx}{Index for naming.}

\item{total_h}{Total height for y-axis flipping.}
}
\value{
A \code{grid::pathGrob} or \code{NULL}.
}
\description{
Convert path segments to a grid pathGrob
}
\keyword{internal}
